#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   05202020    Update for PJ45657 support for Real-time Runtime      #
#               Metrics Collection.                                   #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

# Validate the input and provide help info
if [ ! $# -ge 1 ]; then
   echo "Missing input parameter!"
   echo "Syntax:"
   echo "  tpf_capture_data_from_kafka.sh scenario_name [port] [cdc|nvpc|jvm|udm|all]"
   echo "  port 9093 is the default"
   echo "  all is the default"
   echo "The scenario_name must begin with scenario_."
   echo "A directory will be created by this name in the tpf_data_sci/tpfReplayScript directory."
   echo "Data will be captured for CDC, NVPC, NVPC vertical, and JVM data."
   exit 1
fi


PORT="9093"
if [ $# -ge 2 ]; then
   PORT=$2
fi

TYPE="all"
if [ $# -eq 3 ]; then
   TYPE=$(echo "$3" | tr '[:upper:]' '[:lower:]')  # Convert the argument to lowercase
fi

if [[ ! $1 =~ scenario_..* ]]; then
   echo "The scenario_name must begin with scenario_ followed by at least 1 character."
   exit 1
fi

# Define variables for our file names
CDC_FILE_NAME="tpfReplayScript/$1/tpf_sample_cdc.json"
NVPC_FILE_NAME="tpfReplayScript/$1/tpf_sample_nvpc.json"
NVPC_VERT_FILE_NAME="tpfReplayScript/$1/tpf_sample_nvpc_vert.json"
JVM_FILE_NAME="tpfReplayScript/$1/tpf_sample_jvm.json"
UDM_FILE_NAME="tpfReplayScript/$1/tpf_sample_udm.json"

# Create the directory for the results.
mkdir -p "../tpfReplayScript/$1"

# We'll need the hostname for this server in a couple of places.
FULLY_QUALIFIED_HOSTNAME="$(hostname --fqdn):$PORT"

CURRENT=$PWD

# Time to do our docker work
cd ./tpf_replay_script_docker_files

# Build the replay script docker image
docker build -t tpf_replay_script_img .

cd $CURRENT

# Run the Replay Script
if [[ ("$TYPE" == "cdc" || "$TYPE" == "all")]]; then
  echo "-------------------------------------------------------------------------------"
  echo "Capturing CDC data to ../$CDC_FILE_NAME"
  echo "-------------------------------------------------------------------------------"
  docker run -t --name tpf-kafka-capture-script-cdc --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar capture $FULLY_QUALIFIED_HOSTNAME TPF_CDC_DATA /home/tpf_files/$CDC_FILE_NAME -consume_timeout 30 -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "nvpc" || "$TYPE" == "all")]]; then
echo "-------------------------------------------------------------------------------"
echo "Capturing NVPC data to ../$NVPC_FILE_NAME"
echo "-------------------------------------------------------------------------------"
docker run -t --name tpf-kafka-capture-script-nvpc --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar capture $FULLY_QUALIFIED_HOSTNAME TPF_NVPC_DATA /home/tpf_files/$NVPC_FILE_NAME -consume_timeout 30 -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "nvpc_vert" || "$TYPE" == "all")]]; then
echo "-------------------------------------------------------------------------------"
echo "Capturing NVPC vertical data to ../$NVPC_VERT_FILE_NAME"
echo "-------------------------------------------------------------------------------"
docker run -t --name tpf-kafka-capture-script-nvpc-vert --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar capture $FULLY_QUALIFIED_HOSTNAME TPF_NVPC_VERT_DATA /home/tpf_files/$NVPC_VERT_FILE_NAME -consume_timeout 30 -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "jvm" || "$TYPE" == "all")]]; then
echo "-------------------------------------------------------------------------------"
echo "Capturing JVM data to ../$JVM_FILE_NAME"
echo "-------------------------------------------------------------------------------"
docker run -t --name tpf-kafka-capture-script-jvm --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar capture $FULLY_QUALIFIED_HOSTNAME TPF_JVM_DATA /home/tpf_files/$JVM_FILE_NAME -consume_timeout 30 -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "udm" || "$TYPE" == "all")]]; then
echo "-------------------------------------------------------------------------------"
echo "Capturing UDM data to ../$UDM_FILE_NAME"
echo "-------------------------------------------------------------------------------"
docker run -t --name tpf-kafka-capture-script-udm --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar capture $FULLY_QUALIFIED_HOSTNAME TPF_UDM_DATA /home/tpf_files/$UDM_FILE_NAME -consume_timeout 30 -kafka_host /home/tpf_files/kafka_hosts.yml
fi

echo "-------------------------------------------------------------------------------"
echo "tpf_capture_data_from_kafka.sh is done processing"
echo "-------------------------------------------------------------------------------"

