/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2024                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Common Dashboard Framework (cdf) implements a set   */
/*               of functions that can be reused to support multiple */
/*               data types.                                         */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';


DROP PROCEDURE IF EXISTS `tpf_cdf_get_data_types`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_get_data_types` (
    IN `in_dashboard_key` TEXT
)

tpf_cdf_get_data_types:BEGIN

DECLARE table_metrics JSON;
DECLARE dashboard_key_array TEXT;

SET table_metrics = tpf_get_metrics_json();

IF in_dashboard_key = "all"
THEN
   SET @dashboard_key_array = 'system,communications,storage,subsystems';
ELSE
   SET @dashboard_key_array = in_dashboard_key;
END IF;


SELECT DISTINCT panel_key FROM JSON_TABLE(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE FIND_IN_SET(jt.dashboard_key, @dashboard_key_array);


END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_cdf_get_analysis_types`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_get_analysis_types` (
    IN `in_panel_key` TEXT
)
tpf_cdf_get_analysis_types:BEGIN

DECLARE table_metrics JSON;
DECLARE attributes_list JSON;
DECLARE analysis_name_list JSON;
DECLARE result JSON;

SET table_metrics = tpf_get_metrics_json();

SELECT JSON_ARRAYAGG(attribute_name)
INTO attributes_list
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_panel_key AND attribute_name != "";

SET analysis_name_list =  tpf_get_analysis_name_func(in_panel_key);

IF attributes_list IS NULL AND analysis_name_list IS NULL
THEN
    SELECT "N/A";
    LEAVE tpf_cdf_get_analysis_types;
END IF;

IF JSON_LENGTH(attributes_list) = 0 AND JSON_LENGTH(analysis_name_list) = 0
THEN
    SELECT "N/A";
    LEAVE tpf_cdf_get_analysis_types;
END IF;

IF attributes_list IS  NULL OR JSON_LENGTH(attributes_list) = 0
THEN
    SET result = analysis_name_list;
ELSEIF analysis_name_list IS NULL OR JSON_LENGTH(analysis_name_list) = 0
THEN
    SET result = attributes_list;
ELSE
    SET result = JSON_MERGE(analysis_name_list, attributes_list);
END IF;

SELECT name
FROM JSON_TABLE(result, "$[*]" COLUMNS(name VARCHAR(64) path "$")) ism;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_cdf_get_analysis_filters`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_get_analysis_filters` (
    IN `in_panel_key` TEXT,
    IN `in_tpf_systems` JSON,
    IN `in_tpf_subsystems` JSON,
      IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_attribute_type` TEXT
)

tpf_cdf_get_analysis_filters:BEGIN

DECLARE table_metrics JSON;
DECLARE table_name TEXT DEFAULT "";
DECLARE attribute_table_var TEXT DEFAULT "";
DECLARE table_column_var TEXT DEFAULT "";
DECLARE attribute_table_column TEXT DEFAULT "";
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE subsystem_available TEXT DEFAULT "";
DECLARE inner_join_stmt TEXT DEFAULT "";
DECLARE analysis_function_found TEXT DEFAULT "";
DECLARE available_results JSON;
DECLARE cur_attribute TEXT DEFAULT "";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

--  Validate in_attribute_types is JSON
IF  in_attribute_type='""'
OR in_attribute_type=""
OR in_attribute_type='"N/A"'
THEN
    SELECT "N/A" AS __text, 0 AS __value;
    LEAVE tpf_cdf_get_analysis_filters;
END IF;

-- Show attributes for z/TPFDF even if there is a function.
SET cur_attribute = tpf_get_input_attribute_type(in_attribute_type);
SET analysis_function_found = tpf_find_analysis_func(cur_attribute, "table_procedure");
IF  analysis_function_found != "" AND in_panel_key != "tpfdf"
THEN
    SELECT "N/A" AS __text, 0 AS __value;
    LEAVE tpf_cdf_get_analysis_filters;
END IF;

IF JSON_TYPE(in_tpf_systems) = "STRING"
THEN
    SET in_tpf_systems = JSON_UNQUOTE(in_tpf_systems);
    SET in_tpf_systems = JSON_ARRAY(in_tpf_systems);
END IF;

IF JSON_TYPE(in_tpf_subsystems) = "STRING"
THEN
    SET in_tpf_subsystems = JSON_UNQUOTE(in_tpf_subsystems);
    SET in_tpf_subsystems = JSON_ARRAY(in_tpf_subsystems);
END IF;

SET table_metrics = tpf_get_metrics_json();

IF in_panel_key = "tpfdf" AND  analysis_function_found != ""
THEN
    SET table_name = "tpf_tpfdf_metrics_table";
    SET table_column_var = "file_id";
ELSE
 SELECT DISTINCT attribute_table,attribute_name,table_column
 INTO attribute_table_var,table_column_var,attribute_table_column
 FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
 ) AS jt
 WHERE jt.panel_key=in_panel_key AND jt.attribute_name = cur_attribute;
 IF attribute_table_var = "" AND table_column_var = ""
 THEN
    SELECT "N/A" AS __text, 0 AS __value;
    LEAVE tpf_cdf_get_analysis_filters;
 END IF;
END IF;

SET table_name = tpf_get_table_name(in_panel_key);

 SET subsystem_available = tpf_get_subsystem_applies(in_panel_key);
 IF subsystem_available = "yes"
 THEN
    SET inner_join_stmt = CONCAT(
        " INNER JOIN JSON_TABLE('",
        in_tpf_subsystems,
        "', '$[*]' COLUMNS(id INT path '$')) sub_its ON tn.subsystem_id=sub_its.id  ");
 END IF;

 IF in_panel_key != "tpfdf" AND   attribute_table_var != ""
 THEN
        SET select_stmt = CONCAT(
            " SELECT DISTINCT ",table_column_var,
             " AS __text, ",
            " tss.id  AS __value \n",
            " FROM ",table_name," tn ",
            "\n INNER JOIN JSON_TABLE('",
            in_tpf_systems,
            "', '$[*]' COLUMNS(id INT path '$')) its ON tn.system_id=its.id \n",
             inner_join_stmt,
            " INNER JOIN ",attribute_table_var," tss ON tn.",attribute_table_column,"=tss.id ",
             "\n WHERE tn.TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'\n");
        CALL tpf_execute_immediate(select_stmt);
        LEAVE tpf_cdf_get_analysis_filters;
  END IF;
  IF in_panel_key != "tpfdf"
  THEN
        SET select_stmt = CONCAT(
            " SELECT DISTINCT ",table_column_var,
            " AS __text,",
              table_column_var," AS __value",
            " FROM ",table_name," tn ",
            " INNER JOIN JSON_TABLE('", in_tpf_systems,"', '$[*]' COLUMNS(id INT path '$')) its ON tn.system_id=its.id",
             inner_join_stmt,
        " WHERE tn.TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'",
        " AND ",table_column_var," != '' ");
        CALL tpf_execute_immediate(select_stmt);
        LEAVE tpf_cdf_get_analysis_filters;
  END IF;
--
-- This is tpfdf
--
SET inner_join_stmt = CONCAT(
    " INNER JOIN JSON_TABLE('", in_tpf_systems,"', '$[*]' COLUMNS(id INT path '$')) its ON tn.system_id=its.id\n",
    CASE WHEN subsystem_available = "yes" THEN CONCAT(" INNER JOIN JSON_TABLE('",in_tpf_subsystems,"', '$[*]' COLUMNS(id INT path '$')) sub_its ON tn.subsystem_id=sub_its.id  ") ELSE "" END,
    "\n INNER JOIN tpf_tpfdf_file_name_alias_table fnat ON file_name_alias_id = fnat.id \n");

  IF analysis_function_found != ""
  THEN
      -- The user selected some analysis function.
      -- We want to differentiate between the file_id_ebcdic and the file_name alias
      -- So we will show the dashes between the two.
      -- If we have neither, then do not show the dash.
      -- B095-B9-
      -- BD95--BD95
      SET select_stmt = CONCAT(
           " SELECT DISTINCT CONCAT(file_id, ",
           "  CASE WHEN file_id_ebcdic != '' AND fnat.file_name_alias != 'N/A' THEN CONCAT('-',file_id_ebcdic,'-',file_name_alias)  ",
           "  \n WHEN file_id_ebcdic != '' THEN CONCAT('-',file_id_ebcdic,'-')  ",
           "  \n WHEN fnat.file_name_alias != 'N/A' THEN CONCAT('--',file_name_alias) ",
           " \n ELSE '' END ",
           ") AS __text,\n  ",
           " file_id AS __value\n",
           " FROM tpf_tpfdf_metrics_table tn \n",
            inner_join_stmt,
           "\n WHERE tn.TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'");
  ELSE
        -- This is the case where the user selected analysis by function for tpfdf.
        -- One of file_id, file_id_ebcdic, or file_name_alias
        SET select_stmt = CONCAT(
          "  SELECT DISTINCT CASE WHEN ",table_column_var," != '' THEN  ", table_column_var, " ELSE 'N/A' END",
            " AS __text,",
            CASE WHEN attribute_table_var != "" THEN " fnat.id " ELSE table_column_var END,
             " AS __value",
            " FROM ",table_name," tn ",
             inner_join_stmt,
        " WHERE tn.TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'");
  END IF;
  CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_create_series_name_string`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_create_series_name_string` (
    `in_num_subsystems` INT,
    `in_agg_type` ENUM('no_agg', 'avg_systems', 'total_systems'),
    `in_attribute_types` TEXT,
    `in_panel_key` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE series_name_stmt TEXT DEFAULT "";
DECLARE series_base_fields TEXT DEFAULT "";
DECLARE file_id_str TEXT DEFAULT "";

-- The following are the abbreviations for the tables:
-- tn: input table name
-- tss: tpf_systems
-- itsitn: temp table with system information: in_temp_sys_id_table_name
-- isst: the list of input subsystems

IF in_agg_type = 'avg_systems'
THEN
    SET series_name_stmt =  "'avg-systems' AS series_name,";
    RETURN series_name_stmt;
ELSEIF in_agg_type = 'total_systems'
THEN
    SET series_name_stmt =  "'total_systems' AS series_name,";
    RETURN series_name_stmt;
END IF;

IF `in_panel_key` = 'tpfdf' AND in_agg_type = 'no_agg'
THEN
    SET file_id_str = tpf_tpfdf_file_id_display_name();
    SET series_name_stmt =  CONCAT(" CONCAT(", file_id_str,
         CASE WHEN in_num_subsystems > 1 THEN CONCAT(" ,'-'", ",tss.subsystem") ELSE "" END,
         " ,'-'", ",itsitn.display_name) AS series_name,");
    RETURN series_name_stmt;
END IF;

SET series_name_stmt =  " CONCAT_WS('-',";
IF in_agg_type = 'no_agg' AND in_num_subsystems > 1
THEN
    SET series_base_fields = "tss.subsystem, itsitn.display_name) ";
ELSEIF in_agg_type = 'no_agg'
THEN
    SET series_base_fields = "itsitn.display_name) ";
ELSE
    SET series_base_fields = "tn.subsystem_id) ";
END IF;

SET series_name_stmt =  CONCAT(series_name_stmt,
    CASE WHEN in_attribute_types != "" THEN CONCAT(in_attribute_types, ",") ELSE "" END,
    series_base_fields, " AS series_name,");

RETURN series_name_stmt;

END//
DELIMITER ;

-- Return false if the metrics requested to show
-- or the analysis metric is not in the list
-- of metrics for the in panel key.
DROP FUNCTION IF EXISTS `tpf_validate_metric_columns`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_validate_metric_columns` (
    `in_panel_key` TEXT,
    `in_show_metrics` JSON,
    `in_metric_types` JSON,
    `in_analysis_metric` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE metric_columns TEXT DEFAULT "";
DECLARE  metrics JSON;
DECLARE  show_metrics JSON;
DECLARE table_metrics JSON;
DECLARE json_array_metrics_result JSON;
DECLARE available_metrics TEXT;
DECLARE available_virtual_metrics JSON;

-- Use the panel key to get the metrics and the virtual metrics
-- and validate that the metrics are in that list
-- because we are writing something generic, this will protect us.

 SELECT JSON_ARRAYAGG(
    CASE WHEN input_fields LIKE '%-%' THEN SUBSTRING_INDEX(input_fields, '-', 1)
    ELSE input_fields END)
 INTO  show_metrics
 FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
 IF JSON_LENGTH(show_metrics) = 0 AND in_analysis_metric = ""
 THEN
    RETURN FALSE;
 END IF;

-- Set up the available metrics including virtual metrics
SET metrics = tpf_get_metrics_available_func(in_panel_key,in_metric_types);
SET available_virtual_metrics = tpf_get_virtual_metrics(in_panel_key,in_metric_types);
IF available_virtual_metrics IS NOT NULL AND JSON_LENGTH(available_virtual_metrics) > 0
THEN
    SET available_metrics = JSON_MERGE(metrics,available_virtual_metrics);
ELSE
    SET available_metrics = metrics;
END IF;

-- Test for the analysis metric
IF in_analysis_metric != "" AND
    JSON_CONTAINS(available_metrics,  CONCAT('"',in_analysis_metric,'"')) IS FALSE
    THEN
        RETURN FALSE;
END IF;

-- Test for the requested metrics
SELECT JSON_ARRAYAGG(cur_metric)
INTO table_metrics
FROM JSON_TABLE(show_metrics ,'$[*]'COLUMNS(cur_metric VARCHAR(64) PATH '$' )) AS jt
WHERE JSON_CONTAINS(available_metrics,  CONCAT('"',jt.cur_metric,'"'));

IF JSON_LENGTH(table_metrics) = JSON_LENGTH(show_metrics)
THEN
     RETURN TRUE;
END IF;
RETURN FALSE;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_create_inner_join_string`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_create_inner_join_string` (
    `in_panel_key` VARCHAR(64),
    `in_temp_sys_id_table_name` VARCHAR(128),
    `in_tpf_subsystems` JSON,
    `in_subsystems_requested` VARCHAR(64),
     `in_cur_attribute` VARCHAR(64),
    `in_filters_selected` JSON,
    `in_system_ids` JSON,
    `in_systems_columns` JSON
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE inner_join_stmt LONGTEXT DEFAULT "";
DECLARE num_subsystems INT DEFAULT 0;
DECLARE table_name VARCHAR(64);
DECLARE attribute_table_name_list JSON;
DECLARE attribute_table_statement LONGTEXT DEFAULT "";
DECLARE table_column_name TEXT DEFAULT "";
DECLARE attribute_table_name TEXT DEFAULT "";
DECLARE table_metrics JSON;

-- The following are the abbreviations for the tables:
-- tn: input table name
-- tss: tpf_systems
-- itsitn: in_temp_sys_id_table_name: table of selected system ids used to filter selected systems
-- isst: the list of input subsystems

-- The inner joins are with this table:
-- " FROM ", table_name, " tn ",
-- temp_sys_id_table_name
IF  in_cur_attribute != ""
THEN
    SET table_metrics = tpf_get_metrics_json();
    SELECT DISTINCT table_column, attribute_table
    INTO table_column_name, attribute_table_name
    FROM json_table(table_metrics, '$[*]'
    COLUMNS(
        panel_key         VARCHAR(64) PATH '$.panel_key',
        dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
        subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
        table_name        VARCHAR(64) PATH '$.table_name',
        NESTED PATH '$.attribute_columns[*]' COLUMNS (
        attribute_name VARCHAR(64) PATH '$.attribute_name',
        attribute_table VARCHAR(64) PATH '$.attribute_table',
        table_column VARCHAR(64) PATH '$.table_column'
        ),
        metric_type       VARCHAR(64) PATH '$.metric_type',
        NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.virtual_metrics[*]' COLUMNS (
        virtual_metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.extra_cols[*]' COLUMNS (
        extra_cols VARCHAR(64) PATH '$'
        ),
        altered_column   VARCHAR(64) PATH '$.altered_column'
     )
    ) AS jt
    WHERE jt.panel_key=in_panel_key AND jt.attribute_name = in_cur_attribute;
END IF;

-- Always run an inner join against the system id
-- Example:  INNER JOIN service_app_metrics_table tn ON tn.id=itn.system_id
IF in_temp_sys_id_table_name IS NOT NULL AND in_temp_sys_id_table_name != ""
THEN
    SET inner_join_stmt = CONCAT("\n INNER JOIN ",
                                in_temp_sys_id_table_name,
                                " itsitn ON tn.system_id=itsitn.id");
ELSEIF in_systems_columns IS NOT NULL AND in_systems_columns != ""
THEN
    SET inner_join_stmt = CONCAT("\n INNER JOIN JSON_TABLE('",
                                in_systems_columns,
                                "', \"$[*]\" COLUMNS(id INT path \"$.id\", display_name VARCHAR(100) path \"$.display_name\")) itsitn ",
                                "ON tn.system_id=itsitn.id");
ELSE
    SET inner_join_stmt = CONCAT("\n INNER JOIN JSON_TABLE('",in_system_ids,"', \"$[*]\" COLUMNS(id INT path \"$\")) its ON tn.system_id=its.id");
END IF;

-- Filter the selected subsystems
SET num_subsystems = JSON_LENGTH(in_tpf_subsystems);
IF in_subsystems_requested = "yes" AND num_subsystems > 0
THEN
    SET inner_join_stmt =
        CONCAT(inner_join_stmt,
        "\n INNER JOIN JSON_TABLE(",
        QUOTE(in_tpf_subsystems),
         ", \"$[*]\" COLUMNS(id INT path \"$\")) isst ON tn.subsystem_id=isst.id");
END IF;

-- Create the JOIN statements for all attribute tables.
IF (in_filters_selected IS NOT NULL AND in_filters_selected != "")
THEN
    SET attribute_table_name_list  =  tpf_get_table_name_list_inner_join_func(in_panel_key);
    SELECT GROUP_CONCAT(att_table_item SEPARATOR ' ')
    INTO attribute_table_statement
    FROM JSON_TABLE(attribute_table_name_list, "$[*]" COLUMNS(att_table_item  VARCHAR(255) path "$")) ism;
END IF;
IF attribute_table_statement IS NOT NULL AND attribute_table_statement != ""
THEN
    SET inner_join_stmt = CONCAT(inner_join_stmt," ", attribute_table_statement);
END IF;

-- This will filter on the selected attributes.
-- If there are attributes, this must be done for all attributes,
-- since all attributes are displayed in the table.
-- There might be more than one attribute table name
IF attribute_table_name IS NOT NULL AND attribute_table_name != ""  AND in_filters_selected != ""
THEN
    SET inner_join_stmt =
        CONCAT(inner_join_stmt,
        "\n INNER JOIN JSON_TABLE(",
        QUOTE(in_filters_selected),
        ", \"$[*]\" COLUMNS(id INT path \"$\")) oi ON ",attribute_table_name,".id = oi.id");
ELSEIF table_column_name != "" AND in_filters_selected != ""
THEN
    SET inner_join_stmt =
        CONCAT(inner_join_stmt,
        "\n INNER JOIN JSON_TABLE(",
        QUOTE(in_filters_selected),
        ", \"$[*]\" COLUMNS(id VARCHAR(16) path \"$\")) oi ON tn.", table_column_name," = oi.id");
END IF;

-- Get the subsystem name
IF in_subsystems_requested = "yes" AND num_subsystems > 0
THEN
    SET inner_join_stmt =
        CONCAT(inner_join_stmt,"\n INNER JOIN tpf_subsystems tss ON tn.subsystem_id=tss.id");
END IF;

RETURN inner_join_stmt;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_create_select_agg_stmt`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_create_select_agg_stmt` (
    `ss_entry` TEXT,
    `at_name_agg_display` TEXT,
    `metric_columns` TEXT,
    `extra_column_avg_display` TEXT,
    `key_entry` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE agg_stmt TEXT DEFAULT "";
SET agg_stmt = CONCAT(
     CASE WHEN  ss_entry = 'yes' THEN  CONCAT(",'N/A' as subsystems ") ELSE "" END,
     CASE WHEN  key_entry != "" THEN  CONCAT( ", 'N/A' as ", key_entry) ELSE "" END,
     CASE WHEN at_name_agg_display != '' THEN  CONCAT(",",at_name_agg_display) ELSE "" END,
     CASE WHEN  metric_columns != '' THEN CONCAT( ", ", metric_columns) ELSE "" END,
     CASE WHEN  extra_column_avg_display != "" THEN  CONCAT(",",extra_column_avg_display) ELSE "" END,
     ", time ");
RETURN agg_stmt;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_cdf_docs`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_docs` (
    IN `in_datatype` VARCHAR(128)
)

tpf_cdf_docs:BEGIN
   DECLARE tpf_static_metrics_table MEDIUMTEXT;
   DECLARE tpf_dasd_queue_metrics_table MEDIUMTEXT;
   DECLARE tpf_dasd_service_time_metrics_table MEDIUMTEXT;
   DECLARE tpf_system_messages_metrics_table MEDIUMTEXT;
   DECLARE tpf_system_blocks_metrics_table MEDIUMTEXT;
   DECLARE tpf_system_lists_metrics_table MEDIUMTEXT;
   DECLARE tpf_tape_metrics_table MEDIUMTEXT;
   DECLARE tpf_vfa_metrics_table MEDIUMTEXT;
   DECLARE tpf_istream_metrics_table MEDIUMTEXT;
   DECLARE tpf_subsystem_users_metrics_table MEDIUMTEXT;
   DECLARE tpf_subsystems_metrics_table MEDIUMTEXT;
   DECLARE tpf_subsystems_utility_switches_metrics_table MEDIUMTEXT;
   DECLARE tpf_dasd_io_metrics_table MEDIUMTEXT;
   DECLARE tpf_dasd_pool_metrics_table MEDIUMTEXT;
   DECLARE tpf_service_jam_metrics_table MEDIUMTEXT;
   DECLARE tpf_mq_summary_metrics_table MEDIUMTEXT;
   DECLARE tpf_mq_queue_metrics_table MEDIUMTEXT;
   DECLARE tpf_mq_channel_metrics_table MEDIUMTEXT;
   DECLARE tpf_remote_data_store_metrics_table MEDIUMTEXT;
   DECLARE tpf_enh_chl_util_metrics_table MEDIUMTEXT;
   DECLARE tpf_lpar_util_metrics_table MEDIUMTEXT;
   DECLARE tpf_mpif_metrics_table MEDIUMTEXT;
   DECLARE tpf_shutdown1_metrics_table MEDIUMTEXT;
   DECLARE tpf_traces_metrics_table MEDIUMTEXT;
   DECLARE tpf_lodic_resources_metrics_table MEDIUMTEXT;
   DECLARE tpf_lodic_utilization_metrics_table MEDIUMTEXT;
   DECLARE tpf_time_slice_ecb_metrics_table MEDIUMTEXT;
   DECLARE tpf_dasd_metrics_table MEDIUMTEXT;
   DECLARE tpf_tpfdf_metrics_table MEDIUMTEXT;
   DECLARE tpf_common_deployment_metrics_table MEDIUMTEXT;
   DECLARE tpf_tcpip_metrics_table MEDIUMTEXT;
   DECLARE tpf_service_app_metrics_table MEDIUMTEXT;
   DECLARE tpf_trad_db_encryption_metrics_table MEDIUMTEXT;

-- Define a table for each CDC data type metrics table
   SET tpf_static_metrics_table = "
   <table>
   <tr><th>Metric</th>               <th>Description</th>                                     <th>Source</th></tr>
   <tr><td>altered</td>              <td>The last time any value in this data type was altered on the z/TPF system.</td> <td>N/A</td></tr>
   <tr><td>time</td>                 <td>The time of the most recent collection of this data type on the z/TPF system.</td> <td>N/A</td></tr>
   <tr><td>systems</td>              <td>The z/TPF processor ID.</td>                         <td>N/A</td></tr>
   <tr><td>subsystem_count</td>      <td>The number of subsystems that are defined.</td>      <td>MA0CNT (ms0at.mac)</td></tr>
   <tr><td>subsystem_user_count</td> <td>The number of subsystem users that are defined.</td> <td>UATBC </td></tr>
   <tr><td>usable_istreams</td>      <td>The number of usable I-streams.</td>                 <td>ISTACTIS (dctist.mac)</td></tr>
   <tr><td>cpu_serial</td>           <td>The CPU machine serial number.</td>                  <td>PI1MSLR (pi1dt.mac)</td></tr>
   <tr><td>cpu_machine_type</td>     <td>The machine type.</td>                               <td>PI1SYSIB111 (pi1dt.mac)</td></tr>
   <tr><td>tpf_release</td>          <td>The short product release name.</td>                 <td>ProdRel (tpfglb.mac)</td></tr>
   <tr><td>tpf_prod_update_level</td><td>The z/TPF product update level.</td>                 <td>CMMPUT</td></tr>
   <tr><td>tpf_complex_name</td>     <td>The z/TPF complex name.</td>                         <td>IC0COMP (ic0ck.mac)</td></tr>
   <tr><td>tpf_enterprise_name</td>  <td>The enterprise name.</td>                            <td>IC0ENTR (ic0ck.mac)</td></tr>
   </table>
   Collection Routine: cdcsta.asm<br>
   Sources:
   <ul>
      <li>ms0at.mac - Subsystem attribute table</li>
      <li>ic0ck.mac - Keypoint record I (CTKI)</li>
      <li>dctist.mac - I-stream status table</li>
      <li>pi1dt.mac - Processor ID table (PIDT)</li>
      <li>tpfglb.mac - Global assembler variables</li>
   </ul>
   Notes:
   <ul>
      <li>Because machine information changes infrequently, a new row is added to the database for this data type table only when collection data changes on the z/TPF system.  The timestimp of when a new row is added is set in the altered column.</li>
      <li>You cannot change the collection frequency of this data type with the ZRTMC command.</li>
   </ul>";

   SET tpf_dasd_queue_metrics_table = "
   <table>
   <tr><th>Metric</th>      <th>Description</th>                           <th>Source</th></tr>
   <tr><td>systems</td>     <td>The z/TPF processor ID.</td>               <td>N/A</td></tr>
   <tr><td>subsystems</td>  <td>The subsystem name.</td>                   <td>N/A</td></tr>
   <tr><td>dev_address</td> <td>The 2-byte symbolic device address.</td>   <td>MF1DAD (dctmfs.mac)</td></tr>
   <tr><td>dev_queue</td>   <td>The I/O queue length.</td>                 <td>MF1QCT (dctmfs.mac)</td></tr>
   <tr><td>dev_mod</td>     <td>The symbolic module number.</td>           <td>N/A</td></tr>
   <tr><td>time</td>        <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcdad.asm<br>
   Source: dctmfs.mac - Module file status table (MFST)<br>
   Note: Only devices with a queue length greater than zero appear in the table.";

   SET tpf_dasd_service_time_metrics_table = "
   <table>
   <tr><th>Metric</th>                   <th>Description</th>                                                           <th>Source</th></tr>
   <tr><td>systems</td>                  <td>The z/TPF processor ID.</td>                                               <td>N/A</td></tr>
   <tr><td>ssid</td>                     <td>The DASD subsystem ID.</td>                                                <td>MF1SSID (dctmfs.mac)</td></tr>
   <tr><td>average_service_time</td>     <td>The average DASD service time in microseconds.<br>
                                             This metric is calculated by looping through the MFST,accumulating the I/O time (MF2IOTIM) and I/O count (MF2IOCNT) values for each online device in the logical subsystem (LSS),<br>
                                             and then dividing the accumulated I/O time by the accumulated I/O count.</td> <td>MF2IOTIM (dctmfs.mac)<br>MF2IOCNT (dctmfs.mac)</td></tr>
   <tr><td>lowest_online_sda_on_lss</td> <td>The lowest online symbolic device address on the LSS.</td>                 <td>MF1DAD (dctmfs.mac)</td></tr>
   <tr><td>time</td>                     <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcdst.asm<br>
   Source: dctmfs.mac - Module file status table (MFST)<br>
   Notes: Only DASD subsystem IDs (ssid) with an average service time greater than zero appear in the table.";

   SET tpf_system_messages_metrics_table = "<table>
   <tr><th>Metric</th>                        <th>Description</th>                                                         <th>Source</th></tr>
   <tr><td>systems</td>                       <td>The z/TPF processor ID.</td>                                             <td>N/A</td></tr>
   <tr><td>high_spd_msgs_per_sec</td>         <td>The number of high-speed messages, also known as agent messages, per second.</td><td>CMMMSG</td></tr>
   <tr><td>tcpip_msgs_per_sec</td>            <td>The number of TCP/IP messages per second.</td>                           <td>CMMMSG</td></tr>
   <tr><td>routed_entries_per_sec</td>        <td>The number of messages routed to another host per second.</td>           <td>CMMMSG</td></tr>
   <tr><td>created_entries_per_sec</td>       <td>The number of entries that are created by CREMC, CREDC, CREXC, and CRETC macros per second.</td> <td>CMMMSG</td></tr>
   <tr><td>commits_per_sec</td>               <td>The number of tx_commit APIs issued per second.</td>                     <td>ICT_TM_CMTC (icrct.mac)</td></tr>
   <tr><td>number_of_istreams</td>            <td>The number of I-streams that the z/TPF system can be IPLed with.</td>    <td>&SAISTRM</td></tr>
   <tr><td>usable_istreams</td>               <td>The number of usable I-streams.</td>                                     <td>ISTUSEIS (dctist.mac)</td></tr>
   <tr><td>processor_util_pct</td>            <td>The average utilization percentage of all usable I-streams.</td>         <td>I-stream status table (dctist.mac)</td></tr>
   <tr><td>gp_processor_util_pct</td>         <td>The general processor (GP) utilization percentage.</td>                  <td>ISTCMGPUTL (dctist.mac)</td></tr>
   <tr><td>gp_max_cap_for_lpar_util_pct</td>  <td>The z/TPF cap for GP usage percentage.</td>                              <td>ISTCMGPCAP (dctist.mac)</td></tr>
   <tr><td>gp_number_of_engines</td>          <td>The number of GP engines used.</td>                                      <td>ISTCMGPENG (dctist.mac)</td></tr>
   <tr><td>istream_cap</td>                   <td>The I-stream cap value.</td>                                             <td>ISTCMCAPIS (dctist.mac)</td></tr>
   <tr><td>speed_boost</td>                   <td>Indicates whether speed boost was enabled (Y) or not (N) at IPL.</td>    <td>ISTCMF2_SPEEDBOOST_ON (dctist.mac)</td></tr>
   <tr><td>istream_boost</td>                 <td>Indicates whether I-stream boost was enabled (Y) or not (N) at IPL.</td> <td>ISTCMF2_ISBOOST_ON< (dctist.mac)</td></tr>
   <tr><td>time</td>                          <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcsym.asm<br>
   Sources:
   <ul>
      <li>CMMMSG - Message counts for data collection</li>
      <li>icrct.mac - Commit and Rollback control table</li>
      <li>dctist.mac - I-stream status table</li>
   </ul>";

   SET tpf_system_blocks_metrics_table = "
   <table>
   <tr><th>Metric</th>            <th>Description</th>                                                     <th>Source</th></tr>
   <tr><td>systems</td>           <td>The z/TPF processor ID.</td>                                         <td>N/A</td></tr>
   <tr><td>allocated</td>         <td>The number of the block type allocated on the z/TPF system.</td>     <td>CMMBLKS</td></tr>
   <tr><td>inuse</td>             <td>The number of the block type in use on the z/TPF system.</td>        <td>CMMBLKS</td></tr>
   <tr><td>inuse pct</td>         <td>The percentage of the block type in use on the z/TPF system.</td>    <td>CMMBLKS</td></tr>
   <tr><td>active_ecbs</td>       <td>The number of total active ECBs on all I-streams.</td>               <td>CM3NEBA</td></tr>
   <tr><td>low_priority_ecbs</td> <td>The number of low-priority utility ECBs on all I-streams.</td>       <td>ILPUL_HDR_ECB_CNT (ilpul.mac)</td></tr>
   <tr><td>time</td>              <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routines: cdcsyb.asm, cdcsyl.asm<br>
   Sources :
   <ul>
      <li>CMMBLKS - Table of valid block types</li>
      <li>CM3NEBA - Number of active ECBs per I-stream</li>
      <li>ilpul.mac - Low priority utility list</li>
   </ul>
   Notes:
   <ul>
      <li>Recovery log tracks are not a system block, but they are a resource that is used by z/TPF.  Use of recovery log tracks should be monitored.</li>
      <li>active_ecbs and low_priority_ecbs are collected by cdcsyl.asm.</li>
   </ul>";

   SET tpf_system_lists_metrics_table = "<table>
   <tr><th>Metric</th>       <th>Description</th>                                                <th>Source</th></tr>
   <tr><td>systems</td>      <td>The z/TPF processor ID.</td>                                    <td>N/A</td></tr>
   <tr><td>cross_list</td>   <td>The number of blocks currently queued on the cross list.        </td><td>NUMLC</td></tr>
   <tr><td>ready_list</td>   <td>The number of blocks currently queued on the ready list.        </td><td>NUMLC</td></tr>
   <tr><td>input_list</td>   <td>The number of blocks currently queued on the input list.        </td><td>NUMLC</td></tr>
   <tr><td>deferred_list</td><td>The number of blocks currently queued on the deferred list.     </td><td>NUMLC</td></tr>
   <tr><td>suspend_list</td> <td>The number of ECBs on the suspended ECBs list.</td>             <td>PFXSUSCT (dctpfx.mac)</td></tr>
   <tr><td>vct_list</td>     <td>The number of items on the virtual file access (VCT) list.</td> <td>PFXVCTCT (dctpfx.mac)</td></tr>
   <tr><td>time</td>         <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcsyl.asm";

   SET tpf_tape_metrics_table = "<table>
   <tr><th>Metric</th>         <th>Description</th>                      <th>Source</th></tr>
   <tr><td>systems</td>        <td>The z/TPF processor ID.</td>          <td>N/A</td></tr>
   <tr><td>subsystems</td>     <td>The subsystem name.</td>              <td>N/A</td></tr>
   <tr><td>bytes_per_sec</td>  <td>The number of bytes per second.</td>  <td>CPMTC0 (itstb.mac)</td></tr>
   <tr><td>writes_per_sec</td> <td>The number of writes per second.</td> <td>CPMTC1 (itstb.mac)</td></tr>
   <tr><td>queue_length</td>   <td>The module queue length.</td>         <td>CPMMQL (itstb.mac)</td></tr>
   <tr><td>device_address</td> <td>The symbolic device address.</td>     <td>CPMTDV (itstb.mac)</td></tr>
   <tr><td>time</td>           <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdctap.asm<br>
   Source: itstb.mac - Tape status table";

   SET tpf_vfa_metrics_table = "<table>
   <tr><th>Metric</th>                    <th>Description</th>                                                 <th>Source</th></tr>
   <tr><td>systems</td>                   <td>The z/TPF processor ID.</td>                                     <td>N/A</td></tr>
   <tr><td>subsystems</td>                <td>The subsystem name.</td>                                         <td>N/A</td></tr>
   <tr><td>data_reads_per_sec</td>        <td>The number of data finds from VFA (without I/O) per second.</td> <td>DCTDFIND (vf0ac.mac)</td></tr>
   <tr><td>finds_per_sec</td>             <td>The number of candidate finds that involved I/O per second.</td> <td>DCTFNDIO (vf0ac.mac)</td></tr>
   <tr><td>candidate_files_per_sec</td>   <td>The number of candidate files per second.</td>                   <td>DCTFILES (vf0ac.mac)</td></tr>
   <tr><td>noncandidate_files_per_sec</td><td>The number of noncandidate files per second.</td>                <td>DCTFILNC (vf0ac.mac)</td></tr>
   <tr><td>force_files_per_sec</td>       <td>The number of files, caused by VFA buffer being required for another record, per second.</td> <td>DCTFILIO (vf0ac.mac)</td></tr>
   <tr><td>file_immediate_per_sec</td>    <td>The number of candidate files immediate per second.</td>         <td>DCTFILNS (vf0ac.mac)</td></tr>
   <tr><td>buffer_unavailable_per_sec</td><td>The number of requests rejected because of no available buffer per second.</td> <td>DCTREJTS (vf0ac.mac)</td></tr>
   <tr><td>SLock_hits_per_sec</td>        <td>The number of VFA-S lock hits per second.</td>                   <td>DCTVFAS (vf0ac.mac)</td></tr>
   <tr><td>XLock_hits_per_sec</td>        <td>The number of VFA-X lock hits per second.</td>                   <td>DCTVFAX (vf0ac.mac)</td></tr>
   <tr><td>sync_misses_per_sec</td>       <td>The number of VFA synchronous misses per second.</td>            <td>DCTVFAM (vf0ac.mac)</td></tr>
   <tr><td>sync_content_per_sec</td>      <td>The number of synchronous contentions per second.</td>           <td>DCTVFAC (vf0ac.mac)</td></tr>
   <tr><td>cached_rht_hts_per_sec</td>    <td>The number of cached record hold table (RHT) lock hits per second.</td><td>DCTRHTH (vf0ac.mac)</td></tr>
   <tr><td>sync_age_outs_per_sec</td>     <td>The number of VFA synchronous age outs per second.</td>          <td>DCTVFAG (vf0ac.mac)</td></tr>
   <tr><td>dispense_4K_per_sec</td>       <td>The number of dispensed 4 KB buffers per second.</td>            <td>DCTDSP4K (vf0ac.mac)</td></tr>
   <tr><td>rht_content_per_sec</td>       <td>The number of RHT contentions per second.</td>                   <td>DCTRHTC (vf0ac.mac)</td></tr>
   <tr><td>time</td>                      <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcvfa.asm<br>
   Source: vf0ac.mac - VFA data collection table";

   SET tpf_istream_metrics_table = "<table>
   <tr><th>Metric</th>                <th>Description</th>                                                                          <th>Source</th></tr>
   <tr><td>systems</td>               <td>The z/TPF processor ID.</td>                                                              <td>N/A</td></tr>
   <tr><td>cross_list</td>            <td>The number of items on the cross list for the specific I-stream.</td>                     <td>NUMLC CLHCRS</td></tr>
   <tr><td>ready_list</td>            <td>The number of items on the ready list for the specific I-stream.</td>                     <td>NUMLC CLHRDY</td></tr>
   <tr><td>input_list</td>            <td>The number of items on the input list for the specific I-stream.</td>                     <td>NUMLC CLHINP</td></tr>
   <tr><td>deferred_list</td>         <td>The number of items on the deferred list for the specific I-stream.</td>                  <td>NUMLC CLHDEF</td></tr>
   <tr><td>suspend_list</td>          <td>The number of items on the suspend list for the specific I-stream.</td>                   <td>PFXSUSCT (dctpfx.mac)</td></tr>
   <tr><td>vct_list</td>              <td>The number of items on the virtual file access (VCT) list for the specific I-stream.</td> <td>PFXVCTCT (dctpfx.mac)</td></tr>
   <tr><td>active_ecbs</td>           <td>The number of active ECBs for this I-stream.</td>                                         <td>CM3NEBA</td></tr>
   <tr><td>routed_entries_per_sec</td><td>The number of routed entries per second.</td>                                             <td>CMMSWIS SAISTRM</td></tr>
   <tr><td>proc_util</td>             <td>The smoothed I-stream utilization.</td>                                                   <td>ISTUTIL (dctist.mac)</td></tr>
   <tr><td>proc_adj_util</td>         <td>The smoothed adjusted I-stream utilization. Does not include time spent while processing tasks that are deferred or delayed.</td><td>ISTADJU (dctist.mac)</td></tr>
   <tr><td>pseudo_util</td>           <td>The smoothed pseudo utilization for this I-stream.</td>                                   <td>ISTEXPUTL_VALUE (dctist.mac)</td></tr>
   <tr><td>low_priority_util</td>     <td>The smoothed low-priority utility utilization for this I-stream.</td>                     <td>ISTEXLPUL_VALUE (dctist.mac)</td></tr>
   <tr><td>time</td>                  <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcist.asm<br>
   Sources:
   <ul>
      <li>CMMSWIS - Data collection SWISC counters</li>
      <li>CM3NEBA - Number of active ECBs per I-stream</li>
      <li>dctist.mac - I-stream status table</li>
      <li>dctpfx.mac - The prefix area for each CPU</li>
   </ul>";

   SET tpf_subsystem_users_metrics_table = "<table>
   <tr><th>Metric</th>                  <th>Description</th></tr>
   <tr><td>systems</td>                 <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>              <td>The subsystem name.</td></tr>
   <tr><td>subsystem_user_name</td>     <td>The subsystem user name.</td></tr>
   <tr><td>high_spd_msgs_per_sec</td>   <td>The number of high-speed messages, also known as agent messages, per second.</td></tr>
   <tr><td>routed_entries_per_sec</td>  <td>The number of messages routed to another host per second.</td></tr>
   <tr><td>created_entries_per_sec</td> <td>The number of entries that are created by CREMC, CREDC, CREXC, and CRETC macros per second.</td></tr>
   <tr><td>ecb_exits_per_sec</td>       <td>The number of ECB exits per second.</td></tr>
   <tr><td>time</td>                    <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: cdcsub.asm<br>
   Source: The source of this data type is the subsystem user table (SSUT) mapped by ms0ut.mac.";

   SET tpf_subsystems_metrics_table = "<table>
   <tr><th>Metric</th>                  <th>Description</th></tr>
   <tr><td>systems</td>                 <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>              <td>The subsystem name.</td></tr>
   <tr><td>high_spd_msgs_per_sec</td>   <td>The number of high-speed messages, also known as agent messages, per second.</td></tr>
   <tr><td>routed_entries_per_sec</td>  <td>The number of messages routed to another host per second.</td></tr>
   <tr><td>created_entries_per_sec</td> <td>The number of entries that are created by CREMC, CREDC, CREXC, and CRETC macros per second.</td></tr>
   <tr><td>ecb_exits_per_sec</td>       <td>The number of ECB exits per second.</td></tr>
   <tr><td>time</td>                    <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: cdcsub.asm<br>
   Source: The source of this data type is the subsystem user table (SSUT) mapped by ms0ut.mac.";

   SET tpf_subsystems_utility_switches_metrics_table = "<table>
   <tr><th>Metric</th>                <th>Description</th></tr>
   <tr><td>altered</td>               <td>The last time any value in this data type was altered on the z/TPF system.</td></tr>
   <tr><td>time</td>                  <td>The time of the most recent collection of this data type on the z/TPF system.</td></tr>
   <tr><td>recoup</td>                <td>Recoup</td></tr>
   <tr><td>selective_file_trace</td>  <td>Selective file trace (SFT)</td></tr>
   <tr><td>RCS_file</td>              <td>Record cache subsystem (RCS) file</td></tr>
   <tr><td>RCS_enable</td>            <td>Record cache subsystem (RCS) enable</td></tr>
   <tr><td>RCS_implement</td>         <td>Record cache subsystem (RCS) implement</td></tr>
   <tr><td>data_col</td>              <td>Data collection</td></tr>
   <tr><td>capture</td>               <td>Capture</td></tr>
   <tr><td>restore</td>               <td>Restore</td></tr>
   <tr><td>SON_copy</td>              <td>System ordinal number (SON) copy</td></tr>
   <tr><td>trap</td>                  <td>z/TPF software profiler</td></tr>
   <tr><td>DB_reorg</td>              <td>Online database reorganization (ODBR)</td></tr>
   <tr><td>CCP</td>                   <td>Communications control program (CCP) trace</td></tr>
   <tr><td>link_trace</td>            <td>Link trace</td></tr>
   </table>
   Collection Routine: cdcsym.asm<br>
   Notes:
   <ul>
      <li>Switches are set by the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zca-tpf-swchc-set-clear-test-lethal-utility-switch'>tpf_swchc</a> function.</li>
      <li>Because switches change infrequently, a new row is added to the database for this data type table only when collection data changes on z/TPF.  The timestamp of when a new row is added is set in the altered column.</li>
   </ul>";

   SET tpf_dasd_io_metrics_table = "<table>
   <tr><th>Metric</th>              <th>Description</th>                                                     <th>Source</th></tr>
   <tr><td>systems</td>             <td>The z/TPF processor ID.</td>                                         <td>N/A</td></tr>
   <tr><td>subsystems</td>          <td>The subsystem name.</td>                                             <td>N/A</td></tr>
   <tr><td>logical_device_type</td> <td>The device name defined in sip.asm.</td>                             <td>N/A</td></tr>
   <tr><td>io_per_sec</td>          <td>The number of data writes and reads that are issued per second.<br>
                                        This metric does not include locking I/Os in a loosely coupled environment.</td> <td>MF2DWT + MF2DRD (dctmfs.mac)</td></tr>
   <tr><td>writes_per_sec</td>      <td>The number of data writes that are issued per second.</td>           <td>MF2DWT (dctmfs.mac)</td></tr>
   <tr><td>reads_per_sec</td>       <td>The number of data reads that are issued per second.</td>            <td>MF2DRD (dctmfs.mac)</td></tr>
   <tr><td>queue_length</td>        <td>The number of I/O control blocks (IOBs).</td>                        <td>MF2QCT (dctmfs.mac)</td></tr>
   <tr><td>time</td>                <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcdap.asm<br>
   Source: dctmfs.mac - Module file status table (MFST)";

   SET tpf_dasd_pool_metrics_table = "<table>
   <tr><th>Metric</th>              <th>Description</th>                                                            <th>Source</th></tr>
   <tr><td>systems</td>             <td>The z/TPF processor ID.</td>                                                <td>N/A</td></tr>
   <tr><td>subsystems</td>          <td>The subsystem name.</td>                                                    <td>N/A</td></tr>
   <tr><td>logical_device_type</td> <td>The device name defined in sip.asm.</td>                                    <td>N/A</td></tr>
   <tr><td>pool_type</td>           <td>One of the following pool types:<br>
                                        SST - Small, short-term pool<br>
                                        SLT - Small, long-term pool<br>
                                        SDP - long-term duplicate pool<br>
                                        LST - Large, short-term pool<br>
                                        LLT - Large, long-term pool<br>
                                        LDP - Large, long-term duplicated pool<br>
                                        4ST - 4 KB short-term pool<br>
                                        4S6 - 4 KB short-term FARF6 pool<br>
                                        4LT - 4 KB long-term pool<br>
                                        4DP - 4 KB long-term duplicated pool<br>
                                        4D6 - 4 KB long-term duplicated FARF6 pool<br>
                                        For more information, see the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=attributes-pool-record-types'>Pool Record Types</a> topic in z/TPF product documentation.</td>                         <td>N/A</td></tr>
   <tr><td>set_size</td>            <td>The number of directories in the set.<br>
                                        For more information, see the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zgxxx-zgfsp-set-change-size-pool-directory-set'>ZGFSP SET</a></td> <td>ICY7NVD (icy7pr.mac)</td></tr>
   <tr><td>reorder_time</td>        <td>The reorder time in milliseconds.</td>                                      <td>ICY7TLR & ICY7TTR (icy7pr.mac)</td></tr>
   <tr><td>dispensed_per_sec</td>   <td>The number of addresses dispensed from the pool section per second.</td>    <td>ICY7DSP  (icy7pr.mac)</td></tr>
   <tr><td>returned_per_sec</td>    <td>The number of returned addresses that hit the pool section per second.</td> <td>ICY7RET (icy7pr.mac)</td></tr>
   <tr><td>time</td>                <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcdap.asm<br>
   Source: icy7pr.mac - Pool directory set control area";

   SET tpf_service_jam_metrics_table = "<table>
   <tr><th>Metric</th>                      <th>Description</th></tr>
   <tr><td>systems</td>                     <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>                  <td>The subsystem name.</td></tr>
   <tr><td>jam_name</td>                    <td>The name of the JAM that is defined in a JAM descriptor.</td></tr>
   <tr><td>internal_operations_per_sec</td> <td>The number of calls that are made to a JAM locally by your z/TPF system per second.</td></tr>
   <tr><td>external_operations_per_sec</td> <td>The number of calls that are made to a JAM by a remote system per second.</td></tr>
   <tr><td>total_operations_per_sec</td>    <td>The number of internal and external operations per second.</td></tr>
   <tr><td>threads_in_use</td>              <td>The number of threads in use.</td></tr>
   <tr><td>threads_in_use_highwater</td>    <td>The highest number of threads in use since the JAM was started.</td></tr>
   <tr><td>threads_config</td>              <td>The maximum number of threads that can be configured for the JAM.</td></tr>
   <tr><td>time</td>                        <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: cdc_collect_service_jam (cdcp_collect.cpp)";

   SET tpf_mq_summary_metrics_table = "<table>
   <tr><th>Metric</th>                           <th>Description</th></tr>
   <tr><td>systems</td>                          <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>                       <td>The subsystem name.</td></tr>
   <tr><td>queue_manager</td>                    <td>The queue manager name.</td></tr>
   <tr><td>queues</td>                           <td>The total number of queues.</td></tr>
   <tr><td>local_queues</td>                     <td>The number of local queues.</td></tr>
   <tr><td>transmit_queues</td>                  <td>The number of transmission queues.</td></tr>
   <tr><td>remote_queues</td>                    <td>The number of remote queues.</td></tr>
   <tr><td>queue_opens_per_sec</td>              <td>The number of MQOPEN calls per second.</td></tr>
   <tr><td>queue_closes_per_sec</td>             <td>The number of MQCLOSE calls per second.</td></tr>
   <tr><td>queue_sweeps_per_sec</td>             <td>The number of queue sweeps per second.</td></tr>
   <tr><td>queue_msgs_persistent_per_sec</td>    <td>The number of persistent messages per second.</td></tr>
   <tr><td>queue_msg_length_persistent</td>      <td>The number of persistent message bytes.</td></tr>
   <tr><td>queue_msgs_nonpersistent_per_sec</td> <td>The number of nonpersistent messages per second.</td></tr>
   <tr><td>queue_msg_length_nonpersistent</td>   <td>The number of nonpersistent message bytes.</td></tr>
   <tr><td>channels</td>                         <td>The number of channels.</td></tr>
   <tr><td>receiver_channels</td>                <td>The number of receiver channels.</td></tr>
   <tr><td>sender_channels</td>                  <td>The number of sender channels.</td></tr>
   <tr><td>server_channels</td>                  <td>The number of sever connection channels.</td></tr>
   <tr><td>channel_msgs_per_sec</td>             <td>The number of messages per second.</td></tr>
   <tr><td>channel_bytes_recvd_per_sec</td>      <td>The number of bytes received per second.</td></tr>
   <tr><td>channel_bytes_sent_per_sec</td>       <td>The number of bytes sent per second.</td></tr>
   <tr><td>channel_bytes_handled_per_sec</td>    <td>The number of bytes sent and received per second.</td></tr>
   <tr><td>channel_batch_size</td>               <td>The negotiated batch size. This value is 0 for server connection channels.</td></tr>
   <tr><td>channel_active_instances</td>         <td>The number of clients with a connection to a server connection channel.</td></tr>
   <tr><td>channel_last_batch_size</td>          <td>The last batch size.  This value is 0 for server connection channels.</td></tr>
   <tr><td>channel_instances_highwater</td>      <td>The high-water mark of clients with a connection to a server connection channel.</td></tr>
   <tr><td>time</td>                             <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: CDC_summary (cmqcol.cpp)<br>
   Note: This pane presents a summary of every local queue entry in the queue hash table and every channel entry in the channel hash table.";

   SET tpf_mq_channel_metrics_table = "<table>
   <tr><th>Metric</th>                <th>Description</th></tr>
   <tr><td>systems</td>               <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>            <td>The subsystem name.</td></tr>
   <tr><td>queue_manager</td>         <td>The queue manager name.</td></tr>
   <tr><td>channel_type</td>          <td>The channel type:<br>S - Sender channel<br>R - Receiver channel<br>V - Server connection channel</td></tr>
   <tr><td>msgs_per_sec</td>          <td>The number of messages per second.</td></tr>
   <tr><td>bytes_recvd_per_sec</td>   <td>The number of bytes received per second.</td></tr>
   <tr><td>bytes_sent_per_sec</td>    <td>The number of bytes sent per second.</td></tr>
   <tr><td>bytes_handled_per_sec</td> <td>The number of bytes sent and received per second.</td></tr>
   <tr><td>batch_size</td>            <td>The negotiated batch size. This value is 0 for server connection channels.</td></tr>
   <tr><td>active_instances</td>      <td>The number of clients with a connection to a server connection channel.</td></tr>
   <tr><td>last_batch_size</td>       <td>The last batch size.  This value is 0 for server connection channels.</td></tr>
   <tr><td>instances_highwater</td>   <td>The high-water mark of clients with a connection to a server connection channel.</td></tr>
   <tr><td>time</td>                  <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: CDC_Cdetail (cmqcol.cpp)";

   SET tpf_mq_queue_metrics_table = "<table>
   <tr><th>Metric</th>                     <th>Description</th></tr>
   <tr><td>systems</td>                    <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>                 <td>The subsystem name.</td></tr>
   <tr><td>queue_manager</td>              <td>The queue manager name.</td></tr>
   <tr><td>queue_name</td>                 <td>The queue name.</td></tr>
   <tr><td>queue_type</td>                 <td>The queue type:<br>L - Local queue<br>X - Transmission queue<br>R - Remote queue</td></tr>
   <tr><td>opens_per_sec</td>              <td>The number of MQOPEN calls per second.</td></tr>
   <tr><td>closes_per_sec</td>             <td>The number of MQCLOSE calls per second.</td></tr>
   <tr><td>sweeps_per_sec</td>             <td>The number of queue sweeps per second.</td></tr>
   <tr><td>msgs_persistent_per_sec</td>    <td>The number of persistent messages per second.</td></tr>
   <tr><td>msg_length_persistent</td>      <td>The number of persistent message bytes.</td></tr>
   <tr><td>msgs_nonpersistent_per_sec</td> <td>The number of nonpersistent messages per second.</td></tr>
   <tr><td>msg_length_nonpersistent</td>   <td>The number of nonpersistent message bytes.</td></tr>
   <tr><td>time</td>                       <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: CDC_Qdetail (cmqcol.cpp)";

   SET tpf_remote_data_store_metrics_table = "<table>
   <tr><th>Metric</th>             <th>Description</th>                                                          <th>Source</th></tr>
   <tr><td>systems</td>            <td>The z/TPF processor ID.</td>                                              <td>N/A</td></tr>
   <tr><td>subsystems</td>         <td>The subsystem name.</td>                                                  <td>N/A</td></tr>
   <tr><td>rds_name</td>           <td>The remote data store name.</td>                                          <td>rdsName (irds.h)</td></tr>
   <tr><td>rds_mode</td>           <td>The mode:<br>LOCAL - Local data only (no remote data)<br>COPY - Keep both local and remote data and always read local data<br>VERIFY - Keep both local and remote data and read remote data with message digest verification<br>REMOTE - Remote data only (no local data)</td> <td>mode (irds.h)</td></tr>
   <tr><td>reader_active_ecbs</td> <td>The number of active reader ECBs.</td>                                    <td>readerActiveECBs (irds.h)</td></tr>
   <tr><td>reader_highwater</td>   <td>The active reader ECB high-water count.</td>                              <td>readerHWtr (irds.h)</td></tr>
   <tr><td>reader_max</td>         <td>The maximum allowed reader ECBs.</td>                                     <td>readerMax (irds.h)</td></tr>
   <tr><td>writer_active_ecbs</td> <td>The number of active writer ECBs.</td>                                    <td>writerActiveECBs (irds.h)</td></tr>
   <tr><td>writer_highwater</td>   <td>The active writer ECB high-water count.</td>                              <td>writeerHWtr (irds.h)</td></tr>
   <tr><td>writer_max</td>         <td>The maximum allowed writer ECBs.</td>                                     <td>writerMax (irds.h)</td></tr>
   <tr><td>reads_per_sec</td>      <td>The number of successful reads per second.</td>                           <td>readCount (irds.h)</td></tr>
   <tr><td>creates_per_sec</td>    <td>The number of successful asynchronous creates per second.</td>            <td>createCount (irds.h)</td></tr>
   <tr><td>updates_per_sec</td>    <td>The number of successful asynchronous updates per second.</td>            <td>updateCount (irds.h)</td></tr>
   <tr><td>deletes_per_sec</td>    <td>The number of successful asynchronous deletes per second.</td>            <td>deleteCount (irds.h)</td></tr>
   <tr><td>total_ops_per_sec</td>  <td>The total number of reads, creates, updates, and deletes per second.</td> <td>N/A</td></tr>
   <tr><td>read_reject_at_max</td> <td>The number of reads rejected because of the maximum reader count.</td>    <td>readRejectAtMax (irds.h)</td></tr>
   <tr><td>timeouts_per_sec</td>   <td>The number of timeouts per second.</td>                                   <td>timeout (irds.h)</td></tr>
   <tr><td>TPF_error</td>          <td>The number of failures other than timeouts on the z/TPF system.</td>      <td>TPFerror (irds.h)</td></tr>
   <tr><td>server_error_mongo</td> <td>The number of failures other than timeouts on MongoDB.</td>               <td>serverError (irds.h)</td></tr>
   <tr><td>time</td>               <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: tpf_collect_rds_data (jcia_collect.c)<br>
   Source: irds.h -  Structure definitions for the remote data store facility<br>
   Note: The reader_max and writer_max fields are not variables; they are constants that are established for each data store by using the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zrfxx-zrdsc-manage-remote-data-stores'>ZRDSC</a> command.";

   -- The source of the Description information is the "Channel path reports" topic in z/TPF product documentation.
   SET tpf_enh_chl_util_metrics_table = "<table>
   <tr><th>Metric</th>                 <th>Description</th></tr>
   <tr><td>systems</td>                <td>The z/TPF processor ID.</td></tr>
   <tr><td>channel_id</td>             <td>The channel path ID (CHPID).</td></tr>
   <tr><td>active</td>                 <td>Indicates whether the channel is active (Y) or not active (N).<br>If the value is N, all metrics are 0.</td></tr>
   <tr><td>type</td>                   <td>The channel type abbreviation.<br>See the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=analysis-channel-path-reports'>Channel path reports</a> topic in z/TPF product documentation for a table of all channel types and abbreviations.<br></td></tr>
   <tr><td>shared</td>                 <td>Indicates whether the channel is dedicated (N) or shared (Y) between logical partitions (LPARs).  Shared channels might not have the same CHPID, but they share the same physical path.</td></tr>
   <tr><td>channel_group</td>          <td>The channel measurement group (CMG).<br>
                                           If the value is 1, only the part_util_pct and total_util_pct metrics contain values, and all other metrics are 0.<br>
                                           If the value is 2, all metrics contain values.</td></tr>
   <tr><td>part_util_pct</td>          <td>The channel path utilization percentage for the partition where the data is collected.</td></tr>
   <tr><td>total_util_pct</td>         <td>The total accumulated channel path utilization percentage for the entire central processor complex (CPC).</td></tr>
   <tr><td>bus_util_pct</td>           <td>The percentage of the bus cycles used for I/O operations on the associated channel path.</td></tr>
   <tr><td>part_mb_read_per_sec</td>   <td>The data transfer rate from the control unit to the channel for the partition (MB per second).</td></tr>
   <tr><td>total_mb_read_per_sec</td>  <td>The data transfer rate from the control unit to the channel for the entire CPC (MB per second).</td></tr>
   <tr><td>part_mb_write_per_sec</td>  <td>The data transfer rate from the channel to the control unit for the partition (MB per second).</td></tr>
   <tr><td>total_mb_write_per_sec</td> <td>The data transfer rate from the channel to the control unit for the entire CPC (MB per second).</td></tr>
   <tr><td>time</td>                   <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Notes:
   <ul>
      <li>This data type is not collected when the z/TPF system is running as a VM guest.</li>
      <li>This data type is collected by noship code.</li>
   </ul>";

   SET tpf_lpar_util_metrics_table = "<table>
   <tr><th>Metric</th>          <th>Description</th>                                                                        <th>Source</th></tr>
   <tr><td>systems</td>         <td>The z/TPF processor ID.</td>                                                            <td>N/A</td></tr>
   <tr><td>lpar_name</td>       <td>The name of the logical partition (LPAR).</td>                                          <td>ilpe_name (c_ilpst.h)</td></tr>
   <tr><td>part_number</td>     <td>The sequential partition number.</td>                                                   <td>ilpe_pn (c_ilpst.h)</td></tr>
   <tr><td>type</td>            <td>The LPAR type: S - Shared<br>
                                    Dedicated and Inactive LPARs are not shown.</td>                                        <td>ilpe_ind (c_ilpst.h)</td></tr>
   <tr><td>shared_cpus</td>     <td>The number of general CPUs that are defined for this partition in the LPAR activation profile.</td>                                             <td>ilpe_scpus (c_ilpst.h)</td></tr>
   <tr><td>recognized_cpus</td> <td>The number of recognized CPUs.</td>                                                     <td>ilpe_cpus (c_ilpst.h)</td></tr>
   <tr><td>weight</td>          <td>The weight that is assigned to this LPAR.</td>                                          <td>ilpe_weight (c_ilpst.h)</td></tr>
   <tr><td>util</td>            <td>The CPU utilization of this partition.<br>
                                    This is calculated by the calculateUtilization function in c_ilpst.h.</td>              <td>c_ilpst.h</td></tr>
   <tr><td>util_vs_cap</td>     <td>The CPU utilization versus the cap for this partition.<br>
                                    This is calculated by dividing the results of the calculateCecUsePercentage / calculateWeightPercentage functions in c_ilpst.h.<br>
                                    If the LPAR is not capped, this value is set to 0.</td>                                 <td>c_ilpst.h</td></tr>
   <tr><td>machine</td>         <td>The percentage of machine use of all the shared partitions that use general CPUs.<br>
                                    This is calculated by the calculateCecUsePercentage function in c_ilpst.h.</td>         <td>c_ilpst.h</td></tr>
   <tr><td>entitle</td>         <td>The percentage of use versus the entitled share for this shared partition.<br>
                                    This is calculated by the calculateEntitlePercentage function in c_ilpst.h.<br>
                                    If a shared partition is using more than its entitlement, this percentage is greater than 100 percent.</td> <td>c_ilpst.h</td></tr>
   <tr><td>time</td>            <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdc_add_lpar_utilization_data (cdcp_add.cpp)<br>
   Notes:<br>
   <ul>
   <li>Each z/TPF system collects data for all LPARs on the machine, including LPARs that are not running z/TPF.</li>
   <li>LPARs are not shown if any of the following are true:
     <ul>
       <li>There are no shared CPUs that are general processors (GPs).</li>
       <li>The total weight of all shared LPARs that use GPs is 0.</li>
       <li>The number of shared CPUs for the current LPAR is 0.</li>
     </ul>
   </li>
   </ul>";

   SET tpf_mpif_metrics_table = "<table>
   <tr><th>Metric</th>                 <th>Description</th>                                                       <th>Source</th></tr>
   <tr><td>systems</td>                <td>The z/TPF processor ID.</td>                                           <td>N/A</td></tr>
   <tr><td>path_name</td>              <td>The path name.</td>                                                    <td>PDTPN (dctpdt.mac)</td></tr>
   <tr><td>device_name</td>            <td>The generic device name that begins with the $ character.</td>         <td>PDT308N (dctpdt.mac)</td></tr>
   <tr><td>class_name</td>             <td>The alphanumeric character code that identifies the path class.</td>   <td>PDTPCL (dctpdt.mac)</td></tr>
   <tr><td>reads_per_sec</td>          <td>The number of reads per second.</td>                                   <td>CWADCNT (dctcwa.mac)</td></tr>
   <tr><td>writes_per_sec</td>         <td>The number of writes per second.</td>                                  <td>CWADCNT (dctcwa.mac)</td></tr>
   <tr><td>bytes_sent_per_sec</td>     <td>The number of data bytes sent per second.</td>                         <td>PDTNCHR (dctpdt.mac)</td></tr>
   <tr><td>msgs_sent_per_sec</td>      <td>The number of messages sent per second.</td>                           <td>PDTNSND (dctpdt.mac)</td></tr>
   <tr><td>queue_overruns_per_sec</td> <td>The number of queue overruns per second.</td>                          <td>PDTORUN (dctpdt.mac)</td></tr>
   <tr><td>time</td>                   <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcmpf.asm<br>
   Sources:
   <ul>
      <li>dctpdt.mac -  Path definition table (PDT)</li>
      <li>dctcwa.mac -  Channel command word area (CWA)</li>
   </ul>";

   -- The source of the Description is the ZACLV command z/TPF product documentation.
   SET tpf_shutdown1_metrics_table = "<table>
   <tr><th>Metric</th>            <th>Description</th>            <th>Source</th></tr>
   <tr><td>systems</td>           <td>The z/TPF processor ID.</td> <td>N/A</td></tr>
   <tr><td>shutdown_type</td>     <td>The shutdown type:<br>
                                      Input List<br>
                                      Deferred List<br>
                                      Time Available Supervisor<br>
                                      CREMC Macro<br>
                                      CREDC Macro<br>
                                      CREXC Macro<br>
                                      3270 Local Input</td> <td>N/A</td></tr>
   <tr><td>reason</td>            <td>Shutdown levels set by the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zaxxx-zaclv-alter-cpu-loop-create-macro-control-levels'>ZACLV</a> command:<br>
                                      MXECBS - Greater than the maximum number of entry control blocks (ECBs) that can be in the z/TPF system<br>
                                      MNFRAI - Less than the minimum number of frames that must be available to service the input list<br>
                                      MNCMBI - Less than the minimum number of common blocks that must be available to service the input list<br>
                                      MNFR1MBI - Less than the minimum number of 1-MB frames that must be available to service the input list<br>
                                      MNECBI - Less than the minimum number of ECBs that must be available to service the input list<br>
                                      MNSWBI - Less than the minimum number of system work blocks (SWBs) that must be available to service the input list<br>
                                      MNIOBI - Less than the minimum number of I/O blocks that must be available to service the input list<br>
                                      MNXWBI - Less than the minimum number of extended system work blocks (XWBs) that must be available to service the input list<br>
                                      MNRLOGI - Less than the minimum number of recovery log tracks that must be available to service the input list<br>
                                      MXECBD - Greater than the maximum number of ECBs in the z/TPF system to service the deferred list<br>
                                      MNXWBM - The CREMC macro was issued with less than the required XWBs available<br>
                                      *MXITEMR - The CREMC macro was issued with too many items on the ready list<br>
                                      MNXWBD - The CREDC macro was issued with less than the required XWBs available<br>
                                      MNXWBX - The CREXC macro was issued with less than the required XWBs available<br>
                                      MNFRAT - Less than the minimum number of frames that must be available to activate the time available supervisor<br>
                                      MNECBT - Less than the minimum number of ECBs that must be available to activate the time available supervisor<br>
                                      MNCMBT - Less than the minimum number of common blocks that must be available to activate the time available supervisor<br>
                                      MNFR1MBT - Less than the minimum number of 1-MB frames that must be available to activate the time available supervisor<br>
                                      MNSWBT - Less than the minimum number of system work blocks (SWBs) that must be available to activate the time available supervisor<br>
                                      MNXWBT - Less than the minimum number of extended system work blocks (XWBs) that must be available to activate the time available supervisor<br>
                                      MNRLOGT - Less than the minimum number of recovery log tracks that must be available to activate the time available supervisor<br>
                                      *MXITEMI - Greater than the 3270 local shutdown level [CTKA (CK1KE)]<br>
                                      * Levels denoted by an asterisk are not set by the ZACLV command.</td> <td>ZACLV Command</td></tr>
   <tr><td>shutdown_occurred</td> <td>A value of 1 indicates that a shutdown occurred as described by the shutdown_type and reason columns.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdcsym.asm";

   SET tpf_traces_metrics_table = "<table>
   This dashboard displays system trace options and system generation option traces.<br>
   <ul>
   <li>System trace options settings are controlled by the ZSTRC command.<br>For information about the various system traces, see the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zsxxx-zstrc-alter-display-system-trace-options'>ZSTRC</a> command documentation.</li>
   <li>System generation option traces, lock_trace (LTRCE) and VFA (DEBUGV), are controlled by the ZSYSG ALTER command.<br>For information about the system generation option traces, see the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=zsxxx-zsysg-alteralter-system-generation-option-values'>ZSYSG ALTER</a> command documentation.</li>
   </ul>
   Collection Routine: cdc_add_system_messages_data (cdcp_add.cpp)<br>
   Note: Because traces change infrequently, a new row is added to the database for this data type table only when collection data changes on z/TPF.  The timestamp of when a new row is added is set in the altered column.";


   SET tpf_lodic_resources_metrics_table = "<table>
   <tr><th>Metric</th>                  <th>Description</th>                                  <th>Source</th></tr>
   <tr><td>systems</td>                 <td>The z/TPF processor ID.</td>                      <td>N/A</td></tr>
   <tr><td>resource_priority_class</td> <td>The resource priority class name.</td>            <td>(ilodic.mac)</td></tr>
   <tr><td>ecbs_using_class</td>        <td>The number of ECBs that are using the class.</td> <td>(ilodic.mac)</td></tr>
   <tr><td>time</td>                    <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdclod.asm<br>
   Source: ilodic.mac -  LODIC information area";

   SET tpf_lodic_utilization_metrics_table = "<table>
   <tr><th>Metric</th>                     <th>Description</th>                                  <th>Source</th></tr>
   <tr><td>systems</td>                    <td>The z/TPF processor ID.</td>                      <td>N/A</td></tr>
   <tr><td>utilization_priority_class</td> <td>The utilization priority class name.</td>         <td>LodicUtilClassTableEntry.className (cdcw.c)</td></tr>
   <tr><td>ecbs_using_class</td>           <td>The number of ECBs that are using the class.</td> <td>LodicUtilClassTableEntry.ECBCount (cdcw.c)</td></tr>
   <tr><td>time</td>                       <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: collect_LodicUtilClass (cdcw.c)";

   SET tpf_time_slice_ecb_metrics_table = "<table>
   <tr><th>Metric</th>                   <th>Description</th>                                       <th>Source</th></tr>
   <tr><td>systems</td>                  <td>The z/TPF processor ID.</td>                           <td>CN/A</td></tr>
   <tr><td>time_sliceable_ecbs</td>      <td>The number of ECBs that run with time slicing.</td>     <td>ITSTNUM (itsnt.mac)</td></tr>
   <tr><td>time_sliced_ecbs_per_sec</td> <td>The number of time slices per second.</td>             <td>ITSSLICE (itsnt.mac)</td></tr>
   <tr><td>time</td>                     <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdclod.asm<br>
   Sources:
   <ul>
      <li>itsnt.mac -  The time slice name table</li>
   </ul>";

   SET tpf_dasd_metrics_table = "<table>
   <tr><th>Metric</th>                       <th>Description</th>                                                                 <th>Source</th></tr>
   <tr><td>systems</td>                      <td>The z/TPF processor ID.</td>                                                     <td>N/A</td></tr>
   <tr><td>dev_indicator</td>                <td>The bits of the indicator:<br>
                                                 MF0MNT   0x80 mount in progress<br>
                                                 MF0CPALL 0x40 copy all in progress<br>
                                                 MF0CPDP  0x20 copy Dupe in progress<br>
                                                 MF0CPY   0x10 module is in copy state<br>
                                                 MF0OFF   0x04 module is off-line<br>
                                                 MF0ELF   0x08 locking for module on CF<br>
                                                 MF0DUP0  0x02 module is duplicated (wholly or selectively)<br>
                                                 MF0DUP1  0x01 module is selectively duplicated<br>
                                                 MF0DUP   (MF0DUP0+MF0DUP1)<br>
                                                 MF0ONL   (MF0MNT+MF0OFF) any flag on says module not online<br>
                                                 MF0AVL   (MF0MNT+MF0OFF) any flag on mod is not available for normal find i/o</td> <td>mf0ind0 (c_mfst.h)</td></tr>
   <tr><td>symbolic_module</td>              <td>The symbolic module number based from the first symbolic module number.</td>     <td>rt_first_symmod (c_conk.h)</td></tr>
   <tr><td>volume_number</td>                <td>The volume serial number.</td>                                                   <td>mf2vsn (c_mfst.h)</td></tr>
   <tr><td>type</td>                         <td>The symbolic device name.  For example, DEVA, DEVB, DEVC, DEVD.</td>             <td>CY5DCn where n is one of A, B, C, D (cy5gt.mac)</td></tr>
   <tr><td>address</td>                      <td>The device address.</td>                                                         <td>mf1dad (c_mfst.h)</td></tr>
   <tr><td>dasd_subsystem_id</td>            <td>The DASD subsystem ID.</td>                                                      <td>mf1ssid (c_mfst.h)</td></tr>
   <tr><td>module_queue</td>                 <td>The number of I/O control blocks.</td>                                           <td>mf2qct (c_mfst.h)</td></tr>
   <tr><td>data_reads_per_sec</td>           <td>The number of data reads issued per second.</td>                                 <td>mf2drd (c_mfst.h)</td></tr>
   <tr><td>data_writes_per_sec</td>          <td>The number of data writes issued per second.</td>                                <td>mf2dwt (c_mfst.h)</td></tr>
   <tr><td>accumulated_response_time</td>    <td>The accumulated response time.</td>                                              <td>mf2iotim (c_mfst.h)</td></tr>
   <tr><td>response_time_call_per_sec</td>   <td>The number of times iotim was incremented per second.</td>                       <td>mf2iocnt (c_mfst.h)</td></tr>
   <tr><td>sioscs_locks_per_sec</td>         <td>The number of lock only SIOSC macros per second.</td>                                  <td>mf2los (c_mfst.h)</td></tr>
   <tr><td>sioscs_unlocks_per_sec</td>       <td>The number of unlock only SIOSC macros per second.</td>                                <td>mf2uos (c_mfst.h)</td></tr>
   <tr><td>mplf_lock_contention_per_sec</td> <td>The number of delayed holds per second.</td>                                     <td>mf2llf (c_mfst.h)</td></tr>
   <tr><td>fdctc_per_sec</td>                <td>The number of completed file data chain transfers (FDCTC macro) per second.</td> <td>mf2fdct (c_mfst.h)</td></tr>
   <tr><td>time</td>                         <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdc_collect_dasd_metrics (cdcp_collect.cpp)<br>
   Sources:
   <ul>
      <li>c_conk.h - CONK table</li>
      <li>c_mfst.h - Module file status table structure definitions</li>
      <li>cy5gt.mac -  SON pool management global table</li>
   </ul>";

   SET tpf_tpfdf_metrics_table = "<table>
   <tr><th>Metric</th>                 <th>Description</th>                                                                          <th>Source</th></tr>
   <tr><td>systems</td>                <td>The z/TPF processor ID.</td>                                                              <td>N/A</td></tr>
   <tr><td>file_id</td>                <td>The z/TPFDF file ID in hexadecimal format.                                                </td> <td>sw02wid (c_sw02sr.h)</td></tr>
   <tr><td>file_id_ebcdic</td>         <td>The z/TPFDF file ID in EBCDIC format, if alphanumeric.</td>                               <td>N/A</td></tr>
   <tr><td>file_name_alias</td>        <td>The z/TPFDF file alias name, which is set by the ALIAS parameter on the DBDEF macro.</td> <td>sw02als (c_sw02sr.h)</td></tr>
   <tr><td><i>counter</i>_per_sec</td> <td>The number of counts per second where <i>counter</i>_per_sec is one of the z/TPFDF counters.<br>For a list of counters, see the <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=collection-ztpfdf-data-counters'>z/TPFDF data collection counters</a> topic in the z/TPFDF product documentation.<br>
                                           The blk_read_depth metric is calculated as blk_read_depth = blk_read_depth / (pfind + cache_read_hit).</td> <td>_cntr[] (i_dfdc.h)<br></td></tr>
   <tr><td>time</td>                   <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Notes on dashboard variables:<br>
   &nbsp&nbsp Analysis Type:<br>
   <ul>
   <li>AllFilesOpened - Select this to display all files with a DBOPN count.</li>
   <li>ModifyAPIsCalled - Select this to display files with a DBOPN count and a count in at least one modify counters.</li>
   <li>NoModifyAPIsCalled - Select this to display files with a DBOPN count and no count in any of modify counters.</li>
   &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp The following are the modify counters documented by <a href='https://www.ibm.com/docs/en/ztpf/latest?topic=collection-ztpfdf-data-counters'>z/TPFDF data collection counters</a>:<br>
   &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp DBADD, DBCKP, DBCPY, DBCRE, DBDEL, DBDIX, DBIDX, DBMOD, DBMRG, DBREP, DBRST, DBSRT, DBTLD, DBTRD, FILNC, FILEC,<br>
   &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp CFILE, PFILE, GETFC, RELFC, BTFIL, LRFIL, RSF_CREATE, RSF_UPDATE, RSF_DELETE<br>
   </ul>
   Notes:
   <ul>
      <li>Only files with a DBOPN count are displayed in this dashboard.</li>
      <li>There are situations, such as rollback, when a call was made to modify a file, but no actual modification of the file occurs.  In such a situation the modify counters are not changed.</li>
   </ul>
   Collection Routine: cdc_collect_tpfdf (cdcp_collect.cpp)<br>
   Sources:
   <ul>
      <li>c_sw02sr.h - z/TPFDF C interface central database descriptor table</li>
      <li>i_dfdc.h - Continuous z/TPFDF data collection</li>
   </ul>";

   SET tpf_common_deployment_metrics_table = "<table>
   <tr><th>Metric</th>        <th>Description</th></tr>
   <tr><td>systems</td>       <td>The z/TPF processor ID.</td></tr>
   <tr><td>subsystems</td>    <td>The subsystem name.</td></tr>
   <tr><td>cd_key</td>        <td>The key name. The key is typically the name that is used internally for the common deployment descriptor file.  For example, if the cd_type metric is BEVEVESTYPE, the common deployment file is a business event specification and the cd_key metric is the name of the business event.</td></tr>
   <tr><td>cd_type</td>       <td>The type. The common deployment file type, which generally correlates to the file extension of the descriptor file.<br>For example, BEVEVESTYPE is a .evspec.xml file and BEVEVDATYPE is a .evda.xml file.</td></tr>
   <tr><td>cd_version</td>    <td>The version. The version is optional.  Not all deployment descriptors use a version.</td></tr>
   <tr><td>usage_per_sec</td> <td>The usage of this in-memory deployment descriptor file per second.</td></tr>
   <tr><td>time</td>          <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: cdc_collect_common_deploy_files (cdcp_collect.cpp)<br>
   Note: The key, type, and version are based on the contents of the deployment descriptor and uniquely identify the deployment descriptor in in-memory structures.";

   -- The source of the Documentation here is the TCP/IP message summary report topic in z/TPF product documentation
   SET tpf_tcpip_metrics_table = "<table>
   <tr><th>Metric</th>                <th>Description</th>                                                                              <th>Source</th></tr>
   <tr><td>systems</td>               <td>TThe z/TPF processor ID.</td>                                                                 <td>N/A</td></tr>
   <tr><td>app_name</td>              <td>The name of the application as defined in the network services database.<br>
                                          This fields display 'OTHER' for applications that are not defined in the network services database (NSDB).</td> <td>INSDB_APP_NAME (insdb.mac)</td></tr>
   <tr><td>port</td>                  <td>The TCP or UDP port number associated with this application.</td>                             <td>INSDB_PORT (insdb.mac)</td></tr>
   <tr><td>msgs_per_sec</td>          <td>The raw number of messages sent and received per second.</td>                                 <td>INSDB_MESSAGES_SENT + INSDB_MESSAGES_REC (insdb.mac)</td></tr>
   <tr><td>msgs_sent_per_sec</td>     <td>The raw number of messages sent by this application per second.</td>                          <td>INSDB_MESSAGES_SENT (insdb.mac)</td></tr>
   <tr><td>msgs_recvd_per_sec</td>    <td>The number of messages received by this application per second.</td>                          <td>INSDB_MESSAGES_REC (insdb.mac)</td></tr>
   <tr><td>bytes_per_sec</td>         <td>The number of bytes sent and received per second.</td>                                        <td>INSDB_BYTES_SENT + INSDB_BYTES_SENT (insdb.mac)</td></tr>
   <tr><td>bytes_sent_per_sec</td>    <td>The number of bytes that were sent by this application per second.</td>                       <td>INSDB_BYTES_SENT (insdb.mac)</td></tr>
   <tr><td>bytes_recvd_per_sec</td>   <td>The number of bytes that were received by this application per second.</td>                   <td>INSDB_BYTES_REC (insdb.mac)</td></tr>
   <tr><td>packets_per_sec</td>       <td>The number of packets sent and received per second.</td>                                      <td>INSDB_PACKETS_SENT + INSDB_PACKETS_REC  (insdb.mac)</td></tr>
   <tr><td>packets_sent_per_sec</td>  <td>The number of packets that contain data that were sent by this application per second.</td>     <td>INSDB_PACKETS_SENT (insdb.mac)</td></tr>
   <tr><td>packets_recvd_per_sec</td> <td>The number of packets that contain data that were received by this application per second.</td> <td>INSDB_PACKETS_REC (insdb.mac)</td></tr>
   <tr><td>weight</td>                <td>The weight factor that is used when the messages received for this application are counted.<br>
                                          If a weight factor is not defined for the application, this column contains None.</td>     <td>INSDB_WEIGHT (insdb.mac)</td></tr>
   <tr><td>time</td>                  <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdctcp.asm<br>
   Source: insdb.mac -  Network services database";

   SET tpf_service_app_metrics_table = "<table>
   <tr><th>Metric</th>                      <th>Description</th></tr>
   <tr><td>systems</td>                     <td>The z/TPF processor ID.</td></tr>
   <tr><td>operation</td>                   <td>The OpenAPI operation ID for the REST service (also known as the REST service name).</td></tr>
   <tr><td>internal_operations_per_sec</td> <td>The number of calls that are made to a service locally by your z/TPF system per second.</td></tr>
   <tr><td>external_operations_per_sec</td> <td>The number of calls that are made to a service by a remote system per second.</td></tr>
   <tr><td>total_operations_per_sec</td>    <td>The number of internal and external operations per second.</td></tr>
   <tr><td>timeouts_per_sec</td>            <td>The number of calls that timed out per second. A timeout is recognized when the timeout value that was specified in the z/TPF service descriptor is exceeded.</td></tr>
   <tr><td>iface_errors_per_sec</td>        <td>The number of interface errors per second. An interface error might be caused by one of the following conditions:<br>
                                                <ul>
                                                <li>One or more arguments are not valid.</li>
                                                <li>The OpenAPI descriptor for the service cannot be found.</li>
                                                <li>The z/TPF service descriptor for the service cannot be found.</li>
                                                <li>Request or response data parsing errors.</li>
                                                <li>The caller is out of memory.</li>
                                                </ul></td></tr>
   <tr><td>srvc_errors_per_sec</td>         <td>The number of errors indicated by a status value of 400 or more that is returned by user-written code that implements the service.<br>
                                                For services that are implemented in Java, service errors also might include errors that are recognized by Java packages that support the service such as JAX-RS.</td></tr>
   <tr><td>unrec_errors_per_sec</td>        <td>The number of unrecoverable errors for a service per second. Unrecoverable errors can be caused by one or more of the following conditions:
                                                <ul>
                                                <li>The JVM exits unexpectedly.</li>
                                                <li>Threads exit unexpectedly.</li>
                                                <li>Other interprocessor communications (IPC) failures.</li>
                                                </ul></td></tr>
   <tr><td>time</td>                       <td>The rightmost time point for which there is data for the time range selected by the time picker.</td></tr>
   </table>
   Collection Routine: cdc_collect_service_app (cdcp_collect.cpp)";

   SET tpf_trad_db_encryption_metrics_table = "<table>
   <tr><th>Metric</th>                           <th>Description</th>                                            <th>Source</th></tr>
   <tr><td>systems</td>                          <td>The z/TPF processor ID.</td>                                <td>N/A</td></tr>
   <tr><td>subsystems</td>                       <td>The subsystem name.</td>                                    <td>N/A</td></tr>
   <tr><td>encryptions_per_sec</td>              <td>The number of encryptions per second.</td>                  <td>iffencrypt CMMFFEDC</td></tr>
   <tr><td>encryptions_compressions_per_sec</td> <td>The number of encryptions and compressions per second.</td> <td>iffcompenc CMMFFEDC</td></tr>
   <tr><td>decryptions_per_sec</td>              <td>The number of decryptions per second.</td>                  <td>iffdecrypt CMMFFEDC</td></tr>
   <tr><td>decryption_expansions_per_sec</td>    <td>The number of decryptions and expansions per second.</td>   <td>iffxpnddec CMMFFEDC</td></tr>
   <tr><td>encryption_errors_per_sec</td>        <td>The number of encryption errors per second.</td>            <td>ifffailenc CMMFFEDC</td></tr>
   <tr><td>compression_errors_per_sec</td>       <td>The number of compression errors per second.</td>           <td>ifffailcom CMMFFEDC</td></tr>
   <tr><td>time</td>                             <td>The rightmost time point for which there is data for the time range selected by the time picker.</td> <td>N/A</td></tr>
   </table>
   Collection Routine: cdc_collect_encrypt_counts (cdcp_collect.cpp)<br>
   Source: CMMFFEDC -  Find and file encryption and decryption counts";

-- SELECT a CDC data type by panel_key in tpf_complex_metrics_json.sql
   CASE in_datatype
      WHEN "machine" THEN SELECT tpf_static_metrics_table;
      WHEN "dasd_queue" THEN SELECT tpf_dasd_queue_metrics_table;
      WHEN "dasd_service_time" THEN SELECT tpf_dasd_service_time_metrics_table;
      WHEN "system_messages" THEN SELECT tpf_system_messages_metrics_table;
      WHEN "system_blocks" THEN SELECT tpf_system_blocks_metrics_table;
      WHEN "system_lists" THEN SELECT tpf_system_lists_metrics_table;
      WHEN "tape" THEN SELECT tpf_tape_metrics_table;
      WHEN "vfa" THEN SELECT tpf_vfa_metrics_table;
      WHEN "istream" THEN SELECT tpf_istream_metrics_table;
      WHEN "subsystem_users" THEN SELECT tpf_subsystem_users_metrics_table;
      WHEN "subsystems" THEN SELECT tpf_subsystems_metrics_table;
      WHEN "subsystem_utility_switches" THEN SELECT tpf_subsystems_utility_switches_metrics_table;
      WHEN "dasd_io" THEN SELECT tpf_dasd_io_metrics_table;
      WHEN "dasd_pools" THEN SELECT tpf_dasd_pool_metrics_table;
      WHEN "service_jam" THEN SELECT tpf_service_jam_metrics_table;
      WHEN "mq_summary" THEN SELECT tpf_mq_summary_metrics_table;
      WHEN "mq_queues" THEN SELECT tpf_mq_queue_metrics_table;
      WHEN "mq_channels" THEN SELECT tpf_mq_channel_metrics_table;
      WHEN "remote_data_store" THEN SELECT tpf_remote_data_store_metrics_table;
      WHEN "channel_utilization" THEN SELECT tpf_enh_chl_util_metrics_table;
      WHEN "lpar_utilization" THEN SELECT tpf_lpar_util_metrics_table;
      WHEN "mpif" THEN SELECT tpf_mpif_metrics_table;
      WHEN "shutdown_events" THEN SELECT tpf_shutdown1_metrics_table;
      WHEN "traces" THEN SELECT tpf_traces_metrics_table;
      WHEN "LODIC_resource_classes" THEN SELECT tpf_lodic_resources_metrics_table;
      WHEN "LODIC_utilization_classes" THEN SELECT tpf_lodic_utilization_metrics_table;
      WHEN "time_slice_ecbs" THEN SELECT tpf_time_slice_ecb_metrics_table;
      WHEN "dasd_metrics" THEN SELECT tpf_dasd_metrics_table;
      WHEN "tpfdf" THEN SELECT tpf_tpfdf_metrics_table;
      WHEN "common_deployment" THEN SELECT tpf_common_deployment_metrics_table;
      WHEN "tcpip" THEN SELECT tpf_tcpip_metrics_table;
      WHEN "rest_services" THEN SELECT tpf_service_app_metrics_table;
      WHEN "trad_db_encryption" THEN SELECT tpf_trad_db_encryption_metrics_table;
      ELSE SELECT " - No documentation available for this data type.";
      END CASE;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_table_name_list_inner_join_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_table_name_list_inner_join_func` (
    `in_type` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE table_metrics JSON;
DECLARE json_array_result JSON;
SET table_metrics = tpf_get_metrics_json();

SELECT DISTINCT  JSON_ARRAYAGG(CONCAT("INNER JOIN ", attribute_table," ON tn.",table_column, "=", attribute_table, ".id"))
    INTO json_array_result
    FROM json_table(table_metrics, '$[*]'
    COLUMNS(
        panel_key         VARCHAR(64) PATH '$.panel_key',
        dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
        subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
        table_name        VARCHAR(64) PATH '$.table_name',
        NESTED PATH '$.attribute_columns[*]' COLUMNS (
          attribute_name VARCHAR(64) PATH '$.attribute_name',
          attribute_table VARCHAR(64) PATH '$.attribute_table',
          table_column VARCHAR(64) PATH '$.table_column'
        ),
        metric_type       VARCHAR(64) PATH '$.metric_type',
        NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.virtual_metrics[*]' COLUMNS (
        virtual_metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.extra_cols[*]' COLUMNS (
        extra_cols VARCHAR(64) PATH '$'
        ),
        altered_column   VARCHAR(64) PATH '$.altered_column'
    )
    ) AS jt
    WHERE jt.panel_key=in_type AND jt.attribute_table IS NOT NULL AND jt.attribute_table != "";
RETURN json_array_result;
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_check_for_keys_in_attribute_list`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_check_for_keys_in_attribute_list` (
    `in_type` TEXT,
    `in_key_list` JSON
)
RETURNS BOOLEAN
DETERMINISTIC
BEGIN
DECLARE found_attribute BOOLEAN DEFAULT FALSE;
DECLARE json_array_result JSON;
DECLARE table_metrics JSON;

-- Just return TRUE if there are no keys.
IF in_key_list IS NULL OR JSON_LENGTH(in_key_list) = 0
THEN
    RETURN TRUE;
END IF;
-- Search for the keys in the attribute list.
SET table_metrics = tpf_get_metrics_json();
SELECT JSON_ARRAYAGG(table_column)
INTO json_array_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type  AND
      jt.table_column != "" AND
      JSON_CONTAINS(
              in_key_list,
              CONCAT('"',jt.table_column,'"')
);

-- Test to see if in_key_list has the same number of elements as json_array_result
-- If they are not the same, then return false
IF json_array_result IS NULL
    OR JSON_LENGTH(json_array_result) = 0
    OR JSON_LENGTH(json_array_result) != JSON_LENGTH(in_key_list)
THEN
    RETURN FALSE;
END IF;

RETURN TRUE;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_create_key_clause_stmt`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_create_key_clause_stmt` (
    `in_type` TEXT,
    `in_table_name` TEXT,
    `in_show_metrics` JSON,
    `in_subsystem_applies` TEXT,
    `in_attribute_types` TEXT,
    `in_inner_join_string` TEXT,
    `in_lt_time` DATETIME,
    `in_rt_time` DATETIME,
    `in_group_by_time_shift` INT,
     `in_sec_per_interval` INT,
     `avg_metric_columns` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE key_clause_stmt  TEXT DEFAULT "";
DECLARE group_by_stmt TEXT DEFAULT "";
DECLARE select_stmt  TEXT DEFAULT "";
DECLARE metric_columns TEXT;

SET group_by_stmt = CONCAT(" GROUP BY system_id, ",
     "  (UNIX_TIMESTAMP(time) + ",  in_group_by_time_shift ,") DIV ", in_sec_per_interval);

IF in_table_name != "tpf_shutdown1_metrics_table"
THEN
    SELECT GROUP_CONCAT("SUM(`",input_fields,"`) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';
ELSE
    SELECT GROUP_CONCAT("AVG(`",input_fields,"`) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';
END IF;

-- NOTE: subsystem is not here because we want to SUM across subsystems, and not
-- treat that separately.
SET key_clause_stmt = CONCAT("WITH key_clause AS ( SELECT time, system_id, itsitn.display_name AS display_name,",
    CASE WHEN in_attribute_types != "" THEN CONCAT(in_attribute_types,",") ELSE "" END,
    metric_columns,
    " FROM ", in_table_name, " tn ", in_inner_join_string,
    " WHERE time BETWEEN '", in_lt_time ,"' AND  '", in_rt_time, "' ",
    group_by_stmt,
--    " GROUP BY system_id, ",
--    CASE WHEN in_subsystem_applies = 'yes' THEN " subsystem," ELSE "" END,
--     "  (UNIX_TIMESTAMP(time) + ",  in_group_by_time_shift ,") DIV ", in_sec_per_interval,
    ")"
);
-- metric columns needs to be AVG not SUM
SET select_stmt =
 CONCAT(key_clause_stmt,
    " SELECT MIN(time) AS time, ",
    "CONCAT_WS('-',",
    "display_name)  AS series_name_test,",
    avg_metric_columns,
    " FROM key_clause tn ",
    group_by_stmt,
    " ORDER BY time ASC"
 );
RETURN select_stmt;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_input_attribute_type`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_input_attribute_type` (
    `in_attribute_types` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE cur_attribute_type  TEXT DEFAULT "";

IF  in_attribute_types=""
THEN
    SET cur_attribute_type = "";
ELSEIF JSON_TYPE(in_attribute_types) = "STRING"
THEN
    IF in_attribute_types='""'  OR in_attribute_types="[]"
    THEN
        SET cur_attribute_type = "";
    ELSE
        SET cur_attribute_type = in_attribute_types;
    END IF;
ELSEIF  JSON_LENGTH(in_attribute_types) = 0 OR JSON_contains(in_attribute_types,'[""]')
THEN
    SET cur_attribute_type = "";
ELSE
    SELECT attribute_entry
    INTO  cur_attribute_type
    FROM JSON_TABLE(in_attribute_types, "$[*]" COLUMNS(attribute_entry VARCHAR(64) path "$")) ism;
END IF;

IF cur_attribute_type = "N/A"
THEN
    SET cur_attribute_type = "";
END IF;

RETURN cur_attribute_type;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_altered_column`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_altered_column` (
    `in_panel_key` TEXT
)
RETURNS BOOLEAN
DETERMINISTIC
BEGIN
DECLARE altered_column_result  INT DEFAULT 0;
DECLARE table_metrics  JSON;

SET table_metrics = tpf_get_metrics_json();

SELECT COUNT(*)
INTO altered_column_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key != ""  AND
      jt.panel_key=in_panel_key AND
      jt.altered_column != "" AND
      jt.altered_column != "no";

RETURN altered_column_result;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_extra_columns_only_cdf_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_extra_columns_only_cdf_table`(
      IN `in_panel_key` VARCHAR(64),
      IN `in_system_ids` JSON,
      IN `in_subsystems` JSON,
      IN `ss_entry` TEXT,
      IN `systems_columns` JSON,
      IN `in_lt_time` DATETIME,
      IN `in_rt_time` DATETIME,
      IN `extra_column_entry_list` JSON,
      IN `table_name` TEXT,
      IN `max_results_value` INT
)

tpf_extra_columns_only_cdf_table:BEGIN

DECLARE has_altered_column INT DEFAULT 0;
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE sp_partition_statement  TEXT DEFAULT "";
DECLARE inner_join_stmt  TEXT DEFAULT "";
DECLARE select_stmt TEXT DEFAULT "";
DECLARE extra_column_entry TEXT DEFAULT "";
DECLARE select_metric_rows_statement TEXT DEFAULT "";
DECLARE rightmost_time DATETIME;


SET has_altered_column = tpf_get_altered_column(in_panel_key);
IF has_altered_column = 0
THEN
SET select_time_stmt = CONCAT(" WHERE time BETWEEN ",
        QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time));
ELSE
        SET select_time_stmt = CONCAT(" WHERE (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)");

END IF;

SET sp_partition_statement = CONCAT(", ROW_NUMBER() OVER (PARTITION BY tn.system_id ",
                " ORDER BY time DESC) AS row_num ");
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
                 "",NULL, in_system_ids, systems_columns);
SET select_stmt =
CONCAT(
    "WITH ordered AS ( \nSELECT TIME, ROW_NUMBER() OVER (PARTITION BY tn.system_id ",
    CASE WHEN  ss_entry = "yes" THEN  ",tn.subsystem_id " ELSE "" END,
    " ORDER BY time DESC) AS row_num ",
    "\n FROM ", table_name, " tn \n", inner_join_stmt, "\n",
    select_time_stmt,
    "\n ORDER BY TIME DESC) \n",
    " SELECT time INTO @rightmost_time \nFROM ordered \nWHERE row_num = 1 \nLIMIT 1"
);
CALL tpf_execute_immediate(select_stmt);
SET rightmost_time = @rightmost_time;

-- If there are no entries found, then stop processing.
IF rightmost_time IS NULL
THEN
    LEAVE tpf_extra_columns_only_cdf_table;
END IF;
SELECT GROUP_CONCAT(extra_fields)
INTO  extra_column_entry
        FROM JSON_TABLE(extra_column_entry_list, "$[*]" COLUMNS(extra_fields VARCHAR(64) path "$")) ism;
        SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
             "",NULL, in_system_ids, systems_columns);
         -- For each selected system, display all rows that match the time picker,
         -- or the latest row before the rightmost time.
         IF has_altered_column > 0
         THEN
             SET select_stmt =
                 CONCAT(
                 " SELECT altered, time, ",
                 "itsitn.display_name AS systems ",
                 CASE WHEN  ss_entry = "yes" THEN  ",tss.subsystem AS subsystems" ELSE "" END,
                 ",", extra_column_entry,
                 "\n FROM ", table_name, " tn \n",
                 inner_join_stmt, "\n", select_time_stmt,
                 "\n ORDER BY systems, altered DESC ",
                  CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END);
             CALL tpf_execute_immediate(select_stmt);
             LEAVE tpf_extra_columns_only_cdf_table;
        END IF;
        SET select_metric_rows_statement = CONCAT(" SELECT ",
            "itsitn.display_name AS systems ",
            CASE WHEN  ss_entry = "yes" THEN  ",tss.subsystem" ELSE "" END,
            ",", extra_column_entry);
        SET select_stmt = CONCAT(select_metric_rows_statement, " FROM ", table_name, " tn ", inner_join_stmt,
           " WHERE time = ", QUOTE(rightmost_time));
        CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_cdf_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_panel_key` VARCHAR(64),
    IN `in_metric_type` JSON,
    IN `in_filter_types` JSON,
    IN `in_filters_selected` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_max_results` TEXT,
    IN `in_user_input` TEXT,
    IN `in_analysis_metric` VARCHAR(64)
)

tpf_cdf_table:BEGIN

DECLARE metric_columns TEXT;
DECLARE table_name VARCHAR(64)  DEFAULT "";
DECLARE temp_sys_id_table_name VARCHAR(128) DEFAULT "";
DECLARE num_subsystems INT DEFAULT 0;
DECLARE inner_join_stmt TEXT DEFAULT "";
DECLARE cur_attribute TEXT DEFAULT "";
DECLARE table_metrics JSON;
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE ss_entry VARCHAR(64)  DEFAULT 0;
DECLARE extra_column_entry TEXT DEFAULT "";
DECLARE extra_column_avg_display TEXT DEFAULT "";
DECLARE extra_column_entry_list JSON;
DECLARE at_name_list JSON;
DECLARE input_fields_metrics_list  JSON;
DECLARE at_name TEXT DEFAULT "";
DECLARE at_table_name_field TEXT DEFAULT "";
DECLARE ordered_select_stmnt TEXT DEFAULT "";
DECLARE partition_statement TEXT DEFAULT "";
DECLARE select_metric_rows_statement TEXT DEFAULT "";
DECLARE average_metric_columns TEXT DEFAULT "";
DECLARE sum_metric_columns TEXT DEFAULT "";
DECLARE at_name_average_display TEXT DEFAULT "";
DECLARE select_sum_statement TEXT DEFAULT "";
DECLARE select_avg_statement TEXT DEFAULT "";
DECLARE systems_columns JSON;
DECLARE rightmost_time DATETIME;
DECLARE key_name_list JSON;
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE key_clause_stmt LONGTEXT DEFAULT "";
DECLARE verify_metric_columns BOOLEAN DEFAULT FALSE;
DECLARE analysis_function_found TEXT DEFAULT "";
DECLARE max_results_value INT DEFAULT 0;
DECLARE in_show_metrics_contained BOOLEAN DEFAULT TRUE;
DECLARE available_results JSON;
DECLARE has_altered_column BOOLEAN DEFAULT FALSE;
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE sp_partition_statement TEXT DEFAULT "";
DECLARE att_display_fields TEXT DEFAULT "";
DECLARE test_metric TEXT DEFAULT "";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';
IF JSON_LENGTH(in_system_ids) = 0 OR JSON_contains(in_system_ids,'[""]') OR
    in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_cdf_table;
END IF;

IF in_panel_key IS NULL OR in_panel_key = ""
THEN
    LEAVE tpf_cdf_table;
END IF;

IF JSON_LENGTH(in_show_metrics) = 0 OR JSON_contains(in_show_metrics,'[""]') OR
    in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_cdf_table;
END IF;

SELECT in_max_results REGEXP '[0-9]' INTO @max_results_value;
IF @max_results_value = 1
THEN
    SELECT CAST(in_max_results AS UNSIGNED INT) into @max_results_value;
    IF @max_results_value >  65535
    THEN
        SELECT CONCAT("ERROR: value ",in_max_results, " is greater than allowed for Limit.");
        LEAVE tpf_cdf_table;
    END IF;
    SET max_results_value = @max_results_value;
ELSEIF in_max_results != '*'
THEN
    SELECT CONCAT("ERROR: Non-numeric value ",in_max_results, " is not allowed for Limit.");
    LEAVE tpf_cdf_table;
END IF;
-- remove the quotes to use the in_analysis_metric in the ORDER BY clause
-- If the analysis metric is like average or totals, assume the user
-- wants the analysis to be based on this metric and remove the suffix.
IF in_analysis_metric != ""
THEN
    SET in_analysis_metric = REPLACE(in_analysis_metric,"'",'');
    -- Remove the averages or totals from the analysis metric
    IF in_analysis_metric LIKE "%-avg_systems" OR
        in_analysis_metric LIKE "%-total_systems"
    THEN
        SET in_analysis_metric = SUBSTRING_INDEX(in_analysis_metric, '-', 1);
    END IF;
    SET in_show_metrics_contained = JSON_CONTAINS(in_show_metrics,  CONCAT('"',in_analysis_metric,'"'));
END IF;

-- Handle optional parameters
SET num_subsystems = JSON_LENGTH(in_subsystems);

-- Generic function to get the table name
SET table_name = tpf_get_table_name(in_panel_key);

SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
-- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
SET inner_join_stmt = CONCAT(" SELECT JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name)) ",
                         " INTO @systems_columns",
                         " FROM ", temp_sys_id_table_name);
CALL tpf_execute_immediate(inner_join_stmt);
SET systems_columns = @systems_columns;


SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @key_name_list ",
         " FROM information_schema.KEY_COLUMN_USAGE ",
         " WHERE TABLE_NAME = '",table_name,"'",
         " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
CALL tpf_execute_immediate(select_stmt);
SET key_name_list = @key_name_list;
SET found_key_attribute = tpf_check_for_keys_in_attribute_list(in_panel_key,key_name_list);

SELECT GROUP_CONCAT(input_fields)
INTO  metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;


SET ss_entry = tpf_get_subsystem_applies(in_panel_key);

SET has_altered_column = tpf_get_altered_column(in_panel_key);
IF has_altered_column = 0
THEN
SET select_time_stmt = CONCAT(" WHERE time BETWEEN ",
        QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time));
ELSE
        SET select_time_stmt = CONCAT(" WHERE ((TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time))");

END IF;

-- This is the case where there are only extra cols to display. There are no metrics.
-- An example is the traces table.
SET extra_column_entry_list =  tpf_get_extra_cols_available_func(in_panel_key);
IF extra_column_entry_list IS NOT NULL AND JSON_LENGTH(extra_column_entry_list) > 0
THEN
    IF metric_columns = "None"
    THEN
        CALL tpf_extra_columns_only_cdf_table(in_panel_key,  in_system_ids,  in_subsystems, ss_entry,
            systems_columns, in_lt_time, in_rt_time, extra_column_entry_list, table_name, max_results_value);
            LEAVE tpf_cdf_table;
    END IF;
    -- This is the case where there are both metrics and extra fields.
    IF table_name != "tpf_tcpip_metrics_table"
    THEN
        SELECT GROUP_CONCAT(extra_fields),  GROUP_CONCAT("'N/A' as ", extra_fields)
        INTO  extra_column_entry, extra_column_avg_display
        FROM JSON_TABLE(extra_column_entry_list, "$[*]" COLUMNS(extra_fields VARCHAR(64) path "$")) ism;
    ELSE
        -- For the tcpip table only, for the extra columns,
        --  if the value for the extra column is less than zero, show None.
        SELECT GROUP_CONCAT(" CASE WHEN ",extra_fields,
                        " < 4294967295 THEN ",extra_fields,
                         " ELSE 'None' END ",
                          "AS ", extra_fields, " "),
        GROUP_CONCAT("'N/A' as ", extra_fields)
        INTO extra_column_entry,
        extra_column_avg_display
        FROM JSON_TABLE(extra_column_entry_list, "$[*]" COLUMNS(extra_fields VARCHAR(64) path "$")) ism;
    END IF;
END IF;

IF metric_columns = "None"
THEN
        LEAVE tpf_cdf_table;
END IF;

-- The metric_columns need to be obtained again
SELECT GROUP_CONCAT(CONCAT("`",input_fields,"`"))
INTO  metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

--  verify  the metric columns
IF  in_analysis_metric LIKE "%-%"
THEN
    SET test_metric = SUBSTRING_INDEX(in_analysis_metric, '-', 1);
ELSE
    SET test_metric = in_analysis_metric;
END IF;
SET verify_metric_columns = tpf_validate_metric_columns(in_panel_key, in_show_metrics, JSON_UNQUOTE(in_metric_type), test_metric);
IF verify_metric_columns IS FALSE
THEN
      LEAVE tpf_cdf_table;
END IF;


-- The filter means we either call a function to filter the results
-- or we filter by attribute.
-- Get either the function to call or get the variable table_column.
-- For example, attribute_table_entry is operation for REST services
SET cur_attribute = tpf_get_input_attribute_type(in_filter_types);
SET analysis_function_found = tpf_find_analysis_func(cur_attribute, "table_procedure");
IF  analysis_function_found != "" AND has_altered_column = 0
THEN
    SET @analysis_function_found = analysis_function_found;
    SET @in_panel_key = in_panel_key;
    SET @in_show_metrics = in_show_metrics;
    SET @in_system_ids = in_system_ids;
    SET @in_subsystems = in_subsystems;
    SET @in_rt_time = in_rt_time;
    SET @in_lt_time = in_lt_time;
    SET @ss_entry = ss_entry;
    SET @in_filters_selected = in_filters_selected;
    SET @in_analysis_metric = in_analysis_metric;
    SET @cur_attribute = cur_attribute;
    SET @in_user_input = "";
    IF  analysis_function_found LIKE "tpf_user_input_%"
    THEN
        SET @in_user_input = in_user_input;
    END IF;
    --  Call tpf_common_table
    SET select_stmt = CONCAT("CALL ", @analysis_function_found ,
        "(@cur_attribute, @in_panel_key, @in_show_metrics, @in_system_ids, @in_subsystems,",
        "@in_lt_time, @in_rt_time,@in_filters_selected, @ss_entry, ",
        "@systems_columns, @in_analysis_metric, @max_results_value, @in_user_input)");
    --  use alternate prepare statement in tpf_cdf_execute_immediate
    -- to avoid recursion when  calling tpf_execute_immediate from
    -- the user exits.
    CALL tpf_cdf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_table;
END IF;

SELECT GROUP_CONCAT("AVG(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  average_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT("SUM(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  sum_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

IF cur_attribute != ""
THEN
    SET at_name_list = tpf_get_attributes_available_func(in_panel_key);
END IF;
IF at_name_list IS NOT NULL AND JSON_LENGTH(at_name_list) > 0
THEN
SET at_table_name_field = tpf_get_table_name_field_available_func(in_panel_key);
    SELECT GROUP_CONCAT("'N/A' as ", at_fields)
    INTO  at_name_average_display
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
    SELECT GROUP_CONCAT(at_fields)
    INTO  at_name
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
    SET att_display_fields = tpf_attribute_case_stmt(in_panel_key, at_name_list);
END IF;

-- Generic method to get the INNER JOIN clause
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
      cur_attribute, in_filters_selected, in_system_ids, systems_columns);

SET  partition_statement = CONCAT(", ROW_NUMBER() OVER (PARTITION BY tn.system_id ",
    CASE WHEN  ss_entry = "yes" THEN  ",tn.subsystem_id " ELSE "" END,
    CASE WHEN  at_table_name_field != "" THEN  CONCAT(",",at_table_name_field) ELSE "" END,
    " ORDER BY time DESC) AS row_num ");
-- NOTE: Using systems instead of system, since system is a mySQL keyword
SET ordered_select_stmnt = CONCAT("\n SELECT ",
    CASE WHEN has_altered_column > 0 THEN  " altered, " ELSE "" END,
    " time, itsitn.display_name AS systems ",
    CASE WHEN  ss_entry = "yes" THEN  ",tss.subsystem AS subsystems" ELSE "" END,
    CASE WHEN  at_table_name_field != "" THEN  CONCAT(",",at_table_name_field) ELSE "" END,
    CASE WHEN  metric_columns != '' THEN  CONCAT( ", ", metric_columns) ELSE "" END,
    CASE WHEN  extra_column_entry != '' THEN  CONCAT( ", ", extra_column_entry) ELSE "" END,
    CASE WHEN  in_show_metrics_contained IS FALSE THEN  CONCAT( ", `", in_analysis_metric,"`") ELSE "" END
    );
IF found_key_attribute IS TRUE
THEN
SET select_metric_rows_statement = CONCAT(" SELECT ",
    CASE WHEN has_altered_column > 0 THEN  " altered, time, " ELSE "" END,
    " systems ",
    CASE WHEN  ss_entry = "yes" THEN  ",subsystems" ELSE "" END,
    CASE WHEN  at_table_name_field != "" THEN CONCAT(",",att_display_fields) ELSE "" END,
    CASE WHEN  metric_columns != '' THEN CONCAT( ", ", metric_columns) ELSE "" END,
    CASE WHEN  extra_column_entry != "" THEN CONCAT(",",extra_column_entry) ELSE "" END,
    CASE WHEN has_altered_column <= 0 THEN  ",time " ELSE "" END);
ELSE
SET select_metric_rows_statement = CONCAT(" SELECT ",
        CASE WHEN has_altered_column > 0 THEN  " altered, time, " ELSE "" END,
        " systems ",
        CASE WHEN  ss_entry = "yes" THEN  ",subsystems" ELSE "" END,
        CASE WHEN  at_table_name_field != "" THEN CONCAT(",",att_display_fields) ELSE "" END,
        CASE WHEN  sum_metric_columns != '' THEN CONCAT( ", ", sum_metric_columns) ELSE "" END,
        CASE WHEN  extra_column_entry != "" THEN CONCAT(",",extra_column_entry) ELSE "" END,
        CASE WHEN has_altered_column <= 0 THEN  ",time " ELSE "" END);
    SET key_clause_stmt = CONCAT("GROUP BY systems ",
        CASE WHEN  ss_entry = "yes" THEN  ",subsystems " ELSE "" END,
        CASE WHEN  at_table_name_field != "" THEN CONCAT(",",at_name) ELSE "" END);
END IF;
SET select_sum_statement = tpf_create_select_agg_stmt(ss_entry, at_name_average_display, sum_metric_columns, extra_column_avg_display, "");
SET select_avg_statement = tpf_create_select_agg_stmt(ss_entry, at_name_average_display, average_metric_columns, extra_column_avg_display,"");

-- NOTE: Always show the last entry for all processors selected
-- for all time points before the right most time point.
-- Allow for filtering on Limit and User Input
IF has_altered_column > 0
THEN
    SET select_stmt =
    CONCAT(
        ordered_select_stmnt, "\n",
        " FROM ", table_name, " tn \n",
        inner_join_stmt, " \n ", select_time_stmt,
        CASE WHEN (in_user_input != "" AND  analysis_function_found LIKE "tpf_user_input_%")
        THEN CONCAT(" AND (`", in_analysis_metric, "` > ", in_user_input, ") ") ELSE "" END,
        "\n ORDER BY  ",
         CASE WHEN in_analysis_metric != "" THEN CONCAT(" `", in_analysis_metric, "` DESC, ") ELSE "" END,
        " altered DESC \n",
        CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END);
     CALL tpf_execute_immediate(select_stmt);
     LEAVE tpf_cdf_table;
END IF;

-- Determine the rightmost time point.
-- If there is data for one processor, but not the others,
-- then just show the processors that are active at that time.
SET select_stmt =
 CONCAT(
    "WITH ordered AS ( SELECT TIME ",
      partition_statement,
    " FROM ", table_name, " tn ",
    inner_join_stmt,
    " WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time), " ORDER BY TIME DESC) ",
   " SELECT time INTO @rightmost_time FROM ordered WHERE row_num = 1 LIMIT 1"
);
CALL tpf_execute_immediate(select_stmt);
SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
    LEAVE tpf_cdf_table;
END IF;


-- NOTE:  ORDER BY from a subquery is ignored. The ORDER BY clause must be at the top level select.
SET select_stmt =
 CONCAT( " WITH ordered AS (\n",
    ordered_select_stmnt,
    partition_statement,
    "\n FROM ", table_name, " tn ",
    inner_join_stmt,
   "\n WHERE time = ", QUOTE(rightmost_time), "), ",
   "\n basic_ordered AS \n(" ,
        "SELECT * \nFROM ordered ",
       CASE WHEN in_analysis_metric != "" THEN CONCAT(" ORDER BY `", in_analysis_metric, "` DESC ") ELSE "" END,
      CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END,
    " ),",
    "\n totals_ordered AS \n(" ,
        "SELECT systems  ",
        select_sum_statement,
        "\n FROM ordered ",
        " GROUP BY ordered.systems",
        CASE WHEN cur_attribute != '' THEN  CONCAT(",ordered.",cur_attribute) ELSE "" END,
    " ),",
    "\n average_ordered AS \n(" ,
        "SELECT systems  ",
        select_sum_statement,
        "\n FROM ordered ",
        " GROUP BY ordered.systems",
    " )\n",
     select_metric_rows_statement,
    "\n FROM basic_ordered ",
    CASE WHEN found_key_attribute IS FALSE THEN key_clause_stmt ELSE "" END,
    " \n UNION ALL \n",
       -- Add the attribute name to the avg, for example, avg_by-device_name-C
     "SELECT CONCAT('avg",
     CASE WHEN cur_attribute != '' THEN  CONCAT("_by-",cur_attribute) ELSE "" END,
     "-',totals_ordered.systems) as systems  ",
     select_avg_statement,
     "\n FROM totals_ordered ",
     " GROUP BY totals_ordered.systems ",
    " \n UNION ALL \n",
     "SELECT CONCAT('total-',ordered.systems) as systems",
     select_sum_statement,
     "\n FROM ordered ",
     " GROUP BY ordered.systems ",
     " \n UNION ALL \n",
     "SELECT 'total-*' as systems",
     select_sum_statement,
     "\n FROM totals_ordered HAVING COUNT(*) > 0"
     " \n UNION ALL \n",
    "SELECT 'avg-*' as systems",
     select_avg_statement,
     "\n FROM average_ordered HAVING COUNT(*) > 0"
 );
 CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_cdf_bar_chart`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_bar_chart`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_bar_chart_metric` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_panel_key` VARCHAR(64),
     IN `in_metric_types` JSON,
    IN `in_filter_types` JSON,
    IN `in_filters_selected` JSON,
    IN `in_max_results` TEXT,
    IN `in_user_input` TEXT
)

tpf_cdf_bar_chart:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE ordered_stmt LONGTEXT DEFAULT "";
DECLARE bar_chart_metric VARCHAR(128) DEFAULT "";
DECLARE table_name TEXT DEFAULT "";
DECLARE at_field_list TEXT DEFAULT "";
DECLARE at_field_list_test TEXT DEFAULT "";
DECLARE table_metrics JSON;
DECLARE att_type  TEXT DEFAULT "";
DECLARE bar_metric_count INT;
DECLARE inner_join_stmt  TEXT DEFAULT "";
DECLARE ss_entry VARCHAR(64)  DEFAULT 0;
DECLARE cmbs_select_clause_str LONGTEXT DEFAULT "";
DECLARE partition_statement TEXT DEFAULT "";
DECLARE at_name_list JSON;
DECLARE set_up_averages BOOLEAN DEFAULT FALSE;
DECLARE set_up_totals   BOOLEAN DEFAULT FALSE;
DECLARE attribute_list TEXT DEFAULT "";
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE systems_columns JSON;
DECLARE rightmost_time DATETIME;
DECLARE totals_ordered_stmt LONGTEXT DEFAULT "";
DECLARE key_name_list JSON;
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE key_attribute_stmt LONGTEXT DEFAULT "";
DECLARE verify_metric_columns BOOLEAN DEFAULT FALSE;
DECLARE available_regular_metrics JSON;
DECLARE available_virtual_metrics JSON;
DECLARE available_metrics JSON;
DECLARE max_results_value INT DEFAULT 0;
DECLARE analysis_function_found TEXT DEFAULT "";
DECLARE test_metric TEXT DEFAULT "";
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE has_altered_column BOOLEAN DEFAULT FALSE;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_system_ids) = 0 OR JSON_contains(in_system_ids,'[""]') OR
    in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;

IF JSON_LENGTH(in_bar_chart_metric) = 0 OR JSON_contains(in_bar_chart_metric,'[""]') OR
    in_bar_chart_metric='""'  OR in_bar_chart_metric="" OR in_bar_chart_metric="[]"
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;

IF in_panel_key IS NULL OR in_panel_key = ""
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;

SELECT in_max_results REGEXP '[0-9]' INTO @max_results_value;
IF @max_results_value = 1
THEN
    SELECT CAST(in_max_results AS UNSIGNED INT) into @max_results_value;
    IF @max_results_value >  65535
    THEN
        LEAVE tpf_cdf_bar_chart;
    END IF;
    SET max_results_value = @max_results_value;
ELSEIF in_max_results != '*'
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;


IF JSON_TYPE(in_bar_chart_metric) != "STRING"
THEN
    SELECT bar_metric
    INTO bar_chart_metric
    FROM JSON_TABLE(in_bar_chart_metric, "$[*]" COLUMNS(bar_metric VARCHAR(64) path "$")) ism2;
ELSE
    SET bar_chart_metric = in_bar_chart_metric;
END IF;

IF bar_chart_metric != "" AND bar_chart_metric LIKE "%-avg_systems"
THEN
    SET bar_chart_metric = SUBSTRING_INDEX(bar_chart_metric, '-', 1);
    SET set_up_averages = TRUE;
ELSEIF bar_chart_metric != "" AND bar_chart_metric LIKE "%-total_systems"
THEN
    SET bar_chart_metric = SUBSTRING_INDEX(bar_chart_metric, '-', 1);
    SET set_up_totals = TRUE;
END IF;

--  verify that the input metric applies for this metric type
IF  bar_chart_metric LIKE "%-%"
THEN
    SET test_metric = SUBSTRING_INDEX(bar_chart_metric, '-', 1);
ELSE
    SET test_metric = bar_chart_metric;
END IF;

SET available_regular_metrics = tpf_get_metrics_available_func(in_panel_key,in_metric_types);
SET available_virtual_metrics = tpf_get_virtual_metrics(in_panel_key,in_metric_types);
IF available_virtual_metrics IS NOT NULL AND JSON_LENGTH(available_virtual_metrics) > 0
THEN
    SET available_metrics = JSON_MERGE(available_regular_metrics,available_virtual_metrics);
ELSE
    SET available_metrics = available_regular_metrics;
END IF;
IF JSON_SEARCH(available_metrics, 'one', test_metric) IS NULL
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;


SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);

SET ss_entry = tpf_get_subsystem_applies(in_panel_key);
SET att_type = tpf_get_input_attribute_type(in_filter_types);

SET analysis_function_found = tpf_find_analysis_func(att_type, "bar_procedure");
IF  analysis_function_found != ""
THEN
    SET @analysis_function_found = analysis_function_found;
    SET @in_panel_key = in_panel_key;
    SET @in_system_ids = in_system_ids;
    SET @in_subsystems = in_subsystems;
    SET @in_rt_time = in_rt_time;
    SET @in_lt_time = in_lt_time;
    SET @ss_entry = ss_entry;
    SET @bar_chart_metric = bar_chart_metric;
    SET @temp_sys_id_table_name = temp_sys_id_table_name;
    SET @att_type = att_type;
    SET @in_user_input = "";
    SET @set_up_averages = set_up_averages;
    SET @set_up_totals = set_up_totals;

    IF  analysis_function_found LIKE "tpf_user_input_%"
    THEN
        SET @in_user_input = in_user_input;
    END IF;
    SET @in_filters_selected = in_filters_selected;
    --  Call tpf_common_bar_chart
    SET select_stmt = CONCAT("CALL ", @analysis_function_found ,
        "(@att_type, @in_panel_key,  @bar_chart_metric, @in_system_ids, @in_subsystems,",
        "@in_lt_time, @in_rt_time, @temp_sys_id_table_name,  @ss_entry, ",
        " @max_results_value, @in_user_input, @in_filters_selected,",
        "@set_up_averages, @set_up_totals)");
    CALL tpf_cdf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_bar_chart;
END IF;

SET inner_join_stmt = CONCAT(" SELECT JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name)) ",
                         " INTO @systems_columns",
                         " FROM ", temp_sys_id_table_name);
CALL tpf_execute_immediate(inner_join_stmt);
SET systems_columns = @systems_columns;

-- Handle case where there are attributes available but grafana did not send down any.
SET  at_name_list = tpf_get_attributes_available_func(in_panel_key);
IF att_type = "" AND at_name_list IS NOT NULL AND JSON_LENGTH(at_name_list) > 0
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;
-- Handle the converse case. Ignore attributes if grafana sent attributes,
-- but none are available. Do not send up an error.
IF att_type != "" AND at_name_list IS NULL OR JSON_LENGTH(at_name_list) = 0
THEN
    SET att_type = "";
END IF;

IF att_type != ""
THEN
    IF at_name_list IS NOT NULL AND JSON_LENGTH(at_name_list) > 0
    THEN
        SELECT GROUP_CONCAT(at_fields, ",'-'"),GROUP_CONCAT(at_fields)
        INTO at_field_list, attribute_list
        FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
       SELECT GROUP_CONCAT(" CASE WHEN ",at_fields,
       " IS NOT NULL THEN ",at_fields,
       " ELSE 'N/A' END ",
       "AS ", at_fields, " ")
        INTO at_field_list_test
        FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
    END IF;
END IF;

SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
      att_type, in_filters_selected, in_system_ids, systems_columns);

SET partition_statement = CONCAT(",    ROW_NUMBER() OVER (PARTITION BY tn.system_id, ",
    CASE WHEN ss_entry = "yes" THEN " tn.subsystem_id," ELSE "" END,
    CASE WHEN attribute_list != "" THEN attribute_list ELSE " tn.system_id" END,
    " ORDER BY time DESC) AS row_num ");

SET table_name = tpf_get_table_name(in_panel_key);

SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @key_name_list ",
         " FROM information_schema.KEY_COLUMN_USAGE ",
         " WHERE TABLE_NAME = '",table_name,"'",
         " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
CALL tpf_execute_immediate(select_stmt);
SET key_name_list = @key_name_list;
SET found_key_attribute = tpf_check_for_keys_in_attribute_list(in_panel_key,key_name_list);

SET has_altered_column = tpf_get_altered_column(in_panel_key);
IF has_altered_column = 0
THEN
       SET select_time_stmt = CONCAT(" WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time));
ELSE
       SET select_time_stmt = CONCAT(" WHERE (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)");
END IF;

-- Determine the rightmost time point.
-- If there is data for one processor, but not the others,
-- then just show the processors that are active at that time.
SET select_stmt =
 CONCAT(
    "WITH ordered AS ( SELECT TIME ",
      partition_statement,
    " FROM ", table_name, " tn ",
    inner_join_stmt, " \n ", select_time_stmt,
    " ORDER BY TIME DESC) ",
   " SELECT time INTO @rightmost_time FROM ordered WHERE row_num = 1 LIMIT 1"
);
CALL tpf_execute_immediate(select_stmt);
SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
    LEAVE tpf_cdf_bar_chart;
END IF;

SET ordered_stmt = CONCAT(" SELECT tn.system_id",
    CASE WHEN  at_field_list_test != "" THEN  CONCAT( ",",at_field_list_test) ELSE "" END,
    ", ",bar_chart_metric," AS cdf_metric,",
    CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
    " AS display_name",
    " FROM ", table_name, " tn ", inner_join_stmt,
      " WHERE TIME = '",rightmost_time ,"' "
);

IF set_up_totals IS FALSE AND set_up_averages IS FALSE
THEN
    IF found_key_attribute IS FALSE
    THEN
        SET key_attribute_stmt = CONCAT(" GROUP BY tn.system_id ",
            CASE WHEN ss_entry = "yes" THEN ",tss.subsystem " ELSE "" END,
            CASE WHEN  attribute_list != "" THEN CONCAT(",",attribute_list) ELSE "" END);
    END IF;
    SET select_stmt = CONCAT(" SELECT tn.system_id,",
        CASE WHEN  at_field_list_test != "" THEN  CONCAT( at_field_list_test,",") ELSE "" END,
        CASE WHEN found_key_attribute IS FALSE THEN CONCAT("SUM(",bar_chart_metric,") ") ELSE CONCAT("`",bar_chart_metric,"`") END,
        " AS cdf_metric,",
        CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
        " AS display_name\n",
        " FROM ", table_name, " tn \n", inner_join_stmt,
        "\n WHERE TIME = '",rightmost_time ,"' ",
        key_attribute_stmt,
        " \n  ORDER BY cdf_metric DESC "
    );

    IF in_panel_key = "tpfdf"
    THEN
        SET  at_field_list = CONCAT(
         "  CASE WHEN file_id_ebcdic != '' AND file_name_alias != 'N/A' THEN CONCAT(file_id,'-',file_id_ebcdic,'-',file_name_alias)  ",
         "  \n WHEN file_id_ebcdic != '' THEN CONCAT(file_id,'-',file_id_ebcdic,'-')  ",
         "  \n WHEN file_name_alias != 'N/A' THEN CONCAT(file_id,'--',file_name_alias) ",
         " \n ELSE file_id  END ,'-'");
    END IF;

    IF att_type = ""
    THEN
        SET cmbs_select_clause_str = " SELECT display_name AS systems, JSON_OBJECT(display_name, cdf_metric) AS nvps ";
    ELSE
        SET cmbs_select_clause_str = CONCAT(" SELECT CONCAT(",
                                        at_field_list,
                                        ", display_name) AS systems, ",
                                        "\n  JSON_OBJECT(CONCAT(",
                                        at_field_list,
                                        ", display_name), cdf_metric) AS nvps  ");
    END IF;
    SET cmbs_select_clause_str =
        CONCAT(cmbs_select_clause_str,"\n FROM ordered ",
               CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END);

    SET select_stmt  = CONCAT("WITH ordered AS (\n ", select_stmt, ")\n ",cmbs_select_clause_str);
     CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_bar_chart;
END IF;

 SET totals_ordered_stmt = CONCAT(
        "WITH totals_ordered AS ( SELECT itsitn.display_name",
        CASE WHEN attribute_list != "" THEN CONCAT( ",",attribute_list) ELSE "" END,
         ", SUM(",bar_chart_metric,") AS cdf_metric ",
        " FROM ", table_name, " tn",
        inner_join_stmt,
        " WHERE TIME = '",rightmost_time ,"' ",
        " GROUP BY system_id) "
 );

IF set_up_totals IS TRUE THEN
    SET select_stmt = CONCAT(totals_ordered_stmt,
        " SELECT CONCAT('total-', display_name) AS systems, ",
        " JSON_OBJECT(CONCAT('total-', display_name), cdf_metric) AS nvps ",
        " FROM totals_ordered "
        " UNION ",
        " SELECT 'total-*' AS systems, ",
        " JSON_OBJECT('total-*', SUM(cdf_metric)) AS nvps FROM totals_ordered");
    CALL tpf_execute_immediate(select_stmt);
END IF;
IF set_up_averages IS FALSE THEN
    LEAVE tpf_cdf_bar_chart;
END IF;

SET totals_ordered_stmt = CONCAT(
        "totals_ordered AS ( SELECT display_name",
        CASE WHEN attribute_list != "" THEN CONCAT( ",",attribute_list) ELSE "" END,
        ", SUM(cdf_metric) AS cdf_metric FROM ordered GROUP BY system_id) "
 );

SET select_stmt = CONCAT(
 "WITH ordered AS ( ", ordered_stmt, "), ",totals_ordered_stmt,
        " SELECT CONCAT('avg",
          CASE WHEN att_type != '' THEN  CONCAT("_by-",att_type) ELSE "" END,
        "-', display_name) AS systems, ",
        " JSON_OBJECT(CONCAT('avg",
          CASE WHEN att_type != '' THEN  CONCAT("_by-",att_type) ELSE "" END,
        "-', display_name), AVG(cdf_metric)) AS nvps  FROM ordered GROUP BY system_id",
        " UNION ",
        " SELECT 'avg-*' AS systems, ",
        " JSON_OBJECT('avg-*', AVG(cdf_metric)) AS nvps FROM totals_ordered");

CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_cdf_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_graph`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_panel_key` VARCHAR(64),
    IN `in_metric_type` JSON,
    IN `in_analysis_metric` TEXT,
    IN `in_attribute_types` JSON,
    IN `in_filters_selected` JSON,
    IN `in_max_results` TEXT,
    IN `in_user_input` TEXT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_cdf_graph:BEGIN

DECLARE metric_columns TEXT;
DECLARE table_name VARCHAR(64)  DEFAULT "";
DECLARE lt_time DATETIME;
DECLARE lt_time_shift INT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE num_subsystems INT DEFAULT 0;
DECLARE group_by_time_shift INT;
DECLARE series_name_stmt LONGTEXT DEFAULT "";
DECLARE inner_join_stmt TEXT DEFAULT "";
DECLARE cur_attribute TEXT DEFAULT "";
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE totals_ordered_stmt LONGTEXT DEFAULT "";
DECLARE basic_ordered_stmt LONGTEXT DEFAULT "";
DECLARE ordered_select_stmt LONGTEXT DEFAULT "";
DECLARE ss_entry VARCHAR(64);
DECLARE at_name_list TEXT DEFAULT "";
DECLARE at_name_json JSON;
DECLARE at_field_display  TEXT DEFAULT "";
DECLARE key_name_list TEXT DEFAULT "";
DECLARE at_name_json_list JSON;
DECLARE metric_columns_json JSON;
DECLARE avg_metric_columns TEXT;
DECLARE metric_columns_list TEXT;
DECLARE verify_metric_columns BOOLEAN DEFAULT FALSE;
DECLARE group_by_stmt LONGTEXT DEFAULT "";
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE key_attribute_stmt LONGTEXT DEFAULT "";
DECLARE available_metrics_json_array JSON;
DECLARE max_results_value INT DEFAULT 0;
DECLARE in_show_metrics_contained BOOLEAN DEFAULT TRUE;
DECLARE analysis_function_found TEXT DEFAULT "";
DECLARE attribute_type TEXT DEFAULT "";
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE has_altered_column BOOLEAN DEFAULT FALSE;
DECLARE test_metric TEXT DEFAULT "";
DECLARE systems_columns JSON;
DECLARE only_summarized_metrics BOOLEAN DEFAULT FALSE;
DECLARE where_stmt TEXT DEFAULT "";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_system_ids) = 0 OR JSON_contains(in_system_ids,'[""]') OR
    in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_cdf_graph;
END IF;

IF JSON_LENGTH(in_show_metrics) = 0 OR JSON_contains(in_show_metrics,'[""]') OR
    in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_cdf_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

IF in_panel_key IS NULL OR in_panel_key = ""
THEN
    LEAVE tpf_cdf_graph;
END IF;

SET has_altered_column = tpf_get_altered_column(in_panel_key);
IF  has_altered_column = 0
THEN
   SET select_time_stmt = CONCAT(" WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time));
ELSE
   SET select_time_stmt = CONCAT(" WHERE (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)");
END IF;

SELECT in_max_results REGEXP '[0-9]' INTO @max_results_value;
IF @max_results_value = 1
THEN
    SELECT CAST(in_max_results AS UNSIGNED INT) INTO @max_results_value;
    IF @max_results_value >  65535
    THEN
        LEAVE tpf_cdf_graph;
    END IF;
    SET max_results_value = @max_results_value;
ELSEIF in_max_results != '*'
THEN
    LEAVE tpf_cdf_graph;
END IF;

-- remove the quotes to use the in_analysis_metric in the ORDER BY clause
-- If the analysis metric is like average or totals, assume the user
-- wants the analysis to be based on this metric and remove the suffix.
IF in_analysis_metric != ""
THEN
    SET in_analysis_metric = REPLACE(in_analysis_metric,"'",'');
    -- Remove the averages or totals from the analysis metric
    IF in_analysis_metric LIKE "%-avg_systems" OR
        in_analysis_metric LIKE "%-total_systems"
    THEN
        SET in_analysis_metric = SUBSTRING_INDEX(in_analysis_metric, '-', 1);
    END IF;
    SET in_show_metrics_contained =  JSON_CONTAINS(in_show_metrics,  CONCAT('"',in_analysis_metric,'"'));
END IF;

-- Handle optional parameters
SET num_subsystems = JSON_LENGTH(in_subsystems);

-- Generic function to get the table name
SET table_name = tpf_get_table_name(in_panel_key);

SET cur_attribute = tpf_get_input_attribute_type(in_attribute_types);
IF  cur_attribute != ""
THEN
    SET at_name_list = tpf_get_table_name_field_available_func(in_panel_key);
   SET at_name_json = tpf_get_attributes_available_func(in_panel_key);
    SELECT GROUP_CONCAT(" CASE WHEN ",at_fields,
                     " IS NOT NULL THEN ",at_fields,
                     " ELSE 'N/A' END ")
    INTO  at_field_display
    FROM JSON_TABLE(at_name_json, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
END IF;


SELECT name
INTO attribute_type
FROM JSON_TABLE(in_attribute_types, "$[*]" COLUMNS(name VARCHAR(64) path "$")) ism;

-- validate metric columns
IF  in_analysis_metric LIKE "%-%"
THEN
    SET test_metric = SUBSTRING_INDEX(in_analysis_metric, '-', 1);
ELSE
    SET test_metric = in_analysis_metric;
END IF;
SET in_metric_type = JSON_UNQUOTE(in_metric_type);
SET verify_metric_columns = tpf_validate_metric_columns(in_panel_key, in_show_metrics, in_metric_type, test_metric);
IF verify_metric_columns IS FALSE
THEN
    LEAVE tpf_cdf_graph;
END IF;

-- call tpf_common_graph
SET analysis_function_found = tpf_find_analysis_func(attribute_type, "line_procedure");
IF  analysis_function_found != ""
THEN
    SET @analysis_function_found = analysis_function_found;
    SET @in_panel_key = in_panel_key;
    SET @in_show_metrics = in_show_metrics;
    SET @in_system_ids = in_system_ids;
    SET @in_subsystems = in_subsystems;
    SET @in_rt_time = in_rt_time;
    SET @in_lt_time = in_lt_time;
    SET @ss_entry = ss_entry;
    SET @in_analysis_metric = in_analysis_metric;
    SET @in_metric_type =  in_metric_type;
    SET @in_sec_per_interval = in_sec_per_interval;
    SET @in_agg_type = in_agg_type;
    SET @attribute_type = attribute_type;
    SET @in_user_input = "";
    IF  analysis_function_found LIKE "tpf_user_input_%"
    THEN
        SET @in_user_input = in_user_input;
    END IF;
    SET @in_filters_selected = in_filters_selected;
    --  Call tpf_common_line
    SET select_stmt = CONCAT("CALL ", @analysis_function_found ,
        "(@attribute_type, @in_panel_key, @in_show_metrics, @in_system_ids, @in_subsystems,",
        "@in_lt_time, @in_rt_time, @ss_entry, ",
        "@in_analysis_metric, @max_results_value, @in_metric_type, ",
        " @in_sec_per_interval, @in_agg_type, @in_user_input,",
        "@in_filters_selected)");
    --  use alternate prepare statement in tpf_cdf_execute_immediate
    -- to avoid recursion when  calling tpf_execute_immediate from
    -- the user exits.
    CALL tpf_cdf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_graph;
END IF;

SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
-- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name)) ",
                              " INTO @systems_columns_list",
                              " FROM ", temp_sys_id_table_name);
CALL tpf_execute_immediate(select_stmt);
SET systems_columns = @systems_columns_list;

IF in_agg_type = 'no_agg'
THEN
    --  set up the metric columns
    SELECT GROUP_CONCAT("AVG(`", input_fields ,"`) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';
    SELECT GROUP_CONCAT("`",input_fields,"`")
    INTO  metric_columns_list
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
     SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$")) showSysList');

    -- remove the no_agg metrics and any metrics not like the selected agg type.
    SET select_stmt = CONCAT(
      " SELECT JSON_ARRAYAGG(input_fields)",
      " INTO @metric_columns_json",
      " FROM JSON_TABLE('",in_show_metrics,"', \"$[*]\" COLUMNS(input_fields VARCHAR(128) path \"$\")) showMetrics",
      " WHERE input_fields LIKE '%-",in_agg_type,"';"
    );
    CALL tpf_execute_immediate(select_stmt);
    SET metric_columns_json=@metric_columns_json;

    IF in_agg_type = 'total_systems'
    THEN
        SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", input_fields, "`")
        INTO  metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    ELSE
        SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", SUBSTRING_INDEX(input_fields, '-', 1) , "`")
        INTO  metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
        SELECT GROUP_CONCAT("AVG(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) AS `", input_fields, "`")
        INTO  avg_metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    END IF;
    SELECT GROUP_CONCAT("`",SUBSTRING_INDEX(input_fields, '-', 1),"`")
    INTO  metric_columns_list
    FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    SET in_show_metrics_contained =  JSON_CONTAINS(metric_columns_json,  CONCAT('"',in_analysis_metric,'"'));
END IF;

IF metric_columns IS NULL OR metric_columns=""
THEN
    LEAVE tpf_cdf_graph;
END IF;


-- Calculate time shift
-- Make sure the intervals start with in_rt_time i.e. make the right n (in_sec_per_interval) seconds part of the same interval
SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
-- make sure that the left time bound of data we select is the lt bound of an interval
-- so that SUM over the interval doesn't look lower than it should be.
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);
SET lt_time = DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND);

SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @key_name_list ",
         " FROM information_schema.KEY_COLUMN_USAGE ",
         " WHERE TABLE_NAME = '",table_name,"'",
         " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
CALL tpf_execute_immediate(select_stmt);
SET key_name_list = @key_name_list;
SET found_key_attribute = tpf_check_for_keys_in_attribute_list(in_panel_key,key_name_list);

SET ss_entry = tpf_get_subsystem_applies(in_panel_key);

-- Generic method to get the series name
SET series_name_stmt = tpf_create_series_name_string(num_subsystems,in_agg_type, at_field_display, in_panel_key);

-- Generic method to get the INNER JOIN clause
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, temp_sys_id_table_name, in_subsystems,
    ss_entry, cur_attribute, in_filters_selected,  in_system_ids, NULL);

IF found_key_attribute IS FALSE AND in_agg_type = 'no_agg'
THEN
    SET key_attribute_stmt = tpf_create_key_clause_stmt(in_panel_key,table_name, in_show_metrics,ss_entry,
        at_name_list,inner_join_stmt,lt_time,in_rt_time, group_by_time_shift, in_sec_per_interval, metric_columns);
    CALL tpf_execute_immediate(key_attribute_stmt);
    LEAVE tpf_cdf_graph;
END IF;

-- NOTE : comment duplicated from tpf_system_state_graph
-- Average values over n second intervals to keep from returning too many data points. Grafana will supply n based on the width of the graph and the width of the time window.
-- For no_agg, we need 1 series per system, so include system_id in the GROUP BY. We always use AVG for these metrics. If there are 1 second intervals, this doesn't change the values.
-- For avg_systems, we are already averaging anyway.
-- For total_systems, we want to sum the values for each system for each second of data, then take the average of these over the n seconds per interval.
-- In order to avoid doing AVG(SUM(...)), which would require two queries, we can do SUM(...)/number of timestamps (see tpf_agg_column_for_time_interval)
-- NOTE: average across systems.

IF in_agg_type = 'no_agg'
THEN

    SET only_summarized_metrics = contains_only_summarized_metrics(in_show_metrics);
    SET where_stmt = CONCAT(CASE WHEN max_results_value > 0 AND  only_summarized_metrics IS TRUE
                THEN CONCAT(" WHERE row_num <= ", max_results_value, " AND entry_is_pruned IS TRUE ")
                WHEN only_summarized_metrics IS TRUE THEN " WHERE entry_is_pruned IS TRUE "
                WHEN max_results_value > 0 THEN CONCAT(" WHERE row_num <= ", max_results_value, " " ) ELSE "" END);

    SET group_by_stmt = CONCAT(" GROUP BY system_id, ",
     CASE WHEN ss_entry = 'yes' THEN "subsystem_id, " ELSE "" END,
      CASE WHEN at_name_list != '' THEN CONCAT(at_name_list, ",") ELSE "" END,
     "(UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval);
     SET in_analysis_metric = CONCAT("`",in_analysis_metric,"`");
    SET select_stmt =
    CONCAT(
        " WITH ordered AS \n (SELECT MIN(time) AS time, ",
        series_name_stmt,
        metric_columns,
        CASE WHEN in_show_metrics_contained IS FALSE THEN  CONCAT(",",in_analysis_metric, " ") ELSE "" END,
        ", \n ROW_NUMBER() OVER (PARTITION BY TIME ORDER BY TIME DESC ",
        CASE WHEN in_analysis_metric != "" THEN CONCAT(",", in_analysis_metric, " DESC ") ELSE "" END,
        " ) AS row_num ",
        CASE WHEN only_summarized_metrics IS TRUE THEN ", entry_is_pruned " ELSE "" END,
        " FROM ", table_name, " tn \n",
        inner_join_stmt,
    " ", select_time_stmt,
        group_by_stmt,
        " ORDER BY time ASC",
        ") \n",
        " SELECT time, series_name, \n ",
        metric_columns_list,
        " FROM ordered ", where_stmt
    );
    CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_graph;
END IF;
-- We are here either for averages or totals.
SET group_by_stmt = CONCAT(" GROUP BY (UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval);

SELECT name
INTO attribute_type
FROM JSON_TABLE(in_attribute_types, "$[*]" COLUMNS(name VARCHAR(64) path "$")) ism;

-- Obtain the INNER JOIN clause again. The temp_sys_id_table_name cannot be reused in
-- a UNION.
 SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
       cur_attribute, in_filters_selected, in_system_ids, systems_columns);
--  avg_by-operation and total for each processor
SET ordered_select_stmt =
    CONCAT(
        " WITH ordered AS (\n SELECT  time, system_id,  itsitn.display_name,",
        metric_columns_list,
        CASE WHEN attribute_type != "" THEN  CONCAT(",`",attribute_type, "` ") ELSE "" END,
        ", \n ROW_NUMBER() OVER (PARTITION BY TIME ORDER BY TIME DESC ",
        CASE WHEN in_analysis_metric != "" THEN CONCAT(",`", in_analysis_metric, "` DESC ") ELSE "" END,
        " ) AS row_num \n",
        " FROM ", table_name, " tn \n",
        inner_join_stmt, " \n ",
        select_time_stmt," \n ",
        " ORDER BY time ASC",
        CASE WHEN in_analysis_metric != "" THEN CONCAT(",`", in_analysis_metric, "` DESC ") ELSE "" END,
        ") \n"
  );
SET totals_ordered_stmt =
    CONCAT("\n ,totals_ordered AS ",
    " (\nSELECT  ",
    " ordered.display_name as systems, ",
    metric_columns,
    " , MIN(TIME) AS TIME, ordered.system_id \n",
    " FROM ordered \n",
    group_by_stmt,
    " , ordered.system_id ",
    ")\n ");

    SET basic_ordered_stmt =
        CONCAT("\n ,basic_ordered AS ",
        " (\nSELECT  ",
        " ordered.display_name as systems, ",
        metric_columns,
        " , MIN(TIME) AS TIME, ordered.system_id \n",
        " FROM ordered \n",
        group_by_stmt,
        " , ordered.system_id ",
        CASE WHEN attribute_type != "" THEN  CONCAT(",ordered.`",attribute_type, "` ") ELSE "" END,
        ")\n ");

SET select_stmt =
 CONCAT(
    " SELECT MIN(time) AS time, \n",
    series_name_stmt, "\n ",
    metric_columns,
    "\n FROM ", table_name, " tn \n",
    inner_join_stmt, "\n ",
    select_time_stmt, "\n ",
    group_by_stmt,
    CASE WHEN in_agg_type = 'avg_systems' THEN ", tn.system_id " ELSE "" END
 );
 IF in_agg_type = 'total_systems'
 THEN
    SET select_stmt = CONCAT(select_stmt,
     "\n UNION \n",
     " SELECT MIN(TIME),",
     "  CONCAT('total-',ordered.display_name) as systems, ",
     metric_columns,
     " FROM  ordered ",
     group_by_stmt,
     ", system_id ",
     "\n ORDER BY time ASC");
    SET select_stmt = CONCAT(ordered_select_stmt, select_stmt);
    CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_cdf_graph;
 END IF;

-- Here for  avg_systems
 -- For a request for average, generate the select statement like this is the totals.
 -- From there, calculate the average.
-- Add the attribute name to the avg, for example, avg_by-device_name-C
SET select_stmt =
    CONCAT(ordered_select_stmt,  totals_ordered_stmt, basic_ordered_stmt,
    " SELECT time, 'avg-systems' AS series_name,\n ",
    avg_metric_columns,
    "\n FROM totals_ordered \n",
    " GROUP BY totals_ordered.time \n",
    " \n UNION \n",
    " SELECT MIN(TIME) as time, \n",
    " CONCAT('avg",
    CASE WHEN attribute_type != '' THEN  CONCAT("_by-",attribute_type) ELSE "" END,
    "-',basic_ordered.systems) as systems, \n ",
    avg_metric_columns,
    " FROM basic_ordered \n",
    " GROUP BY basic_ordered.time, basic_ordered.system_id \n",
    " ORDER BY time ASC "
    );
CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;
