/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2022, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
/*   20240716   PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Stored procedures common to all databases           */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

-- MySQL doesn't support EXECUTE IMMEDIATE syntax. Replicate with this procedure.
DROP PROCEDURE IF EXISTS `tpf_execute_immediate`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_execute_immediate` (
    IN `in_query` LONGTEXT
)

tpf_execute_immediate:BEGIN

    SET @dynamic_query = in_query;
    PREPARE dynamic_stmt FROM @dynamic_query;
    EXECUTE dynamic_stmt;
    DEALLOCATE PREPARE dynamic_stmt;

END//
DELIMITER ;

-- MySQL doesn't support JSON_VALUE until 8.0.21. Replace with this function.
DROP FUNCTION IF EXISTS `tpf_json_value`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_json_value` (
    `json_doc` JSON,
    `path` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
    RETURN JSON_UNQUOTE(JSON_EXTRACT(json_doc, path));
END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_does_table_exist`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_does_table_exist` (
    IN `in_database` VARCHAR(128),
    IN `in_table_name` VARCHAR(128),
    OUT `out_table_exists` BOOLEAN
)

tpf_does_table_exist:BEGIN
-- Checks if a table exists in the database
-- Input Parameters:
--     in_database          the name of the database
--     in_table_name        the name of the table
-- Output Parameters:
--     out_table_exists     TRUE of the table exists in the database, false otherwise

    SELECT EXISTS(
        SELECT 1
        FROM information_schema.TABLES
        WHERE TABLE_SCHEMA=in_database AND TABLE_NAME=in_table_name
        LIMIT 1
    ) INTO out_table_exists;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_does_column_exist`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_does_column_exist` (
    `database_name` VARCHAR(128),
    `db_table_name` VARCHAR(128),
    `col_name` VARCHAR(128)
)
RETURNS BOOLEAN
READS SQL DATA
BEGIN
    DECLARE column_exists BOOLEAN;

    SELECT EXISTS (
        SELECT 1
        FROM information_schema.COLUMNS
        WHERE TABLE_SCHEMA=database_name AND TABLE_NAME=db_table_name AND COLUMN_NAME=col_name
    ) INTO column_exists;

    RETURN column_exists;
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_column_type`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_column_type` (
    `database_name` VARCHAR(128),
    `db_table_name` VARCHAR(128),
    `col_name` VARCHAR(128)
)
RETURNS LONGTEXT
READS SQL DATA
BEGIN
    DECLARE col_type LONGTEXT;

    SELECT COLUMN_TYPE
    FROM information_schema.COLUMNS
    WHERE TABLE_SCHEMA=database_name AND TABLE_NAME=db_table_name AND COLUMN_NAME=col_name
    INTO col_type;

    RETURN col_type;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_remove_elements_from_json_array`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_remove_elements_from_json_array` (
    `in_json_array` TEXT,
    `in_remove_json_array` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE i INT DEFAULT 0;
DECLARE result TEXT;
DECLARE temp TEXT;

IF in_remove_json_array IS NULL OR in_remove_json_array = ""
THEN
   RETURN in_json_array;
END IF;

SET result = in_json_array;
WHILE i < JSON_LENGTH(in_remove_json_array) DO
   SELECT JSON_EXTRACT(in_remove_json_array,CONCAT('$[',i,']')) INTO @temp;
   IF @temp IS NOT NULL AND @temp!="null" THEN
      SET @temp = JSON_UNQUOTE(JSON_SEARCH(result,'one',JSON_UNQUOTE(@temp)));
      IF @temp IS NOT NULL THEN
         SET result = JSON_REMOVE(result,@temp);
      END IF;
   END IF;
   SELECT i + 1 INTO i;
END WHILE;
RETURN result;
END//
DELIMITER ;
