/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2022, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20230719    PJ46904 - User-Defined Metrics                       */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Stored procedures common to all databases           */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP PROCEDURE IF EXISTS `tpf_system_state_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_system_state_graph` (
    IN `in_tpf_systems` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_system_state_graph:BEGIN

DECLARE metric_columns TEXT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

DECLARE working_statement TEXT;

-- DECLARE EXIT HANDLER FOR SQLEXCEPTION    BEGIN        ROLLBACK;    END;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_tpf_systems,'[""]') OR in_tpf_systems='""' OR in_tpf_systems="" OR in_tpf_systems="[]"
THEN
    LEAVE tpf_system_state_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_system_state_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

IF in_agg_type = 'no_agg'
THEN
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table("system_state_graph_temp_selected_systems", in_tpf_systems);
    SET temp_sys_id_table_name = "system_state_graph_temp_selected_systems";

    -- remove agg metrics
    SELECT JSON_ARRAYAGG(metric_col)
    INTO in_show_metrics
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) showMetrics
    WHERE metric_col NOT LIKE "%-total_systems" AND metric_col NOT LIKE "%-avg_systems";
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_tpf_systems), ', "$[*]" COLUMNS(id INT path "$"))');

    -- remove the no_agg metrics
    SET working_statement = CONCAT(
      " SELECT JSON_ARRAYAGG(metric_col)",
      " INTO @in_show_metrics",
      " FROM JSON_TABLE('",in_show_metrics,"', \"$[*]\" COLUMNS(metric_col VARCHAR(128) path \"$\")) showMetrics",
      " WHERE metric_col LIKE '%-",in_agg_type,"';"
    );
    CALL tpf_execute_immediate(working_statement);
    SET in_show_metrics=@in_show_metrics;
END IF;

IF in_show_metrics IS NULL OR in_show_metrics=""
THEN
    LEAVE tpf_system_state_graph;
END IF;
SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, 's.time', in_agg_type,"system_metrics_table");

IF metric_columns IS NULL OR metric_columns=""
THEN
    LEAVE tpf_system_state_graph;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

CALL tpf_execute_immediate(CONCAT(
    " SELECT MIN(s.time) AS time,",
                  -- By having a non-numeric column, grafana will display one series per distinct value (per numeric metric column) in this column.
             CASE WHEN in_agg_type = 'no_agg' THEN "t.display_name AS 'system',"
                  -- In the above case for 'no_agg', we will have 1 series per system selected. The 'system' column will have the display_name for each of the selected systems.
                  -- In the agg cases, we agg all the selected systems, so there is only 1 series. We don't need the system column in this case. The series names
                  -- will look like the metric with agg type: actual_cpu-avg_systems, inuse_ecb-total_systems
                  ELSE ""
             END,
             metric_columns,
    " FROM ", temp_sys_id_table_name, " t INNER JOIN system_metrics_table s ON t.id=s.system_id",
    " WHERE s.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    -- Average values over n second intervals to keep from returning too many data points. Grafana will supply n based on the width of the graph and the width of the time window.
    -- For no_agg, we need 1 series per system, so include system_id in the GROUP BY. We always use AVG for these metrics. If there are 1 second intervals, this doesn't change the values.
    -- For avg_systems, we are already averaging anyway.
    -- For total_systems, we want to sum the values for each system for each second of data, then take the average of these over the n seconds per interval.
    -- In order to avoid doing AVG(SUM(...)), which would require two queries, we can do SUM(...)/number of timestamps (see tpf_agg_column_for_time_interval)
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "s.system_id, " ELSE "" END, "(UNIX_TIMESTAMP(s.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
));



DROP TEMPORARY TABLE IF EXISTS system_state_graph_temp_selected_systems;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_nvp_metrics_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_nvp_metrics_graph` (
    IN `in_tpf_systems` JSON,
    IN `in_dimension_value_ids` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_nvp_metrics_graph:BEGIN

DECLARE metric_columns TEXT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;
DECLARE where_col_list LONGTEXT;

DECLARE display_statement TEXT;
DECLARE working_statement TEXT;

-- DECLARE EXIT HANDLER FOR SQLEXCEPTION    BEGIN        ROLLBACK;    END;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_tpf_systems,'[""]') OR in_tpf_systems='""' OR in_tpf_systems="" OR in_tpf_systems="[]"
THEN
    LEAVE tpf_nvp_metrics_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_nvp_metrics_graph;
END IF;

-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

IF in_agg_type = 'no_agg'
THEN
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table("nvp_metrics_graph_temp_selected_systems", in_tpf_systems);
    SET temp_sys_id_table_name = "nvp_metrics_graph_temp_selected_systems";

    -- remove agg metrics
    SELECT JSON_ARRAYAGG(metric_col)
    INTO @in_show_metrics
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) showMetrics
    WHERE metric_col NOT LIKE "%-total_systems" AND metric_col NOT LIKE "%-avg_systems";
ELSE
    -- This creates a table of selected system ids which we will join to the nvp_message_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_tpf_systems), ', "$[*]" COLUMNS(id INT path "$"))');

    -- remove the no_agg metrics
    SET working_statement = CONCAT(
      " SELECT JSON_ARRAYAGG(metric_col)",
      " INTO @in_show_metrics",
      " FROM JSON_TABLE('",in_show_metrics,"', \"$[*]\" COLUMNS(metric_col VARCHAR(128) path \"$\")) showMetrics",
      " WHERE metric_col LIKE '%-",in_agg_type,"';"
    );
    CALL tpf_execute_immediate(working_statement);
    SET in_show_metrics=@in_show_metrics;
END IF;


IF in_show_metrics IS NULL OR in_show_metrics=""
THEN
    LEAVE tpf_nvp_metrics_graph;
END IF;
SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, 'm.time', in_agg_type,"nvp_message_metrics_table");
IF metric_columns IS NULL OR metric_columns=""
THEN
    LEAVE tpf_nvp_metrics_graph;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

    SET display_statement =
    CONCAT(
    " SELECT MIN(m.time) AS time,",
             CASE WHEN in_agg_type = 'no_agg' THEN "CONCAT_WS('-', v.value, t.display_name) AS series_name,"
                  ELSE "v.value AS series_name,"
             END,
             metric_columns,
    " FROM ", temp_sys_id_table_name, " t INNER JOIN nvp_message_metrics_table m ON t.id=m.system_id",
           " INNER JOIN JSON_TABLE(", QUOTE(in_dimension_value_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) sv ON m.value_id=sv.id INNER JOIN app_dimension_values v ON sv.id=v.id",
    " WHERE m.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "m.system_id, " ELSE "" END, "m.value_id, (UNIX_TIMESTAMP(m.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
);
CALL tpf_execute_immediate(display_statement);

DROP TEMPORARY TABLE IF EXISTS nvp_metrics_graph_temp_selected_systems;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_nvp_metrics_bar_chart`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_nvp_metrics_bar_chart` (
    IN `in_tpf_systems` JSON,
    IN `in_dimension_value_ids` JSON,
    IN `in_bar_chart_metric` VARCHAR(128),
    IN `in_rt_time` DATETIME,
    IN `in_sec_to_avg` INT
)

tpf_nvp_metrics_bar_chart:BEGIN

DECLARE bar_chart_metric VARCHAR(128);
SET bar_chart_metric = CONCAT('n.',in_bar_chart_metric);
IF bar_chart_metric="n."
THEN
    LEAVE tpf_nvp_metrics_bar_chart;
END IF;

CALL tpf_create_system_display_name_temp_table("nvp_metrics_bar_chart_temp_selected_systems", in_tpf_systems);

CALL tpf_execute_immediate(CONCAT(
" WITH nvps_by_system AS (",
"     SELECT system_id,",
"            JSON_OBJECTAGG(v.value, lv.nvp_metric) AS nvps",
"     FROM (",
"         SELECT n.system_id, n.value_id, AVG(",bar_chart_metric,") AS nvp_metric",
"         FROM nvp_message_metrics_table n",
"              INNER JOIN JSON_TABLE('",in_tpf_systems,"', \"$[*]\" COLUMNS(id INT path \"$\")) its ON n.system_id=its.id",
"              INNER JOIN JSON_TABLE('",in_dimension_value_ids,"', \"$[*]\" COLUMNS(id INT path \"$\")) idvi ON n.value_id=idvi.id",
"         WHERE TIME BETWEEN DATE_SUB('",in_rt_time,"', INTERVAL '",in_sec_to_avg,"' SECOND) AND '",in_rt_time,"'",
"         GROUP BY system_id, value_id",
"     ) lv",
"     INNER JOIN app_dimension_values v ON lv.value_id=v.id",
"     GROUP BY lv.system_id",
" )",
" SELECT ts.display_name AS 'system', nbs.nvps",
" FROM nvps_by_system nbs",
"      INNER JOIN nvp_metrics_bar_chart_temp_selected_systems ts ON nbs.system_id=ts.id",
-- Grafana currently has a bug in their Bar Chart. If there is only 1 group of bars (in this case, we group by system)
-- Then the chart won't show the group. This ensures we always have a minimum of two groups.
-- When we upgrade to a version where this is fixed then we can get rid of this.
" UNION",
" SELECT '' AS 'system', '{}' AS 'nvps'",
" FROM nvps_by_system",
" WHERE 1=(SELECT COUNT(*) FROM nvps_by_system);"
));

DROP TEMPORARY TABLE IF EXISTS nvp_metrics_bar_chart_temp_selected_systems;


END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_complex_create_selected_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_complex_create_selected_metrics` (
    `complex_show_metrics` JSON,
    `time_col` VARCHAR(64),
    `in_agg_type` ENUM('no_agg', 'avg_systems', 'total_systems'),
    `in_table_name` VARCHAR(64)
)
RETURNS TEXT
DETERMINISTIC
BEGIN
    DECLARE metric_columns TEXT;
    DECLARE agg_type_functions JSON DEFAULT '
    {
        "avg_systems": "AVG",
        "total_systems": "SUM",
        "no_agg": "AVG"
    }
    ';

    WITH selected_metrics AS (
        SELECT metric AS metric_with_agg_type,
               -- system cpu metrics need to be divided by 10 to become a percent.
               -- est_msg_cpu and cpu_used are nvp metrics that are not cpu percentages.
               -- They are total micro seconds used by a message.
               CASE WHEN metric LIKE "%-avg_systems"  THEN SUBSTRING_INDEX(metric, '-', 1)
                    WHEN metric LIKE "%-total_systems"  THEN SUBSTRING_INDEX(metric, '-', 1)
                    ELSE metric
               END AS metric_col,
               CASE WHEN metric LIKE "%-avg_systems"  THEN "avg_systems"
                    WHEN metric LIKE "%-total_systems"  THEN "total_systems"
                    ELSE "no_agg"
               END AS agg_type,
               tpf_json_value(agg_type_functions, (SELECT CONCAT('$.',agg_type))) AS agg_function
        FROM JSON_TABLE(complex_show_metrics, "$[*]" COLUMNS(metric VARCHAR(64) path "$")) metrics
    )
    -- For no_agg, we want this: SELECT AVG(`actual_cpu`) AS 'actual_cpu', AVG(workload_cpu) AS workload_cpu;
    -- For avg_systems we want this: SELECT AVG(`actual_cpu`) AS 'actual_cpu-avg_systems', AVG(workload_cpu) AS 'workload_cpu-avg_systems';
    -- For total_systems we want this: SELECT SUM(`est_msg_rate`)/num timestamps AS 'est_msg_rate-total_systems'
    SELECT GROUP_CONCAT(tpf_agg_column_for_time_interval(agg_function, metric_col, metric_with_agg_type, time_col,in_table_name))
    INTO metric_columns
    FROM selected_metrics
    WHERE agg_type=in_agg_type;

    RETURN metric_columns;

END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_agg_column_for_time_interval`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_agg_column_for_time_interval` (
    `agg_function` VARCHAR(32),
    `col_name` VARCHAR(64),
    `col_alias` VARCHAR(64),
    `time_col` VARCHAR(64),
    `in_table_name` VARCHAR(64)
)
RETURNS TEXT
DETERMINISTIC
BEGIN

    IF in_table_name = "system_metrics_table"
    THEN
        -- this calculation only applies to cpu metrics in the system metrics table
        SET col_name = CONCAT("`",col_name,"`",IF(col_name NOT LIKE '%est_msg_cpu%' AND col_name LIKE '%_cpu%', '/10', ''));
    ELSEIF col_name  != "*"
    THEN
        SET col_name = CONCAT("`",col_name,"`");
    END IF;

    IF agg_function IS NULL OR agg_function = ""
    THEN
        RETURN CONCAT(col_name, " AS ", QUOTE(col_alias));
    ELSEIF agg_function = "SUM"
    THEN
        RETURN CONCAT("SUM(", col_name, ")/COUNT(DISTINCT ", time_col, ") AS ", QUOTE(col_alias));
    ELSE
        RETURN CONCAT(agg_function, "(", col_name, ") AS ", QUOTE(col_alias));
    END IF;
END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_get_table_name`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_table_name` (
    `in_panel_key` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE table_metrics JSON;
DECLARE table_name_return JSON;

SET table_metrics = tpf_get_metrics_json();

SELECT DISTINCT JSON_QUOTE(table_name) INTO table_name_return FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_panel_key;

RETURN JSON_UNQUOTE(table_name_return);

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_get_metrics_types`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_metrics_types` (
    `in_type` TEXT
)
tpf_get_metrics_types:BEGIN

DECLARE table_metrics JSON;
DECLARE metrics_result JSON;
DECLARE result JSON;
DECLARE tpfdf_result JSON;
DECLARE available_results JSON;

SET table_metrics = tpf_get_metrics_json();
SELECT DISTINCT JSON_ARRAYAGG(metric_type)
INTO metrics_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type;

-- get the list of metric profiles.
SET available_results = tpf_get_metric_profiles_available_func();
SELECT JSON_ARRAYAGG(name)
INTO tpfdf_result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.name != "" AND jt.datatype = in_type;

IF tpfdf_result IS NOT NULL
THEN
    SET result = JSON_MERGE_PRESERVE(metrics_result,tpfdf_result);
ELSE
    SET result = metrics_result;
END IF;

SELECT DISTINCT at_fields
FROM JSON_TABLE(result, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_extra_cols_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_extra_cols_available_func` (
    `in_type` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE table_metrics JSON;
DECLARE json_array_extras_result TEXT;
SET table_metrics = tpf_get_metrics_json();
SELECT JSON_ARRAYAGG(extra_cols) INTO json_array_extras_result FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type AND
      jt.extra_cols != "null";
RETURN json_array_extras_result;
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_metrics_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_metrics_available_func` (
    `in_type` TEXT,
    `in_metric_types` JSON
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE table_metrics JSON;
DECLARE json_array_metrics_result TEXT;
DECLARE available_results JSON;
DECLARE tpfdf_result JSON;
DECLARE result JSON;

SET table_metrics = tpf_get_metrics_json();
SELECT JSON_ARRAYAGG(metrics) INTO json_array_metrics_result FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type AND
      JSON_CONTAINS(
        in_metric_types,
        CONCAT('"',jt.metric_type,'"')
      ) AND
      jt.metrics!="null";

IF json_array_metrics_result IS NULL
THEN
    SET json_array_metrics_result = '[]';
END IF;

-- get the list of metric profiles.
SET available_results = tpf_get_metric_profiles_available_func();
SELECT JSON_ARRAYAGG(metrics)
INTO tpfdf_result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.metrics IS NOT NULL
AND jt.datatype = in_type
AND JSON_CONTAINS(in_metric_types,CONCAT('"',jt.name,'"')) ;

IF tpfdf_result IS NOT NULL
THEN
    SET result = JSON_MERGE_PRESERVE(json_array_metrics_result,tpfdf_result);
ELSE
    SET result = json_array_metrics_result;
END IF;

RETURN result;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_virtual_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_virtual_metrics` (
    `in_type` TEXT,
    `in_metric_types` JSON
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE i INT DEFAULT 0;
DECLARE table_metrics JSON;
DECLARE json_array_virtual_metrics_result TEXT;
DECLARE result TEXT;
DECLARE temp TEXT;
DECLARE available_results JSON;
DECLARE tpfdf_result JSON;

SET table_metrics = tpf_get_metrics_json();

SELECT JSON_ARRAYAGG(virtual_metrics)
INTO json_array_virtual_metrics_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type AND
      JSON_CONTAINS(
        in_metric_types,
        CONCAT('"',jt.metric_type,'"')
      ) AND
      jt.virtual_metrics!="";

IF json_array_virtual_metrics_result IS NULL
THEN
    SET json_array_virtual_metrics_result = '[]';
END IF;

-- get the list of metric profiles.
SET available_results = tpf_get_metric_profiles_available_func();
SELECT JSON_ARRAYAGG(virtual_metrics)
INTO tpfdf_result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.virtual_metrics IS NOT NULL
AND jt.datatype = in_type
AND JSON_CONTAINS(in_metric_types,CONCAT('"',jt.name,'"')) ;

IF tpfdf_result IS NOT NULL
THEN
    SET result = JSON_MERGE_PRESERVE(json_array_virtual_metrics_result,tpfdf_result);
ELSE
    SET result = json_array_virtual_metrics_result;
END IF;

RETURN result;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_subsystem_applies`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_subsystem_applies` (
    `in_type` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE table_metrics JSON;
DECLARE subsystem_applies_indicator TEXT;

SET table_metrics = tpf_get_metrics_json();
SELECT DISTINCT  subsystem_applies
INTO subsystem_applies_indicator
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type;

RETURN subsystem_applies_indicator;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_table_name_field_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_table_name_field_available_func` (
    `in_type` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN
DECLARE table_metrics JSON;
DECLARE json_array_result TEXT;
SET table_metrics = tpf_get_metrics_json();
SELECT GROUP_CONCAT(CASE WHEN attribute_table = "" THEN "tn" ELSE attribute_table END ,".",attribute_name)
INTO json_array_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type  AND
      jt.attribute_name != "";
RETURN json_array_result;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_attributes_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_attributes_available_func` (
    `in_type` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE table_metrics JSON;
DECLARE json_array_result JSON;
SET table_metrics = tpf_get_metrics_json();
SELECT JSON_ARRAYAGG(attribute_name)
INTO json_array_result
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_type  AND
      jt.attribute_name != "";
RETURN json_array_result;
END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_get_metrics_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_metrics_available` (
    IN `in_type` TEXT,
    IN `in_metric_types` JSON
)
tpf_get_metrics_available:BEGIN

DECLARE available_metrics_json_array TEXT;
SET available_metrics_json_array = tpf_get_metrics_available_func(in_type,in_metric_types);
SELECT metrics FROM JSON_TABLE(available_metrics_json_array,'$[*]'
  COLUMNS(
    metrics VARCHAR(64) PATH '$'
  )
) AS jt;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_nvp_metrics_dimensions_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_nvp_metrics_dimensions_available` (
    IN `in_tpf_systems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_nvp_metrics_dimensions_available:BEGIN

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

WITH selected_systems AS (
    SELECT n.value_id
    FROM nvp_message_metrics_table n INNER JOIN JSON_TABLE(in_tpf_systems, "$[*]" COLUMNS(id INT path "$")) l ON n.system_id=l.id
    WHERE n.TIME BETWEEN in_lt_time AND in_rt_time
    GROUP BY n.value_id
)
SELECT dimension AS __text, v.dimension_id AS __value
FROM app_dimensions d INNER JOIN app_dimension_values v ON d.id=v.dimension_id
WHERE d.dimension != 'TCPIP' AND EXISTS (
    SELECT 1
    FROM selected_systems s
    WHERE s.value_id=v.id
    LIMIT 1
)
GROUP BY v.dimension_id
ORDER BY dimension;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_nvp_metrics_dimension_values_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_nvp_metrics_dimension_values_available` (
    IN `in_tpf_systems` JSON,
    IN `in_dimension_ids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_nvp_metrics_dimensions_available:BEGIN

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

WITH selected_systems AS (
    SELECT n.value_id
    FROM nvp_message_metrics_table n INNER JOIN JSON_TABLE(in_tpf_systems, "$[*]" COLUMNS(id INT path "$")) l ON n.system_id=l.id
    WHERE n.TIME BETWEEN in_lt_time AND in_rt_time
    GROUP BY n.value_id
)
SELECT v.value AS __text, v.id AS __value
FROM app_dimension_values v INNER JOIN JSON_TABLE(in_dimension_ids, "$[*]" COLUMNS(id INT path "$")) idi ON v.dimension_id=idi.id
WHERE EXISTS (
    SELECT 1
    FROM selected_systems s
    WHERE s.value_id=v.id
    LIMIT 1
)
ORDER BY v.value;


END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_calc_group_by_time_shift`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_calc_group_by_time_shift` (
    `rt_time` DATETIME,
    `sec_per_interval` INT
)
RETURNS INT
DETERMINISTIC
BEGIN
    -- Make sure the intervals start with in_rt_time i.e. make the right n (in_sec_per_interval) seconds part of the same interval
    RETURN sec_per_interval - 1 - MOD(UNIX_TIMESTAMP(rt_time), sec_per_interval);
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_calc_lt_time_shift`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_calc_lt_time_shift` (
    `lt_time` DATETIME,
    `sec_per_interval` INT,
    `group_by_time_shift` INT
)
RETURNS INT
DETERMINISTIC
BEGIN
    -- make sure that the left time bound of data we select is the lt bound of an interval so that SUM over the interval doesn't look lower than it should be.
    RETURN MOD(UNIX_TIMESTAMP(lt_time), sec_per_interval) + group_by_time_shift;
END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_create_system_display_name_temp_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_create_system_display_name_temp_table` (
    IN `in_temp_tpf_systems_table_name` VARCHAR(64),
    IN `in_tpf_systems_list` JSON
)

tpf_create_system_display_name_temp_table:BEGIN
-- Creates a temporary table with columns id, display_name. The display name is the
-- fewest number of columns in tpf_systems that, when concatenated, uniquely identify all the tpf_systems
-- in in_tpf_systems_list.
-- Input Parameters:
--     in_temp_tpf_systems_table_name  The name of the temporary table to be created.
--     in_tpf_systems_list      A JSON list of tpf_system ids. Example: '[1, 2, 3]' or '["1", "2", "3"]'

DECLARE num_systems_selected INT;

SET num_systems_selected = JSON_LENGTH(in_tpf_systems_list);

DROP TEMPORARY TABLE IF EXISTS temp_tpf_systems_create_procedure_table;

-- Figure out the most concise way to uniquely identify each tpf system. Make the series name as short as possible in Grafana
IF (SELECT COUNT(DISTINCT processor_id) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, processor_id AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT system_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, system_name AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, ip_address AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT complex_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, complex_name AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, system_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, system_name) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, complex_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, complex_name) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT system_name, complex_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', system_name, complex_name) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT system_name, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', system_name, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT complex_name, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', complex_name, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, system_name, complex_name) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, system_name, complex_name) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, system_name, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, system_name, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT processor_id, complex_name, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', processor_id, complex_name, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSEIF (SELECT COUNT(DISTINCT system_name, complex_name, ip_address) FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id) = num_systems_selected
THEN
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', system_name, complex_name, ip_address) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
ELSE
    CREATE TEMPORARY TABLE temp_tpf_systems_create_procedure_table AS
    SELECT s.id, CONCAT_WS('~', system_name, complex_name, processor_id, ip_address, environment, cpc_name) AS display_name
    FROM tpf_systems s INNER JOIN JSON_TABLE(in_tpf_systems_list, "$[*]" COLUMNS(id INT path "$")) l ON s.id=l.id;
END IF;


CALL tpf_execute_immediate(CONCAT(
    " DROP TEMPORARY TABLE IF EXISTS ", in_temp_tpf_systems_table_name
));

CALL tpf_execute_immediate(CONCAT(
    " ALTER TABLE temp_tpf_systems_create_procedure_table RENAME TO ",
    in_temp_tpf_systems_table_name
));



END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_pruning_variations_json`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_pruning_variations_json` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE pruning_variations JSON;

SET pruning_variations = '
  [
  ]';

RETURN pruning_variations;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_variations_json`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_variations_json` (
        `in_panel_key` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;
DECLARE variations JSON;
DECLARE altered_column_result INT DEFAULT 0;
DECLARE table_metrics JSON;

SET variations = '
  [
	  {"display_string":"per_second",
	   "metric_suffix":""
	   },
	  {"display_string":"multi-system",
	   "metric_suffix":"-total_systems"
	   },
	  {"display_string":"multi-system",
	   "metric_suffix":"-avg_systems"
	   }
  ]';

IF in_panel_key != ""
THEN
    SET table_metrics = tpf_get_metrics_json();
    SELECT COUNT(*)
    INTO altered_column_result
    FROM json_table(table_metrics, '$[*]'
    COLUMNS(
        panel_key         VARCHAR(64) PATH '$.panel_key',
        dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
        subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
        table_name        VARCHAR(64) PATH '$.table_name',
        NESTED PATH '$.attribute_columns[*]' COLUMNS (
          attribute_name VARCHAR(64) PATH '$.attribute_name',
        attribute_table VARCHAR(64) PATH '$.attribute_table',
        table_column VARCHAR(64) PATH '$.table_column'
        ),
        metric_type       VARCHAR(64) PATH '$.metric_type',
        NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.virtual_metrics[*]' COLUMNS (
        virtual_metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.extra_cols[*]' COLUMNS (
        extra_cols VARCHAR(64) PATH '$'
        ),
        altered_column   VARCHAR(64) PATH '$.altered_column'
    )
    ) AS jt
    WHERE jt.panel_key != ""  AND
          jt.panel_key=in_panel_key AND
          jt.altered_column != "" AND
          jt.altered_column != "no";
END IF;

IF altered_column_result = 0
THEN
    SET result = JSON_MERGE(variations,tpf_get_pruning_variations_json());
ELSE
    SET result = variations;
END IF;

RETURN result;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_get_variations`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_variations` (
    IN `in_panel_key` TEXT
)
tpf_get_variations:BEGIN

DECLARE variation_details JSON;

SET variation_details = tpf_get_variations_json(in_panel_key);
IF in_panel_key="Lock_DD"
THEN
   SELECT DISTINCT display_string FROM json_table(variation_details, '$[*]'
     COLUMNS(
       display_string VARCHAR(64) PATH '$.display_string',
       metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
     )
   ) AS jt
   WHERE display_string!="multi-system";
ELSE
   SELECT DISTINCT display_string FROM json_table(variation_details, '$[*]'
COLUMNS(
    display_string VARCHAR(64) PATH '$.display_string',
    metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
)
) AS jt;
END IF;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_get_show_metrics_list`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_show_metrics_list` (
    IN `in_type` TEXT,
    IN `in_metric_types` JSON,
    IN `in_variation_types` TEXT
)
tpf_get_show_metrics_list:BEGIN


DECLARE variation_details JSON;
DECLARE available_metrics_json_array JSON;
DECLARE avaialble_virtual_metrics_json_array JSON;
DECLARE metrics_array JSON;


IF LOCATE("[",in_metric_types)=0
THEN
   SET in_metric_types = JSON_ARRAY(JSON_UNQUOTE(in_metric_types));
END IF;

IF LOCATE("[",in_variation_types)=0
THEN
   SET in_variation_types = JSON_ARRAY(JSON_UNQUOTE(in_variation_types));
END IF;

SET variation_details = tpf_get_variations_json(in_type);

SET metrics_array = tpf_get_metrics_available_func(in_type,in_metric_types);
SET avaialble_virtual_metrics_json_array =  tpf_get_virtual_metrics(in_type,in_metric_types);
IF avaialble_virtual_metrics_json_array IS NOT NULL AND JSON_LENGTH(avaialble_virtual_metrics_json_array) > 0
THEN
    SET available_metrics_json_array = JSON_MERGE(metrics_array, avaialble_virtual_metrics_json_array);
ELSE
    SET available_metrics_json_array = metrics_array;
END IF;

SELECT CONCAT(jt1.metric_name, jt2.metric_suffix) AS metric
FROM json_table(available_metrics_json_array, '$[*]'
     COLUMNS(
        metric_name VARCHAR(64) PATH '$'
     )
   ) AS jt1
CROSS JOIN json_table(variation_details, '$[*]'
  COLUMNS(
    display_string VARCHAR(64) PATH '$.display_string',
    metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
  )
) AS jt2
WHERE jt2.display_string IN (
  SELECT jt3.variation
  FROM json_table(in_variation_types, '$[*]'
  COLUMNS(
      variation VARCHAR(64) PATH '$'
  )
) AS jt3);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_get_table_metrics_list`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_table_metrics_list` (
    IN `in_type` TEXT,
    IN `in_metric_types` JSON,
    IN `in_variation_types` JSON
)
tpf_get_table_metrics_list:BEGIN


DECLARE variation_details JSON;
DECLARE available_metrics_json_array JSON;
DECLARE avaialble_virtual_metrics_json_array JSON;
DECLARE metrics_virtual_array JSON;
DECLARE metrics_array JSON;
DECLARE results JSON;
DECLARE variation_types_num_elements INT DEFAULT 0;
DECLARE search_result TEXT DEFAULT "";


IF LOCATE("[",in_metric_types)=0
THEN
   SET in_metric_types = JSON_ARRAY(JSON_UNQUOTE(in_metric_types));
END IF;

IF LOCATE("[",in_variation_types)=0
THEN
   SET in_variation_types = JSON_ARRAY(JSON_UNQUOTE(in_variation_types));
END IF;

SET variation_details = tpf_get_variations_json(in_type);

--
-- NOTE: If they have only selected multi-system,
-- this does not make sense in the table
-- because there are no columns to show.
-- The multi-system information is always the bottom rows anyway.
-- Because we want to show something, we are just going to default
-- to per_second in the multi-system case.
--
SET search_result =  JSON_SEARCH(in_variation_types, 'one', 'multi-system');
IF search_result IS NOT NULL
THEN
    SET variation_types_num_elements = JSON_LENGTH(in_variation_types);
    IF variation_types_num_elements = 1
    THEN
        SET in_variation_types = JSON_ARRAY("per_second");
    ELSE
SET in_variation_types = tpf_remove_elements_from_json_array(in_variation_types, JSON_ARRAY("multi-system"));
    END IF;
END IF;

SET available_metrics_json_array = tpf_get_metrics_available_func(in_type,in_metric_types);
SET avaialble_virtual_metrics_json_array =  tpf_get_virtual_metrics(in_type,in_metric_types);
IF avaialble_virtual_metrics_json_array IS NOT NULL AND JSON_LENGTH(avaialble_virtual_metrics_json_array) > 0
THEN
    SET metrics_array = JSON_MERGE(available_metrics_json_array, avaialble_virtual_metrics_json_array);
ELSE
    SET metrics_array = available_metrics_json_array;
END IF;

SELECT JSON_ARRAYAGG(CONCAT(jt1.metric_name, jt2.metric_suffix)) AS metric
 INTO results
FROM json_table(metrics_array, '$[*]'
     COLUMNS(
        metric_name VARCHAR(64) PATH '$'
     )
   ) AS jt1
CROSS JOIN json_table(variation_details, '$[*]'
  COLUMNS(
    display_string VARCHAR(64) PATH '$.display_string',
    metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
  )
) AS jt2
WHERE jt2.display_string IN (
  SELECT jt3.variation
  FROM json_table(in_variation_types, '$[*]'
  COLUMNS(
      variation VARCHAR(64) PATH '$'
  )
) AS jt3);

 IF results IS NOT NULL  AND  JSON_LENGTH(results) > 0
 THEN
    SELECT result_entry
    FROM JSON_TABLE(results, "$[*]" COLUMNS(result_entry VARCHAR(64) path "$")) ism
    WHERE result_entry != "";
 ELSE
    SELECT "None";
 END IF;

END//
DELIMITER ;
