/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2022, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20220131    PJ46608 - multi-JVM and system selector support      */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20230719    PJ46904 - User-Defined Metrics                       */
/*  20231015    PJ47106 - Display multiple Kafka producers           */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     JVM related stored procedures                       */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP FUNCTION IF EXISTS `tpf_get_jvm_table_name`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jvm_table_name` (
    `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App', 'JVM_GC')
)
RETURNS VARCHAR(128)
DETERMINISTIC
BEGIN
    DECLARE panel_key_table_mappings JSON DEFAULT '
    {
        "JAM_Sum": "jvm_jvm_metrics_table",
        "JVM_Sum": "jvm_jvm_metrics_table",
        "JVM_DD": "jvm_jvm_metrics_table",
        "Lock_DD": "jvm_lock_metrics_table",
        "JAX-RS_DD": "jvm_jam_jax_rs_metrics_table",
        "JAX-RS-Op_DD": "jvm_jam_jax_rs_op_metrics_table",
        "JVM_GC": "jvm_jvm_metrics_table"
    }';

    RETURN tpf_json_value(panel_key_table_mappings, CONCAT("$.\"", in_panel_key, "\""));

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_cpcs_complexes_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_cpcs_complexes_available` (
    IN `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App', 'JVM_GC'),
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(128)
)

tpf_jvm_cpcs_complexes_available:BEGIN

DECLARE table_name VARCHAR(128);

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- Do everything in UTC time.
SET time_zone = '+00:00';

IF in_panel_key = 'JMX_App'
THEN
    IF in_jmx_app = ''
    THEN
        SELECT 1 FROM DUAL WHERE FALSE;
        LEAVE tpf_jvm_cpcs_complexes_available;
    END IF;

    SET table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");
ELSE
    SET table_name = tpf_get_jvm_table_name(in_panel_key);
END IF;

-- __text, __value creates key-value pairs in Grafana dropdown.
CALL tpf_execute_immediate(CONCAT(
    " WITH cpcs_complexes_available AS (",
        " SELECT ts.cpc_name, ts.complex_name",
        " FROM tpf_systems ts INNER JOIN tpf_jams tj ON ts.id=tj.system_id",
        " WHERE EXISTS (",
            " SELECT 1",
            " FROM ", table_name, " jjmt",
            " WHERE tj.id=jjmt.jam_id AND TIME BETWEEN '", in_lt_time, "' AND '", in_rt_time, "'",
            " LIMIT 1",
        " ) ",
    " )",
    " (SELECT CONCAT('CPC: ', cpc_name) AS __text, cpc_name AS __value",
    "  FROM cpcs_complexes_available",
    "  GROUP BY cpc_name)",
    " UNION ALL",
    " (SELECT CONCAT('Complex: ', complex_name) AS __text, complex_name AS __value",
    "  FROM cpcs_complexes_available",
    "  GROUP BY complex_name)",
    " ORDER BY __text"
));


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_systems_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_systems_available` (
    IN `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App', 'JVM_GC'),
    IN `in_cpc_complexes` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(128)
)

tpf_jvm_systems_available:BEGIN

DECLARE table_name VARCHAR(128);
DECLARE select_stmt LONGTEXT DEFAULT "";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF in_panel_key = 'JMX_App'
THEN
    IF in_jmx_app = ''
    THEN
        SELECT 1 FROM DUAL WHERE FALSE;
        LEAVE tpf_jvm_systems_available;
    END IF;

    SET table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");
ELSE
    SET table_name = tpf_get_jvm_table_name(in_panel_key);
END IF;

-- __text, __value create key-value pairs in Grafana dropdown. This way, Grafana passes the system ids back
-- to the stored procedures
SET select_stmt =
CONCAT(
    " SELECT CONCAT_WS('~', system_name, complex_name, processor_id, ip_address, environment, cpc_name) AS __text,",
           " ts.id AS __value",
    " FROM tpf_systems ts INNER JOIN tpf_jams tj ON ts.id=tj.system_id",
    " WHERE EXISTS (",
        " SELECT 1",
        " FROM ", table_name, " jjmt",
        " WHERE tj.id=jjmt.jam_id AND TIME BETWEEN '", in_lt_time, "' AND '", in_rt_time, "'",
        " LIMIT 1",
    " ) ", CASE WHEN in_cpc_complexes != '' THEN CONCAT(" AND (cpc_name IN (", in_cpc_complexes, ") OR complex_name IN (", in_cpc_complexes, "))") ELSE "" END,
    " GROUP BY ts.id"
);
CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_subsystems_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_subsystems_available` (
    IN `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App', 'JVM_GC'),
    IN `in_system_ids` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(128)
)

tpf_jvm_subsystems_available:BEGIN

DECLARE table_name VARCHAR(128);

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF in_panel_key = 'JMX_App'
THEN
    IF in_jmx_app = ''
    THEN
        SELECT 1 FROM DUAL WHERE FALSE;
        LEAVE tpf_jvm_subsystems_available;
    END IF;

    SET table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");
ELSE
    SET table_name = tpf_get_jvm_table_name(in_panel_key);
END IF;


CALL tpf_execute_immediate(CONCAT(
    " SELECT tj.subsystem",
    " FROM tpf_systems ts INNER JOIN tpf_jams tj ON ts.id=tj.system_id",
    " WHERE EXISTS (",
        " SELECT 1",
        " FROM ", table_name, " jjmt",
        " WHERE tj.id=jjmt.jam_id AND TIME BETWEEN '", in_lt_time, "' AND '", in_rt_time, "'",
        " LIMIT 1",
    " ) ", CASE WHEN in_system_ids != '' THEN CONCAT(" AND tj.system_id IN (", in_system_ids, ")") ELSE "" END,
    " GROUP BY tj.subsystem"
));


END//
DELIMITER ;




DROP PROCEDURE IF EXISTS `tpf_jvm_jams_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_jams_available` (
    IN `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App', 'JVM_GC'),
    IN `in_system_ids` TEXT,
    IN `in_subsystems` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(128)
)

tpf_jvm_jams_available:BEGIN

DECLARE table_name VARCHAR(128);

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF in_panel_key = 'JMX_App'
THEN
    IF in_jmx_app = ''
    THEN
        SELECT 1 FROM DUAL WHERE FALSE;
        LEAVE tpf_jvm_jams_available;
    END IF;

    SET table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");
ELSE
    SET table_name = tpf_get_jvm_table_name(in_panel_key);
END IF;

CALL tpf_execute_immediate(CONCAT(
    " SELECT tj.jam",
    " FROM tpf_systems ts INNER JOIN tpf_jams tj ON ts.id=tj.system_id",
    " WHERE EXISTS (",
        " SELECT 1",
        " FROM ", table_name, " jjmt",
        " WHERE tj.id=jjmt.jam_id AND TIME BETWEEN '", in_lt_time, "' AND '", in_rt_time, "'",
        " LIMIT 1",
    " ) ", CASE WHEN in_system_ids != '' THEN CONCAT(" AND tj.system_id IN (", in_system_ids, ")") ELSE "" END,
    CASE WHEN in_subsystems != '' THEN CONCAT(" AND tj.subsystem IN (", in_subsystems, ")") ELSE "" END,
    " GROUP BY tj.jam"
));

END//
DELIMITER ;

-- Dumping structure for procedure TPFDataSciDB.tpf_jmx_apps_available
DROP PROCEDURE IF EXISTS `tpf_jvm_jmx_apps_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_jmx_apps_available`(
    IN `in_lt_time` VARCHAR(50),
    IN `in_rt_time` VARCHAR(50)
)
tpf_jvm_jmx_apps_available:BEGIN
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Determines the JMX applications available.          */
/*                                                                   */
/* Description:  JMX applications are recorded in individual tables  */
/*               within TPFDataSciDB. These tables are prefixed by   */
/*               'jvm_jmx_app_' and suffixed by '_table'. This       */
/*               procedure finds all available applications in the   */
/*               time range and strips the prefix and suffix         */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is Grafana.                          */
/*                                                                   */
/* Input Parameters:                                                 */
/*       None.                                                       */
/*                                                                   */
/* Tables accessed:                                                  */
/*      information_schema.TABLES and JMX tables prefixed by         */
/*      "jvm_jmx_app"                                                */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Result:                                                           */
/*      JMX application names within the given time range            */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*                                                                   */
/*********************************************************************/

DECLARE lt_time DATETIME;
DECLARE rt_time DATETIME;

DECLARE n INT DEFAULT 0;
DECLARE i INT DEFAULT 0;

DECLARE current_database VARCHAR(64);

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SET lt_time = STR_TO_DATE(in_lt_time, '%Y-%m-%d %H:%i:%s');
SET rt_time = STR_TO_DATE(in_rt_time, '%Y-%m-%d %H:%i:%s');

--  Error conditions. Since this will populate the drop-down menus in grafana and could be passed to other
--  stored procedures (as "in_selectors" or "in_show_groups") we'll use only one error for simplicity (same as other JVM procedures).
IF lt_time IS NULL OR rt_time IS NULL OR lt_time > rt_time
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
    LEAVE tpf_jvm_jmx_apps_available;
END IF;

DROP TEMPORARY TABLE IF EXISTS temp_jmx_tables_available;
CREATE TEMPORARY TABLE temp_jmx_tables_available (
    `jmx_table_name` VARCHAR(256) DEFAULT NULL,
    `is_data_available` BOOLEAN DEFAULT NULL
) ENGINE=InnoDB;

SELECT DATABASE() INTO current_database;

--  Find all the tables with the 'jvm_jmx_app' prefix
INSERT INTO temp_jmx_tables_available (jmx_table_name)
    SELECT TABLE_NAME
      FROM information_schema.TABLES
      WHERE TABLE_SCHEMA=current_database AND TABLE_NAME LIKE 'jvm_jmx_app%';

SELECT COUNT(*) FROM temp_jmx_tables_available INTO n;

--  Loop through each table we found
WHILE i<n DO
	SELECT jmx_table_name INTO @jmx_table FROM temp_jmx_tables_available LIMIT i,1;

	--  Is there any data in this jmx_table in the time range?
	CALL tpf_execute_immediate(CONCAT(
	    "UPDATE temp_jmx_tables_available",
	    " SET is_data_available=(SELECT EXISTS(SELECT 1 FROM ", @jmx_table, " WHERE time BETWEEN '", lt_time, "' AND '", rt_time, "' LIMIT 1))",
	    " WHERE jmx_table_name='",@jmx_table,"'"
	));
	SET i = i + 1;
END WHILE;

SELECT NOT EXISTS(SELECT 1
  FROM temp_jmx_tables_available
  WHERE is_data_available
  LIMIT 1) INTO @is_none_available;

IF @is_none_available
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
ELSE
    --  We will identify tables for JMX applications in the database with
    --  table names prefixed by "jvm_jmx_app_". We will strip the prefix
    --  and suffix in order to get the app name
    SELECT TRIM(LEADING 'jvm_jmx_app_' FROM TRIM(TRAILING '_table' FROM jmx_table_name))
      FROM temp_jmx_tables_available
      WHERE is_data_available;
END IF;

DROP TEMPORARY TABLE IF EXISTS temp_jmx_tables_available;

END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_get_jvm_metrics_json`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jvm_metrics_json` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;
SET result = '
  [
	  {"panel_key":"JAM_Sum",
	   "metric_type":"primary",
       "metrics":["cpu",
                  "avg_response_time",
                  "max_jam_threads_observed",
                  "num_jam_threads_inuse",
                  "num_jam_threads_total",
                  "num_jvms_actual",
                  "num_jvms_expected"
	             ],
       "virtual_metrics":[]
      },
	  {"panel_key":"JVM_Sum",
	   "metric_type":"primary",
       "metrics":["cpu",
                  "avg_gc_interval_global",
                  "avg_gc_pause_global",
                  "avg_response_time",
                  "heap_size"
	             ],
       "virtual_metrics":[]
      },
	  {"panel_key":"JVM_DD",
	   "metric_type":"primary",
       "metrics":["avg_gc_interval",
                  "avg_gc_interval_global",
                  "avg_gc_pause",
                  "avg_gc_pause_global",
                  "heap_size",
                  "num_jvm_threads_daemon",
                  "num_jvm_threads_latest",
                  "used_heap"
	             ],
       "virtual_metrics":[]
      },
	  {"panel_key":"Lock_DD",
	   "metric_type":"primary",
       "metrics":["gets",
                  "gets_rec",
                  "gets_slow",
                  "hold_time",
                  "miss_pct",
                  "util_pct"
  	             ],
       "virtual_metrics":[]
      },
	  {"panel_key":"JAX-RS_DD",
	   "metric_type":"primary",
       "metrics":["app_faults_checked",
                  "app_faults_unchecked",
                  "avg_response_time",
                  "handling_time",
                  "max_response_time",
                  "min_response_time",
                  "runtime_faults_app",
                  "runtime_faults_logical"
  	             ],
       "virtual_metrics":[]
      },
	  {"panel_key":"JAX-RS-Op_DD",
	   "metric_type":"primary",
       "metrics":["app_faults_checked",
                  "app_faults_unchecked",
                  "avg_response_time",
                  "handling_time",
                  "max_response_time",
                  "min_response_time",
                  "runtime_faults_app",
                  "runtime_faults_logical"
  	             ],
       "virtual_metrics":[]
      }
  ]';

RETURN result;


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_get_jvm_metrics_types`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_jvm_metrics_types` (
    `in_type` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App')
)
tpf_get_jvm_metrics_types:BEGIN

DECLARE jvm_table_metrics JSON;

SET jvm_table_metrics = tpf_get_jvm_metrics_json();

SELECT DISTINCT metric_type FROM json_table(jvm_table_metrics, '$[*]'
  COLUMNS(
    panel_key    VARCHAR(64) PATH '$.panel_key',
    metric_type  VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.panel_key=in_type;

END//
DELIMITER ;

-- Dumping structure for procedure TPFDataSciDB.tpf_jvm_table_metrics_available
DROP PROCEDURE IF EXISTS `tpf_jvm_table_metrics_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_table_metrics_available`(
    IN `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App'),
    IN `in_jmx_app` VARCHAR(128),
    IN `in_metric_types` JSON,
    IN `in_variation_types` JSON
)
tpf_jvm_table_metrics_available:BEGIN
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Determines the metrics available for a dashboard.   */
/*                                                                   */
/* Description:  Finds all the metrics (columns) available for a     */
/*               given JVM dashboard to display.                     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is Grafana.                          */
/*                                                                   */
/* Input Parameters:                                                 */
/*       in_dashboard_name      - Grafana dashboard/panel name       */
/*       in_panel_key           - Used by JMX_App dashboard to pass  */
/*                                the app name. Used to reconstruct  */
/*                                the DB table name                  */
/*                                                                   */
/* Tables accessed:                                                  */
/*      information_schema.COLUMNS                                   */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Result:                                                           */
/*      JVM database table metric names                              */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*                                                                   */
/*********************************************************************/

DECLARE jvm_table_name VARCHAR(100);
DECLARE current_database VARCHAR(64);

DECLARE jvm_table_metrics JSON;
DECLARE variation_details JSON;
DECLARE variation_metric_suffixes TEXT;
-- DECLARE available_metrics_json_array TEXT;
DECLARE available_metrics_json_array JSON;

DECLARE jmx_metric_columns TEXT;

IF LOCATE("[",in_variation_types) = 0
THEN
    SET in_variation_types = JSON_ARRAY(JSON_UNQUOTE(in_variation_types));
END IF;
SET jvm_table_metrics = tpf_get_jvm_metrics_json();

--  Error condition for JMX App dashboard.
IF in_panel_key = 'JMX_App' AND (in_jmx_app IS NULL OR in_jmx_app = '')
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
    LEAVE tpf_jvm_table_metrics_available;
END IF;

IF in_panel_key = 'JMX_App'
THEN
    SELECT DATABASE() INTO current_database;

    SET variation_details = tpf_get_variations_json("");

--  JMX data does not have it's columns in JSON like the other IBM tables do.
--  So we have to interrogate the table definition to get the metric columns.

--  Our final result is cross join of our desired variations
--  so for example, suppose our columns are a, b,... and use selected variation max and multi system
--  Our metric list will be: a-max_per_second, a-avg_system,b-max_per_second, b-avg_system, ...
--  CONCAT merges the variation (from json list) and column name (from scehma definition)
    SELECT CONCAT(t1.COLUMN_NAME, jt2.metric_suffix) AS metric
    FROM information_schema.COLUMNS AS t1
    CROSS JOIN json_table(variation_details, '$[*]'
      COLUMNS(
        display_string VARCHAR(64) PATH '$.display_string',
        metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
      )
    ) AS jt2
    WHERE t1.TABLE_SCHEMA=current_database
    AND t1.TABLE_NAME=CONCAT("jvm_jmx_app_", in_jmx_app, "_table")
    AND t1.DATA_TYPE IN ('int', 'double', 'bigint', 'mediumint', 'smallint', 'tinyint')
    AND NOT t1.COLUMN_KEY='PRI'
--  This block removes the columns like a-max_per_second as we'll be adding those in the CONCAT above
    AND NOT SUBSTRING_INDEX(t1.COLUMN_NAME, '-', -1) IN  (
        SELECT SUBSTRING_INDEX(jt4.metric_suffix, '-', -1)
        FROM json_table(variation_details, '$[*]'
           COLUMNS(
             display_string VARCHAR(64) PATH '$.display_string',
             metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
           )
        ) AS jt4
        WHERE NOT display_string="per_second"
        )
    AND NOT t1.COLUMN_NAME="entry_is_pruned"
    AND NOT jt2.metric_suffix="-total_systems"
--  This block limits the variations we'll use to what the user selected.
    AND jt2.display_string IN (
      SELECT jt3.variation
      FROM json_table(in_variation_types, '$[*]'
      COLUMNS(
          variation VARCHAR(64) PATH '$'
      )
    ) AS jt3);

    LEAVE tpf_jvm_table_metrics_available;
END IF;

SET variation_details = tpf_get_variations_json("");
SET available_metrics_json_array = tpf_get_jvm_metrics_available_func(in_panel_key,in_metric_types);
SELECT CONCAT(jt1.metric_name, jt2.metric_suffix) AS metric
FROM json_table(available_metrics_json_array, '$[*]'
     COLUMNS(
        metric_name VARCHAR(64) PATH '$'
     )
   ) AS jt1
CROSS JOIN json_table(variation_details, '$[*]'
  COLUMNS(
    display_string VARCHAR(64) PATH '$.display_string',
    metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
  )
) AS jt2
WHERE NOT jt2.metric_suffix="-total_systems"
AND jt2.display_string IN (
  SELECT jt3.variation
  FROM json_table(in_variation_types, '$[*]'
  COLUMNS(
      variation VARCHAR(64) PATH '$'
  )
) AS jt3);

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_jvm_metrics_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jvm_metrics_available_func` (
    `in_panel_key` ENUM('JAM_Sum', 'JVM_Sum', 'JVM_DD', 'Lock_DD', 'JAX-RS_DD', 'JAX-RS-Op_DD', 'JMX_App'),
    `in_metric_types` JSON
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE json_array_metrics_result TEXT;
DECLARE jvm_table_metrics JSON;

SET jvm_table_metrics = tpf_get_jvm_metrics_json();


    SELECT JSON_ARRAYAGG(metrics) INTO json_array_metrics_result FROM json_table(jvm_table_metrics, '$[*]'
      COLUMNS(
        panel_key    VARCHAR(64) PATH '$.panel_key',
        metric_type  VARCHAR(64) PATH '$.metric_type',
        NESTED PATH '$.metrics[*]' COLUMNS (
          metrics VARCHAR(64) PATH '$'
        ),
        NESTED PATH '$.virtual_metrics[*]' COLUMNS (
          virtual_metrics VARCHAR(64) PATH '$'
        )
      )
    ) AS jt
    WHERE jt.panel_key=in_panel_key AND
          JSON_CONTAINS(
             in_metric_types,
             CONCAT('"',jt.metric_type,'"')
          ) AND
          jt.metrics!="null";

    RETURN json_array_metrics_result;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_jvm_pids_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_pids_available`(
    IN `in_panel_key` ENUM('JVM_Sum', 'JVM_DD', 'Lock_DD', 'JMX_App'),
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(128)
)
tpf_jvm_pids_available:BEGIN

DECLARE table_name VARCHAR(128);
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
    LEAVE tpf_jvm_pids_available;
END IF;

IF in_panel_key = 'JMX_App'
THEN
    IF in_jmx_app = ''
    THEN
        SELECT 1 FROM DUAL WHERE FALSE;
        LEAVE tpf_jvm_pids_available;
    END IF;

    SET table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");
ELSE
    SET table_name = tpf_get_jvm_table_name(in_panel_key);
END IF;

CALL tpf_execute_immediate(CONCAT(
    " SELECT pid",
    " FROM ", table_name, " j INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) ji ON j.jam_id=ji.id",
    " WHERE time BETWEEN '", in_lt_time, "' AND '", in_rt_time, "'",
    " GROUP BY pid"
));

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_locks_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_locks_available`(
    IN `in_system_id` INT,
    IN `in_subsystem` VARCHAR(4),
    IN `in_jam` VARCHAR(32),
    IN `in_pid` INT,
    IN `in_focus_metric` TEXT,
    IN `in_lock_limit` INT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)
tpf_jvm_locks_available:BEGIN

DECLARE working_statement TEXT;
DECLARE jam_id_lookup INT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SELECT id
INTO jam_id_lookup
FROM tpf_jams
WHERE system_id=in_system_id AND subsystem=in_subsystem AND jam=in_jam;

SET working_statement=CONCAT(
  " WITH ordered AS ( ",
  " SELECT l.lock, ROW_NUMBER() OVER (PARTITION BY l.lock ORDER BY l.time DESC) AS row_num,",
  " l.", in_focus_metric, " AS focus_col",
  " FROM jvm_lock_metrics_table l",
  " WHERE l.jam_id=", jam_id_lookup,
  " AND pid=",in_pid,
  " AND time BETWEEN '", in_lt_time, "' AND '", in_rt_time, "' )",
  " SELECT o.lock",
  " FROM ordered o",
  " WHERE row_num = 1",
  " ORDER BY focus_col DESC",
  " LIMIT ", in_lock_limit
);
CALL tpf_execute_immediate(working_statement);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_jvm_jax_rs_apps_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_jax_rs_apps_available`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)
tpf_jvm_jax_rs_apps_available:BEGIN

DECLARE table_name VARCHAR(128);
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
    LEAVE tpf_jvm_jax_rs_apps_available;
END IF;

SELECT app
FROM jvm_jam_jax_rs_metrics_table jr INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) ji ON jr.jam_id=ji.id
WHERE jr.time BETWEEN in_lt_time AND in_rt_time
GROUP BY app;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_jvm_jax_rs_operations_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_jax_rs_operations_available`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_apps` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)
tpf_jvm_jax_rs_operations_available:BEGIN

DECLARE table_name VARCHAR(128);
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    SELECT 1 FROM DUAL WHERE FALSE;
    LEAVE tpf_jvm_jax_rs_operations_available;
END IF;

WITH ops_selected_apps AS (
    SELECT jro.op_id
    FROM jvm_jam_jax_rs_op_metrics_table jro INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) ji ON jro.jam_id=ji.id
                                             INNER JOIN JSON_TABLE(in_apps, "$[*]" COLUMNS(app VARCHAR(128) path "$")) ia ON jro.app=ia.app
    WHERE jro.TIME BETWEEN in_lt_time AND in_rt_time
    GROUP BY jro.op_id
)
SELECT jops.operation AS __text, jops.id AS __value
FROM jvm_operations jops
WHERE EXISTS (
    SELECT 1
    FROM ops_selected_apps osa
    WHERE osa.op_id=jops.id
    LIMIT 1
)
ORDER BY jops.operation;

END//
DELIMITER ;




DROP FUNCTION IF EXISTS `tpf_get_jam_ids`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jam_ids` (
    `system_ids` JSON,
    `subsystems` JSON,
    `jams` JSON
)
RETURNS JSON
DETERMINISTIC
BEGIN
    DECLARE jam_ids JSON;

    SELECT JSON_ARRAYAGG(tj.id)
    INTO jam_ids
    FROM tpf_jams tj INNER JOIN JSON_TABLE(system_ids, "$[*]" COLUMNS(id INT path "$")) si ON tj.system_id=si.id
                     INNER JOIN JSON_TABLE(subsystems, "$[*]" COLUMNS(subsystem VARCHAR(4) path "$")) ss ON tj.subsystem=ss.subsystem
                     INNER JOIN JSON_TABLE(jams, "$[*]" COLUMNS(jam VARCHAR(32) path "$")) j ON tj.jam=j.jam;

    RETURN jam_ids;
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_num_non_jam_apps`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_num_non_jam_apps` (
    `jam_ids` JSON
)
RETURNS INT
DETERMINISTIC
BEGIN
    DECLARE num_non_jam_apps INT;

    SELECT COUNT(NULLIF(tj.is_jam_app,1))
    INTO num_non_jam_apps
    FROM tpf_jams tj INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) ji ON tj.id=ji.id;

    RETURN num_non_jam_apps;
END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_jam_summary_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jam_summary_graph`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_show_metrics` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_jam_summary_graph:BEGIN

DECLARE num_subsystems INT;
DECLARE num_non_jam_apps INT;
DECLARE jam_ids JSON;

DECLARE metric_columns TEXT;
DECLARE non_jam_app_metric_columns TEXT;
DECLARE temp_sys_id_table_name LONGTEXT;
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

DECLARE select_stmt LONGTEXT DEFAULT "";

DECLARE cpu_window_func_calc VARCHAR(128) DEFAULT "AVG(AVG(`cpu`)) OVER (PARTITION BY j.jam_id ORDER BY UNIX_TIMESTAMP(j.time) RANGE BETWEEN 1 PRECEDING AND 1 FOLLOWING) AS cpu";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_system_ids,'[""]') OR in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_jam_summary_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_jam_summary_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    LEAVE tpf_jam_summary_graph;
END IF;

SET num_non_jam_apps = tpf_get_num_non_jam_apps(jam_ids);
SET num_subsystems = JSON_LENGTH(in_subsystems);

IF in_agg_type = 'no_agg'
THEN
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table("jam_summary_graph_temp_selected_systems", in_system_ids);

    -- Playing weird games to avoid referring to a temp table twice in the large query that follows
    SELECT CONCAT(
        "JSON_TABLE(",
        QUOTE(JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name))), -- Get the table rows as JSON
        ", '$[*]' COLUMNS(id INT path '$.id', display_name VARCHAR(100) path '$.display_name'))"
    )
    INTO temp_sys_id_table_name
    FROM jam_summary_graph_temp_selected_systems;
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

IF in_agg_type = 'no_agg'
THEN
    -- Need to aggregate differently whether the PIDS are JAM apps or non-JAM apps
    -- CPU events come once per 3 seconds per PID. Within a 3 second window for a JAM, we should have one CPU value per PID.
    -- These values could be at any 1 of the 3 time stamps within the window, so we need to average over minimum 3 second windows
    -- within the overall time range in order to accurately display the CPU graph for a JAM. If CPU is selected and in_sec_per_interval
    -- is less than 3, we'll use a window function to get the 3 second average
    SELECT GROUP_CONCAT(
        CASE WHEN metric LIKE 'num_%' OR metric LIKE 'max_%' THEN tpf_agg_column_for_time_interval('MAX', metric, metric, 'j.time', 'jvm_jvm_metrics_table')
             WHEN in_sec_per_interval < 3 AND metric = 'cpu' THEN cpu_window_func_calc
             ELSE tpf_agg_column_for_time_interval('AVG', metric, metric, 'j.time', 'jvm_jvm_metrics_table')
        END
    ),
    GROUP_CONCAT(
        CASE WHEN metric = 'num_jvms_actual' THEN tpf_agg_column_for_time_interval('COUNT', '*', metric, 'j.time', 'jvm_jvm_metrics_table')
             WHEN metric LIKE 'num_%' OR metric LIKE 'max_%' THEN tpf_agg_column_for_time_interval('SUM', metric, metric, 'j.time', 'jvm_jvm_metrics_table')
             WHEN in_sec_per_interval < 3 AND metric = 'cpu' THEN cpu_window_func_calc
             ELSE tpf_agg_column_for_time_interval('AVG', metric, metric, 'j.time', 'jvm_jvm_metrics_table')
        END
    )
    INTO metric_columns, non_jam_app_metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric VARCHAR(64) path "$")) metrics
    WHERE metric NOT LIKE "%-avg_systems" AND metric NOT LIKE "%-total_systems" ;

    IF metric_columns IS NULL
    THEN
        LEAVE tpf_jam_summary_graph;
    END IF;

    SET select_stmt = CONCAT(
        " WITH sys_id_table_cte AS (",
            " SELECT *",
            " FROM ", temp_sys_id_table_name, " tsitn",
        " )",
        " SELECT MIN(j.time) AS time,",
                 -- Only need the subsystem when there is more than 1.
                 -- This is no_agg, so we need the system display_name
                 CASE WHEN num_subsystems > 1 THEN "CONCAT_WS('-', tj.jam, tj.subsystem, tsi.display_name) AS series_name,"
                      ELSE "CONCAT_WS('-', tj.jam, tsi.display_name) AS series_name,"
                 END,
                 metric_columns,
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id",
                         " INNER JOIN sys_id_table_cte tsi ON tj.system_id=tsi.id",
                         " INNER JOIN jvm_jvm_metrics_table j ON tj.id=j.jam_id",
        " WHERE j.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time), " AND tj.is_jam_app",
        " GROUP BY j.jam_id, (UNIX_TIMESTAMP(j.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval
    );

    IF num_non_jam_apps = 0
    THEN
        SET select_stmt = CONCAT(
            select_stmt,
            " ORDER BY time ASC"
        );
    ELSE
        SET select_stmt = CONCAT(
            select_stmt,
            " UNION ALL",
            " SELECT MIN(j.time) AS time,",
                     CASE WHEN num_subsystems > 1 THEN "CONCAT_WS('-', tj.jam, tj.subsystem, tsi.display_name) AS series_name,"
                          ELSE "CONCAT_WS('-', tj.jam, tsi.display_name) AS series_name,"
                     END,
                     non_jam_app_metric_columns,
            " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id",
                             " INNER JOIN sys_id_table_cte tsi ON tj.system_id=tsi.id",
                             " INNER JOIN jvm_jvm_metrics_table j ON tj.id=j.jam_id",
            " WHERE j.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time), " AND NOT tj.is_jam_app",
            " GROUP BY j.jam_id, (UNIX_TIMESTAMP(j.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
            " ORDER BY time ASC"
        );
    END IF;
    CALL tpf_execute_immediate(select_stmt);
ELSE
    SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, 'j.time', in_agg_type,"jvm_jvm_metrics_table");

    IF metric_columns IS NULL
    THEN
        LEAVE tpf_jam_summary_graph;
    END IF;

    IF in_sec_per_interval < 3
    THEN
        SET metric_columns = REPLACE(metric_columns, "AVG(`cpu`) AS 'cpu-avg_systems'", "AVG(AVG(`cpu`)) OVER (PARTITION BY j.jam_id, j.pid ORDER BY UNIX_TIMESTAMP(j.time) RANGE BETWEEN 1 PRECEDING AND 1 FOLLOWING) AS 'cpu-avg_systems'");
    END IF;

    CALL tpf_execute_immediate(CONCAT(
        " SELECT MIN(j.time) AS time,",
                 CASE WHEN num_subsystems > 1 THEN "CONCAT_WS('-', tj.jam, tj.subsystem) AS series_name,"
                      ELSE "tj.jam AS series_name,"
                 END,
                 metric_columns,
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id", -- jam_id filters selected systems, subsystems, and jams
                         " INNER JOIN ", temp_sys_id_table_name, " tsi ON tj.system_id=tsi.id", -- Get the system display_name, which is the shortest concatenation of system cols to uniquely identify
                         " INNER JOIN jvm_jvm_metrics_table j ON tj.id=j.jam_id", -- Get the metrics
        " WHERE j.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
        " GROUP BY tj.subsystem, tj.jam, (UNIX_TIMESTAMP(j.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
        " ORDER BY time ASC"
    ));
END IF;


DROP TEMPORARY TABLE IF EXISTS jam_summary_graph_temp_selected_systems;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_jvm_summary_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_summary_graph`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems'),
    IN `in_jmx_app` VARCHAR(64)
)

tpf_jvm_summary_graph:BEGIN

DECLARE jam_ids JSON;
DECLARE num_subsystems INT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE metric_columns TEXT;
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;
DECLARE jvm_table_name VARCHAR(64);

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_system_ids,'[""]') OR in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_jvm_summary_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_jvm_summary_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    LEAVE tpf_jvm_summary_graph;
END IF;

SET num_subsystems = JSON_LENGTH(in_subsystems);

IF in_agg_type = 'no_agg'
THEN
    SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;

SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, 'j.time', in_agg_type,"jvm_table_name");

IF metric_columns IS NULL
THEN
    LEAVE tpf_jvm_summary_graph;
END IF;

SET jvm_table_name = CASE WHEN in_jmx_app != '' THEN CONCAT("jvm_jmx_app_", in_jmx_app, "_table") ELSE 'jvm_jvm_metrics_table' END;

-- In this case, we are aggregating cpu values for PIDs. CPU values come once per 3 seconds, so the values won't line
-- up on the same second boundary. Average over 3 second intervals.
-- Don't do this for a jmx app table, only jvm_jvm_metrics_table
IF in_sec_per_interval < 3 AND in_agg_type != 'no_agg' AND in_jmx_app = ''
THEN
    SET metric_columns = REPLACE(metric_columns, "AVG(`cpu`) AS 'cpu-avg_systems'", "AVG(AVG(`cpu`)) OVER (PARTITION BY j.jam_id, j.pid ORDER BY UNIX_TIMESTAMP(j.time) RANGE BETWEEN 1 PRECEDING AND 1 FOLLOWING) AS 'cpu-avg_systems'");
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

CALL tpf_execute_immediate(CONCAT(
    " SELECT MIN(j.time) AS time,",
             -- Only need the system name in the series name when it is no_agg (otherwise we are aggregating all systems)
             -- Only need subsystem in the series name when there are more than 1
             CASE WHEN in_agg_type = 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', j.pid, tj.subsystem, tsi.display_name) AS series_name,"
                  WHEN in_agg_type = 'no_agg' AND num_subsystems <= 1 THEN "CONCAT_WS('-', j.pid, tsi.display_name) AS series_name,"
                  WHEN in_agg_type != 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', tj.subsystem) AS series_name,"
                  ELSE ""
             END,
             metric_columns,
    " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id", -- jam_id filters selected systems, subsystems, and jams
                     " INNER JOIN ", temp_sys_id_table_name, " tsi ON tj.system_id=tsi.id", -- Get us the system display_name, which is the shortest concatenation of system cols to uniquely identify
                     " INNER JOIN ", jvm_table_name, " j ON tj.id=j.jam_id", -- Get the metrics
                     " INNER JOIN JSON_TABLE(", QUOTE(in_pids), ", \"$[*]\" COLUMNS(pid INT path \"$\")) ip ON j.pid=ip.pid", -- Filter the selected PIDs
    " WHERE j.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "j.jam_id, j.pid, " ELSE "tj.subsystem, tj.jam, " END, "(UNIX_TIMESTAMP(j.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
));


CALL tpf_execute_immediate(CONCAT("DROP TEMPORARY TABLE IF EXISTS ", temp_sys_id_table_name));

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_lock_drill_down_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_lock_drill_down_graph`(
    IN `in_system_id` INT,
    IN `in_subsystem` VARCHAR(4),
    IN `in_jam` VARCHAR(32),
    IN `in_pid` INT,
    IN `in_locks` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT
)

tpf_lock_drill_down_graph:BEGIN

DECLARE jam_id_lookup INT;
DECLARE metric_columns TEXT;

DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF in_system_id=0
THEN
    LEAVE tpf_lock_drill_down_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_lock_drill_down_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

SELECT id
INTO jam_id_lookup
FROM tpf_jams
WHERE system_id=in_system_id AND subsystem=in_subsystem AND jam=in_jam;

SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, 'l.time', 'no_agg',"jvm_lock_metrics_table");

IF metric_columns IS NULL OR jam_id_lookup IS NULL
THEN
    LEAVE tpf_lock_drill_down_graph;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

CALL tpf_execute_immediate(CONCAT(
    " SELECT MIN(l.time) AS time,",
           " CONCAT('[', TRIM(LEADING '0' FROM TRIM(LEADING '[' FROM l.lock))) AS series_name,",
             metric_columns,
    " FROM jvm_lock_metrics_table l INNER JOIN JSON_TABLE(", QUOTE(in_locks), ", \"$[*]\" COLUMNS(lock_name VARCHAR(18) path \"$\")) il ON l.lock=il.lock_name", -- Filter selected locks
    " WHERE l.jam_id=", jam_id_lookup, " AND l.pid=", in_pid,
          " AND l.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    " GROUP BY l.lock, (UNIX_TIMESTAMP(l.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
));


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_lock_drill_down_bar_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_lock_drill_down_bar_graph`(
    IN `in_system_id` INT,
    IN `in_subsystem` VARCHAR(4),
    IN `in_jam` VARCHAR(32),
    IN `in_pid` INT,
    IN `in_locks` JSON,
    IN `in_show_metrics` ENUM('gets', 'gets_rec', 'gets_slow', 'hold_time', 'miss_pct', 'util_pct'),
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_lock_drill_down_bar_graph:BEGIN

DECLARE jam_id_lookup INT;
DECLARE lt_time DATETIME;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Everything related to MariaDB is in UTC time. We let the caller (i.e. Grafana) handle time zone conversions
SET time_zone = '+00:00';

SELECT id
INTO jam_id_lookup
FROM tpf_jams
WHERE system_id=in_system_id AND subsystem=in_subsystem AND jam=in_jam;

--  Limit time window so we don't take forever
--  Look at 10 min for Lock_DD because data points are less frequent
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 10 MINUTE));

WITH top_locks_by_miss_pct AS (
    SELECT l.lock, l.time, l.miss_pct, l.util_pct, l.hold_time, ROW_NUMBER() OVER (PARTITION BY l.lock ORDER BY l.time DESC) AS row_num,
           CASE WHEN in_show_metrics='gets' THEN l.gets
                WHEN in_show_metrics='gets_rec' THEN l.gets_rec
                WHEN in_show_metrics='gets_slow' THEN l.gets_slow
                WHEN in_show_metrics='hold_time' THEN l.hold_time
                WHEN in_show_metrics='miss_pct' THEN l.miss_pct
                WHEN in_show_metrics='util_pct' THEN l.util_pct
                ELSE NULL
           END AS lock_metric
    FROM jvm_lock_metrics_table l INNER JOIN JSON_TABLE(in_locks, "$[*]" COLUMNS(lock_name VARCHAR(18) path "$")) il ON l.lock=il.lock_name
    WHERE l.jam_id=jam_id_lookup AND l.pid=in_pid AND l.time BETWEEN lt_time AND in_rt_time
)
SELECT CONCAT('[', TRIM(LEADING '0' FROM TRIM(LEADING '[' FROM tl.lock))) AS series_name, tl.lock_metric
FROM top_locks_by_miss_pct tl
WHERE row_num = 1
ORDER BY miss_pct DESC, hold_time DESC, util_pct DESC, tl.time ASC
LIMIT 5;


END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_get_jvm_jam_jax_rs_metrics_summarized_columns`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jvm_jam_jax_rs_metrics_summarized_columns` (
    `in_show_metrics` JSON,
    `in_agg_type` ENUM('no_agg', 'avg_systems', 'total_systems'),
    `time_col` VARCHAR(64)
)
RETURNS TEXT
DETERMINISTIC
BEGIN
    DECLARE metric_columns TEXT;

    SELECT GROUP_CONCAT(
       CASE WHEN in_agg_type='no_agg' THEN
                 CASE WHEN metric_name LIKE "%-avg_systems" THEN NULL
                      WHEN metric_name LIKE "%-total_systems" THEN NULL
                      WHEN metric_name LIKE "avg_response_time%"  THEN CONCAT(" AVG(`",metric_name,"`) AS '",metric_name,"'")
                      WHEN metric_name LIKE "min_response_time%"  THEN CONCAT(" MIN(`",metric_name,"`) AS '",metric_name,"'")
                      WHEN metric_name LIKE "max_response_time%"  THEN CONCAT(" MAX(`",metric_name,"`) AS '",metric_name,"'")
                      ELSE CONCAT(" SUM(`",metric_name,"`) AS '",metric_name,"'")
                 END
            ELSE
                 CASE WHEN metric_name LIKE "%-avg_systems" THEN
                           CASE WHEN metric_name LIKE "avg_response_time%"  THEN CONCAT(" AVG(`",SUBSTRING_INDEX(metric_name, '-', 1),"`) AS '",metric_name,"'")
                                WHEN metric_name LIKE "min_response_time%"  THEN CONCAT(" MIN(`",SUBSTRING_INDEX(metric_name, '-', 1),"`) AS '",metric_name,"'")
                                WHEN metric_name LIKE "max_response_time%"  THEN CONCAT(" MAX(`",SUBSTRING_INDEX(metric_name, '-', 1),"`) AS '",metric_name,"'")
                                ELSE CONCAT(" SUM(`",SUBSTRING_INDEX(metric_name, '-', 1),"`) AS '",metric_name,"'")
                           END
                      ELSE NULL
                 END
       END
    )
    INTO metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric_name VARCHAR(64) path "$")) metrics;

    RETURN metric_columns;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_jvm_jam_jax_rs_metrics_columns`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_jvm_jam_jax_rs_metrics_columns` (
    `in_show_metrics` JSON,
    `in_agg_type` ENUM('no_agg', 'avg_systems', 'total_systems'),
    `time_col` VARCHAR(64)
)
RETURNS TEXT
DETERMINISTIC
BEGIN
    DECLARE metric_columns TEXT;

    SELECT GROUP_CONCAT(
       CASE WHEN in_agg_type='no_agg' THEN
                 CASE WHEN metric_name LIKE "%-avg_systems" THEN NULL
                      WHEN metric_name LIKE "%-total_systems" THEN NULL
                      ELSE CONCAT(" AVG(`",metric_name,"`) AS '",metric_name,"'")
                 END
            ELSE
                 CASE WHEN metric_name LIKE "%-avg_systems" THEN CONCAT(" AVG(`",metric_name,"`) AS '",metric_name,"'")
                      WHEN metric_name LIKE "%-total_systems" THEN CONCAT(" AVG(`",metric_name,"`) AS '",metric_name,"'")
                      ELSE NULL
                 END
       END
    )
    INTO metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric_name VARCHAR(64) path "$")) metrics;

    RETURN metric_columns;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jax_rs_drill_down_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jax_rs_drill_down_graph`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_apps` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_jax_rs_drill_down_graph:BEGIN

DECLARE jam_ids JSON;
DECLARE num_subsystems INT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE metric_columns TEXT;
DECLARE jvm_summarized_metric_columns TEXT;
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;
DECLARE working_statement TEXT;


-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_system_ids,'[""]') OR in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_jax_rs_drill_down_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_jax_rs_drill_down_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    LEAVE tpf_jax_rs_drill_down_graph;
END IF;

SET num_subsystems = JSON_LENGTH(in_subsystems);

IF in_agg_type = 'no_agg'
THEN
    SET temp_sys_id_table_name = "jax_rs_drill_down_graph_temp_selected_systems";
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;

SET metric_columns = tpf_get_jvm_jam_jax_rs_metrics_columns(in_show_metrics, in_agg_type, 'agg.time');
SET jvm_summarized_metric_columns = tpf_get_jvm_jam_jax_rs_metrics_summarized_columns(in_show_metrics, in_agg_type, 'jr.time');

IF metric_columns IS NULL OR jvm_summarized_metric_columns IS NULL
THEN
    LEAVE tpf_jax_rs_drill_down_graph;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

SET working_statement = CONCAT(
    " WITH agg AS (",
        " SELECT tj.system_id, jr.jam_id, tj.subsystem, jr.time, tj.jam, jr.app,",
               jvm_summarized_metric_columns,
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id", -- jam_id filters selected systems, subsystems, and jams
                         " INNER JOIN jvm_jam_jax_rs_metrics_table jr ON tj.id=jr.jam_id", -- Get the metrics
                         " INNER JOIN JSON_TABLE(", QUOTE(in_apps), ", \"$[*]\" COLUMNS(app VARCHAR(128) path \"$\")) ia ON jr.app=ia.app", -- Filter the selected apps
        " WHERE jr.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
        " GROUP BY jr.jam_id, jr.app, jr.time"
    " )",
    " SELECT MIN(agg.time) AS time,",
             -- Only need the system name in the series name when it is no_agg (otherwise we are aggregating all systems)
             -- Only need subsystem in the series name when there are more than 1
             CASE WHEN in_agg_type = 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', agg.app, agg.subsystem, tsi.display_name) AS series_name,"
                  WHEN in_agg_type = 'no_agg' AND num_subsystems <= 1 THEN "CONCAT_WS('-', agg.app, tsi.display_name) AS series_name,"
                  WHEN in_agg_type != 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', agg.app, agg.subsystem) AS series_name,"
                  ELSE "agg.app AS series_name,"
             END,
             metric_columns,
    " FROM agg INNER JOIN ", temp_sys_id_table_name, " tsi ON agg.system_id=tsi.id", -- Get the system display_name, which is the shortest concatenation of system cols to uniquely identify
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "agg.jam_id, agg.app, " ELSE "agg.subsystem, agg.jam, agg.app, " END, "(UNIX_TIMESTAMP(agg.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
);
CALL tpf_execute_immediate(working_statement);


DROP TEMPORARY TABLE IF EXISTS jax_rs_drill_down_graph_temp_selected_systems;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jax_rs_op_drill_down_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jax_rs_op_drill_down_graph`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_apps` JSON,
    IN `in_op_ids` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_jax_rs_op_drill_down_graph:BEGIN

DECLARE jam_ids JSON;
DECLARE num_subsystems INT;
DECLARE temp_sys_id_table_name VARCHAR(128);
DECLARE metric_columns TEXT;
DECLARE jvm_summarized_metric_columns TEXT;
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_system_ids,'[""]') OR in_system_ids='""' OR in_system_ids="" OR in_system_ids="[]"
THEN
    LEAVE tpf_jax_rs_op_drill_down_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_jax_rs_op_drill_down_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF jam_ids IS NULL
THEN
    LEAVE tpf_jax_rs_op_drill_down_graph;
END IF;

SET num_subsystems = JSON_LENGTH(in_subsystems);

IF in_agg_type = 'no_agg'
THEN
    SET temp_sys_id_table_name = "jax_rs_op_drill_down_graph_temp_selected_systems";
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
ELSE
    -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;

SET metric_columns = tpf_get_jvm_jam_jax_rs_metrics_columns(in_show_metrics, in_agg_type, 'agg.time');
SET jvm_summarized_metric_columns = tpf_get_jvm_jam_jax_rs_metrics_summarized_columns(in_show_metrics, in_agg_type, 'jro.time');

IF metric_columns IS NULL OR jvm_summarized_metric_columns IS NULL
THEN
    LEAVE tpf_jax_rs_op_drill_down_graph;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

CALL tpf_execute_immediate(CONCAT(
    " WITH agg AS (",
        " SELECT tj.system_id, jro.jam_id, tj.subsystem, tj.jam, jro.time, jro.app, jro.op_id,",
               jvm_summarized_metric_columns,
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id", -- jam_id filters selected systems, subsystems, and jams
                         " INNER JOIN jvm_jam_jax_rs_op_metrics_table jro ON tj.id=jro.jam_id", -- Get the metrics
                         " INNER JOIN JSON_TABLE(", QUOTE(in_apps), ", \"$[*]\" COLUMNS(app VARCHAR(128) path \"$\")) ia ON jro.app=ia.app", -- Filter the selected apps
                         " INNER JOIN JSON_TABLE(", QUOTE(in_op_ids), ", \"$[*]\" COLUMNS(op_id INT path \"$\")) ioi ON jro.op_id=ioi.op_id", -- Filter the selected op_ids
        " WHERE jro.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
        " GROUP BY jro.jam_id, jro.app, jro.op_id, jro.time"
    " )",
    " SELECT MIN(agg.time) AS time,",
             -- Only need the system name in the series name when it is no_agg (otherwise we are aggregating all systems)
             -- Only need subsystem in the series name when there are more than 1
             CASE WHEN in_agg_type = 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', jops.operation, agg.subsystem, tsi.display_name) AS series_name,"
                  WHEN in_agg_type = 'no_agg' AND num_subsystems <= 1 THEN "CONCAT_WS('-', jops.operation, tsi.display_name) AS series_name,"
                  WHEN in_agg_type != 'no_agg' AND num_subsystems > 1 THEN "CONCAT_WS('-', jops.operation, agg.subsystem) AS series_name,"
                  ELSE "jops.operation AS series_name,"
             END,
             metric_columns,
    " FROM agg INNER JOIN ", temp_sys_id_table_name, " tsi ON agg.system_id=tsi.id", -- Get the system display_name, which is the shortest concatenation of system cols to uniquely identify
             " INNER JOIN jvm_operations jops ON agg.op_id=jops.id",
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "agg.jam_id, agg.app, agg.op_id, " ELSE "agg.subsystem, agg.jam, agg.app, agg.op_id, " END, "(UNIX_TIMESTAMP(agg.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
));


DROP TEMPORARY TABLE IF EXISTS jax_rs_op_drill_down_graph_temp_selected_systems;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_jam_summary_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jam_summary_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jam_summary_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;


-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_jams) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jam_summary_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jam_summary_current_summary_table;
END IF;


WITH with_max_time AS ( -- Get the max time for each jam_id so we only so we can limit the data in the table to the last 10 seconds for each JAM
    SELECT time, jam_id, pid, cpu, num_jvms_expected, num_jvms_actual, num_jam_threads_total, num_jam_threads_inuse, max_jam_threads_observed, avg_response_time,
           MAX(time) OVER (PARTITION BY jam_id) AS max_time
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_row_num AS (
    SELECT *, ROW_NUMBER() OVER (PARTITION BY jam_id, pid ORDER BY time DESC) AS row_num -- Mark the first row for each PID
    FROM with_max_time
    WHERE TIMESTAMPDIFF(SECOND,time,max_time) <= 10 -- Limit to last 10 seconds for each JAM
), with_first_values AS ( -- Get the first non-null value for each col.
    SELECT row_num,
           jam_id,
           FIRST_VALUE(cpu) OVER (PARTITION BY jam_id,pid ORDER BY cpu IS NULL,time DESC) AS cpu,
           FIRST_VALUE(num_jvms_expected) OVER (PARTITION BY jam_id,pid ORDER BY num_jvms_expected IS NULL,time DESC) AS num_jvms_expected,
           FIRST_VALUE(num_jvms_actual) OVER (PARTITION BY jam_id,pid ORDER BY num_jvms_actual IS NULL,time DESC) AS num_jvms_actual,
           FIRST_VALUE(num_jam_threads_total) OVER (PARTITION BY jam_id,pid ORDER BY num_jam_threads_total IS NULL,time DESC) AS num_jam_threads_total,
           FIRST_VALUE(num_jam_threads_inuse) OVER (PARTITION BY jam_id,pid ORDER BY num_jam_threads_inuse IS NULL,time DESC) AS num_jam_threads_inuse,
           FIRST_VALUE(max_jam_threads_observed) OVER (PARTITION BY jam_id,pid ORDER BY max_jam_threads_observed IS NULL,time DESC) AS max_jam_threads_observed,
           FIRST_VALUE(avg_response_time) OVER (PARTITION BY jam_id,pid ORDER BY avg_response_time IS NULL,time DESC) AS avg_response_time
    FROM with_row_num
), with_agg AS ( -- Need to aggregate PIDs differently for JAM vs non-JAM apps
    SELECT tj.system_id, tj.subsystem, tj.jam, tj.id AS jam_id,
           SUM(cpu) AS total_cpu,AVG(cpu) AS cpu,MAX(num_jvms_expected) AS num_jvms_expected,
           MAX(num_jvms_actual) AS num_jvms_actual,MAX(num_jam_threads_total) AS num_jam_threads_total,
           MAX(num_jam_threads_inuse) AS num_jam_threads_inuse,MAX(max_jam_threads_observed) AS max_jam_threads_observed,
           AVG(avg_response_time) AS avg_response_time
    FROM with_first_values wfv INNER JOIN tpf_jams tj ON wfv.jam_id=tj.id
    WHERE row_num = 1 AND tj.is_jam_app -- Select only the first row for each PID, otherwise we get duplicates
    GROUP BY tj.id
    UNION ALL
    SELECT tj.system_id, tj.subsystem, tj.jam, tj.id AS jam_id,
           SUM(cpu) AS total_cpu,AVG(cpu) AS cpu,NULL AS num_jvms_expected,
           COUNT(*) AS num_jvms_actual,NULL AS num_jam_threads_total,
           NULL AS num_jam_threads_inuse,NULL AS max_jam_threads_observed,
           AVG(avg_response_time) AS avg_response_time
    FROM with_first_values wfv INNER JOIN tpf_jams tj ON wfv.jam_id=tj.id
    WHERE row_num = 1 AND NOT tj.is_jam_app
    GROUP BY tj.id
)
SELECT CONCAT("var-tpf_system_ids=", wa.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wa.subsystem AS subsystem,
       wa.jam AS jam,
       total_cpu,
       cpu,
       num_jvms_actual/num_jvms_expected AS jvms_exp_act_pct,
       CONCAT(IFNULL(num_jvms_expected, "         "), ' / ', num_jvms_actual) AS jvms_exp_act,
       CONCAT(num_jam_threads_total, ' / ', num_jam_threads_inuse, ' / ', max_jam_threads_observed) AS threads_tot_inuse,
       avg_response_time
FROM with_agg wa INNER JOIN tpf_systems ts ON wa.system_id=ts.id
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", wa.system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       AVG(total_cpu) AS total_cpu,
       AVG(cpu) AS cpu,
       NULL AS jvms_exp_act_pct,
       NULL AS jvms_exp_act,
       NULL AS threads_tot_inuse,
       AVG(avg_response_time) AS avg_response_time
FROM with_agg wa
GROUP BY subsystem, jam;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_summary_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_summary_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jvm_summary_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_pids) = 0
THEN
    -- Need the args_hidden json column here to satisfy a transformation in Grafana
    -- See comment in final SELECT below for column args_hidden
    SELECT 'No data in time range' AS 'Error Message', '{"jam_app_no_args_hidden": ""}' AS args_hidden;
    LEAVE tpf_jvm_summary_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message', '{"jam_app_no_args_hidden": ""}' AS args_hidden;
    LEAVE tpf_jvm_summary_current_summary_table;
END IF;


WITH with_max_time AS ( -- Get the max time for each pid so we only so we can limit the data in the table to the last 10 seconds for each JAM
    SELECT time, jam_id, j.pid, cpu, avg_response_time, avg_gc_interval_global, avg_gc_pause_global, heap_size,
           MAX(time) OVER (PARTITION BY jam_id, pid) AS max_time
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id -- Filter the selected jam_ids
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid -- Filter the selected pids
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_row_num AS (
    SELECT *, ROW_NUMBER() OVER (PARTITION BY jam_id, pid ORDER BY time DESC) AS row_num -- Mark the first row for each PID
    FROM with_max_time
    WHERE TIMESTAMPDIFF(SECOND,time,max_time) <= 10 -- Limit to the last 10 seconds of data for each PID
), with_first_values AS ( -- Get the first non-null value for each col.
    SELECT row_num,
           tj.system_id, tj.subsystem, tj.jam,
           jam_id,
           pid,
           -- I set up a "Extract fields" transformation on the JVM_Summary dashboard. It will create a new column based on the JSON in this column
           -- For non-JAM apps, we want to display the Link to Args link. The name of the added column will be "args", which will display the link.
           -- For JAM apps, we don't want to display this (there are no args for JAM apps). The name of the added column will be "jam_app_no_args_hidden".
           -- There is an override on columns ending with "hidden" so they are not displayed in the table.
           IF(tj.is_jam_app, '{"jam_app_no_args_hidden": ""}', '{"args": "Link to Args"}') AS args_hidden,
           FIRST_VALUE(cpu) OVER (PARTITION BY jam_id, pid ORDER BY cpu IS NULL, time DESC) AS cpu,
           FIRST_VALUE(avg_response_time) OVER (PARTITION BY jam_id, pid ORDER BY avg_response_time IS NULL,time DESC) AS avg_response_time,
           FIRST_VALUE(avg_gc_interval_global) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_interval_global IS NULL, time DESC) AS avg_gc_interval_global,
           FIRST_VALUE(avg_gc_pause_global) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_pause_global IS NULL,time DESC) AS avg_gc_pause_global,
           FIRST_VALUE(heap_size) OVER (PARTITION BY jam_id, pid ORDER BY heap_size IS NULL, time DESC) AS heap_size
    FROM with_row_num wrn INNER JOIN tpf_jams tj ON wrn.jam_id=tj.id
)
SELECT CONCAT("var-tpf_system_ids=", wfv.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wfv.subsystem AS subsystem,
       wfv.jam AS jam,
       wfv.pid,
       args_hidden,
       cpu,
       avg_response_time,
       avg_gc_interval_global,
       avg_gc_pause_global,
       heap_size
FROM with_first_values wfv INNER JOIN tpf_systems ts ON wfv.system_id=ts.id
WHERE row_num=1
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       NULL AS pid,
       args_hidden,
       AVG(cpu) AS cpu,
       AVG(avg_response_time) AS avg_response_time,
       AVG(avg_gc_interval_global) AS avg_gc_interval_global,
       AVG(avg_gc_pause_global) AS avg_gc_pause_global,
       AVG(heap_size) AS heap_size
FROM with_first_values
WHERE row_num=1
GROUP BY subsystem, jam;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_drill_down_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_drill_down_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jvm_drill_down_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_pids) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jvm_drill_down_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jvm_drill_down_current_summary_table;
END IF;

WITH with_max_time AS ( -- Get the max time for each pid so we only so we can limit the data in the table to the last 10 seconds for each JAM
    SELECT time, jam_id, j.pid, num_jvm_threads_daemon, num_jvm_threads_latest, used_heap, heap_size,
           MAX(time) OVER (PARTITION BY jam_id, pid) AS max_time
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id -- Filter the selected jam_ids
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid -- Filter the selected pids
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_row_num AS (
    SELECT *, ROW_NUMBER() OVER (PARTITION BY jam_id, pid ORDER BY time DESC) AS row_num -- Mark the first row for each PID
    FROM with_max_time
    WHERE TIMESTAMPDIFF(SECOND,time,max_time) <= 10 -- Limit to the last 10 seconds of data for each PID
), with_first_values AS ( -- Get the first non-null value for each col.
   SELECT row_num,
          tj.system_id, tj.subsystem, tj.jam,
          jam_id,
          pid,
          FIRST_VALUE(num_jvm_threads_daemon) OVER (PARTITION BY jam_id, pid ORDER BY num_jvm_threads_daemon IS NULL,time DESC) AS num_jvm_threads_daemon,
          FIRST_VALUE(num_jvm_threads_latest) OVER (PARTITION BY jam_id, pid ORDER BY num_jvm_threads_latest IS NULL,time DESC) AS num_jvm_threads_latest,
          FIRST_VALUE(used_heap) OVER (PARTITION BY jam_id, pid ORDER BY used_heap IS NULL,time DESC) AS used_heap,
          FIRST_VALUE(heap_size) OVER (PARTITION BY jam_id, pid ORDER BY heap_size IS NULL,time DESC) AS heap_size
   FROM with_row_num wrn INNER JOIN tpf_jams tj ON wrn.jam_id=tj.id
), with_miss_pct AS (
    SELECT tj.subsystem, tj.jam, l.jam_id, l.pid, l.miss_pct, ROW_NUMBER() OVER (PARTITION BY l.jam_id, l.pid, l.lock ORDER BY l.time DESC) as row_num
    FROM jvm_lock_metrics_table l INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON l.jam_id=iji.id -- Filter the selected jam_ids
                                  INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON l.pid=ip.pid -- Filter the selected pids
                                  INNER JOIN tpf_jams tj ON l.jam_id=tj.id
    WHERE time BETWEEN lt_time AND in_rt_time
)
SELECT CONCAT("var-tpf_system_ids=", wfv.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wfv.subsystem AS subsystem,
       wfv.jam AS jam,
       wfv.pid,
       'Link to Static Data' AS static_data_link,
       CONCAT(IFNULL(num_jvm_threads_daemon,''), ' / ',IFNULL(num_jvm_threads_latest,'')) AS jvm_threads_daemon_appl,
       'Link to GC Table' AS gc_link,
       used_heap,
       heap_size,
       (SELECT MAX(miss_pct) FROM with_miss_pct wmp WHERE row_num=1 AND wfv.jam_id=wmp.jam_id AND wfv.pid=wmp.pid) AS lock_status -- What is the max value out of the most recent miss_pct for each lock
FROM with_first_values wfv INNER JOIN tpf_systems ts ON wfv.system_id=ts.id
WHERE row_num=1
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       NULL AS pid,
       'Link to Static Data' AS static_data_link,
       CONCAT(IFNULL(ROUND(AVG(num_jvm_threads_daemon)),''), ' / ',IFNULL(ROUND(AVG(num_jvm_threads_latest)),'')) AS jvm_threads_daemon_appl,
       'Link to GC Table' AS gc_link,
       AVG(used_heap) AS used_heap,
       AVG(heap_size) AS heap_size,
       NULL AS lock_status
FROM with_first_values wfv
WHERE row_num=1
GROUP BY subsystem, jam;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_lock_drill_down_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_lock_drill_down_current_summary_table`(
    IN `in_system_id` INT,
    IN `in_subsystem` VARCHAR(4),
    IN `in_jam` VARCHAR(32),
    IN `in_pid` INT,
    IN `in_locks` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_lock_drill_down_current_summary_table:BEGIN

DECLARE jam_id_lookup INT;
DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

SELECT id
INTO jam_id_lookup
FROM tpf_jams
WHERE system_id=in_system_id AND subsystem=in_subsystem AND jam=in_jam;

IF JSON_LENGTH(in_locks) = 0 OR jam_id_lookup IS NULL
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_lock_drill_down_current_summary_table;
END IF;

--  Limit time window so we don't take forever
--  Look at 10 min for Lock_DD because data points are less frequent
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 10 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_lock_metrics_table l INNER JOIN JSON_TABLE(in_locks, "$[*]" COLUMNS(lock_name VARCHAR(18) path "$")) il ON l.lock=il.lock_name
    WHERE l.jam_id=jam_id_lookup AND l.pid=in_pid AND l.time BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first 10 minutes of the time range.' AS 'Error Message';
    LEAVE tpf_lock_drill_down_current_summary_table;
END IF;

WITH with_row_num AS (
    SELECT l.lock, l.description, l.miss_pct, l.util_pct, l.gets, l.gets_slow, l.gets_rec, l.hold_time,
           ROW_NUMBER() OVER (PARTITION BY l.lock ORDER BY l.time DESC) AS row_num
    FROM jvm_lock_metrics_table l INNER JOIN JSON_TABLE(in_locks, "$[*]" COLUMNS(lock_name VARCHAR(18) path "$")) il ON l.lock=il.lock_name
    WHERE l.jam_id=jam_id_lookup AND l.pid=in_pid AND l.time BETWEEN lt_time AND in_rt_time
)
SELECT wrn.lock, wrn.description, wrn.miss_pct, wrn.util_pct,
       CONCAT_WS(' / ', wrn.gets, wrn.gets_slow, wrn.gets_rec) AS gets_slow_recursive,
       wrn.hold_time
FROM with_row_num wrn
WHERE wrn.row_num = 1
ORDER BY wrn.miss_pct DESC, wrn.hold_time DESC, wrn.util_pct DESC;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jax_rs_drill_down_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jax_rs_drill_down_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_apps` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jax_rs_drill_down_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_apps) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jax_rs_drill_down_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jam_jax_rs_metrics_table jr INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON jr.jam_id=iji.id
                                         INNER JOIN JSON_TABLE(in_apps, "$[*]" COLUMNS(app VARCHAR(128) path "$")) ia ON jr.app=ia.app
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jax_rs_drill_down_current_summary_table;
END IF;

WITH with_row_num AS (
    SELECT tj.system_id, jr.jam_id, tj.subsystem, tj.jam, jr.app, jr.pid, jr.min_response_time, jr.max_response_time,
           jr.avg_response_time, jr.app_faults_checked, jr.app_faults_unchecked,
           jr.runtime_faults_logical, jr.runtime_faults_app, jr.handling_time,
           ROW_NUMBER() OVER (PARTITION BY jr.jam_id, jr.pid ORDER BY jr.time DESC) AS row_num -- Mark the first row for each PID
    FROM jvm_jam_jax_rs_metrics_table jr INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON jr.jam_id=iji.id
                                         INNER JOIN JSON_TABLE(in_apps, "$[*]" COLUMNS(app VARCHAR(128) path "$")) ia ON jr.app=ia.app
                                         INNER JOIN tpf_jams tj ON jr.jam_id=tj.id
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_agg AS (
    SELECT system_id, jam_id, subsystem, jam, app, GROUP_CONCAT(pid) AS pids,
           MIN(min_response_time) AS min_response_time, MAX(max_response_time) AS max_response_time, AVG(avg_response_time) AS avg_response_time,
           SUM(app_faults_checked) AS app_faults_checked, SUM(app_faults_unchecked) AS app_faults_unchecked,
           SUM(runtime_faults_logical) AS runtime_faults_logical, SUM(runtime_faults_app) AS runtime_faults_app,
           SUM(handling_time) AS handling_time
    FROM with_row_num
    WHERE row_num = 1
    GROUP BY jam_id, app
)
SELECT CONCAT("var-tpf_system_ids=", wa.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wa.subsystem,
       wa.jam,
       wa.app,
       CONCAT_WS(' / ', ROUND(min_response_time, 2), ROUND(max_response_time, 2), ROUND(avg_response_time, 2)) AS resp_min_max_avg,
       app_faults_checked + app_faults_unchecked + runtime_faults_logical + runtime_faults_app AS total_faults,
       CONCAT_WS(' / ', app_faults_checked, app_faults_unchecked) AS app_faults_checked_unchecked,
       CONCAT_WS(' / ', runtime_faults_logical, runtime_faults_app) AS runtime_faults_log_app,
       handling_time,
       pids
FROM with_agg wa INNER JOIN tpf_systems ts ON wa.system_id=ts.id
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       app,
       CONCAT_WS(' / ', ROUND(AVG(min_response_time), 2), ROUND(AVG(max_response_time), 2), ROUND(AVG(avg_response_time), 2)) AS resp_min_max_avg,
       AVG(app_faults_checked + app_faults_unchecked + runtime_faults_logical + runtime_faults_app) as total_faults,
       CONCAT_WS(' / ', ROUND(AVG(app_faults_checked), 1), ROUND(AVG(app_faults_unchecked), 1)) AS app_faults_checked_unchecked,
       CONCAT_WS(' / ', ROUND(AVG(runtime_faults_logical), 1), ROUND(AVG(runtime_faults_app), 1)) AS runtime_faults_log_app,
       AVG(handling_time) AS handling_time,
       NULL AS pids
FROM with_agg
GROUP BY subsystem, jam, app;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jax_rs_op_drill_down_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jax_rs_op_drill_down_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_apps` JSON,
    IN `in_op_ids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jax_rs_op_drill_down_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_apps) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jax_rs_op_drill_down_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jam_jax_rs_op_metrics_table jro INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON jro.jam_id=iji.id
                                             INNER JOIN JSON_TABLE(in_apps, "$[*]" COLUMNS(app VARCHAR(128) path "$")) ia ON jro.app=ia.app
                                             INNER JOIN JSON_TABLE(in_op_ids, "$[*]" COLUMNS(op_id INT path "$")) ioi ON jro.op_id=ioi.op_id
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jax_rs_op_drill_down_current_summary_table;
END IF;

WITH with_row_num AS (
    SELECT tj.system_id, jro.jam_id, tj.subsystem, tj.jam, jro.app, jro.op_id, jops.operation,
           jro.min_response_time, jro.max_response_time, jro.avg_response_time,
           jro.app_faults_checked, jro.app_faults_unchecked, jro.runtime_faults_logical, jro.runtime_faults_app,
           jro.handling_time,
           ROW_NUMBER() OVER (PARTITION BY jro.jam_id, jro.pid, jro.op_id ORDER BY jro.time DESC) AS row_num -- Mark the first row for each op_id
    FROM jvm_jam_jax_rs_op_metrics_table jro INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON jro.jam_id=iji.id
                                             INNER JOIN JSON_TABLE(in_apps, "$[*]" COLUMNS(app VARCHAR(128) path "$")) ia ON jro.app=ia.app
                                             INNER JOIN JSON_TABLE(in_op_ids, "$[*]" COLUMNS(op_id INT path "$")) ioi ON jro.op_id=ioi.op_id
                                             INNER JOIN tpf_jams tj ON jro.jam_id=tj.id
                                             INNER JOIN jvm_operations jops ON jro.op_id=jops.id
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_agg AS (
    SELECT system_id, jam_id, subsystem, jam, app, op_id, operation,
           MIN(min_response_time) AS min_response_time, MAX(max_response_time) AS max_response_time, AVG(avg_response_time) AS avg_response_time,
           SUM(app_faults_checked) AS app_faults_checked, SUM(app_faults_unchecked) AS app_faults_unchecked,
           SUM(runtime_faults_logical) AS runtime_faults_logical, SUM(runtime_faults_app) AS runtime_faults_app,
           SUM(handling_time) AS handling_time
    FROM with_row_num
    WHERE row_num = 1
    GROUP BY jam_id, app, op_id
)
SELECT CONCAT("var-tpf_system_ids=", wa.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wa.subsystem,
       wa.jam,
       wa.app,
       wa.operation,
       CONCAT_WS(' / ', ROUND(min_response_time, 2), ROUND(max_response_time, 2), ROUND(avg_response_time, 2)) AS resp_min_max_avg,
       app_faults_checked + app_faults_unchecked + runtime_faults_logical + runtime_faults_app AS total_faults,
       CONCAT_WS(' / ', app_faults_checked, app_faults_unchecked) AS app_faults_checked_unchecked,
       CONCAT_WS(' / ', runtime_faults_logical, runtime_faults_app) AS runtime_faults_log_app,
       handling_time
FROM with_agg wa INNER JOIN tpf_systems ts ON wa.system_id=ts.id
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       app,
       operation,
       CONCAT_WS(' / ', ROUND(AVG(min_response_time), 2), ROUND(AVG(max_response_time), 2), ROUND(AVG(avg_response_time), 2)) AS resp_min_max_avg,
       AVG(app_faults_checked + app_faults_unchecked + runtime_faults_logical + runtime_faults_app) as total_faults,
       CONCAT_WS(' / ', ROUND(AVG(app_faults_checked), 1), ROUND(AVG(app_faults_unchecked), 1)) AS app_faults_checked_unchecked,
       CONCAT_WS(' / ', ROUND(AVG(runtime_faults_logical), 1), ROUND(AVG(runtime_faults_app), 1)) AS runtime_faults_log_app,
       AVG(handling_time) AS handling_time
FROM with_agg
GROUP BY subsystem, jam, app, op_id;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jmx_app_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jmx_app_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_jmx_app` VARCHAR(64)
)

tpf_jmx_app_current_summary_table:BEGIN

DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

DECLARE jvm_table_name VARCHAR(64);
DECLARE show_metrics TEXT;
DECLARE show_metrics_json JSON;
DECLARE variation_details JSON;
DECLARE working_statement TEXT;
DECLARE temp_table_working_statement TEXT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

IF JSON_LENGTH(in_pids) = 0 OR jam_ids IS NULL
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jmx_app_current_summary_table;
END IF;

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SET jvm_table_name = CONCAT("jvm_jmx_app_", in_jmx_app, "_table");

CALL tpf_execute_immediate(CONCAT(
    " SELECT EXISTS(",
        " SELECT 1",
        " FROM ", jvm_table_name, " j INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON j.jam_id=iji.id",
                                    " INNER JOIN JSON_TABLE(", QUOTE(in_pids), ", \"$[*]\" COLUMNS(pid INT path \"$\")) ip ON j.pid=ip.pid",
        " WHERE TIME BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
        " LIMIT 1",
    " )",
    " INTO @is_data_available"
));

IF NOT @is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jmx_app_current_summary_table;
END IF;

-- Get all the metrics we can show
SET variation_details = tpf_get_variations_json("");
SELECT GROUP_CONCAT(t1.COLUMN_NAME), JSON_ARRAYAGG(t1.COLUMN_NAME)
INTO show_metrics, show_metrics_json
FROM information_schema.COLUMNS AS t1
WHERE t1.TABLE_NAME=jvm_table_name
AND t1.DATA_TYPE IN ('int', 'double', 'bigint', 'mediumint', 'smallint', 'tinyint')
AND NOT t1.COLUMN_KEY='PRI'
--  This block removes the columns like a-max_per_second as we'll be adding those in the CONCAT above
AND NOT SUBSTRING_INDEX(t1.COLUMN_NAME, '-', -1) IN  (
    SELECT SUBSTRING_INDEX(jt4.metric_suffix, '-', -1)
    FROM json_table(variation_details, '$[*]'
       COLUMNS(
         display_string VARCHAR(64) PATH '$.display_string',
         metric_suffix  VARCHAR(64) PATH '$.metric_suffix'
       )
    ) AS jt4
    WHERE NOT display_string="per_second"
    )
AND NOT t1.COLUMN_NAME="entry_is_pruned";

IF show_metrics IS NULL
THEN
    SELECT "No columns available" AS "Error Message";
    LEAVE tpf_jmx_app_current_summary_table;
END IF;

IF jvm_table_name = "jvm_jmx_app_Kafka_table"
THEN
    DROP TEMPORARY TABLE IF EXISTS temp_with_row_num_table;
    SET temp_table_working_statement=CONCAT(
        " CREATE TEMPORARY TABLE temp_with_row_num_table AS ",
        " WITH with_row_num AS (",
        " SELECT j.*, tj.system_id, tj.subsystem, tj.jam, RANK() OVER (PARTITION BY j.jam_id, j.pid, j.producer_id  ORDER BY time DESC) AS row_num",
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id",
        " INNER JOIN ", jvm_table_name, " j ON tj.id=j.jam_id",
        " INNER JOIN JSON_TABLE(", QUOTE(in_pids), ", \"$[*]\" COLUMNS(pid INT path \"$\")) ip ON j.pid=ip.pid",
        " WHERE j.time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
        " AND j.entry_is_pruned IS FALSE"
        " )",
        " SELECT system_id, subsystem, jam, row_num,",
        " SUM(`req_total`) AS 'req_total',SUM(`reqs_in_flight`) AS 'reqs_in_flight',AVG(`avg_batch_size`) AS 'avg_batch_size',AVG(`rec_send_rate`) AS 'rec_send_rate',AVG(`avg_rec_size`) AS 'avg_rec_size',AVG(`avg_req_latency`) AS 'avg_req_latency',SUM(`waiting_threads`) AS 'waiting_threads' ",
        " FROM with_row_num",
        " WHERE row_num=1",
        " GROUP BY system_id, subsystem, jam");
    CALL tpf_execute_immediate(temp_table_working_statement);
    SET working_statement=CONCAT(
        " WITH with_row_num AS (",
        " SELECT j.*, tj.system_id, tj.subsystem, tj.jam, RANK() OVER (PARTITION BY j.jam_id, j.pid, j.producer_id  ORDER BY time DESC) AS row_num",
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id",
                         " INNER JOIN ", jvm_table_name, " j ON tj.id=j.jam_id",
                         " INNER JOIN JSON_TABLE(", QUOTE(in_pids), ", \"$[*]\" COLUMNS(pid INT path \"$\")) ip ON j.pid=ip.pid",
        " WHERE j.time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
        " AND j.req_total IS NOT NULL"
        " )",
    " SELECT CONCAT('var-tpf_system_ids=', wrn.system_id) AS system_query_param_hidden,",
           " CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',",
           " wrn.subsystem,",
           " wrn.jam,",
           " wrn.pid,",
           " pj.producer_name,",
           show_metrics,
    " FROM with_row_num wrn INNER JOIN tpf_systems ts ON wrn.system_id=ts.id",
    " INNER JOIN tpf_jmx_producer_table pj ON wrn.producer_id=pj.id",
    " WHERE wrn.row_num=1 ",
    " GROUP BY system_id, subsystem, jam, pid, producer_name "
    " UNION ALL",
    " SELECT GROUP_CONCAT(CONCAT('var-tpf_system_ids=', system_id) SEPARATOR '&') AS system_query_param_hidden,",
           " 'avg_systems' AS 'system',",
           " subsystem,",
           " jam,",
           " NULL AS pid,",
           " NULL AS producer_name,",
           -- Creates: AVG(col1) AS 'col1', AVG(col2) AS 'col2'
           tpf_complex_create_selected_metrics(show_metrics_json, 'subsystem, jam', 'no_agg',jvm_table_name),
    " FROM temp_with_row_num_table",
    " GROUP BY subsystem, jam"
    );
ELSE
SET working_statement=CONCAT(
    " WITH with_row_num AS (",
        " SELECT j.*, tj.system_id, tj.subsystem, tj.jam, ROW_NUMBER() OVER (PARTITION BY j.jam_id, j.pid ORDER BY time DESC) AS row_num",
        " FROM tpf_jams tj INNER JOIN JSON_TABLE(", QUOTE(jam_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) iji ON tj.id=iji.id",
                         " INNER JOIN ", jvm_table_name, " j ON tj.id=j.jam_id",
                         " INNER JOIN JSON_TABLE(", QUOTE(in_pids), ", \"$[*]\" COLUMNS(pid INT path \"$\")) ip ON j.pid=ip.pid",
        " WHERE j.time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
    " )",
    " SELECT CONCAT('var-tpf_system_ids=', wrn.system_id) AS system_query_param_hidden,",
           " CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',",
           " wrn.subsystem,",
           " wrn.jam,",
           " wrn.pid,",
           show_metrics,
    " FROM with_row_num wrn INNER JOIN tpf_systems ts ON wrn.system_id=ts.id",
    " WHERE wrn.row_num=1",
    " UNION ALL",
    " SELECT GROUP_CONCAT(CONCAT('var-tpf_system_ids=', system_id) SEPARATOR '&') AS system_query_param_hidden,",
           " 'avg_systems' AS 'system',",
           " subsystem,",
           " jam,",
           " NULL AS pid,",
           -- Creates: AVG(col1) AS 'col1', AVG(col2) AS 'col2'
           tpf_complex_create_selected_metrics(show_metrics_json, 'subsystem, jam', 'no_agg',jvm_table_name),
    " FROM with_row_num",
    " WHERE row_num=1",
    " GROUP BY subsystem, jam"
);
END IF;

CALL tpf_execute_immediate(working_statement);
IF jvm_table_name = "jvm_jmx_app_Kafka_table"
THEN
    DROP TEMPORARY TABLE IF EXISTS temp_with_row_num_table;
END IF;
END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_jvm_gc_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_gc_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jvm_gc_current_summary_table:BEGIN

DECLARE is_data_available BOOLEAN;
DECLARE lt_time DATETIME;
DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_pids) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jvm_gc_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

--  Look at a max of a 1 minute window so we don't take forever
SET lt_time = GREATEST(in_lt_time, DATE_SUB(in_rt_time, INTERVAL 1 MINUTE));

SELECT EXISTS(
    SELECT 1
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid
    WHERE TIME BETWEEN lt_time AND in_rt_time
    LIMIT 1
)
INTO is_data_available;

IF NOT is_data_available
THEN
    SELECT 'No data available in the first minute of the time range.' AS 'Error Message';
    LEAVE tpf_jvm_gc_current_summary_table;
END IF;

WITH with_max_time AS ( -- Get the max time for each pid so we only so we can limit the data in the table to the last 10 seconds for each JAM
    SELECT time, jam_id, j.pid, avg_gc_interval, avg_gc_pause, avg_gc_interval_global, avg_gc_pause_global, used_heap, heap_size,
           MAX(time) OVER (PARTITION BY jam_id, pid) AS max_time
    FROM jvm_jvm_metrics_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id -- Filter the selected jam_ids
                                 INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid -- Filter the selected pids
    WHERE TIME BETWEEN lt_time AND in_rt_time
), with_row_num AS (
    SELECT *, ROW_NUMBER() OVER (PARTITION BY jam_id, pid ORDER BY time DESC) AS row_num -- Mark the first row for each PID
    FROM with_max_time
    WHERE TIMESTAMPDIFF(SECOND,time,max_time) <= 10 -- Limit to the last 10 seconds of data for each PID
), with_first_values AS ( -- Get the first non-null value for each col.
   SELECT row_num,
          tj.system_id, tj.subsystem, tj.jam,
          jam_id,
          pid,
          FIRST_VALUE(avg_gc_interval) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_interval IS NULL,time DESC) AS avg_gc_interval,
          FIRST_VALUE(avg_gc_pause) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_pause IS NULL,time DESC) AS avg_gc_pause,
          FIRST_VALUE(avg_gc_interval_global) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_interval_global IS NULL,time DESC) AS avg_gc_interval_global,
          FIRST_VALUE(avg_gc_pause_global) OVER (PARTITION BY jam_id, pid ORDER BY avg_gc_pause_global IS NULL,time DESC) AS avg_gc_pause_global,
          FIRST_VALUE(used_heap) OVER (PARTITION BY jam_id, pid ORDER BY used_heap IS NULL,time DESC) AS used_heap,
          FIRST_VALUE(heap_size) OVER (PARTITION BY jam_id, pid ORDER BY heap_size IS NULL,time DESC) AS heap_size
   FROM with_row_num wrn INNER JOIN tpf_jams tj ON wrn.jam_id=tj.id
)
SELECT CONCAT("var-tpf_system_ids=", wfv.system_id) AS system_query_param_hidden, -- Create the URL variable for tpf_system_ids. These rows are for a single system, so just need the system_id
       CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       wfv.subsystem AS subsystem,
       wfv.jam AS jam,
       wfv.pid,
       avg_gc_interval,
       avg_gc_pause,
       avg_gc_interval_global,
       avg_gc_pause_global,
       used_heap,
       heap_size
FROM with_first_values wfv INNER JOIN tpf_systems ts ON wfv.system_id=ts.id
WHERE row_num=1
UNION ALL
SELECT GROUP_CONCAT(DISTINCT CONCAT("var-tpf_system_ids=", system_id) SEPARATOR "&") AS system_query_param_hidden, -- These rows are for aggregating systems, so need to repeat for each system_id
       'avg_systems' AS 'system',
       subsystem,
       jam,
       NULL AS pid,
       AVG(avg_gc_interval) AS avg_gc_interval,
       AVG(avg_gc_pause) AS avg_gc_pause,
       AVG(avg_gc_interval_global) AS avg_gc_interval_global,
       AVG(avg_gc_pause_global) AS avg_gc_pause_global,
       AVG(used_heap) AS used_heap,
       AVG(heap_size) AS heap_size
FROM with_first_values wfv
WHERE row_num=1
GROUP BY subsystem, jam;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_static_data_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_static_data_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jvm_static_data_current_summary_table:BEGIN

DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_pids) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jvm_static_data_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

SELECT CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       tj.subsystem,
       tj.jam,
       j.pid,
       j.start_time,
       j.log_dir,
       ts.id AS system_id_hidden
FROM jvm_static_data_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
                             INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid
                             INNER JOIN tpf_jams tj ON j.jam_id=tj.id
                             INNER JOIN tpf_systems ts ON tj.system_id=ts.id
WHERE start_time <= in_rt_time AND last_time >= in_lt_time
ORDER BY start_time;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_jvm_args_current_summary_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_jvm_args_current_summary_table`(
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_jams` JSON,
    IN `in_pids` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_jvm_args_current_summary_table:BEGIN

DECLARE jam_ids JSON;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_pids) = 0
THEN
    SELECT 'No data in time range' AS 'Error Message';
    LEAVE tpf_jvm_args_current_summary_table;
END IF;

SET jam_ids = tpf_get_jam_ids(in_system_ids, in_subsystems, in_jams);

SELECT CONCAT_WS('~', ts.system_name, ts.complex_name, ts.processor_id, ts.ip_address, ts.environment, ts.cpc_name) AS 'system',
       tj.subsystem,
       tj.jam,
       j.pid,
       j.args,
       ts.id AS system_id_hidden
FROM jvm_static_data_table j INNER JOIN JSON_TABLE(jam_ids, "$[*]" COLUMNS(id INT path "$")) iji ON j.jam_id=iji.id
                             INNER JOIN JSON_TABLE(in_pids, "$[*]" COLUMNS(pid INT path "$")) ip ON j.pid=ip.pid
                             INNER JOIN tpf_jams tj ON j.jam_id=tj.id
                             INNER JOIN tpf_systems ts ON tj.system_id=ts.id
WHERE start_time <= in_rt_time AND last_time >= in_lt_time AND NOT tj.is_jam_app
ORDER BY start_time;

END//
DELIMITER ;

-- Old procedures
DROP PROCEDURE IF EXISTS `tpf_jvm_current_summary_table`;
DROP PROCEDURE IF EXISTS `tpf_jvm_groups_available`;
DROP PROCEDURE IF EXISTS `tpf_jvm_metrics_graph`;
