/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2024                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20240104    PJ47174 - Remove ip address from tpf_systems index   */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Migrate database before applying PJ47174.           */
/*                                                                   */
/*********************************************************************/

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP PROCEDURE IF EXISTS `tpf_migrate_PJ47174`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_migrate_PJ47174` ()
tpf_migrate_PJ47174:BEGIN

DECLARE working_statement TEXT;

DECLARE is_pruning_installed TEXT;
DECLARE is_UDM_installed TEXT;

DECLARE array_of_matching_system_id_arrays TEXT;
DECLARE array_of_arrays_index INT DEFAULT 0;
DECLARE array_of_matching_system_ids TEXT;
DECLARE array_of_ids_index INT DEFAULT 0;
DECLARE system_id_replacement TEXT;
DECLARE system_id_to_remove TEXT;

DECLARE last_system_index INT DEFAULT 0;
DECLARE system_id_last INT DEFAULT 0;
DECLARE system_id_iterator INT DEFAULT 0;

DECLARE table_list TEXT;
DECLARE table_name TEXT;
DECLARE join_list TEXT;
DECLARE join_string TEXT;
DECLARE metric_list TEXT;
DECLARE metric_name TEXT;
DECLARE pruning_columns TEXT;

DEClARE jam_id_table TEXT;
DEClARE jam_id_table_entry TEXT;
DECLARE jam_id_to_remove TEXT;
DECLARE jam_id_replacement TEXT;
DECLARE i INT DEFAULT 0;

DECLARE not_updated_count INT DEFAULT 0;

-- todo comment out Debugging statement
-- DECLARE EXIT HANDLER FOR SQLEXCEPTION    BEGIN        ROLLBACK;    END;

SET SESSION group_concat_max_len = 10000;

-- is pruning installed?
SET is_pruning_installed="NO";
SELECT IF(c.COLUMN_NAME IS NULL,"NO","YES") as answer
INTO is_pruning_installed
FROM information_schema.COLUMNS c
WHERE c.TABLE_SCHEMA = "TPFDataSciDB" AND c.TABLE_NAME   = "version_history" AND c.COLUMN_NAME = "pruning_version";

-- is UDM installed?
SET is_UDM_installed="NO";
SELECT IF(c.TABLE_NAME IS NULL,"NO","YES") as answer
INTO is_UDM_installed
FROM information_schema.COLUMNS c
WHERE c.TABLE_SCHEMA = "TPFDataSciDB" AND c.TABLE_NAME   = "udm_metrics_table"
LIMIT 1;


-- get arrays of system ids that have the same details but ip_address and id
-- result looks like this: "[[1],[7,8],[3,4,5,6]]"
SELECT JSON_ARRAYAGG(matching_system_id_array)
INTO array_of_matching_system_id_arrays
FROM (
SELECT JSON_ARRAYAGG(id) AS matching_system_id_array
FROM TPFDataSciDB.tpf_systems
GROUP BY system_name,complex_name,processor_id,cpc_name,environment
) AS matching_system_id_array_select;


 -- NOTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 -- If customers have UDM data, they should implement this section themselves.
 -- NOTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 -- Let's take care of IBM sample UDM data now.  It's just IBM pretend data and there is no pruning so it's simpler

-- loop through the arrays and determine the ids to remove/replace
SET array_of_arrays_index = 0;
WHILE array_of_arrays_index < JSON_LENGTH(array_of_matching_system_id_arrays) DO
   -- get an array of ids
   SET array_of_matching_system_ids =  JSON_EXTRACT(array_of_matching_system_id_arrays,CONCAT('$[',array_of_arrays_index,']'));
   SET array_of_arrays_index = array_of_arrays_index + 1;

   -- If there's only one id, there's nothing to do, so we'll skip it.
   IF JSON_LENGTH(array_of_matching_system_ids) > 1
   THEN
      -- we'll assume that the last id is the most recent, it might not be, but it's a good enough guess
      SET system_id_replacement = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',JSON_LENGTH(array_of_matching_system_ids) - 1,']'));
      -- loop through the ids and remove/replace
      SET array_of_ids_index = 0;
      WHILE array_of_ids_index < JSON_LENGTH(array_of_matching_system_ids) - 1 DO
         -- get an id to remove
         SET system_id_to_remove = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
         SET array_of_ids_index = array_of_ids_index + 1;

         IF is_UDM_installed="YES"
         THEN
            SET working_statement = CONCAT(
               " UPDATE IGNORE TPFDataSciDB.udm_metrics_table",
               " SET system_id=",system_id_replacement,
               " WHERE system_id=",system_id_to_remove,";"
            );
            CALL tpf_execute_immediate(working_statement);
         END IF;
      END WHILE;
   END IF;
END WHILE;


-- Next we'll go through JVM/JAM processing since that depends on the system id

-- Create the list of jvm tables to be processed
SET table_list = 'jvm_jam_jax_rs_metrics_table,jvm_jam_jax_rs_op_metrics_table,jvm_jmx_app_JMBeansDrv_table,jvm_jmx_app_Kafka_table,jvm_jvm_metrics_table,jvm_lock_metrics_table,jvm_static_data_table,';
SET metric_list = 'op_count,avg_response_time,ecb,req_total,num_jvms_expected,miss_pct,last_time,';
SET join_list = 'AND a.app=b.app,AND a.app=b.app AND a.op_id=b.op_id,,AND a.producer_id=b.producer_id,,AND a.lock=b.lock,,';

-- Loop through our list of tables
WHILE (LOCATE(',', table_list) > 0)
DO
    SET table_name =  SUBSTRING_INDEX(table_list,',',1);
    SET table_list = SUBSTRING(table_list, LOCATE(',',table_list) + 1);

    SET metric_name = SUBSTRING_INDEX(metric_list,',',1);
    SET metric_list = SUBSTRING(metric_list, LOCATE(',',metric_list) + 1);

    SET join_string =  SUBSTRING_INDEX(join_list,',',1);
    SET join_list = SUBSTRING(join_list, LOCATE(',',join_list) + 1);

    -- if pruning is installed, get the columns we need to set to null for this table.
    IF is_pruning_installed="YES" AND table_name!="jvm_static_data_table"
    THEN
       -- get the pruning metric columns
       SET working_statement = CONCAT(
          " SELECT concat(group_concat('a.`', COLUMN_NAME, '` = NULL' SEPARATOR ', '))",
          " INTO @pruning_columns",
          " FROM information_schema.COLUMNS c",
          " WHERE c.TABLE_SCHEMA = 'TPFDataSciDB'",
          " AND c.TABLE_NAME = '",table_name,"'",
          " AND (c.COLUMN_NAME LIKE '%-max_per_second' OR c.COLUMN_NAME LIKE '%-min_per_second' OR c.COLUMN_NAME LIKE '%-per_minute');"
       );
       CALL tpf_execute_immediate(working_statement);
    END IF;

    -- loop through the arrays and determine the ids to remove/replace
    SET array_of_arrays_index = 0;
    WHILE array_of_arrays_index < JSON_LENGTH(array_of_matching_system_id_arrays) DO
       -- get an array of ids
       SET array_of_matching_system_ids =  JSON_EXTRACT(array_of_matching_system_id_arrays,CONCAT('$[',array_of_arrays_index,']'));
       SET array_of_arrays_index = array_of_arrays_index + 1;

       -- If there's only one id, there's nothing to do, so we'll skip it.
       IF JSON_LENGTH(array_of_matching_system_ids) > 1
       THEN
          -- we'll assume that the last id is the most recent, it might not be, but it's a good enough guess
          SET system_id_replacement = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',JSON_LENGTH(array_of_matching_system_ids) - 1,']'));

          -- if pruning is installed, we need to handle any summarized entries that crossed 1 minute
          -- we'll take care of these first to help identify them
          IF is_pruning_installed="YES" AND table_name!="jvm_static_data_table"
          THEN
             -- loop through all combinations of system ids (ie 1, 2, 3: 3,2 - 3,1 - 2,1)
             -- set the summarized values to NULL
             SET last_system_index = JSON_LENGTH(array_of_matching_system_ids) - 1;
             WHILE last_system_index > 0 DO
                SET system_id_last = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',last_system_index,']'));

                -- loop through the ids and remove/replace
                SET array_of_ids_index = 0;
                WHILE array_of_ids_index < last_system_index DO
                   -- get an id to remove
                   SET system_id_iterator = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
                   SET array_of_ids_index = array_of_ids_index + 1;

                   -- select jam id pairs based upon the system id pairs
                   SET working_statement = CONCAT(
                      " WITH replacement_entries AS (SELECT * FROM TPFDataSciDB.tpf_jams WHERE system_id=",system_id_last,")"
                      " SELECT JSON_ARRAYAGG(JSON_OBJECT('jam_id_to_remove',a.id,'jam_id_replacement', b.id))"
                      " INTO @jam_id_table"
                      " FROM TPFDataSciDB.tpf_jams AS a, replacement_entries AS b"
                      " WHERE a.system_id = ",system_id_iterator,
                      " AND a.jam = b.jam AND a.subsystem = b.subsystem AND a.is_jam_app = b.is_jam_app AND a.id != b.id;"
                   );
                   CALL tpf_execute_immediate(working_statement);

                   -- if we have any JVM ids, we need to replace/remove them
                   IF @jam_id_table IS NOT NULL AND @jam_id_table != ""
                   THEN
                      -- loop through the jam_id remove/replace pairs
                      SET i = 0;
                      WHILE i < JSON_LENGTH(@jam_id_table) DO
                         SET jam_id_table_entry =  JSON_EXTRACT(@jam_id_table,CONCAT('$[',i,']'));
                         SET i = i + 1;

                         SET jam_id_to_remove = json_value(jam_id_table_entry,'$.jam_id_to_remove');
                         SET jam_id_replacement = json_value(jam_id_table_entry,'$.jam_id_replacement');

                         -- Identify the entries to be deleted, but we need to delete them later.
                         SET working_statement = CONCAT(
                            " UPDATE TPFDataSciDB.",table_name," a",
                            " JOIN ( ",
                            "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE jam_id=",jam_id_replacement," AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                            "      ) b ON a.time=b.time AND a.pid=b.pid ",join_string,
                            " SET a.entry_is_pruned=2",
                            " WHERE a.jam_id=",jam_id_to_remove," AND a.",metric_name," IS NULL AND a.entry_is_pruned=1;"
                         );
                         CALL tpf_execute_immediate(working_statement);
                         SET working_statement = CONCAT(
                            " UPDATE TPFDataSciDB.",table_name," a",
                            " JOIN ( ",
                            "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE jam_id=",jam_id_to_remove," AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                            "      ) b ON a.time=b.time AND a.pid=b.pid ",join_string,
                            " SET a.entry_is_pruned=2",
                            " WHERE a.jam_id=",jam_id_replacement," AND a.",metric_name," IS NULL AND a.entry_is_pruned=1;"
                         );
                         CALL tpf_execute_immediate(working_statement);

                         -- remove summarized metrics for entries with real metrics that collide with summarized entries from another jam id that don't have real metrics
                         SET working_statement = CONCAT(
                            " UPDATE TPFDataSciDB.",table_name," a",
                            " JOIN ( ",
                            "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE jam_id=",jam_id_replacement," AND ",metric_name," IS NULL AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                            "      ) b ON a.time=b.time AND a.pid=b.pid ",join_string,
                            " SET a.entry_is_pruned=0,",@pruning_columns,
                            " WHERE a.jam_id=",jam_id_to_remove," AND a.",metric_name," IS NOT NULL AND a.entry_is_pruned=1;"
                         );
                         CALL tpf_execute_immediate(working_statement);
                         SET working_statement = CONCAT(
                            " UPDATE TPFDataSciDB.",table_name," a",
                            " JOIN ( ",
                            "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE jam_id=",jam_id_to_remove," AND ",metric_name," IS NULL AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                            "      ) b ON a.time=b.time AND a.pid=b.pid ",join_string,
                            " SET a.entry_is_pruned=0,",@pruning_columns,
                            " WHERE a.jam_id=",jam_id_replacement," AND a.",metric_name," IS NOT NULL AND a.entry_is_pruned=1;"
                         );
                         CALL tpf_execute_immediate(working_statement);

                      END WHILE; -- jam id while loop
                   END IF;
                END WHILE;
                SET last_system_index = last_system_index - 1;
             END WHILE; -- system id while loop

             -- remove any rows that have pruning but no real metrics for our jam id to remove
             SET working_statement = CONCAT(
                " DELETE FROM TPFDataSciDB.",table_name,
                " WHERE ",metric_name," IS NULL AND entry_is_pruned=2;"
             );
             CALL tpf_execute_immediate(working_statement);
          END IF; -- If pruning

         -- loop through the ids again to update all entries to the replacement system id
          SET array_of_ids_index = 0;
          WHILE array_of_ids_index < JSON_LENGTH(array_of_matching_system_ids) - 1 DO
             -- get an id to remove
             SET system_id_to_remove = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
             SET array_of_ids_index = array_of_ids_index + 1;

             -- select jam id pairs based upon the system id to remove and it's replacement
             SET working_statement = CONCAT(
                " WITH replacement_entries AS (SELECT * FROM TPFDataSciDB.tpf_jams WHERE system_id=",system_id_replacement,")"
                " SELECT JSON_ARRAYAGG(JSON_OBJECT('jam_id_to_remove',a.id,'jam_id_replacement', b.id))"
                " INTO @jam_id_table"
                " FROM TPFDataSciDB.tpf_jams AS a, replacement_entries AS b"
                " WHERE a.system_id = ",system_id_to_remove,
                " AND a.jam = b.jam AND a.subsystem = b.subsystem AND a.is_jam_app = b.is_jam_app AND a.id != b.id;"
             );
             CALL tpf_execute_immediate(working_statement);

             -- if we have any JVM ids, we need to replace/remove them
             IF @jam_id_table IS NOT NULL AND @jam_id_table != ""
             THEN
                -- loop through the jam_id remove/replace pairs
                SET i = 0;
                WHILE i < JSON_LENGTH(@jam_id_table) DO
                   SET jam_id_table_entry =  JSON_EXTRACT(@jam_id_table,CONCAT('$[',i,']'));
                   SET i = i + 1;

                   SET jam_id_to_remove = json_value(jam_id_table_entry,'$.jam_id_to_remove');
                   SET jam_id_replacement = json_value(jam_id_table_entry,'$.jam_id_replacement');

                   -- convert everything left to the new system id.
                   SET working_statement = CONCAT(
                      " UPDATE IGNORE TPFDataSciDB.",table_name,
                      " SET jam_id=",jam_id_replacement,
                      " WHERE jam_id=",jam_id_to_remove,";"
                   );
                   CALL tpf_execute_immediate(working_statement);

                   -- count the number of rows that were not removed.
                   SET working_statement = CONCAT(
                      " SELECT COUNT(*)",
                      " INTO @not_updated_count",
                      " FROM TPFDataSciDB.",table_name,
                      " WHERE jam_id=",jam_id_to_remove,";"
                   );
                   CALL tpf_execute_immediate(working_statement);

                   -- if we have any rows that we couldn't updated, we need to remove them now.
                   -- This can happen for rows that are created from multiple writes to the same row.
                   -- ie                                        time  system_id   ECBs   actual_cpu
                   -- system block writes >  2023-12-20 19:23:45.000          1     50
                   -- system msg   writes >  2023-12-20 19:23:45.000          2                 300
                   IF @not_updated_count IS NOT NULL AND @not_updated_count > 0
                   THEN
                      -- print rows we're going to remove
                      -- static data gets repeated so we'll just delete this without warning.
                      IF table_name!="jvm_static_data_table"
                      THEN
                         SELECT CONCAT("Due to conflicts with jam_id ",jam_id_replacement,", following rows are being removed from ",table_name," for jam_id ", jam_id_to_remove," system_id ",system_id_to_remove) as warning_message;
                         SET working_statement = CONCAT(
                            " SELECT * FROM TPFDataSciDB.",table_name,
                            " WHERE jam_id=",jam_id_to_remove,";"
                         );
                         CALL tpf_execute_immediate(working_statement);
                      END IF;

                      -- delete the conflicting rows
                      SET working_statement = CONCAT(
                         " DELETE FROM TPFDataSciDB.",table_name,
                         " WHERE jam_id=",jam_id_to_remove,";"
                      );
                      CALL tpf_execute_immediate(working_statement);
                   END IF;
                END WHILE;
             END IF;
          END WHILE;
       END IF;
    END WHILE;
END WHILE;


-- loop through the arrays and determine the ids to remove/replace
SET array_of_arrays_index = 0;
WHILE array_of_arrays_index < JSON_LENGTH(array_of_matching_system_id_arrays) DO
   -- get an array of ids
   SET array_of_matching_system_ids =  JSON_EXTRACT(array_of_matching_system_id_arrays,CONCAT('$[',array_of_arrays_index,']'));
   SET array_of_arrays_index = array_of_arrays_index + 1;

   -- If there's only one id, there's nothing to do, so we'll skip it.
   IF JSON_LENGTH(array_of_matching_system_ids) > 1
   THEN
      SET system_id_replacement = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',JSON_LENGTH(array_of_matching_system_ids) - 1,']'));

      -- we'll assume that the last id is the most recent, it might not be, but it's a good enough guess
      SET array_of_ids_index = 0;
      WHILE array_of_ids_index < JSON_LENGTH(array_of_matching_system_ids) - 1 DO
         -- get an id to remove
         SET system_id_to_remove = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
         SET array_of_ids_index = array_of_ids_index + 1;

         -- select jam id pairs based upon the system id to remove and it's replacement
         SET working_statement = CONCAT(
            " WITH replacement_entries AS (SELECT * FROM TPFDataSciDB.tpf_jams WHERE system_id=",system_id_replacement,")"
            " SELECT JSON_ARRAYAGG(JSON_OBJECT('jam_id_to_remove',a.id,'jam_id_replacement', b.id))"
            " INTO @jam_id_table"
            " FROM TPFDataSciDB.tpf_jams AS a, replacement_entries AS b"
            " WHERE a.system_id = ",system_id_to_remove,
            " AND a.jam = b.jam AND a.subsystem = b.subsystem AND a.is_jam_app = b.is_jam_app AND a.id != b.id;"
         );
         CALL tpf_execute_immediate(working_statement);
         -- if we have any JVM ids, we need to replace/remove them
         IF @jam_id_table IS NOT NULL AND @jam_id_table != ""
         THEN
            -- loop through the jam_id remove/replace pairs
            SET i = 0;
            WHILE i < JSON_LENGTH(@jam_id_table) DO
               SET jam_id_table_entry =  JSON_EXTRACT(@jam_id_table,CONCAT('$[',i,']'));
               SET i = i + 1;
               SET jam_id_to_remove = json_value(jam_id_table_entry,'$.jam_id_to_remove');
               SET jam_id_replacement = json_value(jam_id_table_entry,'$.jam_id_replacement');

               -- remove the id from system table
               SET working_statement = CONCAT(
                  " DELETE FROM TPFDataSciDB.tpf_jams WHERE id=",jam_id_to_remove,";"
               );
               CALL tpf_execute_immediate(working_statement);
            END WHILE;
         END IF;

         -- There might be a jam_id for the system id to remove, but no replacement.
         -- Here we can just, replace the system id
         SET working_statement = CONCAT(
            " UPDATE TPFDataSciDB.tpf_jams",
            " SET system_id=",system_id_replacement,
            " WHERE system_id=",system_id_to_remove,";"
         );
         CALL tpf_execute_immediate(working_statement);
      END WHILE;
   END IF;
END WHILE;

-- We're done handling the JAM ids so now we can take care of the system ids in the metrics tables


-- Create the list of tables to be processed
SET table_list = 'common_deploy_metrics_table,nvp_message_metrics_table,service_app_metrics_table,system_metrics_table,system_tcpip_table,';
SET join_list = 'AND a.subsystem_id = b.subsystem_id AND a.cd_key_id = b.cd_key_id,AND a.value_id = b.value_id,AND a.subsystem_id = b.subsystem_id AND a.operation_id = b.operation_id,,AND a.app_name_id = b.app_name_id,';

-- Loop through our list of tables
WHILE (LOCATE(',', table_list) > 0)
DO
    SET table_name =  SUBSTRING_INDEX(table_list,',',1);
    SET table_list = SUBSTRING(table_list, LOCATE(',',table_list) + 1);

    SET join_string =  SUBSTRING_INDEX(join_list,',',1);
    SET join_list = SUBSTRING(join_list, LOCATE(',',join_list) + 1);

    -- if pruning is installed, get the columns we need to set to null for this table.
    IF is_pruning_installed="YES"
    THEN
       -- get the pruning metric columns
       SET working_statement = CONCAT(
          " SELECT concat(group_concat('a.`', COLUMN_NAME, '` = NULL' SEPARATOR ', '))",
          " INTO @pruning_columns",
          " FROM information_schema.COLUMNS c",
          " WHERE c.TABLE_SCHEMA = 'TPFDataSciDB'",
          " AND c.TABLE_NAME = '",table_name,"'",
          " AND (c.COLUMN_NAME LIKE '%-max_per_second' OR c.COLUMN_NAME LIKE '%-min_per_second' OR c.COLUMN_NAME LIKE '%-per_minute');"
       );
       CALL tpf_execute_immediate(working_statement);
    END IF;

    -- loop through the arrays and determine the ids to remove/replace
    SET array_of_arrays_index = 0;
    WHILE array_of_arrays_index < JSON_LENGTH(array_of_matching_system_id_arrays) DO
       -- get an array of ids
       SET array_of_matching_system_ids =  JSON_EXTRACT(array_of_matching_system_id_arrays,CONCAT('$[',array_of_arrays_index,']'));
       SET array_of_arrays_index = array_of_arrays_index + 1;

       -- If there's only one id, there's nothing to do, so we'll skip it.
       IF JSON_LENGTH(array_of_matching_system_ids) > 1
       THEN
          -- we'll assume that the last id is the most recent, it might not be, but it's a good enough guess
          SET system_id_replacement = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',JSON_LENGTH(array_of_matching_system_ids) - 1,']'));

          -- if pruning is installed, we need to handle any summarized entries that crossed 1 minute
          -- we'll take care of these first to help identify them
          IF is_pruning_installed="YES"
          THEN

             -- loop through all combinations of ids (ie 1, 2, 3: 3,2 - 3,1 - 2,1)
             -- set the summarized values to NULL
             SET last_system_index = JSON_LENGTH(array_of_matching_system_ids) - 1;
             WHILE last_system_index > 0 DO
                SET system_id_last = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',last_system_index,']'));

                SET array_of_ids_index = 0;
                WHILE array_of_ids_index < last_system_index DO
                   -- get an id to iterate on
                   SET system_id_iterator = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
                   SET array_of_ids_index = array_of_ids_index + 1;

                   -- Identify the entries to be deleted, but we need to delete them later.
                   SET working_statement = CONCAT(
                      " UPDATE TPFDataSciDB.",table_name," a",
                      " JOIN ( ",
                      "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE system_id=",system_id_last," AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                      "      ) b ON a.time=b.time ",join_string,
                      " SET a.entry_is_pruned=2",
                      " WHERE a.system_id=",system_id_iterator," AND a.duration IS NULL AND a.entry_is_pruned=1;"
                   );
                   CALL tpf_execute_immediate(working_statement);
                   SET working_statement = CONCAT(
                      " UPDATE TPFDataSciDB.",table_name," a",
                      " JOIN ( ",
                      "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE system_id=",system_id_iterator," AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                      "      ) b ON a.time=b.time ",join_string,
                      " SET a.entry_is_pruned=2",
                      " WHERE a.system_id=",system_id_last," AND a.duration IS NULL AND a.entry_is_pruned=1;"
                   );
                   CALL tpf_execute_immediate(working_statement);

                   -- remove summarized metrics for entries with real metrics that collide with summarized entries from another system id that don't have real metrics
                   SET working_statement = CONCAT(
                      " UPDATE TPFDataSciDB.",table_name," a",
                      " JOIN ( ",
                      "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE system_id=",system_id_last," AND duration IS NULL AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                      "      ) b ON a.time=b.time ",join_string,
                      " SET a.entry_is_pruned=0,",@pruning_columns,
                      " WHERE a.system_id=",system_id_iterator," AND a.duration IS NOT NULL AND a.entry_is_pruned=1;"
                   );
                   CALL tpf_execute_immediate(working_statement);
                   SET working_statement = CONCAT(
                      " UPDATE TPFDataSciDB.",table_name," a",
                      " JOIN ( ",
                      "       WITH cte AS (SELECT * FROM TPFDataSciDB.",table_name," WHERE system_id=",system_id_iterator," AND duration IS NULL AND (entry_is_pruned=1 OR entry_is_pruned=2)) SELECT * FROM cte"
                      "      ) b ON a.time=b.time ",join_string,
                      " SET a.entry_is_pruned=0,",@pruning_columns,
                      " WHERE a.system_id=",system_id_last," AND a.duration IS NOT NULL AND a.entry_is_pruned=1;"
                   );
                   CALL tpf_execute_immediate(working_statement);
                END WHILE;
                SET last_system_index = last_system_index - 1;
             END WHILE;

             -- remove any rows that have pruning but no real metrics
             SET working_statement = CONCAT(
                " DELETE FROM TPFDataSciDB.",table_name,
                " WHERE duration IS NULL AND entry_is_pruned=2;"
             );
             CALL tpf_execute_immediate(working_statement);
          END IF;

          -- loop through the ids again to update all entries to the replacement system id
          SET array_of_ids_index = 0;
          WHILE array_of_ids_index < JSON_LENGTH(array_of_matching_system_ids) - 1 DO
             -- get an id to remove
             SET system_id_to_remove = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
             SET array_of_ids_index = array_of_ids_index + 1;

             -- convert everything left to the new system id.
             SET working_statement = CONCAT(
                " UPDATE IGNORE TPFDataSciDB.",table_name,
                " SET system_id=",system_id_replacement,
                " WHERE system_id=",system_id_to_remove,";"
             );
             CALL tpf_execute_immediate(working_statement);

             -- count the number of rows that were not removed.
             SET working_statement = CONCAT(
                " SELECT COUNT(*)",
                " INTO @not_updated_count",
                " FROM TPFDataSciDB.",table_name,
                " WHERE system_id=",system_id_to_remove,";"
             );
             CALL tpf_execute_immediate(working_statement);

             -- if we have any rows that we couldn't updated, we need to remove them now.
             -- This can happen for rows that are created from multiple writes to the same row.
             -- ie                                        time  system_id   ECBs   actual_cpu
             -- system block writes >  2023-12-20 19:23:45.000          1     50
             -- system msg   writes >  2023-12-20 19:23:45.000          2                 300
             IF @not_updated_count IS NOT NULL AND @not_updated_count > 0
             THEN
                -- print rows we're going to remove
                SELECT CONCAT("Due to conflicts with system_id ",system_id_replacement,", following rows are being removed from ",table_name," for system_id ", system_id_to_remove) as warning_message;
                SET working_statement = CONCAT(
                   " SELECT * FROM TPFDataSciDB.",table_name,
                   " WHERE system_id=",system_id_to_remove,";"
                );
                CALL tpf_execute_immediate(working_statement);

                -- delete the conflicting rows
                SET working_statement = CONCAT(
                   " DELETE FROM TPFDataSciDB.",table_name,
                   " WHERE system_id=",system_id_to_remove,";"
                );
                CALL tpf_execute_immediate(working_statement);
             END IF;
          END WHILE;
       END IF;
    END WHILE;
END WHILE;

-- loop through the arrays and determine the ids to remove/replace
SET array_of_arrays_index = 0;
WHILE array_of_arrays_index < JSON_LENGTH(array_of_matching_system_id_arrays) DO
   -- get an array of ids
   SET array_of_matching_system_ids =  JSON_EXTRACT(array_of_matching_system_id_arrays,CONCAT('$[',array_of_arrays_index,']'));
   SET array_of_arrays_index = array_of_arrays_index + 1;

   -- If there's only one id, there's nothing to do, so we'll skip it.
   IF JSON_LENGTH(array_of_matching_system_ids) > 1
   THEN
      -- we'll assume that the last id is the most recent, it might not be, but it's a good enough guess
      SET array_of_ids_index = 0;
      WHILE array_of_ids_index < JSON_LENGTH(array_of_matching_system_ids) - 1 DO
         -- get an id to remove
         SET system_id_to_remove = JSON_EXTRACT(array_of_matching_system_ids,CONCAT('$[',array_of_ids_index,']'));
         SET array_of_ids_index = array_of_ids_index + 1;

         -- remove the id from system table
         SET working_statement = CONCAT(
            " DELETE FROM TPFDataSciDB.tpf_systems WHERE id=",system_id_to_remove,";"
         );
         CALL tpf_execute_immediate(working_statement);
      END WHILE;
   END IF;
END WHILE;

SELECT CONCAT("Finished PJ47174 migration");

END//
DELIMITER ;
