/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2022, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20220131    PJ46608 - multi-JVM and system selector support      */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
/*  10052022    PJ46853 - Change skip factor to fractional scale     */
/*                        factor                                     */
/*  20230601    PJ47066 - RTMC database migration failures can cause */
/*                        unrecoverable version state                */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20230719    PJ47106 - Display multiple Kafka producers           */
/*  20240104    PJ47174 - Remove ip address from tpf_systems index   */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Creates a new ZRTMC (TPFDataSciDB) database or      */
/*               upgrades existing schema                            */
/*                                                                   */
/*********************************************************************/

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP PROCEDURE IF EXISTS `tpf_get_zrtmc_schema_version`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_zrtmc_schema_version` (
    OUT `out_db_version` VARCHAR(16)
)
tpf_get_zrtmc_schema_version:BEGIN

    DECLARE current_database VARCHAR(64);
    DECLARE version_table_exists BOOLEAN;

    SELECT DATABASE() INTO current_database;

    SELECT EXISTS (
        SELECT 1
        FROM information_schema.TABLES
        WHERE TABLE_SCHEMA=current_database AND TABLE_NAME='version_history'
        LIMIT 1
    ) INTO version_table_exists;

    IF NOT version_table_exists
    THEN
        SET out_db_version = '0.0';
    ELSE
        SELECT version
        FROM version_history
        ORDER BY major DESC, minor DESC
        LIMIT 1
        INTO out_db_version;
    END IF;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_update_version_history`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_update_version_history` (
    IN `in_major_from` INT,
    IN `in_minor_from` INT,
    IN `in_major_to` INT,
    IN `in_minor_to` INT
)
tpf_update_version_history:BEGIN

    DECLARE from_version VARCHAR(32);
    DECLARE to_version VARCHAR(32);

    SET from_version = CONCAT_WS('.', in_major_from, in_minor_from);
    SET to_version = CONCAT_WS('.', in_major_to, in_minor_to);

    -- Update version history
    INSERT INTO `version_history` (`version`, `valid_from`, `valid_to`, `major`, `minor`)
    VALUES (to_version, UTC_TIMESTAMP(), NULL, in_major_to, in_minor_to);

    UPDATE `version_history` v_new, `version_history` v_old
    SET v_old.valid_to=v_new.valid_from
    WHERE v_old.version = from_version AND v_new.version = to_version;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_compare_version`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_compare_version` (
    `version_lt` VARCHAR(16),
    `version_rt` VARCHAR(16)
)
RETURNS INT
DETERMINISTIC
BEGIN
    DECLARE major_lt INT;
    DECLARE minor_lt INT;
    DECLARE major_rt INT;
    DECLARE minor_rt INT;

    SET major_lt = CAST(SUBSTRING_INDEX(version_lt, '.', 1) AS SIGNED);
    SET minor_lt = CASE WHEN version_lt LIKE "%.%" THEN CAST(SUBSTRING_INDEX(version_lt, '.', -1) AS SIGNED) ELSE 0 END;

    SET major_rt = CAST(SUBSTRING_INDEX(version_rt, '.', 1) AS SIGNED);
    SET minor_rt = CASE WHEN version_rt LIKE "%.%" THEN CAST(SUBSTRING_INDEX(version_rt, '.', -1) AS SIGNED) ELSE 0 END;

    IF major_lt != major_rt
    THEN
        RETURN SIGN(major_lt - major_rt);
    ELSE
        RETURN SIGN(minor_lt - minor_rt);
    END IF;
END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_create_zrtmc_schema_1_0`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_create_zrtmc_schema_1_0` ()
tpf_create_zrtmc_schema_1_0:BEGIN

    SELECT 'Upgrading to schema version 1.0'; -- Info message

    CREATE TABLE IF NOT EXISTS `version_history` (
        `version_id` INT UNSIGNED AUTO_INCREMENT,
        `version` VARCHAR(16) NOT NULL,
        `valid_from` DATETIME NOT NULL,
        `valid_to` DATETIME,
        `major` INT NOT NULL,
        `minor` INT NOT NULL,
        PRIMARY KEY (`version_id`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    INSERT INTO `version_history` (`version`, `valid_from`, `valid_to`, `major`, `minor`)
    VALUES ('1.0', UTC_TIMESTAMP(), NULL, 1, 0);

    SELECT 'Creating new tables'; -- Info message

    CREATE TABLE IF NOT EXISTS `system_metrics_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `actual_cpu` int(10) unsigned zerofill DEFAULT NULL,
      `calculated_cpu` int(10) unsigned zerofill DEFAULT NULL,
      `modeled_cpu` int(10) unsigned zerofill DEFAULT NULL,
      `modeled_cpu_data_points` tinyint(3) unsigned zerofill DEFAULT NULL,
      `inuse_istreams` int(10) unsigned zerofill DEFAULT NULL,
      `active_istreams` int(10) unsigned zerofill DEFAULT NULL,
      `inuse_ecb` int(10) unsigned zerofill DEFAULT NULL,
      `inuse_iob` int(10) unsigned zerofill DEFAULT NULL,
      PRIMARY KEY (`time`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `system_tcpip_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `app_name` varchar(255) DEFAULT NULL,
      `port` int(10) unsigned zerofill DEFAULT NULL,
      `msg_cnt` int(10) unsigned zerofill DEFAULT NULL,
      `tot_bytes` int(10) unsigned zerofill DEFAULT NULL,
      `bytes_sent` int(10) unsigned zerofill DEFAULT NULL,
      `bytes_recvd` int(10) unsigned zerofill DEFAULT NULL,
      `msg_sent` int(10) unsigned zerofill DEFAULT NULL,
      `msg_recvd` int(10) unsigned zerofill DEFAULT NULL,
      UNIQUE KEY `system_tcpip_table_time_IDX` (`time`,`app_name`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `msg_types_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `avg_nvp_skip_factor` int(10) unsigned zerofill DEFAULT NULL,
      `msg_type` varchar(255) DEFAULT NULL,
      `msg_cnt` int(10) unsigned zerofill DEFAULT NULL,
      `avg_cpu` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_exist_time` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_cret` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find_file` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_file` bigint(20) unsigned zerofill DEFAULT NULL,
      UNIQUE KEY `msg_types_table_time_IDX` (`time`,`msg_type`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `horizontal_nvps_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `avg_nvp_skip_factor` int(10) unsigned zerofill DEFAULT NULL,
      `nvps` varchar(255) DEFAULT NULL,
      `msg_cnt` int(10) unsigned zerofill DEFAULT NULL,
      `avg_cpu` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_exist_time` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_cret` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find_file` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_file` bigint(20) unsigned zerofill DEFAULT NULL,
      UNIQUE KEY `horizontal_nvps_table_time_IDX` (`time`,`nvps`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `code_packages_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `avg_nvp_skip_factor` int(10) unsigned zerofill DEFAULT NULL,
      `code_package` varchar(255) DEFAULT NULL,
      `msg_cnt` int(10) unsigned zerofill DEFAULT NULL,
      `avg_cpu` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_exist_time` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_cret` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find_file` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_file` bigint(20) unsigned zerofill DEFAULT NULL,
      UNIQUE KEY `code_packages_table_time_IDX` (`time`,`code_package`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `msg_types_and_code_packages_table` (
      `time` datetime(3) NOT NULL,
      `duration` int(10) unsigned zerofill DEFAULT NULL,
      `avg_nvp_skip_factor` int(10) unsigned zerofill DEFAULT NULL,
      `nvps` varchar(500) DEFAULT NULL,
      `msg_cnt` int(10) unsigned zerofill DEFAULT NULL,
      `avg_cpu` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_exist_time` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_cret` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find_file` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_find` bigint(20) unsigned zerofill DEFAULT NULL,
      `avg_file` bigint(20) unsigned zerofill DEFAULT NULL,
      UNIQUE KEY `msg_types_and_code_packages_table_time_IDX` (`time`,`nvps`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_jvm_metrics_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `pid` int(10) NOT NULL,
        `num_jvms_expected` int(10) unsigned DEFAULT NULL,
        `num_jvms_actual` int(10) unsigned DEFAULT NULL,
        `num_jam_threads_total` int(10) unsigned DEFAULT NULL,
        `num_jam_threads_inuse` int(10) unsigned DEFAULT NULL,
        `max_jam_threads_observed` int(10) unsigned DEFAULT NULL,
        `cpu` DOUBLE unsigned DEFAULT NULL,
        `avg_response_time` DOUBLE unsigned DEFAULT NULL,
        `num_jvm_threads_daemon` int(10) unsigned DEFAULT NULL,
        `num_jvm_threads_latest` int(10) unsigned DEFAULT NULL,
        `avg_gc_interval` DOUBLE DEFAULT NULL,
        `avg_gc_pause` DOUBLE DEFAULT NULL,
        `avg_gc_interval_global` DOUBLE DEFAULT NULL,
        `avg_gc_pause_global` DOUBLE DEFAULT NULL,
        `used_heap` DOUBLE unsigned DEFAULT NULL,
        `heap_size` DOUBLE unsigned DEFAULT NULL,
        `is_jam_app` BOOLEAN DEFAULT NULL,
        `num_gc_events` int unsigned DEFAULT 0,
        UNIQUE KEY `jvm_jvm_metrics_table_time_IDX` (`time`,`jam`,`pid`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_jam_jax_rs_metrics_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `app` VARCHAR(128) NOT NULL,
        `pid` int(10) NOT NULL,
        `op_count` int(10) unsigned DEFAULT NULL,
        `min_response_time` DOUBLE unsigned DEFAULT NULL,
        `max_response_time` DOUBLE unsigned DEFAULT NULL,
        `avg_response_time` DOUBLE unsigned DEFAULT NULL,
        `app_faults_checked` int(10) unsigned DEFAULT NULL,
        `app_faults_unchecked` int(10) unsigned DEFAULT NULL,
        `runtime_faults_logical` int(10) unsigned DEFAULT NULL,
        `runtime_faults_app` int(10) unsigned DEFAULT NULL,
        `handling_time` DOUBLE unsigned DEFAULT NULL,
        UNIQUE KEY `jvm_jam_jax_rs_metrics_table_time_IDX` (`time`,`jam`,`pid`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_jam_jax_rs_op_metrics_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `app` VARCHAR(128) NOT NULL,
        `pid` int(10) NOT NULL,
        `op_id` VARCHAR(128) NOT NULL,
        `min_response_time` DOUBLE unsigned DEFAULT NULL,
        `max_response_time` DOUBLE unsigned DEFAULT NULL,
        `avg_response_time` DOUBLE unsigned DEFAULT NULL,
        `app_faults_checked` int(10) unsigned DEFAULT NULL,
        `app_faults_unchecked` int(10) unsigned DEFAULT NULL,
        `runtime_faults_logical` int(10) unsigned DEFAULT NULL,
        `runtime_faults_app` int(10) unsigned DEFAULT NULL,
        `handling_time` DOUBLE unsigned DEFAULT NULL,
        UNIQUE KEY `jvm_jam_jax_rs_op_metrics_table_time_IDX` (`time`,`jam`,`pid`,`op_id`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_lock_metrics_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `pid` int(10) NOT NULL,
        `lock` VARCHAR(18) NOT NULL,
        `description` VARCHAR(256) DEFAULT NULL,
        `miss_pct` DOUBLE(6,3) unsigned DEFAULT NULL,
        `util_pct` DOUBLE(6,3) unsigned DEFAULT NULL,
        `gets` BIGINT unsigned DEFAULT NULL,
        `gets_slow` int(10) unsigned DEFAULT NULL,
        `gets_rec` int(10) unsigned DEFAULT NULL,
        `hold_time` BIGINT unsigned DEFAULT NULL,
        `is_jam_app` BOOLEAN DEFAULT NULL,
        UNIQUE KEY `jvm_lock_metrics_table_time_IDX` (`time`,`jam`,`pid`,`lock`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_static_data_table` (
        `jam` VARCHAR(32) NOT NULL,
        `pid` int(10) NOT NULL,
        `start_time` datetime(3) NOT NULL,
        `log_dir` TEXT DEFAULT NULL,
        `args` TEXT DEFAULT NULL,
        `last_time` datetime(3) DEFAULT NULL,
        UNIQUE KEY `jvm_static_info_table_IDX` (`jam`,`pid`,`start_time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_jmx_app_Kafka_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `pid` int(10) NOT NULL,
        `req_total` BIGINT DEFAULT NULL,
        `reqs_in_flight` int(10) DEFAULT NULL,
        `avg_batch_size` DOUBLE DEFAULT NULL,
        `rec_send_rate` DOUBLE DEFAULT NULL,
        `avg_rec_size` DOUBLE DEFAULT NULL,
        `avg_req_latency` DOUBLE DEFAULT NULL,
        `waiting_threads` int(10) DEFAULT NULL,
        UNIQUE KEY `jvm_jmx_app_kafka_table_time_IDX` (`time`,`jam`,`pid`) USING BTREE
    )  ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_jmx_app_JMBeansDrv_table` (
        `time` datetime(3) NOT NULL,
        `jam` VARCHAR(32) NOT NULL,
        `pid` int(10) NOT NULL,
        `buff` TEXT DEFAULT NULL,
        `ecb` BIGINT DEFAULT NULL,
        `frm` BIGINT DEFAULT NULL,
        `frm1mb` BIGINT DEFAULT NULL,
        `req_counter` BIGINT DEFAULT NULL,
        `swb` BIGINT DEFAULT NULL,
        `sequence_num` BIGINT DEFAULT NULL,
        UNIQUE KEY `jvm_jmx_app_JMBeansDrv_table_time_IDX` (`time`,`jam`,`pid`) USING BTREE
    )  ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Done upgrading to schema version 1.0'; -- Info message


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_0_to_1_1`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_0_to_1_1` ()
tpf_upgrade_zrtmc_schema_1_0_to_1_1:BEGIN

    SELECT 'Upgrading to schema version 1.1'; -- Info message

    -- Update version history
    INSERT INTO `version_history` (`version`, `valid_from`, `valid_to`, `major`, `minor`)
    VALUES ('1.1', UTC_TIMESTAMP(), NULL, 1, 1);

    UPDATE `version_history` v_new, `version_history` v_old
    SET v_old.valid_to=v_new.valid_from
    WHERE v_old.version = '1.0' AND v_new.version = '1.1';

    SELECT 'Creating new tables'; -- Info message

    -- Add new tables
    CREATE TABLE IF NOT EXISTS `tpf_systems` (
        `system_name` VARCHAR(8) NOT NULL,
        `complex_name` VARCHAR(8) NOT NULL,
        `processor_id` VARCHAR(1) NOT NULL,
        `ip_address` VARCHAR(15) NOT NULL,
        `cec_name` VARCHAR(16) NOT NULL,
        `environment` VARCHAR(10) NULL DEFAULT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`processor_id`, `system_name`, `ip_address`, `complex_name`, `cec_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    INSERT INTO `tpf_systems` (`id`, `system_name`, `complex_name`, `processor_id`, `ip_address`, `cec_name`, `environment`)
    VALUES (1, '?', '?', '?', '?', '?', '?');

    CREATE TABLE IF NOT EXISTS `tpf_jams` (
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem` VARCHAR(4) NOT NULL DEFAULT '?',
        `jam` VARCHAR(32) NOT NULL,
        `id` MEDIUMINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`system_id`, `subsystem`, `jam`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `jvm_operations` (
        `operation` VARCHAR(128) NOT NULL,
        `id` MEDIUMINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`operation`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Altering existing table schemas'; -- Info message

    -- Add system_id columns and set all to default of 0 (unknown system)
    ALTER TABLE `system_metrics_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        ADD COLUMN `istream_cap` INT(10) UNSIGNED ZEROFILL DEFAULT NULL AFTER `inuse_istreams`,
        DROP COLUMN active_istreams,
        DROP PRIMARY KEY,
        ADD PRIMARY KEY (`system_id`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `system_tcpip_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        DROP KEY `system_tcpip_table_time_IDX`,
        MODIFY COLUMN `app_name` VARCHAR(255) NOT NULL,
        ADD PRIMARY KEY (`system_id`, `app_name`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `msg_types_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        ADD COLUMN `est_msg_rate` BIGINT(20) UNSIGNED AS (`msg_cnt` * `avg_nvp_skip_factor` / NULLIF(`duration`, 0)) VIRTUAL,
        DROP KEY `msg_types_table_time_IDX`,
        MODIFY COLUMN `msg_type` VARCHAR(255) NOT NULL,
        ADD PRIMARY KEY (`system_id`, `msg_type`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `horizontal_nvps_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        DROP KEY horizontal_nvps_table_time_IDX,
        MODIFY COLUMN `nvps` VARCHAR(255) NOT NULL,
        ADD PRIMARY KEY (`system_id`, `nvps`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `code_packages_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        DROP KEY `code_packages_table_time_IDX`,
        MODIFY COLUMN `code_package` VARCHAR(255) NOT NULL,
        ADD PRIMARY KEY (`system_id`, `code_package`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `msg_types_and_code_packages_table`
        ADD COLUMN `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`' AFTER `time`,
        DROP KEY `msg_types_and_code_packages_table_time_IDX`,
        MODIFY COLUMN `nvps` VARCHAR(500) NOT NULL,
        ADD PRIMARY KEY (`system_id`, `nvps`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;


    SELECT 'Updating existing data to new table schemas'; -- Info message

    -- Find all the distinct jams and assign jam_id values in tpf_jams
    INSERT INTO `tpf_jams` (jam)
    SELECT DISTINCT jam FROM `jvm_jvm_metrics_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_jam_jax_rs_metrics_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_jam_jax_rs_op_metrics_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_lock_metrics_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_static_data_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_jmx_app_Kafka_table`
    UNION
    SELECT DISTINCT jam FROM `jvm_jmx_app_JMBeansDrv_table`;

    -- Find all the distinct jam operations and assign INT op_id values in jvm_operations
    INSERT INTO `jvm_operations` (`operation`)
    SELECT DISTINCT `op_id` FROM `jvm_jam_jax_rs_op_metrics_table`;

    -- Convert all jam name strings to a jam_id
    UPDATE `jvm_jvm_metrics_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_jam_jax_rs_metrics_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_jam_jax_rs_op_metrics_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_lock_metrics_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_static_data_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_jmx_app_Kafka_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';

    UPDATE `jvm_jmx_app_JMBeansDrv_table` j, `tpf_jams` tj
    SET j.jam=tj.id
    WHERE j.jam=tj.jam AND tj.system_id=1 AND tj.subsystem='?';


    -- Convert all op_id strings to INT op_id
    UPDATE `jvm_jam_jax_rs_op_metrics_table` j, `jvm_operations` jo
    SET j.op_id=jo.id
    WHERE j.op_id=jo.operation;

    -- Add id columns and update PRIMARY KEY
    ALTER TABLE `jvm_jvm_metrics_table`
        DROP KEY `jvm_jvm_metrics_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `jvm_jam_jax_rs_metrics_table`
        DROP KEY `jvm_jam_jax_rs_metrics_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `jvm_jam_jax_rs_op_metrics_table`
        DROP KEY `jvm_jam_jax_rs_op_metrics_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        MODIFY COLUMN `op_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `jvm_operations`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `op_id`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `jvm_lock_metrics_table`
        DROP KEY `jvm_lock_metrics_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `lock`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `jvm_static_data_table`
        DROP KEY `jvm_static_info_table_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `start_time`) USING BTREE;

    ALTER TABLE `jvm_jmx_app_Kafka_table`
        DROP KEY `jvm_jmx_app_kafka_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    ALTER TABLE `jvm_jmx_app_JMBeansDrv_table`
        DROP KEY `jvm_jmx_app_JMBeansDrv_table_time_IDX`,
        CHANGE COLUMN `jam` `jam_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_jams`',
        ADD PRIMARY KEY (`jam_id`, `pid`, `time`) USING BTREE,
        ADD INDEX `time_index` (`time`) USING BTREE;

    SELECT 'Done upgrading to schema version 1.1'; -- Info message


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_1_to_1_2`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_1_to_1_2` ()
tpf_upgrade_zrtmc_schema_1_1_to_1_2:BEGIN

    DECLARE dimension_value_rows TEXT;

    SELECT 'Upgrading to schema version 1.2'; -- Info message

    -- Change cec_name to cpc_name
    ALTER TABLE `tpf_systems`
        RENAME COLUMN `cec_name` TO `cpc_name`;

    SELECT 'Creating new name-value pair tables'; -- Info message

    -- Condense all nvp tables to a single metrics table (plus lookup tables)
    CREATE TABLE IF NOT EXISTS `app_dimensions` (
        `dimension` VARCHAR(64) NOT NULL,
        `id` MEDIUMINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`dimension`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `app_dimension_values` (
        `dimension_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `app_dimensions`',
        `value` VARCHAR(500) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`dimension_id`, `value`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `nvp_message_metrics_table` (
        `time` datetime(3) NOT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `value_id` INT UNSIGNED NOT NULL COMMENT 'References `id` in `app_dimension_values`',
        `duration` INT UNSIGNED DEFAULT NULL,
        `avg_nvp_skip_factor` INT UNSIGNED DEFAULT NULL,
        `msg_cnt` INT UNSIGNED DEFAULT NULL,
        `avg_cpu` BIGINT UNSIGNED DEFAULT NULL,
        `avg_exist_time` BIGINT UNSIGNED DEFAULT NULL,
        `avg_cret` BIGINT UNSIGNED DEFAULT NULL,
        `avg_find_file` BIGINT UNSIGNED DEFAULT NULL,
        `avg_find` BIGINT UNSIGNED DEFAULT NULL,
        `avg_file` BIGINT UNSIGNED DEFAULT NULL,
        `est_msg_rate` BIGINT UNSIGNED AS (`msg_cnt` * (`avg_nvp_skip_factor` + 1)  / NULLIF(`duration`, 0)) VIRTUAL,
        PRIMARY KEY (`system_id`, `value_id`, `time`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Populating new lookup tables'; -- Info Message

    -- Set up app_dimensions_table
    INSERT INTO `app_dimensions` (`dimension`) VALUES
        ('Message Type'),
        ('Horizontal NVPs'),
        ('Code Package'),
        ('Message Type and Code Package'),
        ('TCPIP');

    -- Find all the unique Message Types
    WITH msg_types AS (
        SELECT m.msg_type, a.id
        FROM `msg_types_table` m INNER JOIN `app_dimensions` a
        WHERE a.dimension='Message Type'
        GROUP BY m.msg_type
    )
    SELECT GROUP_CONCAT(CONCAT("(", id, ",'", msg_type, "')"))
    INTO dimension_value_rows
    FROM msg_types;

    IF dimension_value_rows IS NOT NULL
    THEN
        CALL tpf_execute_immediate(CONCAT("INSERT INTO `app_dimension_values` (`dimension_id`, `value`) VALUES ", dimension_value_rows));
    END IF;

    SET dimension_value_rows = NULL;

    -- Find all the unique Horizontal NVPs
    WITH hor_nvps AS (
        SELECT h.nvps, a.id
        FROM `horizontal_nvps_table` h INNER JOIN `app_dimensions` a
        WHERE a.dimension='Horizontal NVPs'
        GROUP BY h.nvps
    )
    SELECT GROUP_CONCAT(CONCAT("(", id, ",'", nvps, "')"))
    INTO dimension_value_rows
    FROM hor_nvps;

    IF dimension_value_rows IS NOT NULL
    THEN
        CALL tpf_execute_immediate(CONCAT("INSERT INTO `app_dimension_values` (`dimension_id`, `value`) VALUES ", dimension_value_rows));
    END IF;

    SET dimension_value_rows = NULL;

    -- Find all the unique Code Packages
    WITH code_pkgs AS (
        SELECT c.code_package, a.id
        FROM `code_packages_table` c INNER JOIN `app_dimensions` a
        WHERE a.dimension='Code Package'
        GROUP BY c.code_package
    )
    SELECT GROUP_CONCAT(CONCAT("(", id, ",'", code_package, "')"))
    INTO dimension_value_rows
    FROM code_pkgs;

    IF dimension_value_rows IS NOT NULL
    THEN
        CALL tpf_execute_immediate(CONCAT("INSERT INTO `app_dimension_values` (`dimension_id`, `value`) VALUES ", dimension_value_rows));
    END IF;

    SET dimension_value_rows = NULL;

    -- Find all the unique Message Type and Code Package
    WITH msg_type_code_pkgs AS (
        SELECT mc.nvps, a.id
        FROM `msg_types_and_code_packages_table` mc INNER JOIN `app_dimensions` a
        WHERE a.dimension='Message Type and Code Package'
        GROUP BY mc.nvps
    )
    SELECT GROUP_CONCAT(CONCAT("(", id, ",'", nvps, "')"))
    INTO dimension_value_rows
    FROM msg_type_code_pkgs;

    IF dimension_value_rows IS NOT NULL
    THEN
        CALL tpf_execute_immediate(CONCAT("INSERT INTO `app_dimension_values` (`dimension_id`, `value`) VALUES ", dimension_value_rows));
    END IF;

    SET dimension_value_rows = NULL;

    -- Find all the unique TCPIP app_names
    WITH tcpip_apps AS (
        SELECT t.app_name, a.id
        FROM `system_tcpip_table` t INNER JOIN `app_dimensions` a
        WHERE a.dimension='TCPIP'
        GROUP BY t.app_name
    )
    SELECT GROUP_CONCAT(CONCAT("(", id, ",'", app_name, "')"))
    INTO dimension_value_rows
    FROM tcpip_apps;

    IF dimension_value_rows IS NOT NULL
    THEN
        CALL tpf_execute_immediate(CONCAT("INSERT INTO `app_dimension_values` (`dimension_id`, `value`) VALUES ", dimension_value_rows));
    END IF;

    SELECT 'Migrating existing data in system_tcpip_table to new schema'; -- Info Message

    -- Convert all TCPIP app_names to their value_id
    UPDATE system_tcpip_table t, app_dimension_values v, app_dimensions d
    SET t.app_name=v.id
    WHERE d.dimension='TCPIP' AND d.id=v.dimension_id AND t.app_name=v.value;

    -- Update system_tcpip_table schema
    ALTER TABLE system_tcpip_table
        DROP PRIMARY KEY,
        CHANGE COLUMN app_name app_name_id INT UNSIGNED NOT NULL COMMENT 'References `id` in `app_dimension_values`',
        ADD PRIMARY KEY (`system_id`, `app_name_id`, `time`) USING BTREE;

    SELECT 'Migrating existing name-value pair data to new table nvp_message_metrics_table'; -- Info Message

    -- Transfer rows from old nvp tables to new nvp table
    INSERT INTO nvp_message_metrics_table (`time`, `system_id`, `value_id`, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
                                           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`)
    SELECT `time`, `system_id`, v.id, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`
    FROM msg_types_table m INNER JOIN app_dimension_values v ON m.msg_type=v.value INNER JOIN app_dimensions d ON d.id=v.dimension_id
    WHERE d.dimension='Message Type';

    INSERT INTO nvp_message_metrics_table (`time`, `system_id`, `value_id`, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
                                           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`)
    SELECT `time`, `system_id`, v.id, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`
    FROM horizontal_nvps_table h INNER JOIN app_dimension_values v ON h.nvps=v.value INNER JOIN app_dimensions d ON d.id=v.dimension_id
    WHERE d.dimension='Horizontal NVPs';

    INSERT INTO nvp_message_metrics_table (`time`, `system_id`, `value_id`, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
                                           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`)
    SELECT `time`, `system_id`, v.id, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`
    FROM code_packages_table c INNER JOIN app_dimension_values v ON c.code_package=v.value INNER JOIN app_dimensions d ON d.id=v.dimension_id
    WHERE d.dimension='Code Package';

    INSERT INTO nvp_message_metrics_table (`time`, `system_id`, `value_id`, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
                                           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`)
    SELECT `time`, `system_id`, v.id, `duration`, `avg_nvp_skip_factor`, `msg_cnt`,
           `avg_cpu`, `avg_exist_time`, `avg_cret`, `avg_find_file`, `avg_find`, `avg_file`
    FROM msg_types_and_code_packages_table mc INNER JOIN app_dimension_values v ON mc.nvps=v.value INNER JOIN app_dimensions d ON d.id=v.dimension_id
    WHERE d.dimension='Message Type and Code Package';

    -- New app_dimension value format is a bit different: [Shopping, PackageB] instead of [MsgType=Shopping, IOwnerHi=PackageB]
    UPDATE app_dimension_values v, app_dimensions d
    SET v.value=REPLACE(REPLACE(v.value, 'MsgType=', ''), 'IOwnerHi=', '')
    WHERE d.dimension='Message Type and Code Package' AND d.id=v.dimension_id;

    SELECT 'Deleting old name-value pair tables'; -- Info Message

    -- Drop old nvp tables
    DROP TABLE IF EXISTS msg_types_table, horizontal_nvps_table, code_packages_table, msg_types_and_code_packages_table;


    SELECT 'Altering system_metrics_table schema'; -- Info message

    -- Add new system cpu metrics and estimated system metrics
    ALTER TABLE system_metrics_table
        ADD COLUMN workload_cpu SMALLINT UNSIGNED DEFAULT NULL AFTER modeled_cpu_data_points,
        ADD COLUMN lp_cpu SMALLINT UNSIGNED DEFAULT NULL AFTER workload_cpu,
        ADD COLUMN sys_pseudo_cpu SMALLINT UNSIGNED DEFAULT NULL AFTER lp_cpu,
        ADD COLUMN fenced_pseudo_cpu SMALLINT UNSIGNED DEFAULT NULL AFTER sys_pseudo_cpu,
        ADD COLUMN est_msg_cpu BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN est_msg_exist BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN est_msg_find_file INT UNSIGNED DEFAULT NULL,
        ADD COLUMN est_msg_rate INT UNSIGNED DEFAULT NULL;


    SELECT 'Altering tpf_jams table schema'; -- Info message

    -- Move is_jam_app column to tpf_jams
    ALTER TABLE tpf_jams
        ADD COLUMN is_jam_app TINYINT UNSIGNED DEFAULT NULL AFTER jam;

    UPDATE tpf_jams
    SET is_jam_app=(
        SELECT is_jam_app
        FROM jvm_jvm_metrics_table
        WHERE jam_id=id
        LIMIT 1
    );

    ALTER TABLE jvm_jvm_metrics_table
        DROP COLUMN is_jam_app;

    ALTER TABLE jvm_lock_metrics_table
        DROP COLUMN is_jam_app;


    CREATE TABLE IF NOT EXISTS `tpf_subsystems` (
        `subsystem` VARCHAR(4) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`subsystem`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating common_deploy_types, common_deploy_keys and common_deploy_metrics_table'; -- Info message

    CREATE TABLE IF NOT EXISTS `common_deploy_types` (
        `cd_type` VARCHAR(16) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`cd_type`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `common_deploy_keys` (
        `cd_type_id` INT UNSIGNED NOT NULL COMMENT 'References `id` in `common_deploy_types`',
        `cd_key` VARCHAR(255) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`cd_type_id`, `cd_key`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `common_deploy_metrics_table` (
        `time` datetime(3) NOT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
        `cd_key_id` INT UNSIGNED NOT NULL COMMENT 'References `id` in `common_deploy_keys`',
        `duration` INT UNSIGNED DEFAULT NULL,
        `usage_count` INT UNSIGNED DEFAULT NULL,
        `count_per_second` BIGINT UNSIGNED AS (`usage_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        PRIMARY KEY (`system_id`, `subsystem_id`, `cd_key_id`, `time`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;


    SELECT 'Creating service_app_operations and service_app_metrics_table'; -- Info message

    -- Create tables for REST
    CREATE TABLE IF NOT EXISTS `service_app_operations` (
        `operation` VARCHAR(16) NOT NULL,
        `jam_name` VARCHAR(16) DEFAULT NULL,
        `id` MEDIUMINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`operation`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `service_app_metrics_table` (
        `time` datetime(3) NOT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
        `operation_id` MEDIUMINT UNSIGNED NOT NULL COMMENT 'References `id` in `service_app_operations`',
        `duration` INT UNSIGNED DEFAULT NULL,
        `internal_operation_count` INT UNSIGNED DEFAULT NULL,
        `internal_operation_rate` INT UNSIGNED AS (`internal_operation_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `external_operation_count` INT UNSIGNED DEFAULT NULL,
        `external_operation_rate` INT UNSIGNED AS (`external_operation_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `total_operation_count` INT UNSIGNED DEFAULT NULL,
        `total_operation_rate` INT UNSIGNED AS (`total_operation_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `timeout_count` INT UNSIGNED DEFAULT NULL,
        `timeout_rate` INT UNSIGNED AS (`timeout_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `iface_error_count` INT UNSIGNED DEFAULT NULL,
        `iface_error_rate` INT UNSIGNED AS (`iface_error_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `srvc_error_count` INT UNSIGNED DEFAULT NULL,
        `srvc_error_rate` INT UNSIGNED AS (`srvc_error_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        `unrec_error_count` INT UNSIGNED DEFAULT NULL,
        `unrec_error_rate` INT UNSIGNED AS (`unrec_error_count` * 1000 / NULLIF(`duration`, 0)) VIRTUAL,
        PRIMARY KEY (`system_id`, `subsystem_id`, `operation_id`, `time`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    -- Update version history
    CALL tpf_update_version_history(1,1,1,2);

    SELECT 'Done upgrading to schema version 1.2'; -- Info message


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_2_to_1_3`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_2_to_1_3` ()
tpf_upgrade_zrtmc_schema_1_2_to_1_3:BEGIN

    DECLARE dimension_value_rows TEXT;

    SELECT 'Upgrading to schema version 1.3'; -- Info message

    SELECT 'Migrating existing name-value pair data skip factor to float scale factor'; -- Info Message

    -- Change avg_nvp_skip_factor to avg_nvp_scale_factor and from int to float and to 1s based instead of 0 based
    UPDATE `nvp_message_metrics_table` SET avg_nvp_skip_factor = avg_nvp_skip_factor + 1;
    ALTER TABLE `nvp_message_metrics_table`
        CHANGE COLUMN `avg_nvp_skip_factor` `avg_nvp_scale_factor` FLOAT UNSIGNED,
        MODIFY COLUMN `est_msg_rate` BIGINT UNSIGNED AS (`msg_cnt` * (`avg_nvp_scale_factor`)  / NULLIF(`duration`, 0)) VIRTUAL;

    -- Update version history
    CALL tpf_update_version_history(1,2,1,3);

    SELECT 'Done upgrading to schema version 1.3'; -- Info message

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_3_to_1_4`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_3_to_1_4` ()
tpf_upgrade_zrtmc_schema_1_3_to_1_4:BEGIN

    DECLARE dimension_value_rows TEXT;

    SELECT 'Upgrading to schema version 1.4'; -- Info message

    SELECT 'Migrating nvp_message_metrics_table existing field names'; -- Info Message

    -- Change avg_nvp_skip_factor to avg_nvp_scale_factor and from int to float and to 1s based instead of 0 based
    ALTER TABLE `nvp_message_metrics_table`
        CHANGE COLUMN `avg_cpu` `cpu_used` BIGINT UNSIGNED,
        CHANGE COLUMN `avg_exist_time` `exist_time` BIGINT UNSIGNED,
        CHANGE COLUMN `avg_cret` `cret` BIGINT UNSIGNED,
        CHANGE COLUMN `avg_find_file` `find_file` BIGINT UNSIGNED,
        CHANGE COLUMN `avg_find` `find` BIGINT UNSIGNED,
        CHANGE COLUMN `avg_file` `file` BIGINT UNSIGNED;

    SELECT 'Add all metrics to nvp_message_metrics_table'; -- Info Message

    ALTER TABLE nvp_message_metrics_table
        ADD COLUMN `dd_time` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `ecb1mf` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `ecb4kf` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `rlog` BIGINT UNSIGNED DEFAULT NULL,

        ADD COLUMN `cmbk` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `cow` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_attac` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_btfil` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_btfnd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cfile` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cfind` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cls_no_rel` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cls_opn` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cls_rels` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_cls_reuse` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbadd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbcls` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbdel` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbdix` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbidx` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbkey` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbmod` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbopn` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbred` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_dbrep` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_detac` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_getcc` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_getfc` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_lrfil` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_lrfnd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_opn_hd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_opn_hnd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_opn_nhd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_opn_nhnd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_pfile` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_pfind` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_relcc` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_relfc` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_rsf_create` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_rsf_delete` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_rsf_read` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `df_rsf_update` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `file_dasd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `file_vfa` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `find_dasd` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `find_vfa` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `gfsl` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `gfss` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `grfs` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `hctl` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `mal` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `relf` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `rout` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `serr` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `snap` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `swbk` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `sysh` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `tour` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `twrt` BIGINT UNSIGNED DEFAULT NULL,
        ADD COLUMN `xwbk` BIGINT UNSIGNED DEFAULT NULL;

    -- Update version history
    CALL tpf_update_version_history(1,3,1,4);

    SELECT 'Done upgrading to schema version 1.4'; -- Info message

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_4_to_1_5`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_4_to_1_5` ()
tpf_upgrade_zrtmc_schema_1_4_to_1_5:BEGIN

    SELECT 'Upgrading to schema version 1.5'; -- Info message

    SELECT 'Migrating jvm_jmx_app_Kafka_table with producer information'; -- Info Message

    -- Change key to include the producer_id so that the producer name can be displayed.

    ALTER TABLE `jvm_jmx_app_Kafka_table`
        ADD COLUMN `producer_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_jmx_producer_table`' AFTER `pid`,
        DROP PRIMARY KEY, ADD  PRIMARY KEY  (`time`,`jam_id`,`pid`,`producer_id`);

    CREATE TABLE IF NOT EXISTS `tpf_jmx_producer_table` (
        `producer_name` VARCHAR(80) NOT NULL DEFAULT 'NOT AVAILABLE',
        `id` MEDIUMINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`producer_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    INSERT INTO `tpf_jmx_producer_table` (`producer_name`)
        VALUES ('Not Available');

    -- Update version history
    CALL tpf_update_version_history(1,4,1,5);

    SELECT 'Done upgrading to schema version 1.5'; -- Info message

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_5_to_1_6`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_5_to_1_6` ()
tpf_upgrade_zrtmc_schema_1_5_to_1_6:BEGIN

    SELECT 'Upgrading to schema version 1.6'; -- Info message

    SELECT 'Removing TCP IP address from tpf_systems table index'; -- Info Message

    ALTER TABLE `tpf_systems`
        DROP KEY `processor_id`, ADD UNIQUE KEY (`processor_id`, `system_name`, `complex_name`, `cpc_name`);

    -- Update version history
    CALL tpf_update_version_history(1,5,1,6);

    SELECT 'Done upgrading to schema version 1.6'; -- Info message

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_1_6_to_1_7`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_1_6_to_1_7` ()
tpf_upgrade_zrtmc_schema_1_6_to_1_7:BEGIN

    SELECT 'Upgrading to schema version 1.7'; -- Info message

    SELECT 'Creating tpf_dasd_queue_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_dasd_queue_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `dev_address` VARCHAR(4) NOT NULL,
    -- extra column
        `dev_mod` VARCHAR(4) NOT NULL,
    -- metrics
        `dev_queue` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `dev_address`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_dasd_service_time_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_dasd_service_time_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `ssid` VARCHAR(4) NOT NULL COMMENT 'SSID is the DASD subsystem id (MF1SSID)',
    -- extra column
        `lowest_online_sda_on_lss` VARCHAR(4) DEFAULT NULL COMMENT 'The lowest online SDA on the logical subsystem',
    -- metrics
        `average_service_time` INT UNSIGNED DEFAULT NULL COMMENT 'Average DASD service time (microseconds per io) between collection intervals',
        PRIMARY KEY (`time`, `system_id`, `ssid`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_system_blocks_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_system_blocks_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- metrics
        `iob_allocated` INT UNSIGNED DEFAULT NULL,
        `iob_inuse` INT UNSIGNED DEFAULT NULL,
        `iob_inuse_pct` FLOAT DEFAULT NULL,
        `ecb_allocated` INT UNSIGNED DEFAULT NULL,
        `ecb_inuse` INT UNSIGNED DEFAULT NULL,
        `ecb_inuse_pct` FLOAT DEFAULT NULL,
        `swb_allocated` INT UNSIGNED DEFAULT NULL,
        `swb_inuse` INT UNSIGNED DEFAULT NULL,
        `swb_inuse_pct` FLOAT DEFAULT NULL,
        `common_allocated` INT UNSIGNED DEFAULT NULL,
        `common_inuse` INT UNSIGNED DEFAULT NULL,
        `common_inuse_pct` FLOAT DEFAULT NULL,
        `frame_allocated` INT UNSIGNED DEFAULT NULL,
        `frame_inuse` INT UNSIGNED DEFAULT NULL,
        `frame_inuse_pct` FLOAT DEFAULT NULL,
        `frame1mb_allocated` INT UNSIGNED DEFAULT NULL,
        `frame1mb_inuse` INT UNSIGNED DEFAULT NULL,
        `frame1mb_inuse_pct` FLOAT DEFAULT NULL,
        `xwb_allocated` INT UNSIGNED DEFAULT NULL,
        `xwb_inuse` INT UNSIGNED DEFAULT NULL,
        `xwb_inuse_pct` FLOAT DEFAULT NULL,
        `rlog_allocated` INT UNSIGNED DEFAULT NULL,
        `rlog_inuse` INT UNSIGNED DEFAULT NULL,
        `rlog_inuse_pct` FLOAT DEFAULT NULL,
        `active_ecbs` INT UNSIGNED DEFAULT NULL,
        `low_priority_ecbs` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_system_lists_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_system_lists_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- metrics
        `cross_list` INT UNSIGNED DEFAULT NULL,
        `ready_list` INT UNSIGNED DEFAULT NULL,
        `input_list` INT UNSIGNED DEFAULT NULL,
        `deferred_list` INT UNSIGNED DEFAULT NULL,
        `suspend_list` INT UNSIGNED DEFAULT NULL,
        `vct_list` INT UNSIGNED DEFAULT NULL COMMENT 'Virtual file access count',
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_tape_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_tape_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `tape_name` VARCHAR(3) NOT NULL COMMENT 'A 3-character tape name`',
    -- metrics
        `bytes_per_sec` FLOAT DEFAULT NULL,
        `writes_per_sec` FLOAT DEFAULT NULL,
        `queue_length` SMALLINT UNSIGNED DEFAULT NULL,
    -- extra column
        `device_address` VARCHAR(4) DEFAULT NULL COMMENT 'A 3 to 4 digit hexadecimal tape device address',
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `tape_name`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_vfa_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_vfa_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- metrics
        `data_reads_per_sec` FLOAT DEFAULT NULL,
        `finds_per_sec` FLOAT DEFAULT NULL,
        `candidate_files_per_sec` FLOAT DEFAULT NULL,
        `noncandidate_files_per_sec` FLOAT DEFAULT NULL,
        `force_files_per_sec` FLOAT DEFAULT NULL,
        `file_immediate_per_sec` FLOAT DEFAULT NULL,
        `buffer_unavailable_per_sec` FLOAT DEFAULT NULL,
        `SLock_hits_per_sec` FLOAT DEFAULT NULL,
        `XLock_hits_per_sec` FLOAT DEFAULT NULL,
        `sync_misses_per_sec` FLOAT DEFAULT NULL,
        `sync_content_per_sec` FLOAT DEFAULT NULL,
        `cached_rht_hts_per_sec` FLOAT DEFAULT NULL,
        `sync_age_outs_per_sec` FLOAT DEFAULT NULL,
        `dispense_4K_per_sec` FLOAT DEFAULT NULL,
        `rht_content_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_static_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_static_metrics_table` (
        `time` datetime(3) NOT NULL,
        `altered` datetime(3) NOT NULL, -- altered is an extra column
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- extra columns
        `cpu_serial` VARCHAR(8) NOT NULL,
        `cpu_machine_type` VARCHAR(8) DEFAULT NULL,
        `tpf_release` VARCHAR(8) DEFAULT NULL,
        `tpf_prod_update_level` VARCHAR(8) DEFAULT NULL,
        `tpf_complex_name` VARCHAR(8) DEFAULT NULL,
        `tpf_enterprise_name` VARCHAR(8) DEFAULT NULL,
        `cpu_Id` VARCHAR(1) NOT NULL,
    -- metrics
        `subsystem_count` TINYINT UNSIGNED DEFAULT NULL,
        `subsystem_user_count` TINYINT UNSIGNED DEFAULT NULL,
        `usable_istreams` TINYINT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_istreams_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_istreams_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `istream` INT UNSIGNED NOT NULL,
    -- extra columns
        `status` VARCHAR(3) NOT NULL,
    -- metrics
        `cross_list` INT UNSIGNED DEFAULT NULL,
        `ready_list` INT UNSIGNED DEFAULT NULL,
        `input_list` INT UNSIGNED DEFAULT NULL,
        `deferred_list` INT UNSIGNED DEFAULT NULL,
        `suspend_list` INT UNSIGNED DEFAULT NULL,
        `vct_list` INT UNSIGNED DEFAULT NULL,
        `active_ecbs` INT UNSIGNED DEFAULT NULL,
        `routed_entries_per_sec` FLOAT DEFAULT NULL,
        `proc_util` FLOAT DEFAULT NULL,
        `proc_adj_util` FLOAT DEFAULT NULL,
        `pseudo_util` FLOAT DEFAULT NULL,
        `low_priority_util` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `istream`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_subsystem_users_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_subsystem_users_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `subsystem_user_name` VARCHAR(3) NOT NULL,
    -- metrics
        `high_spd_msgs_per_sec` FLOAT DEFAULT NULL,
        `routed_entries_per_sec` FLOAT DEFAULT NULL,
        `created_entries_per_sec` FLOAT DEFAULT NULL,
        `ecb_exits_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `subsystem_user_name`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_subsystems_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_subsystems_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- metrics
        `high_spd_msgs_per_sec` FLOAT DEFAULT NULL,
        `routed_entries_per_sec` FLOAT DEFAULT NULL,
        `created_entries_per_sec` FLOAT DEFAULT NULL,
        `ecb_exits_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_subsystems_utility_switches_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_subsystems_utility_switches_metrics_table` (
        `time` datetime(3) NOT NULL,
        `altered` datetime(3) NOT NULL, -- altered is an extra column
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- extra columns
        `recoup` VARCHAR(3) NOT NULL,
        `selective_file_trace` VARCHAR(3) NOT NULL,
        `RCS_file` VARCHAR(3) NOT NULL,
        `RCS_enable` VARCHAR(3) NOT NULL,
        `RCS_implement` VARCHAR(3) NOT NULL,
        `data_col` VARCHAR(3) NOT NULL,
        `capture` VARCHAR(3) NOT NULL,
        `restore` VARCHAR(3) NOT NULL,
        `SON_copy` VARCHAR(3) NOT NULL,
        `trap` VARCHAR(3) NOT NULL,
        `DB_reorg` VARCHAR(3) NOT NULL,
        `CCP` VARCHAR(3) NOT NULL,
        `link_trace` VARCHAR(3) NOT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_dasd_io_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_dasd_io_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `logical_device_type` VARCHAR(4) NOT NULL,
    -- metrics
        `io_per_sec` FLOAT DEFAULT NULL,
        `writes_per_sec` FLOAT DEFAULT NULL,
        `reads_per_sec` FLOAT DEFAULT NULL,
        `queue_length` SMALLINT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `logical_device_type`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_dasd_pool_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_dasd_pool_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `logical_device_type` VARCHAR(4) NOT NULL,
        `pool_type` VARCHAR(3) NOT NULL,
    -- metrics
        `set_size` INT UNSIGNED DEFAULT NULL,
        `reorder_time` INT UNSIGNED DEFAULT NULL,
        `dispensed_per_sec` FLOAT DEFAULT NULL,
        `returned_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `logical_device_type`, `pool_type`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_service_jam_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_service_jam_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `jam_name` VARCHAR(16) NOT NULL,
    -- metrics
        `internal_operations_per_sec` FLOAT DEFAULT NULL,
        `external_operations_per_sec` FLOAT DEFAULT NULL,
        `total_operations_per_sec` FLOAT DEFAULT NULL,
        `threads_in_use` INT UNSIGNED DEFAULT NULL,
        `threads_in_use_highwater` INT UNSIGNED DEFAULT NULL,
        `threads_config` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `jam_name`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_system_messages_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_system_messages_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- metrics
        `high_spd_msgs_per_sec` FLOAT DEFAULT NULL,
    -- sscp_input_msgs_per_sec commented out because all transactions come in via TCP/IP nowadays
    --  `sscp_input_msgs_per_sec` FLOAT DEFAULT NULL,
        `tcpip_msgs_per_sec` FLOAT DEFAULT NULL,
        `routed_entries_per_sec` FLOAT DEFAULT NULL,
        `created_entries_per_sec` FLOAT DEFAULT NULL,
        `commits_per_sec` FLOAT DEFAULT NULL,
    -- cf_requests_per_sec is commented out because CF is not currently being used by customers.
    --  `cf_requests_per_sec` FLOAT DEFAULT NULL,
        `number_of_istreams` TINYINT UNSIGNED DEFAULT NULL,
        `usable_istreams` TINYINT UNSIGNED DEFAULT NULL COMMENT 'Also added to system_metrics_table',
    -- Note: active_ecbs and low_priority_ecbs are from the SYSTEM_LISTS CDC data type
        `processor_util_pct` FLOAT DEFAULT NULL COMMENT 'Also added to system_metrics_table',
    -- The shutdown1, shutdown2, traces in the SYSTEM_MESSAGES data are being intentionally skipped here.
    -- They're stored in other tables.
    -- `shutdown1` INT UNSIGNED DEFAULT NULL,
    -- `shutdown2` INT UNSIGNED DEFAULT NULL,
    -- `traces` INT UNSIGNED DEFAULT NULL,
        `gp_processor_util_pct` FLOAT DEFAULT NULL,
        `gp_max_cap_for_lpar_util_pct` FLOAT DEFAULT NULL,
        `gp_number_of_engines` INT UNSIGNED DEFAULT NULL,
        `istream_cap` SMALLINT UNSIGNED DEFAULT NULL COMMENT 'Also added to system_metrics_table',
    -- extra columns
        `speed_boost` VARCHAR(3) DEFAULT NULL,
        `istream_boost` VARCHAR(3) DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_mq_queue_manager_table and tpf_mq_summary_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_mq_queue_manager_table` (
        `queue_manager` VARCHAR(48) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`queue_manager`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_mq_summary_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `queue_manager_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mq_queue_manager_table`',
    -- metrics
        `queues` INT UNSIGNED DEFAULT NULL,
        `local_queues` INT UNSIGNED DEFAULT NULL,
        `transmit_queues` INT UNSIGNED DEFAULT NULL,
        `remote_queues` INT UNSIGNED DEFAULT NULL,
        `queue_opens_per_sec` FLOAT DEFAULT NULL,
        `queue_closes_per_sec` FLOAT DEFAULT NULL,
        `queue_sweeps_per_sec` FLOAT DEFAULT NULL,
        `queue_msgs_persistent_per_sec` FLOAT DEFAULT NULL,
        `queue_msg_length_persistent` INT UNSIGNED DEFAULT NULL,
        `queue_msgs_nonpersistent_per_sec` FLOAT DEFAULT NULL,
        `queue_msg_length_nonpersistent` INT UNSIGNED DEFAULT NULL,
        `channels` INT UNSIGNED DEFAULT NULL,
        `receiver_channels` INT UNSIGNED DEFAULT NULL,
        `sender_channels` INT UNSIGNED DEFAULT NULL,
        `server_channels` INT UNSIGNED DEFAULT NULL,
        `channel_msgs_per_sec` FLOAT DEFAULT NULL,
        `channel_bytes_recvd_per_sec` FLOAT DEFAULT NULL,
        `channel_bytes_sent_per_sec` FLOAT DEFAULT NULL,
        `channel_bytes_handled_per_sec` FLOAT DEFAULT NULL,
        `channel_batch_size` INT UNSIGNED DEFAULT NULL,
        `channel_active_instances` INT UNSIGNED DEFAULT NULL,
        `channel_last_batch_size` INT UNSIGNED DEFAULT NULL,
        `channel_instances_highwater` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `queue_manager_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_mq_queue_name_table and tpf_mq_queue_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_mq_queue_name_table` (
        `queue_name` VARCHAR(48) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`queue_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_mq_queue_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `queue_manager_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mq_queue_manager_table`',
        `queue_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mq_queue_name_table`',
        `queue_type` VARCHAR(1) NOT NULL,
    -- metrics
        `opens_per_sec` FLOAT DEFAULT NULL,
        `closes_per_sec` FLOAT DEFAULT NULL,
        `sweeps_per_sec` FLOAT DEFAULT NULL,
        `msgs_persistent_per_sec` FLOAT DEFAULT NULL,
        `msg_length_persistent` INT UNSIGNED DEFAULT NULL,
        `msgs_nonpersistent_per_sec` FLOAT DEFAULT NULL,
        `msg_length_nonpersistent` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `queue_manager_id`, `queue_name_id`,`queue_type`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_mq_channel_name_table and tpf_mq_channel_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_mq_channel_name_table` (
        `channel_name` VARCHAR(20) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`channel_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_mq_channel_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `queue_manager_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mq_queue_manager_table`',
        `channel_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mq_channel_name_table`',
        `channel_type` VARCHAR(1) NOT NULL,
    -- metrics
        `msgs_per_sec` FLOAT DEFAULT NULL,
        `bytes_recvd_per_sec` FLOAT DEFAULT NULL,
        `bytes_sent_per_sec` FLOAT DEFAULT NULL,
        `bytes_handled_per_sec` FLOAT DEFAULT NULL,
        `batch_size` INT UNSIGNED DEFAULT NULL,
        `active_instances` INT UNSIGNED DEFAULT NULL,
        `last_batch_size` INT UNSIGNED DEFAULT NULL,
        `instances_highwater` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `queue_manager_id`, `channel_name_id`,`channel_type`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_tcpip_name_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_tcpip_name_table` (
        `app_name` VARCHAR(255) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`app_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Altering system_tcpip_table'; -- Info Message
    -- For v1.7 system_tcpip_table, we:
    -- 1. MODIFY some columns.
    --    ADD _per_sec columns which replace existing count columns.  This was done for column naming consistency across tables.
    -- 2. UPDATE to calculate and set _per_sec values in all existing non-summarized rows. (summarized columns contain NULL values)
    -- 3. DROP existing count columns replaced by the new _per_sec columns.
    ALTER TABLE system_tcpip_table
        MODIFY COLUMN `app_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_tcpip_name_table`',
        MODIFY COLUMN `port` INT UNSIGNED DEFAULT NULL,
        ADD COLUMN `weight` INT UNSIGNED DEFAULT NULL,
        ADD COLUMN `msgs_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `bytes_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `packets_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `bytes_sent_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `bytes_recvd_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `msgs_sent_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `msgs_recvd_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `packets_sent_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `packets_recvd_per_sec` FLOAT DEFAULT NULL;
    UPDATE system_tcpip_table SET
        msgs_per_sec = msg_cnt/duration,
        bytes_per_sec = tot_bytes/duration,
        bytes_sent_per_sec = bytes_sent/duration,
        bytes_recvd_per_sec = bytes_recvd/duration,
        msgs_sent_per_sec = msg_sent/duration,
        msgs_recvd_per_sec = msg_recvd/duration WHERE duration != NULL;
    ALTER TABLE system_tcpip_table
        DROP COLUMN `msg_cnt`,
        DROP COLUMN `tot_bytes`,
        DROP COLUMN `bytes_sent`,
        DROP COLUMN `bytes_recvd`,
        DROP COLUMN `msg_sent`,
        DROP COLUMN `msg_recvd`;


    SELECT 'Creating tpf_rds_mode_table, tpf_rds_mode_table and tpf_remote_data_store_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_rds_name_table` (
        `rds_name` VARCHAR(30) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`rds_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_rds_mode_table` (
        `rds_mode` VARCHAR(8) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`rds_mode`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    -- Remote Data Store Modes are static so we insert now.
    -- Note: The insert order matches the order of modes in the z/TPF documentation "Remote data store modes" topic.
    --       According to docs, the order here is the typical order the mode is changed in.
    INSERT INTO `tpf_rds_mode_table` (`rds_mode`)
        VALUES ('LOCAL'),
               ('COPY'),
               ('VERIFY'),
               ('REMOTE');

    CREATE TABLE IF NOT EXISTS `tpf_remote_data_store_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `rds_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_rds_name_table`',
        `rds_mode_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_rds_mode_table`',
    -- metrics
        `reader_active_ecbs` INT UNSIGNED DEFAULT NULL,
        `reader_highwater` INT UNSIGNED DEFAULT NULL,
        `reader_max` INT UNSIGNED DEFAULT NULL,
        `writer_active_ecbs` INT UNSIGNED DEFAULT NULL,
        `writer_highwater` INT UNSIGNED DEFAULT NULL,
        `writer_max` INT UNSIGNED DEFAULT NULL,
        `reads_per_sec` FLOAT DEFAULT NULL,
        `creates_per_sec` FLOAT DEFAULT NULL,
        `updates_per_sec` FLOAT DEFAULT NULL,
        `deletes_per_sec` FLOAT DEFAULT NULL,
        `total_ops_per_sec` FLOAT DEFAULT NULL,
        `read_reject_at_max` INT UNSIGNED DEFAULT NULL,
        `timeouts_per_sec` FLOAT DEFAULT NULL,
        `TPF_error` INT UNSIGNED DEFAULT NULL,
        `server_error_mongo` INT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `rds_name_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_chl_util_type_table and tpf_enh_chl_util_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_chl_util_type_table` (
        `type` VARCHAR(7) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`type`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_enh_chl_util_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `channel_id` VARCHAR(2) NOT NULL,
        `active` VARCHAR(1) NOT NULL,
        `type_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_chl_util_type_table`',
        `shared` VARCHAR(1) NOT NULL,
        `channel_group` INT UNSIGNED DEFAULT NULL,
    -- metrics
        `part_util_pct` FLOAT DEFAULT NULL,
        `total_util_pct` FLOAT DEFAULT NULL,
        `bus_util_pct` FLOAT DEFAULT NULL,
        `part_mb_read_per_sec` FLOAT DEFAULT NULL,
        `total_mb_read_per_sec` FLOAT DEFAULT NULL,
        `part_mb_write_per_sec` FLOAT DEFAULT NULL,
        `total_mb_write_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `channel_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_lpar_name_table and tpf_lpar_util_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_lpar_name_table` (
        `lpar_name` VARCHAR(8) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`lpar_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_lpar_util_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `part_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_lpar_name_table`',
        `part_number` INT UNSIGNED DEFAULT NULL,
        `type` VARCHAR(1) NOT NULL,
    -- metrics
        `shared_cpus` INT UNSIGNED DEFAULT NULL,
        `recognized_cpus` INT UNSIGNED DEFAULT NULL,
        `weight` INT UNSIGNED DEFAULT NULL,
        `util` FLOAT DEFAULT NULL,
        `util_vs_cap` FLOAT DEFAULT NULL,
        `machine` FLOAT DEFAULT NULL,
        `entitle` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `part_name_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_mpif_path_name_table, tpf_mpif_device_name_table and tpf_mpif_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_mpif_path_name_table` (
        `path_name` VARCHAR(8) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`path_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_mpif_device_name_table` (
        `device_name` VARCHAR(8) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`device_name`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_mpif_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `path_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mpif_path_name_table`',
        `device_name_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_mpif_device_name_table`',
        `class_name` VARCHAR(1) NOT NULL,
    -- metrics
        `reads_per_sec` FLOAT DEFAULT NULL,
        `writes_per_sec` FLOAT DEFAULT NULL,
        `bytes_sent_per_sec` FLOAT DEFAULT NULL,
        `msgs_sent_per_sec` FLOAT DEFAULT NULL,
        `queue_overruns_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `path_name_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_shutdown1_shutdown_type_table and tpf_shutdown1_reason_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_shutdown1_shutdown_type_table` (
        `shutdown_type` VARCHAR(25) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`shutdown_type`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    -- Shutdown types are static so we insert now.
    INSERT INTO `tpf_shutdown1_shutdown_type_table` (`shutdown_type`)
        VALUES ('Input List'),
               ('Deferred List'),
               ('Time Available Supervisor'),
               ('CREM Macro'),
               ('CRED Macro'),
               ('CREX Macro'),
               ('3270 Local Input');

    CREATE TABLE IF NOT EXISTS `tpf_shutdown1_reason_table` (
        `reason` VARCHAR(150) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`reason`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    -- Shutdown reasons are static so we insert now.
    -- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    -- NOTE: These reason strings must be kept in synch with strings in the create_shutdown1_row function in system_messages_db.py
    --       The shutdown reasons here are taken from the ZACLV command documentation.  With the exception of MXITEMR and MXITEMI
    --       which are "made up" and will be explained in the Shutdown Event documentation pane.
    -- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    INSERT INTO `tpf_shutdown1_reason_table` (`reason`)
        VALUES ('MXECBS'),
               ('MNFRAI'),
               ('MNCMBI'),
               ('MNFR1MBI'),
               ('MNECBI'),
               ('MNSWBI'),
               ('MNIOBI'),
               ('MNXWBI'),
               ('MNRLOGI'),
               ('MXECBD'),
               ('MNXWBM'),
               ('MXITEMR'),
               ('MNXWBD'),
               ('MNXWBX'),
               ('MNFRAT'),
               ('MNECBT'),
               ('MNCMBT'),
               ('MNFR1MBT'),
               ('MNSWBT'),
               ('MNXWBT'),
               ('MNRLOGT'),
               ('MXITEMI');

    SELECT 'Creating tpf_shutdown1_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_shutdown1_metrics_table` (
        `time` datetime(3) NOT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `shutdown_type_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_shutdown1_shutdown_type_table`',
        `reason_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_shutdown1_reason_table`',
    -- metrics
        `shutdown_occurred` TINYINT UNSIGNED DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `shutdown_type_id`, `reason_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_traces_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_traces_metrics_table` (
        `time` datetime(3) NOT NULL,
        `altered` datetime(3) NOT NULL, -- altered is an extra column
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- extra columns
        `block_check_mode` VARCHAR(3) NOT NULL,
        `branch` VARCHAR(3) NOT NULL,
        `branch_relative_target_check_mode` VARCHAR(3) NOT NULL,
        `ECB_heap` VARCHAR(3) NOT NULL,
        `ECB_macro` VARCHAR(3) NOT NULL,
        `enter_back` VARCHAR(3) NOT NULL,
        `extended_C_function` VARCHAR(3) NOT NULL,
        `function_trace` VARCHAR(3) NOT NULL,
        `heap_check_mode` VARCHAR(3) NOT NULL,
        `IO` VARCHAR(3) NOT NULL,
        `IBM_DBUGC_log` VARCHAR(3) NOT NULL,
        `IBM_DBUGC_test` VARCHAR(3) NOT NULL,
        `idle_timer` VARCHAR(3) NOT NULL,
        `IO_CCW` VARCHAR(3) NOT NULL,
        `lock_trace` VARCHAR(3) NOT NULL,
        `reg_this_IPL` VARCHAR(3) NOT NULL,
        `socket` VARCHAR(3) NOT NULL,
        `system_log` VARCHAR(3) NOT NULL,
        `user_DBUGC_log` VARCHAR(3) NOT NULL,
        `user_DBUGC_test` VARCHAR(3) NOT NULL,
        `VFA` VARCHAR(3) NOT NULL,
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_lodic_resource_name_table and tpf_lodic_resources_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_lodic_resource_name_table` (
        `resource_priority_class` VARCHAR(8) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`resource_priority_class`)

    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_lodic_resources_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `class_name_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_lodic_resource_name_table`',
    -- metrics
        `ecbs_using_class` SMALLINT UNSIGNED  DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `class_name_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_lodic_utilization_name_table and tpf_lodic_utilization_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_lodic_utilization_name_table` (
        `utilization_priority_class` VARCHAR(8) NOT NULL,
        `id` TINYINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`utilization_priority_class`)

    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE IF NOT EXISTS `tpf_lodic_utilization_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- attributes
        `class_name_id` TINYINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_lodic_utilization_name_table`',
    -- metrics
        `ecbs_using_class` SMALLINT UNSIGNED  DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `class_name_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_time_slice_ecb_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_time_slice_ecb_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
    -- metrics
        `time_sliceable_ecbs` INT UNSIGNED DEFAULT NULL,
        `time_sliced_ecbs_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_dasd_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_dasd_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- attributes
        `dev_indicator` VARCHAR(2) NOT NULL,
        `symbolic_module` VARCHAR(4) NOT NULL,
        `volume_number` VARCHAR(6) NOT NULL,
        `type` VARCHAR(4) NOT NULL,
        `address` VARCHAR(4) NOT NULL,
    -- Note: dasd_subsystem_id is received from z/TPF as record_cache_subsystem_id
        `dasd_subsystem_id` VARCHAR(4) NOT NULL,
    -- metrics
        `module_queue` INT UNSIGNED DEFAULT NULL,
        `data_reads_per_sec` FLOAT DEFAULT NULL,
        `data_writes_per_sec` FLOAT DEFAULT NULL,
        `accumulated_response_time` BIGINT UNSIGNED DEFAULT NULL,
        `response_time_call_per_sec` FLOAT DEFAULT NULL,
        `sioscs_locks_per_sec` FLOAT DEFAULT NULL,
        `sioscs_unlocks_per_sec` FLOAT DEFAULT NULL,
        `mplf_lock_contention_per_sec` FLOAT DEFAULT NULL,
        `fdctc_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `volume_number`, `address`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_tpfdf_file_name_alias_table'; -- Info Message
    CREATE TABLE IF NOT EXISTS `tpf_tpfdf_file_name_alias_table` (
        `file_name_alias` VARCHAR(16) NOT NULL,
        `id` SMALLINT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`file_name_alias`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Creating tpf_tpfdf_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_tpfdf_metrics_table` (
        `time` datetime(3) NOT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
        `duration` INT UNSIGNED DEFAULT NULL,
        `file_id` VARCHAR(4) NOT NULL,
        `file_id_ebcdic` VARCHAR(2) NOT NULL,
        `file_name_alias_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_tpfdf_file_name_alias_table`',
    -- metrics
        `dbadd_per_sec` FLOAT DEFAULT NULL COMMENT 'ADD        command count 0',
        `dbadr_per_sec` FLOAT DEFAULT NULL COMMENT 'ADR        command count 1',
        `dbckp_per_sec` FLOAT DEFAULT NULL COMMENT 'CHECKPOINT command count 2',
        `dbcls_per_sec` FLOAT DEFAULT NULL COMMENT 'CLOSE      command count 3',
        `dbcpy_per_sec` FLOAT DEFAULT NULL COMMENT 'COPY       command count 4',
        `dbcre_per_sec` FLOAT DEFAULT NULL COMMENT 'CREATE     command count 5',
        `dbdel_per_sec` FLOAT DEFAULT NULL COMMENT 'DELETE     command count 6',
        `dbdix_per_sec` FLOAT DEFAULT NULL COMMENT 'DEINDEX    command count 7',
        `dbdsp_per_sec` FLOAT DEFAULT NULL COMMENT 'DISPLAY    command count 8',
        `dbidx_per_sec` FLOAT DEFAULT NULL COMMENT 'INDEX      command count 9',
        `dbkey_per_sec` FLOAT DEFAULT NULL COMMENT 'KEY        command count 10',
        `dbmod_per_sec` FLOAT DEFAULT NULL COMMENT 'MODIFY     command count 11',
        `dbmrg_per_sec` FLOAT DEFAULT NULL COMMENT 'MERGE      command count 12',
        `dbopn_per_sec` FLOAT DEFAULT NULL COMMENT 'OPEN       command count 13',
        `dbred_per_sec` FLOAT DEFAULT NULL COMMENT 'READ       command count 14',
        `dbrep_per_sec` FLOAT DEFAULT NULL COMMENT 'REPLACE    command count 15',
        `dbret_per_sec` FLOAT DEFAULT NULL COMMENT 'RETAIN     command count 16',
        `dbrst_per_sec` FLOAT DEFAULT NULL COMMENT 'RESTORE    command count 17',
        `dbspa_per_sec` FLOAT DEFAULT NULL COMMENT 'SPACE      command count 18',
        `dbsrt_per_sec` FLOAT DEFAULT NULL COMMENT 'SORT       command count 19',
        `dbtld_per_sec` FLOAT DEFAULT NULL COMMENT 'TAPELOAD   command count 20',
        `dbtlg_per_sec` FLOAT DEFAULT NULL COMMENT 'TAPELOG    command count 21',
        `dbtrd_per_sec` FLOAT DEFAULT NULL COMMENT 'TAPEREAD   command count 22',
        `dbuky_per_sec` FLOAT DEFAULT NULL COMMENT 'UKY        command count 23',
        `dbhold_per_sec` FLOAT DEFAULT NULL COMMENT 'DBHOLD    command count 24',
        `dbuhld_per_sec` FLOAT DEFAULT NULL COMMENT 'DBUHLD    command count 25',
        `filnc_per_sec` FLOAT DEFAULT NULL COMMENT 'FILNC      macro count 26',
        `filec_per_sec` FLOAT DEFAULT NULL COMMENT 'FILEC      macro count 27',
                                                                     -- Reserved for IBM 28
        `cfile_per_sec` FLOAT DEFAULT NULL COMMENT 'FILE CHAIN macro count 29',
        `pfile_per_sec` FLOAT DEFAULT NULL COMMENT 'FILE PRIME macro count 30',
        `twrtc_per_sec` FLOAT DEFAULT NULL COMMENT 'TWRTC      macro count 31',
        `getfc_per_sec` FLOAT DEFAULT NULL COMMENT 'GETFC      macro count 32',
        `relfc_per_sec` FLOAT DEFAULT NULL COMMENT 'RELFC      macro count 33',
        `detac_per_sec` FLOAT DEFAULT NULL COMMENT 'DETAC      macro count 34',
        `attac_per_sec` FLOAT DEFAULT NULL COMMENT 'ATTAC      macro count 35',
        `fsysw_per_sec` FLOAT DEFAULT NULL COMMENT 'FSYS WRITE macro count 36',
        `getcc_per_sec` FLOAT DEFAULT NULL COMMENT 'GETCC      macro count 37',
        `relcc_per_sec` FLOAT DEFAULT NULL COMMENT 'RELCC      macro count 38',
        `cfind_per_sec` FLOAT DEFAULT NULL COMMENT 'FIND CHAIN macro count 39',
        `pfind_per_sec` FLOAT DEFAULT NULL COMMENT 'FIND PRIME macro count 40',
        `btfil_per_sec` FLOAT DEFAULT NULL COMMENT 'B+TREE FILE macro count 41',
        `btfnd_per_sec` FLOAT DEFAULT NULL COMMENT 'B+TREE FIND macro count 42',
        `lrfil_per_sec` FLOAT DEFAULT NULL COMMENT 'LLR FILE   macro count 43',
        `lrfnd_per_sec` FLOAT DEFAULT NULL COMMENT 'LLR FIND   macro count 44',
        `opn_hd_per_sec` FLOAT DEFAULT NULL COMMENT 'OPEN HOLD DETAC 45',
        `opn_nhd_per_sec` FLOAT DEFAULT NULL COMMENT 'OPEN NOHOLD DETAC 46',
        `opn_hnd_per_sec` FLOAT DEFAULT NULL COMMENT 'OPEN HOLD NODETAC 47',
        `opn_nhnd_per_sec` FLOAT DEFAULT NULL COMMENT 'OPEN NOHOLD NODETAC 48',
        `cls_rel_per_sec` FLOAT DEFAULT NULL COMMENT 'CLOSE RELEASE 49',
        `cls_norel_per_sec` FLOAT DEFAULT NULL COMMENT 'CLOSE NORELEASE 50',
        `cls_reuse_per_sec` FLOAT DEFAULT NULL COMMENT 'CLOSE REUSE 51',
        `cls_opn_per_sec` FLOAT DEFAULT NULL COMMENT 'CLOSE OPEN (UBC1) 52',
        `prefp_per_sec` FLOAT DEFAULT NULL COMMENT 'PREFETCH PRIME 53',
        `prefpw_per_sec` FLOAT DEFAULT NULL COMMENT 'PREFETCH PRIME WAITC 54',
        `cache_prtl_iocnt_per_sec` FLOAT DEFAULT NULL COMMENT 'Cache partial IO cnt 55',
        `cache_prtl_ioreq_per_sec` FLOAT DEFAULT NULL COMMENT 'Cache partial IO req 56',
        `cache_read_hit_per_sec` FLOAT DEFAULT NULL COMMENT 'Cache read hit 57',
        `cache_read_miss_per_sec` FLOAT DEFAULT NULL COMMENT 'Cache read miss 58',
        `blk_read_depth` FLOAT DEFAULT NULL COMMENT 'Block read depth 59',
        `rsf_create_per_sec` FLOAT DEFAULT NULL COMMENT 'Remote subfile create 60',
        `rsf_read_per_sec` FLOAT DEFAULT NULL COMMENT 'Remote subfile read 61',
        `rsf_update_per_sec` FLOAT DEFAULT NULL COMMENT 'Remote subfile update 62',
        `rsf_delete_per_sec` FLOAT DEFAULT NULL COMMENT 'Remote subfile delete 63',
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `file_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

   -- Add normalization tables for the common_deploy_metrics_table
    SELECT 'Creating tpf_trad_db_encryption_metrics_table'; -- Info Message

    CREATE TABLE IF NOT EXISTS `tpf_trad_db_encryption_metrics_table` (
        `time` datetime(3) NOT NULL,
        `duration` INT UNSIGNED DEFAULT NULL,
        `system_id` SMALLINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'References `id` in `tpf_systems`',
        `subsystem_id` SMALLINT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_subsystems`',
    -- metrics
        `encryptions_per_sec` FLOAT DEFAULT NULL,
        `encryptions_compressions_per_sec` FLOAT DEFAULT NULL,
        `decryptions_per_sec` FLOAT DEFAULT NULL,
        `decryption_expansions_per_sec` FLOAT DEFAULT NULL,
        `encryption_errors_per_sec` FLOAT DEFAULT NULL,
        `compression_errors_per_sec` FLOAT DEFAULT NULL,
        PRIMARY KEY (`time`, `system_id`, `subsystem_id`) USING BTREE,
        INDEX `time_index` (`time`) USING BTREE
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;


    -- Add normalization tables for the common_deploy_metrics_table
    CREATE TABLE IF NOT EXISTS `tpf_common_deployment_types` (
        `cd_type` VARCHAR(16) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`cd_type`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;
    CREATE TABLE IF NOT EXISTS `tpf_common_deployment_versions` (
        `cd_version` VARCHAR(16) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`cd_version`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;
    CREATE TABLE IF NOT EXISTS `tpf_common_deployment_keys` (
        `cd_key` VARCHAR(255) NOT NULL,
        `id` INT UNSIGNED AUTO_INCREMENT,
        PRIMARY KEY (`id`),
        UNIQUE KEY (`cd_key`)
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    SELECT 'Altering common_deploy_metrics_table'; -- Info Message
    -- For v1.7 common_deploy_metrics_table, we:
    -- 1. ADD the usage_per_sec column which replaces existing count and rate columns.
    --    This was done for column naming consistency across tables.  And because we decided to not show counts.
    -- 2. UPDATE to copy count_per_second values in all existing non-summarized rows to the new usage_per_sec column.
    --    (summarized columns contain NULL values)
    -- 3. DROP the usage_count column and count_per_second virtual column since we're now using the new usage_per_sec column.
    --    Also DROP the duration column because the count_per_second Virtual column is also dropped.
    --    Also updates to use the new normalization tables.
    ALTER TABLE `common_deploy_metrics_table`
        ADD COLUMN usage_per_sec FLOAT DEFAULT NULL;
    UPDATE common_deploy_metrics_table SET usage_per_sec = count_per_second WHERE duration != NULL;
    ALTER TABLE `common_deploy_metrics_table`
        DROP COLUMN `usage_count`,
        DROP COLUMN `count_per_second`,
        CHANGE COLUMN `cd_key_id` `key_id` INT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_common_deployment_keys`',
        ADD COLUMN type_id INT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_common_deployment_types`',
        ADD COLUMN version_id INT UNSIGNED NOT NULL COMMENT 'References `id` in `tpf_common_deployment_versions`',
        DROP PRIMARY KEY, ADD  PRIMARY KEY (`time`, `system_id`, `subsystem_id`, `key_id`, `type_id`, `version_id`);

    SELECT 'Altering service_app_metrics_table'; -- Info Message
    -- For v1.7 service_app_metrics_table, we:
    -- 1. ADD _per_sec columns which replace existing rate columns.  This was done for column naming consistency across tables.
    -- 2. UPDATE to copy rate values in all existing non-summarized rows to the new _per_sec columns. (summarized columns contain NULL values)
    -- 3. DROP existing count and rate columns replaced by the new _per_sec columns.
    --    Also DROP the duration column because all Virtual columns that require it are also dropped.
    ALTER TABLE `service_app_metrics_table`
        ADD COLUMN `internal_operations_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `external_operations_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `total_operations_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `timeouts_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `iface_errors_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `srvc_errors_per_sec` FLOAT DEFAULT NULL,
        ADD COLUMN `unrec_errors_per_sec` FLOAT DEFAULT NULL;
    UPDATE service_app_metrics_table SET
       internal_operations_per_sec = internal_operation_rate,
       external_operations_per_sec = external_operation_rate,
       total_operations_per_sec = total_operation_rate,
       timeouts_per_sec = timeout_rate,
       iface_errors_per_sec = iface_error_rate,
       srvc_errors_per_sec = srvc_error_rate,
       unrec_errors_per_sec = unrec_error_rate WHERE duration != NULL;
    ALTER TABLE `service_app_metrics_table`
        DROP COLUMN `internal_operation_count`,
        DROP COLUMN `external_operation_count`,
        DROP COLUMN `total_operation_count`,
        DROP COLUMN `timeout_count`,
        DROP COLUMN `iface_error_count`,
        DROP COLUMN `srvc_error_count`,
        DROP COLUMN `unrec_error_count`,
        DROP COLUMN `internal_operation_rate`,
        DROP COLUMN `external_operation_rate`,
        DROP COLUMN `total_operation_rate`,
        DROP COLUMN `timeout_rate`,
        DROP COLUMN `iface_error_rate`,
        DROP COLUMN `srvc_error_rate`,
        DROP COLUMN `unrec_error_rate`;

    SELECT 'Renaming common_deploy_metrics_table, system_tcpip_table and service_app_metrics_table'; -- Info Message

    -- Rename tables such that all CDC data type tables use the naming convention: tpf_<data_type>_metrics_table
    RENAME TABLE common_deploy_metrics_table TO tpf_common_deployment_metrics_table,
        system_tcpip_table TO tpf_tcpip_metrics_table,
        service_app_metrics_table TO tpf_service_app_metrics_table;

    SELECT 'Create a user id with limited authorization/privileges'; -- Info Message
    CREATE USER IF NOT EXISTS tpfreadonlyuser IDENTIFIED BY 'admin_passw0rd';
    GRANT SELECT, CREATE TEMPORARY TABLES, EXECUTE ON *.* TO 'tpfreadonlyuser';
    FLUSH PRIVILEGES;

    -- Update version history
    CALL tpf_update_version_history(1,6,1,7);

    SELECT 'Done upgrading to schema version 1.7'; -- Info message

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema` ()
tpf_upgrade_zrtmc_schema:BEGIN

    DECLARE db_max_available_version VARCHAR(16) DEFAULT '1.7';
    DECLARE db_version VARCHAR(16);
    DECLARE db_upgrade_to_version VARCHAR(16);

    -- If user variable not set, upgrade to the max available version
    SET db_upgrade_to_version = IFNULL(@db_upgrade_to_version, db_max_available_version);

    -- Initial DB, no version
    CALL tpf_get_zrtmc_schema_version(db_version);
    SET db_version = IFNULL(db_version, '0.0');
    IF db_version = '0.0' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_create_zrtmc_schema_1_0();
    END IF;

    -- 1.0 to 1.1
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.0' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_0_to_1_1();
    END IF;

    -- 1.1 to 1.2
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.1' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_1_to_1_2();
    END IF;

    -- 1.2 to 1.3
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.2' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_2_to_1_3();
    END IF;

    -- 1.3 to 1.4
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.3' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_3_to_1_4();
    END IF;

    -- 1.4 to 1.5
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.4' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_4_to_1_5();
    END IF;

    -- 1.5 to 1.6
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.5' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_5_to_1_6();
    END IF;

    -- 1.6 to 1.7
    CALL tpf_get_zrtmc_schema_version(db_version);
    IF db_version = '1.6' AND tpf_compare_version(db_version, db_upgrade_to_version) < 0
    THEN
        CALL tpf_upgrade_zrtmc_schema_1_6_to_1_7();
    END IF;

END//
DELIMITER ;

CALL tpf_upgrade_zrtmc_schema();


DROP PROCEDURE IF EXISTS tpf_get_zrtmc_schema_version;
DROP PROCEDURE IF EXISTS tpf_update_version_history;
DROP FUNCTION IF EXISTS tpf_compare_version;
DROP PROCEDURE IF EXISTS tpf_create_zrtmc_schema_1_0;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_0_to_1_1;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_1_to_1_2;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_2_to_1_3;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_3_to_1_4;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_4_to_1_5;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_5_to_1_6;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_1_6_to_1_7;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema;
