#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   05202020    Update for PJ45657 support for Real-time Runtime      #
#               Metrics Collection.                                   #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

# Validate the input and provide help info
if [ ! $# -ge 1 ]; then
   echo "Missing input parameter!"
   echo "Syntax:"
   echo "  tpf_start_replay_script.sh scenario_name [port] [cdc|nvpc|jvm|udm|all]"
   echo "  port 9093 is the default"
   echo "  all is the default"
   echo "Available directories:"
   cd   "../tpfReplayScript"
   find . -type d -name "scenario_*" | while read DIR_NAME; do
      echo "  "${DIR_NAME:2}
   done
   exit 1
fi

PORT="9093"
if [ $# -ge 2 ]; then
   PORT=$2
fi

TYPE="all"
if [ $# -eq 3 ]; then
   TYPE=$(echo "$3" | tr '[:upper:]' '[:lower:]')  # Convert the argument to lowercase
fi


DATE_FORMAT="+%Y-%m-%d %H:%M:%S"
SED_DATE_REGEX="[0-9]\{4,4\}-[0-9]\{2,2\}-[0-9]\{2,2\} [0-9]\{2,2\}:[0-9]\{2,2\}:[0-9]\{2,2\}\.[0-9]\{3,3\}"


# Define variables for our file names
CDC_FILE_NAME="tpfReplayScript/$1/tpf_sample_cdc.json"
NVPC_FILE_NAME="tpfReplayScript/$1/tpf_sample_nvpc.json"
NVPC_VERT_FILE_NAME="tpfReplayScript/$1/tpf_sample_nvpc_vert.json"
JVM_FILE_NAME="tpfReplayScript/$1/tpf_sample_jvm.json"
UDM_FILE_NAME="tpfReplayScript/$1/tpf_sample_udm.json"

#CDC data must exist for all scenarios but we'll allow NVP data to be missing
if [ ! -f "../$CDC_FILE_NAME" ]; then
   echo "CDC file ../$CDC_FILE_NAME does not exist."
   echo "NVPC, NVPC vertical, and JVM files can be missing but CDC data is required."
   exit 1
elif [ ! -s "../$CDC_FILE_NAME" ]; then
   echo "CDC file ../$CDC_FILE_NAME is empty."
   echo "NVPC, NVPC vertical, and JVM files can be empty but CDC data is required."
   exit 1
fi

# We'll need the hostname for this server in a couple of places.
FULLY_QUALIFIED_HOSTNAME="$(hostname --fqdn):$PORT"

CURRENT=$PWD

# Time to do our docker work
cd ./tpf_replay_script_docker_files

# Build the replay script Docker image
docker build -t tpf_replay_script_img .

cd $CURRENT

TIMESTAMP=`date --utc "$DATE_FORMAT"`
TIMESTAMP_SECONDS=$(date --utc -d "$TIMESTAMP" +%s)

# The replay script jar takes the difference between the timestamp -sub parameter and the timestamp of the first line of the file,
# and shifts each timestamp in the file by that amount. In order to preserve the same relative timestamps across each file
# when running the replay, we need to make sure this difference is the same for each container. Therefore, we adjust
# the -sub parameter by the difference between the timestamp of the first line of subsequent files and the
# timestamp of the first line of the CDC file.

# Extract the "collection_start_utc_time" from the first line of the file, convert to unix time seconds
CDC_SECONDS=$(date --utc -d "$(head -n 1 ../"$CDC_FILE_NAME" | sed "s/.*\"collection_start_utc_time\"\s*:\s*\"\($SED_DATE_REGEX\)\".*/\1/")" +%s)

# Run the Replay Script

if [[ ("$TYPE" == "cdc" || "$TYPE" == "all") && -f "../$CDC_FILE_NAME" ]]; then
	docker run -d -t --name tpf-replay-script-cdc --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar replay $FULLY_QUALIFIED_HOSTNAME TPF_CDC_DATA /home/tpf_files/$CDC_FILE_NAME -sub $TIMESTAMP -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "nvpc" || "$TYPE" == "all") && -f "../$NVPC_FILE_NAME" ]]; then
   NVPC_SECONDS=$(date --utc -d "$(head -n 1 ../"$NVPC_FILE_NAME" | sed "s/.*\"collection_start_utc_time\"\s*:\s*\"\($SED_DATE_REGEX\)\".*/\1/")" +%s)

   # Adjust the timestamp and convert back to proper date format
   NVPC_TIMESTAMP=$(date --utc -d @"$((NVPC_SECONDS - CDC_SECONDS + TIMESTAMP_SECONDS))" "$DATE_FORMAT")
   docker run -d -t --name tpf-replay-script-nvpc --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar replay $FULLY_QUALIFIED_HOSTNAME TPF_NVPC_DATA /home/tpf_files/$NVPC_FILE_NAME -sub $NVPC_TIMESTAMP -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "nvpc_vert" || "$TYPE" == "all") && -f "../$NVPC_VERT_FILE_NAME" ]]; then
   NVPC_VERT_SECONDS=$(date --utc -d "$(head -n 1 ../"$NVPC_VERT_FILE_NAME" | sed "s/.*\"collection_start_utc_time\"\s*:\s*\"\($SED_DATE_REGEX\)\".*/\1/")" +%s)
   NVPC_VERT_TIMESTAMP=$(date --utc -d @"$((NVPC_VERT_SECONDS - CDC_SECONDS + TIMESTAMP_SECONDS))" "$DATE_FORMAT")
   docker run -d -t --name tpf-replay-script-nvpc-vert --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar replay $FULLY_QUALIFIED_HOSTNAME TPF_NVPC_VERT_DATA /home/tpf_files/$NVPC_VERT_FILE_NAME -sub $NVPC_VERT_TIMESTAMP -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "jvm" || "$TYPE" == "all") && -f "../$JVM_FILE_NAME" ]]; then
   JVM_SECONDS=$(date --utc -d "$(head -n 1 ../"$JVM_FILE_NAME" | sed "s/.*\"collection_start_utc_time\"\s*:\s*\"\($SED_DATE_REGEX\)\".*/\1/")" +%s)
   JVM_TIMESTAMP=$(date --utc -d @"$((JVM_SECONDS - CDC_SECONDS + TIMESTAMP_SECONDS))" "$DATE_FORMAT")
   docker run -d -t --name tpf-replay-script-jvm --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar replay $FULLY_QUALIFIED_HOSTNAME TPF_JVM_DATA /home/tpf_files/$JVM_FILE_NAME -sub $JVM_TIMESTAMP -kafka_host /home/tpf_files/kafka_hosts.yml
fi

if [[ ("$TYPE" == "udm" || "$TYPE" == "all") && -f "../$UDM_FILE_NAME" ]]; then
   UDM_SECONDS=$(date --utc -d "$(head -n 1 ../"$UDM_FILE_NAME" | sed "s/.*\"collection_start_utc_time\"\s*:\s*\"\($SED_DATE_REGEX\)\".*/\1/")" +%s)
   UDM_TIMESTAMP=$(date --utc -d @"$((UDM_SECONDS - CDC_SECONDS + TIMESTAMP_SECONDS))" "$DATE_FORMAT")
   docker run -d -t --name tpf-replay-script-udm --rm --volume=$PWD/../tpfReplayScript:/home/tpf_files/tpfReplayScript tpf_replay_script_img java -jar /home/tpf_files/tpfReplayScript/tpf_ReplayScripts.jar replay $FULLY_QUALIFIED_HOSTNAME TPF_UDM_DATA /home/tpf_files/$UDM_FILE_NAME -sub $UDM_TIMESTAMP -kafka_host /home/tpf_files/kafka_hosts.yml
fi