#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from pymysql.cursors import Cursor

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.common_deploy import CommonDeployFiles, CommonDeployEntry
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_data_manager import id_manager
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

import logging
import copy

logger = logging.getLogger('tpf_zrtmc_analyzer')

class CommonDeployHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_common_deployment_metrics_table'

    db_field_names = ['time', 'system_id', 'subsystem_id', 'key_id', 'duration', 'type_id', 'version_id', 'usage_per_sec']
    db_key_fields = {'time', 'system_id', 'subsystem_id', 'key_id', 'type_id', 'version_id'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: CommonDeployFiles, parent: CdcData = None):

        #logger.info(f'CommonDeployHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for cd_entry in data.cdc_record_array:

            duration = cd_entry.cdc_record_header.collection_interval_length
            timestamp = cd_entry.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = "common_deploy" + str(system_id) + cd_entry.ss_name + cd_entry.key + cd_entry.type + cd_entry.version

            # Create an array of metrics to test for 0
            metrics = [cd_entry.usage_count]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                      system_id,
                      self.get_subsystem_id(db_cursor, cd_entry),
                      self.get_common_deployment_key_id(db_cursor, cd_entry.key),
                      duration,
                      self.get_common_deployment_type_id(db_cursor, cd_entry.type),
                      self.get_common_deployment_version_id(db_cursor, cd_entry.version),
                      self.calculate_rate(cd_entry.usage_count, duration)]

               #logger.info(f'CommonDeployHandler row: {row}')

               rows.append(row)
               
        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, data: CommonDeployEntry):
        return id_manager.get_tpf_subsystem_id(data.ss_name, db_cursor)

    def get_common_deployment_key_id(self, db_cursor: Cursor, key):
        return id_manager.get_common_deployment_key_id(key, db_cursor)

    def get_common_deployment_type_id(self, db_cursor: Cursor, type):
        return id_manager.get_common_deployment_type_id(type, db_cursor)

    def get_common_deployment_version_id(self, db_cursor: Cursor, version):
        return id_manager.get_common_deployment_version_id(version, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)