#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes DASDMetric objects to a relational database.      #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.dasd_metrics import DASDMetrics, DASDMetric
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class DASDMetricsHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_dasd_metrics_table'

    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'dev_indicator', 'symbolic_module',
       'volume_number', 'type', 'address','dasd_subsystem_id', 'module_queue',
       'data_reads_per_sec', 'data_writes_per_sec', 'accumulated_response_time', 'response_time_call_per_sec',
       'sioscs_locks_per_sec', 'sioscs_unlocks_per_sec', 'mplf_lock_contention_per_sec', 'fdctc_per_sec']
    db_key_fields = {'time', 'system_id', 'subsystem_id', 'volume_number', 'address'}

    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: DASDMetrics, parent: CdcData = None):

        #logger.warning(f'DASDMetricsHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for dasd_metric in data.cdc_record_array:

            duration = dasd_metric.cdc_record_header.collection_interval_length
            timestamp = dasd_metric.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor),

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + dasd_metric.devSubSystem + dasd_metric.devVolumeNumber + dasd_metric.devDeviceAddress

            # Create an array of metrics to test for 0
            metrics = [dasd_metric.devModuleQueue,
               dasd_metric.devDataReads,
               dasd_metric.devDataWrites,
               dasd_metric.devAccumulatedResponseTime,
               dasd_metric.devResponseTimeCallCounter,
               dasd_metric.devSioscsLocks,
               dasd_metric.devSioscsUnLocks,
               dasd_metric.devMplfLockContention,
               dasd_metric.devFdctcCount]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                      duration,
                      system_id,
                      self.get_subsystem_id(db_cursor, dasd_metric.devSubSystem),
                      dasd_metric.devIndicator,
                      dasd_metric.devSymbolicModule,
                      dasd_metric.devVolumeNumber,
                      dasd_metric.devDeviceType,
                      dasd_metric.devDeviceAddress,
                      dasd_metric.devRecordCacheSubsystemId,
                      dasd_metric.devModuleQueue,
                      self.calculate_rate(dasd_metric.devDataReads, duration),
                      self.calculate_rate(dasd_metric.devDataWrites, duration),
                      dasd_metric.devAccumulatedResponseTime,
                      self.calculate_rate(dasd_metric.devResponseTimeCallCounter, duration),
                      self.calculate_rate(dasd_metric.devSioscsLocks, duration),
                      self.calculate_rate(dasd_metric.devSioscsUnLocks, duration),
                      self.calculate_rate(dasd_metric.devMplfLockContention, duration),
                      self.calculate_rate(dasd_metric.devFdctcCount, duration)]

               #logger.info(f'DASDMetricsHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)