#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes DASD POOL objects to a relational database.       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.dasd_pools import DASDPools, DASDPool, DASDPoolDevices, DASDPoolsData
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class DASDPoolsHandler(DbDataHandler, CdcDbHelper):
    io_table_name = 'tpf_dasd_io_metrics_table'
    io_db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'logical_device_type',
       'io_per_sec', 'writes_per_sec', 'reads_per_sec', 'queue_length']
    io_db_key_fields = {'system_id', 'subsystem_id', 'logical_device_type'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    io_pre_sql_insert = create_pre_sql_insert(io_table_name, io_db_field_names, io_db_key_fields)

    pool_table_name = 'tpf_dasd_pool_metrics_table'
    pool_db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'logical_device_type',
       'pool_type', 'set_size', 'reorder_time', 'dispensed_per_sec', 'returned_per_sec']
    pool_db_key_fields = {'system_id', 'subsystem_id', 'logical_device_type', 'pool_type'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pool_pre_sql_insert = create_pre_sql_insert(pool_table_name, pool_db_field_names, pool_db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: DASDPools, parent: CdcData = None):

        #logger.info(f'DASDPoolsHandler io_pre_sql_insert: {self.io_pre_sql_insert}')
        #logger.info(f'DASDPoolsHandler pool_pre_sql_insert: {self.pool_pre_sql_insert}')

        insert_manys: List[InsertMany] = []
        io_rows = []
        pool_rows = []
        loopCounter = 0

        for dasdpool in data.cdc_record_array:
            #logger.info(f'for dasdpool for dasdpool.subsystem_name {dasdpool.subsystem_name}')

            for dasddev in dasdpool.devices_array:

               duration = dasdpool.cdc_record_header.collection_interval_length
               timestamp = dasdpool.cdc_record_header.timestamp
               system_id = self.get_tpf_system_id(parent.header, db_cursor)

               # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
               if loopCounter == 0:
                  self.heartbeatSeconds += duration
               loopCounter += 1

                   # order matters here! it must match with db_field_names
               io_row = [dasdpool.cdc_record_header.timestamp,
                  duration,
                  system_id,
                  self.get_subsystem_id(db_cursor, dasdpool.subsystem_name),
                  dasddev.logical_device_type,
                  self.calculate_rate(dasddev.dev_writes + dasddev.dev_reads, duration),
                  self.calculate_rate(dasddev.dev_writes, duration),
                  self.calculate_rate(dasddev.dev_reads, duration),
                  dasddev.dev_queue]

               #logger.info(f'DASDPoolsHandler deviceName: {dasddev.logical_device_type}  io_row: {io_row}')

               io_rows.append(io_row)

               for dasdpooldata in dasddev.pools_array:

                  # Create a unique string from the db_key_fields for the row of data
                  heartbeatKey = str(system_id) + dasdpool.subsystem_name + dasddev.logical_device_type + dasdpooldata.pool_type

                  # Create an array of metrics to test for 0
                  # Only metrics needed to determine dasd pool inactivity are included in the array.
                  metrics = [dasdpooldata.dispensed,
                      dasdpooldata.returned]

                  if self.checkRowInsert(heartbeatKey, metrics) == True:

                      # order matters here! it must match with db_field_names
                      # reorder_time is divided by 1000 to convert milliseconds to seconds for display
                      pool_row = [dasdpool.cdc_record_header.timestamp,
                         duration,
                         system_id,
                         self.get_subsystem_id(db_cursor, dasdpool.subsystem_name),
                         dasddev.logical_device_type,
                         dasdpooldata.pool_type,
                         dasdpooldata.set_size,
                         dasdpooldata.reorder_time / 1000,
                         self.calculate_rate(dasdpooldata.dispensed, duration),
                         self.calculate_rate(dasdpooldata.returned, duration)]

                      #logger.info(f'DASDPoolsHandler poolType: {dasdpooldata.pool_type}  pool_row: {pool_row}')

                      pool_rows.append(pool_row)

        insert_manys.append(InsertMany(self.io_pre_sql_insert, rows=io_rows))
        insert_manys.append(InsertMany(self.pool_pre_sql_insert, rows=pool_rows))

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.io_table_name, cls.pool_table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)

