#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes ENH_CHL_UTILIZATION objects to a relational       #
#  database.                                                          #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.enh_chl_util import EnhChlUtils, EnhChlUtil, ChannelDescriptor
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class EnhChlUtilHandler(DbDataHandler, CdcDbHelper):

    table_name = 'tpf_enh_chl_util_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'channel_id', 'active', 'type_id', 'shared', 'channel_group',
       'part_util_pct', 'total_util_pct', 'bus_util_pct',
       'part_mb_read_per_sec', 'total_mb_read_per_sec', 'part_mb_write_per_sec', 'total_mb_write_per_sec']
    db_key_fields = {'time', 'system_id', 'channel_id'}
    # Generate an SQL INSERT statement for tpf_enh_chl_util_metrics_table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: EnhChlUtils, parent: CdcData = None):

        #logger.info(f'EnhChlUtilHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for util in data.cdc_record_array:

            duration = util.cdc_record_header.collection_interval_length
            timestamp = util.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + util.channel_descriptor.channel_id

            # Create an array of metrics to test for 0
            metrics = [util.part_util,
               util.total_util,
               util.bus_util,
               util.part_mb_read,
               util.total_mb_read,
               util.part_mb_write,
               util.total_mb_write]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               row = self.create_channel_row(db_cursor, system_id, util)
               #logger.info(f'EnhChlUtilHandler row: {row}')
               if row:
                   rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_chl_type_id(self, db_cursor: Cursor, type):
        return id_manager.get_chl_type_id(type, db_cursor)

    def create_channel_row(self, db_cursor, system_id, util):
        """
        Create and return a row for the INSERT of an enhanced channel.
        The if:elif:else statements here are taken from the logic in the Tivoli Monitor Agent for z/TPF
        which came before the RTMC Starter Kit.
        Order of metrics added to the row matters. It must match with db_field_names.
        """
        row = []

        duration = util.cdc_record_header.collection_interval_length

        if util.channel_descriptor.active == "N" or util.channel_descriptor.valid == 0:
            # The channel is not active or is not valid.  Set all metric data to 0.
            row = [util.cdc_record_header.timestamp,
               util.cdc_record_header.collection_interval_length,
               system_id,
               util.channel_descriptor.channel_id,
               util.channel_descriptor.active,
               self.get_chl_type_id(db_cursor, util.channel_descriptor.type),
               util.channel_descriptor.shared,
               util.channel_descriptor.channel_group,
               0,
               0,
               0,
               0,
               0,
               0,
               0]
        elif util.channel_descriptor.channel_group < 2:
            # The channel_group is less than 2.  Set metric data other than part_util and total_util to 0.
            # The online collector multiplies percents by 100 so we divide by 100 here.
            row = [util.cdc_record_header.timestamp,
               util.cdc_record_header.collection_interval_length,
               system_id,
               util.channel_descriptor.channel_id,
               util.channel_descriptor.active,
               self.get_chl_type_id(db_cursor, util.channel_descriptor.type),
               util.channel_descriptor.shared,
               util.channel_descriptor.channel_group,
               util.part_util / 100,
               util.total_util / 100,
               0,
               0,
               0,
               0,
               0]
        else:
            # The online collector multiplies percents by 100 so we divide by 100 here.
            row = [util.cdc_record_header.timestamp,
               util.cdc_record_header.collection_interval_length,
               system_id,
               util.channel_descriptor.channel_id,
               util.channel_descriptor.active,
               self.get_chl_type_id(db_cursor, util.channel_descriptor.type),
               util.channel_descriptor.shared,
               util.channel_descriptor.channel_group,
               util.part_util / 100,
               util.total_util / 100,
               util.bus_util / 100,
               self.calculate_rate(util.part_mb_read, duration),
               self.calculate_rate(util.total_mb_read, duration),
               self.calculate_rate(util.part_mb_write, duration),
               self.calculate_rate(util.total_mb_write, duration)]

        return row

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)