#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
#   20240725    PJ48071 - workload cpu for dedicated istreams issue   #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from collections import namedtuple
from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.istreams import IStreams
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

CpuValues = namedtuple('CpuValues', ['workload_cpu', 'lp_cpu', 'sys_pseudo_cpu', 'fenced_pseudo_cpu', 'num_istreams',
                                     'fenced_istreams'])


class IStreamsHandler(DbDataHandler, CdcDbHelper):
    table_name = 'system_metrics_table'
    db_field_names = ['time', 'workload_cpu', 'lp_cpu', 'sys_pseudo_cpu', 'fenced_pseudo_cpu']
    db_key_fields = {'time'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)
    db_field_names_v2 = db_field_names + ['system_id']
    db_key_fields_v2 = db_key_fields.union({'system_id'})
    pre_sql_insert_v2 = create_pre_sql_insert(table_name, db_field_names_v2, db_key_fields_v2)

    # Local variables to insert the complete set of istream metrics into the tpf_system_blocks_metrics_table
    is_table_name = 'tpf_istreams_metrics_table'
    is_db_field_names = ['time', 'duration', 'system_id', 'istream', 'status',
                         'cross_list', 'ready_list', 'input_list', 'deferred_list', 'suspend_list', 'vct_list',
                         'active_ecbs', 'routed_entries_per_sec',
                         'proc_util', 'proc_adj_util', 'pseudo_util', 'low_priority_util']
    is_db_key_fields = {'time', 'system_id', 'istream'}
    is_pre_sql_insert = create_pre_sql_insert(is_table_name, is_db_field_names, is_db_key_fields)

    def calculate_cpu_values(self, istreams: IStreams, parent: CdcData):
        """
        Calculates the workload CPU and the low-priority CPU.

        :param istreams: an IStreams object
        :return: CpuValues namedtuple
        """

        workload_cpu = 0
        lp_cpu = 0
        sys_pseudo_cpu = 0
        num_istreams = 0
        found_cap = False

        fenced_pseudo_cpu = 0
        fenced_istreams = 0

        # CPU metrics for inuse/cap istreams (use all istreams up to max(cap, inuse) inclusive)
        for istream in istreams.cdc_record_array:
            # istream_status could be None for certain versions of tpfrtmc
            # istream_status should be a 3 byte string
            if istream.istream_status is None or len(istream.istream_status) == 0:
                return CpuValues(0, 0, 0, 0, 0, 0)

            if found_cap and 'U' not in istream.istream_status:
                break

            num_istreams += 1

            # On dedicated istreams and under VM, LP and PSEUDO include IO interrupt handling while ADJ does not.
            if parent.header.environment == "PR/SM-SH":
               workload_cpu += istream.processor_adj_utilization_percent_istream - istream.low_priority_utilization_istream
            else: # this covers PR/SM-D and VM configurations
               workload_cpu += istream.pseudo_utilization_istream - istream.low_priority_utilization_istream

            lp_cpu += istream.low_priority_utilization_istream
            sys_pseudo_cpu += istream.pseudo_utilization_istream

            if 'C' in istream.istream_status:
                found_cap = True

        # CPU metrics for fenced istreams
        for fenced_is in istreams.cdc_record_array[num_istreams:]:
            if fenced_is.istream_status is None or len(fenced_is.istream_status) == 0:
                return CpuValues(0, 0, 0, 0, 0, 0)

            if 'F' not in fenced_is.istream_status:
                break

            fenced_istreams += 1
            fenced_pseudo_cpu += fenced_is.pseudo_utilization_istream

        if num_istreams == 0:
            return CpuValues(0, 0, 0, 0, 0, 0)
        else:
            return CpuValues(workload_cpu=round(workload_cpu/num_istreams),
                             lp_cpu=round(lp_cpu/num_istreams),
                             sys_pseudo_cpu=round(sys_pseudo_cpu/num_istreams),
                             fenced_pseudo_cpu=0 if fenced_istreams == 0 else round(fenced_pseudo_cpu/fenced_istreams),
                             num_istreams=num_istreams,
                             fenced_istreams=fenced_istreams)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: IStreams, parent: CdcData = None):

        cpu_values = self.calculate_cpu_values(data,parent)

        insert_manys: List[InsertMany] = []

        if cpu_values.num_istreams == 0:
            return []

        version = 1 if parent is None else parent.header.version_num

        # ---------------------------------------------------------------------
        # First create the InsertMany statement for the system_metric_table
        # ---------------------------------------------------------------------
        # Base version
        v_insert = self.pre_sql_insert

        # Arguments must be in order of db_field_names
        row = [data.cdc_record_header.timestamp,
               cpu_values.workload_cpu,
               cpu_values.lp_cpu,
               cpu_values.sys_pseudo_cpu,
               cpu_values.fenced_pseudo_cpu]

        # Version 2
        if version >= 2:
            v_insert = self.pre_sql_insert_v2
            row += [self.get_tpf_system_id(parent.header, db_cursor)]

        rows = [row]
        insert_manys.append(InsertMany(v_insert, rows=rows))

        # ---------------------------------------------------------------------
        # Second create the InsertMany statement for the tpf_istreams_metrics_table
        # ---------------------------------------------------------------------
        is_rows = []
        loopCounter = 0

        for istream in data.cdc_record_array:

            duration = istream.cdc_record_header.collection_interval_length
            timestamp = istream.cdc_record_header.timestamp
            sysid = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(sysid) + str(istream.istream)

            # Create an array of metrics to test for 0
            metrics = [istream.cross_list_istream,
               istream.ready_list_istream,
               istream.input_list_istream,
               istream.deferred_list_istream,
               istream.suspend_list_istream,
               istream.virtual_file_access_count_list_istream,
               istream.active_event_control_block_istream,
               istream.routed_entries_istream,
               istream.processor_utility_percent_istream,
               istream.processor_adj_utilization_percent_istream,
               istream.pseudo_utilization_istream,
               istream.low_priority_utilization_istream]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # Arguments must be in order of db_field_names
               is_row = [timestamp,
                  duration,
                  self.get_tpf_system_id(parent.header, db_cursor),
                  istream.istream,
                  istream.istream_status,
                  istream.cross_list_istream,
                  istream.ready_list_istream,
                  istream.input_list_istream,
                  istream.deferred_list_istream,
                  istream.suspend_list_istream,
                  istream.virtual_file_access_count_list_istream,
                  istream.active_event_control_block_istream,
                  self.calculate_rate(istream.routed_entries_istream, duration),
                  self.calculate_percentage(istream.processor_utility_percent_istream),
                  self.calculate_percentage(istream.processor_adj_utilization_percent_istream),
                  self.calculate_percentage(istream.pseudo_utilization_istream),
                  self.calculate_percentage(istream.low_priority_utilization_istream)]

               #logger.info(f'IStreamsHandler row: {is_row}')

               is_rows.append(is_row)

        insert_manys.append(InsertMany(self.is_pre_sql_insert, rows=is_rows))

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name, cls.is_table_name]

    def calculate_percentage(self, count):
       return count / 10

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)

