#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file generates SQL INSERT statements to insert into the       #
#  tpf_lodic_resources_metrics_table and                              #
#  tpf_time_slice_ecb_metrics_table                                   #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_cdc.cdc_record_types.lodic_resource import LodicResources
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LodicResourcesHandler(DbDataHandler, CdcDbHelper):

    # Local variables to insert into the tpf_lodic_resources_metrics_table
    table_name = 'tpf_lodic_resources_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'class_name_id', 'ecbs_using_class']
    db_key_fields = {'time', 'system_id', 'class_name_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)
    NUM_RESOURCE_PRIORITY_CLASSES = 14

    # Local variables to insert into the tpf_time_slice_ecb_metrics_table
    ts_table_name = 'tpf_time_slice_ecb_metrics_table'
    ts_db_field_names = ['time', 'duration', 'system_id', 'time_sliceable_ecbs', 'time_sliced_ecbs_per_sec']
    ts_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    ts_pre_sql_insert = create_pre_sql_insert(ts_table_name, ts_db_field_names, ts_db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: LodicResources, parent: CdcData = None):

        insert_manys: List[InsertMany] = []

        system_id = self.get_tpf_system_id(parent.header, db_cursor)

        #################################################
        # 1. create INSERT statements to insert into tpf_lodic_resources_metrics_table
        #################################################
        resource_rows = []

        # ---------------------------------------------------------------------
        # Create the InsertMany statement for the tpf_lodic_resources_metrics_table
        # One row per resource priority class.
        # Currently the LODIC data type from TPF has 14 resource priority classes.
        # ---------------------------------------------------------------------
        i = 0
        while i < self.NUM_RESOURCE_PRIORITY_CLASSES:
           resource_row = self.create_resources_row(db_cursor, system_id, data, i)
           #logger.info(f'LodicResourcesHandler resource_row: {resource_row}')
           if resource_row:
              resource_rows.append(resource_row)
           i += 1
        insert_manys.append(InsertMany(self.pre_sql_insert, rows=resource_rows))

        #################################################
        # 2. Create INSERT statements to insert into tpf_time_slice_ecb_metrics_table.
        #    Time slice values are inserted upon every call to this handler.
        #################################################
        time_slice_rows = []

        duration = data.cdc_record_header.collection_interval_length

        # Arguments must be in order of ts_db_field_names
        time_slice_row = [data.cdc_record_header.timestamp,
               duration,
               self.get_tpf_system_id(parent.header, db_cursor),
               data.TimeSliceableEcbs,
               self.calculate_rate(data.TimeSlicedEcbs, duration)]

        #logger.info(f'LodicResourcesHandler time_slice_row: {time_slice_row}')

        time_slice_rows = [time_slice_row]
        insert_manys.append(InsertMany(self.ts_pre_sql_insert, rows=time_slice_rows))


        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name, cls.ts_table_name]

    def create_resources_row(self, db_cursor, system_id, data, type):
        """
        Create and return a row for the INSERT of a resource priority class.
        The 'type' parameter is one of the NUM_RESOURCE_PRIORITY_CLASSES
        Returns an empty row if the 'type' parameter is not known.
        """
        row = []
        if type == 0:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass1Name),
               data.resourcePriorityClass1Ecb]
           return row
        elif type == 1:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass2Name),
               data.resourcePriorityClass2Ecb]
           return row
        elif type == 2:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass3Name),
               data.resourcePriorityClass3Ecb]
           return row
        elif type == 3:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass4Name),
               data.resourcePriorityClass4Ecb]
           return row
        elif type == 4:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass5Name),
               data.resourcePriorityClass5Ecb]
           return row
        elif type == 5:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass6Name),
               data.resourcePriorityClass6Ecb]
           return row
        elif type == 6:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass7Name),
               data.resourcePriorityClass7Ecb]
           return row
        elif type == 7:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass8Name),
               data.resourcePriorityClass8Ecb]
           return row
        elif type == 8:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass9Name),
               data.resourcePriorityClass9Ecb]
           return row
        elif type == 9:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass10Name),
               data.resourcePriorityClass10Ecb]
           return row
        elif type == 10:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass11Name),
               data.resourcePriorityClass11Ecb]
           return row
        elif type == 11:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass12Name),
               data.resourcePriorityClass12Ecb]
           return row
        elif type == 12:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass13Name),
               data.resourcePriorityClass13Ecb]
           return row
        elif type == 13:
           row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               self.get_lodic_resource_name_id(db_cursor, data.resourcePriorityOfEcbsClass14Name),
               data.resourcePriorityClass14Ecb]
           return row

        return row

    def get_lodic_resource_name_id(self, db_cursor: Cursor, name):
        return id_manager.get_lodic_resource_name_id(name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)