#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes LPAR_UTILIZATION objects to a relational database.#
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.lpar_util import LPARUtils, LPARUtil, LPARInfo
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LPARlUtilHandler(DbDataHandler, CdcDbHelper):

    table_name = 'tpf_lpar_util_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'part_name_id', 'part_number', 'type', 'shared_cpus', 'recognized_cpus', 'weight',
       'util', 'util_vs_cap', 'machine', 'entitle']
    db_key_fields = {'time', 'system_id', 'part_name'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: LPARUtils, parent: CdcData = None):

        #logger.info(f'LPARlUtilHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for lpar in data.cdc_record_array:

            duration = lpar.cdc_record_header.collection_interval_length
            timestamp = lpar.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + lpar.lpar_info.part_name

            # Create an array of metrics to test for 0
            # Only metrics needed to determine lpar inactivity are included in the array.
            metrics = [lpar.util,
               lpar.util_vs_cap,
               lpar.machine,
               lpar.entitle]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                  duration,
                  system_id,
                  self.get_lpar_name_id(db_cursor, lpar.lpar_info.part_name),
                  lpar.lpar_info.part_number,
                  lpar.type,
                  lpar.shared_cpus,
                  lpar.lpar_info.recognized_cpus,
                  lpar.lpar_info.weight,
                  self.normalizePercentage(lpar.util),
                  self.normalizePercentage(lpar.util_vs_cap),
                  self.normalizePercentage(lpar.machine),
                  lpar.entitle / 100]  # The divide by 100 is due to a multiply in the online collector.  If a shared partition is using more than its entitlement, this percentage is greater than 100 percent.

               #logger.info(f'LPARlUtilHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_lpar_name_id(self, db_cursor: Cursor, lpar_name):
        return id_manager.get_lpar_name_id(lpar_name, db_cursor)

    def normalizePercentage(self, percentage):
        # This is looking for percentages slightly over 100.00 (e.g. 102 pct) and adjust it to be 100.00 percent.
        # This logic here is copied from the VERY old CDC Java Client.
        # The divide by 100 is due to a multiply in the online collector.
        normalizedPercentage = percentage / 100
        if(normalizedPercentage > 100):
           normalizedPercentage = 100
        return normalizedPercentage