#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes MQQueue objects to a relational database.       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.mq_queues import MQQueues, MQQueue
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class MQQueueHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_mq_queue_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'queue_manager_id', 'queue_name_id', 'queue_type',
     'opens_per_sec', 'closes_per_sec', 'sweeps_per_sec', 'msgs_persistent_per_sec', 'msg_length_persistent',
     'msgs_nonpersistent_per_sec', 'msg_length_nonpersistent']
    db_key_fields = {'system_id', 'subsystem_id', 'queue_manager_id', 'queue_name_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: MQQueues, parent: CdcData = None):

        #logger.info(f'MQQueueHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for queue in data.cdc_record_array:

            # Bring some object info into local variables to be used later in this function.
            duration = queue.cdc_record_header.collection_interval_length
            timestamp = queue.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + queue.subsystem_name + queue.queue_manager + queue.queue_name

            # Create an array of metrics to test for 0
            metrics = [queue.opens,
               queue.opens,
               queue.sweeps,
               queue.msgs_persistent,
               queue.msg_length_persistent,
               queue.msgs_nonpersistent,
               queue.msg_length_nonpersistent]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                   duration,
                   system_id,
                   self.get_subsystem_id(db_cursor, queue.subsystem_name),
                   self.get_mq_queue_manager_id(db_cursor, queue.queue_manager),
                   self.get_mq_queue_name_id(db_cursor, queue.queue_name),
                   queue.queue_type,
                   self.calculate_rate(queue.opens, duration),
                   self.calculate_rate(queue.closes, duration),
                   self.calculate_rate(queue.sweeps, duration),
                   self.calculate_rate(queue.msgs_persistent, duration),
                   queue.msg_length_persistent,
                   self.calculate_rate(queue.msgs_nonpersistent, duration),
                   queue.msg_length_nonpersistent]

               #logger.info(f'MQQueueHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def get_mq_queue_manager_id(self, db_cursor: Cursor, queue_manager):
        return id_manager.get_mq_queue_manager_id(queue_manager, db_cursor)

    def get_mq_queue_name_id(self, db_cursor: Cursor, queue_name):
        return id_manager.get_mq_queue_name_id(queue_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
