#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes MQSummary objects to a relational database.       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.mq_summary import MQSummarys, MQSummary
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class MQSummaryHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_mq_summary_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'queue_manager_id',
     'queues', 'local_queues', 'transmit_queues', 'remote_queues',
     'queue_opens_per_sec', 'queue_closes_per_sec', 'queue_sweeps_per_sec',
     'queue_msgs_persistent_per_sec', 'queue_msg_length_persistent', 'queue_msgs_nonpersistent_per_sec', 'queue_msg_length_nonpersistent',
     'channels', 'receiver_channels', 'sender_channels', 'server_channels',
     'channel_msgs_per_sec', 'channel_bytes_recvd_per_sec', 'channel_bytes_sent_per_sec', 'channel_bytes_handled_per_sec',
     'channel_batch_size', 'channel_active_instances', 'channel_last_batch_size', 'channel_instances_highwater']
    db_key_fields = {'system_id', 'subsystem_id', 'queue_manager_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: MQSummarys, parent: CdcData = None):

        #logger.info(f'MQSummaryHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for summary in data.cdc_record_array:

            duration = summary.cdc_record_header.collection_interval_length
            timestamp = summary.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + summary.subsystem_name + summary.queue_manager

            # Create an array of metrics to test for 0
            # Only metrics needed to determine channel and queue inactivity are included in the array.
            metrics = [summary.queue_opens,
               summary.queue_closes,
               summary.queue_sweeps,
               summary.queue_msgs_persistent,
               summary.queue_msg_length_persistent,
               summary.queue_msgs_nonpersistent,
               summary.queue_msg_length_nonpersistent,
               summary.channel_msgs,
               summary.channel_bytes_recvd,
               summary.channel_bytes_sent,
               summary.channel_bytes_handled]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                   duration,
                   system_id,
                   self.get_subsystem_id(db_cursor, summary.subsystem_name),
                   self.get_mq_queue_manager_id(db_cursor, summary.queue_manager),
                   summary.queues,
                   summary.local_queues,
                   summary.transmit_queues,
                   summary.remote_queues,
                   self.calculate_rate(summary.queue_opens, duration),
                   self.calculate_rate(summary.queue_closes, duration),
                   self.calculate_rate(summary.queue_sweeps, duration),
                   self.calculate_rate(summary.queue_msgs_persistent, duration),
                   summary.queue_msg_length_persistent,
                   self.calculate_rate(summary.queue_msgs_nonpersistent, duration),
                   summary.queue_msg_length_nonpersistent,
                   summary.channels,
                   summary.receiver_channels,
                   summary.sender_channels,
                   summary.server_channels,
                   self.calculate_rate(summary.channel_msgs, duration),
                   self.calculate_rate(summary.channel_bytes_recvd, duration),
                   self.calculate_rate(summary.channel_bytes_sent, duration),
                   self.calculate_rate(summary.channel_bytes_handled, duration),
                   summary.channel_batch_size,
                   summary.channel_active_instances,
                   summary.channel_last_batch_size,
                   summary.channel_instances_highwater]

               #logger.info(f'MQSummaryHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def get_mq_queue_manager_id(self, db_cursor: Cursor, queue_manager):
        return id_manager.get_mq_queue_manager_id(queue_manager, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)