#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes REMOTE_DATA_STORE objects to a relational database#
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.remote_data_store import RemoteDataStores, RemoteDataStore
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class RemoteDataStoreHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_remote_data_store_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id',
     'rds_name_id', 'rds_mode_id',
     'reader_active_ecbs', 'reader_highwater', 'reader_max',
     'writer_active_ecbs', 'writer_highwater', 'writer_max',
     'reads_per_sec', 'creates_per_sec', 'updates_per_sec', 'deletes_per_sec', 'total_ops_per_sec',
     'read_reject_at_max', 'timeouts_per_sec', 'TPF_error', 'server_error_mongo']
    db_key_fields = {'system_id', 'subsystem_id', 'rds_name_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: RemoteDataStores, parent: CdcData = None):

        #logger.info(f'RemoteDataStoreHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for rds in data.cdc_record_array:

            duration = rds.cdc_record_header.collection_interval_length
            timestamp = rds.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + rds.subsystem_name + rds.rds_name

            # Create an array of metrics to test for 0
            # Only metrics needed to determine remote data store inactivity are included in the array.
            metrics = [rds.reader_active_ecbs,
               rds.writer_active_ecbs,
               rds.read_count,
               rds.create_count,
               rds.update_count,
               rds.delete_count,
               rds.total_ops_count]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                   duration,
                   system_id,
                   self.get_subsystem_id(db_cursor, rds.subsystem_name),
                   self.get_rds_name_id(db_cursor, rds.rds_name),
                   self.get_rds_mode_id(db_cursor, rds.rds_mode),
                   rds.reader_active_ecbs,
                   rds.reader_highwater,
                   rds.reader_max,
                   rds.writer_active_ecbs,
                   rds.writer_highwater,
                   rds.writer_max,
                   self.calculate_rate(rds.read_count, duration),
                   self.calculate_rate(rds.create_count, duration),
                   self.calculate_rate(rds.update_count, duration),
                   self.calculate_rate(rds.delete_count, duration),
                   self.calculate_rate(rds.total_ops_count, duration),
                   rds.read_reject_at_max,
                   self.calculate_rate(rds.timeout_count, duration),
                   rds.TPFerror,
                   rds.server_error_mongo]

               #logger.info(f'RemoteDataStoreHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def get_rds_name_id(self, db_cursor: Cursor, rds_name):
        return id_manager.get_rds_name_id(rds_name, db_cursor)

    def get_rds_mode_id(self, db_cursor: Cursor, rds_mode):
        return id_manager.get_rds_mode_id(rds_mode, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)