#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from pymysql.cursors import Cursor

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.service_app import ServiceApps, ServiceApp
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_data_manager import id_manager
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert

import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')

class ServiceAppHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_service_app_metrics_table'
    db_field_names = ['time', 'system_id', 'subsystem_id', 'operation_id', 'duration',
                      'internal_operations_per_sec', 'external_operations_per_sec', 'total_operations_per_sec',
                      'timeouts_per_sec', 'iface_errors_per_sec', 'srvc_errors_per_sec', 'unrec_errors_per_sec']
    db_key_fields = {'time', 'system_id', 'subsystem_id', 'operation_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: ServiceApps, parent: CdcData = None):

        rows = []
        loopCounter = 0

        #logger.info(f'ServiceAppHandler entered')

        for service_app in data.cdc_record_array:

            duration = service_app.cdc_record_header.collection_interval_length
            timestamp = service_app.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + service_app.subsystem_name + service_app.operation

            # Create an array of metrics to test for 0
            metrics = [service_app.internal_operation_count,
               service_app.external_operation_count,
               service_app.total_operation_count,
               service_app.timeout_count,
               service_app.iface_error_count,
               service_app.srvc_error_count,
               service_app.unrec_error_count]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # Arguments must be in order of db_field_names
               # Base version db fields
               row = [timestamp,
                   system_id,
                   self.get_subsystem_id(db_cursor, service_app),
                   self.get_operation_id(db_cursor, service_app.operation, service_app.jam_name),
                   duration,
                   self.calculate_rate(service_app.internal_operation_count, duration),
                   self.calculate_rate(service_app.external_operation_count, duration),
                   self.calculate_rate(service_app.total_operation_count, duration),
                   self.calculate_rate(service_app.timeout_count, duration),
                   self.calculate_rate(service_app.iface_error_count, duration),
                   self.calculate_rate(service_app.srvc_error_count, duration),
                   self.calculate_rate(service_app.unrec_error_count, duration)]

               #logger.info(f'ServiceAppHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, data: ServiceApp):
        return id_manager.get_tpf_subsystem_id(data.subsystem_name, db_cursor)

    def get_operation_id(self, db_cursor: Cursor, operation_name, jam_name):
        return id_manager.get_service_app_operation_id(operation_name, jam_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
