#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file inserts ServiceJam objects to a relational database.     #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from pymysql.cursors import Cursor

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.service_jam import ServiceJams, ServiceJam
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_data_manager import id_manager
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert

import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')

class ServiceJamHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_service_jam_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'jam_name',
                      'internal_operations_per_sec', 'external_operations_per_sec', 'total_operations_per_sec',
                      'threads_in_use', 'threads_in_use_highwater', 'threads_config']
    db_key_fields = {'time', 'system_id', 'subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: ServiceJams, parent: CdcData = None):

        #logger.info(f'ServiceJamHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for service_jam in data.cdc_record_array:

            duration = service_jam.cdc_record_header.collection_interval_length
            timestamp = service_jam.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + service_jam.subsystem_name

            # Create an array of metrics to test for 0
            # Only metrics needed to determine service jam inactivity are included in the array.
            metrics = [service_jam.internal_operation_count,
               service_jam.external_operation_count,
               service_jam.total_operation_count,
               service_jam.threads_in_use]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                   duration,
                   system_id,
                   self.get_subsystem_id(db_cursor, service_jam.subsystem_name),
                   service_jam.jam_name,
                   self.calculate_rate(service_jam.internal_operation_count, duration),
                   self.calculate_rate(service_jam.external_operation_count, duration),
                   self.calculate_rate(service_jam.total_operation_count, duration),
                   service_jam.threads_in_use,
                   service_jam.threads_in_use_highwater,
                   service_jam.threads_config]

               #logger.info(f'ServiceJamHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
