#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file generates SQL INSERT statements to insert into the       #
#  tpf_static_metrics_table.                                          #
#  If new collection data matches data already saved to the database  #
#  for the system id, then an UPDATE is performed to update the time  #
#  column.                                                            #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_utils.db_utils import create_pre_sql_update
from tpf_cdc.cdc_record_types.static import Static
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor

logger = logging.getLogger('tpf_zrtmc_analyzer')

class StaticHandler(DbDataHandler, CdcDbHelper):

    # Local variables to insert the static metric into the tpf_static_metrics_table
    table_name = 'tpf_static_metrics_table'
    db_field_names = ['time', 'altered', 'duration', 'system_id', 'cpu_serial', 'cpu_machine_type',
                      'tpf_release', 'tpf_prod_update_level', 'tpf_complex_name', 'tpf_enterprise_name',
                      'cpu_Id', 'subsystem_count', 'subsystem_user_count', 'usable_istreams']
    db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    # dataDict is a static cache of Static data by system id   e.g. {2:<Static>}
    dataDict = {}
    # insertTimeDict is a static cache of row insert times by system id   e.g. {2:<timestamp>}
    insertTimeDict = {}

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: Static, parent: CdcData = None):

        insert_manys: List[InsertMany] = []

        # ---------------------------------------------------------------------
        # First create the InsertMany statement for the system_metric_table
        # ---------------------------------------------------------------------
        rows = []

        system_id = self.get_tpf_system_id(parent.header, db_cursor)
        timestamp = data.cdc_record_header.timestamp

        # Get the last Static object for this system id from local cache
        cachedStatic = self.dataDict.get(system_id)
        if cachedStatic == None or cachedStatic != data:
            # This Static object is not in the cache -or-
            # it is in the cache but the data is different in this collection,
            # so add it to the cache and create an INSERT statement for it.

            # Update this Static object in the cache for this system id
            self.dataDict.update({system_id:data})

            # Arguments must be in order of db_field_names
            row = [timestamp,
               timestamp,
               data.cdc_record_header.collection_interval_length,
               system_id,
               data.cpu_serial,
               data.cpu_machine_type,
               data.tpf_release,
               data.tpf_prod_update_level,
               data.tpf_complex_name,
               data.tpf_enterprise_name,
               data.cpu_Id,
               data.subsystem_count,
               data.subsystem_user_count,
               data.usable_istreams]

            #logger.info(f'StaticHandler INSERT row: {row}')

            rows = [row]
            insert_manys.append(InsertMany(self.pre_sql_insert, rows=rows))

            # Save the row insert time in local cache for this system id
            self.insertTimeDict.update({system_id:timestamp})
        else:
            # Create an SQL UPDATE statement to update the time in the row with insert time for this system id
            sql = create_pre_sql_update(self.table_name, timestamp, self.insertTimeDict.get(system_id), system_id)
            #logger.info(f'StaticHandler UPDATE sql: {sql}')
            insert_manys.append(InsertMany(sql))

        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]
