#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file inserts SubsystemUser objects to a relational database.  #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.subsystem_users import SubsystemUsers, SubsystemUser
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SubsystemUserHandler(DbDataHandler, CdcDbHelper):

    table_name = 'tpf_subsystem_users_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id', 'subsystem_user_name',
     'high_spd_msgs_per_sec', 'routed_entries_per_sec', 'created_entries_per_sec', 'ecb_exits_per_sec']
    db_key_fields = {'time', 'system_id', 'subsystem_id', 'subsystem_user_name'}
    # Generate an SQL INSERT INTO statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    # This def is called from tpf_cdc_manager.py
    def get_mysql_insert_manys(self, db_cursor: Cursor, data: SubsystemUsers, parent: CdcData = None):

        #logger.info(f'SubsystemUserHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for ssu in data.cdc_record_array:

            duration = ssu.cdc_record_header.collection_interval_length
            timestamp = ssu.cdc_record_header.timestamp
            sysid = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(sysid) + ssu.subsystem_name + ssu.subsystem_user_name

            # Create an array of metrics to test for 0
            metrics = [ssu.high_spd_msgs,
               ssu.routed_entries,
               ssu.created_entries,
               ssu.ecb_exits]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # Arguments must be in order of db_field_names
               row = [timestamp,
                   duration,
                   self.get_tpf_system_id(parent.header, db_cursor),
                   self.get_subsystem_id(db_cursor, ssu.subsystem_name),
                   ssu.subsystem_user_name,
                   self.calculate_rate(ssu.high_spd_msgs, duration),
                   self.calculate_rate(ssu.routed_entries, duration),
                   self.calculate_rate(ssu.created_entries, duration),
                   self.calculate_rate(ssu.ecb_exits, duration)]

               #logger.info(f'SubsystemUserHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
