#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file inserts Subsystem object data to a relational database.  #
#  For tpf_subsystems_utility_switches_metrics_table, if new          #
#  collection data matches data already saved to the database for the #
#  system id, then an UPDATE is performed to update the time column.  #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_db_handlers.subsystems_utility_switches import SubsystemUtilitySwitches
from tpf_cdc.cdc_record_types.subsystems import Subsystems, Subsystem
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_utils.db_utils import create_pre_sql_update
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SubsystemHandler(DbDataHandler, CdcDbHelper):

    ss_table_name = 'tpf_subsystems_metrics_table'
    ss_db_field_names = ['time', 'duration', 'system_id', 'subsystem_id',
     'high_spd_msgs_per_sec', 'routed_entries_per_sec', 'created_entries_per_sec', 'ecb_exits_per_sec']
    ss_db_key_fields = {'time', 'system_id', 'subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    ss_pre_sql_insert = create_pre_sql_insert(ss_table_name, ss_db_field_names, ss_db_key_fields)

    util_sw_table_name = 'tpf_subsystems_utility_switches_metrics_table'
    util_sw_db_field_names = ['time','altered','system_id','subsystem_id','recoup','selective_file_trace','RCS_file','RCS_enable','RCS_implement',
       'data_col','capture','restore','SON_copy','trap','DB_reorg','CCP','link_trace']
    util_sw_db_key_fields = {'time','system_id','subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    util_sw_pre_sql_insert = create_pre_sql_insert(util_sw_table_name, util_sw_db_field_names, util_sw_db_key_fields)

    RECOUP_ON               = 0x80000000
    SELECTIVE_FILE_TRACE_ON = 0x40000000
    RCS_FILE_ON	            = 0x20000000
    RCS_ENABLE_ON           = 0x10000000
    RCS_IMPLEMENT_ON        = 0x08000000
    DATA_COLLECTION_ON      = 0x04000000
    SWITCH_BIT_06           = 0x02000000
    SWITCH_BIT_07           = 0x01000000
    SWITCH_BIT_08           = 0x00800000
    SWITCH_BIT_09           = 0x00400000
    SWITCH_BIT_10           = 0x00200000
    SWITCH_BIT_11           = 0x00100000
    # REAL_TIME_TRACE_ON    = 0x00080000 # Real-time trace was on TPF 4.1.  It's not in z/TPF.
    CAPTURE_ON              = 0x00040000
    RESTORE_ON              = 0x00020000
    SWITCH_BIT_15           = 0x00010000
    SWITCH_BIT_16           = 0x00008000
    SWITCH_BIT_17           = 0x00004000
    SWITCH_BIT_18           = 0x00002000
    SWITCH_BIT_19           = 0x00001000
    SON_COPY_ON             = 0x00000800
    # DUMP_3705_ON          = 0x00000400 # This was for a 3705 Communications Controller long ago obsoleted.
    SWITCH_BIT_22           = 0x00000200
    SWITCH_BIT_23           = 0x00000100
    TRAP_ON	                = 0x00000080
    DB_REORG_ON	            = 0x00000020
    SWITCH_BIT_25           = 0x00000040
    SWITCH_BIT_27           = 0x00000010
    SWITCH_BIT_28           = 0x00000008
    SWITCH_BIT_29           = 0x00000004
    CCP_TRACE_ON            = 0x00000002
    LINK_TRACE_ON           = 0x00000001

    # A static cache of system_id:{Dict}
    # where Dict is a Dictionary containing a SubsystemUtilitySwitches object by subsystem
    # and SubsystemUtilitySwitches is an object that contains the utility switches and the time new switches were inserted into a row
    systemDict = {}

    # This def is called from tpf_cdc_manager.py
    def get_mysql_insert_manys(self, db_cursor: Cursor, data: Subsystems, parent: CdcData = None):

        #logger.info(f'SubsystemHandler ss_pre_sql_insert: {self.ss_pre_sql_insert}')
        #logger.info(f'SubsystemHandler util_sw_pre_sql_insert: {self.util_sw_pre_sql_insert}')

        insert_manys: List[InsertMany] = []

        ss_rows = []
        util_sw_rows = []
        loopCounter = 0

        for ss in data.cdc_record_array:
            row_inserted = False

            system_id = self.get_tpf_system_id(parent.header, db_cursor)
            subsystem_id = self.get_subsystem_id(db_cursor, ss.subsystem_name)

            duration = ss.cdc_record_header.collection_interval_length
            timestamp = ss.cdc_record_header.timestamp

            ####################################################
            # First handle Subsystem Utility Switches
            # Create INSERT statement only if the the switch is in the Dict and is a new value
            ####################################################

            subsystemDict = self.systemDict.get(system_id)
            if subsystemDict == None:
                # This system id has NOT been encounter yet, so update the systemDict and create an INSERT statement

                # Create a tuple of SubsystemUtilitySwitches by subsystem
                switchesObj = SubsystemUtilitySwitches(ss.utility_switches, timestamp)
                subsystemTuple = {ss.subsystem_name:switchesObj}
                # Create subsystem dictionary
                subsystemDict = {}
                subsystemDict.update(subsystemTuple)
                # Update the subsystem dictionary and system dictionary.
                systemTuple = {system_id:subsystemDict}
                self.systemDict.update(systemTuple)

                # Arguments must be in order of db_field_names
                util_sw_row = [timestamp,
                        timestamp,
                        system_id,
                        subsystem_id,
                        "ON" if ss.utility_switches & self.RECOUP_ON else "OFF",
                        "ON" if ss.utility_switches & self.SELECTIVE_FILE_TRACE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_FILE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_ENABLE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_IMPLEMENT_ON else "OFF",
                        "ON" if ss.utility_switches & self.DATA_COLLECTION_ON else "OFF",
                        "ON" if ss.utility_switches & self.CAPTURE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RESTORE_ON else "OFF",
                        "ON" if ss.utility_switches & self.SON_COPY_ON else "OFF",
                        "ON" if ss.utility_switches & self.TRAP_ON else "OFF",
                        "ON" if ss.utility_switches & self.DB_REORG_ON else "OFF",
                        "ON" if ss.utility_switches & self.CCP_TRACE_ON else "OFF",
                        "ON" if ss.utility_switches & self.LINK_TRACE_ON else "OFF"
                        ]

                #logger.info(f'SubsystemHandler1 util_sw_row: {util_sw_row}')
                util_sw_rows.append(util_sw_row)

                row_inserted = True

            else:
                # This system id has been encountered before.

                switchesObj = subsystemDict.get(ss.subsystem_name)
                if switchesObj == None or switchesObj.switches != ss.utility_switches:
                    # Either this subsystem has not been added for this system -or-
                    # the subsystem is there but the switches have change.  So create an INSERT statement.

                    # Update the subsystem dictionary and system dictionary
                    switchesObj = SubsystemUtilitySwitches(ss.utility_switches, timestamp)
                    subsystemTuple = {ss.subsystem_name:switchesObj}
                    subsystemDict.update(subsystemTuple)
                    systemTuple = {system_id:subsystemDict}
                    self.systemDict.update(systemTuple)

                    # Arguments must be in order of db_field_names
                    util_sw_row = [timestamp,
                        timestamp,
                        system_id,
                        subsystem_id,
                        "ON" if ss.utility_switches & self.RECOUP_ON else "OFF",
                        "ON" if ss.utility_switches & self.SELECTIVE_FILE_TRACE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_FILE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_ENABLE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RCS_IMPLEMENT_ON else "OFF",
                        "ON" if ss.utility_switches & self.DATA_COLLECTION_ON else "OFF",
                        "ON" if ss.utility_switches & self.CAPTURE_ON else "OFF",
                        "ON" if ss.utility_switches & self.RESTORE_ON else "OFF",
                        "ON" if ss.utility_switches & self.SON_COPY_ON else "OFF",
                        "ON" if ss.utility_switches & self.TRAP_ON else "OFF",
                        "ON" if ss.utility_switches & self.DB_REORG_ON else "OFF",
                        "ON" if ss.utility_switches & self.CCP_TRACE_ON else "OFF",
                        "ON" if ss.utility_switches & self.LINK_TRACE_ON else "OFF"
                        ]

                    #logger.info(f'SubsystemHandler2 util_sw_row: {util_sw_row}')
                    util_sw_rows.append(util_sw_row)

                    row_inserted = True

            if row_inserted == False:
                # The above code did no create an INSERT for the collection, so...
                # Create an SQL UPDATE statement to update the time in the row with insert for this system id and subsystem_id
                switchesObj = subsystemDict.get(ss.subsystem_name)
                sql = create_pre_sql_update(self.util_sw_table_name, timestamp, switchesObj.altered_time, system_id, subsystem_id)
                insert_manys.append(InsertMany(sql))

            ####################################################
            # Second handle the Subsystem values
            ####################################################

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + ss.subsystem_name

            # Create an array of metrics to test for 0
            metrics = [ss.high_spd_msgs,
               ss.routed_entries,
               ss.created_entries,
               ss.ecb_exits]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # Arguments must be in order of db_field_names
               ss_row = [timestamp,
                   duration,
                   system_id,
                   subsystem_id,
                   self.calculate_rate(ss.high_spd_msgs, duration),
                   self.calculate_rate(ss.routed_entries, duration),
                   self.calculate_rate(ss.created_entries, duration),
                   self.calculate_rate(ss.ecb_exits, duration)]

               #logger.info(f'SubsystemHandler ss_row: {ss_row}')
               ss_rows.append(ss_row)

        insert_manys.append(InsertMany(self.ss_pre_sql_insert, rows=ss_rows))
        insert_manys.append(InsertMany(self.util_sw_pre_sql_insert, rows=util_sw_rows))

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.ss_table_name, cls.util_sw_table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)