#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file generates SQL INSERT statements to insert into both      #
#  the system_metrics_table and the tpf_system_blocks_metrics_table.  #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_cdc.cdc_record_types.system_blocks import SystemBlocks
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SystemBlocksHandler(DbDataHandler, CdcDbHelper):

    # Local variables to insert a subset of system block metrics into the system_metrics_table
    # (Josh refers to this table as the Bellweather metrics table intended to show the overall health of the zTPF system.)
    table_name = 'system_metrics_table'
    db_field_names = ['time', 'duration', 'inuse_ecb', 'inuse_iob']
    db_key_fields = {'time'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)
    db_field_names_v2 = db_field_names + ['system_id']
    db_key_fields_v2 = db_key_fields.union({'system_id'})
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert_v2 = create_pre_sql_insert(table_name, db_field_names_v2, db_key_fields_v2)

    # Local variables to insert the complete set of system block metrics into the tpf_system_blocks_metrics_table
    sb_table_name = 'tpf_system_blocks_metrics_table'
    sb_db_field_names = ['time', 'duration', 'system_id',
                         'iob_allocated', 'iob_inuse', 'iob_inuse_pct',
                         'ecb_allocated', 'ecb_inuse', 'ecb_inuse_pct',
                         'swb_allocated', 'swb_inuse', 'swb_inuse_pct',
                         'common_allocated', 'common_inuse', 'common_inuse_pct',
                         'frame_allocated', 'frame_inuse', 'frame_inuse_pct',
                         'frame1mb_allocated', 'frame1mb_inuse', 'frame1mb_inuse_pct',
                         'xwb_allocated', 'xwb_inuse', 'xwb_inuse_pct',
                         'rlog_allocated', 'rlog_inuse', 'rlog_inuse_pct']
    sb_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    sb_pre_sql_insert = create_pre_sql_insert(sb_table_name, sb_db_field_names, sb_db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: SystemBlocks, parent: CdcData = None):

        insert_manys: List[InsertMany] = []

        # ---------------------------------------------------------------------
        # First create the InsertMany statement for the system_metric_table
        # ---------------------------------------------------------------------
        # Base version
        v_insert = self.pre_sql_insert

        # Arguments must be in order of db_field_names
        row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               data.ecb_inuse,
               data.iob_inuse]

        #logger.info(f'SystemBlocksHandler row: {row}')

        # Version 2
        if parent is not None and parent.header.version_num >= 2:
            v_insert = self.pre_sql_insert_v2
            row += [self.get_tpf_system_id(parent.header, db_cursor)]

        rows = [row]
        insert_manys.append(InsertMany(v_insert, rows=rows))

        # ---------------------------------------------------------------------
        # Second create the InsertMany statement for the tpf_system_blocks_metric_table
        # ---------------------------------------------------------------------
        sb_rows = []

        # Arguments must be in order of db_field_names
        sb_row = [data.cdc_record_header.timestamp,
                  data.cdc_record_header.collection_interval_length,
                  self.get_tpf_system_id(parent.header, db_cursor),
                  data.iob_allocated,
                  data.iob_inuse,
                  self.calculate_percentage(data.iob_inuse, data.iob_allocated),
                  data.ecb_allocated,
                  data.ecb_inuse,
                  self.calculate_percentage(data.ecb_inuse, data.ecb_allocated),
                  data.swb_allocated,
                  data.swb_inuse,
                  self.calculate_percentage(data.swb_inuse, data.swb_allocated),
                  data.common_allocated,
                  data.common_inuse,
                  self.calculate_percentage(data.common_inuse, data.common_allocated),
                  data.frame_allocated,
                  data.frame_inuse,
                  self.calculate_percentage(data.frame_inuse, data.frame_allocated),
                  data.frame1mb_allocated,
                  data.frame1mb_inuse,
                  self.calculate_percentage(data.frame1mb_inuse, data.frame1mb_allocated),
                  data.xwb_allocated,
                  data.xwb_inuse,
                  self.calculate_percentage(data.xwb_inuse, data.xwb_allocated),
                  data.rlog_allocated,
                  data.rlog_inuse,
                  self.calculate_percentage(data.rlog_inuse, data.rlog_allocated)]

        #logger.info(f'SystemBlocksHandler sb_row: {sb_row}')

        sb_rows = [sb_row]
        insert_manys.append(InsertMany(self.sb_pre_sql_insert, rows=sb_rows))

        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name, cls.sb_table_name]

    def calculate_percentage(self, inuse, allocated):
       if allocated == 0:
          return 0
       return ((inuse / allocated)  * 100)
