 #######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file generates SQL INSERT statements to insert into           #
#  multiple tables.                                                   #
#  For tpf_traces_metrics_table, if new collection data matches data  #
#  already saved to the database for the system id, then an UPDATE is #
#  performed to update the time column.                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_utils.db_utils import create_pre_sql_update
from tpf_cdc.cdc_record_types.system_messages import SystemMessages
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SystemMessagesHandler(DbDataHandler, CdcDbHelper):

    ##############################################
    # system_metrics_table:
    # definitions for inserting system message data into the system_metrics_table
    ##############################################
    table_name = 'system_metrics_table'
    db_field_names = ['time', 'duration', 'actual_cpu', 'inuse_istreams', 'istream_cap']
    db_key_fields = {'time'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)
    db_field_names_v2 = db_field_names + ['system_id']
    db_key_fields_v2 = db_key_fields.union({'system_id'})
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert_v2 = create_pre_sql_insert(table_name, db_field_names_v2, db_key_fields_v2)

    ##############################################
    # tpf_system_messages_metrics_table:
    # definitions for inserting system message data into the tpf_system_messages_metrics_table
    ##############################################
    sys_msgs_table_name = 'tpf_system_messages_metrics_table'
    sys_msgs_db_field_names = ['time', 'duration', 'system_id',
                               'high_spd_msgs_per_sec', 'tcpip_msgs_per_sec',
                               'routed_entries_per_sec', 'created_entries_per_sec', 'commits_per_sec',
                               'number_of_istreams', 'usable_istreams', 'processor_util_pct',
                               'gp_processor_util_pct', 'gp_max_cap_for_lpar_util_pct', 'gp_number_of_engines',
                               'istream_cap', 'speed_boost', 'istream_boost']
    sys_msgs_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    sys_msgs_pre_sql_insert = create_pre_sql_insert(sys_msgs_table_name, sys_msgs_db_field_names, sys_msgs_db_key_fields)
    # Following are the supported bits in ipl_boost_options (only two currently)
    SPEED_BOOST = 0x80
    ISTREAM_BOOST = 0x40

    ##############################################
    # tpf_shutdown1_metrics_table:
    # definitions for inserting system message data into the tpf_shutdown1_metrics_table
    ##############################################
    shutdown1_table_name = 'tpf_shutdown1_metrics_table'
    shutdown1_db_field_names = ['time', 'system_id', 'shutdown_type_id', 'reason_id', 'shutdown_occurred']
    shutdown1_db_key_fields = {'time', 'system_id', 'shutdown_type_id', 'reason_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    shutdown1_pre_sql_insert = create_pre_sql_insert(shutdown1_table_name, shutdown1_db_field_names, shutdown1_db_key_fields)
    # Following are the possible bits in shutdown1 as defined by /base/macro/dc0rt.mac
    # Note: z/TPF Data Reduction creates it's shutdown count report in file base/ol/jrs3.pli   See "DO SI" loop.
    #                        bit          ZACLV    Reason
    # SHUTDOWN1 1st Byte - CPU Loop shutdown controls
    INPUT_LIST_HIGH_ECB    = 0x80000000 # MXECBS - Greater than maximum number of entry control blocks (ECBs) allowed in the z/TPF system
    INPUT_LIST_LOW_FRM	   = 0x40000000 # MNFRAI - Less than minimum number of frames that must be available to service the input list
    INPUT_LIST_LOW_COM	   = 0x20000000 # MNCMBI - Less than minimum number of common blocks that must be available to service the input list
    INPUT_LIST_LOW_1MB_FRM = 0x10000000 # MNFR1MBI - Less than minimum number of 1-MB frames that must be available to service the input list
    INPUT_LIST_LOW_ECB	   = 0x08000000 # MNECBI - Less than minimum number of ECBs that must be available to service the input list
    INPUT_LIST_LOW_SWB	   = 0x04000000 # MNSWBI - Less than minimum number of system work blocks (SWBs) that must be available to service the input list
    INPUT_LIST_LOW_IOB	   = 0x02000000 # MNIOBI - Less than minimum number of I/O blocks that must be available to service the input list
    DEFERRED_LIST_HIGH_ECB = 0x01000000 # MXECBD - Greater than maximum number of ECBs in the z/TPF system to service the deferred list
    # SHUTDOWN1 2nd Byte - Create shutdown controls plus a few others
    CREM_LOW_XWB           = 0x00800000 # MNXWBM - CREM macro was issued with less than required XWBs available
    CRED_LOW_XWB           = 0x00400000 # MNXWBD - CRED macro was issued with less than required XWBs available
    CREX_LOW_XWB           = 0x00200000 # MNXWBX - CREX macro was issued with less than required XWBs available
    CREM_HIGH_READY_LIST   = 0x00100000 # MXITEMR (not a real ZACLV code) - CREM macro was issued with too many items on the ready list (Note: the level is hardcoded in cp/cicr.cpy - see: MAX_BLOCKS_ON_LIST  DC  F'50')
    INPUT_LIST_RLOG_LOW    = 0x00080000 # MNRLOGI - Less than minimum number of recovery log tracks that must be available to service the input list
    TAS_RLOG_LOW           = 0x00040000 # MNRLOGT - Less than minimum number of recovery log tracks that must be available to activate the time available supervisor
    INPUT_LIST_LOW_XWB     = 0x00020000 # MNXWBI - Less than minimum number of extended system work blocks (XWBs) that must be available to service the input list
    TAS_LOW_XWB            = 0x00010000 # MNXWBT - Less than minimum number of extended system work blocks (XWBs) that must be available to activate the time available supervisor
    # SHUTDOWN1 3rd Byte - CCP shutdown controls
    # AI_INPUT             = 0x00004000 # ------ - Greater than AI INPUT   ,.GT. X INPUT LIST ITEMS - Obsoleted
    # BSC_INPUT            = 0x00000800 # ------ - Greater than BSC INPUT  ,.GT. X INPUT LIST ITEMS - Obsoleted
    INPUT_3270_LOCAL       = 0x00000100 # MXITEMI (not a real ZACLV code) - Greater than 3270 local shutdown level (CTKA (CK1KE))- 3270L INPUT,.GT. X INPUT LIST ITEMS
    # SHUTDOWN1 4th Byte - Time Available Supervisor controls
    #                      = 0x00000080 # ------ - NCP SLOWDOWN - Obsoleted
    #                      = 0x00000040 # ------ - SNA CTC INPUT SLOWDOWN - Obsoleted
    #                      = 0x00000020 # ------ - SNA CTC OUTPUT SLOWDOWN - Obsoleted
    TAS_LOW_FRM            = 0x00000010 # MNFRAT - Less than minimum number of frames that must be available to activate the time available supervisor
    TAS_LOW_COM            = 0x00000008 # MNECBT - Less than minimum number of ECBs that must be available to activate the time available supervisor
    TAS_LOW_ECB            = 0x00000004 # MNCMBT - Less than minimum number of common blocks that must be available to activate the time available supervisor
    TAS_LOW_1MB_FRM        = 0x00000002 # MNFR1MBT - Less than minimum number of 1-MB frames that must be available to activate the time available supervisor
    TAS_LOW_SWB            = 0x00000001 # MNSWBT - Less than minimum number of system work blocks (SWBs) that must be available to activate the time available supervisor
    NUM_SHUTDOWN_BITS = 22
    # Text for the normalization tpf_shutdown_reason_table
    INPUT_LIST_TEXT = "Input List"
    DEFERRED_LIST_TEXT = "Deferred List"
    TIME_AVAILABLE_SUPERVISOR_TEXT = "Time Available Supervisor"
    CREM_MACRO_TEXT = "CREM Macro"
    CRED_MACRO_TEXT = "CRED Macro"
    CREX_MACRO_TEXT = "CREX Macro"
    LOCAL_3270_INPUT_TEXT = "3270 Local Input"

    ##############################################
    # tpf_traces_metrics_table:
    # definitions for inserting system message data into the tpf_traces_metrics_table
    ##############################################
    traces_table_name = 'tpf_traces_metrics_table'
    traces_db_field_names = ['time', 'altered', 'system_id', 'block_check_mode', 'branch', 'branch_relative_target_check_mode',
       'ECB_heap', 'ECB_macro', 'enter_back', 'extended_C_function', 'function_trace', 'heap_check_mode',
       'IO', 'IBM_DBUGC_log', 'IBM_DBUGC_test', 'idle_timer', 'IO_CCW', 'lock_trace', 'reg_this_IPL',
       'socket', 'system_log', 'user_DBUGC_log', 'user_DBUGC_test', 'VFA']
    traces_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    traces_pre_sql_insert = create_pre_sql_insert(traces_table_name, traces_db_field_names, traces_db_key_fields)
    # dataDict is a static cache of SystemMessages by system id   e.g. {2:<SystemMessages>}
    dataDict = {}
    # insertTimeDict is a static cache of insert times by system id   e.g. {2:<timestamp>}
    insertTimeDict = {}
    # Following are the possible bits in traces
    ECB_MACRO_TRACE_ON           = 0x80000000
    ENTER_BACK_TRACE_ON          = 0x40000000
    SYSTEM_LOG_TRACE_ON          = 0x20000000
    IO_TRACE_ON                  = 0x10000000
    BRANCH_TRACE_ON              = 0x08000000
    REGS_TRACE_THIS_IPL_ON       = 0x04000000
    SOCKET_TRACE_ON	             = 0x02000000
    BLOCK_CHECK_MODE_ON          = 0x01000000
    IO_CCW_TRACE_ON	             = 0x00800000
    EXTENDED_C_FUNCTION_TRACE_ON = 0x00400000
    ZTPF_FUNCTION_TRACE_ON       = 0x00200000
    BRANCH_TARGET_CHECK_MODE_ON  = 0x00080000
    IDLE_TIMER_ON                = 0x00040000
    ECB_HEAP_TRACE_ON            = 0x00020000
    HEAP_CHECK_MODE_ON           = 0x00010000
    IBM_DBUGC_TEST_ON            = 0x00008000
    IBM_DBUGC_LOG_ON             = 0x00004000
    USER_DBUGC_TEST_ON           = 0x00002000
    USER_DBUGC_LOG_ON            = 0x00001000
    VFA_TRACE_ON                 = 0x00000080
    LOCK_TRACE_ON                = 0x00000040


    def get_mysql_insert_manys(self, db_cursor: Cursor, data: SystemMessages, parent: CdcData = None):
        #logger.info(f'SystemMessagesHandler pre_sql_insert: {self.pre_sql_insert}')
        #logger.info(f'SystemMessagesHandler sys_msgs_pre_sql_insert: {self.sys_msgs_pre_sql_insert}')
        #logger.info(f'SystemMessagesHandler shutdown1_pre_sql_insert: {self.shutdown1_pre_sql_insert}')

        insert_manys: List[InsertMany] = []

        system_id = self.get_tpf_system_id(parent.header, db_cursor)
        timestamp = data.cdc_record_header.timestamp

        #######################################################
        # 1. Create insert statement for insert into system_metrics_table
        #######################################################
        # Base version
        v_insert = self.pre_sql_insert

        # Arguments must be in order of db_field_names
        row = [timestamp,
               data.cdc_record_header.collection_interval_length,
               data.processor_utilization,
               data.usable_istreams,
               data.istream_cap]

        # Version 2
        if parent is not None and parent.header.version_num >= 2:
            v_insert = self.pre_sql_insert_v2
            row += [self.get_tpf_system_id(parent.header, db_cursor)]

        #logger.info(f'SystemMessagesHandler row: {row}')

        rows = [row]
        insert_manys.append(InsertMany(v_insert, rows=rows))

        #######################################################
        # 2. Create insert statement for insert into tpf_system_messages_metrics_table
        #######################################################
        sys_msgs_rows = []
        duration = data.cdc_record_header.collection_interval_length
         # Arguments must be in order of db_field_names
        sys_msgs_row = [timestamp,
                   duration,
                   self.get_tpf_system_id(parent.header, db_cursor),
                   self.calculate_rate(data.high_spd_msgs, duration),
                   self.calculate_rate(data.tcpip_msgs, duration),
                   self.calculate_rate(data.routed_entries, duration),
                   self.calculate_rate(data.created_entries, duration),
                   self.calculate_rate(data.commits, duration),
                   data.number_of_istreams,
                   data.usable_istreams,
                   data.processor_utilization / 10,
                   data.gp_processor_utilization / 10,
                   data.gp_max_cap_for_lpar_utilization / 10,
                   data.gp_num_of_engines,
                   data.istream_cap,
                   "ON" if data.ipl_boost_options & self.SPEED_BOOST else "OFF",
                   "ON" if data.ipl_boost_options & self.ISTREAM_BOOST else "OFF"]

        #logger.info(f'SystemMessagesHandler sys_msgs_row: {sys_msgs_row}')

        sys_msgs_rows = [sys_msgs_row]
        insert_manys.append(InsertMany(self.sys_msgs_pre_sql_insert, rows=sys_msgs_rows))

        #######################################################
        # 3. Create insert statements for insert into tpf_shutdown1_metrics_table
        #######################################################
        shutdown1_rows = []
        i = 0
        # Loop once for each of the shutdown1 bits to handle
        while i < self.NUM_SHUTDOWN_BITS:
           shutdown1_row = self.create_shutdown1_row(db_cursor, system_id, data, i)
           if shutdown1_row:
              #logger.info(f'SystemMessagesHandler inserting shutdown row: {shutdown1_row}')
              shutdown1_rows.append(shutdown1_row)
           i += 1
        insert_manys.append(InsertMany(self.shutdown1_pre_sql_insert, rows=shutdown1_rows))

        #######################################################
        # 4. Create insert statement for insert into tpf_traces_metrics_table
        #######################################################
        traces_row = self.create_traces_row(system_id, data)
        if traces_row:
           # INSERT the new row
           traces_rows = [traces_row]
           #logger.info(f'SystemMessagesHandler inserting traces_row: {traces_row}')
           insert_manys.append(InsertMany(self.traces_pre_sql_insert, rows=traces_rows))

           # Save the row insert time in local cache for this system id
           self.insertTimeDict.update({system_id:data.cdc_record_header.timestamp})
        else:
           # UPDATE the time column in the existing row
           sql = create_pre_sql_update(self.traces_table_name, timestamp, self.insertTimeDict.get(system_id), system_id)
           insert_manys.append(InsertMany(sql))

        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name, cls.sys_msgs_table_name, cls.traces_table_name]


    def create_shutdown1_row(self, db_cursor, system_id, data, loop):
        """
        Create and return a row for the INSERT of shutdown1 counts.
        The 'data' parameter is used to access the shutdown1 field in the system messages data type
        The 'loop' parameter is simply used to check only a single bit in data.shutdown1 upon the call to this function
        Returns an empty row if the 'type' parameter is not known.
        Note: Relevant Data Reduction files that indicate how the Shutdown and Reason fields are determined:
              /base/ol/jrs3.pli - generates DC Shutdown Indicators section in a DR report
              /base/macro/dc0rt.mac - defines DC Shutdown bits
        """

        # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        # NOTE: These reason strings here must be kept in synch with strings in the tpf_shutdown1_reason_table in tpf_zrtmc_db_schema_upgrade.sql
        # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        # Present all Input List shutdown events together
        if loop == 0:
            if (data.shutdown1 & self.INPUT_LIST_HIGH_ECB) == self.INPUT_LIST_HIGH_ECB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MXECBS"),
                  1]
        elif loop == 1:
            if (data.shutdown1 & self.INPUT_LIST_LOW_FRM) == self.INPUT_LIST_LOW_FRM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNFRAI"),
                  1]
        elif loop == 2:
            if (data.shutdown1 & self.INPUT_LIST_LOW_COM) == self.INPUT_LIST_LOW_COM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNCMBI"),
                  1]
        elif loop == 3:
            if (data.shutdown1 & self.INPUT_LIST_LOW_1MB_FRM) == self.INPUT_LIST_LOW_1MB_FRM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNFR1MBI"),
                  1]
        elif loop == 4:
            if (data.shutdown1 & self.INPUT_LIST_LOW_ECB) == self.INPUT_LIST_LOW_ECB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNECBI"),
                  1]
        elif loop == 5:
            if (data.shutdown1 & self.INPUT_LIST_LOW_SWB) == self.INPUT_LIST_LOW_SWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNSWBI"),
                  1]
        elif loop == 6:
            if (data.shutdown1 & self.INPUT_LIST_LOW_IOB) == self.INPUT_LIST_LOW_IOB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNIOBI"),
                  1]
        elif loop == 7:
            if (data.shutdown1 & self.INPUT_LIST_LOW_XWB) == self.INPUT_LIST_LOW_XWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNXWBI"),
                  1]
        elif loop == 8:
            if (data.shutdown1 & self.INPUT_LIST_RLOG_LOW) == self.INPUT_LIST_RLOG_LOW:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.INPUT_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNRLOGI"),
                  1]

        # Present all Deferred List shutdown events together
        elif loop == 9:
            if (data.shutdown1 & self.DEFERRED_LIST_HIGH_ECB) == self.DEFERRED_LIST_HIGH_ECB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.DEFERRED_LIST_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MXECBD"),
                  1]

        # Present all Create Macro shutdown events together
        elif loop == 10:
            if (data.shutdown1 & self.CREM_LOW_XWB) == self.CREM_LOW_XWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.CREM_MACRO_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNXWBM"),
                  1]
        elif loop == 11:
            if (data.shutdown1 & self.CREM_HIGH_READY_LIST) == self.CREM_HIGH_READY_LIST:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.CREM_MACRO_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MXITEMR"),
                  1]
        elif loop == 12:
            if (data.shutdown1 & self.CRED_LOW_XWB) == self.CRED_LOW_XWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.CRED_MACRO_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNXWBD"),
                  1]
        elif loop == 13:
            if (data.shutdown1 & self.CREX_LOW_XWB) == self.CREX_LOW_XWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.CREX_MACRO_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNXWBX"),
                  1]

        # Present all Time Available Supervisor shutdown events together
        elif loop == 14:
            if (data.shutdown1 & self.TAS_LOW_FRM) == self.TAS_LOW_FRM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNFRAT"),
                  1]
        elif loop == 15:
            if (data.shutdown1 & self.TAS_LOW_ECB) == self.TAS_LOW_ECB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNECBT"),
                  1]
        elif loop == 16:
            if (data.shutdown1 & self.TAS_LOW_COM) == self.TAS_LOW_COM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNCMBT"),
                  1]
        elif loop == 17:
            if (data.shutdown1 & self.TAS_LOW_1MB_FRM) == self.TAS_LOW_1MB_FRM:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNFR1MBT"),
                  1]
        elif loop == 18:
            if (data.shutdown1 & self.TAS_LOW_SWB) == self.TAS_LOW_SWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNSWBT"),
                  1]
        elif loop == 19:
            if (data.shutdown1 & self.TAS_LOW_XWB) == self.TAS_LOW_XWB:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNXWBT"),
                  1]
        elif loop == 20:
            if (data.shutdown1 & self.TAS_RLOG_LOW) == self.TAS_RLOG_LOW:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.TIME_AVAILABLE_SUPERVISOR_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MNRLOGT"),
                  1]

        # Present all CCP (Communications Control Program) events together
        elif loop == 21:
            if (data.shutdown1 & self.INPUT_3270_LOCAL) == self.INPUT_3270_LOCAL:
               return [data.cdc_record_header.timestamp,
                  system_id,
                  self.get_shutdown1_shutdown_type_id(db_cursor, self.LOCAL_3270_INPUT_TEXT),
                  self.get_shutdown1_reason_id(db_cursor, "MXITEMI"),
                  1]

        return []

    def create_traces_row(self, system_id, data):
        """
        Create and return a row for the INSERT of traces.
        Returns an empty row if the 'type' parameter is not known.
        """
        traces_row = []
        
        cachedData = self.dataDict.get(system_id)
        # Note: cachedTraces is None if this is the first collection since the zrtmc_analyzer started
        if cachedData == None or cachedData.traces != data.traces:
           # This system_id has NOT been encountered yet or the system_id has been encountered but the
           # traces data has changed since the last collection, so update the dataDict and create a row
           self.dataDict.update({system_id:data})
           traces_row = [data.cdc_record_header.timestamp,
                     data.cdc_record_header.timestamp,
                     system_id,
                     "ON" if data.traces & self.BLOCK_CHECK_MODE_ON else "OFF",
                     "ON" if data.traces & self.BRANCH_TRACE_ON else "OFF",
                     "ON" if data.traces & self.BRANCH_TARGET_CHECK_MODE_ON else "OFF",
                     "ON" if data.traces & self.ECB_HEAP_TRACE_ON else "OFF",
                     "ON" if data.traces & self.ECB_MACRO_TRACE_ON else "OFF",
                     "ON" if data.traces & self.ENTER_BACK_TRACE_ON else "OFF",
                     "ON" if data.traces & self.EXTENDED_C_FUNCTION_TRACE_ON else "OFF",
                     "ON" if data.traces & self.ZTPF_FUNCTION_TRACE_ON else "OFF",
                     "ON" if data.traces & self.HEAP_CHECK_MODE_ON else "OFF",
                     "ON" if data.traces & self.IO_TRACE_ON else "OFF",
                     "ON" if data.traces & self.IBM_DBUGC_TEST_ON else "OFF",
                     "ON" if data.traces & self.IBM_DBUGC_LOG_ON else "OFF",
                     "ON" if data.traces & self.IDLE_TIMER_ON else "OFF",
                     "ON" if data.traces & self.IO_CCW_TRACE_ON else "OFF",
                     "ON" if data.traces & self.LOCK_TRACE_ON else "OFF",
                     "ON" if data.traces & self.REGS_TRACE_THIS_IPL_ON else "OFF",
                     "ON" if data.traces & self.SOCKET_TRACE_ON else "OFF",
                     "ON" if data.traces & self.SYSTEM_LOG_TRACE_ON else "OFF",
                     "ON" if data.traces & self.USER_DBUGC_TEST_ON else "OFF",
                     "ON" if data.traces & self.USER_DBUGC_LOG_ON else "OFF",
                     "ON" if data.traces & self.VFA_TRACE_ON else "OFF"
                     ]

        return traces_row

    def get_shutdown1_shutdown_type_id(self, db_cursor: Cursor, shutdown1_shutdown_type):
        return id_manager.get_shutdown1_shutdown_type_id(shutdown1_shutdown_type, db_cursor)

    def get_shutdown1_reason_id(self, db_cursor: Cursor, shutdown1_reason):
        return id_manager.get_shutdown1_reason_id(shutdown1_reason, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)

