#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_data_manager import id_manager
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_cdc.cdc_record_types.tcp_ip import TcpIp, TcpIpApp
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor

logger = logging.getLogger('tpf_zrtmc_analyzer')

class TcpIpHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_tcpip_metrics_table'
    db_field_names = ['time', 'system_id', 'duration', 'app_name_id', 'port', 'weight',
       'msgs_per_sec', 'bytes_per_sec', 'packets_per_sec',
       'bytes_sent_per_sec', 'bytes_recvd_per_sec',
       'msgs_sent_per_sec', 'msgs_recvd_per_sec',
       'packets_sent_per_sec', 'packets_recvd_per_sec']
    db_key_fields = {'time', 'app_name_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: TcpIp, parent: CdcData = None):

        rows = []
        loopCounter = 0

        for tcp_ip_app in data.cdc_record_array:

            duration = tcp_ip_app.cdc_record_header.collection_interval_length
            timestamp = tcp_ip_app.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + tcp_ip_app.application_name

            # Create an array of metrics to test for 0
            # Only metrics needed to determine tcpip application inactivity are included in the array.
            metrics = [tcp_ip_app.messages_sent,
               tcp_ip_app.messages_received,
               tcp_ip_app.bytes_sent,
               tcp_ip_app.bytes_received,
               tcp_ip_app.packets_sent,
               tcp_ip_app.packets_received]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # Arguments must be in order of db_field_names
               row = [timestamp,
                   system_id,
                   duration,
                   self.get_tcpip_name_id(db_cursor, tcp_ip_app.application_name),
                   tcp_ip_app.port,
                   tcp_ip_app.weight,
                   self.calculate_rate(tcp_ip_app.messages_sent + tcp_ip_app.messages_received, duration),
                   self.calculate_rate(tcp_ip_app.bytes_sent + tcp_ip_app.bytes_received, duration),
                   self.calculate_rate(tcp_ip_app.packets_sent + tcp_ip_app.packets_received, duration),
                   self.calculate_rate(tcp_ip_app.bytes_sent, duration),
                   self.calculate_rate(tcp_ip_app.bytes_received, duration),
                   self.calculate_rate(tcp_ip_app.messages_sent, duration),
                   self.calculate_rate(tcp_ip_app.messages_received, duration),
                   self.calculate_rate(tcp_ip_app.packets_sent, duration),
                   self.calculate_rate(tcp_ip_app.packets_received, duration)]

               #logger.info(f'TcpIpHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_tcpip_name_id(self, db_cursor: Cursor, app_name):
        return id_manager.get_tcpip_name_id(app_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
