#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes TPFDF objects to a relational database.            #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.tpfdf import TPFDFFiles, TPFDFFile
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class TPFDFHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_tpfdf_metrics_table'
    db_field_names = ['time', 'system_id', 'subsystem_id', 'duration', 'file_id', 'file_id_ebcdic','file_name_alias_id',
       'dbadd_per_sec', 'dbadr_per_sec', 'dbckp_per_sec', 'dbcls_per_sec',
       'dbcpy_per_sec', 'dbcre_per_sec', 'dbdel_per_sec', 'dbdix_per_sec',
       'dbdsp_per_sec', 'dbidx_per_sec', 'dbkey_per_sec', 'dbmod_per_sec',
       'dbmrg_per_sec', 'dbopn_per_sec', 'dbred_per_sec', 'dbrep_per_sec',
       'dbret_per_sec', 'dbrst_per_sec', 'dbspa_per_sec', 'dbsrt_per_sec',
       'dbtld_per_sec', 'dbtlg_per_sec', 'dbtrd_per_sec', 'dbuky_per_sec',
       'dbhold_per_sec', 'dbuhld_per_sec', 'filnc_per_sec', 'filec_per_sec',
       'cfile_per_sec', 'pfile_per_sec', 'twrtc_per_sec', 'getfc_per_sec',
       'relfc_per_sec', 'detac_per_sec', 'attac_per_sec', 'fsysw_per_sec',
       'getcc_per_sec', 'relcc_per_sec', 'cfind_per_sec', 'pfind_per_sec',
       'btfil_per_sec', 'btfnd_per_sec', 'lrfil_per_sec', 'lrfnd_per_sec',
       'opn_hd_per_sec', 'opn_nhd_per_sec', 'opn_hnd_per_sec', 'opn_nhnd_per_sec',
       'cls_rel_per_sec', 'cls_norel_per_sec', 'cls_reuse_per_sec', 'cls_opn_per_sec',
       'prefp_per_sec', 'prefpw_per_sec', 'cache_prtl_iocnt_per_sec', 'cache_prtl_ioreq_per_sec',
       'cache_read_hit_per_sec', 'cache_read_miss_per_sec', 'blk_read_depth',
       'rsf_create_per_sec','rsf_read_per_sec', 'rsf_update_per_sec', 'rsf_delete_per_sec']
    db_key_fields = {'system_id', 'subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: TPFDFFiles, parent: CdcData = None):

        #logger.info(f'TPFDFHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []

        for dffile in data.cdc_record_array:
           #logger.info(f'TPFDFHandler dffile.subsystem_name={dffile.subsystem_name}')
           if(dffile.subsystem_name == "---"):
              # The online collector sent a row with a subsystem_name of three dashes
              logger.warning(f'The CDC_TPFDF data type was not collected for the collection period by the online ZRTMC collector.\nReason: The collector allows for up to {dffile.dbadd} files to be collected but {dffile.dbadr} files are defined on the system.  Contact IBM.')
              break
           elif(dffile.subsystem_name == "+++"):
              # The online collector sent a row with a subsystem_name of three dashes
              logger.warning(f'The CDC_TPFDF data type was not collected for the collection period by the online ZRTMC collector.\nReason: The collector allows for up to {dffile.dbadd} files with a DBOPEN to be collected but {dffile.dbadr} files with a DBOPN were encountered.  Contact IBM.')
              break
           else:
              duration = data.cdc_record_header.collection_interval_length

              block_read_depth = 0
              denominator = dffile.pfind + dffile.cache_read_hit
              if dffile.blk_read_depth > 0 and denominator > 0:
                 block_read_depth = dffile.blk_read_depth / denominator
              # Note: The block_read_depth calculation is taken from /ol/src/tpfdataredux/tpfdr_RecordAnalysis.cpp

              # Arguments must be in order of db_field_names
              row = [data.cdc_record_header.timestamp,
                 self.get_tpf_system_id(parent.header, db_cursor),
                 self.get_subsystem_id(db_cursor, dffile.subsystem_name),
                 duration,
                 dffile.fileId,
                 dffile.fileIdEbcdic,
                 self.get_tpfdf_file_name_alias_id(db_cursor, dffile.fileNameAlias),
                 self.calculate_rate(dffile.dbadd, duration),
                 self.calculate_rate(dffile.dbadr, duration),
                 self.calculate_rate(dffile.dbckp, duration),
                 self.calculate_rate(dffile.dbcls, duration),
                 self.calculate_rate(dffile.dbcpy, duration),
                 self.calculate_rate(dffile.dbcre, duration),
                 self.calculate_rate(dffile.dbdel, duration),
                 self.calculate_rate(dffile.dbdix, duration),
                 self.calculate_rate(dffile.dbdsp, duration),
                 self.calculate_rate(dffile.dbidx, duration),
                 self.calculate_rate(dffile.dbkey, duration),
                 self.calculate_rate(dffile.dbmod, duration),
                 self.calculate_rate(dffile.dbmrg, duration),
                 self.calculate_rate(dffile.dbopn, duration),
                 self.calculate_rate(dffile.dbred, duration),
                 self.calculate_rate(dffile.dbrep, duration),
                 self.calculate_rate(dffile.dbret, duration),
                 self.calculate_rate(dffile.dbrst, duration),
                 self.calculate_rate(dffile.dbspa, duration),
                 self.calculate_rate(dffile.dbsrt, duration),
                 self.calculate_rate(dffile.dbtld, duration),
                 self.calculate_rate(dffile.dbtlg, duration),
                 self.calculate_rate(dffile.dbtrd, duration),
                 self.calculate_rate(dffile.dbuky, duration),
                 self.calculate_rate(dffile.dbhold, duration),
                 self.calculate_rate(dffile.dbuhld, duration),
                 self.calculate_rate(dffile.filnc, duration),
                 self.calculate_rate(dffile.filec, duration),
                 self.calculate_rate(dffile.cfile, duration),
                 self.calculate_rate(dffile.pfile, duration),
                 self.calculate_rate(dffile.twrtc, duration),
                 self.calculate_rate(dffile.getfc, duration),
                 self.calculate_rate(dffile.relfc, duration),
                 self.calculate_rate(dffile.detac, duration),
                 self.calculate_rate(dffile.attac, duration),
                 self.calculate_rate(dffile.fsysw, duration),
                 self.calculate_rate(dffile.getcc, duration),
                 self.calculate_rate(dffile.relcc, duration),
                 self.calculate_rate(dffile.cfind, duration),
                 self.calculate_rate(dffile.pfind, duration),
                 self.calculate_rate(dffile.btfil, duration),
                 self.calculate_rate(dffile.btfnd, duration),
                 self.calculate_rate(dffile.lrfil, duration),
                 self.calculate_rate(dffile.lrfnd, duration),
                 self.calculate_rate(dffile.opn_hd, duration),
                 self.calculate_rate(dffile.opn_nhd, duration),
                 self.calculate_rate(dffile.opn_hnd, duration),
                 self.calculate_rate(dffile.opn_nhnd, duration),
                 self.calculate_rate(dffile.cls_rel, duration),
                 self.calculate_rate(dffile.cls_norel, duration),
                 self.calculate_rate(dffile.cls_reuse, duration),
                 self.calculate_rate(dffile.cls_opn, duration),
                 self.calculate_rate(dffile.prefp, duration),
                 self.calculate_rate(dffile.prefpw, duration),
                 self.calculate_rate(dffile.cache_prtl_iocnt, duration),
                 self.calculate_rate(dffile.cache_prtl_ioreq, duration),
                 self.calculate_rate(dffile.cache_read_hit, duration),
                 self.calculate_rate(dffile.cache_read_miss, duration),
                 block_read_depth,
                 self.calculate_rate(dffile.rsf_create, duration),
                 self.calculate_rate(dffile.rsf_read, duration),
                 self.calculate_rate(dffile.rsf_update, duration),
                 self.calculate_rate(dffile.rsf_delete, duration)]

              #logger.info(f'TPFDFHandler row: {row}')
              rows.append(row)

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)

    def get_tpfdf_file_name_alias_id(self, db_cursor: Cursor, file_name_alias):
        return id_manager.get_tpfdf_file_name_alias_id(file_name_alias, db_cursor)
