#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes Trad_DB_Encryption objects to a relational        #
#  database.                                                          #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.trad_db_encrypt import Trad_DB_Encryptions, Trad_DB_Encryption
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class Trad_DB_EncryptionHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_trad_db_encryption_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id',
                      'encryptions_per_sec', 'encryptions_compressions_per_sec',
                      'decryptions_per_sec', 'decryption_expansions_per_sec',
                      'encryption_errors_per_sec', 'compression_errors_per_sec']
    db_key_fields = {'time', 'system_id', 'subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: Trad_DB_Encryptions, parent: CdcData = None):

        #logger.info(f'Trad_DB_EncryptionHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for encryption in data.cdc_record_array:

            # Bring some object info into local variables to be used later in this function.
            duration = encryption.cdc_record_header.collection_interval_length
            timestamp = encryption.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + encryption.subsystemName

            # Create an array of metrics to test for 0
            metrics = [encryption.numEncryption,
               encryption.numEncryptionComp,
               encryption.numDecryption,
               encryption.numDecryptionExpansions,
               encryption.numErrorEncryption,
               encryption.numErrorCompression]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [encryption.cdc_record_header.timestamp,
                      duration,
                      system_id,
                      self.get_subsystem_id(db_cursor, encryption.subsystemName),
                      self.calculate_rate(encryption.numEncryption, duration),
                      self.calculate_rate(encryption.numEncryptionComp, duration),
                      self.calculate_rate(encryption.numDecryption, duration),
                      self.calculate_rate(encryption.numDecryptionExpansions, duration),
                      self.calculate_rate(encryption.numErrorEncryption, duration),
                      self.calculate_rate(encryption.numErrorCompression, duration)]

               #logger.info(f'Trad_DB_EncryptionHandler row: {row}')
               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)
