#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes VFA objects to a relational database.             #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.vfa import VFAs, VFA
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class VFAHandler(DbDataHandler, CdcDbHelper):
    table_name = 'tpf_vfa_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'subsystem_id',
    'data_reads_per_sec', 'finds_per_sec', 'candidate_files_per_sec', 'noncandidate_files_per_sec', 'force_files_per_sec', 'file_immediate_per_sec',
    'buffer_unavailable_per_sec', 'SLock_hits_per_sec', 'XLock_hits_per_sec', 'sync_misses_per_sec', 'sync_content_per_sec', 'cached_rht_hts_per_sec',
    'sync_age_outs_per_sec', 'dispense_4K_per_sec', 'rht_content_per_sec']
    db_key_fields = {'system_id', 'subsystem_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: VFAs, parent: CdcData = None):

        #logger.info(f'VFAHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []
        loopCounter = 0

        for vfa in data.cdc_record_array:

            duration = vfa.cdc_record_header.collection_interval_length
            timestamp = vfa.cdc_record_header.timestamp
            system_id = self.get_tpf_system_id(parent.header, db_cursor)

            # Upon the first iteration of this for loop, increment the heartbeatSeconds by the duration of this collection.
            if loopCounter == 0:
               self.heartbeatSeconds += duration
            loopCounter += 1

            # Create a unique string from the db_key_fields for the row of data
            heartbeatKey = str(system_id) + vfa.subsystem_name

            # Create an array of metrics to test for 0
            metrics = [vfa.data_reads,
               vfa.finds,
               vfa.candidate_files,
               vfa.noncandidate_files,
               vfa.force_files,
               vfa.file_immediate,
               vfa.buffer_unavailable,
               vfa.SLock_hits,
               vfa.XLock_hits,
               vfa.sync_misses,
               vfa.sync_content,
               vfa.cached_rht_hts,
               vfa.sync_age_outs,
               vfa.dispense_4K,
               vfa.rht_content]

            if self.checkRowInsert(heartbeatKey, metrics) == True:

               # order matters here! it must match with db_field_names
               row = [timestamp,
                   duration,
                   system_id,
                   self.get_subsystem_id(db_cursor, vfa.subsystem_name),
                   self.calculate_rate(vfa.data_reads, duration),
                   self.calculate_rate(vfa.finds, duration),
                   self.calculate_rate(vfa.candidate_files, duration),
                   self.calculate_rate(vfa.noncandidate_files, duration),
                   self.calculate_rate(vfa.force_files, duration),
                   self.calculate_rate(vfa.file_immediate, duration),
                   self.calculate_rate(vfa.buffer_unavailable, duration),
                   self.calculate_rate(vfa.SLock_hits, duration),
                   self.calculate_rate(vfa.XLock_hits, duration),
                   self.calculate_rate(vfa.sync_misses, duration),
                   self.calculate_rate(vfa.sync_content, duration),
                   self.calculate_rate(vfa.cached_rht_hts, duration),
                   self.calculate_rate(vfa.sync_age_outs, duration),
                   self.calculate_rate(vfa.dispense_4K, duration),
                   self.calculate_rate(vfa.rht_content, duration)]

               #logger.info(f'VFAHandler row: {row}')

               rows.append(row)

        # Reset the heartbeat if the threshold has passed.
        self.resetHeartbeat()

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_subsystem_id(self, db_cursor: Cursor, subsystem_name):
        return id_manager.get_tpf_subsystem_id(subsystem_name, db_cursor)

    def calculate_rate(self, count, duration):
        return id_manager.calculate_rate(count, duration)