#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class CommonDeployEntry(object):

    required_fields = ['usageCount', 'ss_name', 'key', 'type']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a CommonDeployEntry object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC COMMON_DEPLOY_FILES record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        try:
            self.usage_count = json_cdc_record["usageCount"]
            self.ss_name = str(json_cdc_record["ss_name"]).strip()
            self.key = str(json_cdc_record["key"]).strip()
            self.type = str(json_cdc_record["type"]).strip()
            # versionName is not a required field.  If not present in the Dictionary then set to an empty string.
            if 'versionName' in json_cdc_record.keys():
               self.version = str(json_cdc_record["versionName"]).strip()
            else:
               self.version = ""

            #logger.info(f'self.usage_count {self.usage_count}')
            #logger.info(f'self.ss_name {self.ss_name}')
            #logger.info(f'self.key {self.key}')
            #logger.info(f'self.type {self.type}')
            #logger.info(f'self.version {self.version}')

        except KeyError:
            msg = f'Unable to deserialize CDC COMMON_DEPLOY_FILES record. Missing fields: ' \
                  f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class CommonDeployFiles(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a CommonDeployFiles object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC COMMON_DEPLOY_FILES record.
        :param cdc_record_header: CdcRecordHeader object for this COMMON_DEPLOY_FILES object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [CommonDeployEntry(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
