#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file is used to convert DASD_METRICS data in JSON format      #
#  to object form.                                                    #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class DASDMetric(object):

    """ Fields in the JSON that are required and match """
    required_fields = ['devIndicator', 'devSymbolicModule', 'devVolumeNumber', 'devSubSystem', 'devDeviceType',
       'devDeviceAddress', 'devRecordCacheSubsystemId', 'devModuleQueue', 'devDataReads', 'devDataWrites',
       'devAccumulatedResponseTime', 'devResponseTimeCallCounter',
       'devSioscsLocks', 'devSioscsUnlocks', 'devMplfLockContention', 'devFdctcCount']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a DASDMetric object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC DASD_METRICS record.
        :param cdc_record_header: CdcRecordHeader object for this DASDMetric object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.devIndicator = str(json_cdc_record['devIndicator']).strip()
            self.devSymbolicModule = str(json_cdc_record['devSymbolicModule']).strip()
            self.devVolumeNumber = str(json_cdc_record['devVolumeNumber']).strip()
            self.devSubSystem = str(json_cdc_record['devSubSystem']).strip()
            self.devDeviceType = str(json_cdc_record['devDeviceType']).strip()
            self.devDeviceAddress = str(json_cdc_record['devDeviceAddress']).strip()
            self.devRecordCacheSubsystemId = str(json_cdc_record['devRecordCacheSubsystemId']).strip()
            self.devModuleQueue = json_cdc_record['devModuleQueue']
            self.devDataReads = json_cdc_record['devDataReads']
            self.devDataWrites = json_cdc_record['devDataWrites']
            self.devAccumulatedResponseTime = json_cdc_record['devAccumulatedResponseTime']
            self.devResponseTimeCallCounter = json_cdc_record['devResponseTimeCallCounter']
            self.devSioscsLocks = json_cdc_record['devSioscsLocks']
            self.devSioscsUnLocks = json_cdc_record['devSioscsUnlocks']
            self.devMplfLockContention = json_cdc_record['devMplfLockContention']
            self.devFdctcCount = json_cdc_record['devFdctcCount']
        except KeyError:
            log_msg = f'Unable to deserialize CDC DASD_METRICS. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)


class DASDMetrics(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a DASDMetrics object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC DASD_METRICS record.
        :param cdc_record_header: CdcRecordHeader object for this DASDMetrics object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [DASDMetric(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
