#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts DASD_POOLS data in JSON format to python object #
#  form.                                                              #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class DASDPoolsData(object):

    """ Fields in the JSON that are required """
    required_fields = ['POOL_TYPE','SET_SIZE', 'REORDER_TIME', 'DISPENSED_ADDR', 'RETURN_HITS']

    def __init__(self, json_pool_data):
        """
        Initializes a DASDPoolsData object.

        :param json_pool_data: dictionary of a single object of pool data
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.json_pool_raw_data = json_pool_data

        try:
            self.pool_type = str(json_pool_data['POOL_TYPE']).strip()
            #logger.info(f'Creating DASDPoolsData object for ss {self.pool_type}')
            self.set_size = json_pool_data['SET_SIZE']
            self.reorder_time = json_pool_data['REORDER_TIME']
            self.dispensed = json_pool_data['DISPENSED_ADDR']
            self.returned = json_pool_data['RETURN_HITS']

            #logger.warning(f'self.pool_type {self.pool_type}')
            #logger.warning(f'self.set_size {self.set_size}')
            #logger.warning(f'self.reorder_time {self.reorder_time}')
            #logger.warning(f'self.dispensed {self.dispensed}')
            #logger.warning(f'self.returned {self.returned}')

        except KeyError:
            msg = f'Unable to deserialize CDC DASD_POOLS record for DASDPoolsData. Missing fields: ' \
                  f'{get_missing_keys(json_pool_data, self.required_fields)}. Data: {json_pool_data}'
            logger.warning(msg)
            raise MissingFieldError(msg)

class DASDPoolDevices(object):

    """ Fields in the JSON that are required """
    required_fields = ['DEVICE_NAME', 'DEV_WRITES', 'DEV_READS', 'DEV_QUEUE', 'POOL_DATA',
                       'POOL_TYPE','SET_SIZE', 'REORDER_TIME', 'DISPENSED_ADDR', 'RETURN_HITS']

    def __init__(self, json_device_data):
        """
        Initializes a DASDPoolDevices object.

        :param json_device_data: dictionary of a single object of device data
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.json_device_raw_data = json_device_data
        self.pools_array = []

        try:
            self.logical_device_type = str(json_device_data['DEVICE_NAME']).strip()
            #logger.info(f'Creating DASDPoolDevices object for ss {self.logical_device_type}')
            self.dev_writes = json_device_data['DEV_WRITES']
            self.dev_reads = json_device_data['DEV_READS']
            self.dev_queue = json_device_data['DEV_QUEUE']
            self.pools_array = [DASDPoolsData(json_pool_data) for json_pool_data in json_device_data['POOL_DATA']]

            #logger.info(f'self.logical_device_type {self.logical_device_type}')
            #logger.info(f'self.dev_writes {self.dev_writes}')
            #logger.info(f'self.dev_reads {self.dev_reads}')
            #logger.info(f'self.dev_queue {self.dev_queue}')
            #logger.info(f'self.pools_array {self.pools_array}')

        except KeyError:
            msg = f'Unable to deserialize CDC DASD_POOLS record for DASDPoolDevices. Missing fields: ' \
                  f'{get_missing_keys(json_device_data, self.required_fields)}. Data: {json_device_data}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class DASDPool(object):

    """ Fields in the JSON that are required """
    required_fields = ['SS_NAME', 'data',
                       'DEVICE_NAME', 'DEV_WRITES', 'DEV_READS', 'DEV_QUEUE', 'POOL_DATA',
                       'POOL_TYPE','SET_SIZE', 'REORDER_TIME', 'DISPENSED_ADDR', 'RETURN_HITS']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a DASDPool object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC DASD_POOLS record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        #logger.info(f'Creating DASDPool json_cdc_record {json_cdc_record}')
        try:
            self.subsystem_name = str(json_cdc_record['SS_NAME']).strip()
            #logger.info(f'Creating DASDPool object for ss {self.subsystem_name}')
            self.devices_array = [DASDPoolDevices(json_device_data) for json_device_data in json_cdc_record['data']]

        except KeyError:
            msg = f'Unable to deserialize CDC DASD_POOLS record. Missing fields: ' \
                  f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class DASDPools(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a DASDPools object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC DASD_POOLS record.
        :param cdc_record_header: CdcRecordHeader object for this DASD_POOLS object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [DASDPool(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
