#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
#   20240725    PJ48071 - workload cpu for dedicated istreams issue   #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts ENH_CHL_UTILIZATION data in JSON format to      #
#  python object form.                                                #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class ChannelDescriptor(object):

    """ Fields in the JSON that are required """
    required_fields = ['channelID', 'isChannelShared', 'channelType', 'channelGroup', 'validDataFlag', 'isChannelActive']

    def __init__(self, json_channel_descriptor_data):
        """
        Initializes a ChannelDescriptor object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC EnhChlUtil record.
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            self.channel_id = str(json_channel_descriptor_data['channelID']).strip()
            if json_channel_descriptor_data['isChannelActive'] == 1:
               self.active = "Y"
            else:
               self.active = "N"
            self.shared = str(json_channel_descriptor_data['isChannelShared']).strip()
            self.type = str(json_channel_descriptor_data['channelType']).strip()
            self.channel_group = json_channel_descriptor_data['channelGroup']
            self.valid = json_channel_descriptor_data['validDataFlag']

            #logger.info(f'self.channel_id {self.channel_id}')
            #logger.info(f'self.active {self.active}')
            #logger.info(f'self.shared {self.shared}')
            #logger.info(f'self.channel_type {self.type}')
            #logger.info(f'self.channel_group {self.channel_group}')
            #logger.info(f'self.valid {self.valid}')

        except KeyError:
            msg = f'Unable to deserialize CDC ENH_CHL_UTILIZATION record for ChannelDescriptor. Missing fields: ' \
                  f'{get_missing_keys(json_channel_descriptor_data, self.required_fields)}. Data: {json_channel_descriptor_data}'
            logger.warning(msg)

            if 'isChannelActive' in get_missing_keys(json_channel_descriptor_data, self.required_fields):
                  logger.warning(f'Apply APAR PJ47254 for the tpfrtmc offline utility to resolve the missing field isChannelActive error.')
            raise MissingFieldError(msg)


class EnhChlUtil(object):

    """ Fields in the JSON that are required """
    required_fields = ['CdcChannelDescriptor', 'channelID', 'isChannelShared', 'channelType', 'channelGroup', 'validDataFlag',
                       'partitionUtilizationPercent', 'totalUtilizationPercent','busUtilizationPercent',
                       'partitionMBRead','totalMBRead', 'partitionMBWriteSPerec','totalMBWrite']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an EnhChlUtil object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC EnhChlUtil record.
        :param cdc_record_header: CdcRecordHeader object for this EnhChlUtil object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.channel_descriptor = ChannelDescriptor(json_cdc_record['CdcChannelDescriptor'])
            self.part_util = json_cdc_record['partitionUtilizationPercent']
            self.total_util = json_cdc_record['totalUtilizationPercent']
            self.bus_util = json_cdc_record['busUtilizationPercent']
            self.part_mb_read = json_cdc_record['partitionMBRead']
            self.total_mb_read = json_cdc_record['totalMBRead']
            self.part_mb_write = json_cdc_record['partitionMBWriteSPerec']
            self.total_mb_write = json_cdc_record['totalMBWrite']
        except KeyError:
            log_msg = f'Unable to deserialize CDC EnhChlUtil. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class EnhChlUtils(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an EnhChlUtils object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC EnhChlUtil record.
        :param cdc_record_header: CdcRecordHeader object for this EnhChlUtils object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [EnhChlUtil(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
