#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class IStream(object):

    required_fields = ['istreamOrdinal', 'processorUtilityPercentIstream', 'crossListIstream', 'readyListIstream',
                       'inputListIstream', 'deferredListIstream', 'activeEventControlBlockIstream', 'suspendListIstream',
                       'virtualFileAccessCountListIstream', 'routedEnteriesIstream', 'processorAdjUtilizationPercentIstream',
                       'psuedoUtilizationIstream', 'lowPriorityUtilizationIstream', 'iStreamStatus']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an IStream object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC ISTREAM record.
        :param cdc_record_header: CdcRecordHeader object for this IStream object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # iStreamStatus was initially not passed through by tpfrtmc, so we can't tell by the version whether it should
        # be in the JSON or not.
        if 'iStreamStatus' not in json_cdc_record:
            json_cdc_record['iStreamStatus'] = None

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        try:
            self.istream = json_cdc_record['istreamOrdinal']
            self.processor_utility_percent_istream = json_cdc_record['processorUtilityPercentIstream']
            self.cross_list_istream = json_cdc_record['crossListIstream']
            self.ready_list_istream = json_cdc_record['readyListIstream']
            self.input_list_istream = json_cdc_record['inputListIstream']
            self.deferred_list_istream = json_cdc_record['deferredListIstream']
            self.active_event_control_block_istream = json_cdc_record['activeEventControlBlockIstream']
            self.suspend_list_istream = json_cdc_record['suspendListIstream']
            self.virtual_file_access_count_list_istream = json_cdc_record['virtualFileAccessCountListIstream']
            self.routed_entries_istream = json_cdc_record['routedEnteriesIstream']
            self.processor_adj_utilization_percent_istream = json_cdc_record['processorAdjUtilizationPercentIstream']
            self.pseudo_utilization_istream = json_cdc_record['psuedoUtilizationIstream']
            self.low_priority_utilization_istream = json_cdc_record['lowPriorityUtilizationIstream']
            self.istream_status = json_cdc_record['iStreamStatus']
        except KeyError:
            log_msg = f'Unable to deserialize CDC ISTREAM. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)


class IStreams(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an IStreams object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC ISTREAM record.
        :param cdc_record_header: CdcRecordHeader object for this IStreams object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [IStream(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
