#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts LODIC data in JSON format to python object form.#
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging
import sys

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LodicResources(CdcRecord):

    required_fields = ["resourcePriorityClass1Ecb", "resourcePriorityClass2Ecb", "resourcePriorityClass3Ecb", "resourcePriorityClass4Ecb",
                       "resourcePriorityClass5Ecb", "resourcePriorityClass6Ecb", "resourcePriorityClass7Ecb", "resourcePriorityClass8Ecb",
                       "resourcePriorityClass9Ecb", "resourcePriorityClass10Ecb", "resourcePriorityClass11Ecb", "resourcePriorityClass12Ecb",
                       "resourcePriorityClass14Ecb", "resourcePriorityClass5Ecb",
                       "TimeSliceEcb", "TimeSlice",
                       "resourcePriorityOfEcbsClass1Name", "resourcePriorityOfEcbsClass2Name", "resourcePriorityOfEcbsClass3Name", "resourcePriorityOfEcbsClass4Name",
                       "resourcePriorityOfEcbsClass5Name", "resourcePriorityOfEcbsClass6Name", "resourcePriorityOfEcbsClass7Name", "resourcePriorityOfEcbsClass8Name",
                       "resourcePriorityOfEcbsClass9Name", "resourcePriorityOfEcbsClass10Name", "resourcePriorityOfEcbsClass11Name", "resourcePriorityOfEcbsClass12Name",
                       "resourcePriorityOfEcbsClass13Name", "resourcePriorityOfEcbsClass14Name"]

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a LodicResources object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC LODIC record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        json_cdc_record = json_cdc_record_array[0]

        # get these from json cdc record
        try:
            self.resourcePriorityClass1Ecb = json_cdc_record["resourcePriorityClass1Ecb"]
            self.resourcePriorityClass2Ecb = json_cdc_record["resourcePriorityClass2Ecb"]
            self.resourcePriorityClass3Ecb = json_cdc_record["resourcePriorityClass3Ecb"]
            self.resourcePriorityClass4Ecb = json_cdc_record["resourcePriorityClass4Ecb"]
            self.resourcePriorityClass5Ecb = json_cdc_record["resourcePriorityClass5Ecb"]
            self.resourcePriorityClass6Ecb = json_cdc_record["resourcePriorityClass6Ecb"]
            self.resourcePriorityClass7Ecb = json_cdc_record["resourcePriorityClass7Ecb"]
            self.resourcePriorityClass8Ecb = json_cdc_record["resourcePriorityClass8Ecb"]
            self.resourcePriorityClass9Ecb = json_cdc_record["resourcePriorityClass9Ecb"]
            self.resourcePriorityClass10Ecb = json_cdc_record["resourcePriorityClass10Ecb"]
            self.resourcePriorityClass11Ecb = json_cdc_record["resourcePriorityClass11Ecb"]
            self.resourcePriorityClass12Ecb = json_cdc_record["resourcePriorityClass12Ecb"]
            self.resourcePriorityClass13Ecb = json_cdc_record["resourcePriorityClass13Ecb"]
            self.resourcePriorityClass14Ecb = json_cdc_record["resourcePriorityClass14Ecb"]
            self.TimeSliceableEcbs = json_cdc_record["TimeSliceEcb"]
            self.TimeSlicedEcbs = json_cdc_record["TimeSlice"]
            self.resourcePriorityOfEcbsClass1Name = str(json_cdc_record["resourcePriorityOfEcbsClass1Name"]).strip()
            self.resourcePriorityOfEcbsClass2Name = str(json_cdc_record["resourcePriorityOfEcbsClass2Name"]).strip()
            self.resourcePriorityOfEcbsClass3Name = str(json_cdc_record["resourcePriorityOfEcbsClass3Name"]).strip()
            self.resourcePriorityOfEcbsClass4Name = str(json_cdc_record["resourcePriorityOfEcbsClass4Name"]).strip()
            self.resourcePriorityOfEcbsClass5Name = str(json_cdc_record["resourcePriorityOfEcbsClass5Name"]).strip()
            self.resourcePriorityOfEcbsClass6Name = str(json_cdc_record["resourcePriorityOfEcbsClass6Name"]).strip()
            self.resourcePriorityOfEcbsClass7Name = str(json_cdc_record["resourcePriorityOfEcbsClass7Name"]).strip()
            self.resourcePriorityOfEcbsClass8Name = str(json_cdc_record["resourcePriorityOfEcbsClass8Name"]).strip()
            self.resourcePriorityOfEcbsClass9Name = str(json_cdc_record["resourcePriorityOfEcbsClass9Name"]).strip()
            self.resourcePriorityOfEcbsClass10Name = str(json_cdc_record["resourcePriorityOfEcbsClass10Name"]).strip()
            self.resourcePriorityOfEcbsClass11Name = str(json_cdc_record["resourcePriorityOfEcbsClass11Name"]).strip()
            self.resourcePriorityOfEcbsClass12Name = str(json_cdc_record["resourcePriorityOfEcbsClass12Name"]).strip()
            self.resourcePriorityOfEcbsClass13Name = str(json_cdc_record["resourcePriorityOfEcbsClass13Name"]).strip()
            self.resourcePriorityOfEcbsClass14Name = str(json_cdc_record["resourcePriorityOfEcbsClass14Name"]).strip()

        except KeyError:
            log_message = f'Unable to deserialize LODIC. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

    # Implement the __eq__ so we can use the == operator in the lodic_resource_db.py file to compare LodicResources objects.
    def __eq__(self, comp):
        if not isinstance(comp, LodicResources):
            # don't attempt to compare against unrelated types
            return NotImplemented

        if sys.getsizeof(self) != sys.getsizeof(comp):
           return False

        # Note that the time slice fields are not included in the comparison.
        return self.resourcePriorityClass1Ecb == comp.resourcePriorityClass1Ecb and \
               self.resourcePriorityClass2Ecb == comp.resourcePriorityClass2Ecb and \
               self.resourcePriorityClass3Ecb == comp.resourcePriorityClass3Ecb and \
               self.resourcePriorityClass4Ecb == comp.resourcePriorityClass4Ecb and \
               self.resourcePriorityClass5Ecb == comp.resourcePriorityClass5Ecb and \
               self.resourcePriorityClass6Ecb == comp.resourcePriorityClass6Ecb and \
               self.resourcePriorityClass7Ecb == comp.resourcePriorityClass7Ecb and \
               self.resourcePriorityClass8Ecb == comp.resourcePriorityClass8Ecb and \
               self.resourcePriorityClass9Ecb == comp.resourcePriorityClass9Ecb and \
               self.resourcePriorityClass10Ecb == comp.resourcePriorityClass10Ecb and \
               self.resourcePriorityClass11Ecb == comp.resourcePriorityClass11Ecb and \
               self.resourcePriorityClass12Ecb == comp.resourcePriorityClass12Ecb and \
               self.resourcePriorityClass13Ecb == comp.resourcePriorityClass13Ecb and \
               self.resourcePriorityClass14Ecb == comp.resourcePriorityClass14Ecb and \
               self.resourcePriorityOfEcbsClass1Name == comp.resourcePriorityOfEcbsClass1Name and \
               self.resourcePriorityOfEcbsClass2Name == comp.resourcePriorityOfEcbsClass2Name and \
               self.resourcePriorityOfEcbsClass3Name == comp.resourcePriorityOfEcbsClass3Name and \
               self.resourcePriorityOfEcbsClass4Name == comp.resourcePriorityOfEcbsClass4Name and \
               self.resourcePriorityOfEcbsClass5Name == comp.resourcePriorityOfEcbsClass5Name and \
               self.resourcePriorityOfEcbsClass6Name == comp.resourcePriorityOfEcbsClass6Name and \
               self.resourcePriorityOfEcbsClass7Name == comp.resourcePriorityOfEcbsClass7Name and \
               self.resourcePriorityOfEcbsClass8Name == comp.resourcePriorityOfEcbsClass8Name and \
               self.resourcePriorityOfEcbsClass9Name == comp.resourcePriorityOfEcbsClass9Name and \
               self.resourcePriorityOfEcbsClass10Name == comp.resourcePriorityOfEcbsClass10Name and \
               self.resourcePriorityOfEcbsClass11Name == comp.resourcePriorityOfEcbsClass11Name and \
               self.resourcePriorityOfEcbsClass12Name == comp.resourcePriorityOfEcbsClass12Name and \
               self.resourcePriorityOfEcbsClass13Name == comp.resourcePriorityOfEcbsClass13Name and \
               self.resourcePriorityOfEcbsClass14Name == comp.resourcePriorityOfEcbsClass14Name
