#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts LODIC_UTIL_CLASS data in JSON format to python  #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LodicUtilization(object):

    """ Fields in the JSON that are required """
    required_fields = ['PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS', 'PRIORITY_UTILITY_className']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a LodicUtilization object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC LODIC_UTIL_CLASS record.
        :param cdc_record_header: CdcRecordHeader object for this LodicUtilization object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.PRIORITY_UTILITY_className = str(json_cdc_record["PRIORITY_UTILITY_className"]).strip()
            self.PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS = json_cdc_record["PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS"]
        except KeyError:
            log_msg = f'Unable to deserialize CDC LODIC_UTIL_CLASS. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)


class LodicUtilizations(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a LodicUtilizations object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC LODIC_UTIL_CLASS record.
        :param cdc_record_header: CdcRecordHeader object for this LodicUtilization object.
        :raise MissingFieldError: If the data is missing a required field.
        """
        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [LodicUtilization(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]

    # Implement the __eq__ so we can use the == operator in the lodic_utilization_db.py file to compare LodicUtilization objects.
    def __eq__(self, comp):
        if not isinstance(comp, LodicUtilizations):
            # don't attempt to compare against unrelated types
            return NotImplemented

        # Test each LodicUtilization class in the arrays.
        # If any array element is not equal then return not equal.
        selflen = len(self.cdc_record_array)
        complen = len(comp.cdc_record_array)

        if selflen != complen:
           # If the arrays are not equal length then the collector on TPF has started sending different data.
           # In which case we want to insert the data into the database.
           return False

        # Loop through the arrays.
        # If any items do not match then return False (not equal).  Else return True (equal).
        i = 0
        while i < selflen:
           equal = self.cdc_record_array[i].PRIORITY_UTILITY_className == comp.cdc_record_array[i].PRIORITY_UTILITY_className and \
                   self.cdc_record_array[i].PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS == comp.cdc_record_array[i].PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS
           i += 1
           if not equal:
              return False
        return True
