#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts LPAR_UTILIZATION data in JSON format to python  #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LPARInfo(object):

    """ Fields in the JSON that are required """
    required_fields = ['partitionName', 'partitionNumber', 'numberOfLogicalCpus', 'weight_Priority']

    def __init__(self, json_lpar_info_data):
        """
        Initializes a LPARInfo object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC LPARInfo record.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.json_lpar_info_raw_data = json_lpar_info_data

        try:
            self.part_name = str(json_lpar_info_data['partitionName']).strip()
            self.part_number = json_lpar_info_data['partitionNumber']
            self.recognized_cpus = json_lpar_info_data['numberOfLogicalCpus']
            self.weight = json_lpar_info_data['weight_Priority']

            #logger.info(f'self.part_name {self.part_name}')
            #logger.info(f'self.part_number {self.part_number}')
            #logger.info(f'self.recognized_cpus {self.recognized_cpus}')
            #logger.info(f'self.weight {self.weight}')

        except KeyError:
            msg = f'Unable to deserialize CDC LPAR_UTILIZATION record for LPARInfo. Missing fields: ' \
                  f'{get_missing_keys(json_lpar_info_data, self.required_fields)}. Data: {json_lpar_info_data}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class LPARUtil(object):

    """ Fields in the JSON that are required """
    required_fields = ['entitlePercentage', 'sharedCpus', 'type',
       'CdcLPARInfo', 'partitionName', 'partitionNumber', 'numberOfLogicalCpus', 'weight_Priority',
       'utilization', 'utilVsCap','cecUsePercentage']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an LPARUtil object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC LPARUtil record.
        :param cdc_record_header: CdcRecordHeader object for this LPARUtil object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.shared_cpus = json_cdc_record['sharedCpus']
            self.type = json_cdc_record['type']
            self.lpar_info = LPARInfo(json_cdc_record['CdcLPARInfo'])
            self.util = json_cdc_record['utilization']
            self.util_vs_cap = json_cdc_record['utilVsCap']
            self.machine = json_cdc_record['cecUsePercentage']
            self.entitle = json_cdc_record['entitlePercentage']
        except KeyError:
            log_msg = f'Unable to deserialize CDC LPARUtil. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class LPARUtils(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an LPARUtils object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC LPARUtil record.
        :param cdc_record_header: CdcRecordHeader object for this LPARUtils object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [LPARUtil(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
