#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts MQ_CHL_DETAIL data in JSON format to python       #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class MQChannel(object):

    """ Fields in the JSON that are required """
    required_fields = ['cdcMqChannel', 'channelType', 'queueManagerName', 'subSystemName',
     'channelMessages', 'channelBytesReceived','channelBytesSent','channelBytesHandled',
     'channelBatchSize', 'channelActiveInstances', 'channelLastBatchSize', 'channelMaxInstances']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQChannel object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC MQ Channel record.
        :param cdc_record_header: CdcRecordHeader object for this MQChannel object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subSystemName"]).strip()
            self.queue_manager = str(json_cdc_record["queueManagerName"]).strip()
            self.channel_name = str(json_cdc_record["cdcMqChannel"]).strip()
            self.channel_type = str(json_cdc_record["channelType"]).strip()
            self.msgs = json_cdc_record["channelMessages"]
            self.bytes_recvd = json_cdc_record["channelBytesReceived"]
            self.bytes_sent = json_cdc_record["channelBytesSent"]
            self.bytes_handled = json_cdc_record["channelBytesHandled"]
            self.batch_size = json_cdc_record["channelBatchSize"]
            self.active_instances = json_cdc_record["channelActiveInstances"]
            self.last_batch_size = json_cdc_record["channelLastBatchSize"]
            self.instances_highwater = json_cdc_record["channelMaxInstances"]

        except KeyError:
            log_msg = f'Unable to deserialize CDC MQ Channel. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class MQChannels(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQChannels object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC MQ Channel record.
        :param cdc_record_header: CdcRecordHeader object for this MQChannels object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [MQChannel(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
