#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts MQ_Q_DETAIL data in JSON format to python       #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class MQQueue(object):

    """ Fields in the JSON that are required """
    required_fields = ['queueName', 'queueType', 'queueManagerName', 'subsystemName',
     'queueOpenCount', 'queueCloseCount','queueSweepCount',
     'queueAddMessageLength', 'queueMessageCount', 'queueMessageLength']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQQueue object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC MQ Queue record.
        :param cdc_record_header: CdcRecordHeader object for this MQQueue object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subsystemName"]).strip()
            self.queue_manager = str(json_cdc_record["queueManagerName"]).strip()
            self.queue_name = str(json_cdc_record["queueName"]).strip()
            self.queue_type = str(json_cdc_record["queueType"]).strip()
            self.opens = json_cdc_record["queueOpenCount"]
            self.closes = json_cdc_record["queueCloseCount"]
            self.sweeps = json_cdc_record["queueSweepCount"]
            self.msgs_persistent = json_cdc_record["queueAddCount"]
            self.msg_length_persistent = json_cdc_record["queueAddMessageLength"]
            self.msgs_nonpersistent = json_cdc_record["queueMessageCount"]
            self.msg_length_nonpersistent = json_cdc_record["queueMessageLength"]

        except KeyError:
            log_msg = f'Unable to deserialize CDC MQ Queue. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class MQQueues(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQQueues object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC MQ Queue record.
        :param cdc_record_header: CdcRecordHeader object for this MQQueues object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [MQQueue(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
