#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts MQ_SUMMARY data in JSON format to python object #
#  form.                                                              #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class MQSummary(object):

    """ Fields in the JSON that are required """
    required_fields = ['queueManagerName', 'subSystemName', 'numQueue', 'numChannels', 'numQueuesLocal', 'numQueueXmitX',
     'numQueueRemoteR', 'queueOpenCount', 'queueCloseCount', 'queueSweepCount', 'queueAddCountPersistent', 'queueAddMessageLengthPersistent',
     'queueAddMessageCountNonPersistent', 'queueAddMessageLengthNonPersistent',
     'numChannelReceiverR', 'numChannelsSendersS', 'numChannelsServerV', 'channelMessages',
     'channelBytesReceived', 'channelBytesSent', 'channelBytesHandled',
     'channelBatchSize', 'channelActiveInstances', 'channelLastBatchSize', 'channelMaxInstances'
     ]

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQSummary object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC MQ Summary record.
        :param cdc_record_header: CdcRecordHeader object for this MQSummary object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.queue_manager = str(json_cdc_record["queueManagerName"]).strip()
            self.subsystem_name = str(json_cdc_record["subSystemName"]).strip()
            self.queues = json_cdc_record["numQueue"]
            self.local_queues = json_cdc_record["numQueuesLocal"]
            self.transmit_queues = json_cdc_record["numQueueXmitX"]
            self.remote_queues = json_cdc_record["numQueueRemoteR"]
            self.queue_opens = json_cdc_record["queueOpenCount"]
            self.queue_closes = json_cdc_record["queueCloseCount"]
            self.queue_sweeps = json_cdc_record["queueSweepCount"]
            self.queue_msgs_persistent = json_cdc_record["queueAddCountPersistent"]
            self.queue_msg_length_persistent = json_cdc_record["queueAddMessageLengthPersistent"]
            self.queue_msgs_nonpersistent = json_cdc_record["queueAddMessageCountNonPersistent"]
            self.queue_msg_length_nonpersistent = json_cdc_record["queueAddMessageLengthNonPersistent"]
            self.channels = json_cdc_record["numChannels"]
            self.receiver_channels = json_cdc_record["numChannelReceiverR"]
            self.sender_channels = json_cdc_record["numChannelsSendersS"]
            self.server_channels = json_cdc_record["numChannelsServerV"]
            self.channel_msgs = json_cdc_record["channelMessages"]
            self.channel_bytes_recvd = json_cdc_record["channelBytesReceived"]
            self.channel_bytes_sent = json_cdc_record["channelBytesSent"]
            self.channel_bytes_handled = json_cdc_record["channelBytesHandled"]
            self.channel_batch_size = json_cdc_record["channelBatchSize"]
            self.channel_active_instances = json_cdc_record["channelActiveInstances"]
            self.channel_last_batch_size = json_cdc_record["channelLastBatchSize"]
            self.channel_instances_highwater = json_cdc_record["channelMaxInstances"]
        except KeyError:
            log_msg = f'Unable to deserialize CDC MQ Summary. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class MQSummarys(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an MQSummarys object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC MQ Summary record.
        :param cdc_record_header: CdcRecordHeader object for this MQSummary object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [MQSummary(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
