#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts REMOTE_DATA_STORE data in JSON format to python #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class RemoteDataStore(object):

    """ Fields in the JSON that are required """
    required_fields = ['subsystemName', 'rdsName', 'rdsMode',
      'readerActiveECBs', 'readerHWtr', 'readerMax', 'writerActiveECBs', 'writerHWtr', 'writerMax',
      'readCount', 'createCount', 'updateCount', 'deleteCount', 'totalOpsCount',
      'readRejectAtMax', 'timeoutCount', 'TPFerror', 'serverErrorMongo']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an RemoteDataStore object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC RemoteDataStore record.
        :param cdc_record_header: CdcRecordHeader object for this RemoteDataStore object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subsystemName"]).strip()
            self.rds_name = str(json_cdc_record["rdsName"]).strip()
            self.rds_mode = str(json_cdc_record["rdsMode"]).strip()
            self.reader_active_ecbs = json_cdc_record["readerActiveECBs"]
            self.reader_highwater = json_cdc_record["readerHWtr"]
            self.reader_max = json_cdc_record["readerMax"]
            self.writer_active_ecbs = json_cdc_record["writerActiveECBs"]
            self.writer_highwater = json_cdc_record["writerHWtr"]
            self.writer_max = json_cdc_record["writerMax"]
            self.read_count = json_cdc_record["readCount"]
            self.create_count = json_cdc_record["createCount"]
            self.update_count = json_cdc_record["updateCount"]
            self.delete_count = json_cdc_record["deleteCount"]
            self.total_ops_count = json_cdc_record["totalOpsCount"]
            self.read_reject_at_max = json_cdc_record["readRejectAtMax"]
            self.timeout_count = json_cdc_record["timeoutCount"]
            self.TPFerror = json_cdc_record["TPFerror"]
            self.server_error_mongo = json_cdc_record["serverErrorMongo"]
        except KeyError:
            log_msg = f'Unable to deserialize CDC RemoteDataStore. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class RemoteDataStores(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an RemoteDataStore object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC RemoteDataStore record.
        :param cdc_record_header: CdcRecordHeader object for this RemoteDataStores object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [RemoteDataStore(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
