#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class ServiceApp(object):

    required_fields = ['subSystemName', 'opId', 'jamName', 'internalOperationCount', 'externalOperationCount',
                       'totalOperationCount', 'timeoutCount', 'ifaceErrorCount', 'srvcErrorCount', 'unrecErrorCount']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a ServiceApp object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC SERVICE_APP record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        try:
            self.subsystem_name = str(json_cdc_record['subSystemName']).strip()
            self.operation = str(json_cdc_record['opId']).strip()
            self.jam_name = str(json_cdc_record['jamName']).strip()
            self.internal_operation_count = json_cdc_record['internalOperationCount']
            self.external_operation_count = json_cdc_record['externalOperationCount']
            self.total_operation_count = json_cdc_record['totalOperationCount']
            self.timeout_count = json_cdc_record['timeoutCount']
            self.iface_error_count = json_cdc_record['ifaceErrorCount']
            self.srvc_error_count = json_cdc_record['srvcErrorCount']
            self.unrec_error_count = json_cdc_record['unrecErrorCount']
        except KeyError:
            msg = f'Unable to deserialize CDC SERVICE_APP record. Missing fields: ' \
                  f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class ServiceApps(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a ServiceApps object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC SERVICE_APP record.
        :param cdc_record_header: CdcRecordHeader object for this SERVICE_APP object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [ServiceApp(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
