#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts STATIC_SYS_INFO data in JSON format to          #
#  python object form.                                                #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class Static(CdcRecord):

    required_fields = ["cpuSerNbr", "cpuMachType", "tpfRelease", "tpfPutLevel",
                       "tpfCplxName", "tpfEntrName", "cpuSerNbr",
                       "ssCount", "ssuCount", "usableIstreams"]

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a Static object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC STATIC_SYS_INFO record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        json_cdc_record = json_cdc_record_array

        # This is for backward compatibility as this constructor used to take a dict, not a list
        # The cdcRecordArray for SYSTEM_LISTS always has one item, so the output doesn't change
        if isinstance(json_cdc_record_array, list) and len(json_cdc_record_array) > 0:
            json_cdc_record = json_cdc_record_array[0]

        # get these from json cdc record
        try:
            self.cpu_serial = str(json_cdc_record["cpuSerNbr"]).strip()
            self.cpu_machine_type = str(json_cdc_record["cpuMachType"]).strip()
            self.tpf_release = str(json_cdc_record["tpfRelease"]).strip()
            self.tpf_prod_update_level = str(json_cdc_record["tpfPutLevel"]).strip()
            self.tpf_complex_name = str(json_cdc_record["tpfCplxName"]).strip()
            self.tpf_enterprise_name = str(json_cdc_record["tpfEntrName"]).strip()
            self.cpu_Id = str(json_cdc_record["cpuId"]).strip()
            self.subsystem_count = json_cdc_record["ssCount"]
            self.subsystem_user_count = json_cdc_record["ssuCount"]
            self.usable_istreams = json_cdc_record["usableIstreams"]

        except KeyError:
            log_message = f'Unable to deserialize CDC STATIC_SYS_INFO. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

    # Implement the __eq__ so we can use the == operator in the static_db.py file to compare Static objects.
    def __eq__(self, comp):
        if not isinstance(comp, Static):
            # don't attempt to compare against unrelated types
            return NotImplemented

        return self.cpu_serial == comp.cpu_serial and \
               self.cpu_machine_type == comp.cpu_machine_type and \
               self.tpf_release == comp.tpf_release and \
               self.tpf_complex_name == comp.tpf_complex_name and \
               self.tpf_enterprise_name == comp.tpf_enterprise_name and \
               self.cpu_Id == comp.cpu_Id and \
               self.subsystem_count == comp.subsystem_count and \
               self.subsystem_user_count == comp.subsystem_user_count and \
               self.usable_istreams == comp.usable_istreams