#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts SUB_SYS_MSGS data in JSON format to python      #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class Subsystem(object):

    """ Fields in the JSON that are required """
    required_fields = ['subSystemName', 'highSpeedSubsystemUserMessages', 'lowSpeedSubsystemUserMessages', \
                       'routedEntriesSubsystemUserMessages', 'createdEntriesSubsystemUserMessages', \
                       'ecbExitSubsystemUserMessages', 'utilitySwitches']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an Subsystem object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC Subsystem record.
        :param cdc_record_header: CdcRecordHeader object for this Subsystem object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subSystemName"]).strip()
            self.high_spd_msgs = json_cdc_record["highSpeedSubsystemUserMessages"]
            self.routed_entries = json_cdc_record["routedEntriesSubsystemUserMessages"]
            self.created_entries = json_cdc_record["createdEntriesSubsystemUserMessages"]
            self.ecb_exits = json_cdc_record["ecbExitSubsystemUserMessages"]
            self.utility_switches = json_cdc_record["utilitySwitches"]
        except KeyError:
            log_msg = f'Unable to deserialize CDC Subsystems. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)


class Subsystems(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a Subsystem object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC Subsystem record.
        :param cdc_record_header: CdcRecordHeader object for this Subsystem object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [Subsystem(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
