#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts SYSTEM_BLOCKS data in JSON format to python     #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class SystemBlocks(CdcRecord):

    required_fields = ["inOutControlBlocksInuse", "inOutControlBlocksAllocated", "ecbBlocksInuse", "ecbBlocksAllocated",
                       "systemWorkBlocksInuse", "systemWorkBlocksAllocated", "commonBlocksInuse",
                       "commonBlocksAllocated", "frameBlocksInuse", "frameBlocksAllocated", "frame1MbBlocksInuse",
                       "extendedSystemWorkBlocksInuse", "extendedSystemWorkBlocksAllocated", "rlogTracksInUse",
                       "rlogTracksAlloc"]

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a SystemBlocks object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC SYSTEM_BLOCKS record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        json_cdc_record = json_cdc_record_array

        # This is for backward compatibility as this constructor used to take a dict, not a list
        # The cdcRecordArray for SYSTEM_BLOCKS always has one item, so the output doesn't change
        if isinstance(json_cdc_record_array, list) and len(json_cdc_record_array) > 0:
            json_cdc_record = json_cdc_record_array[0]

        # get these from json cdc record
        try:
            self.iob_inuse = json_cdc_record["inOutControlBlocksInuse"]
            self.iob_allocated = json_cdc_record["inOutControlBlocksAllocated"]
            self.ecb_inuse = json_cdc_record["ecbBlocksInuse"]
            self.ecb_allocated = json_cdc_record["ecbBlocksAllocated"]
            self.swb_inuse = json_cdc_record["systemWorkBlocksInuse"]
            self.swb_allocated = json_cdc_record["systemWorkBlocksAllocated"]
            self.common_inuse = json_cdc_record["commonBlocksInuse"]
            self.common_allocated = json_cdc_record["commonBlocksAllocated"]
            self.frame_inuse = json_cdc_record["frameBlocksInuse"]
            self.frame_allocated = json_cdc_record["frameBlocksAllocated"]
            self.frame1mb_inuse = json_cdc_record["frame1MbBlocksInuse"]
            self.frame1mb_allocated = json_cdc_record["frame1MbBlocksAllocated"]
            self.xwb_inuse = json_cdc_record["extendedSystemWorkBlocksInuse"]
            self.xwb_allocated = json_cdc_record["extendedSystemWorkBlocksAllocated"]
            self.rlog_inuse = json_cdc_record["rlogTracksInUse"]
            self.rlog_allocated = json_cdc_record["rlogTracksAlloc"]

        except KeyError:
            log_message = f'Unable to deserialize CDC SYSTEM_BLOCKS. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
