#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts SYSTEM_LISTS data in JSON format to python      #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SystemLists(CdcRecord):

    required_fields = ["entriesOnCrossList", "entriesOnReadyList", "entriesOnInputList", "entriesOnDeferredList",
                       "numberOfActiveEcbs", "entriesOnSuspendList", "entriesOnVitualFileAccessCountList",
                       "sharedReadyList", "sharedInputList", "sharedDeferredList", "accumCount",
                       "lowPriorityEcbs"]

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a SystemLists object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC SYSTEM_LISTS record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        json_cdc_record = json_cdc_record_array

        # This is for backward compatibility as this constructor used to take a dict, not a list
        # The cdcRecordArray for SYSTEM_LISTS always has one item, so the output doesn't change
        if isinstance(json_cdc_record_array, list) and len(json_cdc_record_array) > 0:
            json_cdc_record = json_cdc_record_array[0]

        # get these from json cdc record
        try:
            self.cross_list = json_cdc_record["entriesOnCrossList"]
            self.ready_list = json_cdc_record["entriesOnReadyList"]
            self.input_list = json_cdc_record["entriesOnInputList"]
            self.deferred_list = json_cdc_record["entriesOnDeferredList"]
            self.active_ecbs = json_cdc_record["numberOfActiveEcbs"]
            self.suspend_list = json_cdc_record["entriesOnSuspendList"]
            self.vct_list = json_cdc_record["entriesOnVitualFileAccessCountList"]
            self.low_priority_ecbs = json_cdc_record["lowPriorityEcbs"]

        except KeyError:
            log_message = f'Unable to deserialize CDC SYSTEM_LISTS. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)