#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts SYSTEM_MESSAGES data in JSON format to python   #
#  bject form.                                                        #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SystemMessages(CdcRecord):

    required_fields = ["timeDiffInMilliseconds", "highSpeedMessages", "lowSpeedMessages", "routedEntries",
                       "createdEntries", "snaScpInputMessages", "processorUtilization", "numberOfIStreams",
                       "usableIStreams", "tcpIpMessagesFromNsd", "commits", "cfRequests", "shutdown1", "shutdown2",
                       "traces", "gpProcessorUtilization", "gpMaxCapForLparUtilization", "gpNumOfEngines", "iStreamCap",
                       "iplboostoptions"]

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a SystemMessages object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC SYSTEM_MESSAGES record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        json_cdc_record = json_cdc_record_array

        # This is for backward compatibility as this constructor used to take a dict, not a list
        # The cdcRecordArray for SYSTEM_MESSAGES always has one item, so the output doesn't change
        if isinstance(json_cdc_record_array, list) and len(json_cdc_record_array) > 0:
            json_cdc_record = json_cdc_record_array[0]

        # get these from json cdc record
        try:
            self.time_diff_in_milliseconds = json_cdc_record["timeDiffInMilliseconds"]
            self.high_spd_msgs = json_cdc_record["highSpeedMessages"]
            self.routed_entries = json_cdc_record["routedEntries"]
            self.created_entries = json_cdc_record["createdEntries"]
            self.processor_utilization = json_cdc_record["processorUtilization"]
            self.number_of_istreams = json_cdc_record["numberOfIStreams"]
            self.usable_istreams = json_cdc_record["usableIStreams"]
            self.tcpip_msgs = json_cdc_record["tcpIpMessagesFromNsd"]
            self.commits = json_cdc_record["commits"]
            self.shutdown1 = json_cdc_record["shutdown1"]
            self.shutdown2 = json_cdc_record["shutdown2"]
            self.traces = json_cdc_record["traces"]
            self.gp_processor_utilization = json_cdc_record["gpProcessorUtilization"]
            self.gp_max_cap_for_lpar_utilization = json_cdc_record["gpMaxCapForLparUtilization"]
            self.gp_num_of_engines = json_cdc_record["gpNumOfEngines"]
            self.istream_cap = json_cdc_record["iStreamCap"]
            self.ipl_boost_options = json_cdc_record["iplboostoptions"]

        except KeyError:
            log_message = f'Unable to deserialize CDC SYSTEM_MESSAGES. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
