#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts TAPE data in JSON format to python object form. #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class Tape(object):

    """ Fields in the JSON that are required """
    required_fields = ['subsystemName', 'tapeName', 'bytes', 'writesPerSec', 'queueLength', 'deviceAddress']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an Tape object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC Tape record.
        :param cdc_record_header: CdcRecordHeader object for this Tape object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subsystemName"]).strip()
            self.tape_name = str(json_cdc_record["tapeName"]).strip()
            self.bytes = json_cdc_record["bytes"]
            self.writes = json_cdc_record["writesPerSec"]
            self.queue_length = json_cdc_record["queueLength"]
            self.device_address = str(json_cdc_record["deviceAddress"]).strip()
        except KeyError:
            log_msg = f'Unable to deserialize CDC Tape. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class Tapes(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an Tapes object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC Tape record.
        :param cdc_record_header: CdcRecordHeader object for this Tapes object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [Tape(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
