#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts TCP_IP data in JSON format to python object form#
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class TcpIpApp(object):

    required_fields = ["applicationName", "port", "weight", "packetsSent", "packetsReceived", "bytesSent",
                       "bytesReceived", "messagesSent", "messagesReceived"]

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):

        """
        Initializes a TcpIp object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC TCP_IP record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        # get these from json cdc record
        try:
            self.application_name = str(json_cdc_record['applicationName']).strip()
            self.port = json_cdc_record["port"]
            self.weight = json_cdc_record["weight"]
            self.packets_sent = json_cdc_record["packetsSent"]
            self.packets_received = json_cdc_record["packetsReceived"]
            self.bytes_sent = json_cdc_record["bytesSent"]
            self.bytes_received = json_cdc_record["bytesReceived"]
            self.messages_sent = json_cdc_record["messagesSent"]
            self.messages_received = json_cdc_record["messagesReceived"]

        except KeyError:
            log_message = f'Unable to deserialize CDC tcp_ip message. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)


class TcpIp(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a TcpIp object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC TCP_IP record.
        :param cdc_record_header: CdcRecordHeader object for this TCP_IP object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [TcpIpApp(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
