#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts TPFDF data in JSON format to python object      #
#  form.                                                              #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class TPFDFFile(object):

    """ Fields in the JSON that are required """
    required_fields = ['subsystemName', 'fileID', 'fileIDEbcdic', 'fileNameAlias', 'deltaCounters']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes a TPFDFFile object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC TPFDF record.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_raw_json = json_cdc_record

        try:

            self.subsystem_name = str(json_cdc_record['subsystemName']).strip()
            self.fileId = str(json_cdc_record['fileID']).strip()
            self.fileIdEbcdic = str(json_cdc_record['fileIDEbcdic']).strip()
            self.fileNameAlias = str(json_cdc_record['fileNameAlias']).strip()
            #logger.info(f'TPFDF subsystem {self.subsystem_name}')
            #logger.info(f'TPFDF fileId {self.fileId}')
            #logger.info(f'TPFDF fileIdEbcdic {self.fileIdEbcdic}')
            #logger.info(f'TPFDF fileNameAlias {self.fileNameAlias}')

            index = 0
            #the order of the TPFDF counters in the deltaCounter List is defined by base/include/tpf/i_idcrct.h
            for deltaCounter in json_cdc_record['deltaCounters']:
               self.setCounterDeltaAtIndex(index, deltaCounter)
               index += 1

            #logger.info(f'TPFDF dbadd {self.dbadd}')

        except KeyError:
            msg = f'Unable to deserialize CDC TPFDF record. Missing fields: ' \
                  f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)

    def setCounterDeltaAtIndex(self, index, deltaCounter):
        """
        Note: Index 28 is ibmreserved and not included in this function.
        """
        if(index == 0):
            self.dbadd = deltaCounter
        elif(index == 1):
            self.dbadr = deltaCounter
        elif(index == 2):
            self.dbckp = deltaCounter
        elif(index == 3):
            self.dbcls = deltaCounter
        elif(index == 4):
            self.dbcpy = deltaCounter
        elif(index == 5):
            self.dbcre = deltaCounter
        elif(index == 6):
            self.dbdel = deltaCounter
        elif(index == 7):
            self.dbdix = deltaCounter
        elif(index == 8):
            self.dbdsp = deltaCounter
        elif(index == 9):
            self.dbidx = deltaCounter
        elif(index == 10):
            self.dbkey = deltaCounter
        elif(index == 11):
            self.dbmod = deltaCounter
        elif(index == 12):
            self.dbmrg = deltaCounter
        elif(index == 13):
            self.dbopn = deltaCounter
        elif(index == 14):
            self.dbred = deltaCounter
        elif(index == 15):
            self.dbrep = deltaCounter
        elif(index == 16):
            self.dbret = deltaCounter
        elif(index == 17):
            self.dbrst = deltaCounter
        elif(index == 18):
            self.dbspa = deltaCounter
        elif(index == 19):
            self.dbsrt = deltaCounter
        elif(index == 20):
            self.dbtld = deltaCounter
        elif(index == 21):
           self.dbtlg = deltaCounter
        elif(index == 22):
            self.dbtrd = deltaCounter
        elif(index == 23):
            self.dbuky = deltaCounter
        elif(index == 24):
            self.dbhold = deltaCounter
        elif(index == 25):
            self.dbuhld = deltaCounter
        elif(index == 26):
            self.filnc = deltaCounter
        elif(index == 27):
            self.filec = deltaCounter
        elif(index == 29):
            self.cfile = deltaCounter
        elif(index == 30):
            self.pfile = deltaCounter
        elif(index == 31):
            self.twrtc = deltaCounter
        elif(index == 32):
            self.getfc = deltaCounter
        elif(index == 33):
            self.relfc = deltaCounter
        elif(index == 34):
            self.detac = deltaCounter
        elif(index == 35):
            self.attac = deltaCounter
        elif(index == 36):
            self.fsysw = deltaCounter
        elif(index == 37):
            self.getcc = deltaCounter
        elif(index == 38):
            self.relcc = deltaCounter
        elif(index == 39):
            self.cfind = deltaCounter
        elif(index == 40):
            self.pfind = deltaCounter
        elif(index == 41):
            self.btfil = deltaCounter
        elif(index == 42):
            self.btfnd = deltaCounter
        elif(index == 43):
            self.lrfil = deltaCounter
        elif(index == 44):
            self.lrfnd = deltaCounter
        elif(index == 45):
            self.opn_hd = deltaCounter
        elif(index == 46):
            self.opn_nhd = deltaCounter
        elif(index == 47):
            self.opn_hnd = deltaCounter
        elif(index == 48):
            self.opn_nhnd = deltaCounter
        elif(index == 49):
            self.cls_rel = deltaCounter
        elif(index == 50):
            self.cls_norel = deltaCounter
        elif(index == 51):
            self.cls_reuse = deltaCounter
        elif(index == 52):
            self.cls_opn = deltaCounter
        elif(index == 53):
            self.prefp = deltaCounter
        elif(index == 54):
            self.prefpw = deltaCounter
        elif(index == 55):
            self.cache_prtl_iocnt = deltaCounter
        elif(index == 56):
            self.cache_prtl_ioreq = deltaCounter
        elif(index == 57):
            self.cache_read_hit = deltaCounter
        elif(index == 58):
            self.cache_read_miss = deltaCounter
        elif(index == 59):
            self.blk_read_depth = deltaCounter
        elif(index == 60):
            self.rsf_create = deltaCounter
        elif(index == 61):
            self.rsf_read = deltaCounter
        elif(index == 62):
            self.rsf_update = deltaCounter
        elif(index == 63):
            self.rsf_delete = deltaCounter
        return

class TPFDFFiles(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes a TPFDFFiles object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC TPFDF record.
        :param cdc_record_header: CdcRecordHeader object for this TPFDFFiles object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [TPFDFFile(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]

