#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file is used to convert TRAD_DB_ENCRYPTION data in JSON       #
#  format to object form.                                             #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class Trad_DB_Encryption(object):

    """ Fields in the JSON that are required and match """
    required_fields = ['numEncryption', 'numEncryptionComp', 'numDecryption', 'numDecryptionExpansions',
       'numErrorEncryption', 'numErrorCompression', 'subsystemName']

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an Trad_DB_Encryption object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC TradDbEncryption record.
        :param cdc_record_header: CdcRecordHeader object for this Trad_DB_Encryption object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.numEncryption = json_cdc_record['numEncryption']
            self.numEncryptionComp= json_cdc_record['numEncryptionComp']
            self.numDecryption = json_cdc_record['numDecryption']
            self.numDecryptionExpansions = json_cdc_record['numDecryptionExpansions']
            self.numErrorEncryption = json_cdc_record['numErrorEncryption']
            self.numErrorCompression = json_cdc_record['numErrorCompression']
            self.subsystemName = str(json_cdc_record['subsystemName']).strip()
        except KeyError:
            log_msg = f'Unable to deserialize CDC TradDbEncryption. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)


class Trad_DB_Encryptions(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an Trad_DB_Encryptions object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC Trad_DB_Encryption record.
        :param cdc_record_header: CdcRecordHeader object for this Trad_DB_Encryptions object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [Trad_DB_Encryption(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
