#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file converts VFA_SUBSYSTEM data in JSON format to python     #
#  object form.                                                       #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')

class VFA(object):

    """ Fields in the JSON that are required """
    required_fields = ['subSystemName', \
                       'vfaDataReads', 'vfaFinds', 'vfaCandidateFiles', 'vfaNonCandidateFls', "vfaForceFiles", \
                       "vfaFileImmediate", "vfaBufferUnavailable", "vfaSLockHits", "vfaXLockHits", \
                       "vfaSyncMisses", "vfaSyncContent", "vfaCachedRhtHts", "vfatSyncAgeOuts", \
                       "vfaDispense4K", "vfaRhtContent"]

    def __init__(self, json_cdc_record, cdc_record_header: CdcRecordHeader):
        """
        Initializes an VFA object.

        :param json_cdc_record: dictionary of a single object in the cdcRecordArray of a CDC VFA record.
        :param cdc_record_header: CdcRecordHeader object for this VFA object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.cdc_record_raw_json = json_cdc_record
        self.cdc_record_header = cdc_record_header

        try:
            self.subsystem_name = str(json_cdc_record["subSystemName"]).strip()
            self.data_reads = json_cdc_record["vfaDataReads"]
            self.finds = json_cdc_record["vfaFinds"]
            self.candidate_files = json_cdc_record["vfaCandidateFiles"]
            self.noncandidate_files = json_cdc_record["vfaNonCandidateFls"]
            self.force_files = json_cdc_record["vfaForceFiles"]
            self.file_immediate = json_cdc_record["vfaFileImmediate"]
            self.buffer_unavailable = json_cdc_record["vfaBufferUnavailable"]
            self.SLock_hits = json_cdc_record["vfaSLockHits"]
            self.XLock_hits = json_cdc_record["vfaXLockHits"]
            self.sync_misses = json_cdc_record["vfaSyncMisses"]
            self.sync_content = json_cdc_record["vfaSyncContent"]
            self.cached_rht_hts = json_cdc_record["vfaCachedRhtHts"]
            self.sync_age_outs = json_cdc_record["vfatSyncAgeOuts"]
            self.dispense_4K = json_cdc_record["vfaDispense4K"]
            self.rht_content = json_cdc_record["vfaRhtContent"]
        except KeyError:
            log_msg = f'Unable to deserialize CDC VFA. Missing fields: ' \
                      f'{get_missing_keys(json_cdc_record, self.required_fields)}. Data: {json_cdc_record}'
            logger.warning(log_msg)
            raise MissingFieldError(log_msg)

class VFAs(CdcRecord):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):
        """
        Initializes an VFAs object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC VFA record.
        :param cdc_record_header: CdcRecordHeader object for this VFAs object.
        :raise MissingFieldError: If the data is missing a required field.
        """
        # sets self.cdc_record_header and self.cdc_record_array_raw_json
        super().__init__(json_cdc_record_array, cdc_record_header)

        self.cdc_record_array = [VFA(json_cdc_record, cdc_record_header) for json_cdc_record in json_cdc_record_array]
