#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from typing import Dict, Type

from exceptions.json_err import MissingFieldError
from tpf_cdc.cdc_record_types.common_deploy import CommonDeployFiles
from tpf_cdc.cdc_record_types.istreams import IStreams
from tpf_cdc.cdc_record_types.service_app import ServiceApps
from tpf_cdc.cdc_record_types.service_jam import ServiceJams
from tpf_cdc.cdc_record_types.dasd_queue import DASDQueues
from tpf_cdc.cdc_record_types.dasd_service_time import DASDServiceTimes
from tpf_cdc.cdc_record_types.dasd_pools import DASDPools
from tpf_cdc.cdc_record_types.tape import Tapes
from tpf_cdc.cdc_record_types.vfa import VFAs
from tpf_cdc.cdc_record_types.system_blocks import SystemBlocks
from tpf_cdc.cdc_record_types.system_lists import SystemLists
from tpf_cdc.cdc_record_types.system_messages import SystemMessages
from tpf_cdc.cdc_record_types.subsystem_users import SubsystemUsers
from tpf_cdc.cdc_record_types.subsystems import Subsystems
from tpf_cdc.cdc_record_types.tcp_ip import TcpIp
from tpf_cdc.cdc_record_types.static import Static
from tpf_cdc.cdc_record_types.mq_summary import MQSummarys
from tpf_cdc.cdc_record_types.mq_queues import MQQueues
from tpf_cdc.cdc_record_types.mq_channels import MQChannels
from tpf_cdc.cdc_record_types.remote_data_store import RemoteDataStores
from tpf_cdc.cdc_record_types.enh_chl_util import EnhChlUtils
from tpf_cdc.cdc_record_types.lpar_util import LPARUtils
from tpf_cdc.cdc_record_types.mpif import MPIFs
from tpf_cdc.cdc_record_types.lodic_resource import LodicResources
from tpf_cdc.cdc_record_types.lodic_utilization import LodicUtilizations
from tpf_cdc.cdc_record_types.dasd_metrics import DASDMetrics
from tpf_cdc.cdc_record_types.trad_db_encrypt import Trad_DB_Encryptions
from tpf_cdc.cdc_record_types.tpfdf import TPFDFFiles
from tpf_cdc.tpf_cdc_record import CdcRecord
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class CdcBody(object):

    required_fields_rlist = ["recordList"]
    required_fields_rheader = ["cdcRecordHeader"]
    required_fields_rarray = ["cdcRecordArray"]

    cdc_data_types: Dict[str, Type[CdcRecord]] = {
        'SYSTEM_MESSAGES': SystemMessages,
        'SYSTEM_BLOCKS': SystemBlocks,
        'TCP_IP': TcpIp,
        'ISTREAM': IStreams,
        'COMMON_DEPLOY_FILES': CommonDeployFiles,
        'SERVICE_APP': ServiceApps,
        'DASD_DEVICES': DASDQueues,
        'DASD_SERVICE_TIME': DASDServiceTimes,
        'TAPE': Tapes,
        'VFA_SUBSYSTEM': VFAs,
        'SYSTEM_LISTS': SystemLists,
        'STATIC_SYS_INFO': Static,
        'SUB_SYS_USER': SubsystemUsers,
        'SUB_SYS_MSGS': Subsystems,
        'DASD_POOLS': DASDPools,
        'SERVICE_JAM': ServiceJams,
        'MQ_SUMMARY': MQSummarys,
        'MQ_Q_DETAIL': MQQueues,
        'MQ_CHL_DETAIL': MQChannels,
        'REMOTE_DATA_STORE': RemoteDataStores,
        'ENH_CHL_UTILIZATION': EnhChlUtils,
        'LPAR_UTILIZATION': LPARUtils,
        'MPIF': MPIFs,
        'LODIC': LodicResources,
        'LODIC_UTIL_CLASS': LodicUtilizations,
        'DASD_METRICS': DASDMetrics,
        'TRAD_DB_ENCRYPTION': Trad_DB_Encryptions,
        'TPFDF': TPFDFFiles,
        'default': CdcRecord
    }

    encountered_data_types = set()

    def __init__(self, json_cdc_body):

        """
        Initializes a CdcBody object.

        :param json_cdc_body: "body" dictionary from CDC JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            record_list = json_cdc_body["recordList"]

        except KeyError:
            log_message = f'Unable to deserialize CDC body. ' \
                          f'Missing fields: {get_missing_keys(json_cdc_body, self.required_fields_rlist)}. ' \
                          f'Data: {json_cdc_body}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

        self.record_list = []

        for record in record_list:

            try:
                cdc_record_header_object = CdcRecordHeader(record['cdcRecordHeader'])
            except KeyError:
                log_message = f'Unable to deserialize CDC body. ' \
                              f'Missing fields: {get_missing_keys(json_cdc_body, self.required_fields_rheader)}. ' \
                              f'Data: {json_cdc_body}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)

            # checks to see if data type is supported
            if cdc_record_header_object.cdc_record_name in self.cdc_data_types:
                cdc_data_type = self.cdc_data_types[cdc_record_header_object.cdc_record_name]
            else:
                # chooses to make CdcRecord object for unsupported data types
                cdc_data_type = self.cdc_data_types['default']

                # notes and logs warning if the unsupported datatype has not yet been encountered
                if cdc_record_header_object.cdc_record_name not in self.encountered_data_types:
                    self.encountered_data_types.add(cdc_record_header_object.cdc_record_name)
                    logger.warning(f'Encountered the following CDC data type for which there is no handler: '
                                   f'{cdc_record_header_object.cdc_record_name}. Passing raw JSON for this type.')

            try:
                self.record_list.append(cdc_data_type(record['cdcRecordArray'], cdc_record_header_object))
            except KeyError:
                log_message = f'Unable to deserialize CDC body. Missing fields: ' \
                              f'{get_missing_keys(record, self.required_fields_rarray)}. Data: {record}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)
