#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_cdc.cdc_db_handlers.common_deploy_db import CommonDeployHandler
from tpf_cdc.cdc_db_handlers.istreams_db import IStreamsHandler
from tpf_cdc.cdc_db_handlers.service_app_db import ServiceAppHandler
from tpf_cdc.cdc_db_handlers.service_jam_db import ServiceJamHandler
from tpf_cdc.cdc_db_handlers.dasd_queue_db import DASDQueueHandler
from tpf_cdc.cdc_db_handlers.dasd_service_time_db import DASDServiceTimeHandler
from tpf_cdc.cdc_db_handlers.dasd_pools_db import DASDPoolsHandler
from tpf_cdc.cdc_db_handlers.tape_db import TapeHandler
from tpf_cdc.cdc_db_handlers.vfa_db import VFAHandler
from tpf_data_manager.source_manager import SourceManager
from tpf_cdc.cdc_db_handlers.system_blocks_db import SystemBlocksHandler
from tpf_cdc.cdc_db_handlers.system_lists_db import SystemListsHandler
from tpf_cdc.cdc_db_handlers.system_messages_db import SystemMessagesHandler
from tpf_cdc.cdc_db_handlers.subsystem_users_db import SubsystemUserHandler
from tpf_cdc.cdc_db_handlers.subsystems_db import SubsystemHandler
from tpf_cdc.cdc_db_handlers.tcp_ip_db import TcpIpHandler
from tpf_cdc.cdc_db_handlers.static_db import StaticHandler
from tpf_cdc.cdc_db_handlers.mq_summary_db import MQSummaryHandler
from tpf_cdc.cdc_db_handlers.mq_queues_db import MQQueueHandler
from tpf_cdc.cdc_db_handlers.mq_channels_db import MQChannelHandler
from tpf_cdc.cdc_db_handlers.remote_data_store_db import RemoteDataStoreHandler
from tpf_cdc.cdc_db_handlers.enh_chl_util_db import EnhChlUtilHandler
from tpf_cdc.cdc_db_handlers.lpar_util_db import LPARlUtilHandler
from tpf_cdc.cdc_db_handlers.mpif_db import MPIFHandler
from tpf_cdc.cdc_db_handlers.lodic_resource_db import LodicResourcesHandler
from tpf_cdc.cdc_db_handlers.lodic_utilization_db import LodicUtilizationHandler
from tpf_cdc.cdc_db_handlers.dasd_metrics_db import DASDMetricsHandler
from tpf_cdc.cdc_db_handlers.trad_db_encrypt_db import Trad_DB_EncryptionHandler
from tpf_cdc.cdc_db_handlers.tpfdf_db import TPFDFHandler
from tpf_cdc.tpf_cdc_data import CdcData
from pymysql.cursors import Cursor
from typing import List, Dict, Type
import logging

from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler


logger = logging.getLogger('tpf_zrtmc_analyzer')


class CdcManager(SourceManager):
    cdc_data_types: Dict[str, Type[DbDataHandler]] = {"SYSTEM_MESSAGES": SystemMessagesHandler,
                                                      "SYSTEM_BLOCKS": SystemBlocksHandler,
                                                      "TCP_IP": TcpIpHandler,
                                                      "ISTREAM": IStreamsHandler,
                                                      "COMMON_DEPLOY_FILES": CommonDeployHandler,
                                                      "SERVICE_APP": ServiceAppHandler,
                                                      "DASD_DEVICES": DASDQueueHandler,
                                                      "DASD_SERVICE_TIME": DASDServiceTimeHandler,
                                                      "TAPE": TapeHandler,
                                                      "VFA_SUBSYSTEM": VFAHandler,
                                                      "SYSTEM_LISTS": SystemListsHandler,
                                                      "STATIC_SYS_INFO": StaticHandler,
                                                      "SUB_SYS_USER": SubsystemUserHandler,
                                                      "SUB_SYS_MSGS": SubsystemHandler,
                                                      "DASD_POOLS": DASDPoolsHandler,
                                                      "SERVICE_JAM": ServiceJamHandler,
                                                      "MQ_SUMMARY": MQSummaryHandler,
                                                      "MQ_Q_DETAIL": MQQueueHandler,
                                                      "MQ_CHL_DETAIL": MQChannelHandler,
                                                      "REMOTE_DATA_STORE": RemoteDataStoreHandler,
                                                      "ENH_CHL_UTILIZATION": EnhChlUtilHandler,
                                                      "LPAR_UTILIZATION": LPARlUtilHandler,
                                                      "MPIF": MPIFHandler,
                                                      "LODIC": LodicResourcesHandler,
                                                      "LODIC_UTIL_CLASS": LodicUtilizationHandler,
                                                      "DASD_METRICS": DASDMetricsHandler,
                                                      "TPFDF": TPFDFHandler,
                                                      "TRAD_DB_ENCRYPTION": Trad_DB_EncryptionHandler
                                                      }
    required_fields = cdc_data_types.keys()

    def __init__(self, active_source_types: List):
        # Sets self.active_source_types
        super().__init__(active_source_types)

        self.unhandled_data_types = set()
        self.active_cdc_data_types: Dict[str, DbDataHandler] = {}

        if active_source_types is not None:
            unrecognized_source_types = []
            for source_type in active_source_types:
                try:
                    self.active_cdc_data_types[source_type] = self.cdc_data_types[source_type]()
                except KeyError:
                    unrecognized_source_types.append(source_type)

            if len(unrecognized_source_types) > 0:
                msg = f'The following data types were not recognized: {unrecognized_source_types}.' \
                      f'The following are recognized CDC data types: {self.cdc_data_types.keys()}. Make ' \
                      f'sure the correct data types are specified in the data_sources.cdc.tpf_processing ' \
                      f'section of the ZRTMC Analyzer profile.'
                logger.error(msg)
                raise ValueError

        if active_source_types is None or len(active_source_types) == 0:
            msg = f'TPF processing is not enabled for CDC.'
            logger.info(msg)
        else:
            msg = f'TPF processing is enabled for the following CDC data types: {active_source_types}'
            logger.info(msg)

    def get_db_tables(self, active_only=True):

        table_names = []
        if active_only:
            for data_handler in self.active_cdc_data_types.values():
                table_names += data_handler.get_table_names()
        else:
            for data_handler in self.cdc_data_types.values():
                table_names += data_handler.get_table_names()
        return table_names

    def deserialize(self, json_doc: Dict):
        try:
            source = json_doc['header']['source']
            if source != 'RTMC_SEND_CDC' and source != 3:
                return None
            return CdcData(json_doc)
        except (KeyError, MissingFieldError):
            msg = f'Failed to deserialize a JSON document. Source: CDC. Data: {json_doc}'
            logger.error(msg)
            return None

    def get_mysql_insert_manys(self, db_cursor: Cursor, analytics_data: CdcData):
        insert_manys: List[InsertMany] = []
        for record in analytics_data.body.record_list:

            cdc_data_type = record.cdc_record_header.cdc_record_name

            if cdc_data_type in self.active_cdc_data_types:
                data_type_handler = self.active_cdc_data_types[cdc_data_type]
                insert_manys += data_type_handler.get_mysql_insert_manys(db_cursor, record, parent=analytics_data)
            elif cdc_data_type not in self.unhandled_data_types:
                msg = f'Encountered a CDC data type for which there is no active TPF processing. ' \
                      f'Data type: {cdc_data_type}'
                logger.warning(msg)
                self.unhandled_data_types.add(cdc_data_type)
        return insert_manys
