#######################################################################
#       5748-T15                                                       #
#       Copyright IBM Corp. 2022, 2024                               #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   10152023    PJ47106 - Display multiple Kafka producers            #
#   20240104    PJ47174 - Remove ip address from tpf_systems index    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging
from collections import namedtuple
from typing import Dict
from pymysql.cursors import Cursor

from tpf_rtmc_db.db_writer import last_executed_query
from tpf_utils.db_utils import create_id_pre_sql_select, create_id_pre_sql_insert

logger = logging.getLogger('tpf_zrtmc_analyzer')

IdCols = namedtuple('IdCols', ['key_cols', 'extra_cols'])

unknown_col = 'N/A'

id_table_cols = {
    'tpf_systems': IdCols(
        key_cols=['complex_name', 'cpc_name', 'system_name', 'processor_id'],
        extra_cols=['environment', 'ip_address']
    ),
    'tpf_jams': IdCols(
        key_cols=['system_id', 'subsystem', 'jam'],
        extra_cols=['is_jam_app']
    ),
    'jvm_operations': IdCols(
        key_cols=['operation'],
        extra_cols=[]
    ),
    'app_dimensions': IdCols(
        key_cols=['dimension'],
        extra_cols=[]
    ),
    'app_dimension_values': IdCols(
        key_cols=['dimension_id', 'value'],
        extra_cols=[]
    ),
    'tpf_subsystems': IdCols(
        key_cols=['subsystem'],
        extra_cols=[]
    ),
    'service_app_operations': IdCols(
        key_cols=['operation'],
        extra_cols=['jam_name']
    ),
    'tpf_jmx_producer_table': IdCols(
        key_cols=['producer_name'],
        extra_cols=[]
    ),
    'tpf_tcpip_name_table': IdCols(
        key_cols=['app_name'],
        extra_cols=[]
    ),
    'tpf_mq_queue_manager_table': IdCols(
        key_cols=['queue_manager'],
        extra_cols=[]
    ),
    'tpf_mq_queue_name_table': IdCols(
        key_cols=['queue_name'],
        extra_cols=[]
    ),
    'tpf_mq_channel_name_table': IdCols(
        key_cols=['channel_name'],
        extra_cols=[]
    ),
    'tpf_chl_util_type_table': IdCols(
        key_cols=['type'],
        extra_cols=[]
    ),
    'tpf_rds_name_table': IdCols(
        key_cols=['rds_name'],
        extra_cols=[]
    ),
    'tpf_rds_mode_table': IdCols(
        key_cols=['rds_mode'],
        extra_cols=[]
    ),
    'tpf_lpar_name_table': IdCols(
        key_cols=['lpar_name'],
        extra_cols=[]
    ),
    'tpf_mpif_path_name_table': IdCols(
        key_cols=['path_name'],
        extra_cols=[]
    ),
    'tpf_mpif_device_name_table': IdCols(
        key_cols=['device_name'],
        extra_cols=[]
    ),
    'tpf_shutdown1_shutdown_type_table': IdCols(
        key_cols=['shutdown_type'],
        extra_cols=[]
    ),
    'tpf_shutdown1_reason_table': IdCols(
        key_cols=['reason'],
        extra_cols=[]
    ),
    'tpf_lodic_resource_name_table': IdCols(
        key_cols=['resource_priority_class'],
        extra_cols=[]
    ),
    'tpf_lodic_utilization_name_table': IdCols(
        key_cols=['utilization_priority_class'],
        extra_cols=[]
    ),
    'tpf_common_deployment_types': IdCols(
        key_cols=['cd_type'],
        extra_cols=[]
    ),
    'tpf_common_deployment_versions': IdCols(
        key_cols=['cd_version'],
        extra_cols=[]
    ),
    'tpf_common_deployment_keys': IdCols(
        key_cols=['cd_key'],
        extra_cols=[]
    ),
    'tpf_tpfdf_file_name_alias_table': IdCols(
        key_cols=['file_name_alias'],
        extra_cols=[]
    )

}

id_tables = {
    table: {
        'insert': create_id_pre_sql_insert(table, id_cols.key_cols + id_cols.extra_cols),
        'select': create_id_pre_sql_select(table, id_cols.key_cols),
        'ids': {}
    }
    for table, id_cols in id_table_cols.items()
}

tpf_systems_ip_addresses = {}

def _get_id(table, db_cursor: Cursor, *db_values):
    """
    Gets a database id. This first checks the local cache for the id. If not present,
    it gets the id from the database, or inserts the record and assigns an id, then
    stores the id in the local cache.

    :param table: The db table.
    :param db_cursor: pymysql.cursors.Cursor to access the database.
    :param db_values: One or more values which make up the database record. The values that make up the
    key should be first, followed by any extra values.
    :return: The id for the record.
    :raise pymysql.DatabaseError:
    """
    ids: Dict = id_tables[table]['ids']

    # The first len(key_cols) values in db_values are the key values we need for the cache lookup
    db_key_values = db_values[:len(id_table_cols[table].key_cols)]

    # The last value is the key for the id in the cache dictionary
    id_key = db_key_values[-1]

    level = ids

    # Access the nested dictionaries until we reach the lowest level, which contains the id_key
    # setdefault associates an empty dict with a key if there is no value, otherwise returns the value
    for db_val in db_key_values[:-1]:
        level = level.setdefault(db_val, {})

    i = level.get(id_key)

    # if we're working with the tpf_systems table, we need to see if the ip_address has changed
    # if it has, we need to update the ip address
    if i is not None and table == 'tpf_systems':
        update_tpf_systems_ip_address(db_cursor,db_values[-1],i)

    # If id_key has no id value, we need to get it from the db
    if i is None:
        # SELECT the id from the table
        # Only need to SELECT based on the key values
        db_cursor.execute(
            id_tables[table]['select'],
            db_key_values
        )

        result = db_cursor.fetchall()

        # If the row doesn't exist, insert it and get the auto_incremented id
        if len(result) == 0:
            affected_rows = db_cursor.execute(id_tables[table]['insert'], db_values)

            if affected_rows != 1:
                logger.warning(f'Duplicate ID row inserted. SQL: {last_executed_query(db_cursor)}')

            # Commit the new ID to the database before we cache it. If the commit fails, it'll throw an exception
            # and we'll never cache the ID. The DB and cache need to be in sync, otherwise data will end up with
            # incorrect IDs. Inserting a new ID will be a very infrequent operation, so isn't much cost.
            db_cursor.connection.commit()
            i = db_cursor.lastrowid
        else:
            i = result[0][0]

        # Cache the id
        level[id_key] = i

        # if we're working with the tpf_systems table, we need to cache the ip_address
        # if the ip address in the DB doesn't match our input, we'll update it
        if table == 'tpf_systems':
            db_cursor.execute(f'SELECT ip_address FROM tpf_systems WHERE id=\'{i}\';')
            result = db_cursor.fetchall()
            tpf_systems_ip_addresses[i]=result[0][0]
            update_tpf_systems_ip_address(db_cursor,db_values[-1],i)

    return i

def update_tpf_systems_ip_address(db_cursor: Cursor, ip_address, i):
    if tpf_systems_ip_addresses.get(i) != ip_address:
       db_cursor.execute(f'UPDATE tpf_systems SET ip_address=\'{ip_address}\' WHERE id=\'{i}\';')
       tpf_systems_ip_addresses[i]=ip_address

def clear_id_cache():
    """
    Clears all the cached ids.
    """
    for table_cache in id_tables.values():
        table_cache['ids'].clear()

def calculate_rate(count, duration):
   if duration == 0:
      return 0
   return ((count * 1000) / duration)

def get_tpf_system_id(complex_name, cpc_name, system_name, processor_id, ip_address, environment, db_cursor: Cursor):
    return _get_id('tpf_systems', db_cursor, complex_name or unknown_col, cpc_name or unknown_col,
                   system_name or unknown_col, processor_id or unknown_col,
                   environment or unknown_col, ip_address or unknown_col)


def get_tpf_jam_id(system_id, subsystem, jam, is_jam_app, db_cursor: Cursor):
    system_id = system_id if system_id is not None else get_tpf_system_id(None, None, None, None, None, None, db_cursor)
    return _get_id('tpf_jams', db_cursor, system_id, subsystem or unknown_col, jam or unknown_col, is_jam_app)


def get_jvm_operation_id(operation, db_cursor: Cursor):
    return _get_id('jvm_operations', db_cursor, operation or unknown_col)


def get_app_dimension_id(dimension, db_cursor: Cursor):
    return _get_id('app_dimensions', db_cursor, dimension or unknown_col)


def get_app_dimension_value_id(dimension_id, dimension_value, db_cursor: Cursor):
    dimension_id = dimension_id if dimension_id is not None else get_app_dimension_id(None, db_cursor)
    return _get_id('app_dimension_values', db_cursor, dimension_id, dimension_value or unknown_col)


def get_tpf_subsystem_id(subsystem, db_cursor: Cursor):
    return _get_id('tpf_subsystems', db_cursor, subsystem or unknown_col)


def get_service_app_operation_id(operation, jam_name, db_cursor: Cursor):
    return _get_id('service_app_operations', db_cursor, operation or unknown_col, jam_name)


def get_jmx_producer_id(producer_name, db_cursor: Cursor):
    return _get_id('tpf_jmx_producer_table', db_cursor, producer_name or unknown_col)


def get_tcpip_name_id(app_name, db_cursor: Cursor):
    return _get_id('tpf_tcpip_name_table', db_cursor, app_name or unknown_col)


def get_mq_queue_manager_id(queue_manager, db_cursor: Cursor):
    return _get_id('tpf_mq_queue_manager_table', db_cursor, queue_manager or unknown_col)


def get_mq_queue_name_id(queue_name, db_cursor: Cursor):
    return _get_id('tpf_mq_queue_name_table', db_cursor, queue_name or unknown_col)


def get_mq_channel_name_id(channel_name, db_cursor: Cursor):
    return _get_id('tpf_mq_channel_name_table', db_cursor, channel_name or unknown_col)


def get_chl_type_id(type, db_cursor: Cursor):
    return _get_id('tpf_chl_util_type_table', db_cursor, type or unknown_col)


def get_rds_name_id(rds_name, db_cursor: Cursor):
    return _get_id('tpf_rds_name_table', db_cursor, rds_name or unknown_col)


def get_rds_mode_id(rds_mode, db_cursor: Cursor):
    return _get_id('tpf_rds_mode_table', db_cursor, rds_mode or unknown_col)


def get_lpar_name_id(lpar_name, db_cursor: Cursor):
    return _get_id('tpf_lpar_name_table', db_cursor, lpar_name or unknown_col)


def get_mpif_path_name_id(path_name, db_cursor: Cursor):
    return _get_id('tpf_mpif_path_name_table', db_cursor, path_name or unknown_col)


def get_mpif_device_name_id(device_name, db_cursor: Cursor):
    return _get_id('tpf_mpif_device_name_table', db_cursor, device_name or unknown_col)


def get_shutdown1_shutdown_type_id(shutdown_type, db_cursor: Cursor):
    return _get_id('tpf_shutdown1_shutdown_type_table', db_cursor, shutdown_type or unknown_col)


def get_shutdown1_reason_id(reason, db_cursor: Cursor):
    return _get_id('tpf_shutdown1_reason_table', db_cursor, reason or unknown_col)


def get_lodic_resource_name_id(class_name, db_cursor: Cursor):
    return _get_id('tpf_lodic_resource_name_table', db_cursor, class_name or unknown_col)


def get_lodic_utilization_name_id(class_name, db_cursor: Cursor):
    return _get_id('tpf_lodic_utilization_name_table', db_cursor, class_name or unknown_col)


def get_common_deployment_type_id(cd_type, db_cursor: Cursor):
    return _get_id('tpf_common_deployment_types', db_cursor, cd_type or unknown_col)


def get_common_deployment_version_id(cd_version, db_cursor: Cursor):
    return _get_id('tpf_common_deployment_versions', db_cursor, cd_version or unknown_col)


def get_common_deployment_key_id(cd_key, db_cursor: Cursor):
    return _get_id('tpf_common_deployment_keys', db_cursor, cd_key or unknown_col)


def get_tpfdf_file_name_alias_id(file_name_alias, db_cursor: Cursor):
    return _get_id('tpf_tpfdf_file_name_alias_table', db_cursor, file_name_alias or unknown_col)
