#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from abc import ABC, abstractmethod
from pymysql.cursors import Cursor
from typing import List, Dict


class SourceManager(ABC):

    def __init__(self, active_source_types: List, **kwargs):
        """
        creates SourceManager object

        :param active_source_types: Python list of strings indicating the active data
        types for tpf processing of a data source
        :raise ValueError: If data type is not recognized
        """
        self.active_source_types = active_source_types

    @abstractmethod
    def get_db_tables(self, active_only=True):
        """
         If active_only (default), return only the table names accessed by the data types that are active.
         If not active_only, return the table names accessed by all the data types this source manager knows about.

        :param active_only: boolean
        :return: a list of table names (strings)
        :rtype: List[str]
        """
        pass

    @abstractmethod
    def deserialize(self, json_doc: Dict):
        """
        Create data object from json doc.

        :param json_doc: a Python dictionary
        :return: a Python object version of the data in json_doc
        """
        pass

    def get_mysql_inserts(self, db_cursor: Cursor, analytics_data):
        """
        Return the SQL insert statements for the input data.

        :param db_cursor: A pymysql.cursors.Cursor object to create the insert statement.
        :param analytics_data: An object instance to be processed for SQL insert statements.
        :return: List of the SQL insert statements.
        :rtype: list[bytes]
        :raise pymysql.DatabaseError:
        """
        inserts = self.get_mysql_insert_manys(db_cursor, analytics_data)
        return [db_cursor.mogrify(insert.pre_insert, row) for insert in inserts for row in insert.rows]

    @abstractmethod
    def get_mysql_insert_manys(self, db_cursor: Cursor, data):
        """
        Gets a list of InsertMany objects for bulk insert into the database.

        :param db_cursor: A pymysql.cursors.Cursor object to access the database.
        :param data: The data to be processed for SQL inserts.
        :return: List of the SQL InsertMany objects
        :rtype: list[tpf_rtmc_db.insert_many.InsertMany]
        :raise pymysql.DatabaseError:
        """
        pass
