#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20240403    PJ48015 - Vertical processing was limited to one      #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
import time

from exceptions.json_err import InvalidAppDimensionConfigError
from tpf_batch.batch_processor import AnalyzerBatchProcessor
from tpf_data_manager import id_manager
from tpf_data_manager.dbms import DBMS
from tpf_data_manager.source_manager import SourceManager
from tpf_jvm.tpf_jvm_manager import JvmManager
from tpf_cdc.tpf_cdc_manager import CdcManager
from tpf_nvpc.nvpc_config.app_dimension import AppDimensionConfigValidator
from tpf_nvpc.nvpc_config.orientation import Orientation
from tpf_nvpc.tpf_nvpc_vertical_manager import NvpcVerticalManager
from tpf_nvpc.tpf_nvpc_horizontal_manager import NvpcHorizontalManager
from tpf_data_manager.data_source import DataSource
from tpf_batch.batch import AnalyzerBatch
from tpf_rtmc_db.db_writer import DbWriter
from pymysql import DatabaseError, OperationalError
from typing import Dict, Type, List
from constants.profile_constants import *
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.profile_utils import are_profile_parameters_present

logger = logging.getLogger('tpf_zrtmc_analyzer')


def group_and_merge_inserts(insert_manys: List[InsertMany]):
    """
    Groups a list of InsertMany objects by pre_insert and merges each groups
    rows.
    :param insert_manys: A list of InsertMany objects.
    :return: An iterable of InsertMany objects, one per unique pre_insert in insert_manys.
    """
    merged: Dict[str, InsertMany] = {}

    for insert in insert_manys:
        i = merged.setdefault(insert.pre_insert, InsertMany(insert.pre_insert))
        i.merge(insert)

    return merged.values()


class ZrtmcManager(AnalyzerBatchProcessor):
    source_managers: Dict[DataSource, Type[SourceManager]] = {DataSource.JVM: JvmManager,
                                                              DataSource.CDC: CdcManager,
                                                              DataSource.NVPC_HORIZONTAL: NvpcHorizontalManager,
                                                              DataSource.NVPC_VERTICAL: NvpcVerticalManager}

    tpf_sources = [tpf_source.name.lower() for tpf_source in source_managers]

    required_parameters_data_source = [ACTIVE, TPF_PROCESSING]
    required_parameters_db = [HOST, PORT, USERNAME, PASSWORD, NAME]

    additional_db_tables = id_manager.id_table_cols.keys()

    def __init__(self, analyzer_profile: dict):
        """
        Initialize a ZrtmcManager object.

        :param analyzer_profile: Python dictionary of tpf_zrtmc_analyzer_profile
        :raise ValueError: If the profile data has an incorrect field, or is missing a required field.
        """
        # num of empty batches encountered
        self.empty_batches = 0
        self.active_source_managers: Dict[DataSource, SourceManager] = {}

        if not are_profile_parameters_present(analyzer_profile, [DATA_SOURCES], None):
            raise ValueError

        sources = analyzer_profile[DATA_SOURCES]
        unrecognized_sources = set()

        for source_name, source_config in sources.items():
            try:
                data_source: DataSource = DataSource[source_name.upper()]
            except KeyError:
                unrecognized_sources.add(source_name)
                continue

            if are_profile_parameters_present(source_config, self.required_parameters_data_source,
                                              f'{DATA_SOURCES}.{source_name}', log_missing=False, warn_missing=True):
                if source_config[ACTIVE]:
                    if data_source not in {DataSource.NVPC_HORIZONTAL, DataSource.NVPC_VERTICAL}:
                        # Initializing source_manager could raise a ValueError if there is an error in the
                        # configuration. We want to raise a ValueError from this init function if there is an error,
                        # so we won't handle it.
                        self.active_source_managers[data_source] = self.source_managers[data_source](
                            source_config[TPF_PROCESSING])
                    else:
                        cur_vertical_limit = 1
                        try:
                            cur_vertical_limit = source_config[MAX_VERTICAL]
                        except Exception:
                            pass
                        app_dim_validator = AppDimensionConfigValidator(num_verticals=cur_vertical_limit)
                        app_dimensions_config = {}

                        # We need to validate all the application_dimensions as a group, so save the config for now then
                        # we'll initialize the nvpc source managers later
                        orientation = Orientation.HORIZONTAL if data_source == DataSource.NVPC_HORIZONTAL else Orientation.VERTICAL
                        # {'app_dimensions': List[AppDimensions], 'tpf_processing': List[String]}
                        try:
                            app_dimensions_config[data_source] = {
                                'app_dimensions': app_dim_validator.validate(
                                    source_config.get('application_dimensions'),
                                    orientation,
                                    f'{DATA_SOURCES}.{source_name}.application_dimensions'),
                                TPF_PROCESSING: source_config[TPF_PROCESSING]
                            }
                        except InvalidAppDimensionConfigError:
                            raise ValueError

                        if data_source == DataSource.NVPC_HORIZONTAL:
                            try:
                                app_dim_validator.validate_group()
                            except InvalidAppDimensionConfigError:
                                raise ValueError

                            sys_indicator = app_dim_validator.get_system_indicator()

                        self.active_source_managers.update(
                            {ds: self.source_managers[ds](config[TPF_PROCESSING], **config,
                                                          **{'system_indicator': sys_indicator}) for
                             ds, config in app_dimensions_config.items()}
                        )

        if len(unrecognized_sources) > 0:
            msg = f'The following data sources found in the RTMC Analytics profile were unrecognized:' \
                  f' {list(unrecognized_sources)} TPF data sources are: {self.tpf_sources}'
            logger.warning(msg)

        if self.active_source_managers == {}:
            msg = f'No active data sources were configured. You must configure one of the following data sources' \
                  f' to be active: {self.tpf_sources}'
            logger.error(msg)
            raise ValueError

        # Log any data sources that were missing
        for ds in self.source_managers:
            if ds not in self.active_source_managers:
                logger.info(f'Data source "{ds.name}" is inactive.')

        # DB section could be either mariadb or database. We'll log when database is missing
        db_section = MARIADB

        if not are_profile_parameters_present(analyzer_profile, [db_section], None, log_missing=False):
            db_section = DATABASE
            if not are_profile_parameters_present(analyzer_profile, [db_section], None):
                raise ValueError

        db_profile = analyzer_profile[db_section]

        if not are_profile_parameters_present(db_profile, self.required_parameters_db, db_section):
            raise ValueError

        host = db_profile[HOST]
        port = db_profile[PORT]
        username = db_profile[USERNAME]
        password = db_profile[PASSWORD]
        database_name = db_profile[NAME]

        dbms = DBMS.MARIADB

        # Require system when db_section is database
        if db_section == DATABASE:
            if not are_profile_parameters_present(db_profile, [SYSTEM], db_section):
                raise ValueError

            try:
                dbms = DBMS[db_profile[SYSTEM].upper()]
            except KeyError:
                logger.error(f'Value {db_profile[SYSTEM]} for {db_section}.{SYSTEM} is not one of the following '
                             f'values: {["mysql", "mariadb"]}')
                raise ValueError

        self.db_writer = DbWriter(host, port, username, password, database_name, dbms)
        self.db_start_time = 0

    def process_batch(self, batch: AnalyzerBatch):
        """
        Insert the json data from batch into the database.

        :param batch: an AnalyzerBatch object
        """

        if batch.is_empty():
            if self.empty_batches < 5:
                self.empty_batches += 1
            else:
                # close db_writer
                self.db_writer.close(commit=False)
            return

        try:
            if not self.db_writer.is_open():
                self.db_writer.open()
                self.check_db_start_time()
        except DatabaseError as e:
            logger.error(f'Lost connection to the database. Attempting to reconnect. Exception: {e}')
            # Clear the cache in case db data was lost. We need the cache to be in sync with the DB
            id_manager.clear_id_cache()
            self.open_db_connection(with_reconnect=True)
            logger.info('Successfully reconnected to the database.')

        self.empty_batches = 0

        # Does nothing if autocommit is already false
        if not self._set_autocommit(False):
            self.db_writer.close(commit=False)
            return

        try:
            # Multi-row inserts for better performance
            insert_manys = group_and_merge_inserts([
                insert_many
                for source, manager in self.active_source_managers.items()
                for data in batch.get_analyzer_data(source)
                for insert_many in manager.get_mysql_insert_manys(self.db_writer.db_cursor, data)
            ])
        except DatabaseError as e:
            logger.warning(f'Failed to create insert statements. Message: {e}')
            self.db_writer.close(commit=False)
            return

        # Split into stored procedures and regular inserts
        regulars, procs = [], []
        for i in insert_manys:
            (procs if i.pre_insert.startswith('CALL') else regulars).append(i)

        for insert_many in regulars:
            if(insert_many.pre_insert.startswith('UPDATE')):
               # This insert_many is an UPDATE of a single row so simply execute the SQL statement
               self.db_writer.execute_sql(insert_many.pre_insert)
            else:
               # This insert_many is an INSERT of potentially multiple rows
               self._exec_insert_many(insert_many)

        if len(procs) > 0:
            # Commit after each procedure
            if not self._set_autocommit(True):
                self.db_writer.close(commit=False)
                return

            for insert_many in procs:
                self._exec_insert_many(insert_many)

            if not self._set_autocommit(False):
                self.db_writer.close(commit=False)
                return

        try:
            self.db_writer.commit()
        except DatabaseError as e:
            logger.warning(f'Failed to commit batch. Message: {e}.')
            # Close the connection. Maybe the DB went down. We'll try to open again next time around.
            self.db_writer.close(commit=False)

    def _exec_insert_many(self, insert_many: InsertMany):
        try:
            self.db_writer.exec_insert_many(insert_many)

            if self.db_writer.last_execution_time.total_seconds() > 3:
                logger.warning(f'Last query execution time was slow.'
                               f' Seconds: {self.db_writer.last_execution_time.total_seconds()}.'
                               f' SQL: {self.db_writer.last_executed()}')

                # recycle connection
                self.db_writer.close()
                time.sleep(0.1)
                self.open_db_connection(with_reconnect=True)
        except OperationalError as e:
            logger.warning(f'Unable to insert data. Message: {e}. SQL: {self.db_writer.last_executed()}.')
            # OperationError is probably a DB connection issue. Try to reconnect, then move on.
            logger.info(f'Attempting to recycle connection.')
            # Clear the cache in case db data was lost. We need the cache to be in sync with the DB
            id_manager.clear_id_cache()
            self.db_writer.close(commit=False)
            self.open_db_connection(with_reconnect=True)
            logger.info('Successfully reconnected to the database.')
        except DatabaseError as e:
            logger.warning(f'Unable to insert data. Message: {e}. SQL: {self.db_writer.last_executed()}.')

    def _set_autocommit(self, autocommit):
        succeeded = True

        try:
            self.db_writer.set_autocommit(autocommit)
        except DatabaseError as e:
            logger.warning(f'Failed to change autocommit value. Message: {e}')
            self.db_writer.close(commit=False)
            succeeded = False

        return succeeded

    def open_db_connection(self, log=False, with_reconnect=False):
        """
        Opens a connection to the database.

        :param log: If True, this function will log success and failure messages. Reconnect messages
        are always logged every 60 seconds.
        :param with_reconnect: If True, this function will attempt to reconnect to the database
        every ten seconds until a successful connection is made.
        :return: the success of the connection to the database
        :rtype: bool
        """

        fail_msg = f'Database connection failed. Check your configuration parameters in the ZRTMC Analyzer profile.'
        success_msg = f'Connected to database {self.db_writer.db_name} at {self.db_writer.host}:{self.db_writer.port}.'
        db_trying_msg = f'Still trying to connect to the database.'

        if self.db_writer.is_open():
            if log:
                logger.info(success_msg)
            return True

        try:
            self.db_writer.open()
        except DatabaseError:
            if log:
                logger.error(fail_msg)
            if not with_reconnect:
                return False
            else:
                db_wait_time = 0
                while not self.open_db_connection():
                    if db_wait_time == 60:
                        db_wait_time = 0
                        logger.info(db_trying_msg)
                    time.sleep(10)
                    db_wait_time += 10

        if log:
            logger.info(success_msg)
        return True

    def check_db_tables(self, log_missing=False):
        """
        Check if tables written to by active source managers exist in the database.

        :param log_missing: boolean
        :return: List of missing tables that do not exist in the database
        :rtype: list
        """

        # returns a List of tables written to by active source managers that do not exist in the database

        db_table_names = set()

        for manager in self.active_source_managers.values():
            db_table_names.update(manager.get_db_tables())

        db_table_names.update(self.additional_db_tables)

        missing_tables = []

        for table in db_table_names:

            try:
                if not self.db_writer.table_exists(table):
                    missing_tables.append(table)

            except DatabaseError as e:
                msg = f'Query for existence of a database table failed. Table: {table}. ' \
                      f'Exception: {e}'
                logger.warning(msg)

        if len(missing_tables) > 0 and log_missing:
            msg = f'The following tables do not exist in the database: {missing_tables}'
            logger.warning(msg)
        return missing_tables

    def deserialize_batch(self, batch_json: Dict[DataSource, List[Dict]]):
        """
        Create AnalyzerBatch object of deserialized batch json data for each data source.

        :param batch_json: dictionary of DataSource mapped to list of json file data
        :return: AnalyzerBatch object consisting of deserialized batch json data
        :rtype: AnalyzerBatch
        """
        return AnalyzerBatch(
            **{source.name.lower(): self.deserialize_type(source, json) for source, json in batch_json.items() if
               source in self.active_source_managers})

    def deserialize_type(self, data_source: DataSource, json_docs: List[Dict]):
        """
        Deserialize json_docs into zrtmc data object structure for specific type of data source.

        :param data_source: DataSource object
        :param json_docs: List of Python dictionaries of json docs
        :return: list of zrtmc data objects for that data source
        :rtype: List
        """

        data_manager = self.active_source_managers[data_source]
        data_objects = []
        for json_doc in json_docs:
            data_object = data_manager.deserialize(json_doc)
            if data_object is not None:
                data_objects.append(data_object)

        return data_objects

    @classmethod
    def get_instance(cls, analyzer_profile):
        """
        Create ZrtmcManager object from analyzer profile information.

        :param analyzer_profile: analyzer_profile Python dictionary
        :return: ZrtmcManager object from analyzer_profile
        :rtype: ZrtmcManager
        """

        try:
            data_manager = ZrtmcManager(analyzer_profile)
        except ValueError:
            return None
        return data_manager

    def close_db_connection(self, commit=True):
        """
        Close db_writer connection with database.

        :param commit: boolean for if the writer should commit before closing or not
        """

        self.db_writer.close(commit)

    def check_db_start_time(self):
        """
        Checks the db start time and compares to the last cached db start time. If they are different,
        clear the id manager id cache because it is possible the cache is different than the db.
        """
        new_db_start_time = self.db_writer.get_db_start_time()

        # Clear the id cache in case the database went down while we were not connected to db
        if new_db_start_time is None:
            id_manager.clear_id_cache()
        elif new_db_start_time - self.db_start_time - 1 > 0:
            id_manager.clear_id_cache()
            self.db_start_time = new_db_start_time
