#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   10152023    PJ47106 - Display multiple Kafka producers            #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from typing import Dict
from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_jvm_data_type import JvmDataType
from tpf_jvm.tpf_relational_data import RelationalData
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmCustomMbeanData(object):

    required_fields = ['Attributes', 'ObjectName']

    def __init__(self, event_time, relational_data: RelationalData, json_custom_mbean: Dict):
        """
        Initializes a JvmCustomMbeanData.

        :param event_time: unix timestamp integer
        :param relational_data: Relational data from the parent JvmJmxData
        :param json_custom_mbean: the JSON python dictionary of a single object in the "customMBeanData" list
        :raise MissingFieldError: If the data is missing a required field.
        """
        self.event_time = event_time
        self.relational_data = relational_data
        self.mbean_attributes = {}
        try:
            # Create a single dict out of list of dicts in mbean_attributes
            for attr in json_custom_mbean['Attributes']:
                self.mbean_attributes.update(attr)
            self.object_name = json_custom_mbean['ObjectName']
        except KeyError:
            log_message = f'Unable to deserialize JVM custom mbean data. Missing fields: ' \
                          f'{get_missing_keys(json_custom_mbean, self.required_fields)}. Data: {json_custom_mbean}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)


class JvmJaxRsData(object):

    required_fields = ['appName', 'operationCount', 'minRespTime', 'maxRespTime', 'avgRespTime', 'checkedFaults',
                       'uncheckedFaults', 'runLogicFaults', 'runFaults', 'totalHandleTime', 'CXFOps']
    ms_per_us = 1000

    def __init__(self, event_time, relational_data: RelationalData, event_data):
        """
        Initializes a JvmJaxRsData object.

        :param event_time: unix timestamp integer
        :param relational_data: Relational data from the parent JvmJmxData
        :param event_data: "eventData" dictionary from the JVM JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """
        self.relational_data = relational_data
        self.event_time = event_time

        try:
            self.app_name = event_data['appName']
            self.op_count = event_data['operationCount']
            self.min_resp_time = event_data['minRespTime'] / self.ms_per_us
            self.max_resp_time = event_data['maxRespTime'] / self.ms_per_us
            self.avg_resp_time = event_data['avgRespTime'] / self.ms_per_us
            self.app_faults_checked = event_data['checkedFaults']
            self.app_faults_unchecked = event_data['uncheckedFaults']
            self.run_faults_logical = event_data['runLogicFaults']
            self.run_faults_app = event_data['runFaults']
            self.handling_time = event_data['totalHandleTime'] / self.ms_per_us

            if len(event_data["CXFOps"]) != self.op_count:
                log_message = f'The "operationCount" and length of "CXFOps" were not equal. '\
                              f'Expected: {self.op_count}. Found: {len(event_data["CXFOps"])}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)

            self.jax_rs_ops = [JvmJaxRsOperationData(self.app_name,
                                                     self.event_time,
                                                     self.relational_data,
                                                     op_data) for op_data in event_data["CXFOps"]]
        except KeyError:
            log_message = f'Unable to deserialize JVM JAX-RS data. Missing fields: ' \
                          f'{get_missing_keys(event_data, self.required_fields)}. Data: {event_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

    def is_not_none(self):
        """
        Checks if this object is actual JAX-RS data. If a JAM is running but is not
        receiving traffic to generate JAX-RS data, then this will be false.

        :return: true if this is actual JAX-RS data, false otherwise
        """
        return self.app_name != "None" or self.op_count != 0


class JvmJaxRsOperationData(object):
    required_fields = ['opName', 'opMinRespTime', 'opMaxRespTime', 'opAvgRespTime', 'opCheckedFaults',
                       'opUnCheckedFaults', 'opRunLogicFaults', 'opRunFaults', 'opTotalHandleTime']
    ms_per_us = 1000

    def __init__(self, app_name, event_time, relational_data: RelationalData, op_data):
        """
        Initializes a JvmJaxRsOperationData object.

        :param app_name: app_name from JvmJaxRsData object
        :param event_time: unix timestamp integer
        :param relational_data: Relational data from the parent JvmJmxData
        :param op_data: dictionary item from "CXFOps" list from JVM JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """
        self.app_name = app_name
        self.relational_data = relational_data
        self.event_time = event_time

        try:
            self.op_name = op_data['opName']
            self.op_min_resp_time = op_data['opMinRespTime'] / self.ms_per_us
            self.op_max_resp_time = op_data['opMaxRespTime'] / self.ms_per_us
            self.op_avg_resp_time = op_data['opAvgRespTime'] / self.ms_per_us
            self.op_app_faults_checked = op_data['opCheckedFaults']
            self.op_app_faults_unchecked = op_data['opUnCheckedFaults']
            self.op_run_faults_logical = op_data['opRunLogicFaults']
            self.op_run_faults_app = op_data['opRunFaults']
            self.op_handling_time = op_data['opTotalHandleTime'] / self.ms_per_us
        except KeyError:
            log_message = f'Unable to deserialize JVM JAX-RS Operation data. Missing fields: ' \
                          f'{get_missing_keys(op_data, self.required_fields)}. Data: {op_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)


class JvmKafkaJmxData(object):
    required_fields = ['requestTotal', 'requestsInFlight', 'batchSizeAverage', 'recordSendRate',
                       'recordSizeAverage', 'requestLatencyAverage', 'waitingThreads']

    def __init__(self, event_time, relational_data: RelationalData, kafka_event_data, version_number):
        """
        Initializes a JvmKafkaJmxData object.

        :param event_time: unix timestamp integer
        :param relational_data: Relational data from the parent JvmJmxData
        :param kafka_event_data: "KafkaEvent" dictionary from the JVM JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """
        self.event_time = event_time
        self.relational_data = relational_data
        self.version_number = version_number

        try:
            if version_number > 1:
                self.producer_name = kafka_event_data['producerName']
            else:
                self.producer_name = "Not Available"
            self.req_total = kafka_event_data['requestTotal']
            self.reqs_in_flight = kafka_event_data['requestsInFlight']
            self.avg_batch_size = kafka_event_data['batchSizeAverage']
            self.rec_send_rate = kafka_event_data['recordSendRate']
            self.avg_rec_size = kafka_event_data['recordSizeAverage']
            self.avg_req_latency = kafka_event_data['requestLatencyAverage']
            self.waiting_threads = kafka_event_data['waitingThreads']
        except KeyError:
            log_message = f'Unable to deserialize JVM Kafka JMX data. Missing fields: ' \
                          f'{get_missing_keys(kafka_event_data, self.required_fields)}. Data: {kafka_event_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)


class JvmJmxData(JvmDataType):
    required_fields = ['eventTime', 'JVMNum', 'JAMCurrentInUseThreads', 'JAMMaxObservedThreads', 'avgRespTime',
                       'logDirectory', 'DaemonThreadCount', 'customMBeanData']
    ms_per_us = 1000

    def __init__(self, jvm_data: dict):

        super().__init__(jvm_data)

        try:
            self.event_time = self.event_data['eventTime']
            self.log_dir = self.event_data['logDirectory']
            self.daemon_thread_count = self.event_data['DaemonThreadCount']
            self.avg_resp_time = self.event_data['avgRespTime'] / self.ms_per_us

            if self.relational_data.is_jam_app:
                self.num_jvms = self.event_data['JVMNum']
                self.num_jam_threads_inuse = self.event_data['JAMCurrentInUseThreads']
                self.max_num_jam_threads = self.event_data['JAMMaxObservedThreads']
            else:
                self.num_jvms = 1
                self.num_jam_threads_inuse = None
                self.max_num_jam_threads = None

                if self.avg_resp_time <= 0:
                    self.avg_resp_time = None

            self.custom_mbean_data_list = self.event_data['customMBeanData']
        except KeyError:
            log_message = f'Unable to deserialize JVM JMX data. Missing fields: ' \
                          f'{get_missing_keys(self.event_data, self.required_fields)}. Data: {self.event_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

        self.custom_mbean_data: [JvmCustomMbeanData] = []
        self.kafka_data: [JvmKafkaJmxData] = []
        # Look for custom MBeans
        for mbean in self.custom_mbean_data_list:
            self.custom_mbean_data.append(JvmCustomMbeanData(self.event_time, self.relational_data, mbean))

        # JAX-RS data is only available for JAM apps
        if self.relational_data.is_jam_app:
            self.jax_rs_data = JvmJaxRsData(self.event_time, self.relational_data, self.event_data)
        else:
            self.jax_rs_data = None

        # KafkaEvent is only available if KafkaEvent in event_data
        if 'KafkaEvent' in self.event_data:
            if self.version_number > 1:
                kafka_event_info_list = self.event_data['KafkaEvent']
            else:
                kafka_event_info_list: [JvmKafkaJmxData] = []
                kafka_event_info_list.append(self.event_data['KafkaEvent'])
            for kafka_event_data in kafka_event_info_list:
                kafka_data_item = JvmKafkaJmxData(self.event_time, self.relational_data, kafka_event_data, self.version_number)
                self.kafka_data.append(kafka_data_item)
        else:
            self.kafka_data = None
