#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from pymysql.cursors import Cursor

from tpf_jvm.jvm_data_types.jvm_gc_data import JvmGcData
from tpf_jvm.jvm_db_handlers.jvm_db_helper import JvmDbHelper
from tpf_jvm.tpf_jvm_data import JvmData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import ms_to_datetime, create_pre_sql_insert


def create_gc_pre_sql_insert(table_name, db_field_names, db_key_fields):
    """
    Creates an SQL insert statement with % placeholders for replacement by actual values.
    This function is used for JvmGcData. JvmGcData is collected multiple times per second. This insert
    statement averages the GC data values, updating the row with a weighted average as they are inserted.

    :param str table_name: The table name to which to insert the data.
    :param db_field_names: List of the field names in the database.
    :param db_key_fields: List of the database key fields.
    :return: A pre SQL insert string.
    :rtype: str
    """

    exclude_fields = {'num_gc_events', 'num_jvm_threads_latest'}

    # Algorithm for averaging: agg = agg + (val - agg) * wVal / (wVal + wAgg)
    # Where: agg = db value, val = incoming value, wAgg = weight db value (num_gc_events), wVal = weight incoming value
    # wVal (VALUES(num_gc_events)) is currently always 1
    return f'{create_pre_sql_insert(table_name, db_field_names, duplicate_key=False)}' \
           f' ON DUPLICATE KEY UPDATE ' \
           f'{", ".join([f"{s}=IFNULL({s},0)+(VALUES({s})-IFNULL({s},0))*VALUES(num_gc_events)/(VALUES(num_gc_events)+num_gc_events)" for s in db_field_names if s not in db_key_fields and s not in exclude_fields])},' \
           f' num_gc_events=num_gc_events+VALUES(num_gc_events), num_jvm_threads_latest=VALUES(num_jvm_threads_latest)'


class JvmGcHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jvm_metrics_table'
    db_field_names = ['time', 'jam_id', 'pid', 'num_jvm_threads_latest', 'avg_gc_interval', 'avg_gc_pause',
                      'avg_gc_interval_global', 'avg_gc_pause_global', 'used_heap', 'heap_size', 'num_gc_events']
    db_key_fields = {'time', 'jam_id', 'pid'}
    pre_sql_insert = create_gc_pre_sql_insert(table_name, db_field_names, db_key_fields)

    bytes_per_mb = 1048576

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_gc_data: JvmGcData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[ms_to_datetime(jvm_gc_data.event_time),
                 self.get_tpf_jam_id(jvm_gc_data.relational_data, parent, db_cursor),
                 jvm_gc_data.relational_data.pid,
                 jvm_gc_data.num_jvm_threads_latest,
                 jvm_gc_data.avg_interval,
                 jvm_gc_data.avg_pause,
                 jvm_gc_data.avg_interval_global,
                 jvm_gc_data.avg_pause_global,
                 jvm_gc_data.used_heap / self.bytes_per_mb,
                 jvm_gc_data.heap_size / self.bytes_per_mb,
                 1]]

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]
