#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   10152023    PJ47106 - Display multiple Kafka producers            #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from typing import Dict
from pymysql.cursors import Cursor
from tpf_jvm.jvm_data_types.jvm_jmx_data import JvmJmxData, JvmKafkaJmxData, JvmJaxRsData, JvmJaxRsOperationData
from tpf_jvm.jvm_db_handlers.jvm_db_helper import JvmDbHelper
from tpf_jvm.jvm_db_handlers.mbean_handlers.qjmb_memory_stat_db import QjmbMemoryStat0Handler
from tpf_jvm.tpf_jvm_data import JvmData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert, ms_to_datetime


def create_static_pre_sql_insert(table_name, db_field_names, db_key_fields):
    """
    Creates an SQL insert statement with %s placeholders for replacement by actual values.
    This function is used for JvmStaticData.

    :param str table_name: The table name to which to insert the data.
    :param db_field_names: List of the field names in the database.
    :param db_key_fields: List of the database key fields.
    :return: A pre SQL insert string.
    :rtype: str
    """
    return f'{create_pre_sql_insert(table_name, db_field_names, duplicate_key=False)}' \
           f' ON DUPLICATE KEY UPDATE {", ".join([f"{s}=IF(last_time>VALUES(last_time), {s}, VALUES({s}))" for s in db_field_names if s not in db_key_fields])}'


class JvmJmxHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jvm_metrics_table'
    db_field_names = ['time', 'jam_id', 'pid', 'num_jvms_expected', 'num_jvms_actual', 'num_jam_threads_total',
                      'num_jam_threads_inuse', 'max_jam_threads_observed', 'avg_response_time',
                      'num_jvm_threads_daemon']
    db_key_fields = {'time', 'jam_id', 'pid'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def __init__(self):
        self.static_data_handler = JvmStaticDataHandler()
        self.jax_rs_handler = JvmJaxRsHandler()
        self.kafka_handler = JvmKafkaHandler()
        self.mbean_db_handlers: Dict[str, DbDataHandler] = {
            'com.ibm.tpf.qjmb:type=Memory,name=MemoryStat0': QjmbMemoryStat0Handler()
        }

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_data: JvmJmxData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[ms_to_datetime(jvm_jmx_data.event_time),
                 self.get_tpf_jam_id(jvm_jmx_data.relational_data, parent, db_cursor),
                 jvm_jmx_data.relational_data.pid,
                 jvm_jmx_data.relational_data.num_jvms_expected,
                 jvm_jmx_data.num_jvms,
                 jvm_jmx_data.relational_data.num_threads_total,
                 jvm_jmx_data.num_jam_threads_inuse,
                 jvm_jmx_data.max_num_jam_threads,
                 jvm_jmx_data.avg_resp_time,
                 jvm_jmx_data.daemon_thread_count]]

        insert_manys = [InsertMany(self.pre_sql_insert, rows=rows)]

        insert_manys += self.static_data_handler.get_mysql_insert_manys(db_cursor, jvm_jmx_data, parent)

        if jvm_jmx_data.jax_rs_data is not None and jvm_jmx_data.jax_rs_data.is_not_none():
            insert_manys += self.jax_rs_handler.get_mysql_insert_manys(db_cursor, jvm_jmx_data.jax_rs_data, parent)

        if jvm_jmx_data.kafka_data is not None:
            for kafka_event_data in jvm_jmx_data.kafka_data:
                insert_manys += self.kafka_handler.get_mysql_insert_manys(db_cursor, kafka_event_data, parent)

        for mbean in jvm_jmx_data.custom_mbean_data:
            obj_name = mbean.object_name
            if obj_name in self.mbean_db_handlers:
                handler = self.mbean_db_handlers[obj_name]
                insert_manys += handler.get_mysql_insert_manys(db_cursor, mbean, parent)

        return insert_manys

    @classmethod
    def get_table_names(cls):
        # Add all inner classes here (ex. newly created application JMX data)
        table_names = [cls.table_name]
        table_names += JvmStaticDataHandler.get_table_names()
        table_names += JvmJaxRsHandler.get_table_names()
        table_names += JvmKafkaHandler.get_table_names()
        table_names += QjmbMemoryStat0Handler.get_table_names()
        return table_names


class JvmStaticDataHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_static_data_table'
    db_field_names = ['jam_id', 'pid', 'start_time', 'log_dir', 'args', 'last_time']
    db_key_fields = {'jam_id', 'pid', 'start_time'}
    pre_sql_insert = create_static_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_data: JvmJmxData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[self.get_tpf_jam_id(jvm_jmx_data.relational_data, parent, db_cursor),
                 jvm_jmx_data.relational_data.pid,
                 ms_to_datetime(jvm_jmx_data.relational_data.start_time_ms),
                 jvm_jmx_data.log_dir,
                 jvm_jmx_data.relational_data.cmd_line_args,
                 ms_to_datetime(jvm_jmx_data.event_time)]]
        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]


class JvmJaxRsOperationHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jam_jax_rs_op_metrics_table'
    db_field_names = ['time', 'jam_id', 'app', 'pid', 'op_id', 'min_response_time',
                      'max_response_time', 'avg_response_time', 'app_faults_checked', 'app_faults_unchecked',
                      'runtime_faults_logical', 'runtime_faults_app', 'handling_time']
    db_key_fields = {'time', 'jam_id', 'pid', 'op_id'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_jaxrsop_data: JvmJaxRsOperationData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[ms_to_datetime(jvm_jmx_jaxrsop_data.event_time),
                 self.get_tpf_jam_id(jvm_jmx_jaxrsop_data.relational_data, parent, db_cursor),
                 jvm_jmx_jaxrsop_data.app_name[:self.app_db_field_len],
                 jvm_jmx_jaxrsop_data.relational_data.pid,
                 self.get_jvm_operation_id(jvm_jmx_jaxrsop_data.op_name, db_cursor),
                 jvm_jmx_jaxrsop_data.op_min_resp_time,
                 jvm_jmx_jaxrsop_data.op_max_resp_time,
                 jvm_jmx_jaxrsop_data.op_avg_resp_time,
                 jvm_jmx_jaxrsop_data.op_app_faults_checked,
                 jvm_jmx_jaxrsop_data.op_app_faults_unchecked,
                 jvm_jmx_jaxrsop_data.op_run_faults_logical,
                 jvm_jmx_jaxrsop_data.op_run_faults_app,
                 jvm_jmx_jaxrsop_data.op_handling_time]]
        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]


class JvmJaxRsHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jam_jax_rs_metrics_table'
    db_field_names = ['time', 'jam_id', 'app', 'pid', 'op_count', 'min_response_time',
                      'max_response_time', 'avg_response_time', 'app_faults_checked', 'app_faults_unchecked',
                      'runtime_faults_logical', 'runtime_faults_app', 'handling_time']
    db_key_fields = {'time', 'jam_id', 'pid'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def __init__(self):
        self.jax_rs_op_handler = JvmJaxRsOperationHandler()

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_jaxrs_data: JvmJaxRsData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[ms_to_datetime(jvm_jmx_jaxrs_data.event_time),
                 self.get_tpf_jam_id(jvm_jmx_jaxrs_data.relational_data, parent, db_cursor),
                 jvm_jmx_jaxrs_data.app_name[:self.app_db_field_len],
                 jvm_jmx_jaxrs_data.relational_data.pid,
                 jvm_jmx_jaxrs_data.op_count,
                 jvm_jmx_jaxrs_data.min_resp_time,
                 jvm_jmx_jaxrs_data.max_resp_time,
                 jvm_jmx_jaxrs_data.avg_resp_time,
                 jvm_jmx_jaxrs_data.app_faults_checked,
                 jvm_jmx_jaxrs_data.app_faults_unchecked,
                 jvm_jmx_jaxrs_data.run_faults_logical,
                 jvm_jmx_jaxrs_data.run_faults_app,
                 jvm_jmx_jaxrs_data.handling_time]]

        insert_manys = [InsertMany(self.pre_sql_insert, rows=rows)]

        for jax_rs_op in jvm_jmx_jaxrs_data.jax_rs_ops:
            insert_manys += self.jax_rs_op_handler.get_mysql_insert_manys(db_cursor, jax_rs_op, parent)

        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.table_name] + JvmJaxRsOperationHandler.get_table_names()


class JvmKafkaHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jmx_app_Kafka_table'
    pre_sql_insert = ''
    db_field_names = ['time', 'jam_id', 'pid', 'producer_id', 'req_total', 'reqs_in_flight', 'avg_batch_size', 'rec_send_rate',
                          'avg_rec_size', 'avg_req_latency', 'waiting_threads']
    db_key_fields = {'time', 'jam_id', 'pid', 'producer_id'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_kafka_data: JvmKafkaJmxData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        temp_producer_id = self.get_jmx_producer_id(jvm_jmx_kafka_data.producer_name,  db_cursor)
        rows = [[ms_to_datetime(jvm_jmx_kafka_data.event_time),
                 self.get_tpf_jam_id(jvm_jmx_kafka_data.relational_data, parent, db_cursor),
                 jvm_jmx_kafka_data.relational_data.pid,
                 temp_producer_id,
                 jvm_jmx_kafka_data.req_total,
                 jvm_jmx_kafka_data.reqs_in_flight,
                 jvm_jmx_kafka_data.avg_batch_size,
                 jvm_jmx_kafka_data.rec_send_rate,
                 jvm_jmx_kafka_data.avg_rec_size,
                 jvm_jmx_kafka_data.avg_req_latency,
                 jvm_jmx_kafka_data.waiting_threads]]
        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

