#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging

from pymysql.cursors import Cursor

from tpf_jvm.jvm_data_types.jvm_jmx_data import JvmCustomMbeanData
from tpf_jvm.jvm_db_handlers.jvm_db_helper import JvmDbHelper
from tpf_jvm.tpf_jvm_data import JvmData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import ms_to_datetime, create_pre_sql_insert
from tpf_utils.json_utils import get_missing_keys

# This class provides an example of processing custom MBeans data.
# NOTE: The QjmbMemoryStat0Handler class is used to process data produced by the driver "JMBeansDrv"
#       This driver can be downloaded from the TPF Drivers download page: https://www.ibm.com/support/pages/node/598321
# You can follow this example with your own MBeans.
# When MBeans are configured and running, you can run tpf_monitor_kafka_topic.sh JVM to see the exact
# JSON structure of your customMbeanData. Then you can write the code to process your MBeans data.
# This data is displayed in Grafana dashboard 10. JMX Application Data.
# The "Select application" drop-down will call SQL that finds all tables with this
# naming convention "jvm_jmx_app_APPNAME_table" and extracts APPNAME for display in the drop-down.
# You can follow this convention for other MBean data if you want to be able to view
# any application from a single dashboard, or you can create a separate dashboard for each
# application for more control over how the data is displayed.

logger = logging.getLogger('tpf_zrtmc_analyzer')


class QjmbMemoryStat0Handler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jmx_app_JMBeansDrv_table'

    db_field_names = ['time', 'jam_id', 'pid', 'buff', 'ecb', 'frm', 'frm1mb', 'req_counter', 'swb', 'sequence_num']
    db_key_fields = {'time', 'jam_id', 'pid'}

    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)
    required_fields = ['BUFF', 'ECB', 'FRM', 'FRM1MB', 'ReqCounter', 'SWB', 'SequenceNumber']

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_jmx_mbean_data: JvmCustomMbeanData, parent: JvmData = None):

        try:
            buff = jvm_jmx_mbean_data.mbean_attributes['BUFF']
            ecb = jvm_jmx_mbean_data.mbean_attributes['ECB']
            frm = jvm_jmx_mbean_data.mbean_attributes['FRM']
            frm1mb = jvm_jmx_mbean_data.mbean_attributes['FRM1MB']
            req_counter = jvm_jmx_mbean_data.mbean_attributes['ReqCounter']
            swb = jvm_jmx_mbean_data.mbean_attributes['SWB']
            seq_num = jvm_jmx_mbean_data.mbean_attributes['SequenceNumber']
        except KeyError:
            log_message = f'Unable to deserialize QjmbMemoryMbean data. Missing fields: ' \
                          f'{get_missing_keys(jvm_jmx_mbean_data.mbean_attributes, self.required_fields)}. ' \
                          f'Data: {jvm_jmx_mbean_data.mbean_attributes}'

            logger.warning(log_message)
            return []

        rows = [[ms_to_datetime(jvm_jmx_mbean_data.event_time),
                 self.get_tpf_jam_id(jvm_jmx_mbean_data.relational_data, parent, db_cursor),
                 jvm_jmx_mbean_data.relational_data.pid,
                 buff,
                 int(ecb),
                 int(frm),
                 int(frm1mb),
                 int(req_counter),
                 int(swb),
                 int(seq_num)]]

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]
