#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from typing import Dict, Type
from exceptions.json_err import MissingFieldError
from tpf_jvm.jvm_data_types.jvm_cpu_data import JvmCpuData
from tpf_jvm.jvm_data_types.jvm_gc_data import JvmGcData
from tpf_jvm.jvm_data_types.jvm_jmx_data import JvmJmxData
from tpf_jvm.jvm_data_types.jvm_lock_data import JvmLockData
from tpf_jvm.tpf_jvm_data_type import JvmDataType
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmBody(object):
    jvm_data_types: Dict[str, Type[JvmDataType]] = {
        'JVM_CPU_DATA': JvmCpuData,
        'JVM_JMX_DATA': JvmJmxData,
        'JVM_GC_DATA': JvmGcData,
        'JVM_LOCK_DATA': JvmLockData,
        'default': JvmDataType
    }

    unhandled_jvm_types = set()

    required_fields_rlist = ['recordList']

    def __init__(self, json_jvm_body, jvm_type):
        """
        Initializes a JvmBody object.

        :param json_jvm_body: "body" dictionary from JVM JSON structure.
        :param jvm_type: JVM data type from JvmHeader object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # checks to see if data type is supported (cpu, jmx, gc, lock)
        if jvm_type in self.jvm_data_types:
            jvm_data_type = self.jvm_data_types[jvm_type]
        else:
            # chooses to make JvmDataType object for unsupported data types
            jvm_data_type = self.jvm_data_types['default']

            # notes and logs warning if the unsupported datatype has been encountered
            if jvm_type not in self.unhandled_jvm_types:
                self.unhandled_jvm_types.add(jvm_type)
                logger.warning(f'Encountered the following JVM data type for which there is no handler: '
                               f'{jvm_type}. Passing raw JSON for this type.')

        try:
            record_list = json_jvm_body["recordList"]
        except KeyError:
            log_message = f'Unable to deserialize JVM body. Missing fields: ' \
                          f'{get_missing_keys(json_jvm_body, self.required_fields_rlist)}. Data: {json_jvm_body}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

        self.record_list = []

        for record in record_list:
            record_object = jvm_data_type(record)
            self.record_list.append(record_object)
