#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys, get_or_default_or_error, fix_json_cpc_name
import logging


logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmHeader(object):
    max_version = 2

    required_fields = ['version', 'complex_name', 'processor_id', 'parameters']
    required_fields_v2 = required_fields + ["system_name", "cpc_name", "ss_name", "ip_address", "environment"]
    required_fields_param = ["data_type"]

    req_fields_versions = {
        0: required_fields,
        1: required_fields,
        2: required_fields_v2
    }

    def __init__(self, json_jvm_header):
        """
        Initializes a JvmHeader object.

        :param json_jvm_header: "header" python dictionary from JVM JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """
        self.version_num = 0

        try:
            # Base version
            self.version = json_jvm_header['version']
            self.version_num = int(float(self.version))
            self.complex_name = json_jvm_header['complex_name']
            self.processor_id = json_jvm_header['processor_id']
            parameters = json_jvm_header['parameters']

            # Version 2
            are_fields_required = self.version_num >= 2
            self.system_name = get_or_default_or_error(json_jvm_header, "system_name", key_error=are_fields_required)

            fix_json_cpc_name(json_jvm_header)

            self.cpc_name = get_or_default_or_error(json_jvm_header, "cpc_name", key_error=are_fields_required)
            self.ss_name = get_or_default_or_error(json_jvm_header, "ss_name", key_error=are_fields_required)
            self.ip_address = get_or_default_or_error(json_jvm_header, "ip_address", key_error=are_fields_required)
            self.environment = get_or_default_or_error(json_jvm_header, "environment", key_error=are_fields_required)
        except KeyError:
            req_fields = self.req_fields_versions.get(min(self.max_version, self.version_num), self.required_fields)
            log_message = f'Unable to deserialize JVM Header. ' \
                          f'Missing fields: {get_missing_keys(json_jvm_header, req_fields)}. ' \
                          f'Data: {json_jvm_header}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

        try:
            self.data_type = parameters['data_type']
        except KeyError:
            log_message = f'Unable to deserialize JVM Header parameters. Missing fields: ' \
                          f'{get_missing_keys(parameters, self.required_fields_param)}. Data: {parameters}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
