#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from typing import Dict, List

from exceptions.json_err import MissingFieldError
from tpf_data_manager.source_manager import SourceManager
from pymysql.cursors import Cursor

from tpf_jvm.jvm_db_handlers.jvm_cpu_db import JvmCpuHandler
from tpf_jvm.jvm_db_handlers.jvm_gc_db import JvmGcHandler
from tpf_jvm.jvm_db_handlers.jvm_jmx_db import JvmJmxHandler
from tpf_jvm.jvm_db_handlers.jvm_lock_db import JvmLockHandler
import logging

from tpf_jvm.tpf_jvm_data import JvmData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmManager(SourceManager):

    jvm_data_types = {"JVM_CPU_DATA": JvmCpuHandler,
                      "JVM_JMX_DATA": JvmJmxHandler,
                      "JVM_GC_DATA": JvmGcHandler,
                      "JVM_LOCK_DATA": JvmLockHandler}

    required_fields = jvm_data_types.keys()

    def __init__(self, active_source_types):
        # Sets self.active_source_types
        super().__init__(active_source_types)

        self.unhandled_data_types = set()
        self.active_jvm_data_types: Dict[str, DbDataHandler] = {}

        if active_source_types is not None:
            unrecognized_source_types = []

            for source_type in active_source_types:
                try:
                    self.active_jvm_data_types[source_type] = self.jvm_data_types[source_type]()
                except KeyError:
                    unrecognized_source_types.append(source_type)

            if len(unrecognized_source_types) > 0:
                msg = f'The following data types were not recognized: {unrecognized_source_types}.' \
                      f'The following are recognized JVM data types: {self.jvm_data_types.keys()}. Make ' \
                      f'sure the correct data types are specified in the data_sources.jvm.tpf_processing ' \
                      f'section of the ZRTMC Analyzer profile.'
                logger.error(msg)
                raise ValueError

        if active_source_types is None or len(active_source_types) == 0:
            msg = f'TPF processing is not enabled for JVM.'
            logger.info(msg)
        else:
            msg = f'TPF processing is enabled for the following JVM data types: {active_source_types}'
            logger.info(msg)

    def get_db_tables(self, active_only=True):
        table_names = []
        if active_only:
            for data_handler in self.active_jvm_data_types.values():
                table_names += data_handler.get_table_names()
        else:
            for data_handler in self.jvm_data_types.values():
                table_names += data_handler.get_table_names()
        return table_names

    def deserialize(self, json_doc):
        try:
            source = json_doc['header']['source']
            if source != 'RTMC_JVM' and source != 7:
                return None
            return JvmData(json_doc)
        except (KeyError, MissingFieldError):
            msg = f'Failed to deserialize a JSON document. Source: JVM. Data: {json_doc}'
            logger.error(msg)
            return None

    def get_mysql_insert_manys(self, db_cursor: Cursor, analytics_data: JvmData):
        insert_manys: List[InsertMany] = []
        jvm_data_type = analytics_data.header.data_type

        if jvm_data_type in self.active_jvm_data_types:
            data_type_handler = self.active_jvm_data_types[jvm_data_type]
        else:
            if jvm_data_type not in self.unhandled_data_types:
                msg = f'Encountered a JVM data type for which there is no active TPF processing. ' \
                      f'Data type: {jvm_data_type}'
                logger.warning(msg)
                self.unhandled_data_types.add(jvm_data_type)
            return insert_manys

        for record in analytics_data.body.record_list:
            insert_manys += data_type_handler.get_mysql_insert_manys(db_cursor, record, parent=analytics_data)
        return insert_manys
