#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class RelationalData(object):
    required_jam_fields = ['ProcessID', 'JAMName', 'JAMNum', 'JVMTotalThreads', 'StartTime',
                           'HealthCenterPort', 'ComFilename']
    required_non_jam_fields = ['ProcessID', 'ObjectName', 'StartTime', 'CmdLineArgs', 'HealthCenterPort']

    def __init__(self, relational_data: dict):
        """
        Initializes a RelationalData object.

        :param relational_data: "relationalData" dictionary from JVM JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # Need one of these to determine if this is a JAM app or not
        if 'JAMName' not in relational_data and 'ObjectName' not in relational_data:
            raise MissingFieldError(f'Unable to deserialize JVM Relational Data. Missing fields: '
                                    f'{get_missing_keys(relational_data, ["JAMName", "ObjectName"])}. '
                                    f'Data: {relational_data}')

        self.is_jam_app = 'JAMName' in relational_data

        if self.is_jam_app:
            try:
                self.pid = relational_data['ProcessID']
                self.group_name = relational_data['JAMName']
                self.num_jvms_expected = relational_data['JAMNum']
                self.num_threads_total = relational_data['JVMTotalThreads'] * self.num_jvms_expected
                self.start_time_ms = relational_data['StartTime']
                self.cmd_line_args = None
                self.health_center_port = relational_data['HealthCenterPort']
                self.com_file_name = relational_data['ComFilename']
            except KeyError:
                log_message = f'Unable to deserialize JAM app Relational Data. Missing fields: ' \
                              f'{get_missing_keys(relational_data, self.required_jam_fields)}. Data: {relational_data}'
                logger.error(log_message)
                raise MissingFieldError(log_message)
        else:
            try:
                self.pid = relational_data['ProcessID']
                self.group_name = relational_data['ObjectName']
                self.num_jvms_expected = None
                self.num_threads_total = None
                self.start_time_ms = relational_data['StartTime']
                self.cmd_line_args = relational_data['CmdLineArgs']
                self.health_center_port = relational_data['HealthCenterPort']
                self.com_file_name = None
            except KeyError:
                log_message = f'Unable to deserialize non-JAM app Relational Data. Missing fields: ' \
                              f'{get_missing_keys(relational_data, self.required_non_jam_fields)}. Data: {relational_data}'
                logger.error(log_message)
                raise MissingFieldError(log_message)

    def get_db_group_name(self, jam_db_field_len):
        """
        Truncates the group name (jam name or object name) by removing the beginning of the String.
        JAM names are limited to 16 characters, so this will only ever truncate an object name. The
        end of a Java object name is more valuable than the beginning (package names are less specific
        at the front), so we want to use the end.

        :param jam_db_field_len: The max length of the database field for jam name. This should be
        the same across all tables so that the jam name remains consistent.
        :return: The (possibly truncated) group name.
        :rtype: str
        """
        return self.group_name[-jam_db_field_len:]


