#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
import threading

from constants.profile_constants import *
from exceptions.json_err import MissingFieldError
from tpf_kafka_structure.tpf_kafka_configuration import KafkaConfiguration
from tpf_utils.json_utils import get_missing_keys
from kafka import KafkaConsumer
import uuid
SECOND_INTERVAL = 1000
INTERVAL = 5000

BYTE_MULTI = 1024


class KafkaContainer(object):
    """
        Container object for Kafka Consumer connection
    """

    def __init__(self, kafka_host_object: dict, curr_host_index: int, consumer_timeout: int):
        self.KafkaConsumer: KafkaConsumer = None
        self.consumer_timeout = consumer_timeout
        self.logger = logging.getLogger('tpf_zrtmc_analyzer')
        self.Initialized = False
        self.encryption_settings = []
        self.encryption_index = 0
        self.offset_reset: str = 'earliest'
        self.hosts = ""
        from tpf_utils.profile_utils import create_missing_profile_parameter_msg
        from tpf_utils.profile_utils import are_profile_parameters_present
        self.encrypted = False
        if ENCRYPTION in kafka_host_object:
            self.encrypted = True
            required_encryption_params = [SECURITY_PROTOCOL, SECURITY_SETTINGS]
            if not are_profile_parameters_present(kafka_host_object[ENCRYPTION], required_encryption_params, KAFKA,
                                                  log_missing=False):
                self.logger.info(create_missing_profile_parameter_msg(get_missing_keys(kafka_host_object[ENCRYPTION],
                                                                                       required_encryption_params),
                                                                      KAFKA))
                raise MissingFieldError(create_missing_profile_parameter_msg(get_missing_keys
                                                                             (kafka_host_object[ENCRYPTION],
                                                                              required_encryption_params), KAFKA))

            for encrypt_setting_config in kafka_host_object[ENCRYPTION][SECURITY_SETTINGS]:
                required_encryption_params = [CA_ROOT_PEM, CA_CERT_PEM, KEY_PASSWORD_PEM, PASSWORD_FOR_KEY_PEM]
                if not are_profile_parameters_present(encrypt_setting_config,
                                                      required_encryption_params, KAFKA,
                                                      log_missing=False):
                    self.logger.info(create_missing_profile_parameter_msg(
                        get_missing_keys(encrypt_setting_config,
                                         required_encryption_params),
                        KAFKA))
                    raise MissingFieldError(create_missing_profile_parameter_msg(get_missing_keys
                                                                                 (encrypt_setting_config,
                                                                                  required_encryption_params), KAFKA))

                current_kafka_encrypt_setting = KafkaConfiguration(kafka_host_object[HOSTS][curr_host_index],
                                                                   kafka_host_object[ENCRYPTION],
                                                                   encrypt_setting_config)
                self.encryption_settings.append(current_kafka_encrypt_setting)
        else:
            self.hosts = kafka_host_object[HOSTS][curr_host_index]

        self.group_id = kafka_host_object[PROG_VAR][GROUPID]
        self.fetch_min = kafka_host_object[PROG_VAR][FETCH_MIN_KB]

    def get_hosts(self) -> str:
        """
            Returns host currently being used.
        """
        if self.encrypted:
            cur_encrypt_index = self.encryption_index
            if cur_encrypt_index >= len(self.encryption_settings):
                cur_encrypt_index -= 1
            return self.encryption_settings[cur_encrypt_index].hosts
        else:
            return self.hosts

    def create_kafka_consumer(self):
        """
            Creates a kafka consumer based on the pre-defined configs.
        """
        if isinstance(self.hosts, list):
            self.hosts = ' '.join(self.hosts)
        if not self.encrypted:
            self.logger.info(
                f"Creating Kafka Consumer with these settings:\n{self.hosts}")
            self.KafkaConsumer = KafkaConsumer(bootstrap_servers=f'{self.hosts}',
                                               group_id=self.group_id,
                                               auto_offset_reset=self.offset_reset,
                                               enable_auto_commit=False,
                                               fetch_min_bytes=self.fetch_min * BYTE_MULTI,
                                               fetch_max_wait_ms=INTERVAL,
                                               session_timeout_ms=30000,
                                               request_timeout_ms=40000,
                                               max_poll_interval_ms=INTERVAL,
                                               reconnect_backoff_ms=1000,  # Wait 1 second between reconnection attempts
                                               #retry_backoff_ms=500,  # Wait 500ms between retry attempts
                                               consumer_timeout_ms=self.consumer_timeout)
            self.logger.info("Allocated Kafka Consumer")
        else:
            if self.encryption_index >= len(self.encryption_settings):
                self.encryption_index = 0
            current_encryption_settings: KafkaConfiguration = self.encryption_settings[self.encryption_index]
            self.encryption_index += 1
            file = open(current_encryption_settings.ssl_password_file, "r")
            first_line = file.readline()
            file.close()
            self.logger.info(f"Creating Encrypted Kafka Consumer with these settings:\n{str(current_encryption_settings)}")
            self.KafkaConsumer = KafkaConsumer(
                bootstrap_servers=f'{current_encryption_settings.hosts}',  # Kafka broker addresses to connect to
                group_id=self.group_id,  # Consumer group ID for tracking offsets
                security_protocol=current_encryption_settings.protocol,  # Security protocol used to connect to brokers
                auto_offset_reset=self.offset_reset,
                # What to do when there is no initial offset in Kafka or the current offset does not exist
                enable_auto_commit=False,  # Whether to commit offsets automatically
                # Maximum amount of time to block waiting for messages during a poll request
                ssl_check_hostname=False,
                # Whether to verify that the hostname in the broker's certificate matches the actual hostname
                ssl_cafile=current_encryption_settings.ssl_cafile,  # Path to a trusted CA certificate file
                ssl_certfile=current_encryption_settings.ssl_certfile,  # Path to the client certificate file
                ssl_keyfile=current_encryption_settings.ssl_keyfile,  # Path to the client private key file
                ssl_password=first_line,  # Password to decrypt the client private key file
                reconnect_backoff_ms=1000,  # Wait 1 second between reconnection attempts
                max_poll_interval_ms=INTERVAL,
                consumer_timeout_ms=self.consumer_timeout
                # Maximum time to block waiting for messages during a single poll request
            )
            self.logger.info("Allocated Encrypted Kafka Consumer")
        self.set_initialized(True)

    def get_kafka_consumer(self) -> KafkaConsumer:
        """
            Returns internal KafkaConsumer
        """
        return self.KafkaConsumer

    def initialized(self) -> bool:
        """
            Is this container currently running
        """
        return self.Initialized

    def set_initialized(self, status):
        """
            Set container status
        """
        self.Initialized = status;

    def __repr__(self):
        temp_index = 0
        if self.encryption_index >= len(self.encryption_settings):
            temp_index = self.encryption_index - 1
        return f"Current KafkaConsumer for this container {str(self.KafkaConsumer)} based on these settings\n" \
               f"{self.encryption_settings[temp_index]}"
