#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2024                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240403    PJ48015 - Vertical processing was limited to one      #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging
from typing import List

from exceptions.json_err import InvalidAppDimensionConfigError
from tpf_nvpc.nvpc_config.orientation import Orientation
from tpf_nvpc.tpf_nvpc_namevaluepair import NameValuePair
from tpf_utils.json_utils import to_json_string
from tpf_utils.profile_utils import are_profile_parameters_present, create_invalid_profile_parameter_msg

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpName(object):
    required_params = ['name', 'orientation']
    orientation_valid_values = {'horizontal', 'vertical'}

    def __init__(self, name, orientation: Orientation):
        """
        Initializes a NvpName object.

        :param name: The name of this Nvp
        :param orientation: The orientation of this Nvp.
        """
        self.name = name
        self.orientation = orientation

    @classmethod
    def from_profile(cls, nvp_name_config, profile_section=None):
        """
        Initializes a NvpName object from a dictionary. This function logs any missing or invalid parameters.

        :param profile_section: The section of the tpf_zrtmc_analyzer_profile for this NvpName
        :param nvp_name_config: dict containing keys 'name' and 'orientation'
        :return: An NvpName object.
        :raise InvalidAppDimensionConfigError: If nvp_name_config is missing a required key or has an invalid parameter.
        """
        if not are_profile_parameters_present(nvp_name_config, cls.required_params, profile_section, log_missing=True):
            raise InvalidAppDimensionConfigError()

        name = nvp_name_config['name']

        if name is None:
            logger.error(create_invalid_profile_parameter_msg('name',
                                                              profile_section,
                                                              'null',
                                                              'Must be of type string'))
            raise InvalidAppDimensionConfigError()

        try:
            orientation: Orientation = Orientation[nvp_name_config['orientation'].upper()]
        except KeyError:
            logger.error(create_invalid_profile_parameter_msg('orientation',
                                                              profile_section,
                                                              nvp_name_config['orientation'],
                                                              f'Must be one of the following values: '
                                                              f'{cls.orientation_valid_values}'))
            raise InvalidAppDimensionConfigError()

        return cls(name, orientation)

    def is_horizontal(self):
        return self.orientation == Orientation.HORIZONTAL

    def is_vertical(self):
        return self.orientation == Orientation.VERTICAL

    def matches_name(self, nvp):
        if isinstance(nvp, NameValuePair):
            return nvp.orientation_enum == self.orientation and nvp.name == self.name
        else:
            return nvp.orientation == self.orientation and nvp.name == self.name


class AppDimension(object):
    required_params = ['dimension', 'nvp_names']

    def __init__(self, dimension, nvp_names: List[NvpName], is_system_indicator=False):
        """
        Initializes an AppDimension object.

        :param dimension: The name of the dimension
        :param nvp_names: List of NvpName objects
        """
        self.dimension = dimension
        self.nvp_names = nvp_names
        self.num_verticals = sum(1 for n in self.nvp_names if n.is_vertical())
        self.is_system_indicator = is_system_indicator

    @classmethod
    def from_profile(cls, app_dimension_config, profile_section=None):
        """
        Initializes an AppDimension object from a dictionary. This function logs any missing or invalid parameters.

        :param app_dimension_config: dict containing keys 'dimension' and 'nvp_names'
        :param profile_section: The section of the tpf_zrtmc_analyzer_profile for this AppDimension
        :return: An AppDimension object.
        :raise InvalidAppDimensionConfigError: If app_dimension_config is missing a required key or has an invalid parameter.
        """
        if not are_profile_parameters_present(app_dimension_config, cls.required_params, profile_section, log_missing=True):
            raise InvalidAppDimensionConfigError()

        dimension = app_dimension_config['dimension']

        if dimension is None:
            logger.error(create_invalid_profile_parameter_msg('dimension', profile_section,
                                                              'null', 'Must be of type string'))
            raise InvalidAppDimensionConfigError()

        is_system_indicator = False
        if 'system_indicator' in app_dimension_config:
            is_system_indicator = app_dimension_config['system_indicator']

            if not isinstance(is_system_indicator, bool):
                logger.error(create_invalid_profile_parameter_msg('system_indicator', profile_section,
                                                                  is_system_indicator, 'Must be of type boolean'))
                raise InvalidAppDimensionConfigError()

        config_nvp_names = app_dimension_config['nvp_names']

        if config_nvp_names is None or len(config_nvp_names) == 0:
            logger.error(create_invalid_profile_parameter_msg('nvp_names', profile_section,
                                                              '[]', 'Must have length > 0'))
            raise InvalidAppDimensionConfigError()

        nvp_names = []
        valid_config = True

        for i in range(len(config_nvp_names)):
            try:
                nvp_names.append(NvpName.from_profile(
                    config_nvp_names[i],
                    profile_section='{0}.nvp_names[{1}]'.format(profile_section or "", i)))
            except InvalidAppDimensionConfigError:
                valid_config = False

        if not valid_config:
            raise InvalidAppDimensionConfigError()

        app_dimension = cls(dimension, nvp_names, is_system_indicator=is_system_indicator)

        for dup_index in app_dimension._check_duplicate_nvp_names():
            valid_config = False
            logger.error(create_invalid_profile_parameter_msg('nvp_names[{0}]'.format(dup_index),
                                                              profile_section,
                                                              to_json_string(config_nvp_names[dup_index]),
                                                              'All nvp_names in an application_dimension must be unique'))

        if not valid_config:
            raise InvalidAppDimensionConfigError()

        return app_dimension

    def generate_dimension_value(self, nvps: List[NameValuePair]):
        """
        Generates the dimension value given a list of name-value pairs. If one or more of the
        NvpNames for this AppDimension is/are missing from the name-value pairs, this function
        returns None.

        For AppDimensions with a single NvpName, the name-value pair value is returned. For
        multiple NvpNames, the name-value pair values are returned as a comma-separated list
        enclosed in square brackets.

        :param nvps: List of NameValuePair objects.
        :return: The dimension value string
        """
        # Not enough nvps, we don't have them all
        if len(nvps) < len(self.nvp_names):
            return None

        nvp_values = []

        # Iterate through self.nvp_names because this is the specified order for values
        for nvp_name in self.nvp_names:
            nvp_value = next((nvp.value for nvp in nvps if nvp_name.matches_name(nvp)), None)

            # NvpName is missing
            if nvp_value is None:
                return None
            else:
                nvp_values.append(nvp_value)

        if len(nvp_values) == 1:
            return nvp_values[0]
        else:
            return f'[{", ".join(nvp_values)}]'

    def _check_duplicate_nvp_names(self):
        # Start at end of self.nvp_names, check each nvp_name before it for duplicates
        # This gives us the positions of all the duplicates
        # Reverse the list so the indexes start from the front
        return [i for i in range(len(self.nvp_names) - 1, 0, -1)
                if next((n for n in self.nvp_names[:i] if self.nvp_names[i].matches_name(n)), None)][::-1]


class AppDimensionConfigValidator(object):

    def __init__(self, max_dimension_len=64, num_verticals=1):
        """
        Initializes an AppDimensionConfigValidator.

        :param max_dimension_len: The maximum length of a dimension.
        :param num_verticals: The valid number(s) of verticals for validating vertical AppDimensions. This can be
        a number, or a set of numbers.
        """
        self.num_verticals = num_verticals
        self.max_dimension_len = max_dimension_len
        self._validated = {}
        self._profile_sections = {}

    def validate(self, app_dimensions_config, orientation: Orientation, profile_section=None):
        """
        Validates an app_dimension config dict. This function logs any missing or invalid parameters.

        :param app_dimensions_config: List of app_dimension config dicts .
        :param orientation: The Orientation to validiate for.
        :param profile_section: The section of the tpf_zrtmc_analyzer_profile for this app_dimensions_config.
        :return: list of AppDimensions
        :raise InvalidAppDimensionConfigError: If app_dimensions_config is missing a required key or has an invalid
        parameter.
        """
        app_dimensions = []

        if app_dimensions_config is None or len(app_dimensions_config) == 0:
            return app_dimensions

        valid_config = True

        for i in range(len(app_dimensions_config)):
            cur_profile_section = '{0}[{1}]'.format(profile_section or "", i)
            try:
                app_dimension = AppDimension.from_profile(app_dimensions_config[i],
                                                          profile_section=cur_profile_section)
            except InvalidAppDimensionConfigError:
                valid_config = False
                continue

            app_dimensions.append(app_dimension)

            if len(app_dimension.dimension) > self.max_dimension_len:
                valid_config = False
                logger.error(create_invalid_profile_parameter_msg('dimension',
                                                                  cur_profile_section,
                                                                  app_dimension.dimension,
                                                                  f'Must have length <= {self.max_dimension_len}'))

            if orientation == Orientation.VERTICAL and app_dimension.num_verticals > self.num_verticals:
                valid_config = False
                logger.error(create_invalid_profile_parameter_msg('nvp_names',
                                                                  cur_profile_section,
                                                                  app_dimension.num_verticals,
                                                                  f'Number of verticals exceeds limit:'
                                                                  f' {self.num_verticals} defined in config"'))
            elif orientation == Orientation.HORIZONTAL and app_dimension.num_verticals != 0:
                valid_config = False
                logger.error(create_invalid_profile_parameter_msg('nvp_names',
                                                                  cur_profile_section,
                                                                  app_dimension.num_verticals,
                                                                  f'Number of verticals must be 0 for '
                                                                  f'horizontal dimensions'))

        if not valid_config:
            raise InvalidAppDimensionConfigError()
        else:
            self._validated[orientation] = app_dimensions
            self._profile_sections[orientation] = profile_section
            return app_dimensions

    def validate_group(self):
        """
        Validates the last horizontal and vertical AppDimensions validated using the validate function of this
        AppDimensionConfigValidator, one for each orientation. The group must have no duplicate dimensions and
        no more than one system_indicator.

        :raise InvalidAppDimensionConfigError: If the group of AppDimensions is invalid.
        """
        horizontals = self._validated.get(Orientation.HORIZONTAL, [])
        hor_profile_section = self._profile_sections.get(Orientation.HORIZONTAL)

        verticals = self._validated.get(Orientation.VERTICAL, [])
        vert_profile_section = self._profile_sections.get(Orientation.VERTICAL)

        app_dimensions = horizontals + verticals

        valid_config = True
        for dup_index in self._check_duplicate_dimensions(app_dimensions):
            valid_config = False
            i = dup_index
            profile_section = hor_profile_section

            if i >= len(horizontals):
                i -= len(horizontals)
                profile_section = vert_profile_section

            logger.error(create_invalid_profile_parameter_msg('dimension',
                                                              '{0}[{1}]'.format(profile_section or "", i),
                                                              app_dimensions[dup_index].dimension,
                                                              'All dimensions must be unique'))

        for dup_index in self._check_duplicate_system_indicators(app_dimensions):
            valid_config = False
            i = dup_index
            profile_section = hor_profile_section

            if i >= len(horizontals):
                i -= len(horizontals)
                profile_section = vert_profile_section

            logger.error(create_invalid_profile_parameter_msg('system_indicator',
                                                              '{0}[{1}]'.format(profile_section or "", i),
                                                              app_dimensions[dup_index].is_system_indicator,
                                                              'Only one application_dimension can be marked '
                                                              'as system_indicator'))

        if not valid_config:
            raise InvalidAppDimensionConfigError()

    def _check_duplicate_dimensions(self, app_dimensions: List[AppDimension]):
        return [i for i in range(len(app_dimensions) - 1, 0, -1)
                if next((a for a in app_dimensions[:i] if app_dimensions[i].dimension == a.dimension), None)][::-1]

    def _check_duplicate_system_indicators(self, app_dimensions: List[AppDimension]):
        return [i for i in range(len(app_dimensions) - 1, 0, -1)
                if next((a for a in app_dimensions[:i]
                         if a.is_system_indicator and app_dimensions[i].is_system_indicator == a.is_system_indicator), None)][::-1]

    def get_system_indicator(self):
        """
        Gets the system_indicator dimension, or None if there is no system_indicator.

        :return: The system_indicator dimension
        :rtype: AppDimension
        """
        validated = self._validated.get(Orientation.HORIZONTAL, []) + self._validated.get(Orientation.VERTICAL, [])
        return next((a for a in validated if a.is_system_indicator), None)
