#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingNvpcProcessingMetrics
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpcAggMetrics(object):

    required_fields = ['CPU_USED', 'CRET', 'EXIST_TIME', 'FILE', 'FIND']

    def __init__(self, group, msg_cnt, metrics):
        """
        Initialize instance of an NvpcAggMetrics object.

        :param group: variable that groups the record metrics
        :param msg_cnt: a message_count variable from NvpcRecord object
        :param metrics: a metrics variable from NvpcRecord object
        :raise MissingNvpcProcessingMetrics: if missing key value in metrics parameter
        """
        self.group = group
        self.msg_cnt = msg_cnt
        self.total = dict()
        try:
            # required fields
            self.total['CPU_USED'] = msg_cnt * metrics['CPU_USED']
            self.total['EXIST_TIME'] = msg_cnt * metrics['EXIST_TIME']
            self.total['CRET'] = msg_cnt * metrics['CRET']
            self.total['FIND_FILE'] = msg_cnt * (metrics['FILE'] + metrics['FIND'])
            self.total['FIND'] = msg_cnt * metrics['FIND']
            self.total['FILE'] = msg_cnt * metrics['FILE']

            # longs
            self.total['DD_TIME'] = msg_cnt * metrics['DD_TIME'] if 'DD_TIME' in metrics else None
            self.total['ECB1MF'] = msg_cnt * metrics['ECB1MF'] if 'ECB1MF' in metrics else None
            self.total['ECB4KF'] = msg_cnt * metrics['ECB4KF'] if 'ECB4KF' in metrics else None
            self.total['RLOG'] = msg_cnt * metrics['RLOG'] if 'RLOG' in metrics else None

            # ints
            self.total['CMBK'] = msg_cnt * metrics['CMBK'] if 'CMBK' in metrics else None
            self.total['COW'] = msg_cnt * metrics['COW'] if 'COW' in metrics else None
            self.total['DF_ATTAC'] = msg_cnt * metrics['DF_ATTAC'] if 'DF_ATTAC' in metrics else None
            self.total['DF_BTFIL'] = msg_cnt * metrics['DF_BTFIL'] if 'DF_BTFIL' in metrics else None
            self.total['DF_BTFND'] = msg_cnt * metrics['DF_BTFND'] if 'DF_BTFND' in metrics else None
            self.total['DF_CFILE'] = msg_cnt * metrics['DF_CFILE'] if 'DF_CFILE' in metrics else None
            self.total['DF_CFIND'] = msg_cnt * metrics['DF_CFIND'] if 'DF_CFIND' in metrics else None
            self.total['DF_CLS_NO_REL'] = msg_cnt * metrics['DF_CLS_NO_REL'] if 'DF_CLS_NO_REL' in metrics else None
            self.total['DF_CLS_OPN'] = msg_cnt * metrics['DF_CLS_OPN'] if 'DF_CLS_OPN' in metrics else None
            self.total['DF_CLS_RELS'] = msg_cnt * metrics['DF_CLS_RELS'] if 'DF_CLS_RELS' in metrics else None
            self.total['DF_CLS_REUSE'] = msg_cnt * metrics['DF_CLS_REUSE'] if 'DF_CLS_REUSE' in metrics else None
            self.total['DF_DBADD'] = msg_cnt * metrics['DF_DBADD'] if 'DF_DBADD' in metrics else None
            self.total['DF_DBCLS'] = msg_cnt * metrics['DF_DBCLS'] if 'DF_DBCLS' in metrics else None
            self.total['DF_DBDEL'] = msg_cnt * metrics['DF_DBDEL'] if 'DF_DBDEL' in metrics else None
            self.total['DF_DBDIX'] = msg_cnt * metrics['DF_DBDIX'] if 'DF_DBDIX' in metrics else None
            self.total['DF_DBIDX'] = msg_cnt * metrics['DF_DBIDX'] if 'DF_DBIDX' in metrics else None
            self.total['DF_DBKEY'] = msg_cnt * metrics['DF_DBKEY'] if 'DF_DBKEY' in metrics else None
            self.total['DF_DBMOD'] = msg_cnt * metrics['DF_DBMOD'] if 'DF_DBMOD' in metrics else None
            self.total['DF_DBOPN'] = msg_cnt * metrics['DF_DBOPN'] if 'DF_DBOPN' in metrics else None
            self.total['DF_DBRED'] = msg_cnt * metrics['DF_DBRED'] if 'DF_DBRED' in metrics else None
            self.total['DF_DBREP'] = msg_cnt * metrics['DF_DBREP'] if 'DF_DBREP' in metrics else None
            self.total['DF_DETAC'] = msg_cnt * metrics['DF_DETAC'] if 'DF_DETAC' in metrics else None
            self.total['DF_GETCC'] = msg_cnt * metrics['DF_GETCC'] if 'DF_GETCC' in metrics else None
            self.total['DF_GETFC'] = msg_cnt * metrics['DF_GETFC'] if 'DF_GETFC' in metrics else None
            self.total['DF_LRFIL'] = msg_cnt * metrics['DF_LRFIL'] if 'DF_LRFIL' in metrics else None
            self.total['DF_LRFND'] = msg_cnt * metrics['DF_LRFND'] if 'DF_LRFND' in metrics else None
            self.total['DF_OPN_HD'] = msg_cnt * metrics['DF_OPN_HD'] if 'DF_OPN_HD' in metrics else None
            self.total['DF_OPN_HND'] = msg_cnt * metrics['DF_OPN_HND'] if 'DF_OPN_HND' in metrics else None
            self.total['DF_OPN_NHD'] = msg_cnt * metrics['DF_OPN_NHD'] if 'DF_OPN_NHD' in metrics else None
            self.total['DF_OPN_NHND'] = msg_cnt * metrics['DF_OPN_NHND'] if 'DF_OPN_NHND' in metrics else None
            self.total['DF_PFILE'] = msg_cnt * metrics['DF_PFILE'] if 'DF_PFILE' in metrics else None
            self.total['DF_PFIND'] = msg_cnt * metrics['DF_PFIND'] if 'DF_PFIND' in metrics else None
            self.total['DF_RELCC'] = msg_cnt * metrics['DF_RELCC'] if 'DF_RELCC' in metrics else None
            self.total['DF_RELFC'] = msg_cnt * metrics['DF_RELFC'] if 'DF_RELFC' in metrics else None
            self.total['DF_RSF_CREATE'] = msg_cnt * metrics['DF_RSF_CREATE'] if 'DF_RSF_CREATE' in metrics else None
            self.total['DF_RSF_DELETE'] = msg_cnt * metrics['DF_RSF_DELETE'] if 'DF_RSF_DELETE' in metrics else None
            self.total['DF_RSF_READ'] = msg_cnt * metrics['DF_RSF_READ'] if 'DF_RSF_READ' in metrics else None
            self.total['DF_RSF_UPDATE'] = msg_cnt * metrics['DF_RSF_UPDATE'] if 'DF_RSF_UPDATE' in metrics else None
            self.total['FILE_DASD'] = msg_cnt * metrics['FILE_DASD'] if 'FILE_DASD' in metrics else None
            self.total['FILE_VFA'] = msg_cnt * metrics['FILE_VFA'] if 'FILE_VFA' in metrics else None
            self.total['FIND_DASD'] = msg_cnt * metrics['FIND_DASD'] if 'FIND_DASD' in metrics else None
            self.total['FIND_VFA'] = msg_cnt * metrics['FIND_VFA'] if 'FIND_VFA' in metrics else None
            self.total['GFSL'] = msg_cnt * metrics['GFSL'] if 'GFSL' in metrics else None
            self.total['GFSS'] = msg_cnt * metrics['GFSS'] if 'GFSS' in metrics else None
            self.total['GRFS'] = msg_cnt * metrics['GRFS'] if 'GRFS' in metrics else None
            self.total['HCTL'] = msg_cnt * metrics['HCTL'] if 'HCTL' in metrics else None
            self.total['MAL'] = msg_cnt * metrics['MAL'] if 'MAL' in metrics else None
            self.total['RELF'] = msg_cnt * metrics['RELF'] if 'RELF' in metrics else None
            self.total['ROUT'] = msg_cnt * metrics['ROUT'] if 'ROUT' in metrics else None
            self.total['SERR'] = msg_cnt * metrics['SERR'] if 'SERR' in metrics else None
            self.total['SNAP'] = msg_cnt * metrics['SNAP'] if 'SNAP' in metrics else None
            self.total['SWBK'] = msg_cnt * metrics['SWBK'] if 'SWBK' in metrics else None
            self.total['SYSH'] = msg_cnt * metrics['SYSH'] if 'SYSH' in metrics else None
            self.total['TOUR'] = msg_cnt * metrics['TOUR'] if 'TOUR' in metrics else None
            self.total['TWRT'] = msg_cnt * metrics['TWRT'] if 'TWRT' in metrics else None
            self.total['XWBK'] = msg_cnt * metrics['XWBK'] if 'XWBK' in metrics else None

        except KeyError:
            msg = f"Required metrics for TPF name-value pair processing were missing. " \
                  f"Make sure these metrics are included in the RTMC Properties file. " \
                  f"Missing metrics: {get_missing_keys(metrics, self.required_fields)}. " \
                  f"Required metrics: {self.required_fields}."
            logger.error(msg)
            raise MissingNvpcProcessingMetrics(msg)

    def add_metrics(self, msg_cnt, metrics):
        """
        Update the total metrics values for group with new metrics values.

        :param msg_cnt: a message_count variable from NvpcRecord object
        :param metrics: a metrics variable from NvpcRecord object
        :raise MissingNvpcProcessingMetrics: if missing key value in metrics parameter
        """
        self.msg_cnt += msg_cnt
        try:
            # required fields
            self.total['CPU_USED'] += msg_cnt * metrics['CPU_USED']
            self.total['EXIST_TIME'] += msg_cnt * metrics['EXIST_TIME']
            self.total['CRET'] += msg_cnt * metrics['CRET']
            self.total['FIND_FILE'] += msg_cnt * (metrics['FILE'] + metrics['FIND'])
            self.total['FIND'] += msg_cnt * metrics['FIND']
            self.total['FILE'] += msg_cnt * metrics['FILE']

            # longs
            if 'DD_TIME' in metrics: self.total['DD_TIME'] += msg_cnt * metrics['DD_TIME']
            if 'ECB1MF' in metrics: self.total['ECB1MF'] += msg_cnt * metrics['ECB1MF']
            if 'ECB4KF' in metrics: self.total['ECB4KF'] += msg_cnt * metrics['ECB4KF']
            if 'RLOG' in metrics: self.total['RLOG'] += msg_cnt * metrics['RLOG']

            # ints
            if 'CMBK' in metrics: self.total['CMBK'] += msg_cnt * metrics['CMBK']
            if 'COW' in metrics: self.total['COW'] += msg_cnt * metrics['COW']
            if 'DF_ATTAC' in metrics: self.total['DF_ATTAC'] += msg_cnt * metrics['DF_ATTAC']
            if 'DF_BTFIL' in metrics: self.total['DF_BTFIL'] += msg_cnt * metrics['DF_BTFIL']
            if 'DF_BTFND' in metrics: self.total['DF_BTFND'] += msg_cnt * metrics['DF_BTFND']
            if 'DF_CFILE' in metrics: self.total['DF_CFILE'] += msg_cnt * metrics['DF_CFILE']
            if 'DF_CFIND' in metrics: self.total['DF_CFIND'] += msg_cnt * metrics['DF_CFIND']
            if 'DF_CLS_NO_REL' in metrics: self.total['DF_CLS_NO_REL'] += msg_cnt * metrics['DF_CLS_NO_REL']
            if 'DF_CLS_OPN' in metrics: self.total['DF_CLS_OPN'] += msg_cnt * metrics['DF_CLS_OPN']
            if 'DF_CLS_RELS' in metrics: self.total['DF_CLS_RELS'] += msg_cnt * metrics['DF_CLS_RELS']
            if 'DF_CLS_REUSE' in metrics: self.total['DF_CLS_REUSE'] += msg_cnt * metrics['DF_CLS_REUSE']
            if 'DF_DBADD' in metrics: self.total['DF_DBADD'] += msg_cnt * metrics['DF_DBADD']
            if 'DF_DBCLS' in metrics: self.total['DF_DBCLS'] += msg_cnt * metrics['DF_DBCLS']
            if 'DF_DBDEL' in metrics: self.total['DF_DBDEL'] += msg_cnt * metrics['DF_DBDEL']
            if 'DF_DBDIX' in metrics: self.total['DF_DBDIX'] += msg_cnt * metrics['DF_DBDIX']
            if 'DF_DBIDX' in metrics: self.total['DF_DBIDX'] += msg_cnt * metrics['DF_DBIDX']
            if 'DF_DBKEY' in metrics: self.total['DF_DBKEY'] += msg_cnt * metrics['DF_DBKEY']
            if 'DF_DBMOD' in metrics: self.total['DF_DBMOD'] += msg_cnt * metrics['DF_DBMOD']
            if 'DF_DBOPN' in metrics: self.total['DF_DBOPN'] += msg_cnt * metrics['DF_DBOPN']
            if 'DF_DBRED' in metrics: self.total['DF_DBRED'] += msg_cnt * metrics['DF_DBRED']
            if 'DF_DBREP' in metrics: self.total['DF_DBREP'] += msg_cnt * metrics['DF_DBREP']
            if 'DF_DETAC' in metrics: self.total['DF_DETAC'] += msg_cnt * metrics['DF_DETAC']
            if 'DF_GETCC' in metrics: self.total['DF_GETCC'] += msg_cnt * metrics['DF_GETCC']
            if 'DF_GETFC' in metrics: self.total['DF_GETFC'] += msg_cnt * metrics['DF_GETFC']
            if 'DF_LRFIL' in metrics: self.total['DF_LRFIL'] += msg_cnt * metrics['DF_LRFIL']
            if 'DF_LRFND' in metrics: self.total['DF_LRFND'] += msg_cnt * metrics['DF_LRFND']
            if 'DF_OPN_HD' in metrics: self.total['DF_OPN_HD'] += msg_cnt * metrics['DF_OPN_HD']
            if 'DF_OPN_HND' in metrics: self.total['DF_OPN_HND'] += msg_cnt * metrics['DF_OPN_HND']
            if 'DF_OPN_NHD' in metrics: self.total['DF_OPN_NHD'] += msg_cnt * metrics['DF_OPN_NHD']
            if 'DF_OPN_NHND' in metrics: self.total['DF_OPN_NHND'] += msg_cnt * metrics['DF_OPN_NHND']
            if 'DF_PFILE' in metrics: self.total['DF_PFILE'] += msg_cnt * metrics['DF_PFILE']
            if 'DF_PFIND' in metrics: self.total['DF_PFIND'] += msg_cnt * metrics['DF_PFIND']
            if 'DF_RELCC' in metrics: self.total['DF_RELCC'] += msg_cnt * metrics['DF_RELCC']
            if 'DF_RELFC' in metrics: self.total['DF_RELFC'] += msg_cnt * metrics['DF_RELFC']
            if 'DF_RSF_CREATE' in metrics: self.total['DF_RSF_CREATE'] += msg_cnt * metrics['DF_RSF_CREATE']
            if 'DF_RSF_DELETE' in metrics: self.total['DF_RSF_DELETE'] += msg_cnt * metrics['DF_RSF_DELETE']
            if 'DF_RSF_READ'   in metrics: self.total['DF_RSF_READ'] += msg_cnt * metrics['DF_RSF_READ']
            if 'DF_RSF_UPDATE' in metrics: self.total['DF_RSF_UPDATE'] += msg_cnt * metrics['DF_RSF_UPDATE']
            if 'FILE_DASD' in metrics: self.total['FILE_DASD'] += msg_cnt * metrics['FILE_DASD']
            if 'FILE_VFA'  in metrics: self.total['FILE_VFA'] += msg_cnt * metrics['FILE_VFA']
            if 'FIND_DASD' in metrics: self.total['FIND_DASD'] = msg_cnt * metrics['FIND_DASD']
            if 'FIND_VFA'  in metrics: self.total['FIND_VFA'] = msg_cnt * metrics['FIND_VFA']
            if 'GFSL' in metrics: self.total['GFSL'] = msg_cnt * metrics['GFSL']
            if 'GFSS' in metrics: self.total['GFSS'] = msg_cnt * metrics['GFSS']
            if 'GRFS' in metrics: self.total['GRFS'] = msg_cnt * metrics['GRFS']
            if 'HCTL' in metrics: self.total['HCTL'] = msg_cnt * metrics['HCTL']
            if 'MAL'  in metrics: self.total['MAL'] = msg_cnt * metrics['MAL']
            if 'RELF' in metrics: self.total['RELF'] = msg_cnt * metrics['RELF']
            if 'ROUT' in metrics: self.total['ROUT'] = msg_cnt * metrics['ROUT']
            if 'SERR' in metrics: self.total['SERR'] = msg_cnt * metrics['SERR']
            if 'SNAP' in metrics: self.total['SNAP'] = msg_cnt * metrics['SNAP']
            if 'SWBK' in metrics: self.total['SWBK'] = msg_cnt * metrics['SWBK']
            if 'SYSH' in metrics: self.total['SYSH'] = msg_cnt * metrics['SYSH']
            if 'TOUR' in metrics: self.total['TOUR'] = msg_cnt * metrics['TOUR']
            if 'TWRT' in metrics: self.total['TWRT'] = msg_cnt * metrics['TWRT']
            if 'XWBK' in metrics: self.total['XWBK'] = msg_cnt * metrics['XWBK']

        except KeyError:
            msg = f"Required metrics for TPF name-value pair processing were missing. " \
                  f"Make sure these metrics are included in the RTMC Properties file. " \
                  f"Missing metrics: {get_missing_keys(metrics, self.required_fields)}. " \
                  f"Required metrics: {self.required_fields}."
            logger.error(msg)
            raise MissingNvpcProcessingMetrics(msg)
