#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   10052022    PJ46853 - Change skip factor to fractional scale      #
#                         factor                                      #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20240304    PJ47249 - Runtime metric collection discards          #
#   name-value pair metric data silently when configured incorrectly  #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging
from typing import Dict, List, Iterable

from pymysql.cursors import Cursor

from tpf_nvpc.nvpc_config.app_dimension import AppDimension
from tpf_nvpc.nvpc_db_handlers.nvpc_agg_metrics import NvpcAggMetrics
from tpf_nvpc.nvpc_db_handlers.nvpc_db_helper import NvpcDbHelper
from tpf_nvpc.nvpc_db_handlers.nvpc_sys_indicator_db import SystemIndicatorHandler, system_indicator_procs
from tpf_nvpc.tpf_nvpc_data import NvpcData
from tpf_nvpc.tpf_nvpc_record import NvpcRecord
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_nvpc_avg_pre_sql_insert, calculate_duration

logger = logging.getLogger('tpf_zrtmc_analyzer')


class AppDimensionHandler(DbDataHandler, NvpcDbHelper):
    table_name = 'nvp_message_metrics_table'
    required_fields = ['CPU_USED', 'CRET', 'EXIST_TIME', 'FILE', 'FIND']

    db_field_names = ['time', 'duration', 'avg_nvp_scale_factor', 'value_id', 'msg_cnt',
                      # required metrics
                      'cpu_used', 'exist_time', 'cret', 'find_file', 'find', 'file',
                      # long metrics
                      'dd_time', 'ecb1mf', 'ecb4kf', 'rlog',
                      # int metrics
                      'cmbk', 'cow',
                      'df_attac', 'df_btfil', 'df_btfnd', 'df_cfile', 'df_cfind', 'df_cls_no_rel',
                      'df_cls_opn', 'df_cls_rels', 'df_cls_reuse', 'df_dbadd', 'df_dbcls',
                      'df_dbdel', 'df_dbdix', 'df_dbidx', 'df_dbkey', 'df_dbmod', 'df_dbopn',
                      'df_dbred', 'df_dbrep', 'df_detac', 'df_getcc', 'df_getfc', 'df_lrfil',
                      'df_lrfnd', 'df_opn_hd', 'df_opn_hnd', 'df_opn_nhd', 'df_opn_nhnd',
                      'df_pfile', 'df_pfind', 'df_relcc', 'df_relfc', 'df_rsf_create',
                      'df_rsf_delete', 'df_rsf_read', 'df_rsf_update',
                      'file_dasd', 'file_vfa', 'find_dasd', 'find_vfa',
                      'gfsl', 'gfss', 'grfs', 'hctl', 'mal', 'relf', 'rout',
                      'serr', 'snap', 'swbk', 'sysh', 'tour', 'twrt', 'xwbk']
    db_key_fields = {'time', 'value_id'}
    pre_sql_insert = create_nvpc_avg_pre_sql_insert(table_name, db_field_names, db_key_fields)

    db_field_names_v2 = db_field_names + ['system_id']
    db_key_fields_v2 = db_key_fields.union({'system_id'})
    pre_sql_insert_v2 = create_nvpc_avg_pre_sql_insert(table_name, db_field_names_v2, db_key_fields_v2)

    def __init__(self, app_dimension: AppDimension):
        self.app_dimension = app_dimension
        self.missing_nvps_count = 0

    def _nvpc_record_filter(self, nvpc_record: NvpcRecord):
        return True

    def _nvpc_data_filter(self, nvpc_data: NvpcData):
        return not nvpc_data.is_vertical

    def aggregate_metrics(self, data: NvpcData):
        dimension_values: Dict[str, NvpcAggMetrics] = {}
        for record in data.body.record_list:
            if self._nvpc_record_filter(record):
                val = self.app_dimension.generate_dimension_value(record.name_value_pairs)

                # If val is None, some NVPs were missing. We can't use this record for this dimension.
                if val is not None:
                    if val in dimension_values:
                        dimension_values[val].add_metrics(record.message_count, record.metrics)
                    else:
                        dimension_values[val] = NvpcAggMetrics(val, record.message_count, record.metrics)

        return dimension_values

    def _create_rows(self, data: NvpcData, dimension_values: Iterable[NvpcAggMetrics], db_cursor: Cursor):
        # Base version
        v_insert = self.pre_sql_insert
        system_id = None

        # Version 2
        if data.header.version_num >= 2:
            v_insert = self.pre_sql_insert_v2
            system_id = self.get_tpf_system_id(data.header, db_cursor)

        time = data.header.collection_interval_utc_time
        duration = calculate_duration(data.header.collection_start_utc_time, time)
        if data.header.average_scale_factor != 0:
            scale_factor = data.header.average_scale_factor
        else:
            #this is a migration path for backlevel versions of TPF
            scale_factor = data.header.average_skip_factor
            if scale_factor != None:
                scale_factor += 1

        rows = []
        for dim_val in dimension_values:
            avg = dict()

            val = dim_val.group
            msg_cnt = dim_val.msg_cnt

            # required fields                        
            avg['CPU_USED'] = dim_val.total['CPU_USED']/msg_cnt
            avg['EXIST_TIME'] = dim_val.total['EXIST_TIME']/msg_cnt
            avg['CRET'] = dim_val.total['CRET']/msg_cnt
            avg['FIND_FILE'] = dim_val.total['FIND_FILE']/msg_cnt
            avg['FIND'] = dim_val.total['FIND']/msg_cnt
            avg['FILE'] = dim_val.total['FILE']/msg_cnt
              
            # longs                                                                                                           
            avg['DD_TIME'] = dim_val.total['DD_TIME']/msg_cnt if 'DD_TIME' in dim_val.total and dim_val.total['DD_TIME'] is not None else None
            avg['ECB1MF'] = dim_val.total['ECB1MF']/msg_cnt if 'ECB1MF' in dim_val.total and dim_val.total['ECB1MF'] is not None else None
            avg['ECB4KF'] = dim_val.total['ECB4KF']/msg_cnt if 'ECB4KF' in dim_val.total and dim_val.total['ECB4KF'] is not None else None
            avg['RLOG'] = dim_val.total['RLOG']/msg_cnt if 'RLOG' in dim_val.total and dim_val.total['RLOG'] is not None else None
            
            # ints                                                                                                         
            avg['CMBK'] = dim_val.total['CMBK']/msg_cnt if 'CMBK' in dim_val.total and dim_val.total['CMBK'] is not None else None
            avg['COW'] = dim_val.total['COW']/msg_cnt if 'COW' in dim_val.total and dim_val.total['COW'] is not None else None
            avg['DF_ATTAC'] = dim_val.total['DF_ATTAC']/msg_cnt if 'DF_ATTAC' in dim_val.total and dim_val.total['DF_ATTAC'] is not None else None
            avg['DF_BTFIL'] = dim_val.total['DF_BTFIL']/msg_cnt if 'DF_BTFIL' in dim_val.total and dim_val.total['DF_BTFIL'] is not None else None
            avg['DF_BTFND'] = dim_val.total['DF_BTFND']/msg_cnt if 'DF_BTFND' in dim_val.total and dim_val.total['DF_BTFND'] is not None else None
            avg['DF_CFILE'] = dim_val.total['DF_CFILE']/msg_cnt if 'DF_CFILE' in dim_val.total and dim_val.total['DF_CFILE'] is not None else None
            avg['DF_CFIND'] = dim_val.total['DF_CFIND']/msg_cnt if 'DF_CFIND' in dim_val.total and dim_val.total['DF_CFIND'] is not None else None
            avg['DF_CLS_NO_REL'] = dim_val.total['DF_CLS_NO_REL']/msg_cnt if 'DF_CLS_NO_REL' in dim_val.total and dim_val.total['DF_CLS_NO_REL'] is not None else None
            avg['DF_CLS_OPN'] = dim_val.total['DF_CLS_OPN']/msg_cnt if 'DF_CLS_OPN' in dim_val.total and dim_val.total['DF_CLS_OPN'] is not None else None
            avg['DF_CLS_RELS'] = dim_val.total['DF_CLS_RELS']/msg_cnt if 'DF_CLS_RELS' in dim_val.total and dim_val.total['DF_CLS_RELS'] is not None else None
            avg['DF_CLS_REUSE'] = dim_val.total['DF_CLS_REUSE']/msg_cnt if 'DF_CLS_REUSE' in dim_val.total and dim_val.total['DF_CLS_REUSE'] is not None else None
            avg['DF_DBADD'] = dim_val.total['DF_DBADD']/msg_cnt if 'DF_DBADD' in dim_val.total and dim_val.total['DF_DBADD'] is not None else None
            avg['DF_DBCLS'] = dim_val.total['DF_DBCLS']/msg_cnt if 'DF_DBCLS' in dim_val.total and dim_val.total['DF_DBCLS'] is not None else None
            avg['DF_DBDEL'] = dim_val.total['DF_DBDEL']/msg_cnt if 'DF_DBDEL' in dim_val.total and dim_val.total['DF_DBDEL'] is not None else None
            avg['DF_DBDIX'] = dim_val.total['DF_DBDIX']/msg_cnt if 'DF_DBDIX' in dim_val.total and dim_val.total['DF_DBDIX'] is not None else None
            avg['DF_DBIDX'] = dim_val.total['DF_DBIDX']/msg_cnt if 'DF_DBIDX' in dim_val.total and dim_val.total['DF_DBIDX'] is not None else None
            avg['DF_DBKEY'] = dim_val.total['DF_DBKEY']/msg_cnt if 'DF_DBKEY' in dim_val.total and dim_val.total['DF_DBKEY'] is not None else None
            avg['DF_DBMOD'] = dim_val.total['DF_DBMOD']/msg_cnt if 'DF_DBMOD' in dim_val.total and dim_val.total['DF_DBMOD'] is not None else None
            avg['DF_DBOPN'] = dim_val.total['DF_DBOPN']/msg_cnt if 'DF_DBOPN' in dim_val.total and dim_val.total['DF_DBOPN'] is not None else None
            avg['DF_DBRED'] = dim_val.total['DF_DBRED']/msg_cnt if 'DF_DBRED' in dim_val.total and dim_val.total['DF_DBRED'] is not None else None
            avg['DF_DBREP'] = dim_val.total['DF_DBREP']/msg_cnt if 'DF_DBREP' in dim_val.total and dim_val.total['DF_DBREP'] is not None else None
            avg['DF_DETAC'] = dim_val.total['DF_DETAC']/msg_cnt if 'DF_DETAC' in dim_val.total and dim_val.total['DF_DETAC'] is not None else None
            avg['DF_GETCC'] = dim_val.total['DF_GETCC']/msg_cnt if 'DF_GETCC' in dim_val.total and dim_val.total['DF_GETCC'] is not None else None
            avg['DF_GETFC'] = dim_val.total['DF_GETFC']/msg_cnt if 'DF_GETFC' in dim_val.total and dim_val.total['DF_GETFC'] is not None else None
            avg['DF_LRFIL'] = dim_val.total['DF_LRFIL']/msg_cnt if 'DF_LRFIL' in dim_val.total and dim_val.total['DF_LRFIL'] is not None else None
            avg['DF_LRFND'] = dim_val.total['DF_LRFND']/msg_cnt if 'DF_LRFND' in dim_val.total and dim_val.total['DF_LRFND'] is not None else None
            avg['DF_OPN_HD'] = dim_val.total['DF_OPN_HD']/msg_cnt if 'DF_OPN_HD' in dim_val.total and dim_val.total['DF_OPN_HD'] is not None else None
            avg['DF_OPN_HND'] = dim_val.total['DF_OPN_HND']/msg_cnt if 'DF_OPN_HND' in dim_val.total and dim_val.total['DF_OPN_HND'] is not None else None
            avg['DF_OPN_NHD'] = dim_val.total['DF_OPN_NHD']/msg_cnt if 'DF_OPN_NHD' in dim_val.total and dim_val.total['DF_OPN_NHD'] is not None else None
            avg['DF_OPN_NHND'] = dim_val.total['DF_OPN_NHND']/msg_cnt if 'DF_OPN_NHND' in dim_val.total and dim_val.total['DF_OPN_NHND'] is not None else None
            avg['DF_PFILE'] = dim_val.total['DF_PFILE']/msg_cnt if 'DF_PFILE' in dim_val.total and dim_val.total['DF_PFILE'] is not None else None
            avg['DF_PFIND'] = dim_val.total['DF_PFIND']/msg_cnt if 'DF_PFIND' in dim_val.total and dim_val.total['DF_PFIND'] is not None else None
            avg['DF_RELCC'] = dim_val.total['DF_RELCC']/msg_cnt if 'DF_RELCC' in dim_val.total and dim_val.total['DF_RELCC'] is not None else None
            avg['DF_RELFC'] = dim_val.total['DF_RELFC']/msg_cnt if 'DF_RELFC' in dim_val.total and dim_val.total['DF_RELFC'] is not None else None
            avg['DF_RSF_CREATE'] = dim_val.total['DF_RSF_CREATE']/msg_cnt if 'DF_RSF_CREATE' in dim_val.total and dim_val.total['DF_RSF_CREATE'] is not None else None
            avg['DF_RSF_DELETE'] = dim_val.total['DF_RSF_DELETE']/msg_cnt if 'DF_RSF_DELETE' in dim_val.total and dim_val.total['DF_RSF_DELETE'] is not None else None
            avg['DF_RSF_READ'] = dim_val.total['DF_RSF_READ']/msg_cnt if 'DF_RSF_READ' in dim_val.total and dim_val.total['DF_RSF_READ'] is not None else None
            avg['DF_RSF_UPDATE'] = dim_val.total['DF_RSF_UPDATE']/msg_cnt if 'DF_RSF_UPDATE' in dim_val.total and dim_val.total['DF_RSF_UPDATE'] is not None else None
            avg['FILE_DASD'] = dim_val.total['FILE_DASD']/msg_cnt if 'FILE_DASD' in dim_val.total and dim_val.total['FILE_DASD'] is not None else None
            avg['FILE_VFA'] = dim_val.total['FILE_VFA']/msg_cnt if 'FILE_VFA' in dim_val.total and dim_val.total['FILE_VFA'] is not None else None
            avg['FIND_DASD'] = dim_val.total['FIND_DASD']/msg_cnt if 'FIND_DASD' in dim_val.total and dim_val.total['FIND_DASD'] is not None else None
            avg['FIND_VFA'] = dim_val.total['FIND_VFA']/msg_cnt if 'FIND_VFA' in dim_val.total and dim_val.total['FIND_VFA'] is not None else None
            avg['GFSL'] = dim_val.total['GFSL']/msg_cnt if 'GFSL' in dim_val.total and dim_val.total['GFSL'] is not None else None
            avg['GFSS'] = dim_val.total['GFSS']/msg_cnt if 'GFSS' in dim_val.total and dim_val.total['GFSS'] is not None else None
            avg['GRFS'] = dim_val.total['GRFS']/msg_cnt if 'GRFS' in dim_val.total and dim_val.total['GRFS'] is not None else None
            avg['HCTL'] = dim_val.total['HCTL']/msg_cnt if 'HCTL' in dim_val.total and dim_val.total['HCTL'] is not None else None
            avg['MAL'] = dim_val.total['MAL']/msg_cnt if 'MAL' in dim_val.total and dim_val.total['MAL'] is not None else None
            avg['RELF'] = dim_val.total['RELF']/msg_cnt if 'RELF' in dim_val.total and dim_val.total['RELF'] is not None else None
            avg['ROUT'] = dim_val.total['ROUT']/msg_cnt if 'ROUT' in dim_val.total and dim_val.total['ROUT'] is not None else None
            avg['SERR'] = dim_val.total['SERR']/msg_cnt if 'SERR' in dim_val.total and dim_val.total['SERR'] is not None else None
            avg['SNAP'] = dim_val.total['SNAP']/msg_cnt if 'SNAP' in dim_val.total and dim_val.total['SNAP'] is not None else None
            avg['SWBK'] = dim_val.total['SWBK']/msg_cnt if 'SWBK' in dim_val.total and dim_val.total['SWBK'] is not None else None
            avg['SYSH'] = dim_val.total['SYSH']/msg_cnt if 'SYSH' in dim_val.total and dim_val.total['SYSH'] is not None else None
            avg['TOUR'] = dim_val.total['TOUR']/msg_cnt if 'TOUR' in dim_val.total and dim_val.total['TOUR'] is not None else None
            avg['TWRT'] = dim_val.total['TWRT']/msg_cnt if 'TWRT' in dim_val.total and dim_val.total['TWRT'] is not None else None
            avg['XWBK'] = dim_val.total['XWBK']/msg_cnt if 'XWBK' in dim_val.total and dim_val.total['XWBK'] is not None else None

            value_id = self.get_nvp_value_id(self.app_dimension.dimension, val, db_cursor)

            # Arguments must be in order of db_field_names
            # Base version
            row = [time, duration, scale_factor, value_id, msg_cnt,
                   # required metrics
                   avg['CPU_USED'], avg['EXIST_TIME'], avg['CRET'], avg['FIND_FILE'], avg['FIND'], avg['FILE'],
                   # long metrics
                   avg['DD_TIME'], avg['ECB1MF'], avg['ECB4KF'], avg['RLOG'],
                   # int metrics
                   avg['CMBK'], avg['COW'],
                   avg['DF_ATTAC'], avg['DF_BTFIL'], avg['DF_BTFND'], avg['DF_CFILE'], avg['DF_CFIND'], avg['DF_CLS_NO_REL'],
                   avg['DF_CLS_OPN'], avg['DF_CLS_RELS'], avg['DF_CLS_REUSE'], avg['DF_DBADD'], avg['DF_DBCLS'],
                   avg['DF_DBDEL'], avg['DF_DBDIX'], avg['DF_DBIDX'], avg['DF_DBKEY'], avg['DF_DBMOD'], avg['DF_DBOPN'],
                   avg['DF_DBRED'], avg['DF_DBREP'], avg['DF_DETAC'], avg['DF_GETCC'], avg['DF_GETFC'], avg['DF_LRFIL'],
                   avg['DF_LRFND'], avg['DF_OPN_HD'], avg['DF_OPN_HND'], avg['DF_OPN_NHD'], avg['DF_OPN_NHND'],
                   avg['DF_PFILE'], avg['DF_PFIND'], avg['DF_RELCC'], avg['DF_RELFC'], avg['DF_RSF_CREATE'],
                   avg['DF_RSF_DELETE'], avg['DF_RSF_READ'], avg['DF_RSF_UPDATE'],
                   avg['FILE_DASD'], avg['FILE_VFA'], avg['FIND_DASD'], avg['FIND_VFA'],
                   avg['GFSL'], avg['GFSS'], avg['GRFS'], avg['HCTL'], avg['MAL'], avg['RELF'], avg['ROUT'],
                   avg['SERR'], avg['SNAP'], avg['SWBK'], avg['SYSH'], avg['TOUR'], avg['TWRT'], avg['XWBK']]

            # Version 2
            if system_id is not None:
                row.append(system_id)

            rows.append(row)

        return v_insert, rows

    def _check_missing_nvps(self, num_dim_values):
        if self.missing_nvps_count > 0:  # Haven't found record with all the nvps yet
            if num_dim_values > 0:  # Found record with no missing nvps
                self.missing_nvps_count = -1
        if num_dim_values <= 0:  # Found record with missing nvps
            logger.warning(f'No records with all of the name-value pairs for dimension '
                           f'"{self.app_dimension.dimension}" have been found. Please check your '
                           f'configurations. Check the nvp_names section of your '
                           f'tpf_zrtmc_analyzer_profile.yml, the nvpc section of your RTMC Properties file,'
                           f' and the name-value pairs that are being collected on your z/TPF system(s).')

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: NvpcData, parent=None):
        if not self._nvpc_data_filter(data):
            return []

        dimension_values = self.aggregate_metrics(data)

        self._check_missing_nvps(len(dimension_values))

        if len(dimension_values) == 0:
            return []

        v_insert, rows = self._create_rows(data, dimension_values.values(), db_cursor)
        return [InsertMany(v_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]


class NvpcHorizontalHandler(DbDataHandler):

    def __init__(self, app_dimensions: List[AppDimension], system_indicator: AppDimension = None):
        self.app_dimension_handlers = [AppDimensionHandler(dim) for dim in (app_dimensions or [])]
        self.sys_indicator_handlers = []

        # If there is no system_indicator, default to Message Type
        # If there is a system indicator, only set it here if it is horizontal
        if system_indicator is None or system_indicator.num_verticals == 0:
            if system_indicator is not None:
                sys_indicator = system_indicator.dimension
                logger.info(f'Using "{sys_indicator}" as the system_indicator.')
            else:
                sys_indicator = 'Message Type'
                logger.info('No system_indicator was specified. Defaulting to "Message Type".')

            self.sys_indicator_handlers = [SystemIndicatorHandler(proc, sys_indicator) for proc in system_indicator_procs]

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: NvpcData, parent=None):
        if data.is_vertical:
            return []

        insert_manys: List[InsertMany] = []

        # No need to pass parent because data is already includes the "parent"
        for handler in self.sys_indicator_handlers:
            insert_manys.extend(handler.get_mysql_insert_manys(db_cursor, data))

        for handler in self.app_dimension_handlers:
            insert_manys.extend(handler.get_mysql_insert_manys(db_cursor, data))

        return insert_manys

    @classmethod
    def get_table_names(cls):
        table_names = []
        table_names.extend(SystemIndicatorHandler.get_table_names())
        table_names.extend(AppDimensionHandler.get_table_names())
        return table_names
