#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging
from typing import List

from pymysql.cursors import Cursor

from tpf_nvpc.nvpc_config.app_dimension import AppDimension
from tpf_nvpc.nvpc_db_handlers.nvpc_horizontal_db import AppDimensionHandler
from tpf_nvpc.nvpc_db_handlers.nvpc_sys_indicator_db import SystemIndicatorHandler, system_indicator_procs
from tpf_nvpc.tpf_nvpc_data import NvpcData
from tpf_nvpc.tpf_nvpc_record import NvpcRecord
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler

logger = logging.getLogger('tpf_zrtmc_analyzer')


class VerticalAppDimensionHandler(AppDimensionHandler):
    def __init__(self, app_dimension: AppDimension):
        super().__init__(app_dimension)

    def _nvpc_record_filter(self, nvpc_record: NvpcRecord):
        verticals = sum(1 for n in nvpc_record.name_value_pairs if n.is_vertical())
        return verticals == self.app_dimension.num_verticals

    def _nvpc_data_filter(self, nvpc_data: NvpcData):
        return nvpc_data.is_vertical


class NvpcVerticalHandler(DbDataHandler):

    def __init__(self, app_dimensions: List[AppDimension], system_indicator: AppDimension = None):
        self.app_dimension_handlers = [VerticalAppDimensionHandler(dim) for dim in (app_dimensions or [])]
        self.sys_indicator_handlers = []

        if system_indicator is not None and system_indicator.num_verticals > 0:
            sys_indicator = system_indicator.dimension

            logger.info(f'Using "{sys_indicator}" as the system_indicator.')

            self.sys_indicator_handlers = [SystemIndicatorHandler(proc, sys_indicator) for proc in system_indicator_procs]

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: NvpcData, parent=None):
        if not data.is_vertical:
            return []

        insert_manys: List[InsertMany] = []

        # No need to pass parent because data is already includes the "parent"
        for handler in self.sys_indicator_handlers:
            insert_manys.extend(handler.get_mysql_insert_manys(db_cursor, data))

        for handler in self.app_dimension_handlers:
            insert_manys.extend(handler.get_mysql_insert_manys(db_cursor, data))

        return insert_manys

    @classmethod
    def get_table_names(cls):
        table_names = []
        table_names.extend(SystemIndicatorHandler.get_table_names())
        table_names.extend(VerticalAppDimensionHandler.get_table_names())
        return table_names
