#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   10052022    PJ46853 - Change skip factor to fractional scale      #
#                         factor                                      #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys, get_or_default_or_error, fix_json_cpc_name

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpcHeader(object):
    max_version = 2

    required_fields_header = ["version", "complex_name", "processor_id", "collection_period",
                              "collection_start_utc_time", "parameter"]
    required_fields_header_v2 = required_fields_header + ["system_name", "cpc_name", "ip_address", "environment"]

    req_fields_header_versions = {
        0: required_fields_header,
        1: required_fields_header,
        2: required_fields_header_v2
    }

    # At some point, "average_scale_factor" should be added as a required field.
    # For now, omitting it for migration purposes: old tpf/old tpfrtmc/new starter kit
    required_fields_parameters = ["total_packets", "packet_number", "average_skip_factor", "inuse_istreams",
                                  "active_istreams", "collection_interval_utc_time"]

    def __init__(self, json_nvpc_header):
        """
        Initialize instance of NvpcHeader object.

        :param json_nvpc_header: "header" object from NVPC JSON structure
        :raise MissingFieldError: if required field is missing from JSON structure
        """

        self.version_num = 0

        try:
            # Base version
            self.version = json_nvpc_header["version"]
            self.version_num = int(float(self.version))
            self.complex_name = json_nvpc_header["complex_name"]
            self.processor_id = json_nvpc_header["processor_id"]
            self.collection_period = json_nvpc_header["collection_period"]
            self.collection_start_utc_time = json_nvpc_header["collection_start_utc_time"]
            json_nvpc_header_parameters = json_nvpc_header["parameters"]

            # Version 2
            are_fields_required = self.version_num >= 2
            self.system_name = get_or_default_or_error(json_nvpc_header, "system_name", key_error=are_fields_required)

            fix_json_cpc_name(json_nvpc_header)

            self.cpc_name = get_or_default_or_error(json_nvpc_header, "cpc_name", key_error=are_fields_required)
            self.ip_address = get_or_default_or_error(json_nvpc_header, "ip_address", key_error=are_fields_required)
            self.environment = get_or_default_or_error(json_nvpc_header, "environment", key_error=are_fields_required)
        except KeyError:
            req_fields = self.req_fields_header_versions.get(min(self.max_version, self.version_num), self.required_fields_header)
            msg = f'Unable to deserialize NVPC header. ' \
                  f'Missing fields: {get_missing_keys(json_nvpc_header, req_fields)}. ' \
                  f'Data: {json_nvpc_header}'
            logger.warning(msg)
            raise MissingFieldError(msg)

        try:
            self.total_packets = json_nvpc_header_parameters["total_packets"]
            self.packet_number = json_nvpc_header_parameters["packet_number"]
            self.average_skip_factor = json_nvpc_header_parameters["average_skip_factor"]
            self.inuse_istreams = json_nvpc_header_parameters["inuse_istreams"]
            self.active_istreams = json_nvpc_header_parameters["active_istreams"]
            self.collection_interval_utc_time = json_nvpc_header_parameters["collection_interval_utc_time"]
            if "average_scale_factor" in json_nvpc_header_parameters:
                self.average_scale_factor = json_nvpc_header_parameters["average_scale_factor"]
            else:
                self.average_scale_factor = 0
        except KeyError:
            msg = f'Unable to deserialize NVPC header parameters. ' \
                  f'Missing fields: {get_missing_keys(json_nvpc_header_parameters, self.required_fields_parameters)}. ' \
                  f'Data: {json_nvpc_header_parameters}'
            logger.warning(msg)
            raise MissingFieldError(msg)
