#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_nvpc.nvpc_config.orientation import Orientation
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NameValuePair(object):

    required_fields = ["name", "value", "orientation"]

    def __init__(self, json_nvp):
        """
        Initializes an instance of a NameValuePair object.

        :param json_nvp: one item from the "name_value_pair" array in NVPC JSON structure
        :raise MissingFieldError: if required field is missing from NVPC JSON structure
        """

        try:
            self.name = json_nvp["name"]
            self.value = json_nvp["value"]
            self.orientation = json_nvp["orientation"]

        except KeyError:
            msg = f'Unable to deserialize NVPC name value pair. Missing fields: ' \
                  f'{get_missing_keys(json_nvp, self.required_fields)}. Data: {json_nvp}'
            logger.warning(msg)
            raise MissingFieldError(msg)

        self.orientation_enum = None

        if isinstance(self.orientation, str):
            try:
                self.orientation_enum = Orientation[self.orientation.upper()]
            except KeyError:
                pass

    def to_string(self):
        """
        Creates a string using nvp name and nvp value ("name=value").

        :return: a string with format "name=value"
        """
        string = self.name + "=" + self.value
        return string

    def is_horizontal(self):
        return self.orientation_enum == Orientation.HORIZONTAL

    def is_vertical(self):
        return self.orientation_enum == Orientation.VERTICAL
