#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from exceptions.json_err import MissingFieldError
from tpf_data_manager.source_manager import SourceManager
from pymysql.cursors import Cursor
import logging

from tpf_nvpc.nvpc_db_handlers.nvpc_vertical_db import NvpcVerticalHandler
from tpf_nvpc.tpf_nvpc_data import NvpcData
from tpf_utils.json_utils import to_json_string

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpcVerticalManager(SourceManager):
    nvpc_vertical = 'nvpcVertical'

    def __init__(self, active_source_types, **kwargs):
        super().__init__(active_source_types)

        app_dimensions = kwargs.get('app_dimensions')
        system_indicator = kwargs.get('system_indicator')

        self.unhandled_vertical_data = False
        self.nvpc_vertical_handler = None

        if active_source_types is not None and len(active_source_types) > 0:
            unrecognized_source_types = set()

            for source_type in active_source_types:
                if self.nvpc_vertical == source_type:
                    if self.nvpc_vertical_handler is None:
                        self.nvpc_vertical_handler = NvpcVerticalHandler(app_dimensions, system_indicator)
                else:
                    unrecognized_source_types.add(source_type)

            if len(unrecognized_source_types) > 0:
                msg = f'The following data types were not recognized: {unrecognized_source_types}.' \
                      f'The following are recognized NVPC Vertical data types: {[self.nvpc_vertical]}. ' \
                      f'Make sure the correct data type is specified in the ' \
                      f'data_sources.nvpc_vertical.tpf_processing section of the ZRTMC Analyzer profile.'
                logger.error(msg)
                raise ValueError

        if self.nvpc_vertical_handler is None:
            msg = f'TPF processing is not enabled for NVPC Vertical.'
            logger.info(msg)
        elif app_dimensions is None or len(app_dimensions) == 0:
            logger.warning(f'There were no application_dimensions specified for NVPC Vertical.')
        else:
            msg = f'TPF processing is enabled for the following NVPC Vertical application_dimensions: ' \
                  f'{[ad.dimension for ad in app_dimensions]} '
            logger.info(msg)

    def get_db_tables(self, active_only=True):
        if active_only and self.nvpc_vertical_handler is None:
            return []
        else:
            return NvpcVerticalHandler.get_table_names()

    def deserialize(self, json_doc):
        try:
            source = json_doc['header']['source']
            if source != 4 and source != 'RTMC_SEND_NVPC' and source != 'RTMC_SEND_NVP':
                return None
            nvpc_data = NvpcData(json_doc)

        except (KeyError, MissingFieldError):
            msg = f'Failed to deserialize a JSON document. Source: NVPC_VERTICAL. Data: {json_doc}'
            logger.error(msg)
            return None

        if not nvpc_data.is_vertical:
            return None

        if nvpc_data.contains_invalid_negatives():
            msg = f'Deserialized NVPC data that included invalid negative numbers. The data was discarded. ' \
                  f'Data: {to_json_string(nvpc_data)}'
            logger.warning(msg)
            return None

        return nvpc_data

    def get_mysql_insert_manys(self, db_cursor: Cursor, analytics_data: NvpcData):
        if self.nvpc_vertical_handler is None:
            if not self.unhandled_vertical_data:
                msg = f'Encountered an NVPC Vertical data type for which there is no active TPF processing. ' \
                      f'Data type: {self.nvpc_vertical}'
                logger.warning(msg)
                self.unhandled_vertical_data = True
            return []

        return self.nvpc_vertical_handler.get_mysql_insert_manys(db_cursor, analytics_data)
