#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from datetime import datetime, timedelta

import pymysql as db
from pymysql.constants import CLIENT
from pymysql.cursors import Cursor

from tpf_data_manager.dbms import DBMS
from tpf_rtmc_db.insert_many import InsertMany


def last_executed_query(cursor: Cursor):
    """
    Get the last executed SQL query from the supplied cursor.

    :param cursor: The pymysql.cursors.Cursor object that executed the query.
    :return: the SQL query string.
    :raise AttributeError: If the cursor has not executed an SQL query yet.
    """
    q = cursor._executed
    if isinstance(q, bytearray) or isinstance(q, bytes):
        q = q.decode()
    return q


class DbWriter(object):

    def __init__(self, host, port, user, password, database_name, dbms):
        """
        Initializes a new DbWriter. This does not attempt to connect to the database.

        :param host: The database host name.
        :param port: The database port.
        :param user: The database username.
        :param password: The database password.
        :param database_name: The database name.
        """

        self.host = host
        self.port = port
        self._user = user
        self._pw = password
        self.db_name = database_name
        self.dbms = dbms
        self._db_conn = None
        self.db_cursor = None
        self.last_execution_time = timedelta(seconds=0)  # Gets updated on calls to execute_sql and exec_insert_many
        self.last_query = None
        self.last_pre_insert = None
        self.last_rows = None

    def open(self):
        """
        Establish a connection to the database.

        :return: A pymysql.connections.Connection object connected to the database
        :raise OperationalError: If the database connection fails.
        """
        if not self.is_open():
            self._db_conn = db.connect(host=self.host,
                                       port=self.port,
                                       user=self._user,
                                       password=self._pw,
                                       db=self.db_name,
                                       client_flag=CLIENT.MULTI_STATEMENTS,
                                       charset='latin1',
                                       init_command="SET SESSION time_zone='+00:00';"
                                                    "SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED;")

            self.db_cursor = self._db_conn.cursor()

    def is_open(self):
        if self._db_conn is not None:
            return self._db_conn.open
        else:
            return False

    def commit(self):
        if self.is_open():
            self._db_conn.commit()

    def close(self, commit=True):
        if self.is_open():
            if commit:
                self.commit()

            self.db_cursor.close()
            self.db_cursor = None

            self._db_conn.close()
            self._db_conn = None

    def table_exists(self, table_name):
        """
        Check if the table exists in the database. This should only be called after :meth:`open`.

        :return: True if the table exists in the database, false otherwise.
        :raise DatabaseError: If the database is unable to perform the query.
        """

        if self.is_open():
            # This could raise a pymysql.DatabaseError, caller needs to handle
            self.db_cursor.execute(f"SELECT COUNT(*) "
                                   f"FROM information_schema.TABLES "
                                   f"WHERE table_schema='{self.db_name}' AND TABLE_NAME='{table_name}'")

            # Fetch all rows (there should only be one), get the first row, get the first value, check if equals 1
            return self.db_cursor.fetchall()[0][0] == 1

    def execute_sql(self, sql_statement):
        """
        Executes the SQL insert statement on the database. Get the execution time of this query (including network
        send/receive time) with self.last_execution_time.

        :param sql_statement: An SQL insert statement string.
        :raise DatabaseError: If the database is unable to perform the query.
        """

        if self.is_open():
            self.last_query = sql_statement
            self.last_pre_insert = None
            self.last_rows = None
            before = datetime.now()
            # This could raise a pymysql.DatabaseError, caller needs to handle
            self.db_cursor.execute(sql_statement)
            after = datetime.now()
            self.last_execution_time = after - before

    def exec_insert_many(self, insert: InsertMany):
        """
        Executes the SQL insert statement with multiple rows of input. Get the execution time of this query (including
        network send/receive time) with self.last_execution_time.

        :param insert: The InsertMany object to insert.
        :raise DatabaseError: If the database is unable to perform the query.
        """
        if self.is_open():
            self.last_query = None
            self.last_pre_insert = insert.pre_insert
            self.last_rows = insert.rows
            before = datetime.now()
            self.db_cursor.executemany(insert.pre_insert, insert.rows)
            after = datetime.now()
            self.last_execution_time = after - before

    def last_executed(self):
        """
        Gets the last SQL query executed.

        :return: the SQL query string.
        :raise AttributeError: If the db cursor has not executed an SQL query yet.
        """
        if self.last_query is not None:
           return self.last_query
        elif self.last_pre_insert is not None:
           return f"'{self.last_pre_insert}' ROWS: '{self.last_rows}'"
        else:
           return f"No SQL statement available."

    def get_db_start_time(self):
        """
        Gets the time the database was started.

        :return: the db start time
        """
        if self.is_open():
            self.db_cursor.execute(f" SELECT UNIX_TIMESTAMP() - VARIABLE_VALUE as start_time"
                                   f" FROM {'performance_schema' if self.dbms == DBMS.MYSQL else 'information_schema'}.global_status"
                                   f" WHERE VARIABLE_NAME='Uptime'")
            try:
                return self.db_cursor.fetchall()[0][0]
            except IndexError:
                return None
        else:
            return None

    def set_autocommit(self, autocommit):
        """
        Sets the autocommit value for the database connection.

        :param autocommit: bool
        :raise DatabaseError: If sending the autocommit value fails.
        """
        self._db_conn.autocommit(autocommit)
