#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from abc import ABC, abstractmethod
from pymysql.cursors import Cursor


class DbDataHandler(ABC):

    # "Heartbeat" class variables:
    # The following class variables are used to insert a row once per minute for each unique row in a collection if,
    # and only if, that unique row has not been inserted in the last minute due to all collections in the last minute
    # containing 0's in all metrics.
    heartbeatSeconds = 0 # A counter to keep track of # of seconds since the last full "heartbeat" insert.
    HEARTBEAT_THRESHOLD = 59000 # 60 seconds.  (some durations of 1 second a 998 - so 60 durations does not add up to 60000)
    heartbeatDict = {} # A dictionary to keep trace of which unique rows were inserted in the last minute. {key:boolean}
                       # where:
                       #   key: Is a unique string formed by the fields from db_key_fields in the Handler classes.
                       #        If this is True then insert the row on the minute heartbeat.
                       #   boolean: True indicates that a heartbeat insert is needed for the row.

    def all_metrics_zero(self, metrics):
        # Check the array of metrics parameter for a non-0 value
        for metric in metrics:
            if(metric != 0):
                return False
        return True

    def resetHeartbeat(self):
        """
        Perform 2 tasks if the heartbeat threshold has passed:
        1. Clear the dictionary.  It will be rebuilt every heartbeat period.
        2. Set heartbeatSeconds to 0 to start a new heartbeat period.
        """
        if self.heartbeatSeconds > self.HEARTBEAT_THRESHOLD:
            self.heartbeatDict.clear()
            self.heartbeatSeconds = 0

    def checkRowInsert(self, heartbeatKey, metrics):
        """
        Using the input parameters, determine if the row should be inserted into the database.
        This function is shared code used by the Handler classes for each data type.

        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        Heartbeat code, represented by lines that start with ###, is available in this function
        is currently commented out but left in place in case a need for it is found in the future.
        !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        :param heartbeatKey: The key in the heartbeatDict to access. (NOT CURRENTLY BEING USED)
        :param metrics: A set of metrics to check for zeros.  If all metrics are zeros then don't insert the row.
        :return: True - yes, insert into the database.
        :        False - no, don't insert into the database.
        """

        ### # If the heartbeatKey is not in the dictionary then add it now and set the flag to True, meaning an
        ### # insert of the row on the minute is needed.  Note that the flag is set to False if non-zero metric data
        ### # is seen (by the following if statement).
        ###if self.heartbeatDict.get(heartbeatKey) is None:
        ###    if heartbeatKey.startswith("common_deploy"):
        ###       logger.info(f'DbDataHandler:create heartbeatKey: {heartbeatKey}')
        ###       logger.info(self.heartbeatDict)
        ###    self.heartbeatDict.update({heartbeatKey:True})

        if self.all_metrics_zero(metrics) == False:
            # This row should be inserted because there is one or more non-zero metrics.

            ### # Because this row is being inserted, clear the flag in the dictionary for this heartbeatKey so it will
            ### # not be inserted when the threshold boundary passes.
            ###if heartbeatKey.startswith("common_deploy"):
            ###   logger.info(f'DbDataHandler:insertThisRow all_metrics_zero=False for heartbeatKey {heartbeatKey} - Set False')
            ###self.heartbeatDict[heartbeatKey] = False

            return True

        ###if self.heartbeatSeconds > self.HEARTBEAT_THRESHOLD and self.heartbeatDict.get(heartbeatKey) == True:
        ###    # This row should be inserted because the number of seconds since the last heartbeat insert has elapsed AND
        ###    # the flag is True in the dictionary for this key (indicating that this row has not been inserted
        ###    # in the last minute).
        ###    if heartbeatKey.startswith("common_deploy"):
        ###       logger.info(f'DbDataHandler:insertThisRow heartbeatSeconds = {self.heartbeatSeconds}, heartbeatKey {heartbeatKey} is True')
        ###    return True

        return False

    def get_mysql_inserts(self, db_cursor: Cursor, data, parent=None):
        """
        Returns the SQL insert statements for the input data.

        :param parent: A parent object containing higher-level information for the data.
        :param db_cursor: A pymysql.cursors.Cursor object to create the insert statement.
        :param data: An object instance to be processed for SQL insert statements.
        :return: List of the SQL insert statements.
        :rtype: list[bytes]
        """
        inserts = self.get_mysql_insert_manys(db_cursor, data, parent=parent)
        return [db_cursor.mogrify(insert.pre_insert, row) for insert in inserts for row in insert.rows]

    @classmethod
    @abstractmethod
    def get_table_names(cls):
        """
        Returns the names of the tables to which this data type inserts.

        :return: List of table names
        :rtype: list[str]
        """
        pass

    @abstractmethod
    def get_mysql_insert_manys(self, db_cursor: Cursor, data, parent=None):
        """
        Gets a list of InsertMany objects for bulk insert into the database.

        :param parent: A parent object containing higher-level information for the data.
        :param db_cursor: A pymysql.cursors.Cursor object to access the database.
        :param data: The data to be processed for SQL inserts.
        :return: List of the SQL InsertMany objects
        :rtype: list[tpf_rtmc_db.insert_many.InsertMany]
        """
        pass
