#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from datetime import datetime, timedelta, timezone

default_date_format = '%Y-%m-%d %H:%M:%S.000'


def ms_to_datetime(unix_ms, date_format=default_date_format):
    """
    Converts a timestamp in unix milliseconds to a datetime string.
    The default date format is '%Y-%m-%d %H:%M:%S.000'.

    :param int unix_ms: The unix milliseconds timestamp.
    :param date_format: The string format that the unix_ms will be translated into the default
    date format is '%Y-%m-%d %H:%M:%S.000'.
    :return: A string representation of the timestamp.
    """
    return (datetime(1970, 1, 1, tzinfo=timezone.utc) + timedelta(milliseconds=unix_ms)).strftime(date_format)


def time_conversion(time):
    """
    Converts string of time into datetime.timedelta format.

    :param time: time string in format 'year-month-day hour:minute:second'
    :return: time in datetime.timedelta format
    """
    return datetime.strptime(time, '%Y-%m-%d %H:%M:%S.%f')


def calculate_duration(start, interval):
    """
    Calculates the duration between start and interval time.

    :param start: starting time in string format 'year-month-day hour:minute:second'
    :param interval: stopping time in string format 'year-month-day hour:minute:second'
    :return: duration in seconds
    """
    interval = time_conversion(interval)
    start = time_conversion(start)
    duration = (interval - start).total_seconds()
    if duration < 1:
        duration = 1
    return duration


def create_pre_sql_update(table_name, new_time, altered_time, system_id, subsystem_id=None):
    """
    Creates an SQL UPDATE statement to update the time column a single database row based upon a WHERE clause.

    :param str table_name: The table name to which to UPDATE the data.
    :param str new_time:     SET clause:   The new time column value.
    :param str altered_time: WHERE clause: The time the row was inserted into tpf_subsystems_utility_switches_metrics_table.
    :param int system_id:    WHERE clause: The system_id column value of the row to update.
    :param int subsystem_id: WHERE clause: The subsystem_id column value of the row to update (optional: if the row contains a subsystem_id column).
    :return: A pre SQL UPDATE string.
    :rtype: str
    """
    if subsystem_id is None:
       # This creates an SQL statement like:
       # UPDATE tpf_traces_metrics_table SET time = '2024-04-15 17:16:07.001' WHERE altered = '2024-04-15 17:16:07.000' AND system_id = 2
       return f'UPDATE {table_name} SET time = "{new_time}" WHERE altered = "{altered_time}" AND system_id = {system_id}'

    # This creates an SQL statement like:
    # UPDATE tpf_traces_metrics_table SET time = '2024-04-15 17:16:07.001' WHERE altered = '2024-04-15 17:16:07.000' AND system_id = 2 AND subsystem_id = 1
    return f'UPDATE {table_name} SET time = "{new_time}" WHERE altered = "{altered_time}" AND system_id = {system_id} AND subsystem_id = {subsystem_id}'


def create_pre_sql_insert(table_name, db_field_names, db_key_fields=None, duplicate_key=True):
    """
    Creates an SQL insert statement with %s placeholders for replacement by actual values.

    :param duplicate_key: Setting this to True will include the 'ON DUPLICATE KEY UPDATE' clause
    to the SQL string.
    :param str table_name: The table name to which to insert the data.
    :param db_field_names: List of the field names in the database.
    :param db_key_fields: List of the database key fields. These fields will not be included in the
    ON DUPLICATE KEY UPDATE clause.
    :return: A pre SQL insert string.
    :rtype: str
    """
    # This creates an SQL statement like: INSERT INTO t1(f1, f2) VALUES (%s, %s)
    pre_sql_insert = f'INSERT INTO {table_name}({", ".join(db_field_names)})' \
                     f' VALUES ({", ".join(["%s" for _ in range(len(db_field_names))])})'

    # This creates an SQL statement like: INSERT INTO t1(f1, f2) VALUES (%s, %s)
    #                                     ON DUPLICATE KEY UPDATE f1=VALUES(f1), f2=VALUES(f2);
    # %s will be replaced during the actual insert function
    # Need ON DUPLICATE KEY UPDATE so we update, not replace, rows when we receive more data
    if duplicate_key:
        if db_key_fields is None:
            db_key_fields = {}

        pre_sql_insert += f' ON DUPLICATE KEY UPDATE ' \
                          f'{", ".join([f"{s}=VALUES({s})" for s in db_field_names if s not in db_key_fields])}'

    return pre_sql_insert


def create_nvpc_avg_pre_sql_insert(table_name, db_field_names, db_key_fields):
    """
    Creates an SQL insert statement with % placeholders for replacement by actual values.
    This insert statement averages database values, updating the row with a weighted average as data is inserted.

    :param str table_name: The table name to which to insert the data.
    :param db_field_names: List of the field names in the database.
    :param db_key_fields: List of the database key fields.
    :return: A pre SQL insert string.
    :rtype: str
    """

    exclude_fields = {'msg_cnt'}

    # Algorithm for averaging: agg = agg + (val - agg) * wVal / (wVal + wAgg)
    # Where: agg = db value, val = incoming value, wAgg = weight db value (msg_cnt), wVal = weight incoming value
    return f'{create_pre_sql_insert(table_name, db_field_names, duplicate_key=False)}' \
           f' ON DUPLICATE KEY UPDATE ' \
           f'{", ".join([f"{s}=IFNULL({s},0)+(VALUES({s})-IFNULL({s},0))*VALUES(msg_cnt)/(VALUES(msg_cnt)+IFNULL(msg_cnt,0))" for s in db_field_names if s not in db_key_fields and s not in exclude_fields])},' \
           f' msg_cnt=IFNULL(msg_cnt,0)+VALUES(msg_cnt)'


def create_nvpc_calc_cpu_pre_sql_insert(table_name, db_field_names):
    """
    Creates the NVPC CalcCpuHandler pre_sql insert. On duplicate key, the calculated_cpu value of the insert
    data is added to the sum already in the database.

    :param table_name: The table name to which to insert the data.
    :param db_field_names: List of the field names in the database.
    :return: A pre SQL insert string.
    :rtype: str
    """

    return f'{create_pre_sql_insert(table_name, db_field_names, duplicate_key=False)}' \
           f'ON DUPLICATE KEY UPDATE calculated_cpu=VALUES(calculated_cpu)+IFNULL(calculated_cpu,0), ' \
           f'inuse_istreams=VALUES(inuse_istreams)'


def create_id_pre_sql_select(table_name, db_field_names):
    """
    Creates an SQL select statement with % placeholders for replacement by actual values.

    :param table_name: The table name
    :param db_field_names: The db columns to use in the WHERE clause, joined by AND.
    :return: A pre SQL select string.
    :rtype: str
    """

    return f'SELECT id FROM {table_name} WHERE {" AND ".join([f"{s}=%s" for s in db_field_names])}'


def create_id_pre_sql_insert(table_name, db_field_names):
    """
    Creates an SQL insert statement with % placeholders for replacement by actual values.
    This statement uses the MySQL function LAST_INSERT_ID to make available the id of
    a duplicate row instead of raising an exception.

    Note: Even though inserting a duplicate row does not add a new row to the table, it
    increments the AUTO_INCREMENT value of the table. If this happens too often, the
    AUTO_INCREMENT value will reach the max value of the id column, which will cause
    subsequent inserts to fail.

    :param table_name: The table name
    :param db_field_names: List of field names in the table.
    :return: A pre SQL insert string with ON DUPLICATE KEY clause.
    :rtype: str
    """
    return f'{create_pre_sql_insert(table_name, db_field_names, duplicate_key=False)}' \
           f' ON DUPLICATE KEY UPDATE id=LAST_INSERT_ID(id)'
