#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import json
from enum import Enum
from typing import Dict


def get_missing_keys(json_data: dict, expected_keys):
    """
    Returns a list of missing keys from a dictionary.

    :param json_data: A dict in which to look for missing keys.
    :param expected_keys: A list of expected dict keys.
    :return: A list of missing dict keys.
    """
    return [k for k in expected_keys if k not in json_data]


def to_json_string(obj, indent=None):
    """
    Serializes an object to a JSON string.

    :param indent: The number of spaces to indent each line, or None for a single line
    :param obj: Any object
    :return: the object as a JSON string.
    :rtype: str
    """

    return json.dumps(obj if not hasattr(obj, '__dict__') else obj.__dict__,
                      default=_to_json_string_default,
                      indent=indent)


def _to_json_string_default(obj):
    if isinstance(obj, Enum):
        return obj.name
    elif not hasattr(obj, '__dict__'):
        return str(obj)
    else:
        return obj.__dict__


def get_or_default_or_error(json_data: Dict, key, default=None, key_error=False):
    """
    Gets an item from a dictionary. If the item is not present, either return a
    default value or raise a KeyError.

    :param json_data: a dictionary of json data
    :param key: the key to get from the dictionary
    :param default: the default value to return if key_error=False
    :param key_error: True if a KeyError should be raised when key is not present, False
    if default should be returned.
    :raise KeyError: If key is not present and key_error=True
    :return: The value associated with key in json_data, or default.
    """

    if key_error:
        return json_data[key]
    else:
        return json_data.get(key, default)


def fix_json_cpc_name(json_data: Dict):
    """
    Fixes incorrect versions of json that use cpc_name instead of cpc_name. If cpc_name
    is a key in the json_data and cpc_name is not a key in the json_data, this function
    adds an entry to the json_data: json_data['cpc_name'] = json_data['cpc_name'].

    :param json_data: a python dictionary of deserialized JSON.
    """

    if 'cec_name' in json_data and 'cpc_name' not in json_data:
        json_data['cpc_name'] = json_data['cec_name']
