#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import json
import logging
import os
import signal
import ssl
import sys
import threading
import time
import traceback
from datetime import datetime, timedelta
from json import JSONDecodeError
from logging.handlers import RotatingFileHandler
from typing import Dict, List, Set

import asyncio
import yaml
from kafka import KafkaConsumer
from kafka.consumer.fetcher import ConsumerRecord
from kafka.errors import NoBrokersAvailable, CommitFailedError, UnrecognizedBrokerVersion, NodeNotReadyError, \
    KafkaProtocolError, KafkaTimeoutError
from kafka.structs import TopicPartition

from constants.profile_constants import *
from exceptions.json_err import MissingNvpcProcessingMetrics
from tpf_batch.batch_processor import AnalyzerBatchProcessor
from tpf_data_manager.data_source import DataSource
from tpf_data_manager.zrtmc_manager import ZrtmcManager
from tpf_user_exit.tpf_zrtmc_analyzer_user_exit import get_analyzer_batch_processor as get_analyzer_batch_user_exit
from tpf_utils.json_utils import get_missing_keys
from tpf_utils.profile_utils import are_profile_parameters_present, create_missing_profile_parameter_msg
from tpf_zrtmc_analyzer_ibm_processing import get_analyzer_batch_processor
from tpf_kafka_structure.tpf_kafka_container import KafkaContainer

Version_Minimum = 3

terminated = False
consumer_poll_timeout_ms = 5000
kafka_hosts = []
next_consumer_index = 0


def read_profile_arg():
    """

    :return:
    """
    if len(sys.argv) <= 1:
        print('Usage:')
        print('    python tpf_zrtmc_analyzer.py path/to/tpf_zrtmc_analyzer_profile.yml')
        return

    profile_path = os.path.realpath(sys.argv[1])
    return profile_path


def read_profile(profile_path):
    if os.path.exists(profile_path):
        try:
            with open(profile_path) as y:
                analyzer_profile = yaml.load(y, Loader=yaml.FullLoader)
                return analyzer_profile
        except (OSError, yaml.YAMLError) as e:
            print(f'Could not read ZRTMC Analyzer profile at "{profile_path}". Exception: {e}')
            return None
    else:
        print(f'ZRTMC Analyzer profile location "{profile_path}" does not exist')
        return None

def create_logger(profile_logging):
    logging_params = [PATH, FILE, MB_PER_FILE, BACKUP_COUNT]
    if not are_profile_parameters_present(profile_logging, logging_params, LOGGING, log_missing=False):
        print(create_missing_profile_parameter_msg(get_missing_keys(profile_logging, logging_params), LOGGING))
        return None
    log_path = profile_logging[PATH]
    log_file = profile_logging[FILE]
    mb_per_file = profile_logging[MB_PER_FILE]
    backup_count = profile_logging[BACKUP_COUNT]

    if not os.path.exists(log_path):
        try:
            os.makedirs(log_path)
        except OSError as e:
            print(f'Could not make logger directory "{log_path}". Exception: {e}')
            return None
    log = logging.getLogger('tpf_zrtmc_analyzer')
    log.setLevel(logging.INFO)
    fh = RotatingFileHandler(f'{log_path}/{log_file}', mode="w", maxBytes=mb_per_file * 1024 * 1024,
                             backupCount=backup_count)
    fh.setLevel(logging.INFO)

    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)

    fmt = logging.Formatter('%(asctime)s - [%(module)s] - [%(levelname)s] - %(message)s')
    fh.setFormatter(fmt)
    ch.setFormatter(fmt)

    log.addHandler(fh)
    log.addHandler(ch)

    return log


def process_old_kafka_settings(kafka_conf: dict, required_fields: list, KAFKA: str, logger):
    for field in required_fields:
        passing = are_profile_parameters_present(kafka_conf, [field], KAFKA, False)
        if passing:
            logger.info(f"Unable to process due to outdated Kafka settings. {required_fields}, in this case {field}")
            raise Exception(
                f"Unable to process due to outdated Kafka settings. {required_fields}, in this case {field}")


def create_kafka_consumer(analyzer_profile: dict, consumer_timeout: int):
    """
        Process property file and create kafka consumer.
    """
    # fetch_min_bytes is set to 50 kb, which means the kafka consumer will wait for 50 kb of data before returning
    # from a poll with a batch of records. The consumer will wait at most fetch_max_wait_ms, which defaults to 500.
    # When data is flowing, the consumer will push at least two batches to the DB per second, more if necessary.
    # The purpose of this is to throttle database connections by this code by having the consumer wait for data
    # as it is coming into the kafka broker, rather than making frequent polls that return small amounts of data.

    logger = logging.getLogger('tpf_zrtmc_analyzer')
    kafka_conf = analyzer_profile[KAFKA]
    kafka_host_file_defined = [KAFKA_HOSTS]

    required_fields = [HOST, PORT, GROUPID, FETCH_MIN_KB]
    process_old_kafka_settings(kafka_conf, required_fields, KAFKA, logger)

    passing = are_profile_parameters_present(kafka_conf, kafka_host_file_defined, KAFKA)
    if not passing:
        return None

    kafka_host_file = read_profile(kafka_conf[KAFKA_HOSTS])

    # No file path was given, the file path does not exist, or the yml was malformed
    if analyzer_profile is None:
        return None

    analyzer_profile.update(kafka_host_file)

    contains_kafka_host = [KAFKA_HOSTS]
    passing = are_profile_parameters_present(kafka_conf, contains_kafka_host, KAFKA_HOSTS)
    if not passing:
        return None

    if analyzer_profile[KAFKA_HOSTS_FILE_VERSION] != 2:
        logger.info(
            f"Unable to process kafka_hosts.yml file due to {KAFKA_HOSTS_FILE_VERSION} != 2:  {analyzer_profile[KAFKA_HOSTS_FILE_VERSION]}")
        raise Exception(
            f"Unable to process kafka_hosts.yml file due to {KAFKA_HOSTS_FILE_VERSION} != 2:  {analyzer_profile[KAFKA_HOSTS_FILE_VERSION]}")

    kafka_hosts_config: list = analyzer_profile[KAFKA_HOSTS]

    if len(kafka_hosts_config) > 10:
        logger.info(f"Unable to process kafka_hosts file due to having {len(kafka_hosts_config)} which is greater than "
                    f"the current limit of 10")
        raise Exception(f"Unable to process kafka_hosts file due to having {len(kafka_hosts_config)} "
                        f"which is greater than the current limit of 10")

    host_to_return = None
    consumer_index = 0
    total_hosts = 0
    for kafka_host in kafka_hosts_config:
        logger.info("Attempting the following hosts: " + str(kafka_host))
        kafka_host_file_required_fields = [HOSTS, PROG_VAR]
        passing = are_profile_parameters_present(kafka_host, kafka_host_file_required_fields, KAFKA_HOSTS)
        if not passing:
            continue

        kafka_hosts_length = len(kafka_host[HOSTS])

        if kafka_hosts_length > 10:
            logger.info(
                f"Unable to process kafka_hosts file due to having {kafka_host[HOSTS]} which is greater than "
                f"the current limit of 10")
            return None

        if total_hosts + kafka_hosts_length > 10:
            logger.info(
                f"Unable to process kafka_hosts file due to having {kafka_host[HOSTS]} which is greater than "
                f"the current limit of 10")
            return None

        total_hosts += kafka_hosts_length

        kafka_host_information = [GROUPID, FETCH_MIN_KB]

        prog_vars_dict = kafka_host[PROG_VAR]

        passing = are_profile_parameters_present(prog_vars_dict, kafka_host_information, KAFKA_HOSTS)
        if not passing:
            return None
        for current_sub_host in range(0, kafka_hosts_length):
            try:
                instantiated_kafka_container: KafkaContainer = KafkaContainer(kafka_host, current_sub_host,
                                                                              consumer_timeout)
                global kafka_hosts
                kafka_hosts.append(instantiated_kafka_container)
                if not host_to_return:
                    instantiated_kafka_container.create_kafka_consumer()
                    if instantiated_kafka_container.initialized():
                        host_to_return = instantiated_kafka_container
                        instantiated_kafka_container.set_initialized(True)
                        global next_consumer_index
                        next_consumer_index = consumer_index
            except NoBrokersAvailable as broker_failure:
                logger.warning("Broker was not available for reconnect due to: " + str(broker_failure) + str(traceback.format_exc()))
                continue
            except ssl.SSLError as ssl_failure:
                logger.warning("SSL certificate failed due to: " + str(ssl_failure) + str(traceback.format_exc()))
                continue
            consumer_index += 1
    return host_to_return


def shutdown(signum, frame):
    global terminated
    terminated = True


def assign_next_kafka_consumer(analyzer_profile, logger):
    kafka_success = False
    kafka_fail_msg = f'Kafka connection failed. Check your configuration parameters in the ZRTMC Analyzer profile.'
    kafka_trying_msg = f'Still trying to connect to Kafka.'
    kafka_wait_time = 0
    kafka_consumer = None
    while not kafka_success:
        try:
            global next_consumer_index
            if len(kafka_hosts) == 0:
                kafka_consumer = create_kafka_consumer(analyzer_profile[KAFKA])
            else:
                if next_consumer_index >= len(kafka_hosts):
                    next_consumer_index = 0
                kafka_consumer: KafkaContainer = kafka_hosts[next_consumer_index]
                if not kafka_consumer.initialized():
                    next_consumer_index += 1
                    continue
        except NoBrokersAvailable:
            if kafka_wait_time == 0:
                logger.error(kafka_fail_msg)
            if kafka_wait_time == 60:
                logger.error(kafka_trying_msg)
                kafka_wait_time = 0
            kafka_wait_time += 10
            time.sleep(10)
        else:
            kafka_success = True
    return kafka_consumer


async def main():
    profile_path = read_profile_arg()
    if profile_path is None:
        return

    analyzer_profile = read_profile(profile_path)

    # No file path was given, the file path does not exist, or the yml was malformed
    if analyzer_profile is None:
        return

    # Create the logger and exit if missing necessary logging values in profile

    passing = are_profile_parameters_present(analyzer_profile, [LOGGING], None, log_missing=False)
    if not passing:
        print(create_missing_profile_parameter_msg(get_missing_keys(analyzer_profile, [LOGGING]), None))
        return

    logger = create_logger(analyzer_profile[LOGGING])

    if logger is None:
        return

    analyzer_manager: ZrtmcManager = get_analyzer_batch_processor(analyzer_profile)
    user_processor: AnalyzerBatchProcessor = get_analyzer_batch_user_exit(analyzer_profile)

    if analyzer_manager is None or user_processor is None:
        return

    passing = are_profile_parameters_present(analyzer_profile, [VERSION], None, log_missing=False)
    if not passing:
        logger.info((create_missing_profile_parameter_msg(get_missing_keys(analyzer_profile, [VERSION]), None)))
        return

    if analyzer_profile[VERSION] < Version_Minimum:
        logger.info(
            f'This version of zrtmc_analyzer is not supported. Minimum version must be version {Version_Minimum}. '
            f'Analyzer will now shutdown')
        return
    # Check MariaDB connection.
    # If unsuccessful, try to connect again after 10-second sleep.
    # Log error message every minute.

    analyzer_manager.open_db_connection(log=True, with_reconnect=True)

    analyzer_manager.check_db_tables(log_missing=True)

    kafka_topic_data_sources: Dict[str, Set[DataSource]] = {}

    passing = are_profile_parameters_present(analyzer_profile, [DATA_SOURCES], None)
    if not passing:
        return

    for data_source, source_config in analyzer_profile[DATA_SOURCES].items():
        # Convert to DataSource enum. Ignore it if unrecognized. ZrtmcManager will log it.
        try:
            source = DataSource[data_source.upper()]
        except KeyError:
            continue

        passing = are_profile_parameters_present(source_config, [ACTIVE, KAFKA_TOPICS], DATA_SOURCES)
        if not passing:
            return

        if source_config[ACTIVE]:
            topics = source_config[KAFKA_TOPICS]

            if topics is None:
                logger.error(f'Data source {data_source} was active but no "kafka_topics" were specified')
                return

            for topic in topics:
                if topic in kafka_topic_data_sources:
                    kafka_topic_data_sources[topic].add(source)
                else:
                    kafka_topic_data_sources[topic] = {source}

    # Create the Kafka consumer.
    # If unsuccessful due to NoBrokersAvailable, try to connect again after 10 second sleep.
    # Log error message every minute.

    passing = are_profile_parameters_present(analyzer_profile, [KAFKA], None)
    if not passing:
        return

    kafka_success = False
    kafka_fail_msg = f'Kafka connection failed. Check your configuration parameters in the ZRTMC Analyzer profile.'
    kafka_trying_msg = f'Still trying to connect to Kafka.'
    kafka_wait_time = 0
    kafka_consumer = None
    while not kafka_success:
        try:
            kafka_consumer = create_kafka_consumer(analyzer_profile, consumer_poll_timeout_ms)
        except NoBrokersAvailable:
            if kafka_wait_time == 0:
                logger.error(kafka_fail_msg)
            if kafka_wait_time == 60:
                logger.error(kafka_trying_msg)
                kafka_wait_time = 0
            kafka_wait_time += 10
            time.sleep(10)
        except UnrecognizedBrokerVersion as e:
            logger.error(f'Kafka exception. If you are using SSL, this error may be due to an incorrect SSL setting.'
                         f' Or maybe your Kafka broker is encrypted with SSL and tpf_zrtmc_analyzer kafka_hosts.yml is '
                         f'not (or vice versa)')
            raise e
        except Exception as e:
            logger.error(f'Kafka exception. If you are using SSL, this error may be due to an incorrect SSL setting.'
                         f' Or maybe your Kafka broker is encrypted with SSL and tpf_zrtmc_analyzer kafka_hosts.yml is '
                         f'not (or vice versa)')
            raise e
        else:
            if kafka_consumer is None:
                time.sleep(1)
                continue
            kafka_success = True

    if kafka_consumer is None:
        logger.error(f'Failed to create the kafka consumer. '
                     f'If you are using SSL, this error may be due to an incorrect SSL setting.'
                     f' Or maybe your Kafka broker is encrypted with SSL and tpf_zrtmc_analyzer kafka_hosts.yml is '
                     f'not (or vice versa)')
        total_attempt = 0
        while kafka_consumer is None:
            time.sleep(1)
            global next_consumer_index
            next_consumer_index += 1
            total_attempt += 1
            if next_consumer_index >= len(kafka_hosts):
                next_consumer_index = 0
            kafka_consumer: KafkaContainer = kafka_hosts[next_consumer_index]
            if not kafka_consumer.initialized():
                kafka_consumer = None
            if total_attempt >= len(kafka_hosts):
                break

    # If the topic doesn't exist, exit. We want the topics to be created by the script so it is configured properly
    broker_topics = kafka_consumer.get_kafka_consumer().topics()
    missing_topics = [k for k in kafka_topic_data_sources.keys() if k not in broker_topics]

    if len(missing_topics) > 0:
        logger.error(f'Kafka topics "{missing_topics}" does not exist. '
                     f'Please create the topic by running "tpf_create_kafka_topics.sh" and make sure the correct '
                     f'topic is specified in "{profile_path}".')
        return

    # Subscribe may auto create the topic, but we want tpf_create_kafka_topics.sh to do it, so subscribe
    # after we check for the topic

    kafka_consumer.get_kafka_consumer().subscribe([*kafka_topic_data_sources])

    logger.info(f'Listening on Kafka topics {[*kafka_topic_data_sources]} at '
                f'{kafka_consumer.get_hosts()}')

    # Register shutdown hook
    signal.signal(signal.SIGTERM, shutdown)
    signal.signal(signal.SIGINT, shutdown)

    msgs_consumed = 0
    last_num_msg_log = datetime.now()
    next_num_msg_log = last_num_msg_log + timedelta(minutes=1)
    empty_poll_count = 0

    # Poll Kafka broker periodically for messages and process them
    while not terminated:
        poll_results: Dict[TopicPartition, List[ConsumerRecord]] = dict()

        try:
           if kafka_consumer.initialized():
              poll_results = poll_kafka_data(kafka_consumer, logger)
           else:
              raise KafkaConsumerException("Kafka consumer not initialized",logger)

           if not poll_results:
              logger.warning("Kafka consumer poll results are empty")
              empty_poll_count += 1
              if empty_poll_count >= 5:
                 empty_poll_count = 0
                 raise KafkaConsumerException("Kafka consumer too many empty poll responses",logger)
              continue

        #catcher for all KafkaConsumerExceptions, we'll try to re-establish the connection
        except:
           if next_consumer_index >= len(kafka_hosts):
               next_consumer_index = 0

           kafka_consumer = kafka_hosts[next_consumer_index]
           try:
               kafka_consumer.create_kafka_consumer()
           except NoBrokersAvailable as broker_failure:
               time.sleep(1)
               logger.warning("Exception creating kafka consumer - Broker was not available for reconnect due to: " + str(broker_failure) + str(traceback.format_exc()))
               next_consumer_index += 1
               continue
           except ssl.SSLError as ssl_failure:
               time.sleep(1)
               logger.warning("Exception creating kafka consumer - SSL certificate failed due to: " + str(ssl_failure) + str(traceback.format_exc()))
               next_consumer_index += 1
               continue
           except Exception as e:
               time.sleep(1)
               logger.warning("Exception creating kafka consumer: " + str(e))
               next_consumer_index += 1
               continue
           kafka_consumer.get_kafka_consumer().subscribe([*kafka_topic_data_sources])
           logger.info(f"Reconnected kafka_consumer Listening on Kafka topics {[*kafka_topic_data_sources]} at "
                       f"{kafka_consumer.get_hosts()}")
           next_consumer_index += 1

        if not kafka_consumer:
            continue

        poll_results_messages: Dict[TopicPartition, List[Dict]] = {}

        for topic_partition, msg_list in poll_results.items():
            msgs_consumed += len(msg_list)
            data_list = []
            for msg in msg_list:
                try:
                    data = json.loads(msg.value)
                    data_list.append(data)
                except JSONDecodeError as j:
                    logger.warning(f'Could not deserialize data to JSON. Message: {j.msg}. Data: {j.doc}')
                    continue
            poll_results_messages[topic_partition] = data_list
        batch_json: Dict[DataSource, List[Dict]] = {}
        for topic_partition, data_list in poll_results_messages.items():
            topic = topic_partition.topic
            for source in kafka_topic_data_sources[topic]:

                if source in batch_json:
                    batch_json[source] += data_list
                else:
                    batch_json[source] = data_list
        analyzer_batch = analyzer_manager.deserialize_batch(batch_json)
        try:
            analyzer_manager.process_batch(analyzer_batch)
        except MissingNvpcProcessingMetrics:
            return
        analyzer_batch = user_processor.deserialize_batch(batch_json)
        user_processor.process_batch(analyzer_batch)
        commit_data_from_kafka(analyzer_profile, kafka_consumer, logger)
        # Log number of messages consumed.
        now = datetime.now()
        if now >= next_num_msg_log:
            diff = now - last_num_msg_log
            diff = diff - timedelta(microseconds=diff.microseconds)  # Remove microseconds
            logger.info(f'{msgs_consumed} Kafka messages consumed in the last {diff}.')
            last_num_msg_log = now
            next_num_msg_log = last_num_msg_log + timedelta(minutes=5)
            msgs_consumed = 0

    # Shutdown ZRTMC Analyzer
    logger.info("Begin ZRTMC Analyzer shutdown process.")
    analyzer_manager.close_db_connection()
    logger.info("Closed database connection.")
    kafka_consumer.get_kafka_consumer().commit()
    logger.info("Committed Kafka offsets.")
    kafka_consumer.get_kafka_consumer().close()
    logger.info("Closed Kafka consumer.")
    logger.info("ZRTMC Analyzer has successfully exited.")

def commit_data_from_kafka(analyzer_profile, kafka_consumer, logger):
    try:
        def commit_offset(consumer):
            try:
                consumer.get_kafka_consumer().commit()
            except KafkaTimeoutError:
                # Handle the timeout error here
                print("Timeout error occurred while committing offset")

        commit_thread = threading.Thread(target=commit_offset, args=(kafka_consumer,))
        commit_thread.start()
        commit_thread.join(timeout=5.0)
        if commit_thread.is_alive():
            # The commit thread is still running, so it timed out
            logger.warning("Timeout occurred while waiting for commit to complete, forced ignored after 5 seconds")
    except CommitFailedError:
        # We will hit this if the DB goes down and reconnect takes longer than max_poll_interval_ms
        logger.warning('Failed to commit offsets because the Kafka Consumer took too much time between polls.')
    except (NoBrokersAvailable, NodeNotReadyError, KafkaProtocolError, KafkaTimeoutError) as kafka_exception:
        logger.warning(f'Failed to process data due to {kafka_exception}. Trying reconnect.')
        kafka_consumer = assign_next_kafka_consumer(analyzer_profile, logger)

class KafkaConsumerException(Exception):
    def __init__(self, message, logger):
        self.message = message
        logger.warning("Kafka poll Exception: " + self.message)
        super().__init__(self.message)

def poll_kafka_data(kafka_consumer, logger):
    data = []
    poll_results: Dict[TopicPartition, List[ConsumerRecord]] = dict()

    def poll_for_data(consumer, data_to_return, logger):
        try:
            data_to_return.append(consumer.get_kafka_consumer().poll(timeout_ms=consumer_poll_timeout_ms))
        except KafkaTimeoutError:
            # Handle the timeout error here
            raise KafkaConsumerException("KafkaTimeoutError",logger)
        except Exception as e:
            raise KafkaConsumerException(str(e),logger)


    # This code is an atrocity due to the nature of the previous code written before me,
    # and the limits of the kafka library
    commit_thread: threading.Thread = threading.Thread(
        target=poll_for_data, args=(kafka_consumer, data, logger))
    commit_thread.start()
    commit_thread.join(timeout=consumer_poll_timeout_ms*1.5/1000)
    if commit_thread.is_alive():
       if len(data) == 0:
          raise KafkaConsumerException("Thread timed out",logger)

    if len(data) >= 1:
        poll_results = data[0]
    return poll_results




if __name__ == "__main__":
    asyncio.run(main())
