/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2024                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     TPFDF Stored procedures                             */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';


DROP FUNCTION IF EXISTS `tpf_get_metric_profile_names_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_metric_profile_names_func` (
    `in_data_type` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE result JSON;
DECLARE available_results JSON;

SET available_results = tpf_get_metric_profiles_available_func();

SELECT JSON_ARRAYAGG(name)
INTO result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    NESTED PATH '$.metrics[*]' COLUMNS (
        metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.name != "" AND jt.datatype = in_data_type;

RETURN result;
END//
DELIMITER ;


--
-- Get the list of available functions
-- to display to the user in the drop down.
--
DROP FUNCTION IF EXISTS `tpf_get_analysis_name_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_analysis_name_func` (
  `in_panel_key` TEXT
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE result JSON;
DECLARE available_results JSON;

SET available_results = tpf_get_available_analysis_func();

SELECT JSON_ARRAYAGG(name)
INTO result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    table_procedure         VARCHAR(64) PATH '$.table_procedure',
    bar_procedure         VARCHAR(64) PATH '$.bar_procedure',
    line_procedure         VARCHAR(64) PATH '$.line_procedure',
    NESTED PATH '$.attributes[*]' COLUMNS (
      attributes VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.name != "" AND (jt.datatype = "all" OR jt.datatype = in_panel_key);

RETURN result;
END//
DELIMITER ;

--
-- Given the name, get the corresponding procedure name.
-- e.g. for name "Top [limit] by [analysis metric]",
-- return the table_procedure tpf_cdf_table
--
DROP FUNCTION IF EXISTS `tpf_find_analysis_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_find_analysis_func` (
    `in_func_name` TEXT,
    `in_procedure_type` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE table_result TEXT DEFAULT "";
DECLARE bar_result TEXT DEFAULT "";
DECLARE line_result TEXT DEFAULT "";
DECLARE available_results JSON;

SET available_results = tpf_get_available_analysis_func();

SELECT table_procedure,bar_procedure,line_procedure
INTO table_result, bar_result, line_result
FROM json_table(available_results, '$[*]'
  COLUMNS(
    datatype         VARCHAR(64) PATH '$.datatype',
    name             VARCHAR(64) PATH '$.name',
    table_procedure         VARCHAR(64) PATH '$.table_procedure',
    bar_procedure         VARCHAR(64) PATH '$.bar_procedure',
    line_procedure         VARCHAR(64) PATH '$.line_procedure',
    NESTED PATH '$.attributes[*]' COLUMNS (
      attributes VARCHAR(64) PATH '$'
    )
  )
) AS jt
WHERE jt.name != "null" AND jt.name = in_func_name;

IF in_procedure_type = "table_procedure"
THEN
    RETURN table_result;
ELSEIF in_procedure_type = "bar_procedure"
THEN
    RETURN bar_result;
END IF;
RETURN line_result;
END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_common_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_common_table` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
     IN `in_filters_selected` JSON,
    IN `ss_entry` TEXT,
    IN `systems_columns` JSON,
    IN `in_analysis_metric` VARCHAR(64),
    IN `max_results_value` INT,
    IN `in_user_input` TEXT
)
tpf_common_table:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE from_stmt TEXT DEFAULT "";
DECLARE get_time_stmt TEXT DEFAULT "";
DECLARE partition_statement TEXT DEFAULT "";
DECLARE ordered_select_stmnt TEXT DEFAULT "";
DECLARE key_clause_stmt TEXT DEFAULT "";
DECLARE select_metric_rows_statement TEXT DEFAULT "";
DECLARE select_sum_statement TEXT DEFAULT "";
DECLARE select_avg_statement TEXT DEFAULT "";
DECLARE available_results JSON;
DECLARE metric_columns TEXT DEFAULT "";
DECLARE average_metric_columns TEXT DEFAULT "";
DECLARE sum_metric_columns TEXT DEFAULT "";
DECLARE key_name_list JSON;
DECLARE key_name TEXT DEFAULT "";
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE table_name TEXT DEFAULT "";
DECLARE at_table_name_field TEXT DEFAULT "";
DECLARE at_name_average_display TEXT DEFAULT "";
DECLARE at_name TEXT DEFAULT "";
DECLARE at_name_display  TEXT DEFAULT "";
DECLARE at_name_list JSON;
DECLARE rightmost_time DATETIME;
DECLARE in_show_metrics_contained BOOLEAN DEFAULT TRUE;
DECLARE extra_column_entry_list JSON;
DECLARE  extra_column_entry TEXT DEFAULT "";
DECLARE extra_column_avg_display TEXT DEFAULT "";
DECLARE analysis_metric_compare TEXT DEFAULT "";
DECLARE inner_join_attr TEXT DEFAULT "";

-- Get the table name
SET table_name = tpf_get_table_name(in_panel_key);
SET at_table_name_field = tpf_get_table_name_field_available_func(in_panel_key);

-- Process metrics
SELECT GROUP_CONCAT(CONCAT("`",input_fields,"`"))
INTO  metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT("AVG(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  average_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT("SUM(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  sum_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SET at_name_list = tpf_get_attributes_available_func(in_panel_key);
SELECT GROUP_CONCAT("'N/A' as ", at_fields)
INTO  at_name_average_display
FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT(at_fields)
INTO  at_name
FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;

SET at_name_display = tpf_attribute_case_stmt(in_panel_key, at_name_list);

-- Process extra columns
SET extra_column_entry_list =  tpf_get_extra_cols_available_func(in_panel_key);
IF extra_column_entry_list IS NOT NULL AND JSON_LENGTH(extra_column_entry_list) > 0
THEN
    -- There are both metrics and extra fields.
    IF table_name != "tpf_tcpip_metrics_table"
    THEN
        SELECT GROUP_CONCAT(extra_fields),  GROUP_CONCAT("'N/A' as ", extra_fields)
        INTO  extra_column_entry, extra_column_avg_display
        FROM JSON_TABLE(extra_column_entry_list, "$[*]" COLUMNS(extra_fields VARCHAR(64) path "$")) ism;
    ELSE
        -- For the tcpip table only, for the extra columns,
        --  if the value for the extra column is less than zero, show None.
        SELECT GROUP_CONCAT(" CASE WHEN ",extra_fields,
                         " < 4294967295 THEN ",extra_fields,
                         " ELSE 'None' END ",
                          "AS ", extra_fields, " "),
        GROUP_CONCAT("'N/A' as ", extra_fields)
        INTO extra_column_entry,
        extra_column_avg_display
        FROM JSON_TABLE(extra_column_entry_list, "$[*]" COLUMNS(extra_fields VARCHAR(64) path "$")) ism;
    END IF;
END IF;

-- remove the quotes to use the in_analysis_metric in the ORDER BY clause
-- If the analysis metric is like average or totals, assume the user
-- wants the analysis to be based on this metric and remove the suffix.
IF in_analysis_metric != ""
THEN
    -- Remove the averages or totals from the analysis metric
    IF in_analysis_metric LIKE "%-avg_systems" OR
        in_analysis_metric LIKE "%-total_systems"
    THEN
        SET in_analysis_metric = SUBSTRING_INDEX(in_analysis_metric, '-', 1);
    END IF;
    SET in_show_metrics_contained = JSON_CONTAINS(in_show_metrics,  CONCAT('"',in_analysis_metric,'"'));
    SET analysis_metric_compare = in_analysis_metric;
    SET in_analysis_metric = CONCAT("`",in_analysis_metric,"`");
END IF;

-- Set up the various statements
SET  partition_statement = CONCAT(", ROW_NUMBER() OVER (PARTITION BY tn.system_id ",
    CASE WHEN  ss_entry = "yes" THEN  ",tn.subsystem_id " ELSE "" END,
    " ORDER BY time DESC) AS row_num ");
--
-- NOTE: This procedure is not invoked when this is filter on an attribute.
-- The exception is the tpfdf data type.
--
SET analysis_metric_compare = CONCAT("`",analysis_metric_compare,"`");
if in_panel_key = "tpfdf"
THEN
    SET  inner_join_attr = "file_id";
END IF;
-- Call a generic method to get the INNER JOIN clause
SET from_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
      inner_join_attr, in_filters_selected, in_system_ids, systems_columns);
SET from_stmt = CONCAT(" FROM ", table_name, " tn\n ", from_stmt);

SET get_time_stmt =
CONCAT(
    "WITH ordered AS ( \nSELECT TIME ",
      partition_statement, "\n",
      from_stmt,
    "\n WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time),
    CASE WHEN in_user_input != "" THEN CONCAT(" AND ", analysis_metric_compare, " > ", in_user_input) ELSE "" END,
    "\n ORDER BY TIME DESC) ",
   "\n SELECT time INTO @rightmost_time \nFROM ordered \nWHERE row_num = 1 \nLIMIT 1"
);
CALL tpf_execute_immediate(get_time_stmt);
SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
    LEAVE tpf_common_table;
END IF;

--
-- Attributes as well as all keys need to be shown in the table.
-- Keys are not necessarily attributes.
-- Get the keys from the schema.
--
SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @key_name_list ",
         " FROM information_schema.KEY_COLUMN_USAGE ",
         " WHERE TABLE_NAME = '",table_name,"'",
         " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
CALL tpf_execute_immediate(select_stmt);
SET key_name_list = tpf_get_keys_not_in_attributes(in_panel_key, @key_name_list, at_name_list);
SELECT GROUP_CONCAT(key_fields)
INTO  key_name
FROM JSON_TABLE(key_name_list, "$[*]" COLUMNS(key_fields VARCHAR(64) path "$")) ism;

-- NOTE: Using systems instead of system, since system is a mySQL keyword
-- show attributes, including the keys
SET ordered_select_stmnt = CONCAT(" SELECT itsitn.display_name AS systems ",
    CASE WHEN  ss_entry = "yes" THEN  ",tss.subsystem AS subsystems" ELSE "" END,
    CASE WHEN at_name != "" THEN  CONCAT(", ", at_name) ELSE "" END,
     ", ", metric_columns,
    CASE WHEN  extra_column_entry != '' THEN  CONCAT( ", ", extra_column_entry) ELSE "" END,
    CASE WHEN  in_show_metrics_contained IS FALSE THEN  CONCAT( ", ", in_analysis_metric) ELSE "" END,
    ", time "
);

SET select_metric_rows_statement = CONCAT(" SELECT systems ",
    CASE WHEN  ss_entry = "yes" THEN  ",subsystems" ELSE "" END,
    CASE WHEN at_name != "" THEN CONCAT(", ", at_name_display) ELSE "" END);

IF found_key_attribute IS TRUE
THEN
SET select_metric_rows_statement = CONCAT(select_metric_rows_statement,
    CASE WHEN  metric_columns != '' THEN CONCAT( ", ", metric_columns) ELSE "" END,
    CASE WHEN  extra_column_entry != "" THEN CONCAT(",",extra_column_entry) ELSE "" END, ",time");
ELSE
    SET select_metric_rows_statement = CONCAT(select_metric_rows_statement,
        CASE WHEN  sum_metric_columns != '' THEN CONCAT( ", ", sum_metric_columns) ELSE "" END,
        CASE WHEN  metric_columns != '' THEN CONCAT( ", ", metric_columns) ELSE "" END,
        CASE WHEN  extra_column_entry != "" THEN CONCAT(",",extra_column_entry) ELSE "" END, ",time");
    SET key_clause_stmt = CONCAT("GROUP BY systems ",
        CASE WHEN  ss_entry = "yes" THEN  ",subsystems " ELSE "" END);
END IF;

SET select_sum_statement = tpf_create_select_agg_stmt(ss_entry,at_name_average_display, sum_metric_columns, extra_column_avg_display, key_name);
SET select_avg_statement = tpf_create_select_agg_stmt(ss_entry,at_name_average_display, average_metric_columns, extra_column_avg_display, key_name);
-- NOTE:  ORDER BY from a subquery is ignored. The ORDER BY clause must be at the top level select.
SET select_stmt =
 CONCAT( " WITH ordered AS (\n",
    ordered_select_stmnt, "\n",
    from_stmt,
   "\n WHERE time = ", QUOTE(rightmost_time), "\n",
   CASE WHEN in_user_input != "" THEN CONCAT(" AND ", analysis_metric_compare, " > ", in_user_input) ELSE "" END,
   "), ",
   "\n basic_ordered AS \n(" ,
        "SELECT * \nFROM ordered ",
       CASE WHEN in_analysis_metric != "" THEN CONCAT(" ORDER BY ", in_analysis_metric, " DESC ") ELSE "" END,
      CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END,
    " ),",
    "\n totals_ordered AS \n(" ,
        "SELECT systems  ",
        select_sum_statement,
        "\n FROM ordered ",
        " GROUP BY ordered.systems",
    " ),",
    "\n average_ordered AS \n(" ,
        "SELECT systems  ",
        select_sum_statement,
        "\n FROM ordered ",
        " GROUP BY ordered.systems",
    " )\n",
     select_metric_rows_statement,
    "\n FROM basic_ordered ",
    " \n UNION ALL \n",
       -- Add the attribute name to the avg, for example, avg_by-device_name-C
     "SELECT CONCAT('avg",
     "-',ordered.systems) as systems  ",
     select_avg_statement,
     "\n FROM ordered ",
     " GROUP BY ordered.systems ",
    " \n UNION ALL \n",
     "SELECT CONCAT('total-',ordered.systems) as systems",
     select_sum_statement,
     "\n FROM ordered ",
     " GROUP BY ordered.systems ",
     " \n UNION ALL \n",
     "SELECT 'total-*' as systems",
     select_sum_statement,
     "\n FROM totals_ordered HAVING COUNT(*) > 0"
     " \n UNION ALL \n",
    "SELECT 'avg-*' as systems",
     select_avg_statement,
     "\n FROM average_ordered HAVING COUNT(*) > 0"
 );
 CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_common_bar_chart`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_common_bar_chart` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_bar_chart_metric` TEXT,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `temp_sys_id_table_name` TEXT,
    IN `ss_entry` TEXT,
    IN `max_results_value` INT,
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON,
    IN `set_up_averages` BOOLEAN,
    IN `set_up_totals` BOOLEAN
)
tpf_common_bar_chart:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE inner_join_stmt TEXT DEFAULT "";
DECLARE ordered_stmt TEXT DEFAULT "";
DECLARE partition_statement TEXT DEFAULT "";
DECLARE totals_ordered_stmt TEXT DEFAULT "";
DECLARE avg_ordered_stmt TEXT DEFAULT "";
DECLARE cmbs_select_clause_str TEXT DEFAULT "";
DECLARE key_attribute_stmt  TEXT DEFAULT "";
DECLARE rightmost_time DATETIME;
DECLARE key_name_list JSON;
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE table_name TEXT DEFAULT "";
DECLARE systems_columns JSON;
DECLARE analysis_metric_compare TEXT DEFAULT "";
DECLARE file_id_str TEXT DEFAULT "";
DECLARE at_field_list TEXT DEFAULT "";
DECLARE at_field_list_test TEXT DEFAULT "";
DECLARE at_name_list JSON;
DECLARE at_field_display  TEXT DEFAULT "";
DECLARE inner_join_attr TEXT DEFAULT "";
DECLARE filter_on_analysis_type TEXT DEFAULT "";
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE has_altered_column INT DEFAULT 0;

IF in_bar_chart_metric != "" AND in_bar_chart_metric LIKE "%-avg_systems"
THEN
    SET in_bar_chart_metric = SUBSTRING_INDEX(in_bar_chart_metric, '-', 1);
    SET set_up_averages = TRUE;
ELSEIF in_bar_chart_metric != "" AND in_bar_chart_metric LIKE "%-total_systems"
THEN
    SET in_bar_chart_metric = SUBSTRING_INDEX(in_bar_chart_metric, '-', 1);
    SET set_up_totals = TRUE;
END IF;
SET analysis_metric_compare = CONCAT("`",in_bar_chart_metric,"`");
SET in_bar_chart_metric = CONCAT("tn.`",in_bar_chart_metric,"`");
-- Get all keys
 SET table_name = tpf_get_table_name(in_panel_key);
 SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @key_name_list ",
          " FROM information_schema.KEY_COLUMN_USAGE ",
          " WHERE TABLE_NAME = '",table_name,"'",
          " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
 CALL tpf_execute_immediate(select_stmt);
 SET key_name_list = @key_name_list;
  SET found_key_attribute = tpf_check_for_keys_in_attribute_list(in_panel_key,key_name_list);
 SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name)) ",
                          " INTO @systems_columns",
                          " FROM ", temp_sys_id_table_name);
CALL tpf_execute_immediate(select_stmt);
SET systems_columns = @systems_columns;

-- Add the parameters for  tpfdf so that we filter on TPFDF file IDs
IF in_panel_key = "tpfdf"
THEN
    SET  inner_join_attr = "file_id";
END IF;

SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, temp_sys_id_table_name, in_subsystems, ss_entry,
   inner_join_attr,in_filters_selected, in_system_ids, NULL);

IF in_panel_key = "tpfdf"
THEN
    SET file_id_str = tpf_tpfdf_file_id_display_name();
    SET file_id_str = CONCAT(", ", file_id_str," AS file_id_display ");
END IF;

SET partition_statement = CONCAT(",\n ROW_NUMBER() OVER (PARTITION BY tn.system_id ",
    CASE WHEN ss_entry = "yes" THEN ", tn.subsystem_id " ELSE "" END,
    " ORDER BY time DESC) AS row_num \n ");

 SET has_altered_column = tpf_get_altered_column(in_panel_key);
 IF has_altered_column = 0
 THEN
        SET select_time_stmt = CONCAT(" WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time));
 ELSE
        SET select_time_stmt = CONCAT(" WHERE ((TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                  " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                  " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time))");
 END IF;

-- Determine the rightmost time point.
-- If there is data for one processor, but not the others,
-- then just show the processors that are active at that time.
 SET select_stmt =
 CONCAT(
    "WITH ordered AS \n( SELECT TIME ",
      partition_statement,
    " \nFROM ", table_name, " tn \n",
    inner_join_stmt,
    select_time_stmt,
    CASE WHEN in_user_input != "" THEN CONCAT(" AND (", analysis_metric_compare, " > ", in_user_input,")") ELSE "" END,
    " ORDER BY TIME DESC) ",
   "\n SELECT time INTO @rightmost_time FROM ordered WHERE row_num = 1 LIMIT 1"
 );
 CALL tpf_execute_immediate(select_stmt);

SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
    LEAVE tpf_common_bar_chart;
END IF;

SET ordered_stmt = CONCAT(" SELECT tn.system_id",
    ", ",in_bar_chart_metric," AS cdf_metric,",
    CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
    " AS display_name",
    " FROM ", table_name, " tn ", inner_join_stmt,
      " WHERE TIME = '",rightmost_time ,"' ",
      CASE WHEN in_user_input != "" THEN CONCAT(" AND ", analysis_metric_compare, " > ", in_user_input) ELSE "" END
);

IF set_up_totals IS FALSE AND set_up_averages IS FALSE
THEN
    IF found_key_attribute IS FALSE
    THEN
        --  the group by must go before the order by
        SET key_attribute_stmt = CONCAT(" GROUP BY tn.system_id ",
            CASE WHEN ss_entry = "yes" THEN ",tss.subsystem " ELSE "" END);
    END IF;
    SET  at_name_list = tpf_get_attributes_available_func(in_panel_key);

    SELECT GROUP_CONCAT(at_fields), GROUP_CONCAT(at_fields,",'-'")
    INTO at_field_list, at_field_display
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
    SELECT GROUP_CONCAT(" CASE WHEN ", at_fields, " IS NOT NULL THEN ", at_fields, " ELSE 'N/A' END AS ", at_fields)
    INTO at_field_list_test
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;


    SET select_stmt = CONCAT("\n SELECT tn.system_id,",
        CASE WHEN found_key_attribute IS FALSE THEN CONCAT("SUM(",in_bar_chart_metric,") ") ELSE in_bar_chart_metric END,
        " AS cdf_metric,\n",
        CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
        " AS display_name \n",
        CASE WHEN file_id_str != "" THEN CONCAT(file_id_str) ELSE "" END,
        CASE WHEN at_field_list_test != "" THEN CONCAT(",", at_field_list_test) ELSE "" END,
        "\n FROM ", table_name, " tn \n", inner_join_stmt,
        "\n WHERE TIME = '",rightmost_time ,"' ",
        CASE WHEN in_user_input != "" THEN CONCAT(" AND ", analysis_metric_compare, " > ", in_user_input) ELSE "" END,
        key_attribute_stmt,
        "\n   ORDER BY cdf_metric DESC \n "
    );
    IF in_panel_key = "tpfdf"
    THEN
        SET cmbs_select_clause_str = CONCAT("\n SELECT CONCAT(file_id_display,'-',display_name) AS systems, \n",
        "  JSON_OBJECT(CONCAT(file_id_display,'-',display_name), cdf_metric) AS nvps  \n");
    ELSEIF at_field_display != ""
    THEN
        SET cmbs_select_clause_str = CONCAT("\n SELECT CONCAT(",at_field_display,",display_name) AS systems, \n",
        " JSON_OBJECT(CONCAT(",at_field_display,",display_name), cdf_metric) AS nvps \n");
    ELSE
        SET cmbs_select_clause_str = "\n SELECT display_name AS systems, JSON_OBJECT(display_name, cdf_metric) AS nvps \n";
    END IF;
    SET cmbs_select_clause_str =
        CONCAT(cmbs_select_clause_str,"\n FROM ordered ",
               CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END);
    SET select_stmt  = CONCAT("WITH ordered AS ( ", select_stmt, ") ",cmbs_select_clause_str);
    CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_common_bar_chart;
END IF;

IF in_panel_key = "tpfdf"
THEN
    SET filter_on_analysis_type =  "file_id";
END IF;
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
      filter_on_analysis_type, in_filters_selected, in_system_ids, systems_columns);

 SET totals_ordered_stmt = CONCAT(
        "WITH totals_ordered AS ( \nSELECT itsitn.display_name",
         ", SUM(",in_bar_chart_metric,") AS cdf_metric \n",
        " FROM ", table_name, " tn\n",
        inner_join_stmt,
        "\n WHERE TIME = '",rightmost_time ,"' \n",
        " GROUP BY system_id)\n "
 );
 SET avg_ordered_stmt = CONCAT(" SELECT tn.system_id",
     ", ",in_bar_chart_metric," AS cdf_metric,",
     CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
     " AS display_name",
     " FROM ", table_name, " tn ", inner_join_stmt,
       " WHERE TIME = '",rightmost_time ,"' ",
       CASE WHEN in_user_input != "" THEN CONCAT(" AND ", analysis_metric_compare, " > ", in_user_input) ELSE "" END
 );

IF set_up_totals IS TRUE THEN
    SET select_stmt = CONCAT(totals_ordered_stmt,
        " SELECT CONCAT('total-', display_name) AS systems, \n",
        " JSON_OBJECT(CONCAT('total-', display_name), cdf_metric) AS nvps \n",
        " FROM totals_ordered \n"
        " UNION ",
        " SELECT 'total-*' AS systems, \n",
        " JSON_OBJECT('total-*', SUM(cdf_metric)) AS nvps \nFROM totals_ordered");
   CALL tpf_execute_immediate(select_stmt);
END IF;
IF set_up_averages IS FALSE THEN
    LEAVE tpf_common_bar_chart;
END IF;

SET totals_ordered_stmt = CONCAT(
        "totals_ordered AS ( SELECT display_name",
        ", SUM(cdf_metric) AS cdf_metric FROM ordered GROUP BY system_id) "
 );

SET select_stmt = CONCAT(
 "WITH ordered AS ( \n", avg_ordered_stmt, "\n),\n ",totals_ordered_stmt,
        "\n SELECT CONCAT('avg",
        "-', display_name) AS systems, \n",
        " JSON_OBJECT(CONCAT('avg",
        "-', display_name), AVG(cdf_metric)) AS nvps  \nFROM ordered \nGROUP BY system_id\n",
        " UNION \n",
        " SELECT 'avg-*' AS systems, \n",
        " JSON_OBJECT('avg-*', AVG(cdf_metric)) AS nvps \nFROM totals_ordered");

CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_common_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_common_graph` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `ss_entry` TEXT,
    IN `in_analysis_metric` VARCHAR(64),
    IN `max_results_value` INT,
    IN `in_metric_type` JSON,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` VARCHAR(64),
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON
)
tpf_common_graph:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE series_name_stmt LONGTEXT DEFAULT "";
DECLARE inner_join_stmt LONGTEXT DEFAULT "";
DECLARE group_by_stmt LONGTEXT DEFAULT "";
DECLARE table_name VARCHAR(64)  DEFAULT "";
DECLARE temp_sys_id_table_name TEXT DEFAULT "";
DECLARE metric_columns TEXT DEFAULT "";
DECLARE metric_columns_list TEXT DEFAULT "";
DECLARE group_by_time_shift INT;
DECLARE lt_time DATETIME;
DECLARE lt_time_shift INT;
DECLARE key_name_list TEXT DEFAULT "";
DECLARE in_show_metrics_contained BOOLEAN DEFAULT TRUE;
DECLARE num_subsystems INT DEFAULT 0;
DECLARE metric_columns_json JSON;
DECLARE avg_metric_columns TEXT DEFAULT "";
DECLARE analysis_metric_compare TEXT DEFAULT "";
DECLARE at_field_display TEXT DEFAULT "";
DECLARE at_name_list JSON;
DECLARE attribute_table_statement TEXT;
DECLARE attribute_table_name_list JSON;
DECLARE inner_join_attr TEXT DEFAULT "";
DECLARE select_time_stmt TEXT DEFAULT "";
DECLARE has_altered_column INT DEFAULT 0;
DECLARE only_summarized_metrics BOOLEAN DEFAULT FALSE;
DECLARE where_stmt TEXT DEFAULT "";

-- Generic function to get the table name
SET table_name = tpf_get_table_name(in_panel_key);

-- Calculate time shift
-- Make sure the intervals start with in_rt_time i.e. make the right n (in_sec_per_interval) seconds part of the same interval
SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
-- make sure that the left time bound of data we select is the lt bound of an interval
-- so that SUM over the interval doesn't look lower than it should be.
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);
SET lt_time = DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND);

IF in_panel_key != "tpfdf"
THEN
    SET at_name_list = tpf_get_attributes_available_func(in_panel_key);
    SELECT GROUP_CONCAT(" CASE WHEN ",at_fields,
                     " IS NOT NULL THEN ",at_fields,
                     " ELSE 'N/A' END ")
    INTO  at_field_display
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
ELSE
    SET at_field_display = tpf_get_table_name_field_available_func(in_panel_key);
END IF;

SET select_stmt = CONCAT(" SELECT GROUP_CONCAT(COLUMN_NAME) INTO @key_name_list ",
         " FROM information_schema.KEY_COLUMN_USAGE ",
         " WHERE TABLE_NAME = '",table_name,"'",
         " AND COLUMN_NAME NOT IN ('time','system_id', 'subsystem_id')");
CALL tpf_execute_immediate(select_stmt);
SET key_name_list = @key_name_list;

SET ss_entry = tpf_get_subsystem_applies(in_panel_key);
SET num_subsystems = JSON_LENGTH(in_subsystems);

SET series_name_stmt = tpf_create_series_name_string(num_subsystems,in_agg_type, at_field_display, in_panel_key);

-- Call a generic method to get the INNER JOIN clause
IF in_agg_type = "no_agg"
THEN
    SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
ELSE
    -- Create a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;
-- Add the parameters for  tpfdf so that we filter on TPFDF file IDs
 IF in_panel_key = "tpfdf"
 THEN
    SET  inner_join_attr = "file_id";
 END IF;

-- Call a generic method to get the INNER JOIN clause
 SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, temp_sys_id_table_name, in_subsystems,
    ss_entry, inner_join_attr,in_filters_selected,  in_system_ids, NULL);

SET has_altered_column = tpf_get_altered_column(in_panel_key);
IF  has_altered_column = 0
THEN
   SET select_time_stmt = CONCAT(" WHERE time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time));
ELSE
   SET select_time_stmt = CONCAT(" WHERE ((TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time))");
END IF;

SET analysis_metric_compare = CONCAT("`",in_analysis_metric,"`");

-- no_agg path
IF in_agg_type = "no_agg"
THEN
    --  set up the metric columns
    SET in_metric_type = JSON_UNQUOTE(in_metric_type);

    SELECT GROUP_CONCAT("AVG(`", input_fields ,"`) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';

    -- This will be blank if no per second metrics are selected.
    IF metric_columns IS NULL OR metric_columns = ""
    THEN
        LEAVE tpf_common_graph;
    END IF;


    SELECT GROUP_CONCAT("`", input_fields, "`")
    INTO  metric_columns_list
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';

    SET only_summarized_metrics = contains_only_summarized_metrics(in_show_metrics);

    SET in_show_metrics_contained = JSON_CONTAINS(in_show_metrics,  CONCAT('"',in_analysis_metric,'"'));
    SET in_analysis_metric = CONCAT('`',in_analysis_metric,'`');
    SET where_stmt = CONCAT(CASE WHEN max_results_value > 0 AND  only_summarized_metrics IS TRUE
                THEN CONCAT(" WHERE row_num <= ", max_results_value, " AND entry_is_pruned IS TRUE ")
                WHEN only_summarized_metrics IS TRUE THEN " WHERE entry_is_pruned IS TRUE "
                WHEN max_results_value > 0 THEN CONCAT(" WHERE row_num <= ", max_results_value, " " ) ELSE "" END);

    SET group_by_stmt = CONCAT(" GROUP BY system_id, ",
        CASE WHEN ss_entry = 'yes' THEN "subsystem_id, " ELSE "" END,
        CASE WHEN key_name_list != '' THEN CONCAT(key_name_list, ",") ELSE "" END,
        "(UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval);
    SET select_stmt =
    CONCAT(
        " WITH ordered AS \n (SELECT MIN(time) AS time, ",
        series_name_stmt,
        CASE WHEN key_name_list != "" THEN CONCAT(key_name_list, ",") ELSE "" END,
        metric_columns,
        CASE WHEN in_show_metrics_contained IS FALSE THEN  CONCAT(",",in_analysis_metric) ELSE "" END,
        ", \n ROW_NUMBER() OVER (PARTITION BY TIME ORDER BY TIME DESC ",
        CASE WHEN in_analysis_metric != "" THEN CONCAT(",", in_analysis_metric, " DESC ") ELSE "" END,
        " ) AS row_num\n ",
        CASE WHEN only_summarized_metrics IS TRUE THEN ", entry_is_pruned " ELSE "" END,
        " FROM ", table_name, " tn \n",
        inner_join_stmt,
        select_time_stmt,
        CASE WHEN in_user_input != "" THEN CONCAT(" AND (", analysis_metric_compare, " > ", in_user_input,") ") ELSE " " END,
        group_by_stmt,
        "\n ORDER BY time ASC \n",
        CASE WHEN in_analysis_metric != "" THEN CONCAT(",", in_analysis_metric, " DESC ") ELSE "" END,
        ") \n",
        " SELECT time, series_name, \n ",
        metric_columns_list,
        " FROM ordered ",where_stmt
    );
    CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_common_graph;
END IF;
-- This is either total or average systems request

-- remove the no_agg metrics and any metrics not like the selected agg type.
SET select_stmt = CONCAT(
    " SELECT JSON_ARRAYAGG(input_fields)",
    " INTO @metric_columns_json",
    " FROM JSON_TABLE('",in_show_metrics,"', \"$[*]\" COLUMNS(input_fields VARCHAR(128) path \"$\")) showMetrics",
    " WHERE input_fields LIKE '%-",in_agg_type,"';"
);
CALL tpf_execute_immediate(select_stmt);
SET metric_columns_json=@metric_columns_json;
IF metric_columns_json IS NULL
THEN
    LEAVE tpf_common_graph;
END IF;
IF JSON_LENGTH(metric_columns_json) = 0
THEN
    LEAVE tpf_common_graph;
END IF;

IF in_agg_type = 'total_systems'
THEN
    SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

ELSE
    SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", SUBSTRING_INDEX(input_fields, '-', 1) , "`")
    INTO  metric_columns
    FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    SELECT GROUP_CONCAT("AVG(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) AS `", input_fields, "`")
    INTO  avg_metric_columns
    FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
END IF;
IF metric_columns IS NULL OR metric_columns = ""
THEN
    LEAVE tpf_common_graph;
END IF;

SET group_by_stmt = CONCAT(" GROUP BY (UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    CASE WHEN in_agg_type = 'avg_systems' THEN ", tn.system_id " ELSE "" END);

SET select_stmt =
 CONCAT(
    " SELECT MIN(time) AS time, ",
    series_name_stmt,
    metric_columns,
    "\n FROM ", table_name, " tn \n",
    inner_join_stmt,
    select_time_stmt,
    CASE WHEN in_user_input != "" THEN CONCAT(" AND (", analysis_metric_compare, " > ", in_user_input,") ") ELSE " " END,
    group_by_stmt,
    "\n ORDER BY time ASC"
 );

 -- For a request for average, generate the select statement like this is the totals.
 -- From there, calculate the average.
 IF in_agg_type = 'avg_systems'
 THEN
    SET select_stmt =
        CONCAT(" WITH totals_ordered AS \n(", select_stmt, ") \n ",
           " SELECT time, 'avg-systems' AS series_name, \n",
           avg_metric_columns,
           "\n FROM totals_ordered ",
           "\n GROUP BY totals_ordered.time ");
END IF;
CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_user_input_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_user_input_table` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_filters_selected` JSON,
    IN `in_ss_entry` TEXT,
    IN `in_systems_columns` JSON,
    IN `in_analysis_metric` VARCHAR(64),
    IN `in_max_results_value` INT,
    IN `in_user_input` TEXT
)
tpf_user_input_table:BEGIN

SELECT in_user_input REGEXP '[^0-9.]' INTO @user_input_value;
IF @user_input_value != 0
THEN
    SELECT CONCAT("ERROR: Non-numeric value ",in_user_input, " is not allowed for User Input.");
    LEAVE tpf_user_input_table;
END IF;
SELECT CAST(in_user_input AS FLOAT) into in_user_input;

CALL tpf_common_table(
    in_analysis_type, in_panel_key, in_show_metrics,
    in_system_ids, in_subsystems,
    in_lt_time, in_rt_time,in_filters_selected, in_ss_entry, in_systems_columns,
    in_analysis_metric, in_max_results_value, in_user_input);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_user_input_bar_chart`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_user_input_bar_chart` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_bar_chart_metric` TEXT,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_temp_sys_id_table_name` TEXT,
    IN `in_ss_entry` TEXT,
    IN `in_max_results_value` INT,
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON,
    IN `in_set_up_averages` BOOLEAN,
    IN `in_set_up_totals` BOOLEAN
)
tpf_user_input_bar_chart:BEGIN

SELECT in_user_input REGEXP '[^0-9.]' INTO @user_input_value;
IF @user_input_value != 0
THEN
    LEAVE tpf_user_input_bar_chart;
END IF;
SELECT CAST(in_user_input AS FLOAT) into in_user_input;
CALL tpf_common_bar_chart(
        in_analysis_type, in_panel_key, in_bar_chart_metric,
        in_system_ids, in_subsystems,
        in_lt_time, in_rt_time, in_temp_sys_id_table_name, in_ss_entry,
        in_max_results_value, in_user_input, in_filters_selected,
        in_set_up_averages,
        in_set_up_totals);
END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_user_input_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_user_input_graph` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_ss_entry` TEXT,
    IN `in_analysis_metric` VARCHAR(64),
    IN `in_max_results_value` INT,
    IN `in_metric_type` JSON,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` VARCHAR(64),
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON
)
tpf_user_input_graph:BEGIN

SELECT in_user_input REGEXP '[^0-9.]' INTO @user_input_value;
IF @user_input_value != 0
THEN
    LEAVE tpf_user_input_graph;
END IF;
SELECT CAST(in_user_input AS FLOAT) into in_user_input;
CALL tpf_common_graph(
        in_analysis_type, in_panel_key, in_show_metrics, in_system_ids,
        in_subsystems, in_lt_time, in_rt_time, in_ss_entry,
        in_analysis_metric, in_max_results_value, in_metric_type,
        in_sec_per_interval, in_agg_type, in_user_input,
        in_filters_selected);
END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_df_files_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_df_files_table` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_filters_selected` JSON,
    IN `ss_entry` TEXT,
    IN `systems_columns` JSON,
    IN `in_analysis_metric` VARCHAR(64),
    IN `max_results_value` INT,
    IN `in_user_input` TEXT
)
tpf_df_files_table:BEGIN

DECLARE table_name TEXT DEFAULT "";
DECLARE at_table_name_field TEXT DEFAULT "";
DECLARE metric_columns TEXT DEFAULT "";
DECLARE average_metric_columns TEXT DEFAULT "";
DECLARE sum_metric_columns TEXT DEFAULT "";
DECLARE at_name_average_display TEXT DEFAULT "";
DECLARE at_name TEXT DEFAULT "";
DECLARE at_field_display TEXT DEFAULT "";
DECLARE at_name_list JSON;
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE from_stmt  LONGTEXT DEFAULT "";
DECLARE partition_statement LONGTEXT DEFAULT "";
DECLARE select_sum_statement LONGTEXT DEFAULT "";
DECLARE select_avg_statement LONGTEXT DEFAULT "";
DECLARE rightmost_time DATETIME;
DECLARE where_clause_modified LONGTEXT DEFAULT "";
DECLARE select_metric_rows_statement TEXT DEFAULT "";


SET where_clause_modified = tpf_tpfdf_modified_where_clause(in_analysis_type);

-- Get the table name
SET table_name = tpf_get_table_name(in_panel_key);
SET at_table_name_field = tpf_get_table_name_field_available_func(in_panel_key);

-- Process metrics
SELECT GROUP_CONCAT(CONCAT("`",input_fields,"`"))
INTO  metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT("AVG(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  average_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

SELECT GROUP_CONCAT("SUM(", "`",input_fields,"`",") AS ", "`",input_fields,"`")
INTO  sum_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;

-- Call a generic method to get the INNER JOIN clause
SET from_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
       "file_id", in_filters_selected, in_system_ids, systems_columns);
SET from_stmt = CONCAT(" FROM ", table_name, " tn\n ", from_stmt);

SET  partition_statement = CONCAT(", ROW_NUMBER() OVER (PARTITION BY tn.system_id,tn.subsystem_id ",
    " ORDER BY time DESC) AS row_num ");

SET at_name_list = tpf_get_attributes_available_func(in_panel_key);
SELECT GROUP_CONCAT("'N/A' as ", at_fields)
INTO  at_name_average_display
FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;

SET at_field_display = tpf_attribute_case_stmt(in_panel_key, at_name_list);
SET select_sum_statement = CONCAT(",'N/A' as subsystems, ", at_name_average_display, ",", sum_metric_columns, ",time");
SET select_avg_statement = CONCAT(",'N/A' as subsystems, ", at_name_average_display, ",", average_metric_columns, ",time");

-- Determine the time point to use
-- Determine the rightmost time point.
-- If there is data for one processor, but not the others,
-- then just show the processors that are active at that time.
SET select_stmt =
 CONCAT(
    "WITH ordered AS \n( SELECT TIME ",
      partition_statement,
      from_stmt,
    " \nWHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time),
    "\n AND (", where_clause_modified, ")\n",
    " \n ORDER BY TIME DESC) ",
   " \n SELECT time INTO @rightmost_time FROM ordered WHERE row_num = 1 LIMIT 1"
);
CALL tpf_execute_immediate(select_stmt);
SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
    LEAVE tpf_df_files_table;
END IF;


SET select_metric_rows_statement = CONCAT(" SELECT systems, subsystems, ",
     at_field_display, ",", metric_columns,  ",time");

SET select_stmt = CONCAT("WITH ordered AS \n",
    " (SELECT itsitn.display_name as systems, tss.subsystem as subsystems, ",
     at_table_name_field, ",", metric_columns,  ",time",
    "\n", from_stmt,
    "\n WHERE time = ", QUOTE(rightmost_time),
    "\n AND (", where_clause_modified, ")\n",
    "\n ORDER BY TIME DESC ),\n",
    "\n basic_ordered AS \n(" ,
        "SELECT * \nFROM ordered ",
      CASE WHEN in_analysis_metric != "" THEN CONCAT(" ORDER BY '", in_analysis_metric, "' DESC ") ELSE "" END,
      CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END,
    " ),",
   " totals_ordered AS (\n" ,
        " SELECT systems ", select_sum_statement,
        "\n FROM ordered ",
        "\n GROUP BY ordered.systems",
    " ),\n",
    " average_ordered AS (\n" ,
        " SELECT systems ", select_sum_statement,
        "\n FROM ordered ",
        "\n GROUP BY ordered.systems",
    " )\n",
     select_metric_rows_statement,
    "\n FROM basic_ordered ",
    " \n UNION \n",
       -- Add the attribute name to the avg, for example, avg_by-device_name-C
     "SELECT CONCAT('avg",
     "-',ordered.systems) as systems  ",
     select_avg_statement,
     "\n FROM ordered ",
     " GROUP BY ordered.systems ",
    " \nUNION\n ",
     "SELECT CONCAT('total-',ordered.systems) as systems ",
     select_sum_statement,
     "\n FROM ordered ",
     " GROUP BY ordered.systems ",
     "\n UNION \n",
     "SELECT 'total-*' as systems",
     select_sum_statement,
     "\n FROM totals_ordered HAVING COUNT(*) > 0"
     "\n UNION  \n",
    " SELECT 'avg-*' as systems",
     select_avg_statement,
     "\n FROM average_ordered HAVING COUNT(*) > 0"
    );
    CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_df_files_bar_chart`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_df_files_bar_chart` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_bar_chart_metric` TEXT,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `temp_sys_id_table_name` TEXT,
    IN `ss_entry` TEXT,
    IN `max_results_value` INT,
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON,
    IN `set_up_averages` BOOLEAN,
    IN `set_up_totals` BOOLEAN
)
tpf_df_files_bar_chart:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE inner_join_stmt TEXT DEFAULT "";
DECLARE ordered_stmt TEXT DEFAULT "";
DECLARE partition_statement TEXT DEFAULT "";
DECLARE totals_ordered_stmt TEXT DEFAULT "";
DECLARE avg_ordered_stmt TEXT DEFAULT "";
DECLARE cmbs_select_clause_str TEXT DEFAULT "";
DECLARE key_attribute_stmt  TEXT DEFAULT "";
DECLARE rightmost_time DATETIME;
DECLARE key_name_list JSON;
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE table_name TEXT DEFAULT "";
DECLARE systems_columns JSON;
DECLARE at_name_list JSON;
DECLARE at_field_list TEXT DEFAULT "";
DECLARE attribute_list TEXT DEFAULT "";
DECLARE where_clause_modified LONGTEXT DEFAULT "";
DECLARE file_id_str TEXT DEFAULT "";

SET where_clause_modified = tpf_tpfdf_modified_where_clause(in_analysis_type);

SET  at_name_list = tpf_get_attributes_available_func(in_panel_key);
SELECT GROUP_CONCAT(at_fields, ",'-'"),GROUP_CONCAT(at_fields)
INTO at_field_list, attribute_list
FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;

SET inner_join_stmt = CONCAT(" SELECT JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name)) ",
                         " INTO @systems_columns",
                         " FROM ", temp_sys_id_table_name);
CALL tpf_execute_immediate(inner_join_stmt);
SET systems_columns = @systems_columns;

SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
       "file_id", in_filters_selected, in_system_ids, systems_columns);

SET partition_statement = CONCAT(",    ROW_NUMBER() OVER (",
    "PARTITION BY tn.system_id, tn.subsystem_id "
    " ORDER BY time DESC) AS row_num ");

-- Get the table name
SET table_name = tpf_get_table_name(in_panel_key);

-- Determine the rightmost time point.
-- If there is data for one processor, but not the others,
-- then just show the processors that are active at that time.
SET select_stmt =
 CONCAT(
    "WITH ordered AS (\n SELECT TIME ",
      partition_statement, "\n",
    " FROM ", table_name, " tn \n",
    inner_join_stmt,
    " WHERE time BETWEEN ", QUOTE(in_lt_time), " AND ", QUOTE(in_rt_time), "\n",
    "\n AND (", where_clause_modified, ")\n",
    " ORDER BY TIME DESC) \n",
    " SELECT time INTO @rightmost_time \nFROM ordered WHERE row_num = 1 LIMIT 1"
);

CALL tpf_execute_immediate(select_stmt);
SET rightmost_time = @rightmost_time;
IF rightmost_time IS NULL
THEN
     LEAVE tpf_df_files_bar_chart;
END IF;
SET file_id_str = tpf_tpfdf_file_id_display_name();
SET file_id_str = CONCAT(file_id_str," AS file_id_display ");

IF set_up_averages IS FALSE AND set_up_totals IS FALSE
THEN
    SET select_stmt = CONCAT(" SELECT tn.system_id, file_id,",
        CASE WHEN found_key_attribute IS FALSE THEN CONCAT("SUM(`",in_bar_chart_metric,"`) ") ELSE CONCAT("`",in_bar_chart_metric,"`") END,
        " AS cdf_metric,",
        "CONCAT(tss.subsystem, '-',itsitn.display_name)",
        " AS display_name, \n",
         file_id_str, " \n",
        " FROM ", table_name, " tn \n", inner_join_stmt, "\n",
        " WHERE TIME = '",rightmost_time, "' ",
         "\n AND (", where_clause_modified, ")\n",
        " GROUP BY tn.system_id,tss.subsystem, file_id \n"
        " ORDER BY cdf_metric DESC "
);
    SET cmbs_select_clause_str = CONCAT("\n SELECT CONCAT(file_id_display,'-',display_name) AS systems, \n",
    "  JSON_OBJECT(CONCAT(file_id_display,'-',display_name), cdf_metric) AS nvps  \n",
    " FROM ordered ",
    CASE WHEN max_results_value > 0 THEN CONCAT(" LIMIT ", max_results_value) ELSE "" END);
    SET select_stmt  = CONCAT("WITH ordered AS \n( ", select_stmt, ")\n ",cmbs_select_clause_str);
    CALL tpf_execute_immediate(select_stmt);
END IF;

-- NOTE: The analysis type for tpfdf is always file_id
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, "", in_subsystems, ss_entry,
      "file_id", in_filters_selected, in_system_ids, systems_columns);

 SET totals_ordered_stmt = CONCAT(
        "WITH totals_ordered AS ( \nSELECT itsitn.display_name",
         ", SUM(",in_bar_chart_metric,") AS cdf_metric \n",
        " FROM ", table_name, " tn\n",
        inner_join_stmt,
        "\n WHERE TIME = '",rightmost_time ,"' \n",
        " GROUP BY system_id)\n "
 );

 SET avg_ordered_stmt = CONCAT(" SELECT tn.system_id",
     ", ",in_bar_chart_metric," AS cdf_metric,",
     CASE WHEN ss_entry = "yes" THEN "CONCAT(tss.subsystem, '-',itsitn.display_name)" ELSE "itsitn.display_name " END,
     " AS display_name",
     " FROM ", table_name, " tn ", inner_join_stmt,
       " WHERE TIME = '",rightmost_time ,"' "
 );


IF set_up_totals IS TRUE THEN
    SET select_stmt = CONCAT(totals_ordered_stmt,
        " SELECT CONCAT('total-', display_name) AS systems, \n",
        " JSON_OBJECT(CONCAT('total-', display_name), cdf_metric) AS nvps \n",
        " FROM totals_ordered \n"
        " UNION ",
        " SELECT 'total-*' AS systems, \n",
        " JSON_OBJECT('total-*', SUM(cdf_metric)) AS nvps \nFROM totals_ordered");
    CALL tpf_execute_immediate(select_stmt);
END IF;
IF set_up_averages IS FALSE THEN
    LEAVE tpf_df_files_bar_chart;
END IF;

SET totals_ordered_stmt = CONCAT(
        "totals_ordered AS ( SELECT display_name",
        ", SUM(cdf_metric) AS cdf_metric FROM ordered GROUP BY system_id) "
 );

SET select_stmt = CONCAT(
 "WITH ordered AS ( \n", avg_ordered_stmt, "\n),\n ",totals_ordered_stmt,
        "\n SELECT CONCAT('avg",
        "-', display_name) AS systems, \n",
        " JSON_OBJECT(CONCAT('avg",
        "-', display_name), AVG(cdf_metric)) AS nvps  \nFROM ordered \nGROUP BY system_id\n",
        " UNION \n",
        " SELECT 'avg-*' AS systems, \n",
        " JSON_OBJECT('avg-*', AVG(cdf_metric)) AS nvps \nFROM totals_ordered");

CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_df_files_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_df_files_graph` (
    IN `in_analysis_type` TEXT,
    IN `in_panel_key` TEXT,
    IN `in_show_metrics` JSON,
    IN `in_system_ids` JSON,
    IN `in_subsystems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `ss_entry` TEXT,
    IN `in_analysis_metric` VARCHAR(64),
    IN `max_results_value` INT,
    IN `in_metric_type` JSON,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` VARCHAR(64),
    IN `in_user_input` TEXT,
    IN `in_filters_selected` JSON
)
tpf_df_files_graph:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE series_name_stmt LONGTEXT DEFAULT "";
DECLARE inner_join_stmt LONGTEXT DEFAULT "";
DECLARE group_by_stmt LONGTEXT DEFAULT "";
DECLARE table_name VARCHAR(64)  DEFAULT "";
DECLARE temp_sys_id_table_name TEXT DEFAULT "";
DECLARE metric_columns TEXT DEFAULT "";
DECLARE metric_columns_list TEXT DEFAULT "";
DECLARE group_by_time_shift INT;
DECLARE lt_time DATETIME;
DECLARE lt_time_shift INT;
DECLARE key_name_list TEXT DEFAULT "";
DECLARE in_show_metrics_contained BOOLEAN DEFAULT TRUE;
DECLARE num_subsystems INT DEFAULT 0;
DECLARE metric_columns_json JSON;
DECLARE avg_metric_columns TEXT DEFAULT "";
DECLARE where_clause_modified LONGTEXT DEFAULT "";
DECLARE file_id_str TEXT DEFAULT "";
DECLARE only_summarized_metrics BOOLEAN DEFAULT FALSE;
DECLARE where_stmt TEXT DEFAULT "";

SET where_clause_modified = tpf_tpfdf_modified_where_clause(in_analysis_type);

-- Get the table name
SET table_name = tpf_get_table_name(in_panel_key);
IF in_agg_type = 'no_agg'
THEN
    SET temp_sys_id_table_name = CONCAT('`tpf_temp_selected_systems_', UUID_SHORT(), '`');
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table(temp_sys_id_table_name, in_system_ids);
    --  set up the metric columns
    SET in_metric_type = JSON_UNQUOTE(in_metric_type);

    SELECT GROUP_CONCAT("AVG(`", input_fields ,"`) AS `", input_fields, "`")
    INTO  metric_columns
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';

    SELECT GROUP_CONCAT("`",input_fields,"`")
    INTO  metric_columns_list
    FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism
    WHERE input_fields NOT LIKE '%-avg_systems' AND input_fields NOT LIKE '%-total_systems';
ELSE
   -- This creates a table of selected system ids which we will join to the system_metrics table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_system_ids), ', "$[*]" COLUMNS(id INT path "$"))');

    -- remove the no_agg metrics and any metrics not like the selected agg type.
    SET select_stmt = CONCAT(
      " SELECT JSON_ARRAYAGG(input_fields)",
      " INTO @metric_columns_json",
      " FROM JSON_TABLE('",in_show_metrics,"', \"$[*]\" COLUMNS(input_fields VARCHAR(128) path \"$\")) showMetrics",
      " WHERE input_fields LIKE '%-",in_agg_type,"';"
    );
    CALL tpf_execute_immediate(select_stmt);
    SET metric_columns_json=@metric_columns_json;

    IF in_agg_type = 'total_systems'
    THEN
        SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", input_fields, "`")
        INTO  metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    ELSE
        SELECT GROUP_CONCAT("SUM(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) /COUNT(DISTINCT time) AS `", SUBSTRING_INDEX(input_fields, '-', 1) , "`")
        INTO  metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
        SELECT GROUP_CONCAT("AVG(`", SUBSTRING_INDEX(input_fields, '-', 1) ,"`) AS `", input_fields, "`")
        INTO  avg_metric_columns
        FROM JSON_TABLE(metric_columns_json, "$[*]" COLUMNS(input_fields VARCHAR(64) path "$")) ism;
    END IF;
END IF;

-- metric_columns will be empty if only per second metrics are requested and in_agg_type is totals or averages
-- Conversely, these will be empty if only multi-system metrics are selected and in_agg_type is no_agg
IF metric_columns IS NULL OR metric_columns=""
THEN
    LEAVE tpf_df_files_graph;
END IF;

-- Calculate time shift
-- Make sure the intervals start with in_rt_time i.e. make the right n (in_sec_per_interval) seconds part of the same interval
SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
-- make sure that the left time bound of data we select is the lt bound of an interval
-- so that SUM over the interval doesn't look lower than it should be.
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);
SET lt_time = DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND);

-- Generic method to get the series name
SET series_name_stmt = tpf_create_series_name_string(num_subsystems,in_agg_type, "file_id",in_panel_key);

-- Method to get the display string of the File ID
SET file_id_str = tpf_tpfdf_file_id_display_name();
SET file_id_str = CONCAT(file_id_str," AS file_id_display ");

-- Generic method to get the INNER JOIN clause
SET inner_join_stmt = tpf_create_inner_join_string(in_panel_key, temp_sys_id_table_name, in_subsystems,
        ss_entry, "file_id", in_filters_selected,  in_system_ids, NULL);
IF in_agg_type = 'no_agg'
THEN
   SET only_summarized_metrics = contains_only_summarized_metrics(in_show_metrics);

    SET group_by_stmt = CONCAT(" GROUP BY system_id, subsystem_id, file_id, file_id_ebcdic, file_name_alias, ",
        "(UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval);

    SET where_stmt = CONCAT(CASE WHEN max_results_value > 0 AND  only_summarized_metrics IS TRUE
                THEN CONCAT(" WHERE row_num <= ", max_results_value, " AND entry_is_pruned IS TRUE ")
                WHEN only_summarized_metrics IS TRUE THEN " WHERE entry_is_pruned IS TRUE "
                WHEN max_results_value > 0 THEN CONCAT(" WHERE row_num <= ", max_results_value, " " ) ELSE "" END);

    SET select_stmt =
        CONCAT(" WITH ordered AS \n (SELECT MIN(time) AS time, ",
            series_name_stmt,
            metric_columns,
            CASE WHEN in_show_metrics_contained IS FALSE THEN  CONCAT(",`",in_analysis_metric, "` ") ELSE "" END,
            ", \n ROW_NUMBER() OVER (PARTITION BY TIME ORDER BY TIME DESC ",
            CASE WHEN in_analysis_metric != "" THEN CONCAT(",`", in_analysis_metric, "` DESC ") ELSE "" END,
            " ) AS row_num \n",
            CASE WHEN only_summarized_metrics IS TRUE THEN ", entry_is_pruned " ELSE "" END,
            " FROM ", table_name, " tn \n",
            inner_join_stmt,
            "\n WHERE time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
            "\n AND (", where_clause_modified, ")\n",
            group_by_stmt,
            "\n ORDER BY time ASC \n",
            CASE WHEN in_analysis_metric != "" THEN CONCAT(",`", in_analysis_metric, "` DESC ") ELSE "" END,
            ") \n",
            " SELECT time, series_name, \n ",
            metric_columns_list,
            "\n FROM ordered ", where_stmt
    );
    CALL tpf_execute_immediate(select_stmt);
    LEAVE tpf_df_files_graph;
END IF;



-- We are here either for averages or totals.
SET group_by_stmt =
    CONCAT(" GROUP BY (UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    CASE WHEN in_agg_type = 'avg_systems' THEN ", tn.system_id " ELSE "" END);

SET select_stmt =
 CONCAT(
    " SELECT MIN(time) AS time, ",
    series_name_stmt,
    metric_columns,
    " FROM ", table_name, " tn ",
    inner_join_stmt,
    " WHERE time BETWEEN ", QUOTE(lt_time), " AND ", QUOTE(in_rt_time),
    group_by_stmt,
    " ORDER BY time ASC"
 );

 -- For a request for average, generate the select statement like this is the totals.
 -- From there, calculate the average.
 IF in_agg_type = 'avg_systems'
 THEN
    SET select_stmt =
        CONCAT(" WITH totals_ordered AS (", select_stmt, ") ",
           " SELECT time, 'avg-systems' AS series_name, ",
           avg_metric_columns,
           " FROM totals_ordered ",
           " GROUP BY totals_ordered.time ");
END IF;
CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_attribute_case_stmt`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_attribute_case_stmt` (
    `in_panel_key` TEXT,
    `at_name_list` JSON
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE at_name_display TEXT DEFAULT "";

IF in_panel_key != 'tpfdf'
THEN
    SELECT GROUP_CONCAT(" CASE WHEN ",at_fields,
                     " IS NOT NULL THEN ",at_fields,
                     " ELSE 'N/A' END ",
                      "AS ", at_fields, " ")
    INTO  at_name_display
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
ELSE
    SELECT GROUP_CONCAT(" CASE WHEN ",at_fields,
                         " != '' THEN ",at_fields,
                         " ELSE 'N/A' END ",
                          "AS ", at_fields, " ")
    INTO  at_name_display
    FROM JSON_TABLE(at_name_list, "$[*]" COLUMNS(at_fields VARCHAR(64) path "$")) ism;
END IF;
RETURN at_name_display;

END//
DELIMITER ;



DROP FUNCTION IF EXISTS `tpf_tpfdf_file_id_display_name`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_tpfdf_file_id_display_name` (
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE display_stmt TEXT DEFAULT "";

SET  display_stmt = CONCAT(
         "  CASE WHEN file_id_ebcdic != '' AND tpf_tpfdf_file_name_alias_table.file_name_alias != 'N/A' THEN CONCAT(file_id,'-',file_id_ebcdic,'-',file_name_alias)  ",
         "  \n WHEN file_id_ebcdic != '' THEN CONCAT(file_id,'-',file_id_ebcdic,'-')  ",
         "  \n WHEN tpf_tpfdf_file_name_alias_table.file_name_alias != 'N/A' THEN CONCAT(file_id,'--',file_name_alias) ",
         " \n ELSE file_id  END ");

RETURN display_stmt;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_tpfdf_modified_where_clause`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_tpfdf_modified_where_clause` (
    `in_analysis_type` TEXT
)
RETURNS TEXT
DETERMINISTIC
BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
IF in_analysis_type = 'AllFilesOpened'
THEN
 SET select_stmt = "dbopn_per_sec > 0 OR `dbopn_per_sec-min_per_second`";
ELSEIF in_analysis_type = 'ModifyAPIsCalled'
THEN
  SET select_stmt = "
    dbadd_per_sec > 0 OR
    `dbadd_per_sec-min_per_second` > 0 OR
    dbckp_per_sec > 0 OR
    `dbckp_per_sec-min_per_second` > 0 OR
    dbcpy_per_sec > 0 OR
    `dbcpy_per_sec-min_per_second` > 0 OR
    dbcre_per_sec > 0 OR
    `dbcre_per_sec-min_per_second` > 0 OR
    dbdel_per_sec > 0 OR
    `dbdel_per_sec-min_per_second` > 0 OR
    dbdix_per_sec > 0 OR
    `dbdix_per_sec-min_per_second` > 0 OR
    dbidx_per_sec > 0 OR
    `dbidx_per_sec-min_per_second` > 0 OR
    dbmod_per_sec > 0 OR
    `dbmod_per_sec-min_per_second` > 0 OR
    dbmrg_per_sec > 0 OR
    `dbmrg_per_sec-min_per_second` > 0 OR
    dbrep_per_sec > 0 OR
    `dbrep_per_sec-min_per_second` > 0 OR
    dbrst_per_sec > 0 OR
    `dbrst_per_sec-min_per_second` > 0 OR
    dbsrt_per_sec > 0 OR
    `dbsrt_per_sec-min_per_second` > 0 OR
    dbtld_per_sec > 0 OR
    `dbtld_per_sec-min_per_second` > 0 OR
    dbtrd_per_sec > 0 OR
    `dbtrd_per_sec-min_per_second` > 0 OR
    filnc_per_sec > 0 OR
    `filnc_per_sec-min_per_second` > 0 OR
    filec_per_sec > 0 OR
    `filec_per_sec-min_per_second` > 0 OR
    cfile_per_sec > 0 OR
    `cfile_per_sec-min_per_second` > 0 OR
    pfile_per_sec > 0 OR
    `pfile_per_sec-min_per_second` > 0 OR
    getfc_per_sec > 0 OR
    `getfc_per_sec-min_per_second` > 0 OR
    relfc_per_sec > 0 OR
    `relfc_per_sec-min_per_second` > 0 OR
    btfil_per_sec > 0 OR
    `btfil_per_sec-min_per_second` > 0 OR
    lrfil_per_sec > 0 OR
    `lrfil_per_sec-min_per_second` > 0 OR
    rsf_create_per_sec > 0 OR
    `rsf_create_per_sec-min_per_second` > 0 OR
    rsf_update_per_sec > 0 OR
    `rsf_update_per_sec-min_per_second` > 0 OR
    rsf_delete_per_sec > 0 OR
    `rsf_delete_per_sec-min_per_second` > 0";
ELSEIF in_analysis_type = 'NoModifyAPIsCalled'
THEN
  SET select_stmt = "
CASE WHEN entry_is_pruned IS FALSE THEN dbadd_per_sec = 0 ELSE `dbadd_per_sec-max_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbadd_per_sec = 0 ELSE `dbadd_per_sec-max_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbckp_per_sec = 0 ELSE `dbckp_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbcpy_per_sec = 0 ELSE `dbcpy_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbcre_per_sec = 0 ELSE `dbcre_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbdel_per_sec = 0 ELSE `dbdel_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbdix_per_sec = 0 ELSE `dbdix_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbidx_per_sec = 0 ELSE `dbidx_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbmod_per_sec = 0 ELSE `dbmod_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbmrg_per_sec = 0 ELSE `dbmrg_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbrep_per_sec = 0 ELSE `dbrep_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbrst_per_sec = 0 ELSE `dbrst_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbsrt_per_sec = 0 ELSE `dbsrt_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbtld_per_sec = 0 ELSE `dbtld_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN dbtrd_per_sec = 0 ELSE `dbtrd_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN filnc_per_sec = 0 ELSE `filnc_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN filec_per_sec = 0 ELSE `filec_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN cfile_per_sec = 0 ELSE `cfile_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN pfile_per_sec = 0 ELSE `pfile_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN getfc_per_sec = 0 ELSE `getfc_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN relfc_per_sec = 0 ELSE `relfc_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN btfil_per_sec = 0 ELSE `btfil_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN lrfil_per_sec = 0 ELSE `lrfil_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN rsf_create_per_sec = 0 ELSE `rsf_create_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN rsf_update_per_sec = 0 ELSE `rsf_update_per_sec-min_per_second` = 0 END
AND CASE WHEN entry_is_pruned IS FALSE THEN rsf_delete_per_sec = 0 ELSE `rsf_delete_per_sec-min_per_second` = 0 END";
END IF;
RETURN select_stmt;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_keys_not_in_attributes`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_keys_not_in_attributes` (
    `in_panel_key` TEXT,
    `key_name_list` JSON,
    `at_name_list` JSON
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE result JSON;
DECLARE available_results JSON;
DECLARE key_attribute_json JSON;
DECLARE table_metrics JSON;
DECLARE key_name_list JSON;
DECLARE key_name TEXT DEFAULT "";
DECLARE found_key_attribute BOOLEAN DEFAULT TRUE;
DECLARE select_stmt LONGTEXT DEFAULT "";


-- If the keys are already in the list of attributes, then remove.
SET key_name_list = tpf_remove_elements_from_json_array(@key_name_list,at_name_list);
SET table_metrics = tpf_get_metrics_json();
SELECT JSON_ARRAYAGG(table_column)
INTO key_attribute_json
FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_panel_key  AND
      jt.table_column != "" AND
      JSON_CONTAINS(
              key_name_list,
              CONCAT('"',jt.table_column,'"')
);
IF key_attribute_json IS NOT NULL
    AND JSON_LENGTH(key_attribute_json) > 0
THEN
    SET key_name_list = tpf_remove_elements_from_json_array(key_attribute_json,key_name_list);
END IF;

RETURN key_name_list;
END//
DELIMITER ;

DROP FUNCTION IF EXISTS `contains_only_summarized_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `contains_only_summarized_metrics` (
    `in_metrics_list` JSON
)
RETURNS BOOLEAN
DETERMINISTIC
BEGIN

DECLARE requested_metrics_list TEXT DEFAULT "";

-- Returns true if the user requested only summarized metrics
SELECT GROUP_CONCAT(selected_metrics)
INTO  requested_metrics_list
FROM JSON_TABLE(in_metrics_list, "$[*]" COLUMNS(selected_metrics VARCHAR(64) path "$")) ism
WHERE selected_metrics NOT LIKE '%-max_per_second' AND selected_metrics NOT LIKE '%-min_per_second' AND selected_metrics NOT LIKE '%-per_minute';

IF requested_metrics_list IS NULL OR requested_metrics_list = ""
THEN
    RETURN TRUE;
END IF;
RETURN FALSE;

END//
DELIMITER ;


-- MySQL doesn't support EXECUTE IMMEDIATE syntax. Replicate with this procedure.
DROP PROCEDURE IF EXISTS `tpf_cdf_execute_immediate`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cdf_execute_immediate` (
    IN `in_query` TEXT
)

tpf_cdf_execute_immediate:BEGIN

    SET @cdf_dynamic_query = in_query;
    PREPARE cdf_dynamic_stmt FROM @cdf_dynamic_query;
    EXECUTE cdf_dynamic_stmt;
    DEALLOCATE PREPARE cdf_dynamic_stmt;

END//
DELIMITER ;
