/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Stored procedures common to all databases           */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procedures.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';


--
-- The following stored procedures are used to modify the database schema
-- to add the columns for the sample pruning event.
--


DROP PROCEDURE IF EXISTS `tpf_get_pruning_version`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_pruning_version` (
    IN `in_major` INT,
    IN `in_minor` INT,
    OUT `out_pruning_version` INT UNSIGNED
)
tpf_get_pruning_version:BEGIN

        SELECT pruning_version
        FROM version_history
        WHERE major=in_major AND minor=in_minor
        LIMIT 1
        INTO out_pruning_version;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_update_version_history_pruning_version`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_update_version_history_pruning_version` (
    IN `in_major` INT,
    IN `in_minor` INT,
    IN `in_pruning_version` INT
)
tpf_update_version_history_pruning_version:BEGIN

    DECLARE in_version VARCHAR(32);

    SET in_version = CONCAT_WS('.', in_major, in_minor);

    UPDATE `version_history`
    SET pruning_version=in_pruning_version
    WHERE version = in_version;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_sample_pruning`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_sample_pruning` ()
tpf_upgrade_zrtmc_schema_sample_pruning:BEGIN

    DECLARE current_database VARCHAR(64);
    DECLARE version_table_exists BOOLEAN;
    DECLARE pruning_version_column_exists BOOLEAN;
    DECLARE pruning_version INT UNSIGNED;

    -- safety check that the version_history table exists, it should always exist
    SELECT DATABASE() INTO current_database;
    SELECT EXISTS (
        SELECT 1
        FROM information_schema.TABLES
        WHERE TABLE_SCHEMA=current_database AND TABLE_NAME='version_history'
        LIMIT 1
    ) INTO version_table_exists;
    IF NOT version_table_exists
    THEN
       LEAVE tpf_upgrade_zrtmc_schema_sample_pruning;
    END IF;

    -- It may be that this is the first time and we have to add the pruning version column
    SELECT EXISTS (
        SELECT 1
        FROM information_schema.COLUMNS
        WHERE TABLE_SCHEMA = 'TPFDataSciDB' AND TABLE_NAME = 'version_history' AND COLUMN_NAME = 'pruning_version'
        LIMIT 1
    ) INTO pruning_version_column_exists;
    IF NOT pruning_version_column_exists
    THEN
       ALTER TABLE version_history
           ADD COLUMN `pruning_version` INT UNSIGNED DEFAULT NULL;
    END IF;


    -- This procedure is going to walk through each known version consecutively and create
    -- pruning columns for the changes made by that version.  This starts at version 1.4.
    -- We'll include a pruning version (instead of a boolean) just in case we have to go
    -- back and fix a pruning update after the fact.

    -- If the 1.4 P1 updates are not applied, then do so
    CALL tpf_get_pruning_version(1,4,pruning_version);
    IF pruning_version IS NULL
    THEN
        CALL tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P1();
    END IF;

    -- EXAMPLE
    -- If the 1.4 P2 updates are not applied, then do so
    -- CALL tpf_get_pruning_version(1,4,pruning_version);
    -- IF pruning_version="1"
    -- THEN
    --     CALL tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P2();
    -- END IF;

    CALL tpf_get_pruning_version(1,7,pruning_version);
    IF pruning_version IS NULL
    THEN
        CALL tpf_upgrade_zrtmc_schema_sample_pruning_1_7_P1();
    END IF;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P1`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P1` ()
tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P1:BEGIN

SELECT 'Upgrading to schema version 1.4.P1'; -- Info message


SELECT 'Adding min and max columns to metric tables.'; -- Info message

ALTER TABLE system_metrics_table
    ADD COLUMN `duration-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `actual_cpu-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `calculated_cpu-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `modeled_cpu-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `workload_cpu-max_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `lp_cpu-max_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `sys_pseudo_cpu-max_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `fenced_pseudo_cpu-max_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `inuse_istreams-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `istream_cap-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_ecb-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_iob-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `est_msg_cpu-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_exist-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_find_file-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `actual_cpu-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `calculated_cpu-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `modeled_cpu-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `workload_cpu-min_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `lp_cpu-min_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `sys_pseudo_cpu-min_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `fenced_pseudo_cpu-min_per_second` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `inuse_istreams-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `istream_cap-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_ecb-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_iob-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `est_msg_cpu-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_exist-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_find_file-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `actual_cpu-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `calculated_cpu-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `modeled_cpu-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `workload_cpu-per_minute` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `lp_cpu-per_minute` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `sys_pseudo_cpu-per_minute` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `fenced_pseudo_cpu-per_minute` SMALLINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `inuse_istreams-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `istream_cap-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_ecb-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `inuse_iob-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `est_msg_cpu-per_minute` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_exist-per_minute` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_find_file-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `est_msg_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE nvp_message_metrics_table
   ADD COLUMN `duration-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `avg_nvp_scale_factor-max_per_second` INT UNSIGNED DEFAULT NULL,
   ADD COLUMN `msg_cnt-max_per_second` INT UNSIGNED DEFAULT NULL,

   ADD COLUMN `cpu_used-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `dd_time-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb1mf-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb4kf-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `exist_time-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rlog-max_per_second` BIGINT UNSIGNED DEFAULT NULL,

   ADD COLUMN `cmbk-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cow-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cret-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_attac-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfil-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfnd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfile-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfind-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_no_rel-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_opn-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_rels-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_reuse-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbadd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbcls-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdel-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdix-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbidx-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbkey-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbmod-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbopn-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbred-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbrep-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_detac-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getcc-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getfc-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfil-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfnd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hnd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhnd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfile-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfind-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relcc-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relfc-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_create-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_delete-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_read-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_update-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_dasd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_vfa-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_dasd-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_vfa-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_file-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfsl-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfss-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `grfs-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `hctl-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `mal-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `relf-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rout-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `serr-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `snap-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `swbk-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `sysh-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `tour-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `twrt-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `xwbk-max_per_second` BIGINT UNSIGNED DEFAULT NULL,

   ADD COLUMN `est_msg_rate-max_per_second` BIGINT UNSIGNED DEFAULT NULL,

   ADD COLUMN `duration-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `avg_nvp_scale_factor-min_per_second` INT UNSIGNED DEFAULT NULL,
   ADD COLUMN `msg_cnt-min_per_second` INT UNSIGNED DEFAULT NULL,

   ADD COLUMN `cpu_used-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `dd_time-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb1mf-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb4kf-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `exist_time-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rlog-min_per_second` BIGINT UNSIGNED DEFAULT NULL,


   ADD COLUMN `cmbk-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cow-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cret-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_attac-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfil-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfnd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfile-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfind-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_no_rel-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_opn-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_rels-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_reuse-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbadd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbcls-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdel-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdix-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbidx-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbkey-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbmod-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbopn-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbred-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbrep-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_detac-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getcc-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getfc-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfil-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfnd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hnd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhnd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfile-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfind-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relcc-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relfc-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_create-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_delete-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_read-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_update-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_dasd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_vfa-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_dasd-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_vfa-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_file-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfsl-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfss-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `grfs-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `hctl-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `mal-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `relf-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rout-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `serr-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `snap-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `swbk-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `sysh-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `tour-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `twrt-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `xwbk-min_per_second` BIGINT UNSIGNED DEFAULT NULL,

   ADD COLUMN `est_msg_rate-min_per_second` BIGINT UNSIGNED DEFAULT NULL,

   ADD COLUMN `cpu_used-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `dd_time-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb1mf-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `ecb4kf-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `exist_time-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rlog-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cmbk-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cow-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `cret-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_attac-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfil-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_btfnd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfile-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cfind-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_no_rel-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_opn-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_rels-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_cls_reuse-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbadd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbcls-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdel-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbdix-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbidx-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbkey-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbmod-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbopn-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbred-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_dbrep-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_detac-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getcc-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_getfc-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfil-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_lrfnd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_hnd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_opn_nhnd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfile-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_pfind-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relcc-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_relfc-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_create-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_delete-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_read-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `df_rsf_update-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_dasd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `file_vfa-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_dasd-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_vfa-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `find_file-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfsl-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `gfss-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `grfs-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `hctl-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `mal-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `relf-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `rout-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `serr-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `snap-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `swbk-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `sysh-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `tour-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `twrt-per_minute` FLOAT UNSIGNED DEFAULT NULL,
   ADD COLUMN `xwbk-per_minute` FLOAT UNSIGNED DEFAULT NULL,

   ADD COLUMN `est_msg_rate-per_minute` BIGINT UNSIGNED DEFAULT NULL,
   ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


SELECT 'ALTER system_tcpip_table now tpf_tcpip_metrics_table'; -- Info message
ALTER TABLE tpf_tcpip_metrics_table
   ADD COLUMN `duration-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_cnt-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `tot_bytes-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_sent-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_recvd-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_sent-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_recvd-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `duration-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_cnt-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `tot_bytes-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_sent-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_recvd-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_sent-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_recvd-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `duration-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_cnt-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `tot_bytes-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_sent-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `bytes_recvd-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_sent-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `msg_recvd-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE jvm_jvm_metrics_table
   ADD COLUMN `num_jvms_expected-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvms_actual-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_total-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_inuse-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `max_jam_threads_observed-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `cpu-max_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `avg_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_daemon-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_latest-max_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `avg_gc_interval-max_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause-max_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_interval_global-max_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause_global-max_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `used_heap-max_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `heap_size-max_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_gc_events-max_per_second` INT UNSIGNED DEFAULT NULL,
   ADD COLUMN `num_jvms_expected-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvms_actual-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_total-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_inuse-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `max_jam_threads_observed-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `cpu-min_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `avg_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_daemon-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_latest-min_per_second` INT unsigned DEFAULT NULL,
   ADD COLUMN `avg_gc_interval-min_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause-min_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_interval_global-min_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause_global-min_per_second` DOUBLE DEFAULT NULL,
   ADD COLUMN `used_heap-min_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `heap_size-min_per_second` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_gc_events-min_per_second` INT UNSIGNED DEFAULT NULL,
   ADD COLUMN `num_jvms_expected-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvms_actual-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_total-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jam_threads_inuse-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `max_jam_threads_observed-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `cpu-per_minute` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `avg_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_daemon-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `num_jvm_threads_latest-per_minute` INT unsigned DEFAULT NULL,
   ADD COLUMN `avg_gc_interval-per_minute` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause-per_minute` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_interval_global-per_minute` DOUBLE DEFAULT NULL,
   ADD COLUMN `avg_gc_pause_global-per_minute` DOUBLE DEFAULT NULL,
   ADD COLUMN `used_heap-per_minute` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `heap_size-per_minute` DOUBLE unsigned DEFAULT NULL,
   ADD COLUMN `num_gc_events-per_minute` INT UNSIGNED DEFAULT NULL,
   ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE jvm_jam_jax_rs_metrics_table
    ADD COLUMN `op_count-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `min_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `op_count-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `min_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `op_count-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `min_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


ALTER TABLE jvm_jam_jax_rs_op_metrics_table
    ADD COLUMN `min_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-max_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `min_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-min_per_second` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `min_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `max_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `avg_response_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_checked-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `app_faults_unchecked-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_logical-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `runtime_faults_app-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `handling_time-per_minute` DOUBLE unsigned DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE jvm_lock_metrics_table
    ADD COLUMN `miss_pct-max_per_second` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `util_pct-max_per_second` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `gets-max_per_second` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `gets_slow-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `gets_rec-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `hold_time-max_per_second` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `miss_pct-min_per_second` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `util_pct-min_per_second` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `gets-min_per_second` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `gets_slow-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `gets_rec-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `hold_time-min_per_second` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `miss_pct-per_minute` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `util_pct-per_minute` DOUBLE(6,3) unsigned DEFAULT NULL,
    ADD COLUMN `gets-per_minute` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `gets_slow-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `gets_rec-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `hold_time-per_minute` BIGINT unsigned DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE jvm_jmx_app_Kafka_table
    ADD COLUMN `req_total-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `reqs_in_flight-max_per_second` INT DEFAULT NULL,
    ADD COLUMN `avg_batch_size-max_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `rec_send_rate-max_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_rec_size-max_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_req_latency-max_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `waiting_threads-max_per_second` INT DEFAULT NULL,
    ADD COLUMN `req_total-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `reqs_in_flight-min_per_second` INT DEFAULT NULL,
    ADD COLUMN `avg_batch_size-min_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `rec_send_rate-min_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_rec_size-min_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_req_latency-min_per_second` DOUBLE DEFAULT NULL,
    ADD COLUMN `waiting_threads-min_per_second` INT DEFAULT NULL,
    ADD COLUMN `req_total-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `reqs_in_flight-per_minute` INT DEFAULT NULL,
    ADD COLUMN `avg_batch_size-per_minute` DOUBLE DEFAULT NULL,
    ADD COLUMN `rec_send_rate-per_minute` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_rec_size-per_minute` DOUBLE DEFAULT NULL,
    ADD COLUMN `avg_req_latency-per_minute` DOUBLE DEFAULT NULL,
    ADD COLUMN `waiting_threads-per_minute` INT DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


ALTER TABLE jvm_jmx_app_JMBeansDrv_table
    ADD COLUMN `ecb-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `frm-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `frm1mb-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `req_counter-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `swb-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `sequence_num-max_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `ecb-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `frm-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `frm1mb-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `req_counter-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `swb-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `sequence_num-min_per_second` BIGINT DEFAULT NULL,
    ADD COLUMN `ecb-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `frm-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `frm1mb-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `req_counter-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `swb-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `sequence_num-per_minute` BIGINT DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


SELECT 'ALTER common_deploy_metrics_table now tpf_common_deployment_metrics_table'; -- Info message
ALTER TABLE tpf_common_deployment_metrics_table
    ADD COLUMN `duration-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `usage_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `count_per_second-max_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `usage_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `count_per_second-min_per_second` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `usage_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `count_per_second-per_minute` BIGINT UNSIGNED DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


SELECT 'ALTER service_app_metrics_table now tpf_service_app_metrics_table'; -- Info message
ALTER TABLE tpf_service_app_metrics_table
    ADD COLUMN `duration-max_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `internal_operation_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `internal_operation_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_count-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_rate-max_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-min_per_second` INT unsigned DEFAULT NULL,
    ADD COLUMN `internal_operation_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `internal_operation_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_count-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_rate-min_per_second` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `duration-per_minute` INT unsigned DEFAULT NULL,
    ADD COLUMN `internal_operation_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `internal_operation_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `external_operation_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `total_operation_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `timeout_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `iface_error_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `srvc_error_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_count-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `unrec_error_rate-per_minute` INT UNSIGNED DEFAULT NULL,
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

CALL tpf_update_version_history_pruning_version(1,4,1);

SELECT 'Done upgrading to schema version 1.4.P1'; -- Info message

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_upgrade_zrtmc_schema_sample_pruning_1_7_P1`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_upgrade_zrtmc_schema_sample_pruning_1_7_P1` ()
tpf_upgrade_zrtmc_schema_sample_pruning_1_7_P1:BEGIN

SELECT 'Upgrading to schema version 1.7.P1'; -- Info message

SELECT 'Adding min and max columns to metric tables.'; -- Info message

ALTER TABLE tpf_dasd_queue_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `dev_queue-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dev_queue-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dev_queue-per_minute` FLOAT DEFAULT NULL,
-- add entry is summarised flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_dasd_service_time_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `average_service_time-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `average_service_time-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `average_service_time-per_minute` FLOAT DEFAULT NULL,
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_tape_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `bytes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_system_lists_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `cross_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cross_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cross_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-per_minute` FLOAT DEFAULT NULL,
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

-- Add pruning columns for the _per_sec metrics
ALTER TABLE tpf_tcpip_metrics_table
   ADD COLUMN `msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_recvd_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_recvd_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `msgs_recvd_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_recvd_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_recvd_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `bytes_recvd_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_recvd_per_sec-max_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_recvd_per_sec-min_per_second` FLOAT DEFAULT NULL,
   ADD COLUMN `packets_recvd_per_sec-per_minute` FLOAT DEFAULT NULL;
-- Populate the _per_sec pruning columns from the existing counter pruning columns for all summarized rows
UPDATE tpf_tcpip_metrics_table SET
   `msgs_per_sec-max_per_second` = `msg_cnt-max_per_second`/`duration-max_per_second`,
   `msgs_per_sec-min_per_second` = `msg_cnt-min_per_second`/`duration-min_per_second`,
   `msgs_per_sec-per_minute` = `msg_cnt-per_minute`/`duration-per_minute`,
   `bytes_per_sec-max_per_second` = `tot_bytes-max_per_second`/`duration-max_per_second`,
   `bytes_per_sec-min_per_second` = `tot_bytes-min_per_second`/`duration-min_per_second`,
   `bytes_per_sec-per_minute` = `tot_bytes-per_minute`/`duration-per_minute`,
   `bytes_sent_per_sec-max_per_second` = `bytes_sent-max_per_second`/`duration-max_per_second`,
   `bytes_sent_per_sec-min_per_second` = `bytes_sent-min_per_second`/`duration-min_per_second`,
   `bytes_sent_per_sec-per_minute` = `bytes_sent-per_minute`/`duration-per_minute`,
   `bytes_recvd_per_sec-max_per_second` = `bytes_recvd-max_per_second`/`duration-max_per_second`,
   `bytes_recvd_per_sec-min_per_second` = `bytes_recvd-min_per_second`/`duration-min_per_second`,
   `bytes_recvd_per_sec-per_minute` = `bytes_recvd-per_minute`/`duration-per_minute`,
   `msgs_sent_per_sec-max_per_second` = `msg_sent-max_per_second`/`duration-max_per_second`,
   `msgs_sent_per_sec-min_per_second` = `msg_sent-min_per_second`/`duration-min_per_second`,
   `msgs_sent_per_sec-per_minute` = `msg_sent-per_minute`/`duration-per_minute`,
   `msgs_recvd_per_sec-max_per_second` = `msg_recvd-max_per_second`/`duration-max_per_second`,
   `msgs_recvd_per_sec-min_per_second` = `msg_recvd-min_per_second`/`duration-min_per_second`,
   `msgs_recvd_per_sec-per_minute` = `msg_recvd-per_minute`/`duration-per_minute` WHERE `entry_is_pruned` = 1;
-- Drop the pruning columns replaced by the new _per_sec pruning columns and other no longer needed.
ALTER TABLE `tpf_tcpip_metrics_table`
   DROP COLUMN `duration-max_per_second`,
   DROP COLUMN `msg_cnt-max_per_second`,
   DROP COLUMN `tot_bytes-max_per_second`,
   DROP COLUMN `bytes_sent-max_per_second`,
   DROP COLUMN `bytes_recvd-max_per_second`,
   DROP COLUMN `msg_sent-max_per_second`,
   DROP COLUMN `msg_recvd-max_per_second`,
   DROP COLUMN `duration-min_per_second`,
   DROP COLUMN `msg_cnt-min_per_second`,
   DROP COLUMN `tot_bytes-min_per_second`,
   DROP COLUMN `bytes_sent-min_per_second`,
   DROP COLUMN `bytes_recvd-min_per_second`,
   DROP COLUMN `msg_sent-min_per_second`,
   DROP COLUMN `msg_recvd-min_per_second`,
   DROP COLUMN `duration-per_minute`,
   DROP COLUMN `msg_cnt-per_minute`,
   DROP COLUMN `tot_bytes-per_minute`,
   DROP COLUMN `bytes_sent-per_minute`,
   DROP COLUMN `bytes_recvd-per_minute`,
   DROP COLUMN `msg_sent-per_minute`,
   DROP COLUMN `msg_recvd-per_minute`;


ALTER TABLE tpf_system_blocks_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `iob_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iob_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `swb_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `common_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `common_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `frame1mb_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `xwb_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_allocated-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_allocated-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_allocated-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rlog_inuse_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_ecbs-per_minute` FLOAT DEFAULT NULL,

-- add entry is summarized flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_vfa_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `data_reads_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_reads_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_reads_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `finds_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `finds_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `finds_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `candidate_files_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `candidate_files_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `candidate_files_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `noncandidate_files_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `noncandidate_files_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `noncandidate_files_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `force_files_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `force_files_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `force_files_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `file_immediate_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `file_immediate_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `file_immediate_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `buffer_unavailable_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `buffer_unavailable_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `buffer_unavailable_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `SLock_hits_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `SLock_hits_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `SLock_hits_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `XLock_hits_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `XLock_hits_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `XLock_hits_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_misses_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_misses_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_misses_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_content_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_content_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_content_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cached_rht_hts_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cached_rht_hts_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cached_rht_hts_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_age_outs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_age_outs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sync_age_outs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dispense_4K_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dispense_4K_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dispense_4K_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rht_content_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rht_content_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rht_content_per_sec-per_minute` FLOAT DEFAULT NULL,

-- add entry is summarized flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;


ALTER TABLE tpf_service_jam_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `internal_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `internal_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `internal_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use_highwater-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use_highwater-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_in_use_highwater-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_config-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_config-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `threads_config-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_subsystems_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `high_spd_msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_subsystem_users_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `high_spd_msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecb_exits_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_istreams_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `cross_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cross_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cross_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ready_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `input_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deferred_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `suspend_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `vct_list-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_ecbs-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_util-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_util-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_util-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_adj_util-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_adj_util-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `proc_adj_util-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `pseudo_util-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pseudo_util-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pseudo_util-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_util-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_util-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `low_priority_util-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_dasd_io_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `io_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `io_per_sec-min_per_second`  FLOAT DEFAULT NULL,
    ADD COLUMN `io_per_sec-per_minute`  FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-min_per_second`  FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-per_minute`  FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-min_per_second`  FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-per_minute`  FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_length-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_dasd_pool_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `set_size-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `set_size-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `set_size-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `reorder_time-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reorder_time-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reorder_time-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dispensed_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dispensed_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dispensed_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `returned_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `returned_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `returned_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_system_messages_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `high_spd_msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `high_spd_msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `tcpip_msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `tcpip_msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `tcpip_msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `routed_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `created_entries_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `commits_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `commits_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `commits_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `number_of_istreams-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `number_of_istreams-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `number_of_istreams-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `usable_istreams-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `usable_istreams-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `usable_istreams-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `processor_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `processor_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `processor_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_processor_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_processor_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_processor_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_max_cap_for_lpar_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_max_cap_for_lpar_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_max_cap_for_lpar_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_number_of_engines-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_number_of_engines-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `gp_number_of_engines-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `istream_cap-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `istream_cap-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `istream_cap-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_shutdown1_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `shutdown_occurred-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `shutdown_occurred-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `shutdown_occurred-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_remote_data_store_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `reader_active_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_active_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_active_ecbs-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_highwater-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_highwater-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_highwater-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_max-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_max-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reader_max-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_active_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_active_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_active_ecbs-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_highwater-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_highwater-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_highwater-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_max-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_max-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writer_max-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `creates_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `creates_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `creates_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `updates_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `updates_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `updates_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `deletes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deletes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `deletes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_ops_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_ops_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_ops_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `read_reject_at_max-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `read_reject_at_max-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `read_reject_at_max-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `TPF_error-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `TPF_error-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `TPF_error-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `server_error_mongo-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `server_error_mongo-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `server_error_mongo-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_mq_summary_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `queues-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queues-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queues-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `local_queues-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `local_queues-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `local_queues-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `transmit_queues-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `transmit_queues-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `transmit_queues-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `remote_queues-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `remote_queues-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `remote_queues-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_opens_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_opens_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_opens_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_closes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_closes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_closes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_sweeps_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_sweeps_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_sweeps_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_persistent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_persistent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_persistent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_persistent-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_persistent-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_persistent-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_nonpersistent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_nonpersistent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msgs_nonpersistent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_nonpersistent-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_nonpersistent-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_msg_length_nonpersistent-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channels-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channels-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channels-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `receiver_channels-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `receiver_channels-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `receiver_channels-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sender_channels-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sender_channels-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sender_channels-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `server_channels-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `server_channels-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `server_channels-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_recvd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_recvd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_recvd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_handled_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_handled_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_bytes_handled_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_batch_size-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_batch_size-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_batch_size-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_active_instances-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_active_instances-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_active_instances-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_last_batch_size-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_last_batch_size-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_last_batch_size-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_instances_highwater-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_instances_highwater-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `channel_instances_highwater-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_mq_queue_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `opens_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opens_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opens_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `closes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `closes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `closes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sweeps_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sweeps_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sweeps_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_persistent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_persistent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_persistent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_persistent-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_persistent-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_persistent-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_nonpersistent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_nonpersistent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_nonpersistent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_nonpersistent-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_nonpersistent-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msg_length_nonpersistent-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_mq_channel_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `msgs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_recvd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_recvd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_recvd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_handled_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_handled_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_handled_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `batch_size-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `batch_size-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `batch_size-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `active_instances-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_instances-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `active_instances-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `last_batch_size-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `last_batch_size-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `last_batch_size-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `instances_highwater-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `instances_highwater-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `instances_highwater-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_mpif_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `reads_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `reads_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `writes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bytes_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_sent_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_sent_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `msgs_sent_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_overruns_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_overruns_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `queue_overruns_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_lpar_util_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `shared_cpus-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `shared_cpus-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `shared_cpus-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `recognized_cpus-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `recognized_cpus-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `recognized_cpus-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `weight-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `weight-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `weight-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `util-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `util-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `util-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `util_vs_cap-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `util_vs_cap-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `util_vs_cap-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `machine-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `machine-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `machine-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `entitle-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `entitle-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `entitle-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_enh_chl_util_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `part_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `bus_util_pct-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bus_util_pct-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `bus_util_pct-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_read_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_read_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_read_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_read_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_read_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_read_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_write_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_write_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `part_mb_write_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_write_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_write_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_mb_write_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_dasd_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `module_queue-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `module_queue-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `module_queue-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `data_reads_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_reads_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_reads_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `data_writes_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_writes_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `data_writes_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `accumulated_response_time-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `accumulated_response_time-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `accumulated_response_time-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `response_time_call_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `response_time_call_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `response_time_call_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_locks_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_locks_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_locks_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_unlocks_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_unlocks_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `sioscs_unlocks_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `mplf_lock_contention_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `mplf_lock_contention_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `mplf_lock_contention_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `fdctc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `fdctc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `fdctc_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_time_slice_ecb_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `time_sliceable_ecbs-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `time_sliceable_ecbs-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `time_sliceable_ecbs-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `time_sliced_ecbs_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `time_sliced_ecbs_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `time_sliced_ecbs_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_trad_db_encryption_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `encryptions_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryptions_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryptions_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `encryptions_compressions_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryptions_compressions_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryptions_compressions_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `decryptions_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `decryptions_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `decryptions_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `decryption_expansions_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `decryption_expansions_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `decryption_expansions_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `encryption_errors_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryption_errors_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `encryption_errors_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `compression_errors_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `compression_errors_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `compression_errors_per_sec-per_minute` FLOAT DEFAULT NULL,

-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_traces_metrics_table
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_subsystems_utility_switches_metrics_table
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_static_metrics_table
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_lodic_resources_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `ecbs_using_class-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecbs_using_class-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecbs_using_class-per_minute` FLOAT DEFAULT NULL,

-- add entry is summarised flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

ALTER TABLE tpf_lodic_utilization_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `ecbs_using_class-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecbs_using_class-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `ecbs_using_class-per_minute` FLOAT DEFAULT NULL,

-- add entry is summarised flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

-- Add pruning columns for the _per_sec metrics
ALTER TABLE  tpf_common_deployment_metrics_table
    ADD COLUMN `usage_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `usage_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `usage_per_sec-per_minute` FLOAT DEFAULT NULL;
-- Populate the _per_sec pruning columns from the existing count_per_second pruning columns for all summarized rows
UPDATE tpf_common_deployment_metrics_table SET
    `usage_per_sec-max_per_second` = `count_per_second-max_per_second`,
    `usage_per_sec-min_per_second` = `count_per_second-min_per_second`,
    `usage_per_sec-per_minute` = `count_per_second-per_minute` WHERE `entry_is_pruned` = 1;
-- Drop the pruning columns replaced by the new _per_sec pruning columns and others no longer needed.
ALTER TABLE `tpf_common_deployment_metrics_table`
    DROP COLUMN `usage_count-max_per_second`,
    DROP COLUMN `usage_count-min_per_second`,
    DROP COLUMN `usage_count-per_minute`,
    DROP COLUMN `count_per_second-max_per_second`,
    DROP COLUMN `count_per_second-min_per_second`,
    DROP COLUMN `count_per_second-per_minute`,
    DROP COLUMN `duration-max_per_second`,
    DROP COLUMN `duration-min_per_second`,
    DROP COLUMN `duration-per_minute`;



-- Add pruning columns for the _per_sec metrics
ALTER TABLE  tpf_service_app_metrics_table
    ADD COLUMN `internal_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `internal_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `internal_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `external_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `total_operations_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `timeouts_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `iface_errors_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iface_errors_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `iface_errors_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `srvc_errors_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `srvc_errors_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `srvc_errors_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `unrec_errors_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `unrec_errors_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `unrec_errors_per_sec-per_minute` FLOAT DEFAULT NULL;
-- Populate the _per_sec pruning columns from the existing rate pruning columns for all summarized rows
UPDATE tpf_service_app_metrics_table SET
   `internal_operations_per_sec-max_per_second` = `internal_operation_rate-max_per_second`,
   `internal_operations_per_sec-min_per_second` = `internal_operation_rate-min_per_second`,
   `internal_operations_per_sec-per_minute` = `internal_operation_rate-per_minute`,
   `external_operations_per_sec-max_per_second` = `external_operation_rate-max_per_second`,
   `external_operations_per_sec-min_per_second` = `external_operation_rate-min_per_second`,
   `external_operations_per_sec-per_minute` = `external_operation_rate-per_minute`,
   `total_operations_per_sec-max_per_second` = `total_operation_rate-max_per_second`,
   `total_operations_per_sec-min_per_second` = `total_operation_rate-min_per_second`,
   `total_operations_per_sec-per_minute` = `total_operation_rate-per_minute`,
   `timeouts_per_sec-max_per_second` = `timeout_rate-max_per_second`,
   `timeouts_per_sec-min_per_second` = `timeout_rate-min_per_second`,
   `timeouts_per_sec-per_minute` = `timeout_rate-per_minute`,
   `iface_errors_per_sec-max_per_second` = `iface_error_rate-max_per_second`,
   `iface_errors_per_sec-min_per_second` = `iface_error_rate-min_per_second`,
   `iface_errors_per_sec-per_minute` = `iface_error_rate-per_minute`,
   `srvc_errors_per_sec-max_per_second` = `srvc_error_rate-max_per_second`,
   `srvc_errors_per_sec-min_per_second` = `srvc_error_rate-min_per_second`,
   `srvc_errors_per_sec-per_minute` = `srvc_error_rate-per_minute`,
   `unrec_errors_per_sec-max_per_second` = `unrec_error_rate-max_per_second`,
   `unrec_errors_per_sec-min_per_second` = `unrec_error_rate-min_per_second`,
   `unrec_errors_per_sec-per_minute` = `unrec_error_rate-per_minute` WHERE `entry_is_pruned` = 1;
-- Drop the pruning columns replaced by the new _per_sec pruning columns and others no longer needed.
ALTER TABLE `tpf_service_app_metrics_table`
    DROP COLUMN `duration-max_per_second`,
    DROP COLUMN `internal_operation_count-max_per_second`,
    DROP COLUMN `internal_operation_rate-max_per_second`,
    DROP COLUMN `external_operation_count-max_per_second`,
    DROP COLUMN `external_operation_rate-max_per_second`,
    DROP COLUMN `total_operation_count-max_per_second`,
    DROP COLUMN `total_operation_rate-max_per_second`,
    DROP COLUMN `timeout_count-max_per_second`,
    DROP COLUMN `timeout_rate-max_per_second`,
    DROP COLUMN `iface_error_count-max_per_second`,
    DROP COLUMN `iface_error_rate-max_per_second`,
    DROP COLUMN `srvc_error_count-max_per_second`,
    DROP COLUMN `srvc_error_rate-max_per_second`,
    DROP COLUMN `unrec_error_count-max_per_second`,
    DROP COLUMN `unrec_error_rate-max_per_second`,
    DROP COLUMN `duration-min_per_second`,
    DROP COLUMN `internal_operation_count-min_per_second`,
    DROP COLUMN `internal_operation_rate-min_per_second`,
    DROP COLUMN `external_operation_count-min_per_second`,
    DROP COLUMN `external_operation_rate-min_per_second`,
    DROP COLUMN `total_operation_count-min_per_second`,
    DROP COLUMN `total_operation_rate-min_per_second`,
    DROP COLUMN `timeout_count-min_per_second`,
    DROP COLUMN `timeout_rate-min_per_second`,
    DROP COLUMN `iface_error_count-min_per_second`,
    DROP COLUMN `iface_error_rate-min_per_second`,
    DROP COLUMN `srvc_error_count-min_per_second`,
    DROP COLUMN `srvc_error_rate-min_per_second`,
    DROP COLUMN `unrec_error_count-min_per_second`,
    DROP COLUMN `unrec_error_rate-min_per_second`,
    DROP COLUMN `duration-per_minute`,
    DROP COLUMN `internal_operation_count-per_minute`,
    DROP COLUMN `internal_operation_rate-per_minute`,
    DROP COLUMN `external_operation_count-per_minute`,
    DROP COLUMN `external_operation_rate-per_minute`,
    DROP COLUMN `total_operation_count-per_minute`,
    DROP COLUMN `total_operation_rate-per_minute`,
    DROP COLUMN `timeout_count-per_minute`,
    DROP COLUMN `timeout_rate-per_minute`,
    DROP COLUMN `iface_error_count-per_minute`,
    DROP COLUMN `iface_error_rate-per_minute`,
    DROP COLUMN `srvc_error_count-per_minute`,
    DROP COLUMN `srvc_error_rate-per_minute`,
    DROP COLUMN `unrec_error_count-per_minute`,
    DROP COLUMN `unrec_error_rate-per_minute`;

    -- this table is used to log errors that occur during pruning
    CREATE TABLE IF NOT EXISTS `tpf_sample_pruning_error_table` (
        `time` datetime(3) NOT NULL,
        `function_name` TEXT DEFAULT NULL,
        `table_name` TEXT DEFAULT NULL,
        `error_text` TEXT DEFAULT NULL,
        `parameters` TEXT DEFAULT NULL
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;
ALTER TABLE tpf_tpfdf_metrics_table
-- add max, min, per minute fields for the metrics field only
    ADD COLUMN `dbadd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbadd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbadd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbadr_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbadr_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbadr_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbckp_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbckp_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbckp_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcls_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcls_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcls_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcpy_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcpy_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcpy_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcre_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcre_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbcre_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdel_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdel_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdel_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdix_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdix_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdix_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdsp_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdsp_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbdsp_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbidx_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbidx_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbidx_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbkey_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbkey_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbkey_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmod_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmod_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmod_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmrg_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmrg_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbmrg_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbopn_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbopn_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbopn_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbred_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbred_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbred_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrep_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrep_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrep_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbret_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbret_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbret_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrst_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrst_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbrst_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbspa_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbspa_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbspa_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbsrt_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbsrt_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbsrt_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtld_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtld_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtld_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtlg_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtlg_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtlg_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtrd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtrd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbtrd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuky_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuky_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuky_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbhold_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbhold_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbhold_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuhld_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuhld_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `dbuhld_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `filnc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `filnc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `filnc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `filec_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `filec_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `filec_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cfile_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cfile_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cfile_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `pfile_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pfile_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pfile_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `twrtc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `twrtc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `twrtc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `getfc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `getfc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `getfc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `relfc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `relfc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `relfc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `detac_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `detac_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `detac_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `attac_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `attac_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `attac_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `fsysw_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `fsysw_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `fsysw_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `getcc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `getcc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `getcc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `relcc_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `relcc_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `relcc_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cfind_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cfind_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cfind_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `pfind_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pfind_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `pfind_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `btfil_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `btfil_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `btfil_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `btfnd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `btfnd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `btfnd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfil_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfil_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfil_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfnd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfnd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `lrfnd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hnd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hnd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_hnd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhnd_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhnd_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `opn_nhnd_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_rel_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_rel_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_rel_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_norel_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_norel_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_norel_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_reuse_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_reuse_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_reuse_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_opn_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_opn_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cls_opn_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `prefp_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `prefp_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `prefp_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `prefpw_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `prefpw_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `prefpw_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_iocnt_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_iocnt_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_iocnt_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_ioreq_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_ioreq_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_prtl_ioreq_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_hit_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_hit_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_hit_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_miss_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_miss_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `cache_read_miss_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `blk_read_depth-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `blk_read_depth-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `blk_read_depth-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_create_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_create_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_create_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_read_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_read_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_read_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_update_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_update_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_update_per_sec-per_minute` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_delete_per_sec-max_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_delete_per_sec-min_per_second` FLOAT DEFAULT NULL,
    ADD COLUMN `rsf_delete_per_sec-per_minute` FLOAT DEFAULT NULL,
-- is the row summarization flag
    ADD COLUMN `entry_is_pruned` BOOLEAN DEFAULT FALSE;

CALL tpf_update_version_history_pruning_version(1,7,2);

SELECT 'Done upgrading to schema version 1.7.P1'; -- Info message

END//
DELIMITER ;


--
-- apply the pruning changes to our schema NOW!
--
CALL tpf_upgrade_zrtmc_schema_sample_pruning();


--
-- Drop our pruning schema change stored procedures
--
DROP PROCEDURE IF EXISTS `tpf_get_pruning_version`;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_sample_pruning;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_sample_pruning_1_4_P1;
DROP PROCEDURE IF EXISTS tpf_upgrade_zrtmc_schema_sample_pruning_1_7_P1;


DROP FUNCTION IF EXISTS `tpf_get_pruning_variations_json`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_pruning_variations_json` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE pruning_variations JSON;

SET pruning_variations = '
  [
	  {"display_string":"per_minute",
	   "metric_suffix":"-per_minute"
	   },
	  {"display_string":"max_per_second",
	   "metric_suffix":"-max_per_second"
	   },
	  {"display_string":"min_per_second",
	   "metric_suffix":"-min_per_second"
	   }
  ]';

RETURN pruning_variations;

END//
DELIMITER ;