/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20231004    PJ47156 - prepare config script does not work in all */
/*                        customer environments                      */
/*  20231015    PJ47106 - Display multiple Kafka producers           */
/*  20231117    PJ47197 - Internal UDM pruning deadlocks             */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Stored procedures common to all databases           */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP PROCEDURE IF EXISTS `tpf_delete_data_time_series_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_delete_data_time_series_table` (
    IN `in_table_name` VARCHAR(128),
    IN `in_time` DATETIME
)

tpf_delete_data_time_series_table:BEGIN

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE we_have_data DATETIME;
DECLARE rows_deleted INT;

-- Do we have any data to delete?
SET @we_have_data = NULL;
SET select_stmt =
            CONCAT(
            " SELECT time INTO @we_have_data",
            " FROM  ", in_table_name,
            " WHERE TIME <= '", in_time, "'",
            " LIMIT 1"
);
CALL tpf_execute_immediate(select_stmt);
IF @we_have_data IS NULL
THEN
    LEAVE tpf_delete_data_time_series_table;
END IF;

-- We want to minimize the impact of deleting data on collection activities
-- so we're going to do the delete of data piecemeal and slowly
deleteLoop: LOOP
   SET rows_deleted = 0;

   -- delete a chunk of data
   SET select_stmt = CONCAT(
      " DELETE FROM ", in_table_name,
      " WHERE TIME <= '", in_time, "'",
      " LIMIT 1000"
   );
   CALL tpf_execute_immediate(select_stmt);
   -- The ideal solution is to use row_count() to see how many rows were deleted as this would:
   --    Prevent infinite loop if errors occur deleting a particular row
   --    Exit the loop when there's no more data left in our time range
   -- However, with mysql 8.0.32 with many test variations (even simple hard coded
   -- variations without prepared statements) row_count() always returned 0.
   -- Maybe this will be fixed eventually and we can revert to this solution.
   -- Testing if a row exists leaves the potential open that if a row failed to delete,
   -- we will end up in an infinite loop.  However, selecting count(*)
   -- rows is too costly for INNODB and likely impacts DB performance for some scenarios.
   -- SELECT ROW_COUNT() INTO rows_deleted;
   -- IF rows_deleted < 5000
   -- THEN
   --    LEAVE deleteLoop;
   -- END IF;

   -- We'll sleep to minimize DB impact to ongoing collection
   DO SLEEP(.01);

   -- Do we have any data to delete?
   SET @we_have_data = NULL;
   SET select_stmt =
               CONCAT(
               " SELECT time INTO @we_have_data",
               " FROM  ", in_table_name,
               " WHERE TIME <= '", in_time, "'",
               " LIMIT 1"
   );
   CALL tpf_execute_immediate(select_stmt);
   IF @we_have_data IS NULL
   THEN
       LEAVE deleteLoop;
   END IF;
END LOOP deleteLoop;

END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_sample_prune_data`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_sample_prune_data` (
    IN `in_per_second_data_days` INT,
    IN `in_delete_data_after_days` INT
)

tpf_sample_prune_data:BEGIN
-- Prunes old time series data. Data will be keep as-is (per second data) for in_per_second_data_days. Then, data
-- will be condensed to per-minute averages and maxes for data that is between in_per_second_data_days and
-- in_delete_data_after_days old. Then, data older than in_delete_data_after_days will be deleted.
--
-- The number of days will start counting from 11:59:59 PM of the previous day (from when this procedure runs).
-- For example, say this procedure is run at 12:01:00 AM on June 28, 2022 with in_per_second_data_days = 15 and
-- in_delete_data_after_days = 30. There will be per-second data from June 13, 2022 12:00:00 AM to current time
-- (June 13 12:00:00 AM to June 27 11:59:59 PM is 15 days). There will be per-minute data from May 29, 2022 12:00:00 AM
-- to June 12 11:59:59 PM. All data May 28, 2022 and earlier will be deleted.
--
-- This procedure considers all dates in UTC time.

DECLARE todays_date DATETIME;
DECLARE per_minute_start_date DATETIME;
DECLARE per_minute_end_date DATETIME;
DECLARE metrics_array JSON;
DECLARE virtual_metrics_array JSON;

-- DECLARE EXIT HANDLER FOR SQLEXCEPTION  BEGIN    ROLLBACK;    END;

--  Do everything in UTC time.
SET time_zone = '+00:00';

SET todays_date = DATE(UTC_TIMESTAMP());
SET per_minute_start_date = DATE_SUB(todays_date, INTERVAL in_delete_data_after_days DAY);
-- BETWEEN is inclusive on both ends, so we subtract 1 second from the day boundary
SET per_minute_end_date = DATE_SUB(todays_date, INTERVAL CONCAT(in_per_second_data_days, ' 00:00:01') DAY_SECOND);

SET metrics_array = tpf_get_metrics_available_func("nvp",'["primary","storage","tpfdf","other"]');
SET virtual_metrics_array  = tpf_get_virtual_metrics("nvp",'["primary","storage","tpfdf","other"]');
SET metrics_array = tpf_remove_elements_from_json_array( metrics_array, virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'nvp_message_metrics_table',
    '["system_id", "value_id"]',
    metrics_array,
    virtual_metrics_array,
    per_minute_start_date,
    per_minute_end_date
);

-- Build list of metrics to average, min, max
-- But we'll remove virtual metrics as these only need min, max
SET metrics_array = '["duration", "avg_nvp_scale_factor", "msg_cnt"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_metrics_available_func("nvp",'["primary", "storage","tpfdf","other"]'));
SET virtual_metrics_array = tpf_get_virtual_metrics("nvp",'["primary", "storage","tpfdf","other"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_sample_prune_sec_time_series_table (
    'nvp_message_metrics_table',
    '["system_id", "value_id"]',
    metrics_array,
    virtual_metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

-- Re-calculates table statistics so the optimizer can make better decisions
ANALYZE TABLE nvp_message_metrics_table;


SET metrics_array = '["duration"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_metrics_available_func("system",'["primary", "nvp_estimated"]'));
SET virtual_metrics_array = tpf_get_virtual_metrics("system",'["primary", "nvp_estimated"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'system_metrics_table',
    '["system_id"]',
    metrics_array,
    virtual_metrics_array,
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'system_metrics_table',
    '["system_id"]',
    metrics_array,
    virtual_metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE system_metrics_table;


SET metrics_array = tpf_get_metrics_available_func("tcpip",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tcpip",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_tcpip_metrics_table',
    '["system_id", "app_name_id"]',
    metrics_array,
    virtual_metrics_array,
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_tcpip_metrics_table',
    '["system_id", "app_name_id"]',
    metrics_array,
    virtual_metrics_array,
    '["port","weight"]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_tcpip_metrics_table;

--  The jvm static data needs to be pruned based on the
-- last_time in the jvm_static_data_table.
DELETE FROM  jvm_static_data_table
WHERE jvm_static_data_table.last_time <=  per_minute_start_date;

-- JVM has no virtual fields at this time
SET metrics_array = '["num_gc_events"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAM_Sum",'["primary"]'));
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JVM_DD",'["primary"]'));

CALL tpf_validate_entries_are_pruned (
    'jvm_jvm_metrics_table',
    '["jam_id", "pid"]',
    metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'jvm_jvm_metrics_table',
    '["jam_id", "pid"]',
    metrics_array,
    '[]',
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_jvm_metrics_table;




SET metrics_array = '["op_count"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAX-RS_DD",'["primary"]'));

CALL tpf_validate_entries_are_pruned (
    'jvm_jam_jax_rs_metrics_table',
    '["jam_id", "pid"]',
    metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table(
    'jvm_jam_jax_rs_metrics_table',
    '["jam_id", "pid"]',
    metrics_array,
    '[]',
    '["app"]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_jam_jax_rs_metrics_table;



SET metrics_array = '[]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAX-RS-Op_DD",'["primary"]'));

CALL tpf_validate_entries_are_pruned (
    'jvm_jam_jax_rs_op_metrics_table',
    '["jam_id", "pid", "op_id"]',
    metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table(
    'jvm_jam_jax_rs_op_metrics_table',
    '["jam_id", "pid", "op_id"]',
    metrics_array,
    '[]',
    '["app"]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_jam_jax_rs_op_metrics_table;


SET metrics_array = '[]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("Lock_DD",'["primary"]'));
CALL tpf_validate_entries_are_pruned (
    'jvm_lock_metrics_table',
    '["jam_id", "pid", "lock"]',
    metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table(
    'jvm_lock_metrics_table',
    '["jam_id", "pid", "lock"]',
    metrics_array,
    '[]',
    '["description"]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_lock_metrics_table;

CALL tpf_validate_entries_are_pruned (
    'jvm_jmx_app_Kafka_table',
    '["jam_id", "pid", "producer_id"]',
    '["req_total", "reqs_in_flight", "avg_batch_size", "rec_send_rate", "avg_rec_size", "avg_req_latency", "waiting_threads"]',
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table(
    'jvm_jmx_app_Kafka_table',
    '["jam_id", "pid", "producer_id"]',
    '["req_total", "reqs_in_flight", "avg_batch_size", "rec_send_rate", "avg_rec_size", "avg_req_latency", "waiting_threads"]',
    '[]',
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_jmx_app_Kafka_table;

CALL tpf_validate_entries_are_pruned (
    'jvm_jmx_app_JMBeansDrv_table',
    '["jam_id", "pid"]',
    '["ecb", "frm", "frm1mb", "req_counter", "swb", "sequence_num"]',
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table(
    'jvm_jmx_app_JMBeansDrv_table',
    '["jam_id", "pid"]',
    '["ecb", "frm", "frm1mb", "req_counter", "swb", "sequence_num"]',
    '[]',
    '["buff"]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE jvm_jmx_app_JMBeansDrv_table;

SET metrics_array = tpf_get_metrics_available_func("common_deployment",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("common_deployment",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

 CALL tpf_validate_entries_are_pruned (
    'tpf_common_deployment_metrics_table',
    '["system_id", "subsystem_id", "key_id", "type_id", "version_id"]',
     metrics_array,
    virtual_metrics_array,
    per_minute_start_date,
    per_minute_end_date
 );

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_common_deployment_metrics_table',
   '["system_id", "subsystem_id", "key_id", "type_id", "version_id"]',
    metrics_array,
    virtual_metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_common_deployment_metrics_table;



 SET metrics_array = tpf_get_metrics_available_func("rest_services",'["All"]');
 SET virtual_metrics_array = tpf_get_virtual_metrics("rest_services",'["All"]');
 SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

 CALL tpf_validate_entries_are_pruned (
    'tpf_service_app_metrics_table',
    '["system_id", "subsystem_id", "operation_id"]',
    metrics_array,
    virtual_metrics_array,
    per_minute_start_date,
    per_minute_end_date
 );

CALL tpf_sample_prune_sec_time_series_table(
    'tpf_service_app_metrics_table',
    '["system_id", "subsystem_id", "operation_id"]',
    metrics_array,
    virtual_metrics_array,
    '[]',
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_service_app_metrics_table;

-- prune tpf_dasd_queue_metrics_table
SET metrics_array = tpf_get_metrics_available_func("dasd_queue",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_queue",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_dasd_queue_metrics_table',                     -- table name
    '["system_id", "subsystem_id", "dev_address"]',     -- primary key columns
    metrics_array,                                      -- metrics columns
    virtual_metrics_array,                              -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_dasd_queue_metrics_table',                     -- table name
    '["system_id", "subsystem_id", "dev_address"]',     -- primary key columns
    metrics_array,                                      -- metrics columns
    virtual_metrics_array,                              -- virtual metrics columns
    '["dev_mod"]',                                      -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_dasd_queue_metrics_table;

-- prune tpf_dasd_service_time_metrics_table
SET metrics_array = tpf_get_metrics_available_func("dasd_service_time",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_service_time",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_dasd_service_time_metrics_table',    -- table name
    '["system_id", "ssid"]',                  -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,                    -- per minute start date
    per_minute_end_date                       -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_dasd_service_time_metrics_table',    -- table name
    '["system_id", "ssid"]',                  -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '["lowest_online_sda_on_lss"]',           -- extra columns
    per_minute_start_date,                    -- per minute start date
    per_minute_end_date                       -- per minute end date
);

ANALYZE TABLE tpf_dasd_service_time_metrics_table;

-- prune tpf_system_blocks_metrics_table
SET metrics_array = tpf_get_metrics_available_func("system_blocks",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_blocks",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_system_blocks_metrics_table',        -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_system_blocks_metrics_table',        -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '[]',                                     -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_system_blocks_metrics_table;



-- prune tpf_tape_metrics_table
SET metrics_array = tpf_get_metrics_available_func("tape",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tape",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_tape_metrics_table',                     -- table name
    '["system_id", "subsystem_id", "tape_name"]', -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_tape_metrics_table',                     -- table name
    '["system_id", "subsystem_id", "tape_name"]', -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '["device_address"]',                         -- extra columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

ANALYZE TABLE tpf_tape_metrics_table;

-- prune tpf_system_lists_metrics_table
SET metrics_array = tpf_get_metrics_available_func("system_lists",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_lists",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_system_lists_metrics_table',         -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,                    -- per minute start date
    per_minute_end_date                       -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_system_lists_metrics_table',         -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '[]',                                     -- extra columns
    per_minute_start_date,                    -- per minute start date
    per_minute_end_date                       -- per minute end date
);

ANALYZE TABLE tpf_system_lists_metrics_table;

-- prune tpf_vfa_metrics_table
SET metrics_array = tpf_get_metrics_available_func("vfa",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("vfa",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_vfa_metrics_table',                  -- table name
    '["system_id", "subsystem_id"]',          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_vfa_metrics_table',                  -- table name
    '["system_id", "subsystem_id"]',          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '[]',                                     -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_vfa_metrics_table;

-- prune tpf_service_jam_metrics_table
SET metrics_array = tpf_get_metrics_available_func("service_jam",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("service_jam",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_service_jam_metrics_table',              -- table name
    '["system_id", "subsystem_id", "jam_name"]',  -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_service_jam_metrics_table',              -- table name
    '["system_id", "subsystem_id", "jam_name"]',  -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '[]',                                         -- extra columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

ANALYZE TABLE tpf_service_jam_metrics_table;

-- prune tpf_subsystems_metrics_table
SET metrics_array = tpf_get_metrics_available_func("subsystems",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("subsystems",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_subsystems_metrics_table',               -- table name
    '["system_id", "subsystem_id"]',              -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_subsystems_metrics_table',               -- table name
    '["system_id", "subsystem_id"]',              -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '[]',                                         -- extra columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

ANALYZE TABLE tpf_subsystems_metrics_table;

-- prune tpf_subsystem_users_metrics_table
SET metrics_array = tpf_get_metrics_available_func("subsystem_users",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("subsystem_users",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_subsystem_users_metrics_table',          -- table name
    '["system_id", "subsystem_id", "subsystem_user_name"]', -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_subsystem_users_metrics_table',          -- table name
    '["system_id", "subsystem_id", "subsystem_user_name"]',  -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '[]',                                         -- extra columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

ANALYZE TABLE tpf_subsystem_users_metrics_table;

-- prune tpf_istreams_metrics_table
SET metrics_array = tpf_get_metrics_available_func("istream",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("istream",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_istreams_metrics_table',                 -- table name
    '["system_id", "istream"]',                   -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_istreams_metrics_table',                 -- table name
    '["system_id", "istream"]',                   -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '["status"]',                                 -- extra columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end date
);

ANALYZE TABLE tpf_istreams_metrics_table;

-- prune tpf_dasd_io_metrics_table
SET metrics_array = tpf_get_metrics_available_func("dasd_io",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_io",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_dasd_io_metrics_table',                    -- table name
    '["system_id", "subsystem_id", "logical_device_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_dasd_io_metrics_table',                    -- table name
    '["system_id", "subsystem_id", "logical_device_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_dasd_io_metrics_table;

-- prune tpf_dasd_pool_metrics_table
SET metrics_array = tpf_get_metrics_available_func("dasd_pools",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_pools",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_dasd_pool_metrics_table',                                -- table name
    '["system_id", "subsystem_id", "logical_device_type", "pool_type"]', -- primary key columns
    metrics_array,                                                -- metrics columns
    virtual_metrics_array,                                        -- virtual metrics columns
    per_minute_start_date,                                        -- per minute start date
    per_minute_end_date                                           -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_dasd_pool_metrics_table',                                -- table name
    '["system_id", "subsystem_id", "logical_device_type", "pool_type"]', -- primary key columns
    metrics_array,                                                -- metrics columns
    virtual_metrics_array,                                        -- virtual metrics columns
    '[]',                                                         -- extra columns
    per_minute_start_date,                                        -- per minute start date
    per_minute_end_date                                           -- per minute end date
);

ANALYZE TABLE tpf_dasd_pool_metrics_table;


-- prune tpf_system_messages_metrics_table
SET metrics_array = tpf_get_metrics_available_func("system_messages",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_messages",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_system_messages_metrics_table',      -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_system_messages_metrics_table',      -- table name
    '["system_id"]',                          -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '["speed_boost", "istream_boost"]',       -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_system_messages_metrics_table;


-- prune tpf_shutdown1_metrics_table
SET metrics_array = tpf_get_metrics_available_func("shutdown_events",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("shutdown_events",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_shutdown1_metrics_table',            -- table name
    '["system_id", "shutdown_type_id", "reason_id"]', -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_shutdown1_metrics_table',            -- table name
    '["system_id", "shutdown_type_id", "reason_id"]', -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '[]',                                     -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_shutdown1_metrics_table;


-- prune tpf_remote_data_store_metrics_table
SET metrics_array = tpf_get_metrics_available_func("remote_data_store",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("remote_data_store",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_remote_data_store_metrics_table',    -- table name
    '["system_id", "subsystem_id", "rds_name_id"]', -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_remote_data_store_metrics_table',    -- table name
    '["system_id", "subsystem_id", "rds_name_id"]', -- primary key columns
    metrics_array,                            -- metrics columns
    virtual_metrics_array,                    -- virtual metrics columns
    '[]',                                     -- extra columns
    per_minute_start_date,
    per_minute_end_date
);

ANALYZE TABLE tpf_remote_data_store_metrics_table;

-- prune tpf_mq_summary_metrics_table
SET metrics_array = tpf_get_metrics_available_func("mq_summary",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_summary",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_mq_summary_metrics_table',                 -- table name
    '["system_id", "subsystem_id", "queue_manager_id"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_mq_summary_metrics_table',                 -- table name
    '["system_id", "subsystem_id", "queue_manager_id"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_mq_summary_metrics_table;

-- prune tpf_mq_queue_metrics_table
SET metrics_array = tpf_get_metrics_available_func("mq_queues",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_queues",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_mq_queue_metrics_table',                   -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "queue_name_id","queue_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_mq_queue_metrics_table',                   -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "queue_name_id","queue_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '["queue_type"]',                               -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_mq_queue_metrics_table;

-- prune tpf_mq_channel_metrics_table
SET metrics_array = tpf_get_metrics_available_func("mq_channels",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_channels",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_mq_channel_metrics_table',                 -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "channel_name_id","channel_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_mq_channel_metrics_table',                 -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "channel_name_id","channel_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_mq_channel_metrics_table;

-- prune tpf_mpif_metrics_table
SET metrics_array = tpf_get_metrics_available_func("mpif",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mpif",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_mpif_metrics_table',                       -- table name
    '["system_id", "path_name_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_mpif_metrics_table',                       -- table name
    '["system_id", "path_name_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_mpif_metrics_table;

-- prune tpf_lpar_util_metrics_table
SET metrics_array = tpf_get_metrics_available_func("lpar_utilization",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("lpar_utilization",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_lpar_util_metrics_table',                  -- table name
    '["system_id", "part_name_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_lpar_util_metrics_table',                  -- table name
    '["system_id", "part_name_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_lpar_util_metrics_table;

-- prune tpf_enh_chl_util_metrics_table
SET metrics_array = tpf_get_metrics_available_func("channel_utilization",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("channel_utilization",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_enh_chl_util_metrics_table',               -- table name
    '["system_id", "channel_id"]',                  -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_enh_chl_util_metrics_table',               -- table name
    '["system_id", "channel_id"]',                  -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_enh_chl_util_metrics_table;

-- tpf_dasd_metrics_table
SET metrics_array = tpf_get_metrics_available_func("dasd_metrics",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_metrics",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_dasd_metrics_table',                       -- table name
    '["system_id", "subsystem_id", "volume_number", "address"]',  -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_dasd_metrics_table',                       -- table name
    '["system_id", "subsystem_id", "volume_number", "address"]',   -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_dasd_metrics_table;

-- prune tpf_time_slice_ecb_metrics_table
SET metrics_array = tpf_get_metrics_available_func("time_slice_ecbs",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("time_slice_ecbs",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_time_slice_ecb_metrics_table',             -- table name
    '["system_id"]',                                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_time_slice_ecb_metrics_table',             -- table name
    '["system_id"]',                                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_time_slice_ecb_metrics_table;

-- prune tpf_trad_db_encryption_metrics_table
SET metrics_array = tpf_get_metrics_available_func("trad_db_encryption",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("trad_db_encryption",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_trad_db_encryption_metrics_table',         -- table name
    '["system_id", "subsystem_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_trad_db_encryption_metrics_table',         -- table name
    '["system_id", "subsystem_id"]',                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_trad_db_encryption_metrics_table;


-- prune tpf_lodic_resources_metrics_table
SET metrics_array = tpf_get_metrics_available_func("LODIC_resource_classes",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("LODIC_resource_classes",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_lodic_resources_metrics_table',            -- table name
    '["system_id", "class_name_id"]',               -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_lodic_resources_metrics_table',            -- table name
    '["system_id", "class_name_id"]',               -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_lodic_resources_metrics_table;


-- prune tpf_lodic_utilization_metrics_table
SET metrics_array = tpf_get_metrics_available_func("LODIC_utilization_classes",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("LODIC_utilization_classes",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_lodic_utilization_metrics_table',          -- table name
    '["system_id", "class_name_id"]',               -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_lodic_utilization_metrics_table',          -- table name
    '["system_id", "class_name_id"]',               -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_lodic_utilization_metrics_table;


-- prune tpf_traces_metrics_table
SET metrics_array = tpf_get_metrics_available_func("traces",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("traces",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

-- NOTE: No summarization for tpf_traces_metrics_table because the table has no metrics.

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_traces_metrics_table',                     -- table name
    '["system_id"]',                                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '["altered","block_check_mode","branch","branch_relative_target_check_mode","ECB_heap","ECB_macro","enter_back","extended_C_function","function_trace","heap_check_mode","IO","IBM_DBUGC_log","IBM_DBUGC_test","idle_timer","IO_CCW","lock_trace","reg_this_IPL","socket","system_log","user_DBUGC_log","user_DBUGC_test","VFA"]', -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_traces_metrics_table;


-- prune tpf_subsystems_utility_switches_metrics_table
SET metrics_array = tpf_get_metrics_available_func("subsystem_utility_switches",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("subsystem_utility_switches",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

-- NOTE: No summarization for tpf_subsystems_utility_switches_metrics_table because the table has no metrics.

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_subsystems_utility_switches_metrics_table', -- table name
    '["system_id","subsystem_id"]',                 -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '["altered","recoup","selective_file_trace","RCS_file","RCS_enable","RCS_implement","data_col","capture","restore","SON_copy","trap","DB_reorg","CCP","link_trace"]', -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_subsystems_utility_switches_metrics_table;


-- prune tpf_static_metrics_table
SET metrics_array = tpf_get_metrics_available_func("machine",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("machine",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

-- NOTE: No summarization for tpf_static_metrics_table because the table metrics won't change unless there is an IPL.

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_static_metrics_table',                     -- table name
    '["system_id"]',                                -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '["altered","cpu_serial","cpu_machine_type","tpf_release","tpf_prod_update_level","tpf_complex_name","tpf_enterprise_name","cpu_Id"]', -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_static_metrics_table;

-- prune tpf_tpfdf_metrics_table
SET metrics_array = tpf_get_metrics_available_func("tpfdf",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tpfdf",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_validate_entries_are_pruned (
    'tpf_tpfdf_metrics_table',                      -- table name
    '["system_id", "subsystem_id", "file_id"]',     -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

CALL tpf_sample_prune_sec_time_series_table (
    'tpf_tpfdf_metrics_table',                      -- table name
    '["system_id", "subsystem_id", "file_id"]',     -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end date
);

ANALYZE TABLE tpf_tpfdf_metrics_table;



END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_sample_prune_sec_time_series_table`;
DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_sample_prune_sec_time_series_table` (
    IN `in_table_name` VARCHAR(128),
    IN `in_primary_key_columns` TEXT,
    IN `in_metric_columns` TEXT,
    IN `in_virtual_metric_columns` TEXT,
    IN `in_extra_columns` TEXT,
    IN `in_per_minute_start_date` DATETIME,
    IN `in_per_minute_end_date` DATETIME
)

tpf_sample_prune_sec_time_series_table:BEGIN
-- Prunes old time series data.
-- Data will be keep as-is (per second data) for in_per_second_data_days.
-- For data that is between in_per_second_data_days and in_delete_data_after_days old,
-- (a) delete rows that do not have values for max/min/per-minute.
-- (b) update the remaining rows to set the values of the per-second columns to null.
-- Then, data older than in_delete_data_after_days will be deleted.

-- The number of days will start counting from 11:59:59 PM of the previous day (from when this procedure runs).
-- For example, say this procedure is run at 12:01:00 AM on June 28, 2022 with in_per_second_data_days = 15 and
-- in_delete_data_after_days = 30. There will be per-second data from June 13, 2022 12:00:00 AM to current time
-- (June 13 12:00:00 AM to June 27 11:59:59 PM is 15 days). There will be per-minute data from May 29, 2022 12:00:00 AM
-- to June 12 11:59:59 PM. All data May 28, 2022 and earlier will be deleted.
--
-- This procedure considers all dates in UTC time.

 DECLARE select_stmt LONGTEXT DEFAULT "";
 DECLARE all_cols LONGTEXT;
 DECLARE we_have_data DATETIME;
 DECLARE rows_deleted INT;
 DECLARE error_text TEXT;
 DECLARE parameters TEXT;

 -- this allows for errors to occur processing a single table, report the error and allow pruning processing to continue
 DECLARE EXIT HANDLER FOR SQLEXCEPTION
 BEGIN
    -- get the reason the error occurred
    GET DIAGNOSTICS CONDITION 1 @error_text = MESSAGE_TEXT;
    -- save error into the database so we can reference it later
    SET parameters=CONCAT(":", in_primary_key_columns,":",in_metric_columns,":",in_virtual_metric_columns,":",in_extra_columns,":",in_per_minute_start_date,":",in_per_minute_start_date,":");
    INSERT INTO tpf_sample_pruning_error_table (time,function_name,table_name,error_text,parameters)
         VALUES (NOW(),"tpf_sample_prune_sec_time_series_table",in_table_name,@error_text,parameters);
    ROLLBACK;
 END;

 -- Data older than in_delete_data_after_days will be deleted.
 CALL tpf_delete_data_time_series_table(in_table_name,in_per_minute_start_date);

 -- Exit the procedure if there are no rows between start and end dates.
 SET @we_have_data = NULL;
 SET select_stmt =
             CONCAT(
             " SELECT TIME INTO @we_have_data",
             " FROM  ", in_table_name,
             " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
             " LIMIT 1"
 );
 CALL tpf_execute_immediate(select_stmt);
 IF @we_have_data IS NULL
 THEN
     LEAVE tpf_sample_prune_sec_time_series_table;
 END IF ;

 -- get the list of columns
 IF in_metric_columns IS NOT NULL AND JSON_LENGTH(in_metric_columns) > 0
 THEN
    SELECT GROUP_CONCAT(REPLACE("`replace_col` = NULL","replace_col", metric_col))
    INTO all_cols
    FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

 -- Set to null the per second values
 -- check the flag in the row that indicates if this row has been pruned.
 -- performance wise, we'll do this update in 1 chunk and then sleep.
    SET select_stmt =
        CONCAT(
        " UPDATE IGNORE ", in_table_name,
        " SET ", all_cols,
        " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
        " AND ", in_table_name, ".entry_is_pruned IS TRUE"
        );
    CALL tpf_execute_immediate(select_stmt);
 END IF;

 -- We'll sleep to minimize DB impact to ongoing collection
 DO SLEEP(.1);

  -- Delete all per second rows
  -- We want to minimize the impact of deleting data on collection activities
  -- so we're going to do the delete of data piecemeal and slowly
  deleteLoop: LOOP
     SET rows_deleted = 0;

     -- delete a chunk of data
     CALL tpf_execute_immediate(CONCAT(
         " DELETE FROM ", in_table_name,
         " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
         " AND ", in_table_name, ".entry_is_pruned IS FALSE",
         " LIMIT 5000"
     ));
     -- The ideal solution is to use row_count() to see how many rows were deleted as this would:
     --    Prevent infinite loop if errors occur deleting a particular row
     --    Exit the loop when there's no more data left in our time range
     -- However, with mysql 8.0.32 with many test variations (even simple hard coded
     -- variations without prepared statements) row_count() always returned 0.
     -- Maybe this will be fixed eventually and we can revert to this solution.
     -- Testing if a row exists leaves the potential open that if a row failed to delete,
     -- we will end up in an infinite loop.  However, selecting count(*)
     -- rows is too costly for INNODB and likely impacts DB performance for some scenarios.
     -- SELECT ROW_COUNT() INTO rows_deleted;
     -- IF rows_deleted < 5000
     -- THEN
     --    LEAVE deleteLoop;
     -- END IF;

     -- We'll sleep to minimize DB impact
     DO SLEEP(.1);

      -- Exit the procedure if there are no rows between start and end dates.
      SET @we_have_data = NULL;
      SET select_stmt =
                  CONCAT(
                  " SELECT TIME INTO @we_have_data",
                  " FROM  ", in_table_name,
                  " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
                  " AND ", in_table_name, ".entry_is_pruned IS FALSE",
                  " LIMIT 1"
      );
      CALL tpf_execute_immediate(select_stmt);
      IF @we_have_data IS NULL
      THEN
          LEAVE deleteLoop;
      END IF ;
  END LOOP deleteLoop;


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_validate_entries_are_pruned`;
DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_validate_entries_are_pruned` (
    IN `in_table_name` VARCHAR(128),
    IN `in_primary_key_columns` TEXT,
    IN `in_metric_columns` TEXT,
    IN `in_virtual_metric_columns` TEXT,
    IN `in_per_minute_start_date` DATETIME,
    IN `in_per_minute_end_date` DATETIME
)

tpf_validate_entries_are_pruned:BEGIN

 DECLARE select_stmt LONGTEXT DEFAULT "";
 DECLARE cur_pruned_date DATETIME;
 DECLARE last_pruned_date DATETIME;
 DECLARE pruned_time_list LONGTEXT DEFAULT "";
 DECLARE pruned_time_string LONGTEXT;
 DECLARE separatorLoc INTEGER;
 DECLARE cur_index INTEGER;
 DECLARE test_field BOOLEAN;
 DECLARE error_text TEXT;
 DECLARE parameters TEXT;

 -- this allows for errors to occur processing a single table, report the error and allow pruning processing to continue
 DECLARE EXIT HANDLER FOR SQLEXCEPTION
 BEGIN
    -- get the reason the error occurred
    GET DIAGNOSTICS CONDITION 1 @error_text = MESSAGE_TEXT;
    -- save error into the database so we can reference it later
    SET parameters=CONCAT(":", in_primary_key_columns,":",in_metric_columns,":",in_virtual_metric_columns,":",in_per_minute_start_date,":",in_per_minute_start_date,":");
    INSERT INTO tpf_sample_pruning_error_table (time,function_name,table_name,error_text,parameters)
         VALUES (NOW(),"tpf_validate_entries_are_pruned",in_table_name,@error_text,parameters);
    ROLLBACK;
 END;

-- test for empty table
-- If there are no entries in the table at all,
-- we do not want to process.
 SET select_stmt =
             CONCAT(
             " SELECT EXISTS ( SELECT 1",
             " FROM  ", in_table_name,
             " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
             " LIMIT 1)",
             " INTO @test_field "
 );
 CALL tpf_execute_immediate(select_stmt);
 IF NOT @test_field
 THEN
     LEAVE tpf_validate_entries_are_pruned;
 END IF ;

-- In order to not check past the last minute, we need to move the
-- last back by one minute.
    SET cur_pruned_date = SUBTIME(in_per_minute_start_date, EXTRACT(SECOND FROM in_per_minute_start_date));
    SET cur_pruned_date = DATE_SUB(cur_pruned_date, INTERVAL -1 MINUTE);
    SET  last_pruned_date = DATE_SUB(in_per_minute_end_date, INTERVAL 1 MINUTE);

  SET  select_stmt = CONCAT(
-- Save the list of times into a JSON array. MySQL does not support JSON_ARRAYAGG DISTINCT - https://bugs.mysql.com/bug.php?id=91993
--              " SELECT JSON_ARRAYAGG(DISTINCT time)"
                " SELECT CONCAT('[', GROUP_CONCAT(DISTINCT JSON_QUOTE(CAST(time AS CHAR))), ']')",
                " INTO @pruned_time_list",
                " FROM  ", in_table_name,
                " WHERE TIME BETWEEN '", in_per_minute_start_date, "' AND '", in_per_minute_end_date, "'",
                " AND ", in_table_name, ".entry_is_pruned IS TRUE",
                " ORDER BY ", in_table_name, ".time ASC"
    );
    CALL tpf_execute_immediate(select_stmt);
    SET pruned_time_list = @pruned_time_list;
    -- Initialize variables needed for the loop to test for missing data points

    SET cur_index = 2;
    SET pruned_time_string = NULL;
        -- Get the first entry
    SET separatorLoc = LOCATE(',',pruned_time_list);
    IF separatorLoc IS NOT NULL AND  separatorLoc != 0 THEN
        SET pruned_time_list = SUBSTRING(pruned_time_list, 2);
        SET pruned_time_string = SUBSTRING(pruned_time_list,  cur_index,separatorLoc-4) ;
        SET pruned_time_list = SUBSTRING(pruned_time_list, (separatorLoc + 1));
    END IF;
    SET cur_index = 1;
    -- Start the loop to test for missing data points
     processVarNames: LOOP
        IF   pruned_time_string IS NOT NULL
        AND  cur_pruned_date =  pruned_time_string
        THEN
            SET separatorLoc = LOCATE(',',pruned_time_list);
            IF separatorLoc IS NULL OR separatorLoc = 0  THEN
                SET separatorLoc = LOCATE(']',pruned_time_list);
                IF separatorLoc IS NULL OR separatorLoc = 0  THEN
                    SET pruned_time_string = NULL;
                ELSE
                    SET pruned_time_string = SUBSTRING(pruned_time_list, cur_index, separatorLoc-4);
                END IF;
            ELSE
                SET pruned_time_string = SUBSTRING(pruned_time_list, cur_index, separatorLoc-4) ;
                SET pruned_time_list = SUBSTRING(pruned_time_list, (separatorLoc + 2));
            END IF;
        ELSE
            CALL tpf_get_per_minute_info(in_table_name,in_primary_key_columns,
                            in_metric_columns,in_virtual_metric_columns, '[]','[]',
                            cur_pruned_date, DATE_SUB(cur_pruned_date, INTERVAL -1 MINUTE));
        END IF;
        SET cur_pruned_date = DATE_SUB(cur_pruned_date, INTERVAL -1 MINUTE);
        IF   cur_pruned_date > last_pruned_date THEN
           LEAVE processVarNames;
         END IF;
    END LOOP processVarNames;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_get_avg_info_per_minute`;
DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_avg_info_per_minute` ()
tpf_get_avg_info_per_minute:BEGIN

DECLARE metrics_array JSON;
DECLARE virtual_metrics_array JSON;
DECLARE per_minute_start_date DATETIME;
DECLARE per_minute_end_date DATETIME;
DECLARE todays_date DATETIME;
DECLARE select_stmt LONGTEXT DEFAULT "";

-- DECLARE EXIT HANDLER FOR SQLEXCEPTION  BEGIN    ROLLBACK;    END;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

-- Call the procedure for the previous minute - 2 minutes ago.
CALL tpf_get_avg_info_start_time("nvp_message_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = tpf_get_metrics_available_func("nvp",'["primary","storage","tpfdf","other"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("nvp",'["primary","storage","tpfdf","other"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);
CALL tpf_get_per_minute_info('nvp_message_metrics_table',
                         '["system_id", "value_id"]',
                         metrics_array,
                         virtual_metrics_array,
                         '[]',
                         '[]',                                    -- averaged columns
                         per_minute_start_date, per_minute_end_date);

CALL tpf_get_avg_info_start_time("system_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = '["duration"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_metrics_available_func("system",'["primary", "nvp_estimated"]'));
SET virtual_metrics_array = tpf_get_virtual_metrics("system",'["primary", "nvp_estimated"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);
CALL tpf_get_per_minute_info('system_metrics_table',
                        '["system_id"]',
                        metrics_array,
                        virtual_metrics_array,
                        '[]',
                        '[]',                                    -- averaged columns
                        per_minute_start_date,  per_minute_end_date
);

CALL tpf_get_avg_info_start_time("tpf_service_app_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = tpf_get_metrics_available_func("rest_services",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("rest_services",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);
CALL tpf_get_per_minute_info('tpf_service_app_metrics_table',
                        '["system_id", "subsystem_id", "operation_id"]',
                        metrics_array,
                        virtual_metrics_array,
                        '[]',
                        '[]',                                    -- averaged columns
                        per_minute_start_date,  per_minute_end_date
 );

CALL tpf_get_avg_info_start_time("tpf_common_deployment_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = tpf_get_metrics_available_func("common_deployment",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("common_deployment",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);
CALL tpf_get_per_minute_info('tpf_common_deployment_metrics_table',
                         '["system_id", "subsystem_id", "key_id", "type_id", "version_id"]',
                        metrics_array,
                        virtual_metrics_array,
                        '[]',
                        '[]',                                    -- averaged columns
                        per_minute_start_date,  per_minute_end_date
 );

CALL tpf_get_avg_info_start_time("tpf_tcpip_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = tpf_get_metrics_available_func("tcpip",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tcpip",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_per_minute_info (
                         'tpf_tcpip_metrics_table',
                         '["system_id", "app_name_id"]',
                         metrics_array,
                         virtual_metrics_array,
                        '["port","weight"]',
                        '[]',                                    -- averaged columns
                         per_minute_start_date, per_minute_end_date);


-- JVM
CALL tpf_get_avg_info_start_time("jvm_jvm_metrics_table",per_minute_start_date,per_minute_end_date);
-- JVM has no virtual fields at this time
SET metrics_array = '["num_gc_events"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAM_Sum",'["primary"]'));
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JVM_DD",'["primary"]'));

CALL tpf_get_per_minute_info('jvm_jvm_metrics_table',
                        '["jam_id", "pid"]',
                        metrics_array,
                        '[]',
                        '[]',
                        '[]',                                    -- averaged columns
                        per_minute_start_date,  per_minute_end_date
);

CALL tpf_get_avg_info_start_time("jvm_lock_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = '[]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("Lock_DD",'["primary"]'));
CALL tpf_get_per_minute_info('jvm_lock_metrics_table',
                        '["jam_id", "pid", "lock"]',
                        metrics_array,
                        '[]',
                        '["description"]',
                        '[]',                                    -- averaged columns
                        per_minute_start_date,  per_minute_end_date
);

CALL tpf_get_avg_info_start_time("jvm_jmx_app_JMBeansDrv_table",per_minute_start_date,per_minute_end_date);
CALL tpf_get_per_minute_info(
    'jvm_jmx_app_JMBeansDrv_table',
    '["jam_id", "pid"]',
    '["ecb", "frm", "frm1mb", "req_counter", "swb", "sequence_num"]',
    '[]',
    '["buff"]',
    '[]',                                    -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_get_avg_info_start_time("jvm_jam_jax_rs_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = '["op_count"]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAX-RS_DD",'["primary"]'));
CALL tpf_get_per_minute_info(
    'jvm_jam_jax_rs_metrics_table',
    '["jam_id", "pid"]',
    metrics_array,
    '[]',
    '["app"]',
    '[]',                                    -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_get_avg_info_start_time("jvm_jam_jax_rs_op_metrics_table",per_minute_start_date,per_minute_end_date);
SET metrics_array = '[]';
SET metrics_array = JSON_MERGE(metrics_array,tpf_get_jvm_metrics_available_func("JAX-RS-Op_DD",'["primary"]'));
CALL tpf_get_per_minute_info(
    'jvm_jam_jax_rs_op_metrics_table',
    '["jam_id", "pid", "op_id"]',
    metrics_array,
    '[]',
    '["app"]',
    '[]',                                    -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

CALL tpf_get_avg_info_start_time("jvm_jmx_app_Kafka_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'jvm_jmx_app_Kafka_table',
    '["jam_id", "pid", "producer_id"]',
    '["req_total", "reqs_in_flight", "avg_batch_size", "rec_send_rate", "avg_rec_size", "avg_req_latency", "waiting_threads"]',
    '[]',
    '[]',
    '[]',                                    -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

-- dasd_queue
SET metrics_array = tpf_get_metrics_available_func("dasd_queue",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_queue",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_dasd_queue_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_dasd_queue_metrics_table',                    -- table anme
    '["system_id", "subsystem_id", "dev_address"]',    -- primary key columns
    metrics_array,                                     -- metrics columns
    virtual_metrics_array,                             -- virtual metrics columns
    '["dev_mod"]',                                     -- extra columns
    '[]',                                              -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

-- dasd_service_time
SET metrics_array = tpf_get_metrics_available_func("dasd_service_time",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_service_time",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_dasd_service_time_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_dasd_service_time_metrics_table',   -- table name
    '["system_id", "ssid"]',                 -- primary key columns
    metrics_array,                           -- metrics columns
    virtual_metrics_array,                   -- virtual metrics columns
    '["lowest_online_sda_on_lss"]',          -- extra columns
    '[]',                                    -- averaged columns
    per_minute_start_date,                   -- per minute start date
    per_minute_end_date                      -- per minute end data
);

-- system_blocks
SET metrics_array = tpf_get_metrics_available_func("system_blocks",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_blocks",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_system_blocks_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_system_blocks_metrics_table',     -- table anme
    '["system_id"]',                       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

-- tape
SET metrics_array = tpf_get_metrics_available_func("tape",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tape",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_tape_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_tape_metrics_table',                     -- table name
    '["system_id", "subsystem_id", "tape_name"]', -- primary key columns
    metrics_array,                                -- metrics columns
    virtual_metrics_array,                        -- virtual metrics columns
    '["device_address"]',                         -- extra columns
    '[]',                                         -- averaged columns
    per_minute_start_date,                        -- per minute start date
    per_minute_end_date                           -- per minute end data
);

-- system_lists
SET metrics_array = tpf_get_metrics_available_func("system_lists",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_lists",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_system_lists_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_system_lists_metrics_table',      -- table name
    '["system_id"]',                       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- vfa
SET metrics_array = tpf_get_metrics_available_func("vfa",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("vfa",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_vfa_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_vfa_metrics_table',               -- table anme
    '["system_id", "subsystem_id"]',       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,
    per_minute_end_date
);

-- service_jam
SET metrics_array = tpf_get_metrics_available_func("service_jam",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("service_jam",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_service_jam_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_service_jam_metrics_table',       -- table name
    '["system_id", "subsystem_id", "jam_name"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '["threads_in_use_highwater"]',        -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end date
);

-- subsystems
SET metrics_array = tpf_get_metrics_available_func("subsystems",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("subsystems",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_subsystems_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_subsystems_metrics_table',        -- table name
    '["system_id", "subsystem_id"]',       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- subsystem users
SET metrics_array = tpf_get_metrics_available_func("subsystem_users",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("subsystem_users",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_subsystem_users_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_subsystem_users_metrics_table',   -- table name
    '["system_id", "subsystem_id", "subsystem_user_name"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- istreams
SET metrics_array = tpf_get_metrics_available_func("istream",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("istream",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_istreams_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_istreams_metrics_table',          -- table name
    '["system_id", "istream"]',            -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["status"]',                          -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- dasd_io
SET metrics_array = tpf_get_metrics_available_func("dasd_io",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_io",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_dasd_io_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_dasd_io_metrics_table',                    -- table name
    '["system_id", "subsystem_id", "logical_device_type"]', -- primary key columns
    metrics_array,                                  -- metrics columns
    virtual_metrics_array,                          -- virtual metrics columns
    '[]',                                           -- extra columns
    '[]',                                           -- averaged columns
    per_minute_start_date,                          -- per minute start date
    per_minute_end_date                             -- per minute end data
);

-- dasd_pool
SET metrics_array = tpf_get_metrics_available_func("dasd_pools",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_pools",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_dasd_pool_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_dasd_pool_metrics_table',                                -- table name
    '["system_id", "subsystem_id", "logical_device_type", "pool_type"]', -- primary key columns
    metrics_array,                                                -- metrics columns
    virtual_metrics_array,                                        -- virtual metrics columns
    '[]',                                                         -- extra columns
    '[]',                                                         -- averaged columns
    per_minute_start_date,                                        -- per minute start date
    per_minute_end_date                                           -- per minute end data
);

-- tpf_system_messages_metrics_table
SET metrics_array = tpf_get_metrics_available_func("system_messages",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("system_messages",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_system_messages_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_system_messages_metrics_table',   -- table name
    '["system_id"]',                       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["speed_boost", "istream_boost"]',    -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- tpf_shutdown1_metrics_table
SET metrics_array = tpf_get_metrics_available_func("shutdown_events",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("shutdown_events",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_shutdown1_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_shutdown1_metrics_table',         -- table name
    '["system_id", "shutdown_type_id", "reason_id"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- tpf_remote_data_store_metrics_table
SET metrics_array = tpf_get_metrics_available_func("remote_data_store",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("remote_data_store",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

-- For remote data store data type, max_reader, max_writer,  high_water_max should be averaged columns
CALL tpf_get_avg_info_start_time("tpf_remote_data_store_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_remote_data_store_metrics_table', -- table name
    '["system_id", "subsystem_id", "rds_name_id"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["rds_mode_id"]',                     -- passing in attribute column as extra columns
    '["reader_highwater", "writer_highwater", "reader_max", "writer_max"]', -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end date
);

-- mq_summary
SET metrics_array = tpf_get_metrics_available_func("mq_summary",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_summary",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_mq_summary_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_mq_summary_metrics_table',        -- table name
    '["system_id", "subsystem_id", "queue_manager_id"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '["channel_instances_highwater"]',     -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end date
);


-- mq_queue
SET metrics_array = tpf_get_metrics_available_func("mq_queues",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_queues",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_mq_queue_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_mq_queue_metrics_table',          -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "queue_name_id","queue_type"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- mq_channel
SET metrics_array = tpf_get_metrics_available_func("mq_channels",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mq_channels",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_mq_channel_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_mq_channel_metrics_table',        -- table name
    '["system_id", "subsystem_id", "queue_manager_id", "channel_name_id","channel_type"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '["instances_highwater"]',             -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- mqif
SET metrics_array = tpf_get_metrics_available_func("mpif",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("mpif",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_mpif_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_mpif_metrics_table',              -- table name
    '["system_id", "path_name_id"]',       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["device_name_id", "class_name"]',    -- passing in attribute columns as extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- lpar_util
SET metrics_array = tpf_get_metrics_available_func("lpar_utilization",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("lpar_utilization",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_lpar_util_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_lpar_util_metrics_table',         -- table name
    '["system_id", "part_name_id"]',       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["part_number", "type"]',             -- passing in attribute columns as extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- enh_chl_util
SET metrics_array = tpf_get_metrics_available_func("channel_utilization",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("channel_utilization",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_enh_chl_util_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_enh_chl_util_metrics_table',      -- table name
    '["system_id", "channel_id"]',         -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["active", "type_id", "shared", "channel_group"]',  -- passing in attribute columns as extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- dasd metrics
SET metrics_array = tpf_get_metrics_available_func("dasd_metrics",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("dasd_metrics",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_dasd_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_dasd_metrics_table',              -- table name
    '["system_id", "subsystem_id", "volume_number", "address"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["dev_indicator", "symbolic_module", "type", "dasd_subsystem_id"]',  -- passing in attribute columns as extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- time_slice_ecb_metrics_table
SET metrics_array = tpf_get_metrics_available_func("time_slice_ecbs",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("time_slice_ecbs",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_time_slice_ecb_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_time_slice_ecb_metrics_table',    -- table name
    '["system_id"]',                       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- tpf_trad_db_encryption_metrics_table
SET metrics_array = tpf_get_metrics_available_func("trad_db_encryption",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("trad_db_encryption",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_trad_db_encryption_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_trad_db_encryption_metrics_table',-- table name
    '["system_id", "subsystem_id"]',       -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- tpf_lodic_resources_metrics_table
SET metrics_array = tpf_get_metrics_available_func("LODIC_resource_classes",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("LODIC_resource_classes",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_lodic_resources_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_lodic_resources_metrics_table',   -- table name
    '["system_id", "class_name_id"]',      -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);

-- tpf_lodic_utilization_metrics_table
SET metrics_array = tpf_get_metrics_available_func("LODIC_utilization_classes",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("LODIC_utilization_classes",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

CALL tpf_get_avg_info_start_time("tpf_lodic_utilization_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_lodic_utilization_metrics_table', -- table name
    '["system_id", "class_name_id"]',      -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '[]',                                  -- extra columns
    '[]',                                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);
-- tpf_tpfdf_metrics_table
SET metrics_array = tpf_get_metrics_available_func("tpfdf",'["All"]');
SET virtual_metrics_array = tpf_get_virtual_metrics("tpfdf",'["All"]');
SET metrics_array = tpf_remove_elements_from_json_array(metrics_array,virtual_metrics_array);

-- NOTE: Adding file_id_ebcdic and file_name_alias_id to the extra columns so that we can copy over
-- the information for the attributes.
CALL tpf_get_avg_info_start_time("tpf_tpfdf_metrics_table",per_minute_start_date,per_minute_end_date);
CALL  tpf_get_per_minute_info(
    'tpf_tpfdf_metrics_table',            -- table name
    '["system_id", "subsystem_id", "file_id"]', -- primary key columns
    metrics_array,                         -- metrics columns
    virtual_metrics_array,                 -- virtual metrics columns
    '["file_id_ebcdic","file_name_alias_id"]', -- passing in attribute columns as extra columns
    '["blk_read_depth"]',                  -- averaged columns
    per_minute_start_date,                 -- per minute start date
    per_minute_end_date                    -- per minute end data
);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_get_per_minute_info`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_per_minute_info` (
    IN `in_table_name` VARCHAR(128),
    IN `in_primary_key_columns` TEXT,
    IN `in_metric_columns` TEXT,
    IN `in_virtual_metric_columns` TEXT,
    IN `in_extra_columns` TEXT,
    IN `avg_metrics_list` JSON,
    IN `in_per_minute_start_date` DATETIME,
    IN `in_per_minute_end_date` DATETIME
)
tpf_get_per_minute_info:BEGIN
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE temp_table_select_cols LONGTEXT;
DECLARE temp_table_select_cols_virtual_metrics LONGTEXT;
DECLARE temp_table_select_cols_2 LONGTEXT;
DECLARE pk_cols TEXT;
DECLARE extra_cols LONGTEXT;
DECLARE extra_update_columns LONGTEXT;
DECLARE all_cols LONGTEXT;
DECLARE all_virtual_cols LONGTEXT;
DECLARE temp_table_update_cols_list LONGTEXT;
DECLARE temp_table_update_cols LONGTEXT;
DECLARE temp_table_update_max LONGTEXT;
DECLARE temp_table_update_min LONGTEXT;
DECLARE virtual_temp_table_update_cols LONGTEXT;
DECLARE temp_table_insert_cols LONGTEXT;
DECLARE temp_table_insert_min  LONGTEXT;
DECLARE temp_table_insert_max  LONGTEXT;
DECLARE where_pk_cols TEXT;
DECLARE error_text TEXT;
DECLARE parameters TEXT;
DECLARE frequency_select_stmt LONGTEXT DEFAULT "";


-- this allows for errors to occur processing a single table, report the error and allow pruning processing to continue
DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
   -- get the reason the error occurred
   GET DIAGNOSTICS CONDITION 1 @error_text = MESSAGE_TEXT;
   -- save error into the database so we can reference it later
   SET parameters=CONCAT(":", in_primary_key_columns,":",in_metric_columns,":",in_virtual_metric_columns,":",in_extra_columns,":",in_per_minute_start_date,":",in_per_minute_start_date,":");
   INSERT INTO tpf_sample_pruning_error_table (time,function_name,table_name,error_text,parameters)
        VALUES (NOW(),"tpf_get_per_minute_info",in_table_name,@error_text,parameters);
   ROLLBACK;
END;


    --  Do everything in UTC time.
SET time_zone = '+00:00';

--  some of our concats for columns are very large
SET SESSION group_concat_max_len = 200000;

IF in_per_minute_start_date IS NULL OR
   in_per_minute_end_date IS NULL
THEN
   LEAVE tpf_get_per_minute_info;
END IF;

--
-- For the per_minute value, in order to account for missing data points,
-- use duration and rate to calculate the counts.
-- Take the SUM of the counts,
-- divide by 60 to get average rate in 1 minute window.
-- When the frequency specified on TPF is less than 60,
-- SUM(duration/1000 * per second rate)/60
--
-- When that frequency is greater than 60
-- then per_minute = per_second value
--
-- Like we're ignoring frequency changes. We're ignoring the possibility
-- of data points missing due to network or other issues.
-- In particular, this calculation is required for very low frequency scenarios like
-- a REST call with a 1 sec collection frequency that is collected 5 times
-- in a minute so we essentially have 55 seconds in that minute of 0s to average in.
--
-- For example, if the frequency is 30, we expect to have 60/30 = 2 data points.
-- If one data point has zero values, there will only be one row and not two.
-- We do not want to take the average because we want to account for the
-- data point with zero values.
-- So use  the expected number of data points, instead of the average.
--
-- For freq > 60, you would never get more than one collection in any one minute.
-- In this case, your min = max = per minute
--
 SET select_stmt = CONCAT(" SELECT JSON_ARRAYAGG(COLUMN_NAME) INTO @column_name_list ",
          " FROM information_schema.COLUMNS  WHERE TABLE_NAME = '",in_table_name,"' AND COLUMN_NAME = 'duration'");
CALL tpf_execute_immediate(select_stmt);

IF @column_name_list IS NOT NULL
THEN
    SET frequency_select_stmt = CONCAT(
         " WITH freqTbl AS (SELECT system_id, ",
         " CASE WHEN round(AVG(duration),0) >= 35000  THEN 0 ",
         " ELSE 1 END  as frequency ",
         " FROM ", in_table_name,
         " WHERE TIME >= '", in_per_minute_start_date, "'",
         " AND TIME < '", in_per_minute_end_date, "' ",
         " AND entry_is_pruned IS FALSE "
         " GROUP BY system_id) "
);
END IF;

CALL tpf_get_summarized_str(@column_name_list,in_metric_columns,avg_metrics_list, temp_table_select_cols,all_cols);

SELECT GROUP_CONCAT(in_table_name,
            REPLACE(".`replace_col-per_minute` = ","replace_col", metric_col),
            " pm_", in_table_name,
            REPLACE(".`replace_col-per_minute`","replace_col", metric_col)
            )
INTO  temp_table_update_cols
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(in_table_name,
            REPLACE(".`replace_col-max_per_second` = ","replace_col", metric_col),
            " pm_", in_table_name,
            REPLACE(".`replace_col-max_per_second`","replace_col", metric_col)
            )
INTO  temp_table_update_max
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(REPLACE("`replace_col`","replace_col", metric_col))
INTO  temp_table_insert_cols
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(REPLACE("`replace_col-max_per_second`","replace_col", metric_col))
INTO  temp_table_insert_max
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(REPLACE("`replace_col-min_per_second`","replace_col", metric_col))
INTO  temp_table_insert_min
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(in_table_name,
                REPLACE(".`replace_col-min_per_second` = ","replace_col", metric_col),
                " pm_", in_table_name,
                REPLACE(".`replace_col-min_per_second`","replace_col", metric_col)
                )
INTO  temp_table_update_min
FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

IF @column_name_list IS NOT NULL
THEN
    -- build the max,min for the virtual metrics, we won't replace the metric itself which is calculated as needed.
    SELECT GROUP_CONCAT(REPLACE(
                            " CASE WHEN freqTbl.frequency > 0 THEN  SUM(ROUND(duration / 1000 * `replace_col`)) / 60 ELSE AVG(`replace_col`) END AS `replace_col-per_minute`,"
                            " MAX(`replace_col`)  AS `replace_col-max_per_second`,"
                            " MIN(`replace_col`)  AS `replace_col-min_per_second`",
                            "replace_col", virtual_metric_col)),
        GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", virtual_metric_col))
    INTO temp_table_select_cols_virtual_metrics,all_virtual_cols
    FROM JSON_TABLE(in_virtual_metric_columns, "$[*]" COLUMNS(virtual_metric_col VARCHAR(128) path "$")) imc;
ELSE
    -- build the max,min for the virtual metrics, we won't replace the metric itself which is calculated as needed.
    SELECT GROUP_CONCAT(REPLACE("AVG(`replace_col`) AS `replace_col-per_minute`,"
                            " MAX(`replace_col`)  AS `replace_col-max_per_second`,"
                            " MIN(`replace_col`)  AS `replace_col-min_per_second`",
                            "replace_col", virtual_metric_col)),
        GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", virtual_metric_col))
    INTO temp_table_select_cols_virtual_metrics,all_virtual_cols
    FROM JSON_TABLE(in_virtual_metric_columns, "$[*]" COLUMNS(virtual_metric_col VARCHAR(128) path "$")) imc;
END IF;

-- merge the virtual columns into the all columns
IF temp_table_select_cols_virtual_metrics IS NOT NULL AND all_virtual_cols IS NOT NULL THEN
    SET temp_table_select_cols = CONCAT(temp_table_select_cols,",",temp_table_select_cols_virtual_metrics);
    SET all_cols = CONCAT(all_cols,",",all_virtual_cols);
END IF;

SELECT GROUP_CONCAT(
            in_table_name,
            REPLACE(".`replace_col-per_minute` = ","replace_col", virtual_metric_col),
            " pm_", in_table_name,
            REPLACE(".`replace_col-per_minute`,","replace_col", virtual_metric_col),
            in_table_name,
            REPLACE(".`replace_col-min_per_second` = ","replace_col", virtual_metric_col),
            " pm_", in_table_name,
            REPLACE(".`replace_col-min_per_second`,","replace_col", virtual_metric_col),
            in_table_name,
            REPLACE(".`replace_col-max_per_second` = ","replace_col", virtual_metric_col),
            " pm_", in_table_name,
            REPLACE(".`replace_col-max_per_second`","replace_col", virtual_metric_col)
            )
INTO  virtual_temp_table_update_cols
FROM JSON_TABLE(in_virtual_metric_columns, "$[*]" COLUMNS(virtual_metric_col VARCHAR(128) path "$")) imc;

SELECT GROUP_CONCAT(
    in_table_name,
    REPLACE(".`replace_col`= pm_","replace_col", extra_col),
    in_table_name,
    REPLACE(".`replace_col`","replace_col", extra_col))
INTO extra_update_columns
FROM JSON_TABLE(in_extra_columns, "$[*]" COLUMNS(extra_col VARCHAR(128) path "$")) iec;

-- ["extra_col", "another_col"] -> "extra_col,another_col"
SELECT GROUP_CONCAT(CONCAT('`', extra_col, '`'))
INTO extra_cols
FROM JSON_TABLE(in_extra_columns, "$[*]" COLUMNS(extra_col VARCHAR(128) path "$")) iec;

-- ["system_id", "value_id"] -> "system_id,value_id"
SELECT GROUP_CONCAT(CONCAT('`', pk_col, '`'))
INTO pk_cols
FROM JSON_TABLE(in_primary_key_columns, "$[*]" COLUMNS(pk_col VARCHAR(128) path "$")) ipkc
WHERE pk_col != 'TIME';

SELECT GROUP_CONCAT(" AND ",
    in_table_name,
    REPLACE(".`replace_col`= pm_","replace_col", pk_col),
    in_table_name,
    REPLACE(".`replace_col`","replace_col", pk_col) SEPARATOR " ")
INTO where_pk_cols
FROM JSON_TABLE(in_primary_key_columns, "$[*]" COLUMNS(pk_col VARCHAR(128) path "$")) ipkcw
WHERE pk_col != 'TIME';

SET temp_table_insert_cols = CONCAT(
        "time,",pk_cols, ", ",temp_table_insert_cols, ",",temp_table_insert_max, ",", temp_table_insert_min,
        CASE WHEN extra_cols IS NULL THEN "" ELSE CONCAT(",", extra_cols) END,
        ",`entry_is_pruned`"
);

SET temp_table_select_cols_2 = CONCAT(
        "SUBTIME(", in_table_name, ".time, EXTRACT(SECOND FROM ",  in_table_name, ".time)) AS TIME,",
        in_table_name,".",
        pk_cols, ", ",temp_table_select_cols,
        CASE WHEN extra_cols IS NULL THEN "" ELSE CONCAT(",", extra_cols) END,
        ",entry_is_pruned"
    );

SET temp_table_select_cols = CONCAT(
        "MIN(TIME) AS TIME,",
        pk_cols, ", ",temp_table_select_cols,
        CASE WHEN extra_cols IS NULL THEN "" ELSE CONCAT(",", extra_cols) END,
         ", entry_is_pruned"
);

SET  temp_table_update_cols_list =
     CONCAT(temp_table_update_cols,
            ",",temp_table_update_max,
            ",",temp_table_update_min,
            CASE WHEN virtual_temp_table_update_cols IS NULL THEN "" ELSE CONCAT(",", virtual_temp_table_update_cols) END,
            CASE WHEN extra_update_columns IS NULL THEN "" ELSE CONCAT(",", extra_update_columns) END,
            ",",in_table_name,".entry_is_pruned = pm_", in_table_name,".entry_is_pruned"
);

-- drop the pm table just in case.
CALL tpf_execute_immediate(CONCAT(
         " DROP TEMPORARY TABLE IF EXISTS pm_", in_table_name
));

-- calculate our avg/max/min and put it into a temp table
IF @column_name_list IS NOT NULL
THEN
    SET select_stmt = CONCAT(
         " CREATE TEMPORARY TABLE pm_", in_table_name, " AS\n",
         frequency_select_stmt,
         "\n SELECT ",temp_table_select_cols_2,
         "\n FROM ", in_table_name,
         " INNER JOIN freqTbl ON freqTbl.system_id =  ", in_table_name, ".system_id \n"
         " WHERE TIME >= '", in_per_minute_start_date, "'\n",
         " AND TIME < '", in_per_minute_end_date, "' \n",
          " GROUP BY ",  pk_cols
    );
ELSE
    SET select_stmt = CONCAT(
         " CREATE TEMPORARY TABLE pm_", in_table_name, " AS",
         " SELECT ",temp_table_select_cols_2,
         " FROM ", in_table_name,
         " WHERE TIME >= '", in_per_minute_start_date, "'",
         " AND TIME < '", in_per_minute_end_date, "' ",
          " GROUP BY ",  pk_cols
    );
END IF;
CALL tpf_execute_immediate(select_stmt);

-- set the entry_is_pruned field in our temp entry
CALL tpf_execute_immediate( CONCAT(
         " UPDATE IGNORE pm_", in_table_name,
         " SET pm_", in_table_name, ".`entry_is_pruned` = TRUE"
));

-- This does an insert of our data from temp table to add an entry
-- at second 00.000 in case it does not exist.  This call may fail
-- if an entry exists at that instant of time.
SET select_stmt = CONCAT(
         " INSERT IGNORE INTO ", in_table_name,
         " (time,",pk_cols,",entry_is_pruned)",
         " SELECT  time,",pk_cols,",entry_is_pruned",
          " FROM pm_", in_table_name
);
CALL tpf_execute_immediate(select_stmt);

-- Next we'll update the 00.000 time entry with the avg/max/min from our
-- temp table.
SET select_stmt =
        CONCAT(
         " UPDATE IGNORE ", in_table_name, ",pm_", in_table_name,
         " SET ", temp_table_update_cols_list,
         " WHERE ", in_table_name, ".TIME = pm_", in_table_name, ".TIME ",
          where_pk_cols
);
CALL tpf_execute_immediate(select_stmt);

-- drop our temp table
CALL tpf_execute_immediate(CONCAT(
        " DROP TEMPORARY TABLE IF EXISTS pm_", in_table_name
));

END//
DELIMITER ;


-- return the most recent 2 minute start time for the requested table
DROP PROCEDURE IF EXISTS `tpf_get_avg_info_start_time`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_avg_info_start_time` (
    IN `in_table_name` VARCHAR(128),
    OUT `out_per_minute_start_date` DATETIME,
    OUT `out_per_minute_end_date` DATETIME
)
tpf_get_avg_info_start_time:BEGIN

DECLARE most_recent_time DATETIME;

CALL tpf_execute_immediate(CONCAT(
    " SELECT ", in_table_name, ".time",
    " INTO @most_recent_time",
    " FROM ", in_table_name,
    " ORDER BY ",in_table_name,".time DESC",
    " LIMIT 1;"
));

SET out_per_minute_start_date = DATE_SUB(@most_recent_time, INTERVAL 2 MINUTE);
SET out_per_minute_start_date = DATE_SUB(out_per_minute_start_date, INTERVAL SECOND(out_per_minute_start_date) SECOND);
SET out_per_minute_end_date = DATE_ADD(out_per_minute_start_date, INTERVAL 1 MINUTE);

END//
DELIMITER ;
--
-- Generate the string that will create the values for per_minute, max, and min.
-- for the metrics.
--
-- If avg_metrics_list is not NULL, that means that for this table,
-- some metrics will be averaged to create the per minute data.
-- For all other metrics, base the per minute value on the frequency.
-- For example, blk_read_depth from in_metric_columns for tpf_tpfdf_metrics_table data type
-- should use AVG but all other metrics in that table should be treated like a frequency.
-- blk_read_depth is looking at the files and counting how many overflow blocks they have.
-- You do not want to average in the zero values.
-- This is an absolute count, and not a rate.
-- use the freqTable for metrics that are rates, and not absolute counts.
--
DROP PROCEDURE IF EXISTS `tpf_get_summarized_str`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_get_summarized_str` (
    IN `column_name_list` TEXT,
    IN `in_metric_columns` JSON,
    IN `avg_metrics_list` JSON,
    OUT `temp_table_select_cols` LONGTEXT,
    OUT `all_cols_result` LONGTEXT
)

tpf_get_summarized_str:BEGIN

DECLARE freq_metrics_list JSON;
DECLARE temp_table_select_cols_avg LONGTEXT;
DECLARE all_cols_result_avg LONGTEXT;
DECLARE highwater_metrics_list JSON;
DECLARE temp_table_select_cols_high_water LONGTEXT;
DECLARE all_cols_result_high_water LONGTEXT;

IF avg_metrics_list IS NOT NULL AND JSON_LENGTH(avg_metrics_list) > 0
THEN
    SELECT JSON_ARRAYAGG(metric_col)
    INTO highwater_metrics_list
    FROM JSON_TABLE(avg_metrics_list, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc
    WHERE  metric_col LIKE '%_highwater';

    SET freq_metrics_list = tpf_remove_elements_from_json_array(in_metric_columns,avg_metrics_list);
    SET avg_metrics_list = tpf_remove_elements_from_json_array(avg_metrics_list,highwater_metrics_list);

    SELECT GROUP_CONCAT(REPLACE("CASE WHEN freqTbl.frequency > 0 THEN SUM(ROUND(duration / 1000 * `replace_col`)) / 60 ELSE AVG(`replace_col`) END AS `replace_col-per_minute`,"
            " MAX(`replace_col`) AS `replace_col-max_per_second`,"
            " MIN(`replace_col`) AS `replace_col-min_per_second`",
            "replace_col", metric_col)),
           -- ["duration", "cpu_used"] -> "`duration`, `max_duration`, `cpu_used`, `max_cpu_used`"
            GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", metric_col))
    INTO temp_table_select_cols, all_cols_result
    FROM JSON_TABLE(freq_metrics_list, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;

    IF avg_metrics_list IS NOT NULL AND JSON_LENGTH(avg_metrics_list) > 0
    THEN
        SELECT GROUP_CONCAT(REPLACE("AVG(`replace_col`) AS `replace_col-per_minute`,"
            " MAX(`replace_col`) AS `replace_col-max_per_second`,"
            " MIN(`replace_col`) AS `replace_col-min_per_second`",
            "replace_col", metric_col)),
           -- ["duration", "cpu_used"] -> "`duration`, `max_duration`, `cpu_used`, `max_cpu_used`"
            GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", metric_col))
        INTO temp_table_select_cols_avg, all_cols_result_avg
        FROM JSON_TABLE(avg_metrics_list, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;
    END IF;

    IF highwater_metrics_list IS NOT NULL AND JSON_LENGTH(highwater_metrics_list) > 0
    THEN
        SELECT GROUP_CONCAT(REPLACE("MAX(`replace_col`) AS `replace_col-per_minute`,"
                " MAX(`replace_col`) AS `replace_col-max_per_second`,"
                " MIN(`replace_col`) AS `replace_col-min_per_second`",
                "replace_col", metric_col)),
               -- ["duration", "cpu_used"] -> "`duration`, `max_duration`, `cpu_used`, `max_cpu_used`"
                GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", metric_col))
        INTO temp_table_select_cols_high_water, all_cols_result_high_water
        FROM JSON_TABLE(highwater_metrics_list, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;
    END IF;

    IF avg_metrics_list IS NOT NULL AND JSON_LENGTH(avg_metrics_list) > 0 AND
        highwater_metrics_list IS NOT NULL AND JSON_LENGTH(highwater_metrics_list) > 0
    THEN
        SET all_cols_result =  CONCAT(all_cols_result, ",", all_cols_result_avg, "," , all_cols_result_high_water);
        SET temp_table_select_cols =  CONCAT(temp_table_select_cols,",", temp_table_select_cols_avg, ",",temp_table_select_cols_high_water);
    ELSEIF highwater_metrics_list IS NOT NULL AND JSON_LENGTH(highwater_metrics_list) > 0
    THEN
        SET all_cols_result =  CONCAT(all_cols_result, ",", all_cols_result_high_water);
        SET temp_table_select_cols =  CONCAT(temp_table_select_cols,",", temp_table_select_cols_high_water);
    ELSE
        SET all_cols_result =  CONCAT(all_cols_result, ",", all_cols_result_avg);
        SET temp_table_select_cols =  CONCAT(temp_table_select_cols,",", temp_table_select_cols_avg);
    END IF;
    LEAVE tpf_get_summarized_str;
END IF;

--
-- column_name_list indicates that duration is one of the columns.
-- Use frequency instead of average in this case.
--
IF @column_name_list IS NOT NULL
THEN
    SELECT GROUP_CONCAT(REPLACE(
            "CASE WHEN freqTbl.frequency > 0 THEN  SUM(ROUND(duration / 1000 * `replace_col`)) / 60 ELSE AVG(`replace_col`) END AS `replace_col-per_minute`,"
            " MAX(`replace_col`) AS `replace_col-max_per_second`,"
            " MIN(`replace_col`) AS `replace_col-min_per_second`",
            "replace_col", metric_col)),
           -- ["duration", "cpu_used"] -> "`duration`, `max_duration`, `cpu_used`, `max_cpu_used`"
            GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", metric_col))
     INTO temp_table_select_cols, all_cols_result
    FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;
ELSE
    SELECT GROUP_CONCAT(REPLACE("AVG(`replace_col`) AS `replace_col-per_minute`,"
            " MAX(`replace_col`) AS `replace_col-max_per_second`,"
            " MIN(`replace_col`) AS `replace_col-min_per_second`",
            "replace_col", metric_col)),
           -- ["duration", "cpu_used"] -> "`duration`, `max_duration`, `cpu_used`, `max_cpu_used`"
            GROUP_CONCAT(REPLACE("`replace_col`", "replace_col", metric_col))
    INTO temp_table_select_cols, all_cols_result
    FROM JSON_TABLE(in_metric_columns, "$[*]" COLUMNS(metric_col VARCHAR(128) path "$")) imc;
END IF;


END//
DELIMITER ;
