/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023, 2024                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20230719    PJ46904 - User-Defined Metrics                       */
/*  20240426    PJ47242 - Improve correlation and stats dashboard    */
/*                        performance                                */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     UDM Stored procedures                               */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';



DROP PROCEDURE IF EXISTS `tpf_sample_udm_data_types_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_sample_udm_data_types_available` (
    IN `in_tpf_systems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)
tpf_sample_udm_data_types_available:BEGIN

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

WITH udm_data_types_available AS (
    SELECT udm_dt.data_type, udm_dt.id
    FROM udm_data_types udm_dt
    WHERE EXISTS (
       SELECT 1
       FROM udm_metrics_table udm_mt
            INNER JOIN JSON_TABLE(in_tpf_systems, "$[*]" COLUMNS(id INT path "$")) its ON udm_mt.system_id=its.id
       WHERE udm_mt.data_type_id=udm_dt.id AND udm_mt.TIME BETWEEN in_lt_time AND in_rt_time
    )
    GROUP BY udm_dt.id
)
SELECT data_type AS __text, id AS __value
FROM udm_data_types_available;

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_sample_udm_group_names_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_sample_udm_group_names_available` (
    IN `in_tpf_systems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)
tpf_sample_udm_group_names_available:BEGIN

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

WITH udm_group_names_available AS (
    SELECT udm_gn.group_name, udm_gn.id
    FROM udm_group_names udm_gn
    WHERE EXISTS (
        SELECT 1
        FROM udm_metrics_table udm_mt
             INNER JOIN JSON_TABLE(in_tpf_systems, "$[*]" COLUMNS(id INT path "$")) its ON udm_mt.system_id=its.id
        WHERE udm_mt.group_name_id=udm_gn.id AND udm_mt.TIME BETWEEN in_lt_time AND in_rt_time
    )
    GROUP BY udm_gn.id
)
SELECT group_name AS __text, id AS __value
FROM udm_group_names_available;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_udm_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_udm_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

SET result = '
  [
    {"panel_key":"User_Defined_Metrics",
     "table_name":"udm_metrics_table",
     "metric_type":"primary",
     "metrics":["metric1", "metric2", "metric3"],
     "virtual_metrics":[]
    }
  ]';

RETURN result;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_user_defined_metrics_graph`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_user_defined_metrics_graph` (
    IN `in_tpf_systems` JSON,
    IN `in_data_types_ids` JSON,
    IN `in_group_name_ids` JSON,
    IN `in_show_metrics` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME,
    IN `in_sec_per_interval` INT,
    IN `in_agg_type` ENUM ('no_agg', 'avg_systems', 'total_systems')
)

tpf_user_defined_metrics_graph:BEGIN

DECLARE num_tpf_systems INT;

DECLARE metric_columns TEXT;
DECLARE sum_metric_columns TEXT;

DECLARE temp_sys_id_table_name LONGTEXT;
DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE cte_stmt LONGTEXT DEFAULT "";

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- with pruning metrics, the group concat can overrun the 1024 character default.
SET SESSION group_concat_max_len = 100000;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_contains(in_tpf_systems,'[""]') OR in_tpf_systems='""' OR in_tpf_systems="" OR in_tpf_systems="[]"
THEN
    LEAVE tpf_user_defined_metrics_graph;
END IF;

-- this can occur if they unselect the default metric type.
IF JSON_contains(in_show_metrics,'[""]') OR in_show_metrics='""'  OR in_show_metrics="" OR in_show_metrics="[]"
THEN
    LEAVE tpf_user_defined_metrics_graph;
END IF;
-- If only 1 metric is selected, SOMETIMES, Grafana doesn't send down an array.
IF JSON_TYPE(in_show_metrics) = "STRING"
THEN
    SET in_show_metrics = JSON_UNQUOTE(in_show_metrics);
    SET in_show_metrics = JSON_ARRAY(in_show_metrics);
END IF;

IF in_agg_type = 'no_agg'
THEN
    -- We only need the display_name for systems when we are not aggregating, i.e. we have one series per selected system
    CALL tpf_create_system_display_name_temp_table("user_defined_metrics_graph_temp_selected_systems", in_tpf_systems);

    -- Playing weird games to avoid referring to a temp table twice in the large query that follows
    SELECT CONCAT(
        "JSON_TABLE(",
        QUOTE(JSON_ARRAYAGG(JSON_OBJECT('id', id, 'display_name', display_name))), -- Get the table rows as JSON
        ", '$[*]' COLUMNS(id INT path '$.id', display_name VARCHAR(100) path '$.display_name'))"
    )
    INTO temp_sys_id_table_name
    FROM user_defined_metrics_graph_temp_selected_systems;
ELSE
    -- This creates a table of selected system ids which we will join to the udm_metrics_table table to filter selected systems
    SET temp_sys_id_table_name = CONCAT('JSON_TABLE(', QUOTE(in_tpf_systems), ', "$[*]" COLUMNS(id INT path "$"))');
END IF;

SET metric_columns = tpf_complex_create_selected_metrics(in_show_metrics, CAST(in_sec_per_interval as CHAR), in_agg_type,"udm_metrics_table");
IF metric_columns IS NULL
THEN
    LEAVE tpf_user_defined_metrics_graph;
END IF;

-- Create metric columns like SUM(internal_operation_count) AS internal_operation_count, SUM(external_operation_count) AS external_operation_count
-- for CTE statements
SELECT GROUP_CONCAT(DISTINCT REPLACE("SUM(metric) AS metric", "metric", SUBSTRING_INDEX(metric, '-', 1)))
INTO sum_metric_columns
FROM JSON_TABLE(in_show_metrics, "$[*]" COLUMNS(metric VARCHAR(64) path "$")) ism;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

SET num_tpf_systems = JSON_LENGTH(in_tpf_systems);

-- Get the sys ids, possibly with display_name (if no_agg)
SET cte_stmt = CONCAT(
    " WITH sys_id_table_cte AS (",
        " SELECT *",
        " FROM ", temp_sys_id_table_name, " tsitn",
    " )"
);

SET select_stmt = CONCAT(
--    This line is used for certain demos
--    " SELECT MIN(udm_mt.time) AS time, udm_mt.metric1 as error_cnt, udm_mt.metric2 as timeout_cnt",
    " SELECT MIN(udm_mt.time) AS time,",
             CASE WHEN in_agg_type = 'no_agg' AND num_tpf_systems > 1  THEN "CONCAT_WS('-', udm_dt.data_type, udm_gn.group_name, t.display_name) AS series_name,"
                  ELSE "CONCAT_WS('-', udm_dt.data_type, udm_gn.group_name) AS series_name,"
             END,
             metric_columns,
    " FROM sys_id_table_cte t", -- Filter the selected systems
           " INNER JOIN udm_metrics_table udm_mt ON t.id=udm_mt.system_id", -- get the metrics
           " INNER JOIN JSON_TABLE(", QUOTE(in_data_types_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) udm_dti ON udm_mt.data_type_id=udm_dti.id", -- Filter the selected data_type
           " INNER JOIN JSON_TABLE(", QUOTE(in_group_name_ids), ", \"$[*]\" COLUMNS(id INT path \"$\")) udm_gni ON udm_mt.group_name_id=udm_gni.id", -- Filter the selected group_name
           " INNER JOIN udm_group_names udm_gn ON udm_mt.group_name_id=udm_gn.id", -- Get the group names
           " INNER JOIN udm_data_types udm_dt ON udm_mt.data_type_id=udm_dt.id", -- Get the group names
    " WHERE udm_mt.time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    " GROUP BY ", CASE WHEN in_agg_type = 'no_agg' THEN "udm_mt.system_id, " ELSE "" END, "udm_mt.data_type_id, udm_mt.group_name_id, (UNIX_TIMESTAMP(udm_mt.time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval
);

SET select_stmt = CONCAT(
    select_stmt,
    " ORDER BY time ASC"
);

CALL tpf_execute_immediate(CONCAT(cte_stmt, select_stmt));


DROP TEMPORARY TABLE IF EXISTS user_defined_metrics_graph_temp_selected_systems;

END//
DELIMITER ;
