#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
import time

from exceptions.json_err import InvalidAppDimensionConfigError
from tpf_batch.batch_processor import AnalyzerBatchProcessor
from tpf_data_manager.zrtmc_manager import ZrtmcManager
from tpf_user_exit.udm import udm_id_manager 
from tpf_data_manager import id_manager
from tpf_data_manager.dbms import DBMS
from tpf_data_manager.source_manager import SourceManager
from tpf_user_exit.udm.udm_manager import UDMManager
from tpf_data_manager.data_source import DataSource
from tpf_nvpc.nvpc_config.app_dimension import AppDimensionConfigValidator
from tpf_nvpc.nvpc_config.orientation import Orientation
from tpf_batch.batch import AnalyzerBatch
from tpf_rtmc_db.db_writer import DbWriter
from pymysql import DatabaseError, OperationalError
from typing import Dict, Type, List
from constants.profile_constants import *
from tpf_user_exit.tpf_user_constants.tpf_user_profile_constants import *
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.profile_utils import are_profile_parameters_present

logger = logging.getLogger('tpf_zrtmc_analyzer')


class UserZrtmcManager(ZrtmcManager):
    source_managers: Dict[DataSource, Type[SourceManager]] = {DataSource.UDM: UDMManager}

    user_sources = [user_source.name.lower() for user_source in source_managers]

    required_parameters_data_source = [ACTIVE, USER_PROCESSING]
    required_parameters_db = [HOST, PORT, USERNAME, PASSWORD, NAME]

    additional_db_tables = udm_id_manager.id_table_cols.keys()


    def __init__(self, analyzer_profile: dict):
        """
        Initialize a ZrtmcManager object.

        :param analyzer_profile: Python dictionary of tpf_zrtmc_analyzer_profile
        :raise ValueError: If the profile data has an incorrect field, or is missing a required field.
        """
        # num of empty batches encountered
        self.empty_batches = 0
        self.active_source_managers: Dict[DataSource, SourceManager] = {}

        if not are_profile_parameters_present(analyzer_profile, [DATA_SOURCES], None, log_missing=True):
            raise ValueError

        sources = analyzer_profile[DATA_SOURCES]
        unrecognized_sources = set()

        for source_name, source_config in sources.items():
            try:
                data_source: DataSource = DataSource[source_name.upper()]
            except KeyError:
                unrecognized_sources.add(source_name)
                continue
            
            if are_profile_parameters_present(source_config, self.required_parameters_data_source,
                                                  f'{DATA_SOURCES}.{source_name}', log_missing=False, warn_missing=True):
               if source_config[ACTIVE]:
                   if data_source not in {DataSource.NVPC_HORIZONTAL, DataSource.NVPC_VERTICAL}:
                       # Initializing source_manager could raise a ValueError if there is an error in the configuration.
                       # We want to raise a ValueError from this init function if there is an error, so we won't handle it.
                       self.active_source_managers[data_source] = self.source_managers[data_source](source_config[USER_PROCESSING])
                   else:
                       app_dim_validator = AppDimensionConfigValidator()
                       app_dimensions_config = {}

                       # We need to validate all the application_dimensions as a group, so save the config for now then
                       # we'll initialize the nvpc source managers later
                       orientation = Orientation.HORIZONTAL if data_source == DataSource.NVPC_HORIZONTAL else Orientation.VERTICAL
                       # {'app_dimensions': List[AppDimensions], 'user_processing': List[String]}
                       try:
                           app_dimensions_config[data_source] = {
                               'app_dimensions': app_dim_validator.validate(
                                   source_config.get('application_dimensions'),
                                   orientation,
                                   f'{DATA_SOURCES}.{source_name}.application_dimensions'),
                               USER_PROCESSING: source_config[USER_PROCESSING]
                           }
                       except InvalidAppDimensionConfigError:
                           raise ValueError

                       if data_source == DataSource.NVPC_HORIZONTAL:
                           try:
                               app_dim_validator.validate_group()
                           except InvalidAppDimensionConfigError:
                               raise ValueError
                        
                           sys_indicator = app_dim_validator.get_system_indicator()


                       self.active_source_managers.update(
                           {ds: self.source_managers[ds](config[USER_PROCESSING], **config, **{'system_indicator': sys_indicator}) for
                            ds, config in app_dimensions_config.items()}
                       )

        if len(unrecognized_sources) > 0:
            msg = f'The following data sources found in the RTMC Analytics profile were unrecognized:' \
                  f' {list(unrecognized_sources)} User data sources are: {self.user_sources}'
            logger.warning(msg)

        if self.active_source_managers == {}:
            msg = f'No active data sources were configured. You must configure one of the following data sources' \
                  f' to be active: {self.user_sources}'
            logger.error(msg)
            raise ValueError

        # Log any data sources that were missing
        for ds in self.source_managers:
            if ds not in self.active_source_managers:
                logger.info(f'Data source "{ds.name}" is inactive.')

        # DB section could be either mariadb or database. We'll log when database is missing
        db_section = MARIADB

        if not are_profile_parameters_present(analyzer_profile, [db_section], None, log_missing=False):
            db_section = DATABASE
            if not are_profile_parameters_present(analyzer_profile, [db_section], None, log_missing=True):
                raise ValueError

        db_profile = analyzer_profile[db_section]

        if not are_profile_parameters_present(db_profile, self.required_parameters_db, db_section, log_missing=True):
            raise ValueError

        host = db_profile[HOST]
        port = db_profile[PORT]
        username = db_profile[USERNAME]
        password = db_profile[PASSWORD]
        database_name = db_profile[NAME]

        dbms = DBMS.MARIADB

        # Require system when db_section is database
        if db_section == DATABASE:
            if not are_profile_parameters_present(db_profile, [SYSTEM], db_section, log_missing=True):
                raise ValueError

            try:
                dbms = DBMS[db_profile[SYSTEM].upper()]
            except KeyError:
                logger.error(f'Value {db_profile[SYSTEM]} for {db_section}.{SYSTEM} is not one of the following '
                             f'values: {["mysql", "mariadb"]}')
                raise ValueError

        self.db_writer = DbWriter(host, port, username, password, database_name, dbms)
        self.db_start_time = 0

    @classmethod
    def get_instance(cls, analyzer_profile):
        """
        Create ZrtmcManager object from analyzer profile information.

        :param analyzer_profile: analyzer_profile Python dictionary
        :return: ZrtmcManager object from analyzer_profile
        :rtype: ZrtmcManager
        """

        try:
            data_manager = UserZrtmcManager(analyzer_profile)
        except ValueError:
            return None
        return data_manager

