#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from typing import Dict, Type

from exceptions.json_err import MissingFieldError
from tpf_user_exit.udm.udm_record_types.qudm_record_type import QUDMEntry
from tpf_user_exit.udm.udm_record_types.qmwr_record_type import QMWREntry
from tpf_user_exit.udm.udm_sample_data import UDMSampleData 
from tpf_user_exit.udm.udm_sample_header import UDMSampleHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class UDMBody(object):

    required_fields_rlist = ["recordList"]
    required_fields_udm_data = ["udm_data"]
    required_fields_rheader = ["header"]
    required_fields_rarray = ["data_array"]

    udm_data_types: Dict[str, Type[UDMSampleData]] = {
        'QUDM': QUDMEntry,
        'QMWR': QMWREntry,
        'default': UDMSampleData 
    }

    encountered_data_types = set()

    def __init__(self, json_body,data_type):

        """
        :param json_body: "body" dictionary from UDM JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            record_list = json_body["recordList"]

        except KeyError:
            log_message = f'Unable to deserialize UDM body. ' \
                          f'Missing fields: {get_missing_keys(json_body, self.required_fields_rlist)}. ' \
                          f'Data: {json_body}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

        self.record_list = []

        for record in record_list:

            try:
                udm_data_list = record['udm_data']
            except KeyError:
                log_message = f'Unable to deserialize UDM body. ' \
                              f'Missing fields: {get_missing_keys(json_body, self.required_fields_udm_data)}. ' \
                              f'Data: {json_body}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)

            try:
                udm_sample_header_object = UDMSampleHeader(udm_data_list['header'],data_type)
            except KeyError:
                log_message = f'Unable to deserialize UDM body. ' \
                              f'Missing fields: {get_missing_keys(json_body, self.required_fields_rheader)}. ' \
                              f'Data: {json_body}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)

            udm_data_object_type = self.udm_data_types[data_type]
            try:
                self.record_list.append(udm_data_object_type(udm_data_list['data_array'], udm_sample_header_object))
            except KeyError:
                log_message = f'Unable to deserialize UDM body. Missing fields: ' \
                              f'{get_missing_keys(record, self.required_fields_rarray)}. Data: {record}'
                logger.warning(log_message)
                raise MissingFieldError(log_message)
