#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_user_exit.udm. tpf_udm_payload import TPF_UDM_Payload
from tpf_user_exit.udm.udm_db_handlers.qudm_db_handler import QUDMHandler
from tpf_user_exit.udm.udm_db_handlers.qmwr_db_handler import QMWRHandler
from tpf_data_manager.source_manager import SourceManager
from pymysql.cursors import Cursor
from typing import List, Dict, Type
import logging

from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler

logger = logging.getLogger('tpf_zrtmc_analyzer')


class UDMManager(SourceManager):
    udm_data_types: Dict[str, Type[DbDataHandler]] = {"QUDM": QUDMHandler,
                                                      "QMWR": QMWRHandler}
    required_fields = udm_data_types.keys()

    def __init__(self, active_source_types: List):
        # Sets self.active_source_types
        super().__init__(active_source_types)

        self.unhandled_data_types = set()
        self.active_udm_data_types: Dict[str, DbDataHandler] = {}

        if active_source_types is not None:
            unrecognized_source_types = []

            for source_type in active_source_types:
                try:
                    self.active_udm_data_types[source_type] = self.udm_data_types[source_type]()
                except KeyError:
                    unrecognized_source_types.append(source_type)

            if len(unrecognized_source_types) > 0:
                msg = f'The following data types were not recognized: {unrecognized_source_types}.' \
                      f'The following are recognized UDM data types: {self.udm_data_types.keys()}. Make ' \
                      f'sure the correct data types are specified in the data_sources.udm.user_processing ' \
                      f'section of the ZRTMC Analyzer profile.'
                logger.error(msg)
                raise ValueError

        if active_source_types is None or len(active_source_types) == 0:
            msg = f'UDM processing is not enabled for UDM.'
            logger.info(msg)
        else:
            msg = f'UDM processing is enabled for the following UDM data types: {active_source_types}'
            logger.info(msg)

    def get_db_tables(self, active_only=True):

        table_names = []
        if active_only:
            for data_handler in self.active_udm_data_types.values():
                table_names += data_handler.get_table_names()
        else:
            for data_handler in self.udm_data_types.values():
                table_names += data_handler.get_table_names()
        return table_names

    def deserialize(self, json_doc: Dict):
        try:
            source = json_doc['header']['source']
            if source != 'RTMC_UDM' and source != 5:
                return None
            if str(json_doc["header"]["parameters"]["data_type"]) not in self.udm_data_types:
                msg = f'User Define Metrics Data type is not defined: {str(json_doc["header"]["parameters"]["data_type"])} ' \
                      f'Data: {json_doc}'
                logger.error(msg)
                return None
            return TPF_UDM_Payload(json_doc)
        except (KeyError, MissingFieldError):
            msg = f'Failed to deserialize a JSON document. Source: UDM. Data: {json_doc}'
            logger.error(msg)
            return None

    def get_mysql_insert_manys(self, db_cursor: Cursor, analytics_data: TPF_UDM_Payload):
        insert_manys: List[InsertMany] = []
        for record in analytics_data.body.record_list:

            udm_data_type = record.udm_header.data_type

            if udm_data_type in self.active_udm_data_types:
                data_type_handler = self.active_udm_data_types[udm_data_type]
                insert_manys += data_type_handler.get_mysql_insert_manys(db_cursor, record, parent=analytics_data)
            elif udm_data_type not in self.unhandled_data_types:
                msg = f'Encountered a UDM data type for which there is no active UDM processing. ' \
                      f'Data type: {udm_data_type}'
                logger.warning(msg)
                self.unhandled_data_types.add(udm_data_type)
        return insert_manys
