#ifndef __QUDM_HPP__
#define __QUDM_HPP__ 1
/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023                                     */
/*********************************************************************/
/*** START OF CHANGE HISTORY******************************************/
/*  DEFECT  DATE     DESCRIPTION                                     */
/*  d26857  20230719 RTMC User Defined Metrics driver.               */
/*                                                                   */
/*                                                                   */
/*** END OF CHANGE HISTORY********************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*                                                                   */
/* Header filename:  include/qudm.hpp                                */
/* Header filetype:  TPF                                             */
/* Abstract:   Header for the sample user-defined metrics driver     */
/*             (QUDM).                                               */
/*                                                                   */
/* Description:  This header contains the following:                 */
/*                                                                   */
/* Sample structure definitions to use to build data to send through */
/* the user-defined metrics (UDM) pipeline.                          */
/* The sample structure definitions in this header are used by the   */
/* sample QUDM driver.                                               */
/* Use these sample structure definitions to build new user-defined  */
/* metrics.                                                          */
/*                                                                   */
/* The internal QUDM table used by the sample QUDM driver.           */
/*                                                                   */
/* The convertStructureToJson function that converts data to JSON    */
/* using a given dfdl schema file.                                   */
/*********************************************************************/
/*                                                                   */
/* Dependencies:   None                                              */
/*                                                                   */
/* Restrictions:   None                                              */
/*                                                                   */
/* Special considerations:  None                                     */
/*                                                                   */
/*********************************************************************/

/**********************************************************************/
/* Defines and Includes                                               */
/**********************************************************************/

#define debug     0
#define UDM_DFDL_ROOT "udm_data"

#include <exception>             // Required for catch
#include <sys/time.h>            // timeval struct, gettimeofday
#if debug
  #include <tpf/reusable.h>      // Required for ASCII_TO_EBCDIC
#endif
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/cdfdl.h>

/**********************************************************************/
/* Sample UDM Structure Definitions:                                  */
/*                                                                    */
/* These are the structures used by the sample QUDM driver to send    */
/* data through the UDM pipeline.                                     */
/* These structures allow binary data to be converted to JSON by      */
/* using the DFDL schema files on the z/TPF system.                   */
/* New versions of these structures need to be built along with the   */
/* DFDL to interpret them in order to send additional user-defined    */
/* metrics through the UDM pipeline.                                  */
/*                                                                    */
/* The header specifies the count of structure instances.             */
/* The data structure is an array of metrics.                         */
/**********************************************************************/

typedef struct _TPF_UDM_SAMPLE_HEADER {
  unsigned  int  version;
  char           timestamp[32];
  unsigned  int  intervalLength;
  unsigned  int  count;
} TPF_UDM_SAMPLE_HEADER;

typedef struct _TPF_UDM_SAMPLE_DATA_STRUCT {
  char           groupName[16];
  unsigned  int  metric1;
  unsigned  int  metric2;
  unsigned  int  metric3;
} TPF_UDM_SAMPLE_DATA_STRUCT;

typedef struct _TPF_UDM_SAMPLE_DATA_LAYOUT {
  TPF_UDM_SAMPLE_HEADER header;
  TPF_UDM_SAMPLE_DATA_STRUCT data_array[];
} TPF_UDM_SAMPLE_DATA_LAYOUT;

/**********************************************************************/
/* QUDM Table for the ZTEST QUDM driver.                              */
/*                                                                    */
/* The table is stored in system heap and is subsystem unique.        */
/* The locking protocol is to do a core hold on the table.            */
/* Each entry in the table is for an iteration of the driver which is */
/* identified by a 3-character token.                                 */
/*                                                                    */
/* If the eyecatcher is not present when the driver runs, the table   */
/* is initialized.                                                    */
/**********************************************************************/

#define TPF_QUDM_EYECATCHER "QUDMTABLE   "

struct TPF_QUDM_HEADER
{
   char eyecatcher[sizeof(TPF_QUDM_EYECATCHER)];
   short  count;                       // number of in-use entries
};

struct TPF_QUDM_ENTRY
{
  char           token[3];             // token assigned to this instance
                                       // of the driver. For example,
                                       // 001 or ABC.
  unsigned  char status;               // indicates whether this instance
                                       // of the driver is started or
                                       // stopped.
    #define  TPF_QUDM_STARTED 'Y'
    #define  TPF_QUDM_STOPPED 'N'
  unsigned  int  ecb_count;            // number of ECBs running this
                                       // instance of the driver.
  unsigned  int  rate;                 // how many times per second the
                                       // data is sent for this instance
                                       // of the driver.
};

/* Define the number of entries which fit in one 4K QUDM table.   */
/* The _GBSZE is the logical size of a 4K block and is defined in */
/* c_syseq.h                                                      */

#define TPF_QUDM_MAX_ENTRIES ((_GBSZE - (sizeof(struct TPF_QUDM_HEADER))) /   \
                              (sizeof(struct TPF_QUDM_ENTRY)))

struct TPF_QUDM_TABLE
{
  TPF_QUDM_HEADER header;
  TPF_QUDM_ENTRY  entry[TPF_QUDM_MAX_ENTRIES];
};

/**********************************************************************/
/* Functions                                                          */
/**********************************************************************/

/**
*  convertStructureToJson()
*  \brief Take an input buffer (char *) and convert the data to json using the
*                 given dfdl file.
*
*  \param [in]    in_data The data that is going to be converted to json via the
*                 dfdl given as input
*  \param [in]    in_data_size The size of the in_data.
*  \param [in]    in_dfdl_file The dfdl file used to convert the json
*  \return     :  char* to the successfully converted json data.
*                 NULL any other failures
*
*/
char * convertStructureToJson( char * in_data, int in_data_size, char * in_dfdl_file) {

    // Initalize
    DFDLHandle dfdl_handle = 0;
    char * json_document = NULL;
    int    json_document_length = 0;

    // Check inputs for illegal values
    if(in_data == NULL || in_data_size == 0 || in_dfdl_file == NULL)
        return NULL;

    // cebic calls
    cebic_save();
    cebic_goto_bss();

    try {
        #if debug
            printf("DFDL Processing: Starting\n");
        #endif

        // DFDL processings
        tpf_dfdl_initialize_handle(&dfdl_handle, in_dfdl_file, UDM_DFDL_ROOT, 0);
        tpf_dfdl_setData(dfdl_handle, in_data, in_data_size);
        json_document = tpf_dfdl_buildDoc(dfdl_handle, &json_document_length, TPF_DFDL_JSON, 0);
        tpf_dfdl_terminate_handle(dfdl_handle);

        #if debug
            printf("DFDL Processing: Completed\n");
        #endif

        // Check if an output was returned
        if(json_document_length != 0 &&
           json_document != NULL)
        {
            #if debug
                ASCII_TO_EBCDIC(json_document,json_document,strlen(json_document));
                printf("Your json is: %s", json_document);
            #endif

            // Restore and return
            cebic_restore();
            return json_document;
        }
    }
    catch(std::exception &dfdl_parsing_failure)
    {
        #if debug
           printf("[ERROR] DFDL failed with error: %s", dfdl_parsing_failure.what());
        #endif
        if (dfdl_handle != 0)
        {
            tpf_dfdl_terminate_handle(dfdl_handle);
        }

        if (json_document != NULL)
        {
            free(json_document);
        }

        // Restore and return
        cebic_restore();
    }
    return NULL;
}

#endif
