/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023                                     */
/*********************************************************************/
/*** START OF CHANGE HISTORY******************************************/
/*  DEFECT  DATE     DESCRIPTION                                     */
/*  26857   20230628 Create QUDM, driver for RTMC User Defined       */
/*                   Metrics (UDM)                                   */
/*                                                                   */
/*                                                                   */
/*** END OF CHANGE HISTORY********************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  QUDM                                               */
/*                                                                   */
/* Abstract:      Main segment for RTMC User Defined Metrics (UDM)   */
/*                driver.                                            */
/*                                                                   */
/* Description:                                                      */
/*   ZTEST QUDM START RATE-xxxx TOKEN-yyy                            */
/*         This starts sending the sawtooth data (see below) with a  */
/*         datatype of QUDM.                                         */
/*         xxxx = the number of times to send the sawtooth data per  */
/*                second.                                            */
/*         yyy  = a unique value like 001 or ABC.                    */
/*                This cannot be ALL (any case).                     */
/*         A 3-entry array of data is created. Each array            */
/*         entry contains 4 elements: a group name, a millisecond    */
/*         value, a microsecond value and a fixed value.             */
/*         - The first entry in the array has a group name of mils,  */
/*           a millisecond value based on the current timestamp, a   */
/*           microsecond value based on the current timestamp and a  */
/*           fixed value of 42.                                      */
/*         - The second entry in the array has a group name of       */
/*           plus1000, a millisecond value based on the current      */
/*           timestamp plus 1000, a microsecond value based on the   */
/*           current timestamp plus 1000 and a fixed value of 777.   */
/*         - The third entry in the array has a group name of        */
/*           plus2000, a millisecond value based on the current      */
/*           timestamp plus 2000, a microsecond value based on the   */
/*           current timestamp plus 2000 and a fixed value of 1337.  */
/*         Because of the increase in values of each element, the    */
/*         data can be thought of as a sawtooth.                     */
/*                                                                   */
/*   ZTEST QUDM STOP TOKEN-yyy|ALL                                   */
/*         This stops sending the sawtooth data (see above).         */
/*         yyy  = the token value specified on the START command     */
/*         ALL  = stop all token values that are running.            */
/*                                                                   */
/*   ZTEST QUDM JSON DATATYPE-datatype FILE-jsonfile PACKAGES-n      */
/*         This is used to send user data that is already in JSON    */
/*         format.                                                   */
/*         Any DATATYPE can be specified except QUDM or QMWR.        */
/*         The jsonfile contains data array entries. Each entry must */
/*         start with { and end with }. For example:                 */
/*    {"groupName":"name1","metric1":51,"metric2":298,"metric3":402} */
/*         There can be 1 entry or many entries. The entries must be */
/*         comma separated.                                          */
/*         The PACKAGES parameter indicates how many times to        */
/*         include the data in the jsonfile before the total data    */
/*         is sent.                                                  */
/*         The default PACKAGES is 1.                                */
/*********************************************************************/

/**********************************************************************/
/* Defines and Includes                                               */
/**********************************************************************/

#define QUDM_MESSAGE_PREFIX "QUDM" // For output messages

#include <qudm.hpp>
#include <tpf/tpfparse.h>
#include <tpf/tpfapi.h>
#include <tpf/sysapi.h>
#include <tpf/reusable.h>
#include <tpf/c_uatbc.h>
#include <ctype.h>

/**********************************************************************/
/* Parser grammar for the command.                                    */
/**********************************************************************/

#define QUDM_GRAMMAR                                                                \
IPRSE_STRICT_GRAMMAR IPRSE_MIXED_CASE_GRAMMAR                                       \
  "{ "                                                                              \
     "STArt Rate-n+++ Token-aaa "                                                   \
  "| "                                                                              \
     "STOp Token-aaa "                                                              \
  "| "                                                                              \
     "JSON File-u* "                                                                \
     "DATatype-a+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ "   \
     "[PACKages-d++++++ ]"                                                          \
  "} "

/*                                                        */
/* Following #defines must match qudm_function[] entries  */
/*                                                        */
#define START        0
#define STOP         1
#define JSON         2
#define NOT_HERE     3

/**********************************************************************/
/* Functions                                                          */
/**********************************************************************/

void  PrintHelp(void);
int   SendUDMData(struct tpf_rtmc_user_defined_metrics_parms *);
void  QUDM_parse();
void  QUDM_start(int, char *);
void  QUDM_stop(char *);
void  QUDM_JSON(char *, char *, int);
void  QUDM_send_sawtooth(char *);
void  manipulateTempData(TPF_UDM_SAMPLE_DATA_LAYOUT *);

/********************************************************************/
/* QUDM -- Initial function for the shared object.                  */
/*                                                                  */
/*   Input:  void                                                   */
/*   Output: void                                                   */
/*                                                                  */
/* This function is entered the following ways:                     */
/* - When the ZTEST QUDM command is entered.                        */
/* - Every second when ZTEST QUDM START is running and sending      */
/*   sawtooth data.                                                 */
/*                                                                  */
/* If entered as a result of the ZTEST QUDM command then the        */
/* function to do the parsing is called.                            */
/* If entered to send the next second of sawtooth data then the     */
/* function to build and send the sawtooth data is called.          */
/********************************************************************/
extern "C" void QUDM(void)
{
    struct swisc_immediate_input swisc_is;
    char qudmtoken[3];
    unsigned int charac = TPF_NAMEVALUE_NO_CHAR;

    /****************************************************************/
    /* Set the MsgType name-value pair for the ECB.                 */
    /****************************************************************/
    tpf_nameValueLocalSet("MsgType",
                          (void *) "IQUDMDriver",
                          strlen("IQUDMDriver"),
                          &charac);

    if(levtest(D0))
    {
      /**************************************************************/
      /* If there is a block on data level 0 then it is the input   */
      /* message block containing the command. Call the parse       */
      /* function.                                                  */
      /**************************************************************/
      QUDM_parse();
    } // end if there is a block containing the command on data level 0

    else
    {
      /**************************************************************/
      /* Called to send the next iteration of QUDM data.            */
      /* Make sure the continuous activity is balanced across       */
      /* I-streams. In order to balance, switch I-streams and let   */
      /* the tightly coupled scheduler select the I-stream by       */
      /* specifying an I-stream number of 0.                        */
      /* The continuous data token is in EBW000-EBW002. Call the    */
      /* function to send the next iteration of QUDM data for the   */
      /* token.                                                     */
      /**************************************************************/
      swisc_is.istream = 0;
      swisc_immediate(&swisc_is);
      memcpy(qudmtoken,&(ecbptr()->ebw000),sizeof(qudmtoken));
      QUDM_send_sawtooth(qudmtoken);
    } // end else entered for continuous QUDM data

    exit(0);

} // end QUDM

/********************************************************************/
/* QUDM_parse -- Parse the input message and call the appropriate   */
/*               function.                                          */
/*                                                                  */
/*   Input:  void                                                   */
/*   Output: void                                                   */
/*                                                                  */
/********************************************************************/
void QUDM_parse()
{
  char wtopc_header[sizeof(struct wtopc_header)];
  struct mi0mi * command_ptr;
  char * parms_in_command_ptr;
  int  numParms;
  int  packages = 1;
  struct IPRSE_output  parsed_data;
  char jsonfile[1024] = {0};
  char data_type[65] = {0};
  int task_code = 0;
  boolean found;
  int rate = 0;
  char token[3];

  /********************************************************************/
  /* qudm_function used to manipulate result of parsed output as      */
  /* integral type in order to use a switch statement.                */
  /********************************************************************/
  char * qudm_function[] = { "STArt",
                             "STOp",
                             "JSON"
                           };

  /****************************************************************/
  /* CVZZ already parsed ZTEST QUDM and changed the command on    */
  /* data level D0 to ZQUDM.                                      */
  /****************************************************************/
  command_ptr = (struct mi0mi *) ecbptr()->ce1cr0;   // command_ptr -> mi0mi struct

  *((char *)&command_ptr->mi0ln0 + command_ptr->mi0cct - 1) = '\0';
  parms_in_command_ptr = command_ptr->mi0acc + strlen("ZQUDM");

  /**************************************************************/
  /* Get a pointer to the input message in mixed case; if it is */
  /* not there, keep pointing to the upper case input message   */
  /* from data level D0.                                        */
  /* The mixed case input message is in the form ZTEST QUDM.    */
  /**************************************************************/
  if (ecbp2()->ce2crsmsg != NULL)
  {
    parms_in_command_ptr = ecbp2()->ce2crsmsg;
    parms_in_command_ptr += strlen("ZTEST QUDM");
  } // end if there is a mixed case version of the input message

  numParms = IPRSE_parse(parms_in_command_ptr, // parameters to parse
                         QUDM_GRAMMAR,         // grammar
                         &parsed_data,         // parsed tokens (linked list)
                         IPRSE_STRICT+         // only spaces are token separators
                         IPRSE_PRINT+          // print all errors +
                         IPRSE_ALLOC+          // get storage for return struct
                         IPRSE_MIXED_CASE,     // accept lower or upper case
                         "QUDM");              // message prefix in case of error

  switch(numParms)
  {
    case IPRSE_HELP:   // explicit help request       or
    case IPRSE_BAD:    // syntax error
      PrintHelp();     // try to help the user
      break;           // from switch

    default:
      /**********************************************************/
      /* The parser did not detect any errors.                  */
      /* Determine which QUDM function is requested.            */
      /**********************************************************/

      found = FALSE;
      while ((found == FALSE) && (task_code < NOT_HERE))
      {
        if (strcmp(parsed_data.IPRSE_parameter,
                   qudm_function[task_code]) == 0)
        {
          found = TRUE;
        }
        else
        {
          task_code++;
        }
      } // end while

      /**********************************************************/
      /* Process the parameters for the QUDM function.          */
      /* Keyword parameters can be in any order in the IPRSE    */
      /* return structure.                                      */
      /**********************************************************/
      numParms--;

      switch(task_code)
      {
        /*----------------------------------------------------------*/
        /*         START                                            */
        /*----------------------------------------------------------*/
        case START:
        {
           while(numParms > 0)
           {
              parsed_data = *(parsed_data.IPRSE_next);
              if (memcmp(parsed_data.IPRSE_parameter, "Rate", strlen("Rate")) == 0)
              {
                 rate = atoi(parsed_data.IPRSE_value);
                 if (rate == 0)
                 {
                    wtopc_insert_header(wtopc_header,
                                        QUDM_MESSAGE_PREFIX,
                                        27,'E',
                                        WTOPC_SUBSYS_TIME);
                    wtopc("A RATE OF ZERO IS NOT VALID.",
                          WTOPC_EBROUT,
                          WTOPC_NO_CHAIN,
                          wtopc_header);
                    return;
                 } // end if rate of zero specified
              } // end if RATE parameter

              else
              {
                 memcpy(token, parsed_data.IPRSE_value, sizeof(token));
              } // end else TOKEN parameter

              numParms--;
           } // end while there are more parameters to parse

           QUDM_start(rate, token);

           break;
        } // end case START

        /*----------------------------------------------------------*/
        /*         STOP                                             */
        /*----------------------------------------------------------*/
        case STOP:
        {
           parsed_data = *(parsed_data.IPRSE_next);
           memcpy(token, parsed_data.IPRSE_value, sizeof(token));

           QUDM_stop(token);

           break;
        } // end case STOP

        /*----------------------------------------------------------*/
        /*         JSON                                             */
        /*----------------------------------------------------------*/
        case JSON:
        {
           while(numParms > 0)
           {
              parsed_data = *(parsed_data.IPRSE_next);
              if (memcmp(parsed_data.IPRSE_parameter, "File", strlen("File")) == 0)
              {

                 if (strlen(parsed_data.IPRSE_value) < 1024)
                 {
                    strcpy(jsonfile, parsed_data.IPRSE_value);
                 } // end if JSON file name is not too long

                 else
                 {
                    wtopc_insert_header(wtopc_header,
                                        QUDM_MESSAGE_PREFIX,
                                        14,'E',
                                        WTOPC_SUBSYS_TIME);
                    wtopc("THE JSON FILE NAME SPECIFIED IS TOO LONG.",
                          WTOPC_EBROUT,
                          WTOPC_NO_CHAIN,
                          wtopc_header);
                    return;
                 } // end else JSON file name is too long

              } // end if FILE parameter

              else if (memcmp(parsed_data.IPRSE_parameter,"DATatype", strlen("DATatype")) == 0)
              {
                 strcpy(data_type, parsed_data.IPRSE_value);
              } // end else if DATATYPE parameter

              else if (memcmp(parsed_data.IPRSE_parameter,"PACKages", strlen("PACKages")) == 0)
              {
                 packages = atoi(parsed_data.IPRSE_value);
                 //todo - check for 0
              } // end else if PACKAGES parameter

              numParms--;

           } // end while there are more parameters to parse

           QUDM_JSON(data_type, jsonfile, packages);

           break;
        } // end case JSON

      } // end switch on QUDM function

  } // end switch on IPRSE result

  return;

} // end QUDM_parse

/********************************************************************/
/* QUDM_start -- start an instance of continuously running sawtooth */
/*               data.                                              */
/*                                                                  */
/*   Input:  rate = the number of times to send the sawtooth data   */
/*                  per second                                      */
/*           token_ptr = pointer to a unique 3-character token that */
/*                  identifies the instance (like 001 or ABC)       */
/*   Output: void                                                   */
/*                                                                  */
/* The instances are kept in a table in system heap. The tables are */
/* subsystem unique.                                                */
/********************************************************************/
void QUDM_start(int rate, char * token_ptr)
{
   char sys_heap_token[ITOKENLEN] = "IQUDM   ";
   char owner[IOWNERLEN] = "drvrQUDM                        ";
   struct uatbc_area uatbc_parm;
   struct TPF_QUDM_TABLE * qudm_table_ptr;
   unsigned int i;
   signed int available_entry = -1;
   char wtopc_header[sizeof(struct wtopc_header)];
   char wtopc_buffer[255];
   char upper_token[3];
   char * copy_token_ptr;

   /*****************************************************************/
   /* A token of ALL (any case) is not allowed. It is a reserved    */
   /* parameter for the STOP parameter.                             */
   /*****************************************************************/
   copy_token_ptr = token_ptr;
   for (i=0; i<sizeof(upper_token); i++)
   {
      upper_token[i]=toupper(*copy_token_ptr);
      copy_token_ptr++;
   } // end for each character in the token

   if (memcmp(upper_token,"ALL",sizeof(upper_token)) == 0)
   {
         wtopc_insert_header(wtopc_header,
                             QUDM_MESSAGE_PREFIX,
                             31,'E',
                             WTOPC_SUBSYS_TIME);
         wtopc("QUDM DRIVER NOT STARTED. THE TOKEN ALL IS NOT ALLOWED.",
               WTOPC_EBROUT,
               WTOPC_NO_CHAIN,
               wtopc_header);
         return;
   } // end if a token of ALL was specified


   /*****************************************************************/
   /* Build the system heap token and owner name for this subsystem.*/
   /*****************************************************************/
   uatbc_parm.uat_call = UAT_ESSU;
   uatbc(&uatbc_parm);
   memcpy(sys_heap_token+strlen("IQUDM"),
          &uatbc_parm.uat_ss_name,
          3);
   memcpy(owner+strlen("drvrQUDM"),
          &uatbc_parm.uat_ss_name,
          sizeof(uatbc_parm.uat_ss_name));

   /*****************************************************************/
   /* Check if the QUDM table for this subsystem already exists.    */
   /*****************************************************************/
   qudm_table_ptr = (struct TPF_QUDM_TABLE *) tpf_fsysc(sys_heap_token, NULL);
   if (qudm_table_ptr == 0)
   {
      /**************************************************************/
      /* The QUDM table for this subsystem does not already exist.  */
      /* Allocate the system heap for the table and initialize it.  */
      /**************************************************************/
      qudm_table_ptr = (struct TPF_QUDM_TABLE *) tpf_gsysc(1, sys_heap_token, owner,
                                                           GSYSC_64BIT+GSYSC_UNIQUE);
      if (qudm_table_ptr == 0)
      {
         wtopc_insert_header(wtopc_header,
                             QUDM_MESSAGE_PREFIX,
                             13,'E',
                             WTOPC_SUBSYS_TIME);
         wtopc("QUDM DRIVER NOT STARTED. UNABLE TO OBTAIN SYSTEM HEAP.",
               WTOPC_EBROUT,
               WTOPC_NO_CHAIN,
               wtopc_header);
         return;
      } // end if unable to allocate system heap for the table

   } // end if system heap for this subsystem does not already exist

   /*****************************************************************/
   /* The locking protocol for the table is to do a core hold on    */
   /* the system heap address.                                      */
   /* Initialize the table if needed.                               */
   /* Check if the table is full.                                   */
   /* If not, check if this token already exists in the table.      */
   /*****************************************************************/
   corhc(qudm_table_ptr);

   if (memcmp(qudm_table_ptr->header.eyecatcher,
              TPF_QUDM_EYECATCHER,
              sizeof(qudm_table_ptr->header.eyecatcher)) != 0)
   {
      tpf_stpoc(TPF_STPOC_ON, NULL);
      memset(qudm_table_ptr, '\0', sizeof(struct TPF_QUDM_TABLE));
      memcpy(qudm_table_ptr->header.eyecatcher,
             TPF_QUDM_EYECATCHER,
             sizeof(qudm_table_ptr->header.eyecatcher));
      qudm_table_ptr->header.count = 0;
      tpf_stpoc(TPF_STPOC_OFF, NULL);
      available_entry = 0;
   } // end if table is not initialized

   else if (qudm_table_ptr->header.count == TPF_QUDM_MAX_ENTRIES)
   {
      coruc(qudm_table_ptr);
      wtopc_insert_header(wtopc_header,
                          QUDM_MESSAGE_PREFIX,
                          23,'E',
                          WTOPC_SUBSYS_TIME);
      wtopc("QUDM DRIVER NOT STARTED. ALREADY AT MAXIMUM NUMBER OF INSTANCES.",
            WTOPC_EBROUT,
            WTOPC_NO_CHAIN,
            wtopc_header);
      return;
   } // end else if table is full

   else
   {
      for (i = 0; i < TPF_QUDM_MAX_ENTRIES; i++)
      {
         if (memcmp(qudm_table_ptr->entry[i].token,
                     token_ptr,
                     sizeof(qudm_table_ptr->entry[i].token)) == 0)
         {
            coruc(qudm_table_ptr);
            wtopc_insert_header(wtopc_header,
                                QUDM_MESSAGE_PREFIX,
                                24,'E',
                                WTOPC_SUBSYS_TIME);
            wtopc("QUDM DRIVER NOT STARTED. AN INSTANCE OF THE DRIVER ALREADY EXISTS WITH THE TOKEN SPECIFIED.",
                  WTOPC_EBROUT,
                  WTOPC_NO_CHAIN,
                  wtopc_header);
            return;
         } // end if the token already exists in the table

         /***********************************************************/
         /* If the entry is not in use, save the index to be used   */
         /* later.                                                  */
         /***********************************************************/
         else if ((memcmp(qudm_table_ptr->entry[i].token,
                  "\0\0\0",
                  sizeof(qudm_table_ptr->entry[i].token)) == 0) &&
                  (available_entry == -1))
         {
            available_entry = i;
         } //end else if entry is available

      } // end for all entries in the table

   } // end else table is initialized and not full

   /*****************************************************************/
   /* The token does not already exist in the table. Add the new    */
   /* entry.                                                        */
   /*****************************************************************/
   tpf_stpoc(TPF_STPOC_ON, NULL);
   qudm_table_ptr->header.count++;
   memcpy(qudm_table_ptr->entry[available_entry].token,
                 token_ptr,
                 sizeof(qudm_table_ptr->entry[available_entry].token));
   qudm_table_ptr->entry[available_entry].rate = rate;
   qudm_table_ptr->entry[available_entry].status = TPF_QUDM_STARTED;
   tpf_stpoc(TPF_STPOC_OFF, NULL);

   coruc(qudm_table_ptr);

   /*****************************************************************/
   /* Start the continuous activity.                                */
   /*****************************************************************/
   cretc(CRETC_SECONDS,QUDM,1,token_ptr);
   sprintf(wtopc_buffer,"QUDM DRIVER STARTED WITH TOKEN %3.3s.", token_ptr);
   wtopc_insert_header(wtopc_header,
                       QUDM_MESSAGE_PREFIX,
                       25,'I',
                       WTOPC_SUBSYS_TIME);
   wtopc(wtopc_buffer,
         WTOPC_EBROUT,
         WTOPC_NO_CHAIN,
         wtopc_header);

   return;
} // end QUDM_start

/********************************************************************/
/* QUDM_stop -- stop one instance or all instances of continuously  */
/*              running sawtooth data.                              */
/*                                                                  */
/*   Input:  token_ptr = pointer to a unique 3-character token that */
/*                  identifies the instance (like 001 or ABC)       */
/*                  -or-                                            */
/*                  ALL to stop all instances                       */
/*   Output: void                                                   */
/*                                                                  */
/* The entry in the QUDM table is marked stopped. If there are no   */
/* more ECBs running this instance of the driver then the QUDM      */
/* table entry is made available. If there are still ECBs running   */
/* this instance of the driver then they'll recognize the instance  */
/* has been stopped and no new ECBs will be created. The last ECB   */
/* to exit will make the QUDM table entry available.                */
/********************************************************************/
void  QUDM_stop(char * token_ptr)
{
  char sys_heap_token[ITOKENLEN] = "IQUDM   ";
  struct uatbc_area uatbc_parm;
  struct TPF_QUDM_TABLE * qudm_table_ptr;
  unsigned int  i;
  boolean found, all_specified = FALSE;
  char wtopc_header[sizeof(struct wtopc_header)];
  char wtopc_buffer[255];
  char upper_token[3];
  char * copy_token_ptr;

  /*****************************************************************/
  /* Determine if the ALL parameter was specified.                 */
  /*****************************************************************/
  copy_token_ptr = token_ptr;
  for (i=0; i<sizeof(upper_token); i++)
  {
     upper_token[i]=toupper(*copy_token_ptr);
     copy_token_ptr++;
  } // end for each character in the token

  if (memcmp(upper_token, "ALL", sizeof(upper_token)) == 0)
  {
     all_specified = TRUE;
  } // end if ALL parameter specified

  /*****************************************************************/
  /* Build the system heap token for this subsystem.               */
  /*****************************************************************/
  uatbc_parm.uat_call = UAT_ESSU;
  uatbc(&uatbc_parm);
  memcpy(sys_heap_token+strlen("IQUDM"),
         &uatbc_parm.uat_ss_name,
         3);

  /*****************************************************************/
  /* Find the QUDM table for this subsystem.                       */
  /*****************************************************************/
  qudm_table_ptr = (struct TPF_QUDM_TABLE *) tpf_fsysc(sys_heap_token, NULL);
  if (qudm_table_ptr == 0)
  {
     /**************************************************************/
     /* The QUDM table for this subsystem does not exist.          */
     /**************************************************************/
     wtopc_insert_header(wtopc_header,
                         QUDM_MESSAGE_PREFIX,
                         30,'E',
                         WTOPC_SUBSYS_TIME);
     wtopc("NO INSTANCES OF THE QUDM DRIVER ARE STARTED.",
           WTOPC_EBROUT,
           WTOPC_NO_CHAIN,
           wtopc_header);
     return;
  } // end if QUDM table does not exist

  /*****************************************************************/
  /* The locking protocol for the table is to do a core hold on    */
  /* the system heap address.                                      */
  /*****************************************************************/
  corhc(qudm_table_ptr);

  if (all_specified == FALSE)
  {
     /**************************************************************/
     /* Find the entry in the QUDM table for the token passed as   */
     /* input.                                                     */
     /**************************************************************/
     for (i = 0; i < TPF_QUDM_MAX_ENTRIES; i++)
     {
        if (memcmp(qudm_table_ptr->entry[i].token,
                   token_ptr,
                   sizeof(qudm_table_ptr->entry[i].token)) == 0)
        {
           found = TRUE;
           break;
        }

     } // end for all entries in the table

     if (found == FALSE)
     {
        /***********************************************************/
        /* The driver for this token does not exist.               */
        /***********************************************************/
        coruc(qudm_table_ptr);
        sprintf(wtopc_buffer,"QUDM DRIVER WITH TOKEN %3.3s IS NOT STARTED.",
                token_ptr);
        wtopc_insert_header(wtopc_header,
                            QUDM_MESSAGE_PREFIX,
                            28,'E',
                            WTOPC_SUBSYS_TIME);
        wtopc(wtopc_buffer,
              WTOPC_EBROUT,
              WTOPC_NO_CHAIN,
              wtopc_header);
        return;
     } // end if token is not in the table

     else if (qudm_table_ptr->entry[i].status != TPF_QUDM_STARTED)
     {
        /***********************************************************/
        /* The driver for this token is not started.               */
        /* If there are no more ECBs running this instance of the  */
        /* driver then remove the entry from the QUDM table.       */
        /***********************************************************/
        if (qudm_table_ptr->entry[i].ecb_count == 0)
        {
           tpf_stpoc(TPF_STPOC_ON, NULL);
           qudm_table_ptr->header.count--;
           memset(&(qudm_table_ptr->entry[i]), '\0',
                  sizeof(struct TPF_QUDM_ENTRY));
           tpf_stpoc(TPF_STPOC_OFF, NULL);
        } // end if no remaining ECBs for this instance of the driver

        coruc(qudm_table_ptr);
        sprintf(wtopc_buffer,"QUDM DRIVER WITH TOKEN %3.3s IS NOT STARTED.",
                token_ptr);
        wtopc_insert_header(wtopc_header,
                            QUDM_MESSAGE_PREFIX,
                            28,'E',
                            WTOPC_SUBSYS_TIME);
        wtopc(wtopc_buffer,
              WTOPC_EBROUT,
              WTOPC_NO_CHAIN,
              wtopc_header);
        return;
     } // end else if driver not started

     tpf_stpoc(TPF_STPOC_ON, NULL);
     qudm_table_ptr->entry[i].status = TPF_QUDM_STOPPED;

     if (qudm_table_ptr->entry[i].ecb_count == 0)
     {
        /***********************************************************/
        /* There are no more ECBs running this instance of the     */
        /* driver. Remove the entry from the QUDM table.           */
        /***********************************************************/
        qudm_table_ptr->header.count--;
        memset(&(qudm_table_ptr->entry[i]), '\0',
               sizeof(struct TPF_QUDM_ENTRY));
     } // end if no remaining ECBs for this instance of the driver

     tpf_stpoc(TPF_STPOC_OFF, NULL);

     coruc(qudm_table_ptr);

     sprintf(wtopc_buffer,"QUDM DRIVER WITH TOKEN %3.3s STOPPED.", token_ptr);
     wtopc_insert_header(wtopc_header,
                         QUDM_MESSAGE_PREFIX,
                         29,'I',
                         WTOPC_SUBSYS_TIME);
     wtopc(wtopc_buffer,
           WTOPC_EBROUT,
           WTOPC_NO_CHAIN,
           wtopc_header);

  } // end if ALL parameter was not specified

  else
  {
     /**************************************************************/
     /* Loop through entries in the QUDM table                     */
     /**************************************************************/
     for (i = 0; i < TPF_QUDM_MAX_ENTRIES; i++)
     {
        if (qudm_table_ptr->entry[i].status == TPF_QUDM_STARTED)
        {
           tpf_stpoc(TPF_STPOC_ON, NULL);
           qudm_table_ptr->entry[i].status = TPF_QUDM_STOPPED;

           if (qudm_table_ptr->entry[i].ecb_count == 0)
           {
              /*****************************************************/
              /* There are no more ECBs running this instance of   */
              /* the driver. Remove the entry from the QUDM table. */
              /*****************************************************/
              qudm_table_ptr->header.count--;
              memset(&(qudm_table_ptr->entry[i]), '\0',
                     sizeof(struct TPF_QUDM_ENTRY));
           } // end if no remaining ECBs for this instance

           tpf_stpoc(TPF_STPOC_OFF, NULL);

        } // end if the entry is started

     } // end for all entries in the table

     coruc(qudm_table_ptr);

     sprintf(wtopc_buffer,"ALL INSTANCES OF THE QUDM DRIVER ARE STOPPED.");
     wtopc_insert_header(wtopc_header,
                         QUDM_MESSAGE_PREFIX,
                         32,'I',
                         WTOPC_SUBSYS_TIME);
     wtopc(wtopc_buffer,
           WTOPC_EBROUT,
           WTOPC_NO_CHAIN,
           wtopc_header);

  } // end else ALL parameter was specified

  return;
} // end QUDM_stop

/********************************************************************/
/* QUDM_send_sawtooth -- Build and send the next second of sawtooth */
/*                       data.                                      */
/*                                                                  */
/*   Input:  token_ptr = pointer to a unique 3-character token that */
/*                  identifies the instance (like 001 or ABC)       */
/*                                                                  */
/*   Output: void                                                   */
/*                                                                  */
/* The entry in the QUDM table in system heap is located for the    */
/* token passed as input. If the iteration of the driver for the    */
/* token hasn't been stopped, then loop on the rate per second and  */
/* build and send the sawtooth data. Kick off another ECB to run    */
/* the driver for the next second.                                  */
/********************************************************************/
void  QUDM_send_sawtooth(char * token_ptr)
{
  char sys_heap_token[ITOKENLEN] = "IQUDM   ";
  struct uatbc_area uatbc_parm;
  struct TPF_QUDM_TABLE * qudm_table_ptr;
  unsigned int i;
  int  n, rc, rate, sleep_interval;
  boolean found = FALSE;
  char wtopc_header[sizeof(struct wtopc_header)];
  char wtopc_buffer[255];
  struct tpf_rtmc_user_defined_metrics_parms send_input_parms = {};
  char * dfdl_sample_file = "tpf_udm_sample_data.dfdl.xsd";
  // Allocate the largest possible message on the stack
  char tempBuffer[sizeof(TPF_UDM_SAMPLE_DATA_LAYOUT) + sizeof(TPF_UDM_SAMPLE_DATA_STRUCT) * 3] = {0};
  TPF_UDM_SAMPLE_DATA_LAYOUT * my_udm_data = (TPF_UDM_SAMPLE_DATA_LAYOUT *) ((char*)tempBuffer);

  /*****************************************************************/
  /* Build the system heap token for this subsystem.               */
  /*****************************************************************/
  uatbc_parm.uat_call = UAT_ESSU;
  uatbc(&uatbc_parm);
  memcpy(sys_heap_token+strlen("IQUDM"),
         &uatbc_parm.uat_ss_name,
         3);

  /*****************************************************************/
  /* Find the QUDM table for this subsystem.                       */
  /*****************************************************************/
  qudm_table_ptr = (struct TPF_QUDM_TABLE *) tpf_fsysc(sys_heap_token, NULL);
  if (qudm_table_ptr == 0)
  {
     /**************************************************************/
     /* The QUDM table for this subsystem no longer exists.        */
     /* Just exit.                                                 */
     /**************************************************************/
     exit(0);
  } // end if QUDM table no longer exists

  corhc(qudm_table_ptr);

  /*****************************************************************/
  /* Find the entry in the QUDM table for the token passed as      */
  /* input.                                                        */
  /*****************************************************************/
  for (i = 0; i < TPF_QUDM_MAX_ENTRIES; i++)
  {
     if (memcmp(qudm_table_ptr->entry[i].token,
                token_ptr,
                sizeof(qudm_table_ptr->entry[i].token)) == 0)
     {
        found = TRUE;
        break;
     }

  } // end for all entries in the table

  if (found == FALSE)
  {
     /*************************************************************/
     /* The token no longer exists in the table, just exit.       */
     /*************************************************************/
     coruc(qudm_table_ptr);
     exit(0);
  } // end if token is not in the table

  if (qudm_table_ptr->entry[i].status != TPF_QUDM_STARTED)
  {
     /***********************************************************/
     /* The driver for this token has been stopped.             */
     /* If there are no more ECBs running this instance of the  */
     /* driver then remove the entry from the QUDM table.       */
     /***********************************************************/
     if (qudm_table_ptr->entry[i].ecb_count == 0)
     {
        tpf_stpoc(TPF_STPOC_ON, NULL);
        qudm_table_ptr->header.count--;
        memset(&(qudm_table_ptr->entry[i]), '\0',
               sizeof(struct TPF_QUDM_ENTRY));
        tpf_stpoc(TPF_STPOC_OFF, NULL);
     } // end if no remaining ECBs for this instance of the driver

     coruc(qudm_table_ptr);
     exit(0);
  } // end if this instance of the driver is no longer started

  /****************************************************************/
  /* Create another ECB to run this instance of the driver in 1   */
  /* second.                                                      */
  /* Increment the number of ECBs running this instance of the    */
  /* driver.                                                      */
  /* Unhold the QUDM table while sending the data.                */
  /****************************************************************/
  cretc(CRETC_SECONDS,QUDM,1,token_ptr);

  tpf_stpoc(TPF_STPOC_ON, NULL);
  qudm_table_ptr->entry[i].ecb_count++;
  tpf_stpoc(TPF_STPOC_OFF, NULL);

  coruc(qudm_table_ptr);

  /****************************************************************/
  /* Based on the rate in the table, calculate how many           */
  /* microseconds to sleep in between each send of the data. This */
  /* will spread the sends evenly over this second.               */
  /* In a loop, build the QUDM data, convert it to JSON and send  */
  /* the data.                                                    */
  /****************************************************************/
  rate = qudm_table_ptr->entry[i].rate;
  sleep_interval = 1000000 / rate;

  send_input_parms.parms_version = TPF_RTMC_UDM_PARM_VER_1;
  strcpy(send_input_parms.data_type, "QUDM");

  for (n=1; n<=rate; n++)
  {
     manipulateTempData(my_udm_data);
     char * jsonBuffer = convertStructureToJson(tempBuffer,
                                                (int) sizeof(tempBuffer),
                                                dfdl_sample_file);
     if ( jsonBuffer != NULL )
     {
        /**********************************************************/
        /* The data converted successfully. Set up and send the   */
        /* data.                                                  */
        /**********************************************************/
        send_input_parms.data = (void *)jsonBuffer;
        send_input_parms.data_size = strlen(jsonBuffer);

        rc = SendUDMData(&send_input_parms);

        free(jsonBuffer);

        if (rc != TPF_RTMC_UDM_OK)
        {
           /*******************************************************/
           /* There was an error sending the data. Indicate this  */
           /* iteration of the driver is stopping.                */
           /*******************************************************/
           sprintf(wtopc_buffer,"STOPPING QUDM DRIVER WITH TOKEN %3.3s.",
                   qudm_table_ptr->entry[i].token);
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               26,'E',
                               WTOPC_SUBSYS_TIME);
           wtopc(wtopc_buffer,
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);

           corhc(qudm_table_ptr);
           tpf_stpoc(TPF_STPOC_ON, NULL);
           qudm_table_ptr->entry[i].status = TPF_QUDM_STOPPED;
           qudm_table_ptr->entry[i].ecb_count--;
           if (qudm_table_ptr->entry[i].ecb_count == 0)
           {
              /****************************************************/
              /* No other ECBs are running this instance of the   */
              /* driver. This entry in the QUDM table can be      */
              /* removed.                                         */
              /****************************************************/
              qudm_table_ptr->header.count--;
              memset(&(qudm_table_ptr->entry[i]), '\0',
                     sizeof(struct TPF_QUDM_ENTRY));
           } // end if no remaining ECBs for this instance of the driver

           tpf_stpoc(TPF_STPOC_OFF, NULL);
           coruc(qudm_table_ptr);

           exit(0);

        } // end if the data could not be sent

     } // end if data converted successfully

     if (n != rate)
     {
        usleep(sleep_interval);
     } // end if not at last send

  } // end for the rate to send this second

  corhc(qudm_table_ptr);
  tpf_stpoc(TPF_STPOC_ON, NULL);
  qudm_table_ptr->entry[i].ecb_count--;

  if (qudm_table_ptr->entry[i].status != TPF_QUDM_STARTED)
  {
     /***********************************************************/
     /* The driver for this token has been stopped.             */
     /* If there are no more ECBs running this instance of the  */
     /* driver then remove the entry from the QUDM table.       */
     /***********************************************************/
     if (qudm_table_ptr->entry[i].ecb_count == 0)
     {
        qudm_table_ptr->header.count--;
        memset(&(qudm_table_ptr->entry[i]), '\0',
               sizeof(struct TPF_QUDM_ENTRY));
     } // end if no remaining ECBs for this instance of the driver

  } // end if this instance of the driver has been stopped

  tpf_stpoc(TPF_STPOC_OFF, NULL);
  coruc(qudm_table_ptr);
  exit(0);

} // end QUDM_send_sawtooth

/********************************************************************/
/* manipulateTempData -- Build the next second of sawtooth data.    */
/*                                                                  */
/*   Input:  udm_payload = pointer to the QUDM data to be filled    */
/*                         in.                                      */
/*                                                                  */
/*   Output: void                                                   */
/*                                                                  */
/* A 3-entry array of data is created. Each array entry contains 4  */
/* elements: a group name, a millisecond value, a microsecond value */
/* and a fixed value.                                               */
/* - The first entry in the array has a group name of mils, a       */
/*   millisecond value based on the current timestamp, a microsecond*/
/*   value based on the current timestamp and a fixed value of 42.  */
/* - The second entry in the array has a group name of plus1000, a  */
/*   millisecond value based on the current timestamp plus 1000, a  */
/*   microsecond value based on the current timestamp plus 1000 and */
/*   a fixed value of 777.                                          */
/* - The third entry in the array has a group name of plus2000, a   */
/*   millisecond value based on the current timestamp plus 2000, a  */
/*   microsecond value based on the current timestamp plus 2000 and */
/*   a fixed value of 1337.                                         */
/********************************************************************/
void manipulateTempData(TPF_UDM_SAMPLE_DATA_LAYOUT * udm_payload) {

    struct timeval timeValue;
    gettimeofday(&timeValue, NULL);
    time_t curr_time = timeValue.tv_sec;
    struct tm *clock_val = gmtime((const time_t *) &curr_time);
    char buffer[32] = {0};
    strftime(buffer, sizeof(buffer), "%Y-%m-%d %H:%M:%S", clock_val);
    char timestamp[32] = {0};
    snprintf(timestamp,sizeof(timestamp),"%s.%3.3ld",buffer,(timeValue.tv_usec / 1000));

    // Define header values
    strncpy(udm_payload->header.timestamp,timestamp,sizeof(udm_payload->header.timestamp));
    udm_payload->header.version = 1;
    udm_payload->header.count = 3;
    udm_payload->header.intervalLength = 1000;

    // Milliseconds/microseconds/fixed value
    strncpy(udm_payload->data_array[0].groupName,"mils",sizeof(udm_payload->data_array[0].groupName));
    udm_payload->data_array[0].metric1 = (clock_val->tm_sec * 1000 + timeValue.tv_usec / 1000) + 5000;  // milliseconds
    udm_payload->data_array[0].metric2 = timeValue.tv_usec % 1000;                            // microseconds
    udm_payload->data_array[0].metric3 = 42;                                                  // fixed values

    // +1,000
    strncpy(udm_payload->data_array[1].groupName,"plus1000",sizeof(udm_payload->data_array[1].groupName));
    udm_payload->data_array[1].metric1 = (clock_val->tm_sec * 1000 + timeValue.tv_usec / 1000) + 5000 + 1000;
    udm_payload->data_array[1].metric2 = (timeValue.tv_usec % 1000) + 1000;
    udm_payload->data_array[1].metric3 = 777;

    // +2,000
    strncpy(udm_payload->data_array[2].groupName,"plus2000",sizeof(udm_payload->data_array[2].groupName));
    udm_payload->data_array[2].metric1 = (clock_val->tm_sec * 1000 + timeValue.tv_usec / 1000) + 5000 + 2000;
    udm_payload->data_array[2].metric2 = (timeValue.tv_usec % 1000) + 2000;
    udm_payload->data_array[2].metric3 = 1337;
} // end manipulateTempData

/********************************************************************/
/* QUDM_JSON -- send user data that is already in JSON format.      */
/*                                                                  */
/*   Input:  - Data type of the JSON data                           */
/*           - File name containing the JSON data                   */
/*           - Number of times to include the JSON data in the      */
/*             total payload sent                                   */
/*   Output: void                                                   */
/********************************************************************/
void  QUDM_JSON(char * data_type, char * jsonfile, int packages)
{
  int  rc;
  char wtopc_header[sizeof(struct wtopc_header)];
  char wtopc_buffer[255];
  int  i, payload_size;
  long file_size;
  FILE * file = NULL;
  char * jsondata;
  char * payload;
  char * payload_copy_ptr;
  struct tpf_rtmc_user_defined_metrics_parms send_input_parms = {};
  char * prepend_ebcdic =
    "{\"udm_data\":{\"header\":{\"version\":1,\"timestamp\":\"2023-03-30 19:26:49.472\",\"intervalLength\":1000},\"data_array\":[";
  int prepend_len = 110;
  char prepend_ascii[prepend_len];
  char comma_char = ',';
  char * comma_char_ptr = &comma_char;
  char * append_ebcdic = "]}}";
  int  append_len = strlen(append_ebcdic);
  char append_ascii[append_len];

  /******************************************************************/
  /* The DATATYPE parameter cannot be QUDM or QMWR (lower case or   */
  /* upper case.                                                    */
  /******************************************************************/

  if ((memcmp(data_type, "QUDM", sizeof("QUDM")) == 0) ||
      (memcmp(data_type, "qudm", sizeof("qudm")) == 0) ||
      (memcmp(data_type, "qmwr", sizeof("qmwr")) == 0) ||
      (memcmp(data_type, "QMWR", sizeof("QMWR")) == 0))
  {
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        16,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc("THE JSON PARAMETER WAS SPECIFIED. THE DATATYPE PARAMETER MUST NOT BE QUDM OR QMWR.",
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if DATATYPE parameter is QUDM or QMWR

  /**************************************************************/
  /* Open the JSON file and determine its length.               */
  /**************************************************************/
  file = fopen(jsonfile,"rb");
  if (file == NULL)
  {
    sprintf(wtopc_buffer,"UNABLE TO OPEN THE JSON FILE SPECIFIED. ERRNO %d REASON ", errno);
    strncat(wtopc_buffer, strerror(errno), (sizeof(wtopc_buffer) - strlen(wtopc_buffer)));
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        17,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if unable to open the JSON file specified

  rc = fseek(file, 0, SEEK_END);
  if (rc != 0)
  {
    sprintf(wtopc_buffer,"UNABLE TO SEEK THE END OF THE JSON FILE SPECIFIED. ERRNO %d REASON ", errno);
    strncat(wtopc_buffer, strerror(errno), (sizeof(wtopc_buffer) - strlen(wtopc_buffer)));
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        18,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if unable to find the end of the JSON file specified

  file_size = ftell(file);
  if (file_size == -1)
  {
    sprintf(wtopc_buffer,"UNABLE TO TELL THE ENDING POSITION OF THE JSON FILE SPECIFIED. ERRNO %d REASON ", errno);
    strncat(wtopc_buffer, strerror(errno), (sizeof(wtopc_buffer) - strlen(wtopc_buffer)));
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        19,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if unable to tell the ending position in the JSON file specified

  /**************************************************************/
  /* Determine the size of the total JSON payload.              */
  /* This is inserted at the beginning:                         */
  /*   {"udm_data":                                             */
  /*   {"header":                                               */
  /*   hard coded header data                                   */
  /*   ,"data_array":[                                          */
  /* Each package of data_array data passed as input is         */
  /* separated by a comma so (packages-1) is added to account   */
  /* for the commas.                                            */
  /* ]}} is put at the end.                                     */
  /**************************************************************/
  payload_size = (prepend_len +
                  (file_size * packages) +
                  (packages-1) +
                  append_len);

  sprintf(wtopc_buffer,"THE JSON FILE SIZE IS %d, THE TOTAL PAYLOAD SIZE IS %d.",
          file_size, payload_size);
  wtopc_insert_header(wtopc_header,
                      QUDM_MESSAGE_PREFIX,
                      15,'I',
                      WTOPC_SUBSYS_TIME);
  wtopc(wtopc_buffer,
        WTOPC_EBROUT,
        WTOPC_NO_CHAIN,
        wtopc_header);

  /**************************************************************/
  /* Get malloc storage for reading the file and the total      */
  /* JSON payload.                                              */
  /**************************************************************/
  jsondata = (char *)malloc64(file_size);
  if (jsondata == NULL)
  {
    sprintf(wtopc_buffer,"UNABLE TO OBTAIN MALLOC STORAGE. SIZE=%d",
            file_size);
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        20,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if malloc failed

  payload = (char *) malloc64(payload_size);
  if (payload == NULL)
  {
    sprintf(wtopc_buffer,"UNABLE TO OBTAIN MALLOC STORAGE. SIZE=%d",
            payload_size);
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        20,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if malloc failed

  memset(jsondata, 0x00, file_size);
  memset(payload, 0x00, payload_size);

  /**************************************************************/
  /* Read the contents of the JSON file.                        */
  /**************************************************************/
  fseek(file, 0, SEEK_SET);
  int fread_ret_size = fread((void *)jsondata,1,file_size,file);
  fclose(file);

  if(file_size != fread_ret_size)
  {
    sprintf(wtopc_buffer,"UNABLE TO READ THE JSON FILE SPECIFIED. ERRNO %d REASON ", errno);
    strncat(wtopc_buffer, strerror(errno), (sizeof(wtopc_buffer) - strlen(wtopc_buffer)));
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        21,'E',
                        WTOPC_SUBSYS_TIME);
    wtopc(wtopc_buffer,
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
    return;
  } // end if unable to read JSON file

  /**************************************************************/
  /* Get ASCII versions of the data prepended before the JSON   */
  /* data, a comma and the data appended after the JSON data.   */
  /**************************************************************/
  memset(prepend_ascii, '\0', prepend_len);
  EBCDIC_TO_ASCII(prepend_ebcdic,
                  prepend_ascii, (size_t)prepend_len);

  EBCDIC_TO_ASCII(comma_char_ptr, comma_char_ptr, 1);

  memset(append_ascii, '\0', append_len);
  EBCDIC_TO_ASCII(append_ebcdic,
                  append_ascii, (size_t)append_len);

  /**************************************************************/
  /* Put the prepend data at the beginning of the payload       */
  /* followed by a copy of the JSON data.                       */
  /**************************************************************/
  memcpy(payload, prepend_ascii, prepend_len);
  payload_copy_ptr = payload + prepend_len;
  memcpy(payload_copy_ptr, jsondata, file_size);
  payload_copy_ptr = payload_copy_ptr + file_size;

  /**************************************************************/
  /* Build the total JSON payload.                              */
  /* Keep repeating the JSON data for as many packages          */
  /* specified.                                                 */
  /* Give up control periodically to avoid a CTL-10.            */
  /**************************************************************/
  for (i=1; i<packages; i++)
  {
    if ((i % 100) == 0)
    {
      dlayc();
    }
    memcpy(payload_copy_ptr, comma_char_ptr, 1);
    payload_copy_ptr = payload_copy_ptr + 1;
    memcpy(payload_copy_ptr, jsondata, file_size);
    payload_copy_ptr = payload_copy_ptr + file_size;
  } // end for number of packages

  /**************************************************************/
  /* Put the append data following the JSON data.               */
  /**************************************************************/
  memcpy(payload_copy_ptr, append_ascii, append_len);

  /**************************************************************/
  /* Set up to send the data.                                   */
  /**************************************************************/
  send_input_parms.parms_version = TPF_RTMC_UDM_PARM_VER_1;
  strcpy(send_input_parms.data_type, data_type);
  send_input_parms.data = (void *)payload;
  send_input_parms.data_size = payload_size;

  rc = SendUDMData(&send_input_parms);

  if (rc == TPF_RTMC_UDM_OK)
  {
    wtopc_insert_header(wtopc_header,
                        QUDM_MESSAGE_PREFIX,
                        22,'I',
                        WTOPC_SUBSYS_TIME);
    wtopc("THE JSON PAYLOAD WAS SENT SUCCESSFULLY.",
          WTOPC_EBROUT,
          WTOPC_NO_CHAIN,
          wtopc_header);
  }

  return;
} // end QUDM_JSON

/********************************************************************/
/* PrintHelp -- display command format.                             */
/********************************************************************/
void PrintHelp(void)  {

  char  PrtBuf[255];      // wtopc_text parameter

  strcpy(PrtBuf, "\nQUDM:  A driver to test RTMC User Defined Metrics");
  strcat(PrtBuf, "\n");
  strcat(PrtBuf, "\nZTEST QUDM Help | ?");
  wtopc(PrtBuf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);

  strcpy(PrtBuf, "ZTEST QUDM STArt Rate-xxxx Token-yyy");
  strcat(PrtBuf, "\n to start continuous sawtooth data");
  strcat(PrtBuf, "\n where xxxx is the number of times to send the data per second");
  strcat(PrtBuf, "\n       yyy  is a unique value like 001 or ABC");
  wtopc(PrtBuf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);

  strcpy(PrtBuf, "ZTEST QUDM STOp Token-yyy|ALL");
  strcat(PrtBuf, "\n to stop continuous sawtooth data");
  strcat(PrtBuf, "\n where yyy is a token value specified on the START command");
  strcat(PrtBuf, "\n       ALL stops all instances of sending the sawtooth data");
  wtopc(PrtBuf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);

  strcpy(PrtBuf, "ZTEST QUDM JSON DATatype-datatype File-jsonfilename (PACKages-nnnnnnn)");
  strcat(PrtBuf, "\n to send JSON data");
  strcat(PrtBuf, "\n where datatype is the type of JSON data being sent (case sensitive)");
  wtopc(PrtBuf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);
  strcpy(PrtBuf, "\n       jsonfilename is the name of the file containing the JSON data (case sensitive)");
  strcat(PrtBuf, "\n       nnnnnnn is the number of times the JSON data is repeated before being sent (optional - default is 1)");
  wtopc(PrtBuf, WTOPC_EBROUT, WTOPC_CHAIN, WTOPC_NO_HEADER);

  wtopc("END OF DISPLAY",
         WTOPC_EBROUT, WTOPC_CHAIN_END, WTOPC_NO_HEADER);

  return;

}  // end of PrintHelp

/********************************************************************/
/* SendUDMData -- call tpf_rtmc_send_user_defined_metrics() and     */
/*                issue messages for any errors.                    */
/*                                                                  */
/*   Input:  pointer to the tpf_rtmc_send_user_defined_metrics()    */
/*           parameter structure.                                   */
/*   Output: return code from tpf_rtmc_send_user_defined_metrics(). */
/********************************************************************/
int SendUDMData(struct tpf_rtmc_user_defined_metrics_parms * send_parms)
{
  int rc;
  char wtopc_header[sizeof(struct wtopc_header)];
  char wtopc_buffer[255];

  rc = tpf_rtmc_send_user_defined_metrics(send_parms);

  if (rc != TPF_RTMC_UDM_OK)
  {

     /***********************************************************/
     /* If sending continuous sawtooth data (QUDM data) then    */
     /* some send errors can be ignored and the driver should   */
     /* continue to attempt to send data. These errors might be */
     /* temporary.                                              */
     /***********************************************************/
     if (memcmp(send_parms->data_type, "QUDM", sizeof("QUDM")) == 0)
     {
        if ((rc == TPF_RTMC_UDM_NOT_ACTIVE) ||
            (rc == TPF_RTMC_UDM_SEND_ERROR) ||
            (rc == TPF_RTMC_UDM_NOT_IN_CORRECT_STATE) ||
            (rc == TPF_RTMC_UDM_TIMEOUT))
        {
           return(TPF_RTMC_UDM_OK);
        } // end if send error can be ignored

     } // end if sending QUDM data

     /***********************************************************/
     /* The data could not be sent. Issue an error message.     */
     /***********************************************************/
     switch (rc)
     {
        case TPF_RTMC_UDM_NOT_ACTIVE :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               1,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("RTRTMC IS NOT PROCESSING UDM DATA.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_BAD_ENDPOINT_NAME :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               2,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("THE ENDPOINT GROUP NAME IS NOT VALID.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_INPUT_NOT_VALID :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               3,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("THE INPUT IS NOT VALID. THE POINTER PASSED IS NULL.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_SEND_ERROR :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               4,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("AN ERROR OCCURRED SENDING THE DATA.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_NOT_IN_CORRECT_STATE :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               5,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("THE SYSTEM IS NOT IN THE CORRECT STATE.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_TIMEOUT :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               6,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("A TIMEOUT OCCURRED WHILE SENDING THE DATA.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

         case TPF_RTMC_UDM_PAYLOAD_EXCEEDS_LIMIT :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               7,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("THE METRICS EXCEED THE MAXIMUM SIZE.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_PAYLOAD_EMPTY :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               8,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("THE METRICS DATA IS EMPTY.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_MALLOC_FAILURE :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               9,'E',
                               WTOPC_SUBSYS_TIME);

           wtopc("INSUFFICIENT MALLOC STORAGE IS AVAILABLE.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_DFDL_EXCEPTION :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               10,'E',
                               WTOPC_SUBSYS_TIME);


           wtopc("A DFDL ERROR OCCURRED WHILE CONVERTING THE STRUCTURE TO JSON.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        case TPF_RTMC_UDM_DATA_NODE_NOT_FOUND :
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               11,'E',
                               WTOPC_SUBSYS_TIME);


           wtopc("THE udm_data JSON NODE WAS NOT FOUND.",
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);
           break;

        default:
           wtopc_insert_header(wtopc_header,
                               QUDM_MESSAGE_PREFIX,
                               12,'E',
                               WTOPC_SUBSYS_TIME);

           sprintf(wtopc_buffer,
                   "UNEXPECTED RETURN CODE RECEIVED FROM tpf_rtmc_send_user_defined_metrics.\n"
                   "RETURN CODE %d",
                   rc);

           wtopc(wtopc_buffer,
                 WTOPC_EBROUT,
                 WTOPC_NO_CHAIN,
                 wtopc_header);

     } // end switch on return code

  } // end if sending the metrics resulted in an error

  return(rc);

} // end of SendUDMData
